package railcraft.client.core;

import cpw.mods.fml.client.FMLClientHandler;
import cpw.mods.fml.client.TextureFXManager;
import cpw.mods.fml.client.registry.*;
import cpw.mods.fml.common.registry.TickRegistry;
import cpw.mods.fml.relauncher.Side;
import java.util.logging.Level;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.client.MinecraftForgeClient;
import net.minecraftforge.common.MinecartRegistry;
import net.minecraftforge.common.MinecraftForge;
import railcraft.client.render.*;
import railcraft.client.render.carts.RenderCart;
import railcraft.client.sounds.SoundHandler;
import railcraft.client.textureFX.TextureCreosoteOilFX;
import railcraft.client.textureFX.TextureSteamFX;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.aesthetics.cube.EntityFallingCube;
import railcraft.common.blocks.machine.beta.*;
import railcraft.common.blocks.machine.gamma.TileLiquidLoader;
import railcraft.common.blocks.machine.gamma.TileLiquidUnloader;
import railcraft.common.carts.EntityCartTank;
import railcraft.common.carts.EntityTunnelBore;
import railcraft.common.core.CommonProxy;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.items.ItemGoggles;
import railcraft.common.util.misc.Game;
import railcraft.common.util.sounds.SoundRegistry;

public class ClientProxy extends CommonProxy {

    @Override
    public World getClientWorld() {
        return FMLClientHandler.instance().getClient().theWorld;
    }

    @Override
    public String getItemDisplayName(ItemStack stack) {
        return stack.getItem().getItemDisplayName(stack);
    }

    @Override
    public int getItemRarityColor(ItemStack stack) {
        return stack.getItem().getRarity(stack).rarityColor;
    }

    @Override
    public int getRenderId() {
        return RenderingRegistry.getNextAvailableRenderId();
    }

    @Override
    public void preInitClient() {
        MinecraftForge.EVENT_BUS.register(SoundHandler.getInstance());
    }

    @Override
    public void initClient() {
        TickRegistry.registerScheduledTickHandler(new LatestVersionMessage(), Side.CLIENT);

        SoundRegistry.setupBlockSounds();

        if (!ItemGoggles.areEnabled()) {
            KeyBindingRegistry.registerKeyBinding(new AuraKeyHandler());
        }

        Game.log(Level.FINER, "Init Start: Renderer");

        MinecraftForgeClient.preloadTexture(RailcraftConstants.MAIN_TEXTURE_FILE);
        MinecraftForgeClient.preloadTexture(RailcraftConstants.TRACK_TEXTURE_FILE);
        MinecraftForgeClient.preloadTexture(RailcraftConstants.MACHINE_TEXTURE_FILE);
        MinecraftForgeClient.preloadTexture(RailcraftConstants.LIQUID_TEXTURE_FILE);

        TextureFXManager.instance().addAnimation(new TextureCreosoteOilFX());
        TextureFXManager.instance().addAnimation(new TextureSteamFX());

        RenderLiquidLoader liquidLoaderRenderer = new RenderLiquidLoader();
        ClientRegistry.bindTileEntitySpecialRenderer(TileLiquidLoader.class, liquidLoaderRenderer);
        ClientRegistry.bindTileEntitySpecialRenderer(TileLiquidUnloader.class, liquidLoaderRenderer);

        ClientRegistry.bindTileEntitySpecialRenderer(TileTankIronGauge.class, new RenderIronTank());
        ClientRegistry.bindTileEntitySpecialRenderer(TileTankIronWall.class, new RenderIronTank());
        ClientRegistry.bindTileEntitySpecialRenderer(TileTankIronValve.class, new RenderIronTank());

        ClientRegistry.bindTileEntitySpecialRenderer(TileEngineSteamHobby.class, new RenderPneumaticEngine(RailcraftConstants.ENGINE_TEXTURE_FOLDER + "steam_hobby.png"));
        ClientRegistry.bindTileEntitySpecialRenderer(TileEngineSteamLow.class, new RenderPneumaticEngine(RailcraftConstants.ENGINE_TEXTURE_FOLDER + "steam_low.png"));
        ClientRegistry.bindTileEntitySpecialRenderer(TileEngineSteamHigh.class, new RenderPneumaticEngine(RailcraftConstants.ENGINE_TEXTURE_FOLDER + "steam_high.png"));

        if (RailcraftBlocks.getBlockTrack() != null) {
            RenderingRegistry.registerBlockHandler(new RenderTrack());
        }

        if (RailcraftBlocks.getBlockElevator() != null) {
            RenderingRegistry.registerBlockHandler(new RenderElevator());
        }

        BlockRenderer renderer = new RenderBlockMachineBeta();
        if (renderer.getBlock() != null) {
            RenderingRegistry.registerBlockHandler(renderer);
            MinecraftForgeClient.registerItemRenderer(renderer.getBlock().blockID, renderer.getItemRenderer());
        }

        renderer = new RenderBlockSignal();
        if (renderer.getBlock() != null) {
            RenderingRegistry.registerBlockHandler(renderer);
            MinecraftForgeClient.registerItemRenderer(renderer.getBlock().blockID, renderer.getItemRenderer());
        }

        renderer = new RenderBlockPost();
        if (renderer.getBlock() != null) {
            RenderingRegistry.registerBlockHandler(renderer);
            MinecraftForgeClient.registerItemRenderer(renderer.getBlock().blockID, renderer.getItemRenderer());
        }

        renderer = new RenderBlockPostMetal();
        if (renderer.getBlock() != null) {
            RenderingRegistry.registerBlockHandler(renderer);
            MinecraftForgeClient.registerItemRenderer(renderer.getBlock().blockID, renderer.getItemRenderer());
        }

        renderer = new RenderBlockOre();
        if (renderer.getBlock() != null) {
            RenderingRegistry.registerBlockHandler(renderer);
            MinecraftForgeClient.registerItemRenderer(renderer.getBlock().blockID, renderer.getItemRenderer());
        }

        renderer = new RenderWall();
        if (renderer.getBlock() != null) {
            RenderingRegistry.registerBlockHandler(renderer);
            MinecraftForgeClient.registerItemRenderer(renderer.getBlock().blockID, renderer.getItemRenderer());
        }

        RenderingRegistry.registerEntityRenderingHandler(EntityFallingCube.class, new RenderFallingCube());
        RenderingRegistry.registerEntityRenderingHandler(EntityTunnelBore.class, new RenderTunnelBore());
        RenderingRegistry.registerEntityRenderingHandler(EntityMinecart.class, new RenderCart());

        ItemStack tankCart = MinecartRegistry.getItemForCart(EntityCartTank.class);
        if (tankCart != null) {
            MinecraftForgeClient.registerItemRenderer(tankCart.itemID, new RenderTankCartItem());
        }

        Game.log(Level.FINER, "Init Complete: Renderer");
    }

    @Override
    public String getSessionUsername() {
        return Minecraft.getMinecraft().session.username;
    }
}
