package railcraft.client.gui;

import java.util.logging.Level;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.entity.player.InventoryPlayer;
import railcraft.common.blocks.detector.TileDetectorAdvanced;
import railcraft.common.blocks.detector.TileDetectorAnimal;
import railcraft.common.blocks.detector.TileDetectorItem;
import railcraft.common.blocks.detector.TileDetectorLocomotive;
import railcraft.common.blocks.detector.TileDetectorSheep;
import railcraft.common.blocks.detector.TileDetectorTank;
import railcraft.common.blocks.detector.TileDetectorTrain;
import railcraft.common.blocks.detector.TileDetectorVillager;
import railcraft.common.blocks.machine.ITankTile;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.alpha.TileBlastFurnace;
import railcraft.common.blocks.machine.alpha.TileCokeOven;
import railcraft.common.blocks.machine.alpha.TileFeedStation;
import railcraft.common.blocks.machine.alpha.TileRockCrusher;
import railcraft.common.blocks.machine.alpha.TileRollingMachine;
import railcraft.common.blocks.machine.alpha.TileSteamTurbine;
import railcraft.common.blocks.machine.alpha.TileAnchorWorld;
import railcraft.common.blocks.machine.alpha.TileSteamOven;
import railcraft.common.blocks.machine.beta.TileBoilerFireboxLiquid;
import railcraft.common.blocks.machine.beta.TileBoilerFireboxSolid;
import railcraft.common.blocks.machine.beta.TileEngineSteam;
import railcraft.common.blocks.machine.beta.TileEngineSteamHobby;
import railcraft.common.blocks.machine.gamma.TileDispenserCart;
import railcraft.common.blocks.machine.gamma.TileDispenserTrain;
import railcraft.common.blocks.machine.gamma.TileEnergyLoader;
import railcraft.common.blocks.machine.gamma.TileEnergyUnloader;
import railcraft.common.blocks.machine.gamma.TileLiquidLoader;
import railcraft.common.blocks.machine.gamma.TileLiquidUnloader;
import railcraft.common.blocks.machine.gamma.TileLoaderItemBase;
import railcraft.common.blocks.signals.TileBoxBlockRelay;
import railcraft.common.blocks.signals.TileBoxCapacitor;
import railcraft.common.blocks.signals.TileBoxController;
import railcraft.common.blocks.signals.TileBoxReceiver;
import railcraft.common.blocks.signals.TileSwitchMotor;
import railcraft.common.blocks.tracks.TileTrack;
import railcraft.common.blocks.tracks.TrackEmbarking;
import railcraft.common.blocks.tracks.TrackLauncher;
import railcraft.common.blocks.tracks.TrackPriming;
import railcraft.common.carts.EntityCartAnchor;
import railcraft.common.carts.EntityCartEnergy;
import railcraft.common.carts.EntityCartTNT;
import railcraft.common.carts.EntityCartTank;
import railcraft.common.carts.EntityCartTrackRelayer;
import railcraft.common.carts.EntityCartUndercutter;
import railcraft.common.carts.EntityCartWork;
import railcraft.common.carts.EntityLocomotiveSteam;
import railcraft.common.carts.EntityTunnelBore;
import railcraft.common.gui.EnumGui;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class FactoryGui {

    public static GuiScreen build(EnumGui gui, InventoryPlayer inv, Object obj) {
        if (obj == null) {
            return null;
        }

        if (obj instanceof TileMultiBlock && !((TileMultiBlock) obj).isStructureValid()) {
            return null;
        }

        try {
            switch (gui) {
                case LOADER_ITEM:
                    return new GuiLoaderItem(inv, (TileLoaderItemBase) obj);
                case LOADER_LIQUID:
                    return new GuiLoaderLiquid(inv, (TileLiquidLoader) obj);
                case UNLOADER_LIQUID:
                    return new GuiUnloaderLiquid(inv, (TileLiquidUnloader) obj);
                case LOADER_ENERGY:
                    return new GuiLoaderEnergy(inv, (TileEnergyLoader) obj);
                case UNLOADER_ENERGY:
                    return new GuiUnloaderEnergy(inv, (TileEnergyUnloader) obj);
                case DETECTOR_ITEM:
                    return new GuiDetectorItem(inv, (TileDetectorItem) obj);
                case DETECTOR_TANK:
                    return new GuiDetectorTank(inv, (TileDetectorTank) obj);
                case DETECTOR_SHEEP:
                    return new GuiDetectorSheep(inv, (TileDetectorSheep) obj);
                case DETECTOR_ANIMAL:
                    return new GuiDetectorAnimal((TileDetectorAnimal) obj);
                case DETECTOR_ADVANCED:
                    return new GuiDetectorAdvanced(inv, (TileDetectorAdvanced) obj);
                case DETECTOR_TRAIN:
                    return new GuiDetectorTrain((TileDetectorTrain) obj);
                case DETECTOR_VILLAGER:
                    return new GuiDetectorVillager((TileDetectorVillager) obj);
                case DETECTOR_LOCOMOTIVE:
                    return new GuiDetectorLocomotive(inv, (TileDetectorLocomotive) obj);
                case CART_DISPENSER:
                    return new GuiDispenserCart(inv, (TileDispenserCart) obj);
                case TRAIN_DISPENSER:
                    return new GuiDispenserTrain(inv, (TileDispenserTrain) obj);
                case COKE_OVEN:
                    return new GuiCokeOven(inv, (TileCokeOven) obj);
                case BLAST_FURNACE:
                    return new GuiBlastFurnace(inv, (TileBlastFurnace) obj);
                case STEAN_OVEN:
                    return new GuiSteamOven(inv, (TileSteamOven) obj);
                case TANK:
                    return new GuiTank(inv, (ITankTile) obj);
                case ROCK_CRUSHER:
                    return new GuiRockCrusher(inv, (TileRockCrusher) obj);
                case ROLLING_MACHINE:
                    return new GuiRollingMachine(inv, (TileRollingMachine) obj);
                case FEED_STATION:
                    return new GuiFeedStation(inv, (TileFeedStation) obj);
                case WORLD_ANCHOR:
                    return new GuiWorldAnchor(inv, (TileAnchorWorld) obj);
                case ENGINE_STEAM:
                    return new GuiEngineSteam(inv, (TileEngineSteam) obj);
                case ENGINE_HOBBY:
                    return new GuiEngineSteamHobby(inv, (TileEngineSteamHobby) obj);
                case BOILER_SOLID:
                    return new GuiBoilerSolid(inv, (TileBoilerFireboxSolid) obj);
                case BOILER_LIQUID:
                    return new GuiBoilerLiquid(inv, (TileBoilerFireboxLiquid) obj);
                case TURBINE:
                    return new GuiTurbine(inv, (TileSteamTurbine) obj);
                case SWITCH_MOTOR:
                    return new GuiAspectAction(RailcraftLanguage.translate("gui.switch.motor.action"), (TileSwitchMotor) obj);
                case BOX_RECEIVER:
                    return new GuiAspectAction(RailcraftLanguage.translate("gui.box.aspect.action"), (TileBoxReceiver) obj);
                case BOX_RELAY:
                    return new GuiAspectAction(RailcraftLanguage.translate("gui.box.aspect.action"), (TileBoxBlockRelay) obj);
                case BOX_CONTROLLER:
                    return new GuiBoxController((TileBoxController) obj);
                case BOX_CAPACITOR:
                    return new GuiBoxCapacitor((TileBoxCapacitor) obj);
                case TRACK_LAUNCHER:
                    return new GuiTrackLauncher((TrackLauncher) ((TileTrack) obj).getTrackInstance());
                case TRACK_PRIMING:
                    return new GuiTrackPriming((TrackPriming) ((TileTrack) obj).getTrackInstance());
                case TRACK_EMBARKING:
                    return new GuiTrackEmbarking((TrackEmbarking) ((TileTrack) obj).getTrackInstance());
                case CART_BORE:
                    return new GuiCartBore(inv, (EntityTunnelBore) obj);
                case CART_ENERGY:
                    return new GuiCartEnergy(inv, (EntityCartEnergy) obj);
                case CART_TANK:
                    return new GuiCartTank(inv, (EntityCartTank) obj);
                case CART_ANCHOR:
                    return new GuiCartAnchor(inv, (EntityCartAnchor) obj);
                case CART_TNT_FUSE:
                    return new GuiCartTNTFuse((EntityCartTNT) obj);
                case CART_WORK:
                    return new GuiCartWork(inv, (EntityCartWork) obj);
                case CART_TRACK_RELAYER:
                    return new GuiCartTrackRelayer(inv, (EntityCartTrackRelayer) obj);
                case CART_UNDERCUTTER:
                    return new GuiCartUndercutter(inv, (EntityCartUndercutter) obj);
                case LOCO_STEAM:
                    return new GuiLocomotiveSteam(inv, (EntityLocomotiveSteam) obj);
                default:
                    return null;
            }
        } catch (ClassCastException ex) {
            Game.log(Level.WARNING, "Error when attempting to build gui {0}: {1}", gui, ex);
        }
        return null;
    }
}
