package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.client.gui.GuiButton;
import railcraft.client.gui.buttons.GuiToggleButton;
import railcraft.common.api.signals.SignalAspect;
import railcraft.common.blocks.signals.IAspectActionManager;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.IGuiReturnHandler;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiAspectAction extends GuiBasic
{

    private final IAspectActionManager tile;
    private final String action;
    private boolean[] aspects = new boolean[SignalAspect.values().length];

    public GuiAspectAction(String action, IAspectActionManager t)
    {
//        super(label, RailcraftConstants.TEXTURE_FOLDER + "gui_long.png", 256, 88);
        super(t.getInvName());
        tile = t;
        this.action = action;
        for(SignalAspect aspect : SignalAspect.values()) {
            aspects[aspect.ordinal()] = tile.doesActionOnAspect(aspect);
        }
    }

    @Override
    public void initGui()
    {
        if(tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        controlList.add(new GuiToggleButton(0, w + 7, h + 35, 50, 20, "Green", aspects[SignalAspect.GREEN.ordinal()]));
        controlList.add(new GuiToggleButton(1, w + 12, h + 60, 70, 20, "Blink Yellow", aspects[SignalAspect.BLINK_YELLOW.ordinal()]));
        controlList.add(new GuiToggleButton(2, w + 63, h + 35, 50, 20, "Yellow", aspects[SignalAspect.YELLOW.ordinal()]));
        controlList.add(new GuiToggleButton(3, w + 94, h + 60, 70, 20, "Blink Red", aspects[SignalAspect.BLINK_RED.ordinal()]));
        controlList.add(new GuiToggleButton(4, w + 119, h + 35, 50, 20, "Red", aspects[SignalAspect.RED.ordinal()]));
    }

    @Override
    protected void drawExtras(int x, int y, float f)
    {
        GuiTools.drawCenteredString(fontRenderer, action, 20);
    }

    @Override
    protected void actionPerformed(GuiButton button)
    {
        if(tile == null) {
            return;
        }
        SignalAspect aspect = SignalAspect.values()[button.id];
        aspects[aspect.ordinal()] = !aspects[aspect.ordinal()];
        ((GuiToggleButton)button).active = aspects[aspect.ordinal()];
    }

    @Override
    public void onGuiClosed()
    {
        if(Game.isNotHost(tile.getWorld()) && tile instanceof IGuiReturnHandler) {
            for(SignalAspect aspect : SignalAspect.values()) {
                tile.doActionOnAspect(aspect, aspects[aspect.ordinal()]);
            }
            PacketGuiReturn pkt = new PacketGuiReturn((IGuiReturnHandler)tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }
}
