package railcraft.client.gui;

import java.util.List;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.inventory.GuiContainer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.inventory.Slot;
import net.minecraftforge.liquids.LiquidStack;
import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.GL11;
import railcraft.client.render.LiquidRenderer;
import railcraft.common.gui.containers.RailcraftContainer;
import railcraft.common.gui.indicator.Indicator;
import railcraft.common.gui.util.ToolTip;
import railcraft.common.liquids.LiquidGauge;

public abstract class GuiContainerRailcraft extends GuiContainer {

    private final RailcraftContainer container;
    public final String texturePath;

    public GuiContainerRailcraft(RailcraftContainer container, String texture) {
        super(container);
        this.container = container;
        this.texturePath = texture;
    }

    /**
     * Draws the screen and all the components in it.
     */
    @Override
    public void drawScreen(int mouseX, int mouseY, float par3) {
        super.drawScreen(mouseX, mouseY, par3);
        int left = this.guiLeft;
        int top = this.guiTop;

        GL11.glDisable(GL11.GL_LIGHTING);
        GL11.glDisable(GL11.GL_DEPTH_TEST);
        GL11.glPushMatrix();
        GL11.glTranslatef((float) left, (float) top, 0.0F);
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        RenderHelper.disableStandardItemLighting();

        InventoryPlayer playerInv = this.mc.thePlayer.inventory;

        if (playerInv.getItemStack() == null) {
            for (LiquidGauge gauge : container.getGauges()) {
                if (isMouseOverGauge(gauge, mouseX, mouseY)) {
                    List<ToolTip> tips = gauge.getToolTip();
                    if (tips != null) {
                        drawToolTips(tips, mouseX, mouseY);
                    }
                    break;
                }
            }
            for (Indicator indicator : container.getIndicators()) {
                if (isMouseOverIndicator(indicator, mouseX, mouseY)) {
                    List<ToolTip> tips = indicator.controller.getToolTip();
                    if (tips != null) {
                        drawToolTips(tips, mouseX, mouseY);
                    }
                    break;
                }
            }
        }

        GL11.glPopMatrix();
        GL11.glEnable(GL11.GL_LIGHTING);
        GL11.glEnable(GL11.GL_DEPTH_TEST);
    }

    @Override
    protected void drawGuiContainerBackgroundLayer(float f, int i, int j) {
        int texture = mc.renderEngine.getTexture(texturePath);
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        mc.renderEngine.bindTexture(texture);
        int x = (width - xSize) / 2;
        int y = (height - ySize) / 2;
        drawTexturedModalRect(x, y, 0, 0, xSize, ySize);

        for (LiquidGauge gauge : container.getGauges()) {
            drawLiquidGauge(gauge);
        }
        for (Indicator indicator : container.getIndicators()) {
            drawIndicator(indicator);
        }
    }

    private void drawButtons(int par1, int par2) {
        for (int var4 = 0; var4 < this.controlList.size(); ++var4) {
            GuiButton var5 = (GuiButton) this.controlList.get(var4);
            var5.drawButton(this.mc, par1, par2);
        }
    }

    @Override
    protected void mouseClicked(int par1, int par2, int mouseButton) {
        super.mouseClicked(par1, par2, mouseButton);

        if (mouseButton == 2) {
            Slot var4 = this.getSlotAtPosition(par1, par2);
            int var5 = this.guiLeft;
            int var6 = this.guiTop;
            boolean var7 = par1 < var5 || par2 < var6 || par1 >= var5 + this.xSize || par2 >= var6 + this.ySize;
            int var8 = -1;

            if (var4 != null) {
                var8 = var4.slotNumber;
            }

            if (var7) {
                var8 = -999;
            }

            if (var8 != -1) {
                boolean var9 = var8 != -999 && (Keyboard.isKeyDown(42) || Keyboard.isKeyDown(54));
                this.handleMouseClick(var4, var8, mouseButton, var9 ? 1 : 0);
            }
        }
    }

    private Slot getSlotAtPosition(int par1, int par2) {
        for (int var3 = 0; var3 < this.inventorySlots.inventorySlots.size(); ++var3) {
            Slot var4 = (Slot) this.inventorySlots.inventorySlots.get(var3);

            if (this.isMouseOverSlot(var4, par1, par2)) {
                return var4;
            }
        }
        return null;
    }

    /**
     * Returns if the passed mouse position is over the specified slot.
     */
    private boolean isMouseOverSlot(Slot par1Slot, int par2, int par3) {
        int var4 = this.guiLeft;
        int var5 = this.guiTop;
        par2 -= var4;
        par3 -= var5;
        return par2 >= par1Slot.xDisplayPosition - 1 && par2 < par1Slot.xDisplayPosition + 16 + 1 && par3 >= par1Slot.yDisplayPosition - 1 && par3 < par1Slot.yDisplayPosition + 16 + 1;
    }

    /**
     * Returns if the passed mouse position is over the specified slot.
     */
    private boolean isMouseOverGauge(LiquidGauge gauge, int mouseX, int mouseY) {
        int left = this.guiLeft;
        int top = this.guiTop;
        mouseX -= left;
        mouseY -= top;
        return mouseX >= gauge.x - 1 && mouseX < gauge.x + gauge.w + 1 && mouseY >= gauge.y - 1 && mouseY < gauge.y + gauge.h + 1;
    }

    /**
     * Returns if the passed mouse position is over the specified slot.
     */
    private boolean isMouseOverIndicator(Indicator indicator, int mouseX, int mouseY) {
        int left = this.guiLeft;
        int top = this.guiTop;
        mouseX -= left;
        mouseY -= top;
        return mouseX >= indicator.x - 1 && mouseX < indicator.x + indicator.w + 1 && mouseY >= indicator.y - 1 && mouseY < indicator.y + indicator.h + 1;
    }

    private void drawToolTips(List<ToolTip> toolTips, int mouseX, int mouseY) {
        if (toolTips.size() > 0) {
            int left = this.guiLeft;
            int top = this.guiTop;
            int lenght = 0;
            int x;
            int y;

            for (ToolTip tip : toolTips) {
                y = this.fontRenderer.getStringWidth(tip.text);

                if (y > lenght) {
                    lenght = y;
                }
            }

            x = mouseX - left + 12;
            y = mouseY - top - 12;
            int var14 = 8;

            if (toolTips.size() > 1) {
                var14 += 2 + (toolTips.size() - 1) * 10;
            }

            this.zLevel = 300.0F;
            itemRenderer.zLevel = 300.0F;
            int var15 = -267386864;
            this.drawGradientRect(x - 3, y - 4, x + lenght + 3, y - 3, var15, var15);
            this.drawGradientRect(x - 3, y + var14 + 3, x + lenght + 3, y + var14 + 4, var15, var15);
            this.drawGradientRect(x - 3, y - 3, x + lenght + 3, y + var14 + 3, var15, var15);
            this.drawGradientRect(x - 4, y - 3, x - 3, y + var14 + 3, var15, var15);
            this.drawGradientRect(x + lenght + 3, y - 3, x + lenght + 4, y + var14 + 3, var15, var15);
            int var16 = 1347420415;
            int var17 = (var16 & 16711422) >> 1 | var16 & -16777216;
            this.drawGradientRect(x - 3, y - 3 + 1, x - 3 + 1, y + var14 + 3 - 1, var16, var17);
            this.drawGradientRect(x + lenght + 2, y - 3 + 1, x + lenght + 3, y + var14 + 3 - 1, var16, var17);
            this.drawGradientRect(x - 3, y - 3, x + lenght + 3, y - 3 + 1, var16, var16);
            this.drawGradientRect(x - 3, y + var14 + 2, x + lenght + 3, y + var14 + 3, var17, var17);

            boolean first = true;
            for (ToolTip tip : toolTips) {
                String line = tip.text;

                if (tip.color == -1) {
                    line = "\u00a77" + line;
                } else {
                    line = "\u00a7" + Integer.toHexString(tip.color) + line;
                }

                this.fontRenderer.drawStringWithShadow(line, x, y, -1);

                if (first) {
                    y += 2;
                    first = false;
                }

                y += 10;
            }

            this.zLevel = 0.0F;
            itemRenderer.zLevel = 0.0F;
        }
    }

    public void drawLiquidGauge(LiquidGauge gauge) {
        int guiX = (width - xSize) / 2;
        int guiY = (height - ySize) / 2;
        if (gauge.tank == null) {
            return;
        }
        LiquidStack liquid = gauge.tank.getLiquid();
        if (liquid == null) {
            return;
        }

        int textureIndex = LiquidRenderer.bindLiquidTexture(liquid);

        if (textureIndex < 0) {
            return;
        }

        int imgLine = textureIndex / 16;
        int imgColumn = textureIndex - imgLine * 16;

        int scale = (int) Math.ceil(((float) Math.min(liquid.amount, gauge.tank.getCapacity()) / (float) gauge.tank.getCapacity()) * gauge.h);

        for (int col = 0; col < gauge.w / 16; col++) {
            int start = 0;
            int s = scale;
            while (true) {
                int x = 0;

                if (s > 16) {
                    x = 16;
                    s -= 16;
                } else {
                    x = s;
                    s = 0;
                }

                drawTexturedModalRect(guiX + gauge.x + col * 16, guiY + gauge.y + gauge.h - x - start,
                        imgColumn * 16, imgLine * 16 + (16 - x), 16, 16 - (16 - x));
                start = start + 16;

                if (x == 0 || s == 0) {
                    break;
                }
            }
        }

        int t = mc.renderEngine.getTexture(texturePath);
        mc.renderEngine.bindTexture(t);
        drawTexturedModalRect(guiX + gauge.x, guiY + gauge.y, xSize, 0, gauge.w, gauge.h);
    }

    public void drawIndicator(Indicator indicator) {
        int x = (this.width - this.xSize) / 2;
        int y = (this.height - this.ySize) / 2;
        int scale = indicator.controller.getScaledLevel(indicator.h);
        this.drawTexturedModalRect(x + indicator.x, y + indicator.y + indicator.h - scale, indicator.u, indicator.v + indicator.h - scale, indicator.w, scale);
    }
}
