package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.client.gui.GuiButton;
import railcraft.client.gui.buttons.GuiBetterButton;
import railcraft.client.gui.buttons.GuiToggleButton;
import railcraft.common.blocks.detector.TileDetectorAnimal;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiDetectorAnimal extends GuiBasic
{

    private static final String LABEL = "Detector - Animal";
    TileDetectorAnimal tile;

    public GuiDetectorAnimal(TileDetectorAnimal t)
    {
        this(LABEL, t);
    }

    protected GuiDetectorAnimal(String label, TileDetectorAnimal t)
    {
        super(label, RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_long.png", 256, 88);
        tile = t;
    }

    @Override
    public void initGui()
    {
        if(tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;

        List<GuiBetterButton> row1 = new ArrayList<GuiBetterButton>();
        row1.add(new GuiToggleButton(0, 0, h + 25, 50, 20, "Chicken", tile.chicken));
        row1.add(new GuiToggleButton(1, 0, h + 25, 50, 20, "Cow", tile.cow));
        row1.add(new GuiToggleButton(2, 0, h + 25, 50, 20, "Pig", tile.pig));
        row1.add(new GuiToggleButton(3, 0, h + 25, 50, 20, "Sheep", tile.sheep));
        GuiTools.newButtonRowAuto(controlList, w, xSize, row1);

        List<GuiBetterButton> row2 = new ArrayList<GuiBetterButton>();
        row2.add(new GuiToggleButton(4, 0, h + 55, 50, 20, "Wolf", tile.wolf));
        row2.add(new GuiToggleButton(5, 0, h + 55, 70, 20, "Mooshroom", tile.mooshroom));
        row2.add(new GuiToggleButton(6, 0, h + 55, 50, 20, "Other", tile.other));
        GuiTools.newButtonRowAuto(controlList, w, xSize, row2);
    }

    @Override
    protected void actionPerformed(GuiButton button)
    {
        if(tile == null) {
            return;
        }
        switch (button.id) {
            case 0:
                tile.chicken = !tile.chicken;
                ((GuiToggleButton)button).active = tile.chicken;
                break;
            case 1:
                tile.cow = !tile.cow;
                ((GuiToggleButton)button).active = tile.cow;
                break;
            case 2:
                tile.pig = !tile.pig;
                ((GuiToggleButton)button).active = tile.pig;
                break;
            case 3:
                tile.sheep = !tile.sheep;
                ((GuiToggleButton)button).active = tile.sheep;
                break;
            case 4:
                tile.wolf = !tile.wolf;
                ((GuiToggleButton)button).active = tile.wolf;
                break;
            case 5:
                tile.mooshroom = !tile.mooshroom;
                ((GuiToggleButton)button).active = tile.mooshroom;
                break;
            case 6:
                tile.other = !tile.other;
                ((GuiToggleButton)button).active = tile.other;
                break;
        }
    }

    @Override
    public void onGuiClosed()
    {
        if(Game.isNotHost(tile.getWorld())) {
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }
}
