package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import java.awt.Color;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Slot;
import org.lwjgl.opengl.GL11;
import railcraft.client.gui.buttons.GuiButtonSmall;
import railcraft.common.blocks.detector.TileDetectorItem;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerDetectorItem;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiDetectorItem extends TileGui
{

    private static final String label = RailcraftLanguage.translate("detector.item");
    private final TileDetectorItem tile;
    private GuiButton filterLeft;
    private GuiButton filterRight;

    public GuiDetectorItem(InventoryPlayer inv, TileDetectorItem tile)
    {
        super(tile, new ContainerDetectorItem(inv, tile), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_detector_item.png");
        this.tile = tile;
        xSize = 176;
        ySize = 166;
    }

    @Override
    public void initGui()
    {
        super.initGui();
        if(tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;

        controlList.add(new GuiButtonSmall(0, w + 10, h + 20, 20, "<"));
        controlList.add(new GuiButtonSmall(1, w + 146, h + 20, 20, ">"));

        controlList.add(filterLeft = new GuiButtonSmall(2, w + 10, h + 40, 20, "<"));
        controlList.add(filterRight = new GuiButtonSmall(3, w + 146, h + 40, 20, ">"));
    }

    @Override
    protected void actionPerformed(GuiButton button)
    {
        if(tile == null) {
            return;
        }
        int primary = tile.getPrimaryMode().ordinal();
        int filter = tile.getFilterMode().ordinal();
        switch (button.id) {
            case 0:
                primary--;
                break;
            case 1:
                primary++;
                break;
            case 2:
                filter--;
                break;
            case 3:
                filter++;
                break;
        }
        if(primary < 0) {
            primary = TileDetectorItem.PrimaryMode.values().length - 1;
        }
        if(primary >= TileDetectorItem.PrimaryMode.values().length) {
            primary = 0;
        }
        tile.setPrimaryMode(TileDetectorItem.PrimaryMode.values()[primary]);

        if(filter < 0) {
            filter = TileDetectorItem.FilterMode.values().length - 1;
        }
        if(filter >= TileDetectorItem.FilterMode.values().length) {
            filter = 0;
        }
        tile.setFilterMode(TileDetectorItem.FilterMode.values()[filter]);
        if(Game.isNotHost(tile.getWorld())) {
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }

    @Override
    protected void drawGuiContainerBackgroundLayer(float f, int i, int j)
    {
        filterLeft.drawButton = tile.getPrimaryMode() == TileDetectorItem.PrimaryMode.FILTERED;
        filterRight.drawButton = tile.getPrimaryMode() == TileDetectorItem.PrimaryMode.FILTERED;

        super.drawGuiContainerBackgroundLayer(f, i, j);
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY)
    {
        int sWidth = fontRenderer.getStringWidth(label);
        int sPos = xSize / 2 - sWidth / 2;
        fontRenderer.drawString(label, sPos, 6, 0x404040);
//        GuiTools.drawCenteredString(fontRenderer, "Default Aspect:", 25);
        GuiTools.drawCenteredString(fontRenderer, tile.getPrimaryMode().toString(), 25);

        if(tile.getPrimaryMode() != TileDetectorItem.PrimaryMode.FILTERED) {
            Color color = new Color(0, 0, 0, 80);
            int displayY;

            GL11.glDisable(GL11.GL_LIGHTING);
            GL11.glDisable(GL11.GL_DEPTH_TEST);
            for(int slotNum = 0; slotNum < 9; slotNum++) {
                Slot slot = (Slot)this.inventorySlots.inventorySlots.get(slotNum);

                int displayX = slot.xDisplayPosition;
                displayY = slot.yDisplayPosition;
                this.drawGradientRect(displayX, displayY, displayX + 16, displayY + 16, color.getRGB(), color.getRGB());
            }
            GL11.glEnable(GL11.GL_LIGHTING);
            GL11.glEnable(GL11.GL_DEPTH_TEST);
        } else {
//        GuiTools.drawCenteredString(fontRenderer, "Redstone Aspect:", 60);
            GuiTools.drawCenteredString(fontRenderer, tile.getFilterMode().toString(), 45);
        }
    }

//    @Override
//    public void onGuiClosed()
//    {
//        if(Game.isNotHost(tile.getWorld())) {
//            PacketGuiReturn pkt = new PacketGuiReturn(tile);
//            PacketDispatcher.sendPacketToServer(pkt.getPacket());
//        }
//    }
}
