package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.entity.player.InventoryPlayer;
import railcraft.client.gui.buttons.GuiMultiButton;
import railcraft.common.blocks.detector.TileDetectorTank;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerDetectorTank;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiDetectorTank extends TileGui {

    private static final String label = RailcraftLanguage.translate("detector.tank");
    private final TileDetectorTank tile;
    private GuiMultiButton button;

    public GuiDetectorTank(InventoryPlayer inv, TileDetectorTank tile) {
        super(tile, new ContainerDetectorTank(inv, tile), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_detector_tank.png");
        this.tile = tile;
        xSize = 176;
        ySize = 140;
    }

    @Override
    public void initGui() {
        super.initGui();
        if (tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        controlList.add(button = new GuiMultiButton(0, w + 95, h + 22, 60, tile.getButtonController()));
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY) {
        int sWidth = fontRenderer.getStringWidth(label);
        int sPos = xSize / 2 - sWidth / 2;
        fontRenderer.drawString(label, sPos, 6, 0x404040);
        fontRenderer.drawString(RailcraftLanguage.translate("gui.filter"), 50, 29, 0x404040);
    }

    @Override
    public void onGuiClosed() {
        super.onGuiClosed();

        if (Game.isNotHost(tile.getWorld())) {
            tile.getButtonController().setCurrentState(button.getController().getCurrentState());
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }
}
