package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import cpw.mods.fml.common.registry.VillagerRegistry;
import java.util.Deque;
import java.util.LinkedList;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.client.renderer.entity.RenderManager;
import net.minecraft.entity.passive.EntityVillager;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL12;
import railcraft.client.gui.buttons.GuiToggleButtonSmall;
import railcraft.common.blocks.detector.TileDetectorVillager;
import railcraft.common.blocks.detector.TileDetectorVillager.Mode;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.RevolvingList;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiDetectorVillager extends GuiBasic {

    private final TileDetectorVillager tile;
    private final EntityVillager villager;
    private Mode mode;
    private GuiToggleButtonSmall any;
    private GuiToggleButtonSmall none;
    private GuiToggleButtonSmall equals;
    private GuiToggleButtonSmall not;
    private RevolvingList<Integer> professions = new RevolvingList<Integer>();

    public GuiDetectorVillager(TileDetectorVillager t) {
        super(t.getInvName(), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_detector_villager.png", 176, 105);
        this.tile = t;
        villager = new EntityVillager(tile.getWorld());
        mode = tile.getMode();

        for (int prof = 0; prof < 5; prof++) {
            professions.add(prof);
        }
        professions.addAll(VillagerRegistry.getRegisteredVillagers());

        professions.setCurrent(tile.getProfession());
        villager.setProfession(professions.getCurrent());
    }

    @Override
    public void initGui() {
        if (tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;

        controlList.add(new GuiButton(0, w + 10, h + 25, 30, 20, "<"));
        controlList.add(new GuiButton(1, w + 135, h + 25, 30, 20, ">"));

        controlList.add(any = new GuiToggleButtonSmall(2, w + 10, h + 55, 45, RailcraftLanguage.translate("gui.detector.villager.any"), mode == Mode.ANY));
        controlList.add(none = new GuiToggleButtonSmall(3, w + 10, h + 75, 45, RailcraftLanguage.translate("gui.detector.villager.none"), mode == Mode.NONE));
        controlList.add(equals = new GuiToggleButtonSmall(4, w + 122, h + 55, 45, RailcraftLanguage.translate("gui.detector.villager.equals"), mode == Mode.EQUALS));
        controlList.add(not = new GuiToggleButtonSmall(5, w + 122, h + 75, 45, RailcraftLanguage.translate("gui.detector.villager.not"), mode == Mode.NOT));
    }

    @Override
    protected void drawExtras(int x, int y, float f) {
        int guiLeft = (width - xSize) / 2;
        int guiTop = (height - ySize) / 2;
        drawVillager(87, 91, 30, (float) (guiLeft + 87) - x, (float) (guiTop + 91 - 50) - y);
    }

    public void drawVillager(int x, int y, int scale, float yaw, float pitch) {
        GL11.glPushAttrib(GL11.GL_ENABLE_BIT);
        GL11.glEnable(GL11.GL_LIGHTING);
        GL11.glEnable(GL11.GL_DEPTH_TEST);
        GL11.glEnable(GL11.GL_COLOR_MATERIAL);
        GL11.glPushMatrix();
        GL11.glTranslatef((float) x, (float) y, 50.0F);
        GL11.glScalef((float) (-scale), (float) scale, (float) scale);
        GL11.glRotatef(180.0F, 0.0F, 0.0F, 1.0F);
        GL11.glRotatef(135.0F, 0.0F, 1.0F, 0.0F);
        RenderHelper.enableStandardItemLighting();
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        GL11.glRotatef(-135.0F, 0.0F, 1.0F, 0.0F);
        GL11.glRotatef(-((float) Math.atan((double) (pitch / 40.0F))) * 20.0F, 1.0F, 0.0F, 0.0F);
        villager.renderYawOffset = (float) Math.atan((double) (yaw / 40.0F)) * 20.0F;
        villager.rotationYaw = (float) Math.atan((double) (yaw / 40.0F)) * 40.0F;
        villager.rotationPitch = -((float) Math.atan((double) (pitch / 40.0F))) * 20.0F;
        villager.rotationYawHead = villager.rotationYaw;
        GL11.glTranslatef(0.0F, villager.yOffset, 0.0F);
        RenderManager.instance.playerViewY = 180.0F;
        RenderManager.instance.renderEntityWithPosYaw(villager, 0.0D, 0.0D, 0.0D, 0.0F, 1.0F);
        GL11.glPopMatrix();
        RenderHelper.disableStandardItemLighting();
        GL11.glDisable(GL12.GL_RESCALE_NORMAL);
        OpenGlHelper.setActiveTexture(OpenGlHelper.lightmapTexUnit);
        GL11.glDisable(GL11.GL_TEXTURE_2D);
        OpenGlHelper.setActiveTexture(OpenGlHelper.defaultTexUnit);
        GL11.glPopAttrib();
    }

    @Override
    protected void actionPerformed(GuiButton button) {
        if (tile == null) {
            return;
        }
        switch (button.id) {
            case 0:
                professions.rotateLeft();
                break;
            case 1:
                professions.rotateRight();
                break;
            case 2:
                mode = Mode.ANY;
                break;
            case 3:
                mode = Mode.NONE;
                break;
            case 4:
                mode = Mode.EQUALS;
                break;
            case 5:
                mode = Mode.NOT;
                break;
        }

        any.active = mode == Mode.ANY;
        none.active = mode == Mode.NONE;
        equals.active = mode == Mode.EQUALS;
        not.active = mode == Mode.NOT;

        villager.setProfession(professions.getCurrent());
    }

    @Override
    public void onGuiClosed() {
        if (Game.isNotHost(tile.getWorld())) {
            tile.setProfession(professions.getCurrent());
            tile.setMode(mode);
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }
}
