package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.tileentity.TileEntity;
import railcraft.client.gui.buttons.GuiToggleButton;
import railcraft.common.blocks.machine.gamma.TileEnergyLoader;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerEnergyLoader;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiLoaderEnergy extends TileGui {

    private static final String BUTTON1 = RailcraftLanguage.translate("gui.energy.loader.empty");
    private static final String BUTTON2 = RailcraftLanguage.translate("gui.energy.loader.fill");
    private TileEnergyLoader tile;
    private boolean waitIfEmpty;
    private boolean waitTillFull;

    public GuiLoaderEnergy(InventoryPlayer inv, TileEnergyLoader tile) {
        super(tile, new ContainerEnergyLoader(inv, tile), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_energy_loader.png");
        this.tile = tile;
        waitIfEmpty = tile.waitIfEmpty();
        waitTillFull = tile.waitTillFull();
    }

    @Override
    public void initGui() {
        super.initGui();
        if (tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        controlList.add(new GuiToggleButton(0, w + 75, h + 18, 70, 20, BUTTON1, waitIfEmpty));
        controlList.add(new GuiToggleButton(1, w + 75, h + 42, 70, 20, BUTTON2, waitTillFull));
    }

    @Override
    protected void actionPerformed(GuiButton guibutton) {
        if (tile == null) {
            return;
        }
        if (guibutton.id == 0) {
            waitIfEmpty = !waitIfEmpty;
            if (!waitIfEmpty) {
                waitTillFull = false;
            }
            ((GuiToggleButton) guibutton).active = waitIfEmpty;
            ((GuiToggleButton) controlList.get(1)).active = waitTillFull;
        }
        if (guibutton.id == 1) {
            waitTillFull = !waitTillFull;
            if (waitTillFull) {
                waitIfEmpty = true;
            }
            ((GuiToggleButton) controlList.get(0)).active = waitIfEmpty;
            ((GuiToggleButton) guibutton).active = waitTillFull;
        }
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY) {
        int sWidth = fontRenderer.getStringWidth(tile.getInvName());
        int sPos = xSize / 2 - sWidth / 2;
        fontRenderer.drawString(tile.getInvName(), sPos, 6, 0x404040);

        fontRenderer.drawString(Integer.toString(tile.getEnergy()), 30, 55, 0x404040);

        StringBuilder capacity = new StringBuilder("/").append(tile.getCapacity());
        fontRenderer.drawString(capacity.toString(), 28, 65, 0x404040);

        fontRenderer.drawString("Rate: " + tile.getTransferRate(), 90, 67, 0x404040);
    }

    @Override
    protected void drawGuiContainerBackgroundLayer(float f, int i, int j) {
        super.drawGuiContainerBackgroundLayer(f, i, j);
        int x = (width - xSize) / 2;
        int y = (height - ySize) / 2;
        
        if (tile.getEnergy() > 0) {
            int energy = tile.getEnergyBarScaled(24);
            drawTexturedModalRect(x + 31, y + 34, 176, 14, energy, 17);
        }
    }

    @Override
    public void onGuiClosed() {
        if (Game.isNotHost(tile.getWorld())) {
            tile.setWaitIfEmpty(waitIfEmpty);
            tile.setWaitTillFull(waitTillFull);
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }

    @Override
    public void updateScreen() {
        super.updateScreen();
        TileEntity t = tile.getWorld().getBlockTileEntity(tile.getX(), tile.getY(), tile.getZ());
        if (t instanceof TileEnergyLoader) {
            tile = (TileEnergyLoader) t;
        } else {
            mc.thePlayer.closeScreen();
        }
    }
}
