package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.InventoryPlayer;
import railcraft.client.gui.buttons.GuiToggleButton;
import railcraft.client.gui.buttons.GuiToggleButtonSmall;
import railcraft.common.blocks.RailcraftTileEntity;
import railcraft.common.blocks.machine.gamma.TileLoaderItemBase;
import railcraft.common.blocks.machine.gamma.TileLoaderItemBase.EnumItemMode;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerItemLoader;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiLoaderItem extends TileGui
{

    private static final String FILTER_LABEL = RailcraftLanguage.translate("gui.filters");
    private static final String CART_FILTER_LABEL = RailcraftLanguage.translate("gui.filters.carts");
    private static final String BUFFER_LABEL = RailcraftLanguage.translate("gui.item.loader.buffer");
    private static final String WAIT_LABEL = RailcraftLanguage.translate("gui.item.loader.wait");
    private static final String TRANSFER_LABEL = RailcraftLanguage.translate("gui.item.loader.transfer");
    private static final String STOCK_LABEL = RailcraftLanguage.translate("gui.item.loader.stock");
    private static final String EXCESS_LABEL = RailcraftLanguage.translate("gui.item.loader.excess");
    private GuiToggleButtonSmall transfer;
    private GuiToggleButtonSmall stock;
    private GuiToggleButtonSmall excess;
    private final TileLoaderItemBase tile;
    private EnumItemMode mode;
    private boolean wait;

    public GuiLoaderItem(InventoryPlayer inv, TileLoaderItemBase tile)
    {
        super((RailcraftTileEntity)tile, new ContainerItemLoader(inv, tile), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_item_loader.png");
        this.tile = tile;
        mode = tile.getMode();
        wait = tile.cartShouldWait();
    }

    @Override
    public void initGui()
    {
        super.initGui();
        if(tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        controlList.add(new GuiToggleButtonSmall(0, w + 68, h + 62, 40, WAIT_LABEL, wait));

        transfer = new GuiToggleButtonSmall(1, w + 66, h + 45, 15, TRANSFER_LABEL, mode == EnumItemMode.TRANSFER);
        stock = new GuiToggleButtonSmall(2, w + 81, h + 45, 15, STOCK_LABEL, mode == EnumItemMode.STOCK);
        excess = new GuiToggleButtonSmall(3, w + 96, h + 45, 15, EXCESS_LABEL, mode == EnumItemMode.EXCESS);
        controlList.add(transfer);
        controlList.add(stock);
        controlList.add(excess);
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY)
    {
        GuiTools.drawCenteredString(fontRenderer, tile.getInvName(), 6);
        fontRenderer.drawString(FILTER_LABEL, 18, 16, 0x404040);
        fontRenderer.drawString(CART_FILTER_LABEL, 75, 16, 0x404040);
        fontRenderer.drawString(BUFFER_LABEL, 126, 16, 0x404040);
    }

    private void updateMode()
    {
        transfer.active = mode == EnumItemMode.TRANSFER;
        stock.active = mode == EnumItemMode.STOCK;
        excess.active = mode == EnumItemMode.EXCESS;
    }

    @Override
    protected void actionPerformed(GuiButton guibutton)
    {
        if(tile == null) {
            return;
        }
        switch (guibutton.id) {
            case 0:
                wait = !wait;
                ((GuiToggleButton)guibutton).active = wait;
                break;
            case 1:
                mode = EnumItemMode.TRANSFER;
                updateMode();
                break;
            case 2:
                mode = EnumItemMode.STOCK;
                updateMode();
                break;
            case 3:
                mode = EnumItemMode.EXCESS;
                updateMode();
                break;
        }
    }

    @Override
    public void onGuiClosed()
    {
        if(Game.isNotHost(tile.getWorld())) {
            tile.setCartShouldWait(wait);
            tile.setMode(mode);
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }
}
