package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.InventoryPlayer;
import railcraft.client.gui.buttons.GuiToggleButtonSmall;
import railcraft.common.blocks.machine.gamma.TileLiquidLoader;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerLiquidLoader;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiLoaderLiquid extends TileGui
{

    private static final String FILTER_LABEL = RailcraftLanguage.translate("gui.filters");
    private final TileLiquidLoader tile;
    private GuiToggleButtonSmall empty;
    private GuiToggleButtonSmall full;

    public GuiLoaderLiquid(InventoryPlayer inv, TileLiquidLoader tile)
    {
        super(tile, new ContainerLiquidLoader(inv, tile), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_liquid_loader.png");
        this.tile = tile;
    }

    @Override
    public void initGui()
    {
        super.initGui();
        if(tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        empty = new GuiToggleButtonSmall(0, w + 40, h + 45, 65, RailcraftLanguage.translate("gui.liquid.loader.empty"), tile.waitIfEmpty());
        controlList.add(empty);
        full = new GuiToggleButtonSmall(1, w + 40, h + 62, 65, RailcraftLanguage.translate("gui.liquid.loader.fill"), tile.waitTillFull());
        controlList.add(full);
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY)
    {
        int sWidth = fontRenderer.getStringWidth(tile.getInvName());
        int sPos = xSize / 2 - sWidth / 2;
        fontRenderer.drawString(tile.getInvName(), sPos, 6, 0x404040);
        fontRenderer.drawString(FILTER_LABEL, 85, 25, 0x404040);
    }

    @Override
    protected void actionPerformed(GuiButton guibutton)
    {
        if(tile == null) {
            return;
        }
        if(guibutton.id == 0) {
            setWaitIfEmpty(!tile.waitIfEmpty());
        }
        if(guibutton.id == 1) {
            setWaitTillFull(!tile.waitTillFull());
        }
    }

    private void setWaitIfEmpty(boolean set)
    {
        tile.setWaitIfEmpty(set);
        empty.active = set;
        if(!set) {
            setWaitTillFull(false);
        }
    }

    private void setWaitTillFull(boolean set)
    {
        tile.setWaitTillFull(set);
        full.active = set;
        if(set) {
            setWaitIfEmpty(true);
        }
    }

    @Override
    public void onGuiClosed()
    {
        tile.setWaitIfEmpty(empty.active);
        tile.setWaitTillFull(full.active);
        if(Game.isNotHost(tile.getWorld())) {
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }
}
