package railcraft.client.gui;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.util.StatCollector;
import railcraft.client.gui.buttons.GuiBetterButton;
import railcraft.client.gui.buttons.GuiToggleButtonSmall;
import railcraft.common.carts.EntityLocomotive.LocoMode;
import railcraft.common.carts.EntityLocomotive.LocoSpeed;
import railcraft.common.carts.EntityLocomotiveSteam;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerLocomotiveSteam;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.network.PacketBuilder;

public class GuiLocomotiveSteam extends EntityGui {

    private final EntityLocomotiveSteam loco;
    private GuiToggleButtonSmall running;
    private GuiToggleButtonSmall idle;
    private GuiToggleButtonSmall shutdown;
    private GuiToggleButtonSmall slowest;
    private GuiToggleButtonSmall slower;
    private GuiToggleButtonSmall slow;
    private GuiToggleButtonSmall max;
    private LocoMode mode;
    private LocoSpeed speed;

    public GuiLocomotiveSteam(InventoryPlayer inv, EntityLocomotiveSteam loco) {
        super(loco, new ContainerLocomotiveSteam(inv, loco), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_locomotive_steam.png");
        ySize = 205;
        this.loco = loco;
        this.mode = loco.getMode();
        this.speed = loco.getSpeed();
    }

    @Override
    public void initGui() {
        super.initGui();
        if (loco == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;

        List<GuiBetterButton> buttons = new ArrayList<GuiBetterButton>();
        running = new GuiToggleButtonSmall(0, 0, h + 76, 55, RailcraftLanguage.translate("gui.locomotive.mode.running"), mode == LocoMode.RUNNING);
        buttons.add(running);
        idle = new GuiToggleButtonSmall(1, 0, h + 76, 45, RailcraftLanguage.translate("gui.locomotive.mode.idle"), mode == LocoMode.IDLE);
        buttons.add(idle);
        shutdown = new GuiToggleButtonSmall(2, 0, h + 76, 55, RailcraftLanguage.translate("gui.locomotive.mode.shutdown"), mode == LocoMode.SHUTDOWN);
        buttons.add(shutdown);

        GuiTools.newButtonRowAuto(controlList, w + 3, 171, buttons);

        List<GuiBetterButton> speedButtons = new ArrayList<GuiBetterButton>();
        slowest = new GuiToggleButtonSmall(3, 0, h + 93, 12, ">", speed == LocoSpeed.SLOWEST);
        speedButtons.add(slowest);
        slower = new GuiToggleButtonSmall(4, 0, h + 93, 17, ">>", speed == LocoSpeed.SLOWER);
        speedButtons.add(slower);
        slow = new GuiToggleButtonSmall(5, 0, h + 93, 22, ">>>", speed == LocoSpeed.SLOW);
        speedButtons.add(slow);
        max = new GuiToggleButtonSmall(6, 0, h + 93, 27, ">>>>", speed == LocoSpeed.MAX);
        speedButtons.add(max);

        GuiTools.newButtonRow(controlList, w + 44, 4, speedButtons);


//        GuiToggleButtonSmall dest = new GuiToggleButtonSmall(2, w + 94, h + 94, 75, "Destination", true);
//        controlList.add(dest);
    }

    @Override
    protected void actionPerformed(GuiButton guibutton) {
        if (loco == null) {
            return;
        }
        switch (guibutton.id) {
            case 0:
                mode = LocoMode.RUNNING;
                break;
            case 1:
                mode = LocoMode.IDLE;
                break;
            case 2:
                mode = LocoMode.SHUTDOWN;
                break;
            case 3:
                speed = LocoSpeed.SLOWEST;
                break;
            case 4:
                speed = LocoSpeed.SLOWER;
                break;
            case 5:
                speed = LocoSpeed.SLOW;
                break;
            case 6:
                speed = LocoSpeed.MAX;
                break;
        }
        updateButtons();
        sendUpdatePacket();
    }

    private void updateButtons() {
        running.active = mode == LocoMode.RUNNING;
        idle.active = mode == LocoMode.IDLE;
        shutdown.active = mode == LocoMode.SHUTDOWN;
        slowest.active = speed == LocoSpeed.SLOWEST;
        slower.active = speed == LocoSpeed.SLOWER;
        slow.active = speed == LocoSpeed.SLOW;
        max.active = speed == LocoSpeed.MAX;
    }

    @Override
    public void onGuiClosed() {
        sendUpdatePacket();
    }

    private void sendUpdatePacket() {
        loco.clientMode = mode;
        loco.clientSpeed = speed;
        PacketBuilder.getInstance().sendGuiReturnPacket(loco);
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY) {
        GuiTools.drawCenteredString(fontRenderer, loco.getInvName(), 6);
        fontRenderer.drawString(StatCollector.translateToLocal("container.inventory"), 8, (ySize - 96) + 2, 0x404040);
    }

    @Override
    protected void drawGuiContainerBackgroundLayer(float par1, int par2, int par3) {
        super.drawGuiContainerBackgroundLayer(par1, par3, par3);
        int x = (this.width - this.xSize) / 2;
        int y = (this.height - this.ySize) / 2;

        if (this.loco.burnTime > 0) {
            int scale = this.loco.getBurnProgressScaled(12);
            this.drawTexturedModalRect(x + 62, y + 34 - scale, 176, 59 - scale, 14, scale + 2);
        }
    }
}
