package railcraft.client.gui;

import net.minecraft.client.gui.GuiButton;
import railcraft.common.blocks.tracks.TrackEmbarking;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.IGuiReturnHandler;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiTrackEmbarking extends GuiBasic
{

    protected byte radius = 2;
    TrackEmbarking track;

    public GuiTrackEmbarking(TrackEmbarking t) {
        super(RailcraftLanguage.translate(t.getTrackType().getTag()));
        track = t;
        if(track != null) {
            radius = track.getArea();
        }
    }

    @Override
    public void initGui() {
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        controlList.add(new GuiButton(0, w + 53, h + 50, 30, 20, "-1"));
        controlList.add(new GuiButton(1, w + 93, h + 50, 30, 20, "+1"));
    }

    @Override
    protected void drawExtras(int x, int y, float f) {
        if(track != null) {
            GuiTools.drawCenteredString(fontRenderer, RailcraftLanguage.translate("gui.track.embarking.radius") + " = " + radius, 25);
        }
    }

    @Override
    protected void actionPerformed(GuiButton guibutton) {
        if(guibutton.id == 0) {
            radius--;
        }
        if(guibutton.id == 1) {
            radius++;
        }

        radius = (byte)Math.max(TrackEmbarking.MIN_AREA, radius);
        radius = (byte)Math.min(TrackEmbarking.MAX_AREA, radius);
    }

    @Override
    public void onGuiClosed() {
        track.setArea(radius);
        if(Game.isNotHost(track.getWorld())) {
            PacketGuiReturn pkt = new PacketGuiReturn((IGuiReturnHandler)track.tileEntity);
            pkt.sendPacket();
        }
    }
}
