package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.tileentity.TileEntity;
import railcraft.client.gui.buttons.GuiToggleButton;
import railcraft.common.blocks.machine.gamma.TileEnergyUnloader;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerEnergyLoader;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiUnloaderEnergy extends TileGui {

    private static final String LABEL = "Energy Unloader";
    private static final String BUTTON1 = "Wait till Empty";
    private TileEnergyUnloader tile;

    public GuiUnloaderEnergy(InventoryPlayer inv, TileEnergyUnloader tile) {
        super(tile, new ContainerEnergyLoader(inv, tile), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_energy_loader.png");
        this.tile = tile;
    }

    @Override
    public void initGui() {
        super.initGui();
        if (tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        controlList.add(new GuiToggleButton(0, w + 75, h + 18, 70, 20, BUTTON1, tile.waitTillEmpty()));
    }

    @Override
    protected void actionPerformed(GuiButton guibutton) {
        if (tile == null) {
            return;
        }
        if (guibutton.id == 0) {
            tile.setWaitTillEmpty(!tile.waitTillEmpty());
            ((GuiToggleButton) guibutton).active = tile.waitTillEmpty();
        }
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY) {
        int sWidth = fontRenderer.getStringWidth(tile.getInvName());
        int sPos = xSize / 2 - sWidth / 2;
        fontRenderer.drawString(LABEL, sPos, 6, 0x404040);

        fontRenderer.drawString(Integer.toString(tile.getEnergy()), 30, 55, 0x404040);

        StringBuilder capacity = new StringBuilder("/").append(tile.getCapacity());
        fontRenderer.drawString(capacity.toString(), 28, 65, 0x404040);

        fontRenderer.drawString("Rate: " + tile.getTransferRate(), 90, 67, 0x404040);
    }

    @Override
    protected void drawGuiContainerBackgroundLayer(float f, int i, int j) {
        super.drawGuiContainerBackgroundLayer(f, i, j);
        int x = (width - xSize) / 2;
        int y = (height - ySize) / 2;
        if (tile.getEnergy() > 0) {
            int energy = tile.getEnergyBarScaled(24);
            drawTexturedModalRect(x + 31, y + 34, 176, 14, energy, 17);
        }
    }

    @Override
    public void onGuiClosed() {
        if (Game.isNotHost(tile.getWorld())) {
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }

    @Override
    public void updateScreen() {
        super.updateScreen();
        TileEntity t = tile.getWorld().getBlockTileEntity(tile.getX(), tile.getY(), tile.getZ());
        if (t instanceof TileEnergyUnloader) {
            tile = (TileEnergyUnloader) t;
        } else {
            mc.thePlayer.closeScreen();
        }
    }
}
