package railcraft.client.gui;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.InventoryPlayer;
import railcraft.client.gui.buttons.GuiToggleButton;
import railcraft.client.gui.buttons.GuiToggleButtonSmall;
import railcraft.common.blocks.machine.gamma.TileLiquidUnloader;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.containers.ContainerLiquidUnloader;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketGuiReturn;

public class GuiUnloaderLiquid extends TileGui {

    private static final String FILTER_LABEL = RailcraftLanguage.translate("gui.filters");
    private final TileLiquidUnloader tile;
    private boolean waitTillEmpty;

    public GuiUnloaderLiquid(InventoryPlayer inv, TileLiquidUnloader tile) {
        super(tile, new ContainerLiquidUnloader(inv, tile), RailcraftConstants.GUI_TEXTURE_FOLDER + "gui_liquid_unloader.png");
        this.tile = tile;
        waitTillEmpty = tile.waitTillEmpty();
    }

    @Override
    public void initGui() {
        super.initGui();
        if (tile == null) {
            return;
        }
        controlList.clear();
        int w = (width - xSize) / 2;
        int h = (height - ySize) / 2;
        String button0 = RailcraftLanguage.translate("gui.liquid.unloader.empty");
        controlList.add(new GuiToggleButtonSmall(0, w + 40, h + 60, 40, button0, waitTillEmpty));
    }

    @Override
    protected void drawGuiContainerForegroundLayer(int mouseX, int mouseY) {
        int sWidth = fontRenderer.getStringWidth(tile.getInvName());
        int sPos = xSize / 2 - sWidth / 2;
        fontRenderer.drawString(tile.getInvName(), sPos, 6, 0x404040);
        fontRenderer.drawString(FILTER_LABEL, 50, 25, 0x404040);
    }

    @Override
    protected void actionPerformed(GuiButton guibutton) {
        if (tile == null) {
            return;
        }
        if (guibutton.id == 0) {
            waitTillEmpty = !waitTillEmpty;
            ((GuiToggleButton) guibutton).active = waitTillEmpty;
        }
    }

    @Override
    public void onGuiClosed() {
        tile.setWaitTillEmpty(waitTillEmpty);
        if (Game.isNotHost(tile.getWorld())) {
            PacketGuiReturn pkt = new PacketGuiReturn(tile);
            PacketDispatcher.sendPacketToServer(pkt.getPacket());
        }
    }
}
