package railcraft.client.render;

import cpw.mods.fml.client.registry.ISimpleBlockRenderingHandler;
import java.util.HashMap;
import java.util.Map;
import net.minecraft.block.Block;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraftforge.client.IItemRenderer;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class BlockRenderer implements ISimpleBlockRenderingHandler, IInvRenderer
{

    private Map<Integer, IBlockRenderer> blockRenderers = new HashMap<Integer, IBlockRenderer>();
    private Map<Integer, IInvRenderer> itemRenderers = new HashMap<Integer, IInvRenderer>();
    private ICombinedRenderer defaultRenderer = new DefaultRenderer();
    private final ItemRenderer itemRenderer;
    private final Block block;

    public BlockRenderer(Block block) {
        this.block = block;
        itemRenderer = new ItemRenderer(this);
    }

    public void addCombinedRenderer(int meta, ICombinedRenderer renderer) {
        blockRenderers.put(meta, renderer);
        itemRenderers.put(meta, renderer);
    }

    public void addBlockRenderer(int meta, IBlockRenderer renderer) {
        blockRenderers.put(meta, renderer);
    }

    public void addItemRenderer(int meta, IInvRenderer renderer) {
        itemRenderers.put(meta, renderer);
    }

    public void setDefaultRenderer(ICombinedRenderer renderer) {
        defaultRenderer = renderer;
    }

    public IItemRenderer getItemRenderer() {
        return itemRenderer;
    }

    @Override
    public void renderInventoryBlock(Block block, int metadata, int modelID, RenderBlocks renderer) {
    }

    @Override
    public boolean renderWorldBlock(IBlockAccess world, int x, int y, int z, Block block, int modelId, RenderBlocks renderblocks) {
        if(!blockRenderers.isEmpty()) {
            int meta = world.getBlockMetadata(x, y, z);
            IBlockRenderer renderer = blockRenderers.get(meta);
            if(renderer != null) {
                renderer.renderBlock(renderblocks, world, x, y, z, block);
                return true;
            }
        }
        defaultRenderer.renderBlock(renderblocks, world, x, y, z, block);
        return true;
    }

    @Override
    public void renderItem(RenderBlocks renderBlocks, ItemStack item, ItemRenderType renderType) {
        IInvRenderer renderer = itemRenderers.get(item.getItemDamage());
        if(renderer != null) {
            renderer.renderItem(renderBlocks, item, renderType);
            return;
        }
        defaultRenderer.renderItem(renderBlocks, item, renderType);
    }

    @Override
    public boolean shouldRender3DInInventory() {
        return false;
    }

    @Override
    public int getRenderId() {
        return block.getRenderType();
    }

    public Block getBlock() {
        return block;
    }

    protected class DoNothingRenderer implements IBlockRenderer
    {

        @Override
        public void renderBlock(RenderBlocks renderblocks, IBlockAccess world, int i, int j, int k, Block block) {
        }
    }

    private class DefaultRenderer implements ICombinedRenderer
    {

        @Override
        public void renderItem(RenderBlocks renderblocks, ItemStack item, ItemRenderType renderType) {
            int meta = item.getItemDamage();
            block.setBlockBounds(0, 0, 0, 1, 1, 1);
            RenderTools.renderBlockOnInventory(renderblocks, block, meta, 1);
//
//            block.setBlockBoundsForItemRender();
//            renderblocks.updateCustomBlockBounds(block);
//
//            if(renderblocks.useInventoryTint) {
//                int color = block.getRenderColor(meta);
//
//                float r = (float)(color >> 16 & 255) / 255.0F;
//                float g = (float)(color >> 8 & 255) / 255.0F;
//                float b = (float)(color & 255) / 255.0F;
//                GL11.glColor4f(r, g, b, 1.0F);
//            }
//
//            GL11.glTranslatef(-0.5F, -0.5F, -0.5F);
//            Tessellator tess = Tessellator.instance;
//            tess.startDrawingQuads();
//            tess.setNormal(0.0F, -1.0F, 0.0F);
//            renderblocks.renderBottomFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(0, meta));
//            tess.draw();
//            tess.startDrawingQuads();
//            tess.setNormal(0.0F, 1.0F, 0.0F);
//            renderblocks.renderTopFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(1, meta));
//            tess.draw();
//            tess.startDrawingQuads();
//            tess.setNormal(0.0F, 0.0F, -1.0F);
//            renderblocks.renderEastFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(2, meta));
//            tess.draw();
//            tess.startDrawingQuads();
//            tess.setNormal(0.0F, 0.0F, 1.0F);
//            renderblocks.renderWestFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(3, meta));
//            tess.draw();
//            tess.startDrawingQuads();
//            tess.setNormal(-1.0F, 0.0F, 0.0F);
//            renderblocks.renderNorthFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(4, meta));
//            tess.draw();
//            tess.startDrawingQuads();
//            tess.setNormal(1.0F, 0.0F, 0.0F);
//            renderblocks.renderSouthFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(5, meta));
//            tess.draw();
//            GL11.glTranslatef(0.5F, 0.5F, 0.5F);
        }

        @Override
        public void renderBlock(RenderBlocks renderblocks, IBlockAccess world, int x, int y, int z, Block block) {
            block.setBlockBounds(0, 0, 0, 1, 1, 1);
            RenderTools.renderStandardBlock(renderblocks, block, x, y, z);
        }
    }
}
