package railcraft.client.render;

import java.util.Map;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.GLAllocation;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.client.ForgeHooksClient;
import net.minecraftforge.liquids.LiquidStack;
import org.lwjgl.opengl.GL11;
import railcraft.common.util.inventory.ItemStackMap;
import railcraft.common.util.misc.FakeBlockRenderInfo;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class LiquidRenderer {

    private static Map<ItemStack, int[]> stage = new ItemStackMap<int[]>();
    public static final int DISPLAY_STAGES = 100;
    private static final FakeBlockRenderInfo liquidBlock = new FakeBlockRenderInfo();

    public static int bindLiquidTexture(LiquidStack liquid) {
        if (liquid == null || liquid.amount <= 0 || liquid.itemID <= 0) {
            return -1;
        }
        String textureFile;
        int textureIndex;

        if (liquid.itemID < Block.blocksList.length && Block.blocksList[liquid.itemID] != null) {
            textureFile = Block.blocksList[liquid.itemID].getTextureFile();
            textureIndex = Block.blocksList[liquid.itemID].getBlockTextureFromSideAndMetadata(1, liquid.itemMeta);
        } else if (Item.itemsList[liquid.itemID] != null) {
            textureFile = Item.itemsList[liquid.itemID].getTextureFile();
            textureIndex = Item.itemsList[liquid.itemID].getIconFromDamage(liquid.itemMeta);
        } else {
            return -1;
        }

        ForgeHooksClient.bindTexture(textureFile, 0);

        return textureIndex;
    }

    public static int[] getLiquidDisplayLists(LiquidStack liquid) {
        if (liquid == null) {
            return null;
        }
        ItemStack stack = liquid.asItemStack();
        int[] diplayLists = stage.get(stack);
        if (diplayLists != null) {
            return diplayLists;
        }

        diplayLists = new int[DISPLAY_STAGES];

        if (liquid.itemID < Block.blocksList.length && Block.blocksList[liquid.itemID] != null) {
            liquidBlock.texture[0] = Block.blocksList[liquid.itemID].getBlockTextureFromSideAndMetadata(1, liquid.itemMeta);
        } else if (Item.itemsList[liquid.itemID] != null) {
            liquidBlock.texture[0] = stack.getIconIndex();
        } else {
            return null;
        }

        stage.put(stack, diplayLists);

        GL11.glDisable(GL11.GL_LIGHTING);
        GL11.glDisable(GL11.GL_BLEND);
        int color = stack.getItem().getColorFromItemStack(stack, 0);
        float c1 = (float) (color >> 16 & 255) / 255.0F;
        float c2 = (float) (color >> 8 & 255) / 255.0F;
        float c3 = (float) (color & 255) / 255.0F;
        GL11.glColor4f(c1, c2, c3, 1);
        for (int s = 0; s < DISPLAY_STAGES; ++s) {
            diplayLists[s] = GLAllocation.generateDisplayLists(1);
            GL11.glNewList(diplayLists[s], 4864 /*GL_COMPILE*/);

            liquidBlock.minX = 0.01f;
            liquidBlock.minY = 0;
            liquidBlock.minZ = 0.01f;

            liquidBlock.maxX = 0.99f;
            liquidBlock.maxY = (float) s / (float) DISPLAY_STAGES;
            liquidBlock.maxZ = 0.99f;

            RenderFakeBlock.renderBlockForEntity(liquidBlock, null, 0, 0, 0, false, true);

            GL11.glEndList();
        }

        GL11.glColor4f(1, 1, 1, 1);
        GL11.glEnable(GL11.GL_BLEND);
        GL11.glEnable(GL11.GL_LIGHTING);

        return diplayLists;
    }
}
