package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.item.ItemStack;
import net.minecraft.world.IBlockAccess;
import net.minecraftforge.client.ForgeHooksClient;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;
import org.lwjgl.opengl.GL11;
import railcraft.common.blocks.ore.BlockOre;
import railcraft.common.blocks.ore.EnumOre;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class RenderBlockOre extends BlockRenderer {

    public RenderBlockOre() {
        super(BlockOre.getBlock());
    }

    @Override
    public boolean renderWorldBlock(IBlockAccess world, int x, int y, int z, Block block, int modelId, RenderBlocks renderblocks) {
        block.setBlockBounds(0, 0, 0, 1, 1, 1);
        if (renderblocks.overrideBlockTexture >= 0) {
            BlockOre.renderPass = 0;
            RenderTools.renderStandardBlock(renderblocks, block, x, y, z);
        } else {
            int meta = world.getBlockMetadata(x, y, z);
            ForgeHooksClient.bindTexture("/terrain.png", 0);
            BlockOre.renderPass = 0;
            RenderTools.renderStandardBlock(renderblocks, block, x, y, z);
            ForgeHooksClient.bindTexture(getBlock().getTextureFile(), 0);
            BlockOre.renderPass = 1;
            RenderTools.renderStandardBlock(renderblocks, block, x, y, z);
        }
        return true;
    }

    @Override
    public void renderItem(RenderBlocks renderBlocks, ItemStack item, ItemRenderType renderType) {
        ForgeHooksClient.bindTexture("/terrain.png", 0);
        BlockOre.renderPass = 0;
        renderItem(renderBlocks, item, getBlock().getBlockTextureFromSideAndMetadata(0, item.getItemDamage()));
        ForgeHooksClient.bindTexture(getBlock().getTextureFile(), 0);
        BlockOre.renderPass = 1;
        renderItem(renderBlocks, item, getBlock().getBlockTextureFromSideAndMetadata(0, item.getItemDamage()));

    }

    private void renderItem(RenderBlocks renderBlocks, ItemStack item, int texture) {
        int meta = item.getItemDamage();

        Block block = getBlock();
        block.setBlockBoundsForItemRender();
        renderBlocks.setRenderBoundsFromBlock(block);

        if (renderBlocks.useInventoryTint) {
            int color = block.getRenderColor(meta);

            float r = (float) (color >> 16 & 255) / 255.0F;
            float g = (float) (color >> 8 & 255) / 255.0F;
            float b = (float) (color & 255) / 255.0F;
            GL11.glColor4f(r, g, b, 1.0F);
        }

        GL11.glTranslatef(-0.5F, -0.5F, -0.5F);
        Tessellator tess = Tessellator.instance;
        tess.startDrawingQuads();
        tess.setNormal(0.0F, -1.0F, 0.0F);
        renderBlocks.renderBottomFace(block, 0.0D, 0.0D, 0.0D, texture);
        tess.draw();
        tess.startDrawingQuads();
        tess.setNormal(0.0F, 1.0F, 0.0F);
        renderBlocks.renderTopFace(block, 0.0D, 0.0D, 0.0D, texture);
        tess.draw();
        tess.startDrawingQuads();
        tess.setNormal(0.0F, 0.0F, -1.0F);
        renderBlocks.renderEastFace(block, 0.0D, 0.0D, 0.0D, texture);
        tess.draw();
        tess.startDrawingQuads();
        tess.setNormal(0.0F, 0.0F, 1.0F);
        renderBlocks.renderWestFace(block, 0.0D, 0.0D, 0.0D, texture);
        tess.draw();
        tess.startDrawingQuads();
        tess.setNormal(-1.0F, 0.0F, 0.0F);
        renderBlocks.renderNorthFace(block, 0.0D, 0.0D, 0.0D, texture);
        tess.draw();
        tess.startDrawingQuads();
        tess.setNormal(1.0F, 0.0F, 0.0F);
        renderBlocks.renderSouthFace(block, 0.0D, 0.0D, 0.0D, texture);
        tess.draw();
        GL11.glTranslatef(0.5F, 0.5F, 0.5F);
    }
}
