package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.world.World;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.signals.SignalAspect;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.aesthetics.post.BlockPost;
import railcraft.common.blocks.signals.BlockSignal;
import railcraft.common.blocks.signals.EnumSignal;
import railcraft.common.blocks.signals.IDualHeadSignal;
import railcraft.common.blocks.signals.Signals;
import railcraft.common.util.misc.Game;

public class RenderDualHeadSignal implements ICombinedRenderer {

    @Override
    public void renderBlock(RenderBlocks renderblocks, IBlockAccess iBlockAccess, int i, int j, int k, Block block) {
        BlockSignal blockStructure = (BlockSignal) block;
        IDualHeadSignal tile = (IDualHeadSignal) iBlockAccess.getBlockTileEntity(i, j, k);
        EnumSignal signalType = tile.getSignalType();
        float pix = RenderTools.PIXEL;
        float f = 3.0F * pix;
        float f1 = 13.0F * pix;
        blockStructure.setBlockBounds(f, 0.0F, f, f1, 1.0F, f1);
        signalType.setUseAltTextures(true);
        signalType.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 16);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        if (tile.getTopAspect().isBlinkAspect() && !SignalAspect.isBlinkOn()) {
            signalType.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 1 + SignalAspect.OFF.getTextureOffset());
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        } else {
            signalType.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 1 + tile.getTopAspect().getTextureOffset());
            RenderTools.renderBlockSideWithBrightness(renderblocks, iBlockAccess, block, i, j, k, tile.getFacing().ordinal(), RenderTools.SIGNAL_BRIGHTNESS);
        }
        if (tile.getBottomAspect().isBlinkAspect() && !SignalAspect.isBlinkOn()) {
            signalType.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 17 + SignalAspect.OFF.getTextureOffset());
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        } else {
            signalType.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 17 + tile.getBottomAspect().getTextureOffset());
            RenderTools.renderBlockSideWithBrightness(renderblocks, iBlockAccess, block, i, j, k, tile.getFacing().ordinal(), RenderTools.SIGNAL_BRIGHTNESS);
        }
        signalType.setUseAltTextures(false);
        if (tile.getFacing().ordinal() == 2 || tile.getFacing().ordinal() == 3) {
            float temp = 13.0F * pix;
            if (tile.getFacing().ordinal() == 2) {
                temp = pix;
            }
            for (int ii = 0; ii < 2; ii++) {
                blockStructure.setBlockBounds(6.0F * pix, 7.0F * pix + ii * 6.0F * pix, temp, 10.0F * pix, 8.0F * pix + ii * 6.0F * pix, temp + 2.0F * pix);
                RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
                blockStructure.setBlockBounds(5.0F * pix, 4.0F * pix + ii * 6.0F * pix, temp, 6.0F * pix, 8.0F * pix + ii * 6.0F * pix, temp + 2.0F * pix);
                RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
                blockStructure.setBlockBounds(10.0F * pix, 4.0F * pix + ii * 6.0F * pix, temp, 11.0F * pix, 8.0F * pix + ii * 6.0F * pix, temp + 2.0F * pix);
                RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
            }
        } else if (tile.getFacing().ordinal() == 4 || tile.getFacing().ordinal() == 5) {
            float temp = 13.0F * pix;
            if (tile.getFacing().ordinal() == 4) {
                temp = pix;
            }
            for (int ii = 0; ii < 2; ii++) {
                blockStructure.setBlockBounds(temp, 7.0F * pix + ii * 6.0F * pix, 6.0F * pix, temp + 2.0F * pix, 8.0F * pix + ii * 6.0F * pix, 10.0F * pix);
                RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
                blockStructure.setBlockBounds(temp, 4.0F * pix + ii * 6.0F * pix, 5.0F * pix, temp + 2.0F * pix, 8.0F * pix + ii * 6.0F * pix, 6.0F * pix);
                RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
                blockStructure.setBlockBounds(temp, 4.0F * pix + ii * 6.0F * pix, 10.0F * pix, temp + 2.0F * pix, 8.0F * pix + ii * 6.0F * pix, 11.0F * pix);
                RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
            }
        }
        World world = Game.getWorld();
        boolean flag1 = false;
        boolean flag2 = false;
        if (BlockPost.connectPostAt(world, i - 1, j, k, ForgeDirection.EAST) || BlockPost.connectPostAt(world, i + 1, j, k, ForgeDirection.WEST)) {
            flag1 = true;
        }
        if (BlockPost.connectPostAt(world, i, j, k - 1, ForgeDirection.SOUTH) || BlockPost.connectPostAt(world, i, j, k + 1, ForgeDirection.NORTH)) {
            flag2 = true;
        }
        boolean flag3 = BlockPost.connectPostAt(world, i - 1, j, k, ForgeDirection.EAST);
        boolean flag4 = BlockPost.connectPostAt(world, i + 1, j, k, ForgeDirection.WEST);
        boolean flag5 = BlockPost.connectPostAt(world, i, j, k - 1, ForgeDirection.SOUTH);
        boolean flag6 = BlockPost.connectPostAt(world, i, j, k + 1, ForgeDirection.NORTH);
        if (!flag1 && !flag2) {
            flag1 = true;
        }
        f = 0.4375F;
        f1 = 0.5625F;
        float f2 = 11 * pix;
        float f3 = 14 * pix;
        float f4 = flag3 ? 0.0F : f;
        float f5 = flag4 ? 1.0F : f1;
        float f6 = flag5 ? 0.0F : f;
        float f7 = flag6 ? 1.0F : f1;
        signalType.setUseAltTextures(true);
        signalType.setAltTextureIndex(Signals.POST_TEXTURE);
        if (flag1) {
            blockStructure.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        if (flag2) {
            blockStructure.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        f2 = 5 * pix;
        f3 = 8 * pix;
        if (flag1) {
            blockStructure.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        if (flag2) {
            blockStructure.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        signalType.setUseAltTextures(false);
        blockStructure.setBlockBounds(0, 0, 0, 1, 1, 1);
    }

    @Override
    public void renderItem(RenderBlocks renderblocks, ItemStack item, ItemRenderType renderType) {
        EnumSignal signalType = EnumSignal.fromId(item.getItemDamage());
        float pix = RenderTools.PIXEL;
        float f = 3.0F * pix;
        float f1 = 13.0F * pix;
        BlockSignal block = (BlockSignal) RailcraftBlocks.getBlockSignal();
        block.setBlockBounds(f, 0.0F, f, f1, 1.0F, f1);
        signalType.setUseAltTextures(true);
        signalType.setUseOffsetTextureOnSide(3, 16);
        RenderTools.renderBlockOnInventory(renderblocks, block, signalType.ordinal(), 1);
        signalType.setUseOffsetTextureOnSide(3, signalType == EnumSignal.DUAL_HEAD_BLOCK_SIGNAL ? 1 : 3);
        RenderTools.renderBlockOnInventory(renderblocks, block, signalType.ordinal(), 1, 3);
        signalType.setUseOffsetTextureOnSide(3, signalType == EnumSignal.DUAL_HEAD_BLOCK_SIGNAL ? 16 + 3 : 16 + 1);
        RenderTools.renderBlockOnInventory(renderblocks, block, signalType.ordinal(), 1, 3);
        signalType.setUseAltTextures(false);
        float temp = 13.0F * pix;
        for (int ii = 0; ii < 2; ii++) {
            block.setBlockBounds(6.0F * pix, 7.0F * pix + ii * 6.0F * pix, temp, 10.0F * pix, 8.0F * pix + ii * 6.0F * pix, temp + 2.0F * pix);
            RenderTools.renderBlockOnInventory(renderblocks, block, signalType.ordinal(), 1);
            block.setBlockBounds(5.0F * pix, 4.0F * pix + ii * 6.0F * pix, temp, 6.0F * pix, 8.0F * pix + ii * 6.0F * pix, temp + 2.0F * pix);
            RenderTools.renderBlockOnInventory(renderblocks, block, signalType.ordinal(), 1);
            block.setBlockBounds(10.0F * pix, 4.0F * pix + ii * 6.0F * pix, temp, 11.0F * pix, 8.0F * pix + ii * 6.0F * pix, temp + 2.0F * pix);
            RenderTools.renderBlockOnInventory(renderblocks, block, signalType.ordinal(), 1);
        }
        block.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    }
}
