package railcraft.client.render;

import net.minecraft.world.IBlockAccess;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import railcraft.common.blocks.aesthetics.cube.BlockCube;
import railcraft.common.util.misc.FakeBlockRenderInfo;

public class RenderFakeBlock {

    public static void renderBlockForEntity(FakeBlockRenderInfo info, IBlockAccess blockAccess, int i, int j, int k, boolean doLight, boolean doTessellating) {
        renderBlock(info, blockAccess, -0.5, -0.5, -0.5, i, j, k, doLight, doTessellating);
    }

    public static void renderAsBlock(FakeBlockRenderInfo info, RenderBlocks render, IBlockAccess blockAccess, double x, double y, double z) {
        BlockCube block = BlockCube.getBlock();
        if (block != null) {
            block.setTextureOveride(info);
            block.setBlockBounds(info.minX, info.minY, info.minZ, info.maxX, info.maxY, info.maxZ);
            RenderTools.renderStandardBlock(render, block, (int) x, (int) y, (int) z);
            block.setBlockBounds(0, 0, 0, 1, 1, 1);
            block.setTextureOveride(null);
        } else {
            renderBlock(info, blockAccess, x, y, z, true, false);
        }
    }

    public static void renderBlock(FakeBlockRenderInfo info, IBlockAccess blockAccess, double x, double y, double z, boolean doLight, boolean doTessellating) {
        renderBlock(info, blockAccess, x, y, z, (int) x, (int) y, (int) z, doLight, doTessellating);
    }

    public static void renderBlock(FakeBlockRenderInfo info, IBlockAccess blockAccess, double x, double y, double z, int lightX, int lightY, int lightZ, boolean doLight, boolean doTessellating) {
        float lightBottom = 0.5F;
        float lightTop = 1.0F;
        float lightEastWest = 0.8F;
        float lightNorthSouth = 0.6F;

        Tessellator tessellator = Tessellator.instance;

        if (blockAccess == null) {
            doLight = false;
        }

        if (doTessellating && !tessellator.isDrawing) {
            tessellator.startDrawingQuads();
        }

        if (info.brightness >= 0) {
            tessellator.setBrightness(info.brightness);
        }

        float light = 0;

        if (doLight) {
            if (info.light < 0) {
                light = info.template.getBlockBrightness(blockAccess, (int) lightX, (int) lightY, (int) lightZ);
                light = light + ((1.0f - light) * 0.4f);
            } else {
                light = info.light;
            }
            int brightness = 0;
            if (info.brightness < 0) {
                brightness = info.template.getMixedBrightnessForBlock(blockAccess, lightX, lightY, lightZ);
            } else {
                brightness = info.brightness;
            }
            tessellator.setBrightness(brightness);
            tessellator.setColorOpaque_F(lightBottom * light, lightBottom * light, lightBottom * light);
        }

        if (info.renderBottom) {
            renderBottomFace(info, x, y, z, info.getBlockTextureFromSide(0));
        }

        if (doLight) {
            tessellator.setColorOpaque_F(lightTop * light, lightTop * light, lightTop * light);
        }

        if (info.renderTop) {
            renderTopFace(info, x, y, z, info.getBlockTextureFromSide(1));
        }

        if (doLight) {
            tessellator.setColorOpaque_F(lightEastWest * light, lightEastWest * light, lightEastWest * light);
        }

        if (info.renderWest) {
            renderEastFace(info, x, y, z, info.getBlockTextureFromSide(2));
        }

        if (doLight) {
            tessellator.setColorOpaque_F(lightEastWest * light, lightEastWest * light, lightEastWest * light);
        }

        if (info.renderEast) {
            renderWestFace(info, x, y, z, info.getBlockTextureFromSide(3));
        }

        if (doLight) {
            tessellator.setColorOpaque_F(lightNorthSouth * light, lightNorthSouth * light, lightNorthSouth * light);
        }

        if (info.renderNorth) {
            renderNorthFace(info, x, y, z, info.getBlockTextureFromSide(4));
        }

        if (doLight) {
            tessellator.setColorOpaque_F(lightNorthSouth * light, lightNorthSouth * light, lightNorthSouth * light);
        }

        if (info.renderSouth) {
            renderSouthFace(info, x, y, z, info.getBlockTextureFromSide(5));
        }

        if (doTessellating && tessellator.isDrawing) {
            tessellator.draw();
        }
    }

    public static void renderBottomFace(FakeBlockRenderInfo info, double d, double d1, double d2, int i) {
        Tessellator tessellator = Tessellator.instance;

        int j = (i & 0xf) << 4;
        int k = i & 0xf0;
        double d3 = (j + info.minX * 16D) / 256D;
        double d4 = ((j + info.maxX * 16D) - 0.01D) / 256D;
        double d5 = (k + info.minZ * 16D) / 256D;
        double d6 = ((k + info.maxZ * 16D) - 0.01D) / 256D;
        if (info.minX < 0.0D || info.maxX > 1.0D) {
            d3 = (j + 0.0F) / 256F;
            d4 = (j + 15.99F) / 256F;
        }
        if (info.minZ < 0.0D || info.maxZ > 1.0D) {
            d5 = (k + 0.0F) / 256F;
            d6 = (k + 15.99F) / 256F;
        }
        double d7 = d + info.minX;
        double d8 = d + info.maxX;
        double d9 = d1 + info.minY;
        double d10 = d2 + info.minZ;
        double d11 = d2 + info.maxZ;

        tessellator.addVertexWithUV(d7, d9, d11, d3, d6);
        tessellator.addVertexWithUV(d7, d9, d10, d3, d5);
        tessellator.addVertexWithUV(d8, d9, d10, d4, d5);
        tessellator.addVertexWithUV(d8, d9, d11, d4, d6);
    }

    public static void renderTopFace(FakeBlockRenderInfo info, double d, double d1, double d2, int i) {
        Tessellator tessellator = Tessellator.instance;

        int j = (i & 0xf) << 4;
        int k = i & 0xf0;
        double d3 = (j + info.minX * 16D) / 256D;
        double d4 = ((j + info.maxX * 16D) - 0.01D) / 256D;
        double d5 = (k + info.minZ * 16D) / 256D;
        double d6 = ((k + info.maxZ * 16D) - 0.01D) / 256D;
        if (info.minX < 0.0D || info.maxX > 1.0D) {
            d3 = (j + 0.0F) / 256F;
            d4 = (j + 15.99F) / 256F;
        }
        if (info.minZ < 0.0D || info.maxZ > 1.0D) {
            d5 = (k + 0.0F) / 256F;
            d6 = (k + 15.99F) / 256F;
        }
        double d7 = d + info.minX;
        double d8 = d + info.maxX;
        double d9 = d1 + info.maxY;
        double d10 = d2 + info.minZ;
        double d11 = d2 + info.maxZ;

        tessellator.addVertexWithUV(d8, d9, d11, d4, d6);
        tessellator.addVertexWithUV(d8, d9, d10, d4, d5);
        tessellator.addVertexWithUV(d7, d9, d10, d3, d5);
        tessellator.addVertexWithUV(d7, d9, d11, d3, d6);
    }

    public static void renderEastFace(FakeBlockRenderInfo info, double d, double d1, double d2, int i) {
        Tessellator tessellator = Tessellator.instance;

        int j = (i & 0xf) << 4;
        int k = i & 0xf0;
        double d3 = (j + info.minX * 16D) / 256D;
        double d4 = ((j + info.maxX * 16D) - 0.01D) / 256D;
        double d5 = (k + info.minY * 16D) / 256D;
        double d6 = ((k + info.maxY * 16D) - 0.01D) / 256D;

        if (info.minX < 0.0D || info.maxX > 1.0D) {
            d3 = (j + 0.0F) / 256F;
            d4 = (j + 15.99F) / 256F;
        }
        if (info.minY < 0.0D || info.maxY > 1.0D) {
            d5 = (k + 0.0F) / 256F;
            d6 = (k + 15.99F) / 256F;
        }
        double d8 = d + info.minX;
        double d9 = d + info.maxX;
        double d10 = d1 + info.minY;
        double d11 = d1 + info.maxY;
        double d12 = d2 + info.minZ;

        tessellator.addVertexWithUV(d8, d11, d12, d4, d5);
        tessellator.addVertexWithUV(d9, d11, d12, d3, d5);
        tessellator.addVertexWithUV(d9, d10, d12, d3, d6);
        tessellator.addVertexWithUV(d8, d10, d12, d4, d6);
    }

    public static void renderWestFace(FakeBlockRenderInfo info, double d, double d1, double d2, int i) {
        Tessellator tessellator = Tessellator.instance;

        int j = (i & 0xf) << 4;
        int k = i & 0xf0;
        double d3 = (j + info.minX * 16D) / 256D;
        double d4 = ((j + info.maxX * 16D) - 0.01D) / 256D;
        double d5 = (k + info.minY * 16D) / 256D;
        double d6 = ((k + info.maxY * 16D) - 0.01D) / 256D;

        if (info.minX < 0.0D || info.maxX > 1.0D) {
            d3 = (j + 0.0F) / 256F;
            d4 = (j + 15.99F) / 256F;
        }
        if (info.minY < 0.0D || info.maxY > 1.0D) {
            d5 = (k + 0.0F) / 256F;
            d6 = (k + 15.99F) / 256F;
        }
        double d8 = d + info.minX;
        double d9 = d + info.maxX;
        double d10 = d1 + info.minY;
        double d11 = d1 + info.maxY;
        double d12 = d2 + info.maxZ;

        tessellator.addVertexWithUV(d8, d11, d12, d3, d5);
        tessellator.addVertexWithUV(d8, d10, d12, d3, d6);
        tessellator.addVertexWithUV(d9, d10, d12, d4, d6);
        tessellator.addVertexWithUV(d9, d11, d12, d4, d5);
    }

    public static void renderNorthFace(FakeBlockRenderInfo info, double d, double d1, double d2, int i) {
        Tessellator tessellator = Tessellator.instance;

        int j = (i & 0xf) << 4;
        int k = i & 0xf0;
        double d3 = (j + info.minZ * 16D) / 256D;
        double d4 = ((j + info.maxZ * 16D) - 0.01D) / 256D;
        double d5 = (k + info.minY * 16D) / 256D;
        double d6 = ((k + info.maxY * 16D) - 0.01D) / 256D;

        if (info.minZ < 0.0D || info.maxZ > 1.0D) {
            d3 = (j + 0.0F) / 256F;
            d4 = (j + 15.99F) / 256F;
        }
        if (info.minY < 0.0D || info.maxY > 1.0D) {
            d5 = (k + 0.0F) / 256F;
            d6 = (k + 15.99F) / 256F;
        }
        double d8 = d + info.minX;
        double d9 = d1 + info.minY;
        double d10 = d1 + info.maxY;
        double d11 = d2 + info.minZ;
        double d12 = d2 + info.maxZ;

        tessellator.addVertexWithUV(d8, d10, d12, d4, d5);
        tessellator.addVertexWithUV(d8, d10, d11, d3, d5);
        tessellator.addVertexWithUV(d8, d9, d11, d3, d6);
        tessellator.addVertexWithUV(d8, d9, d12, d4, d6);
    }

    public static void renderSouthFace(FakeBlockRenderInfo info, double d, double d1, double d2, int i) {
        Tessellator tessellator = Tessellator.instance;

        int j = (i & 0xf) << 4;
        int k = i & 0xf0;
        double d3 = (j + info.minZ * 16D) / 256D;
        double d4 = ((j + info.maxZ * 16D) - 0.01D) / 256D;
        double d5 = (k + info.minY * 16D) / 256D;
        double d6 = ((k + info.maxY * 16D) - 0.01D) / 256D;

        if (info.minZ < 0.0D || info.maxZ > 1.0D) {
            d3 = (j + 0.0F) / 256F;
            d4 = (j + 15.99F) / 256F;
        }
        if (info.minY < 0.0D || info.maxY > 1.0D) {
            d5 = (k + 0.0F) / 256F;
            d6 = (k + 15.99F) / 256F;
        }
        double d8 = d + info.maxX;
        double d9 = d1 + info.minY;
        double d10 = d1 + info.maxY;
        double d11 = d2 + info.minZ;
        double d12 = d2 + info.maxZ;

        tessellator.addVertexWithUV(d8, d9, d12, d3, d6);
        tessellator.addVertexWithUV(d8, d9, d11, d4, d6);
        tessellator.addVertexWithUV(d8, d10, d11, d4, d5);
        tessellator.addVertexWithUV(d8, d10, d12, d3, d5);
    }
}
