package railcraft.client.render;

import net.minecraft.tileentity.TileEntity;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.item.Item;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.LiquidStack;
import org.lwjgl.opengl.GL11;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.beta.TileTankIron;
import railcraft.common.blocks.machine.beta.TileTankIronValve;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.liquids.TankManager;
import railcraft.common.util.misc.FakeBlockRenderInfo;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class RenderIronTank extends TileEntitySpecialRenderer {

    private static final FakeBlockRenderInfo fillBlock = new FakeBlockRenderInfo();

    public RenderIronTank() {
        float pix = RenderTools.PIXEL;
        fillBlock.minX = 5 * pix;
        fillBlock.minZ = 5 * pix;

        fillBlock.maxX = 11 * pix;
        fillBlock.maxZ = 11 * pix;
    }

    private int bindLiquidTexture(LiquidStack liquid) {
        fillBlock.texture = new int[]{liquid.asItemStack().getIconIndex()};
        return LiquidRenderer.bindLiquidTexture(liquid);
    }

    private float getVerticalScaleSide(TileMultiBlock tile) {
        int y = tile.getPatternPositionY();
        if (!RailcraftConfig.allowTankStacking()) {
            y--;
        }
        return y - RenderTools.PIXEL * 5;
    }

    private int getTankHeight(TileMultiBlock tile) {
        int height = tile.getPattern().getPatternHeight();
        if (!RailcraftConfig.allowTankStacking()) {
            height -= 2;
        }
        return height;
    }

    private void draw() {
        preGL();
        RenderFakeBlock.renderBlockForEntity(fillBlock, null, 0, 0, 0, false, true);
        postGL();
    }

    private void preGL() {
        GL11.glPushAttrib(GL11.GL_ENABLE_BIT);
        GL11.glDisable(GL11.GL_LIGHTING);
        GL11.glDisable(GL11.GL_BLEND);
    }

    private void postGL() {
        GL11.glPopAttrib();
    }

    @Override
    public void renderTileEntityAt(TileEntity tile, double x, double y, double z, float f) {
        if (!((TileMultiBlock) tile).isStructureValid()) {
            return;
        }

        if (tile instanceof TileTankIronValve) {
            TileTankIronValve valve = (TileTankIronValve) tile;

            if (valve.isFilling()) {
                TankManager tMan = valve.getTankManager();
                LiquidStack liquid = null;
                if (tMan != null) {
                    liquid = tMan.getTank(0).getLiquid();
                }
                if (liquid != null && liquid.amount > 0 && Item.itemsList[liquid.itemID] != null) {
                    GL11.glPushMatrix();
                    GL11.glDisable(2896 /*GL_LIGHTING*/);
                    if (valve.getPattern().getPatternMarkerChecked(valve.getPatternPositionX(), valve.getPatternPositionY() - 1, valve.getPatternPositionZ()) == 'A') {

                        bindLiquidTexture(liquid);

                        int height = getTankHeight(valve);
                        float yOffset = height / 2f;
                        float vScale = height - 2f;
                        GL11.glTranslatef((float) x + 0.5F, (float) y + yOffset - height + 1, (float) z + 0.5F);
                        GL11.glScalef(1f, vScale, 1f);

                        draw();
                    } else if (valve.getPattern().getPatternMarkerChecked(valve.getPatternPositionX() - 1, valve.getPatternPositionY(), valve.getPatternPositionZ()) == 'A') {

                        bindLiquidTexture(liquid);

                        float vScale = getVerticalScaleSide(valve);
                        float yOffset = 0.5f - vScale / 2f + RenderTools.PIXEL * 3;
                        GL11.glTranslatef((float) x - 0.5F + RenderTools.PIXEL * 5, (float) y + yOffset, (float) z + 0.5F);
                        GL11.glScalef(1f, vScale, 1f);

                        draw();
                    } else if (valve.getPattern().getPatternMarkerChecked(valve.getPatternPositionX() + 1, valve.getPatternPositionY(), valve.getPatternPositionZ()) == 'A') {

                        bindLiquidTexture(liquid);

                        float vScale = getVerticalScaleSide(valve);
                        float yOffset = 0.5f - vScale / 2f + RenderTools.PIXEL * 3;
                        GL11.glTranslatef((float) x + 1.5F - RenderTools.PIXEL * 5, (float) y + yOffset, (float) z + 0.5F);
                        GL11.glScalef(1f, vScale, 1f);

                        draw();
                    } else if (valve.getPattern().getPatternMarkerChecked(valve.getPatternPositionX(), valve.getPatternPositionY(), valve.getPatternPositionZ() - 1) == 'A') {

                        bindLiquidTexture(liquid);

                        float vScale = getVerticalScaleSide(valve);
                        float yOffset = 0.5f - vScale / 2f + RenderTools.PIXEL * 3;
                        GL11.glTranslatef((float) x + 0.5F, (float) y + yOffset, (float) z - 0.5F + RenderTools.PIXEL * 5);
                        GL11.glScalef(1f, vScale, 1f);

                        draw();
                    } else if (valve.getPattern().getPatternMarkerChecked(valve.getPatternPositionX(), valve.getPatternPositionY(), valve.getPatternPositionZ() + 1) == 'A') {

                        bindLiquidTexture(liquid);

                        float vScale = getVerticalScaleSide(valve);
                        float yOffset = 0.5f - vScale / 2f + RenderTools.PIXEL * 3;
                        GL11.glTranslatef((float) x + 0.5F, (float) y + yOffset, (float) z + 1.5F - RenderTools.PIXEL * 5);
                        GL11.glScalef(1f, vScale, 1f);

                        draw();
                    }
                    GL11.glEnable(2896 /*GL_LIGHTING*/);
                    GL11.glPopMatrix();
                }
            }
        }

        TileTankIron ironTank = (TileTankIron) tile;
        if (!ironTank.isMaster() || ironTank.isInvalid()) {
            return;
        }
        int height = getTankHeight(ironTank);
        float yOffset = height / 2f;
        float vScale = height - 2;
        float hScale = ironTank.getPattern().getPatternWidthX() - 2;

        TankManager tankManager = ironTank.getTankManager();
        if (tankManager == null) {
            return;
        }
        ILiquidTank tank = tankManager.getTank(0);
        if (tank == null) {
            return;
        }



        LiquidStack liquid = tank.getLiquid();
        if (liquid != null && liquid.amount > 0 && Item.itemsList[liquid.itemID] != null) {

            GL11.glPushMatrix();
            GL11.glTranslatef((float) x + 0.5F, (float) y + yOffset + 0.01f, (float) z + 0.5F);
            preGL();
            GL11.glScalef(hScale, vScale, hScale);
            
//        GL11.glScalef(0.999f, 1, 0.999f);
            int[] displayLists = LiquidRenderer.getLiquidDisplayLists(liquid);
            if (displayLists != null) {
                GL11.glPushMatrix();

                bindLiquidTexture(liquid);

                float cap = tank.getCapacity();
                float level = (float) Math.min(liquid.amount, cap) / cap;

                GL11.glCallList(displayLists[(int) (level * (float) (LiquidRenderer.DISPLAY_STAGES - 1))]);

                GL11.glPopMatrix();
            }


            postGL();
            GL11.glPopMatrix();
        }

    }
}
