package railcraft.client.render;

import net.minecraft.tileentity.TileEntity;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraftforge.client.ForgeHooksClient;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import org.lwjgl.opengl.GL11;
import railcraft.common.blocks.RailcraftTileEntity;
import railcraft.common.blocks.machine.gamma.TileLiquidLoader;
import railcraft.common.blocks.machine.gamma.TileLoaderLiquidBase;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.util.misc.FakeBlockRenderInfo;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class RenderLiquidLoader extends TileEntitySpecialRenderer {

    private static final float PIPE_OFFSET = 5 * RenderTools.PIXEL;
    private static final int[] PIPE_TEXTURE = new int[]{228, 228, 212, 212, 212, 212};
    private static final FakeBlockRenderInfo backDrop = new FakeBlockRenderInfo();
    private static final FakeBlockRenderInfo pipe = new FakeBlockRenderInfo();

    public RenderLiquidLoader() {
        backDrop.minX = 0.011f;
        backDrop.minY = 0.01f;
        backDrop.minZ = 0.011f;

        backDrop.maxX = 0.989f;
        backDrop.maxY = 0.99f;
        backDrop.maxZ = 0.989f;

        pipe.texture = PIPE_TEXTURE;

        pipe.minX = PIPE_OFFSET;
        pipe.minZ = PIPE_OFFSET;

        pipe.maxX = 1 - PIPE_OFFSET;
        pipe.maxY = RenderTools.PIXEL;
        pipe.maxZ = 1 - PIPE_OFFSET;
    }

    @Override
    public void renderTileEntityAt(TileEntity tile, double x, double y, double z, float f) {
        TileLoaderLiquidBase base = (TileLoaderLiquidBase) tile;
        GL11.glPushMatrix();
        GL11.glPushAttrib(GL11.GL_ENABLE_BIT);
        GL11.glTranslatef((float) x + 0.5F, (float) y + 0.5F, (float) z + 0.5F);
        GL11.glDisable(GL11.GL_LIGHTING);
        GL11.glDisable(GL11.GL_BLEND);
        GL11.glScalef(1f, 0.6f, 1f);

        ILiquidTank tank = base.getTankManager().getTank(0);

        LiquidStack liquid = tank.getLiquid();
        if (liquid != null && liquid.amount > 0) {
            int[] displayLists = LiquidRenderer.getLiquidDisplayLists(liquid);
            if (displayLists != null) {
                GL11.glPushMatrix();

                if (LiquidRenderer.bindLiquidTexture(liquid) != -1) {

                    float cap = tank.getCapacity();
                    float level = (float) Math.min(liquid.amount, cap) / cap;

                    GL11.glCallList(displayLists[(int) (level * (float) (LiquidRenderer.DISPLAY_STAGES - 1))]);
                }

                GL11.glPopMatrix();
            }
        }
        ForgeHooksClient.bindTexture(RailcraftConstants.MACHINE_TEXTURE_FILE, 0);

//        GL11.glScalef(0.994f, 1.05f, 0.994f);
        GL11.glPopAttrib();
        GL11.glPopMatrix();

        backDrop.texture = new int[]{25};
        RenderFakeBlock.renderBlock(backDrop, base.getWorld(), x, y, z, false, true);

        if (tile.getClass() == TileLiquidLoader.class) {
            TileLiquidLoader loader = (TileLiquidLoader) tile;

            pipe.minY = RenderTools.PIXEL - loader.getPipeLenght();

            RenderFakeBlock.renderBlock(pipe, loader.getWorld(), x, y, z, false, true);
        }

        ForgeHooksClient.unbindTexture();

    }
}
