package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.world.World;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;
import net.minecraftforge.common.ForgeDirection;
import org.lwjgl.opengl.GL11;
import railcraft.common.blocks.aesthetics.post.BlockPost;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class RenderPost implements ICombinedRenderer
{

    @Override
    public void renderBlock(RenderBlocks renderblocks, IBlockAccess iBlockAccess, int x, int y, int z, Block block) {
        BlockPost blockPost = (BlockPost)block;
        float pix = RenderTools.PIXEL;

        World world = Game.getWorld();

        evaluateCenterColumn(renderblocks, world, x, y, z, blockPost);

        boolean east_west = false;
        boolean north_south = false;
        if(blockPost.connectToBlockAt(world, x - 1, y, z, ForgeDirection.EAST) || blockPost.connectToBlockAt(world, x + 1, y, z, ForgeDirection.WEST)) {
            east_west = true;
        }
        if(blockPost.connectToBlockAt(world, x, y, z - 1, ForgeDirection.SOUTH) || blockPost.connectToBlockAt(world, x, y, z + 1, ForgeDirection.NORTH)) {
            north_south = true;
        }
        boolean east = blockPost.connectToBlockAt(world, x - 1, y, z, ForgeDirection.EAST);
        boolean west = blockPost.connectToBlockAt(world, x + 1, y, z, ForgeDirection.WEST);
        boolean south = blockPost.connectToBlockAt(world, x, y, z - 1, ForgeDirection.SOUTH);
        boolean north = blockPost.connectToBlockAt(world, x, y, z + 1, ForgeDirection.NORTH);
        if(!east_west && !north_south) {
            east_west = true;
        }
        float f = 0.4375F;
        float f1 = 0.5625F;
        float f2 = 11 * pix;
        float f3 = 14 * pix;
        float f4 = east ? 0.0F : f;
        float f5 = west ? 1.0F : f1;
        float f6 = south ? 0.0F : f;
        float f7 = north ? 1.0F : f1;
        if(east_west) {
            blockPost.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, blockPost, x, y, z);
        }
        if(north_south) {
            blockPost.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, blockPost, x, y, z);
        }
        f2 = 5 * pix;
        f3 = 8 * pix;
        if(east_west) {
            blockPost.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, blockPost, x, y, z);
        }
        if(north_south) {
            blockPost.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, blockPost, x, y, z);
        }
        blockPost.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    }

    public void evaluateCenterColumn(RenderBlocks renderblocks, World world, int i, int j, int k, BlockPost block) {
        int belowId = world.getBlockId(i, j - 1, k);
        int aboveId = world.getBlockId(i, j + 1, k);

        boolean renderColumm = false;
        if(!BlockRail.isRailBlock(belowId)) {
            Block aboveBlock = Block.blocksList[aboveId];
            Block belowBlock = Block.blocksList[belowId];
            if(belowBlock != null && belowBlock.isBlockSolidOnSide(Game.getWorld(), i, j - 1, k, ForgeDirection.UP)) {
                renderColumm = true;
            } else if(block.connectToBlockAt(world, i, j - 1, k, ForgeDirection.UP) || aboveBlock instanceof BlockPost) {
                renderColumm = true;
            }
        }

        boolean railAbove = BlockRail.isRailBlock(aboveId);

        if(renderColumm) {
            renderCenterColumn(renderblocks, i, j, k, block);
        } else if(!world.isAirBlock(i, j + 1, k) && !railAbove) {
            renderMiniPlatform(renderblocks, i, j, k, block);
        }

        if(railAbove) {
            renderPlatform(renderblocks, i, j, k, block);
        }
    }

    public void renderPlatform(RenderBlocks renderblocks, int i, int j, int k, Block block) {
        block.setBlockBounds(0.0F, (16 - getPlatformThinkness()) * RenderTools.PIXEL, 0.0F, 1.0F, 1.0F, 1.0F);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    public void renderCenterColumn(RenderBlocks renderblocks, int i, int j, int k, Block block) {
        float w = getCenterColumnWidth();
        float b1 = (8 - w / 2) * RenderTools.PIXEL;
        float b2 = (8 + w / 2) * RenderTools.PIXEL;
        block.setBlockBounds(b1, 0.0F, b1, b2, 1.0F, b2);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    public void renderMiniPlatform(RenderBlocks renderblocks, int i, int j, int k, Block block) {
        float w = getCenterColumnWidth();
        float b1 = (8 - w / 2) * RenderTools.PIXEL;
        float b2 = (8 + w / 2) * RenderTools.PIXEL;
        block.setBlockBounds(b1, 12 * RenderTools.PIXEL, b1, b2, 1.0F, b2);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    public float getPlatformThinkness() {
        return 2;
    }

    public float getCenterColumnWidth() {
        return 4;
    }

    @Override
    public void renderItem(RenderBlocks renderblocks, ItemStack item, ItemRenderType renderType) {
        int texture = item.getItem().getIconFromDamage(item.getItemDamage());

        Block block = Block.blocksList[((ItemBlock)item.getItem()).getBlockID()];
        if(block == null) {
            return;
        }
        for(int i1 = 0; i1 < 4; i1++) {
            float f4 = 0.125F;
            if(i1 == 0) {
                block.setBlockBounds(0.5F - f4, 0.0F, 0.0F, 0.5F + f4, 1.0F, f4 * 2.0F);
            }
            if(i1 == 1) {
                block.setBlockBounds(0.5F - f4, 0.0F, 1.0F - f4 * 2.0F, 0.5F + f4, 1.0F, 1.0F);
            }
            f4 = 0.0625F;
            if(i1 == 2) {
                block.setBlockBounds(0.5F - f4, 1.0F - f4 * 3.0F, -f4 * 2.0F, 0.5F + f4, 1.0F - f4, 1.0F + f4 * 2.0F);
            }
            if(i1 == 3) {
                block.setBlockBounds(0.5F - f4, 0.5F - f4 * 3.0F, -f4 * 2.0F, 0.5F + f4, 0.5F - f4, 1.0F + f4 * 2.0F);
            }
            renderblocks.setRenderBoundsFromBlock(block);
            GL11.glTranslatef(-0.5F, -0.5F, -0.5F);
            Tessellator tessellator = Tessellator.instance;
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, -1.0F, 0.0F);
            renderblocks.renderBottomFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 1.0F, 0.0F);
            renderblocks.renderTopFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 0.0F, -1.0F);
            renderblocks.renderEastFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 0.0F, 1.0F);
            renderblocks.renderWestFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(-1.0F, 0.0F, 0.0F);
            renderblocks.renderNorthFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(1.0F, 0.0F, 0.0F);
            renderblocks.renderSouthFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            GL11.glTranslatef(0.5F, 0.5F, 0.5F);
        }
        block.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    }
}
