package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.world.World;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;
import net.minecraftforge.common.ForgeDirection;
import org.lwjgl.opengl.GL11;
import railcraft.common.blocks.aesthetics.post.BlockPost;
import railcraft.common.util.misc.Game;

public class RenderPostStone extends RenderPost
{

    @Override
    public float getPlatformThinkness() {
        return 4;
    }

    @Override
    public float getCenterColumnWidth() {
        return 6;
    }

    @Override
    public void renderBlock(RenderBlocks renderblocks, IBlockAccess iBlockAccess, int x, int y, int z, Block block) {
        BlockPost blockPost = (BlockPost)block;
        float pix = RenderTools.PIXEL;

        World world = Game.getWorld();

        evaluateCenterColumn(renderblocks, world, x, y, z, blockPost);

        boolean east_west = false;
        boolean north_south = false;
        if(blockPost.connectToBlockAt(world, x - 1, y, z, ForgeDirection.EAST) || blockPost.connectToBlockAt(world, x + 1, y, z, ForgeDirection.WEST)) {
            east_west = true;
        }
        if(blockPost.connectToBlockAt(world, x, y, z - 1, ForgeDirection.SOUTH) || blockPost.connectToBlockAt(world, x, y, z + 1, ForgeDirection.NORTH)) {
            north_south = true;
        }
        boolean east = blockPost.connectToBlockAt(world, x - 1, y, z, ForgeDirection.EAST);
        boolean west = blockPost.connectToBlockAt(world, x + 1, y, z, ForgeDirection.WEST);
        boolean south = blockPost.connectToBlockAt(world, x, y, z - 1, ForgeDirection.SOUTH);
        boolean north = blockPost.connectToBlockAt(world, x, y, z + 1, ForgeDirection.NORTH);
        if(!east_west && !north_south) {
            east_west = true;
        }
        float f = 6 * pix;
        float f1 = 10 * pix;
        float f2 = 4 * pix;
        float f3 = 15 * pix;
        float f4 = east ? 0.0F : f;
        float f5 = west ? 1.0F : f1;
        float f6 = south ? 0.0F : f;
        float f7 = north ? 1.0F : f1;
        if(east_west) {
            block.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, block, x, y, z);
        }
        if(north_south) {
            block.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, block, x, y, z);
        }
        block.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    }

    @Override
    public void renderItem(RenderBlocks renderblocks, ItemStack item, ItemRenderType renderType) {
        int texture = item.getItem().getIconFromDamage(item.getItemDamage());

        Block block = Block.blocksList[((ItemBlock)item.getItem()).getBlockID()];
        if(block == null) {
            return;
        }
        float pix = RenderTools.PIXEL;
        for(int i1 = 0; i1 < 3; i1++) {
            float f4 = 3 * pix;
            if(i1 == 0) {
                block.setBlockBounds(0.5F - f4, 0.0F, 0.0F, 0.5F + f4, 1.0F, f4 * 2.0F);
            }
            if(i1 == 1) {
                block.setBlockBounds(0.5F - f4, 0.0F, 1.0F - f4 * 2.0F, 0.5F + f4, 1.0F, 1.0F);
            }
            f4 = 0.0625F;
            if(i1 == 2) {
                block.setBlockBounds(0.5F - f4, 0.5F - f4 * 3.0F, -f4 * 2.0F, 0.5F + f4, 1.0F - f4, 1.0F + f4 * 2.0F);
            }
//            if(i1 == 3)
//            {
//                block.setBlockBounds(0.5F - f4, 0.5F - f4 * 3.0F, -f4 * 2.0F, 0.5F + f4, 0.5F - f4, 1.0F + f4 * 2.0F);
//            }
            renderblocks.setRenderBoundsFromBlock(block);
            GL11.glTranslatef(-0.5F, -0.5F, -0.5F);
            Tessellator tessellator = Tessellator.instance;
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, -1.0F, 0.0F);
            renderblocks.renderBottomFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 1.0F, 0.0F);
            renderblocks.renderTopFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 0.0F, -1.0F);
            renderblocks.renderEastFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 0.0F, 1.0F);
            renderblocks.renderWestFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(-1.0F, 0.0F, 0.0F);
            renderblocks.renderNorthFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            tessellator.startDrawingQuads();
            tessellator.setNormal(1.0F, 0.0F, 0.0F);
            renderblocks.renderSouthFace(block, 0.0, 0.0, 0.0, texture);
            tessellator.draw();
            GL11.glTranslatef(0.5F, 0.5F, 0.5F);
        }
        block.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    }
}
