package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.world.World;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.signals.SignalAspect;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.aesthetics.post.BlockPost;
import railcraft.common.blocks.signals.BlockSignal;
import railcraft.common.blocks.signals.EnumSignal;
import railcraft.common.blocks.signals.Signals;
import railcraft.common.blocks.signals.TileSignalBase;
import railcraft.common.util.misc.Game;

public class RenderSignal implements ICombinedRenderer
{

    private EnumSignal structure;
    private SignalAspect defaultAspect;

    public RenderSignal(EnumSignal structure, SignalAspect defaultAspect) {
        this.structure = structure;
        this.defaultAspect = defaultAspect;
    }

    @Override
    public void renderBlock(RenderBlocks renderblocks, IBlockAccess iBlockAccess, int i, int j, int k, Block block) {
        BlockSignal blockStructure = (BlockSignal)block;
        TileSignalBase tile = (TileSignalBase)iBlockAccess.getBlockTileEntity(i, j, k);
        if(tile == null) {
            return;
        }
        float pix = RenderTools.PIXEL;
        float f = 6 * pix;
        float f1 = 10 * pix;
        Block below = Block.blocksList[iBlockAccess.getBlockId(i, j - 1, k)];
        World world = Game.getWorld();
        if(BlockPost.connectPostAt(world, i, j - 1, k, ForgeDirection.UP) || (below != null && below.isBlockSolidOnSide(Game.getWorld(), i, j - 1, k, ForgeDirection.UP))) {
            structure.setUseAltTextures(true);
            structure.setAltTextureIndex(Signals.POST_TEXTURE);
            blockStructure.setBlockBounds(f, 0, f, f1, 15 * pix, f1);
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
            structure.setUseAltTextures(false);
        }
        f = 3 * pix;
        f1 = 13 * pix;
        blockStructure.setBlockBounds(f, 6 * pix, f, f1, 1, f1);
        structure.setUseAltTextures(true);
        structure.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 16);
//        structure.setAltTextureOffset(16);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        if(tile.getSignalAspect().isBlinkAspect() && !SignalAspect.isBlinkOn()) {
            structure.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 2 + SignalAspect.OFF.getTextureOffset());
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        } else {
            structure.setUseOffsetTextureOnSide(tile.getFacing().ordinal(), 2 + tile.getSignalAspect().getTextureOffset());
            RenderTools.renderBlockSideWithBrightness(renderblocks, world, block, i, j, k, tile.getFacing().ordinal(), RenderTools.SIGNAL_BRIGHTNESS);
        }
        structure.setUseAltTextures(false);
        if(tile.getFacing().ordinal() == 2 || tile.getFacing().ordinal() == 3) {
            float temp = 13 * pix;
            if(tile.getFacing().ordinal() == 2) {
                temp = pix;
            }
            blockStructure.setBlockBounds(6 * pix, 13 * pix, temp, 10 * pix, 14 * pix, temp + 2 * pix);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
            blockStructure.setBlockBounds(5 * pix, 10 * pix, temp, 6 * pix, 14 * pix, temp + 2 * pix);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
            blockStructure.setBlockBounds(10 * pix, 10 * pix, temp, 11 * pix, 14 * pix, temp + 2 * pix);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        } else if(tile.getFacing().ordinal() == 4 || tile.getFacing().ordinal() == 5) {
            float temp = 13 * pix;
            if(tile.getFacing().ordinal() == 4) {
                temp = pix;
            }
            blockStructure.setBlockBounds(temp, 13 * pix, 6 * pix, temp + 2 * pix, 14 * pix, 10 * pix);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
            blockStructure.setBlockBounds(temp, 10 * pix, 5 * pix, temp + 2 * pix, 14 * pix, 6 * pix);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
            blockStructure.setBlockBounds(temp, 10 * pix, 10 * pix, temp + 2 * pix, 14 * pix, 11 * pix);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        boolean flag1 = false;
        boolean flag2 = false;
        if(BlockPost.connectPostAt(world, i - 1, j, k, ForgeDirection.EAST) || BlockPost.connectPostAt(world, i + 1, j, k, ForgeDirection.WEST)) {
            flag1 = true;
        }
        if(BlockPost.connectPostAt(world, i, j, k - 1, ForgeDirection.SOUTH) || BlockPost.connectPostAt(world, i, j, k + 1, ForgeDirection.NORTH)) {
            flag2 = true;
        }
        boolean flag3 = BlockPost.connectPostAt(world, i - 1, j, k, ForgeDirection.EAST);
        boolean flag4 = BlockPost.connectPostAt(world, i + 1, j, k, ForgeDirection.WEST);
        boolean flag5 = BlockPost.connectPostAt(world, i, j, k - 1, ForgeDirection.SOUTH);
        boolean flag6 = BlockPost.connectPostAt(world, i, j, k + 1, ForgeDirection.NORTH);
        if(!flag1 && !flag2) {
            flag1 = true;
        }
        f = 0.4375F;
        f1 = 0.5625F;
        float f2 = 11 * pix;
        float f3 = 14 * pix;
        float f4 = flag3 ? 0 : f;
        float f5 = flag4 ? 1 : f1;
        float f6 = flag5 ? 0 : f;
        float f7 = flag6 ? 1 : f1;
        structure.setUseAltTextures(true);
        structure.setAltTextureIndex(Signals.POST_TEXTURE);
        if(flag1) {
            blockStructure.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        if(flag2) {
            blockStructure.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        f2 = 5 * pix;
        f3 = 8 * pix;
        if(flag1) {
            blockStructure.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        if(flag2) {
            blockStructure.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, blockStructure, i, j, k);
        }
        structure.setUseAltTextures(false);
        blockStructure.setBlockBounds(0, 0, 0, 1, 1, 1);
    }

    @Override
    public void renderItem(RenderBlocks renderblocks, ItemStack item, ItemRenderType renderType) {
        BlockSignal blockStructure = (BlockSignal)RailcraftBlocks.getBlockSignal();
        float pix = RenderTools.PIXEL;
        float f = 6 * pix;
        float f1 = 10 * pix;
        structure.setUseAltTextures(true);
        structure.setAltTextureIndex(Signals.POST_TEXTURE);
        blockStructure.setBlockBounds(f, 0, f, f1, 15 * pix, f1);
        RenderTools.renderBlockOnInventory(renderblocks, blockStructure, structure.ordinal(), 1);
        structure.setUseAltTextures(false);
        f = 3 * pix;
        f1 = 13 * pix;
        blockStructure.setBlockBounds(f, 6 * pix, f, f1, 1, f1);
        RenderTools.renderBlockOnInventory(renderblocks, blockStructure, structure.ordinal(), 1);
        structure.setUseAltTextures(true);
        structure.setUseAltTextureOnSide(3, structure.getOffsetTexture(2 + defaultAspect.getTextureOffset()));
//        structure.setAltTextureOffset(4);
        RenderTools.renderBlockOnInventory(renderblocks, blockStructure, structure.ordinal(), 1, 3);
        structure.setUseAltTextures(false);
        float temp = 13 * pix;
        blockStructure.setBlockBounds(6 * pix, 13 * pix, temp, 10 * pix, 14 * pix, temp + 2 * pix);
        RenderTools.renderBlockOnInventory(renderblocks, blockStructure, structure.ordinal(), 1);
        blockStructure.setBlockBounds(5 * pix, 10 * pix, temp, 6 * pix, 14 * pix, temp + 2 * pix);
        RenderTools.renderBlockOnInventory(renderblocks, blockStructure, structure.ordinal(), 1);
        blockStructure.setBlockBounds(10 * pix, 10 * pix, temp, 11 * pix, 14 * pix, temp + 2 * pix);
        RenderTools.renderBlockOnInventory(renderblocks, blockStructure, structure.ordinal(), 1);
        blockStructure.setBlockBounds(0, 0, 0, 1, 1, 1);
    }
}
