package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;
import railcraft.common.api.signals.SignalAspect;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.signals.BlockSignal;
import railcraft.common.blocks.signals.EnumSignal;
import railcraft.common.blocks.signals.ItemSignal;
import railcraft.common.blocks.signals.TileBoxBase;

public class RenderSignalBox implements ICombinedRenderer
{

    @Override
    public void renderBlock(RenderBlocks renderblocks, IBlockAccess world, int i, int j, int k, Block block) {
        TileBoxBase tile = (TileBoxBase)world.getBlockTileEntity(i, j, k);
        EnumSignal structure = tile.getSignalType();
        float pix = RenderTools.PIXEL;
//        block.setBlockBounds(pix, 13 * pix, pix, 15 * pix, 15 * pix, 15 * pix);
//        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);

        boolean eastWest = false;
        boolean northSouth = false;
        if(tile.connectToBoxAt(i - 1, j, k, 5) || tile.connectToBoxAt(i + 1, j, k, 4)) {
            eastWest = true;
        }
        if(tile.connectToBoxAt(i, j, k - 1, 3) || tile.connectToBoxAt(i, j, k + 1, 2)) {
            northSouth = true;
        }
        boolean side2 = tile.connectToBoxAt(i, j, k - 1, 5);
        boolean side3 = tile.connectToBoxAt(i, j, k + 1, 4);
        boolean side4 = tile.connectToBoxAt(i - 1, j, k, 3);
        boolean side5 = tile.connectToBoxAt(i + 1, j, k, 2);
        if(!eastWest && !northSouth) {
            eastWest = true;
        }
        float f = 7 * pix;
        float f1 = 9 * pix;
        float f2 = 10 * pix;
        float f3 = 12 * pix;
        float f4 = side4 ? 0.0F : f;
        float f5 = side5 ? 1.0F : f1;
        float f6 = side2 ? 0.0F : f;
        float f7 = side3 ? 1.0F : f1;

        structure.setUseAltTextures(true);
        structure.setUseAltTextureOnSide(0, structure.getSecondaryTexture());
        structure.setUseAltTextureOnSide(1, structure.getSecondaryTexture());
        if(side2) {
            structure.setUseOffsetTextureOnSide(2, 16);
        }
        if(side3) {
            structure.setUseOffsetTextureOnSide(3, 16);
        }
        if(side4) {
            structure.setUseOffsetTextureOnSide(4, 16);
        }
        if(side5) {
            structure.setUseOffsetTextureOnSide(5, 16);
        }
        block.setBlockBounds(2 * pix, 0, 2 * pix, 14 * pix, 15 * pix, 14 * pix);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        if(!world.isAirBlock(i, j + 1, k)) {
            structure.setUseAltTextureOnSide(1, structure.getPrimaryTexture() + 16 * 3 - 1);
            block.setBlockBounds(5 * pix, 15 * pix, 5 * pix, 11 * pix, 16 * pix, 11 * pix);
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
            block.setBlockBounds(2 * pix, 0, 2 * pix, 14 * pix, 15 * pix, 14 * pix);
            structure.setUseAltTextureOnSide(1, structure.getSecondaryTexture());
        }
        if(tile.getBoxSignalAspect().isBlinkAspect() && !SignalAspect.isBlinkOn()) {
            int offset = 1 + SignalAspect.OFF.getTextureOffset();
            for(int ii = 2; ii < 6; ii++) {
                if(structure.getBlockTextureFromSide(ii) == structure.getPrimaryTexture()) {
                    structure.setUseOffsetTextureOnSide(ii, offset);
                }
            }
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        } else {
            int offset = 1 + tile.getBoxSignalAspect().getTextureOffset();
            if(!side2) {
                structure.setUseOffsetTextureOnSide(2, offset);
                RenderTools.renderBlockSideWithBrightness(renderblocks, world, block, i, j, k, 2, RenderTools.BOX_BRIGHTNESS);
            }
            if(!side3) {
                structure.setUseOffsetTextureOnSide(3, offset);
                RenderTools.renderBlockSideWithBrightness(renderblocks, world, block, i, j, k, 3, RenderTools.BOX_BRIGHTNESS);
            }
            if(!side4) {
                structure.setUseOffsetTextureOnSide(4, offset);
                RenderTools.renderBlockSideWithBrightness(renderblocks, world, block, i, j, k, 4, RenderTools.BOX_BRIGHTNESS);
            }
            if(!side5) {
                structure.setUseOffsetTextureOnSide(5, offset);
                RenderTools.renderBlockSideWithBrightness(renderblocks, world, block, i, j, k, 5, RenderTools.BOX_BRIGHTNESS);
            }
        }

        structure.setAltTextureIndex(structure.getOffsetTexture(16 * 3));
        if(eastWest) {
            block.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        }
        if(northSouth) {
            block.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        }
        f2 = 5 * pix;
        f3 = 7 * pix;
        if(eastWest) {
            block.setBlockBounds(f4, f2, f, f5, f3, f1);
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        }
        if(northSouth) {
            block.setBlockBounds(f, f2, f6, f1, f3, f7);
            RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        }
        structure.setUseAltTextures(false);
        block.setBlockBounds(0, 0, 0, 1, 1, 1);
    }

    @Override
    public void renderItem(RenderBlocks renderblocks, ItemStack item, ItemRenderType renderType) {
        EnumSignal structure = null;
        if(item.getItem() instanceof ItemSignal) {
            structure = EnumSignal.fromId(item.getItemDamage());
        } else {
            return;
        }
        BlockSignal block = (BlockSignal)RailcraftBlocks.getBlockSignal();
        float pix = RenderTools.PIXEL;
        block.setBlockBounds(2 * pix, 0, 2 * pix, 14 * pix, 15 * pix, 14 * pix);
        RenderTools.renderBlockOnInventory(renderblocks, block, structure.ordinal(), 1);
        structure.setUseAltTextures(true);
        int texture = structure.getPrimaryTexture() + 1;
        texture += structure == EnumSignal.BOX_RECEIVER ? SignalAspect.RED.getTextureOffset() : SignalAspect.GREEN.getTextureOffset();
        structure.setUseAltTextureOnSide(0, structure.getSecondaryTexture());
        structure.setUseAltTextureOnSide(1, structure.getSecondaryTexture());
        structure.setUseAltTextureOnSide(2, texture);
        structure.setUseAltTextureOnSide(3, texture);
        structure.setUseAltTextureOnSide(4, texture);
        structure.setUseAltTextureOnSide(5, texture);
        RenderTools.renderBlockOnInventory(renderblocks, block, structure.ordinal(), 1);
        structure.setUseAltTextures(false);
        block.setBlockBounds(0, 0, 0, 1, 1, 1);
    }
}
