package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraftforge.client.IItemRenderer.ItemRenderType;
import org.lwjgl.opengl.GL11;
import railcraft.common.api.tracks.ITrackSwitch;
import railcraft.common.api.tracks.ITrackSwitch.ArrowDirection;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.signals.BlockSignal;
import railcraft.common.blocks.signals.EnumSignal;
import railcraft.common.blocks.signals.TileSwitchBase;

public class RenderSwitch implements ICombinedRenderer
{

    private static final float PIX = RenderTools.PIXEL;
    private static final int LEVER_HEIGHT = 10;
    private EnumSignal type;

    public RenderSwitch(EnumSignal str) {
        type = str;
    }

    @Override
    public void renderBlock(RenderBlocks renderblocks, IBlockAccess world, int x, int y, int z, Block block) {
        BlockSignal structure = (BlockSignal)block;
        TileSwitchBase tile = (TileSwitchBase)world.getBlockTileEntity(x, y, z);
        float f = 4.0F * PIX;
        float f1 = 12.0F * PIX;
        structure.setBlockBounds(f, 0.0F, f, f1, 5.0F * PIX, f1);
        RenderTools.renderStandardBlock(renderblocks, structure, x, y, z);
        f = 7.0F * PIX;
        f1 = 9.0F * PIX;
        structure.setBlockBounds(f, 0.0F, f, f1, 8.0F * PIX, f1);

        RenderTools.renderStandardBlock(renderblocks, structure, x, y, z);
        structure.setBlockBounds(f, 11.0F * PIX, f, f1, 12.0F * PIX, f1);
        RenderTools.renderStandardBlock(renderblocks, structure, x, y, z);

        if(tile.getFacing() == 2 || tile.getFacing() == 3) {
            structure.setBlockBounds(6.0F * PIX, 0.0F, 0.0F, 10.0F * PIX, 3.0F * PIX, 1.0F);
            RenderTools.renderStandardBlock(renderblocks, structure, x, y, z);
        } else {
            structure.setBlockBounds(0.0F, 0.0F, 6.0F * PIX, 1.0F, 3.0F * PIX, 10.0F * PIX);
            RenderTools.renderStandardBlock(renderblocks, structure, x, y, z);
        }

        ITrackSwitch track = tile.getSwitchTrack();

        if(track == null) {
            type.setUseAltTextures(true);
            type.setAltTextureOffset(1);
            renderTargetNorthSouth(renderblocks, structure, x, y, z);
            type.setAltTextureOffset(2);
            renderTargetEastWest(renderblocks, structure, x, y, z);
            type.setUseAltTextures(false);
            return;
        }
        
        type.setUseAltTextures(true);
        type.setAltTextureOffset(1);
        ArrowDirection whiteArrow = track.getWhiteSignDirection();
        renderTarget(whiteArrow, renderblocks, block, x, y, z);

        type.setAltTextureOffset(2);
        ArrowDirection redArrow = track.getRedSignDirection();
        renderTarget(redArrow, renderblocks, block, x, y, z);
        type.setUseAltTextures(false);

        if(type == EnumSignal.SWITCH_LEVER) {
            switch (tile.getFacing()) {
                case 2:
                    structure.setBlockBounds(PIX * 7, 0, PIX * 14, PIX * 9, PIX * LEVER_HEIGHT, 1);
                    break;
                case 3:
                    structure.setBlockBounds(PIX * 7, 0, 0, PIX * 9, PIX * LEVER_HEIGHT, PIX * 2);
                    break;
                case 4:
                    structure.setBlockBounds(PIX * 14, 0, PIX * 7, 1, PIX * LEVER_HEIGHT, PIX * 9);
                    break;
                case 5:
                    structure.setBlockBounds(0, 0, PIX * 7, PIX * 2, PIX * LEVER_HEIGHT, PIX * 9);
                    break;
            }
            RenderTools.renderStandardBlock(renderblocks, structure, x, y, z);
        }
        structure.setBlockBounds(0, 0, 0, 1, 1, 1);

    }

    private void renderTarget(ArrowDirection arrow, RenderBlocks renderblocks, Block block, int x, int y, int z) {
        switch (arrow) {
            case NORTH:
                renderTargetNorth(renderblocks, block, x, y, z);
                break;
            case SOUTH:
                renderTargetSouth(renderblocks, block, x, y, z);
                break;
            case EAST:
                renderTargetEast(renderblocks, block, x, y, z);
                break;
            case WEST:
                renderTargetWest(renderblocks, block, x, y, z);
                break;
            case NORTH_SOUTH:
                renderTargetNorthSouth(renderblocks, block, x, y, z);
                break;
            case EAST_WEST:
                renderTargetEastWest(renderblocks, block, x, y, z);
                break;
        }
    }

    private void renderTargetNorthSouth(RenderBlocks renderblocks, Block block, int i, int j, int k) {
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 4.0F * PIX, 9.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(7.0F * PIX, 8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 11.0F * PIX, 12.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    private void renderTargetEastWest(RenderBlocks renderblocks, Block block, int i, int j, int k) {
        block.setBlockBounds(4.0F * PIX, 8.0F * PIX, 7.0F * PIX, 8.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 12.0F * PIX, 11.0F * PIX, 9.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    private void renderTargetNorth(RenderBlocks renderblocks, Block block, int i, int j, int k) {
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 4.0F * PIX, 9.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(7.0F * PIX, 8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 11.0F * PIX, 12.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(8.0F * PIX, 9.0F * PIX, 3.0F * PIX, 9.0F * PIX, 10.0F * PIX, 4.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(7.0F * PIX, 8.0F * PIX, 12.0F * PIX, 8.0F * PIX, 9.0F * PIX, 13.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(7.0F * PIX, 10.0F * PIX, 12.0F * PIX, 8.0F * PIX, 11.0F * PIX, 13.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    private void renderTargetSouth(RenderBlocks renderblocks, Block block, int i, int j, int k) {
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 4.0F * PIX, 9.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(7.0F * PIX, 8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 11.0F * PIX, 12.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(7.0F * PIX, 9.0F * PIX, 12.0F * PIX, 8.0F * PIX, 10.0F * PIX, 13.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 3.0F * PIX, 9.0F * PIX, 9.0F * PIX, 4.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(8.0F * PIX, 10.0F * PIX, 3.0F * PIX, 9.0F * PIX, 11.0F * PIX, 4.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    private void renderTargetEast(RenderBlocks renderblocks, Block block, int i, int j, int k) {
        block.setBlockBounds(4.0F * PIX, 8.0F * PIX, 7.0F * PIX, 8.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 12.0F * PIX, 11.0F * PIX, 9.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(3.0F * PIX, 9.0F * PIX, 7.0F * PIX, 4.0F * PIX, 10.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(12.0F * PIX, 8.0F * PIX, 8.0F * PIX, 13.0F * PIX, 9.0F * PIX, 9.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(12.0F * PIX, 10.0F * PIX, 8.0F * PIX, 13.0F * PIX, 11.0F * PIX, 9.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    private void renderTargetWest(RenderBlocks renderblocks, Block block, int i, int j, int k) {
        block.setBlockBounds(4.0F * PIX, 8.0F * PIX, 7.0F * PIX, 8.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 12.0F * PIX, 11.0F * PIX, 9.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(12.0F * PIX, 9.0F * PIX, 8.0F * PIX, 13.0F * PIX, 10.0F * PIX, 9.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(3.0F * PIX, 8.0F * PIX, 7.0F * PIX, 4.0F * PIX, 9.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
        block.setBlockBounds(3.0F * PIX, 10.0F * PIX, 7.0F * PIX, 4.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderStandardBlock(renderblocks, block, i, j, k);
    }

    @Override
    public void renderItem(RenderBlocks renderblocks, ItemStack item, ItemRenderType renderType) {
        switch (renderType) {
            case EQUIPPED:
            case ENTITY:
                float scale = 1.5f;
                GL11.glScalef(scale, scale, scale);
        }

        BlockSignal block = (BlockSignal)RailcraftBlocks.getBlockSignal();
        float f = 4.0F * PIX;
        float f1 = 12.0F * PIX;
        block.setBlockBounds(f, 0.0F, f, f1, 5.0F * PIX, f1);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        f = 7.0F * PIX;
        f1 = 9.0F * PIX;
        block.setBlockBounds(f, 0.0F, f, f1, 8.0F * PIX, f1);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        block.setBlockBounds(f, 11.0F * PIX, f, f1, 12.0F * PIX, f1);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        block.setBlockBounds(6.0F * PIX, 0.0F, 0.0F, 10.0F * PIX, 3.0F * PIX, 1.0F);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);

        // Red target
        type.setUseAltTextures(true);
        type.setAltTextureOffset(2);
        block.setBlockBounds(4.0F * PIX, 8.0F * PIX, 7.0F * PIX, 8.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 12.0F * PIX, 11.0F * PIX, 9.0F * PIX);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        block.setBlockBounds(12.0F * PIX, 9.0F * PIX, 8.0F * PIX, 13.0F * PIX, 10.0F * PIX, 9.0F * PIX);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        block.setBlockBounds(3.0F * PIX, 8.0F * PIX, 7.0F * PIX, 4.0F * PIX, 9.0F * PIX, 8.0F * PIX);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        block.setBlockBounds(3.0F * PIX, 10.0F * PIX, 7.0F * PIX, 4.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        type.setUseAltTextures(false);

        // White target
        type.setUseAltTextures(true);
        type.setAltTextureOffset(1);
        block.setBlockBounds(8.0F * PIX, 8.0F * PIX, 4.0F * PIX, 9.0F * PIX, 11.0F * PIX, 8.0F * PIX);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        block.setBlockBounds(7.0F * PIX, 8.0F * PIX, 8.0F * PIX, 8.0F * PIX, 11.0F * PIX, 12.0F * PIX);
        RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        type.setUseAltTextures(false);

        if(type == EnumSignal.SWITCH_LEVER) {

//            structure.setBlockBounds(PIX * 7, 0, PIX * 14, PIX * 9, PIX * LEVER_HEIGHT, 1);

            block.setBlockBounds(PIX * 7, 0, 0, PIX * 9, PIX * LEVER_HEIGHT, PIX * 2);
            RenderTools.renderBlockOnInventory(renderblocks, block, type.ordinal(), 1);
        }

        block.setBlockBounds(0, 0, 0, 1, 1, 1);
    }
}
