package railcraft.client.render;

import net.minecraft.block.Block;
import net.minecraft.client.renderer.EntityRenderer;
import net.minecraft.world.IBlockAccess;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import org.lwjgl.opengl.GL11;

public class RenderTools
{

    public static final int SIGNAL_BRIGHTNESS = 210;
    public static final int BOX_BRIGHTNESS = 165;
    public static final float PIXEL = 0.0625f;

    public static boolean renderStandardBlock(RenderBlocks renderblocks, Block block, int x, int y, int z) {
        renderblocks.setRenderBoundsFromBlock(block);
        return renderblocks.renderStandardBlock(block, x, y, z);
    }

    public static boolean renderStandardBlockWithColorMultiplier(RenderBlocks renderblocks, Block block, int x, int y, int z) {
        renderblocks.setRenderBoundsFromBlock(block);
        int mult = block.colorMultiplier(renderblocks.blockAccess, x, y, z);
        float r = (float)(mult >> 16 & 255) / 255.0F;
        float g = (float)(mult >> 8 & 255) / 255.0F;
        float b = (float)(mult & 255) / 255.0F;

        if(EntityRenderer.anaglyphEnable) {
            float var9 = (r * 30.0F + g * 59.0F + b * 11.0F) / 100.0F;
            float var10 = (r * 30.0F + g * 70.0F) / 100.0F;
            float var11 = (r * 30.0F + b * 70.0F) / 100.0F;
            r = var9;
            g = var10;
            b = var11;
        }

        return renderblocks.renderStandardBlockWithColorMultiplier(block, x, y, z, r, g, b);
    }

    public static void renderBlockSideWithBrightness(RenderBlocks renderblocks, IBlockAccess world, Block block, int i, int j, int k, int side, int brightness) {
        renderblocks.setRenderBoundsFromBlock(block);
        renderblocks.enableAO = false;
        Tessellator tessellator = Tessellator.instance;
        tessellator.setColorOpaque_F(1.0F, 1.0F, 1.0F);
        tessellator.setBrightness(brightness);
        if(side == 0) {
            renderblocks.renderBottomFace(block, i, j, k, block.getBlockTexture(world, i, j, k, 0));
        } else if(side == 1) {
            renderblocks.renderTopFace(block, i, j, k, block.getBlockTexture(world, i, j, k, 1));
        } else if(side == 2) {
            renderblocks.renderEastFace(block, i, j, k, block.getBlockTexture(world, i, j, k, 2));
        } else if(side == 3) {
            renderblocks.renderWestFace(block, i, j, k, block.getBlockTexture(world, i, j, k, 3));
        } else if(side == 4) {
            renderblocks.renderNorthFace(block, i, j, k, block.getBlockTexture(world, i, j, k, 4));
        } else if(side == 5) {
            renderblocks.renderSouthFace(block, i, j, k, block.getBlockTexture(world, i, j, k, 5));
        }
    }

    public static void renderBlockOnInventory(RenderBlocks renderblocks, Block block, int meta, float light) {
        renderBlockOnInventory(renderblocks, block, meta, light, -1);
    }

    public static void renderBlockOnInventory(RenderBlocks renderblocks, Block block, int meta, float light, int side) {
        Tessellator tessellator = Tessellator.instance;
        boolean flag = block.blockID == Block.grass.blockID;
        if(renderblocks.useInventoryTint) {
            int j = block.getRenderColor(meta);
            if(flag) {
                j = 0xffffff;
            }
            float red = (float)(j >> 16 & 0xff) / 255F;
            float green = (float)(j >> 8 & 0xff) / 255F;
            float blue = (float)(j & 0xff) / 255F;
            GL11.glColor4f(red * light, green * light, blue * light, 1.0F);
        }
        block.setBlockBoundsForItemRender();
        renderblocks.setRenderBoundsFromBlock(block);
        GL11.glTranslatef(-0.5F, -0.5F, -0.5F);
        if(side == 0 || side == -1) {
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, -1F, 0.0F);
            renderblocks.renderBottomFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(0, meta));
            tessellator.draw();
        }
        if(flag && renderblocks.useInventoryTint) {
            int k1 = block.getRenderColor(meta);
            float f6 = (float)(k1 >> 16 & 0xff) / 255F;
            float f8 = (float)(k1 >> 8 & 0xff) / 255F;
            float f9 = (float)(k1 & 0xff) / 255F;
            GL11.glColor4f(f6 * light, f8 * light, f9 * light, 1.0F);
        }
        if(side == 1 || side == -1) {
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 1.0F, 0.0F);
            renderblocks.renderTopFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(1, meta));
            tessellator.draw();
        }
        if(flag && renderblocks.useInventoryTint) {
            GL11.glColor4f(light, light, light, 1.0F);
        }
        if(side == 2 || side == -1) {
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 0.0F, -1F);
            renderblocks.renderEastFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(2, meta));
            tessellator.draw();
        }
        if(side == 3 || side == -1) {
            tessellator.startDrawingQuads();
            tessellator.setNormal(0.0F, 0.0F, 1.0F);
            renderblocks.renderWestFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(3, meta));
            tessellator.draw();
        }
        if(side == 4 || side == -1) {
            tessellator.startDrawingQuads();
            tessellator.setNormal(-1F, 0.0F, 0.0F);
            renderblocks.renderNorthFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(4, meta));
            tessellator.draw();
        }
        if(side == 5 || side == -1) {
            tessellator.startDrawingQuads();
            tessellator.setNormal(1.0F, 0.0F, 0.0F);
            renderblocks.renderSouthFace(block, 0.0D, 0.0D, 0.0D, block.getBlockTextureFromSideAndMetadata(5, meta));
            tessellator.draw();
        }
        GL11.glTranslatef(0.5F, 0.5F, 0.5F);
    }
}
