package railcraft.client.render;

import cpw.mods.fml.client.registry.ISimpleBlockRenderingHandler;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.world.IBlockAccess;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.client.ForgeHooksClient;
import railcraft.common.api.tracks.ITrackInstance;
import railcraft.common.api.tracks.ITrackSwitch;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.tracks.TileTrack;
import railcraft.common.blocks.tracks.TrackBufferStop;
import railcraft.common.blocks.tracks.TrackGated;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.util.misc.FakeBlockRenderInfo;

public class RenderTrack implements ISimpleBlockRenderingHandler {

    @Override
    public void renderInventoryBlock(Block block, int metadata, int modelID, RenderBlocks renderer) {
    }

    @Override
    public boolean shouldRender3DInInventory() {
        return false;
    }

    @Override
    public int getRenderId() {
        return RailcraftBlocks.getBlockTrack().getRenderType();
    }

    @Override
    public boolean renderWorldBlock(IBlockAccess world, int i, int j, int k, Block block, int modelId, RenderBlocks renderblocks) {
        int meta = ((BlockRail) block).getBasicRailMetadata(world, null, i, j, k);

        // Changed from getBlockTextureFromSideAndMetadata
        int texture = block.getBlockTexture(world, i, j, k, 0);

        boolean fallback = false;
        if (texture < 0) {
            texture = 8 * 16;
            fallback = true;
        }


        TileEntity tile = world.getBlockTileEntity(i, j, k);
        ITrackInstance track = null;
        if (tile instanceof TileTrack) {
            track = ((TileTrack) tile).getTrackInstance();
        } else {
            texture = 8 * 16;
            fallback = true;
        }

        if (renderblocks.overrideBlockTexture >= 0) {
            texture = renderblocks.overrideBlockTexture;
        }

        int j1 = (texture & 0xf) << 4;
        int k1 = texture & 0xf0;
        double uv1 = (float) j1 / 256F;
        double uv2 = ((float) j1 + 15.99F) / 256F;
        double uv3 = (float) k1 / 256F;
        double uv4 = ((float) k1 + 15.99F) / 256F;
        double d4 = 0.0625D;
        double d5 = i + 1;
        double d6 = i + 1;
        double d7 = i + 0;
        double d8 = i + 0;
        double d9 = k + 0;
        double d10 = k + 1;
        double d11 = k + 1;
        double d12 = k + 0;
        double d13 = (double) j + d4;
        double d14 = (double) j + d4;
        double d15 = (double) j + d4;
        double d16 = (double) j + d4;
        if (meta != 1 && meta != 2 && meta != 3 && meta != 7) {
            if (meta == 8) {
                d5 = d6 = i + 0;
                d7 = d8 = i + 1;
                d9 = d12 = k + 1;
                d10 = d11 = k + 0;
            } else if (meta == 9) {
                d5 = d8 = i + 0;
                d6 = d7 = i + 1;
                d9 = d10 = k + 0;
                d11 = d12 = k + 1;
            }
        } else {
            d5 = d8 = i + 1;
            d6 = d7 = i + 0;
            d9 = d10 = k + 1;
            d11 = d12 = k + 0;
        }
        if (meta != 2 && meta != 4) {
            if (meta == 3 || meta == 5) {
                d14++;
                d15++;
            }
        } else {
            d13++;
            d16++;
        }

        if (track != null) {
            if (track instanceof ITrackSwitch) {
                ITrackSwitch switchTrack = (ITrackSwitch) track;
                if (switchTrack.isMirrored()) {
                    double temp = uv1;
                    uv1 = uv2;
                    uv2 = temp;
                    temp = uv3;
                    uv3 = uv4;
                    uv4 = temp;
                }
            } else if (track instanceof TrackGated) {
                renderGatedTrack(renderblocks, (TrackGated) track, i, j, k, meta);
            } else if (track instanceof TrackBufferStop) {
                renderBufferTrack(renderblocks, (TrackBufferStop) track, world, i, j, k, meta);
            }

            if (renderblocks.overrideBlockTexture < 0) {
                if (fallback) {
                    ForgeHooksClient.bindTexture("/terrain.png", 0);
                } else {
                    ForgeHooksClient.bindTexture(track.getTrackSpec().getTextureFile(), 0);
                }
            }
        } else {
            if (renderblocks.overrideBlockTexture < 0) {
                ForgeHooksClient.bindTexture("/terrain.png", 0);
            }
        }

        Tessellator tessellator = Tessellator.instance;
        tessellator.setBrightness(block.getMixedBrightnessForBlock(world, i, j, k));
        tessellator.setColorOpaque_F(1.0F, 1.0F, 1.0F);

        tessellator.addVertexWithUV(d5, d13, d9, uv2, uv3);
        tessellator.addVertexWithUV(d6, d14, d10, uv2, uv4);
        tessellator.addVertexWithUV(d7, d15, d11, uv1, uv4);
        tessellator.addVertexWithUV(d8, d16, d12, uv1, uv3);
        tessellator.addVertexWithUV(d8, d16, d12, uv1, uv3);
        tessellator.addVertexWithUV(d7, d15, d11, uv1, uv4);
        tessellator.addVertexWithUV(d6, d14, d10, uv2, uv4);
        tessellator.addVertexWithUV(d5, d13, d9, uv2, uv3);

        ForgeHooksClient.unbindTexture();

        return true;
    }

    private static void renderBufferTrack(RenderBlocks render, TrackBufferStop track, IBlockAccess world, int i, int j, int k, int meta) {
        boolean useTexture = false;
        if (render.overrideBlockTexture < 0) {
            useTexture = true;
            ForgeHooksClient.bindTexture(RailcraftConstants.MAIN_TEXTURE_FILE, 0);
        }

        float pix = RenderTools.PIXEL;

        FakeBlockRenderInfo board = new FakeBlockRenderInfo();
        board.texture = new int[]{69};
        board.minX = pix * 2;
        board.minY = pix * 6;
        board.minZ = pix * 4;
        board.maxX = pix * 14;
        board.maxY = pix * 10;
        board.maxZ = pix * 6;

        FakeBlockRenderInfo bumper1 = new FakeBlockRenderInfo();
        bumper1.texture = new int[]{83};
        bumper1.minX = pix * 3;
        bumper1.minY = pix * 7;
        bumper1.minZ = pix * 3;
        bumper1.maxX = pix * 5;
        bumper1.maxY = pix * 9;
        bumper1.maxZ = pix * 4;

        FakeBlockRenderInfo bumper2 = new FakeBlockRenderInfo();
        bumper2.texture = new int[]{83};
        bumper2.minX = pix * 11;
        bumper2.minY = pix * 7;
        bumper2.minZ = pix * 3;
        bumper2.maxX = pix * 13;
        bumper2.maxY = pix * 9;
        bumper2.maxZ = pix * 4;

        FakeBlockRenderInfo baseBig = new FakeBlockRenderInfo();
        baseBig.texture = new int[]{84};
        baseBig.minX = pix * 4;
        baseBig.minY = pix * 0;
        baseBig.minZ = pix * 5;
        baseBig.maxX = pix * 12;
        baseBig.maxY = pix * 9;
        baseBig.maxZ = pix * 9;

        FakeBlockRenderInfo baseSmall = new FakeBlockRenderInfo();
        baseSmall.texture = new int[]{85};
        baseSmall.minX = pix * 5;
        baseSmall.minY = pix * 0;
        baseSmall.minZ = pix * 9;
        baseSmall.maxX = pix * 11;
        baseSmall.maxY = pix * 7;
        baseSmall.maxZ = pix * 13;



        if (meta == 0) {
            if (!track.isReversed()) {
                board.reverseZ();
                bumper1.reverseZ();
                bumper2.reverseZ();
                baseBig.reverseZ();
                baseSmall.reverseZ();
            }
        } else {
            board.texture = new int[]{68, 68, 69, 69, 69, 69};
            board.rotate();
            bumper1.rotate();
            bumper2.rotate();
            baseBig.rotate();
            baseSmall.rotate();

            if (track.isReversed()) {
                board.reverseX();
                bumper1.reverseX();
                bumper2.reverseX();
                baseBig.reverseX();
                baseSmall.reverseX();
            }
        }

//        Tessellator tessellator = Tessellator.instance;

//        tessellator.setBrightness(Block.stone.getMixedBrightnessForBlock(world, i, j, k));
//        tessellator.setBrightness(983055);

        RenderFakeBlock.renderAsBlock(board, render, world, i, j, k);
        RenderFakeBlock.renderAsBlock(bumper1, render, world, i, j, k);
        RenderFakeBlock.renderAsBlock(bumper2, render, world, i, j, k);
        RenderFakeBlock.renderAsBlock(baseBig, render, world, i, j, k);
        RenderFakeBlock.renderAsBlock(baseSmall, render, world, i, j, k);

        if (useTexture) {
//            render.overrideBlockTexture = -1;
        }

//        block.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    }

    private static void renderGatedTrack(RenderBlocks render, TrackGated track, int i, int j, int k, int meta) {
        ForgeHooksClient.bindTexture("/terrain.png", 0);
        boolean open = track.isGateOpen();
        Block gate = Block.fenceGate;
        if (meta == 0) {
            float f = 0.0F;
            float f8 = 0.125F;
            float f4 = 0.4375F;
            float f12 = 0.5625F;
            gate.setBlockBounds(f, 0.3125F, f4, f8, 1.0F, f12);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            f = 0.875F;
            f8 = 1.0F;
            gate.setBlockBounds(f, 0.3125F, f4, f8, 1.0F, f12);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
        } else {
            float f1 = 0.4375F;
            float f9 = 0.5625F;
            float f5 = 0.0F;
            float f13 = 0.125F;
            gate.setBlockBounds(f1, 0.3125F, f5, f9, 1.0F, f13);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            f5 = 0.875F;
            f13 = 1.0F;
            gate.setBlockBounds(f1, 0.3125F, f5, f9, 1.0F, f13);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
        }
        if (!open) {
            if (meta == 0) {
                float f2 = 0.375F;
                float f10 = 0.5F;
                float f6 = 0.4375F;
                float f14 = 0.5625F;
                gate.setBlockBounds(f2, 0.375F, f6, f10, 0.9375F, f14);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                f2 = 0.5F;
                f10 = 0.625F;
                gate.setBlockBounds(f2, 0.375F, f6, f10, 0.9375F, f14);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                f2 = 0.625F;
                f10 = 0.875F;
                gate.setBlockBounds(f2, 0.375F, f6, f10, 0.5625F, f14);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                gate.setBlockBounds(f2, 0.75F, f6, f10, 0.9375F, f14);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                f2 = 0.125F;
                f10 = 0.375F;
                gate.setBlockBounds(f2, 0.375F, f6, f10, 0.5625F, f14);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                gate.setBlockBounds(f2, 0.75F, f6, f10, 0.9375F, f14);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
            } else {
                float f3 = 0.4375F;
                float f11 = 0.5625F;
                float f7 = 0.375F;
                float f15 = 0.5F;
                gate.setBlockBounds(f3, 0.375F, f7, f11, 0.9375F, f15);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                f7 = 0.5F;
                f15 = 0.625F;
                gate.setBlockBounds(f3, 0.375F, f7, f11, 0.9375F, f15);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                f7 = 0.625F;
                f15 = 0.875F;
                gate.setBlockBounds(f3, 0.375F, f7, f11, 0.5625F, f15);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                gate.setBlockBounds(f3, 0.75F, f7, f11, 0.9375F, f15);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                f7 = 0.125F;
                f15 = 0.375F;
                gate.setBlockBounds(f3, 0.375F, f7, f11, 0.5625F, f15);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
                gate.setBlockBounds(f3, 0.75F, f7, f11, 0.9375F, f15);
                RenderTools.renderStandardBlock(render, gate, i, j, k);
            }
        } else if (meta == 1 && !track.isReversed()) {
            gate.setBlockBounds(0.8125F, 0.375F, 0.0F, 0.9375F, 0.9375F, 0.125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.8125F, 0.375F, 0.875F, 0.9375F, 0.9375F, 1.0F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.5625F, 0.375F, 0.0F, 0.8125F, 0.5625F, 0.125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.5625F, 0.375F, 0.875F, 0.8125F, 0.5625F, 1.0F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.5625F, 0.75F, 0.0F, 0.8125F, 0.9375F, 0.125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.5625F, 0.75F, 0.875F, 0.8125F, 0.9375F, 1.0F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
        } else if (meta == 1 && track.isReversed()) {
            gate.setBlockBounds(0.0625F, 0.375F, 0.0F, 0.1875F, 0.9375F, 0.125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.0625F, 0.375F, 0.875F, 0.1875F, 0.9375F, 1.0F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.1875F, 0.375F, 0.0F, 0.4375F, 0.5625F, 0.125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.1875F, 0.375F, 0.875F, 0.4375F, 0.5625F, 1.0F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.1875F, 0.75F, 0.0F, 0.4375F, 0.9375F, 0.125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.1875F, 0.75F, 0.875F, 0.4375F, 0.9375F, 1.0F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
        } else if (meta == 0 && track.isReversed()) {
            gate.setBlockBounds(0.0F, 0.375F, 0.8125F, 0.125F, 0.9375F, 0.9375F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.875F, 0.375F, 0.8125F, 1.0F, 0.9375F, 0.9375F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.0F, 0.375F, 0.5625F, 0.125F, 0.5625F, 0.8125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.875F, 0.375F, 0.5625F, 1.0F, 0.5625F, 0.8125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.0F, 0.75F, 0.5625F, 0.125F, 0.9375F, 0.8125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.875F, 0.75F, 0.5625F, 1.0F, 0.9375F, 0.8125F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
        } else if (meta == 0 && !track.isReversed()) {
            gate.setBlockBounds(0.0F, 0.375F, 0.0625F, 0.125F, 0.9375F, 0.1875F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.875F, 0.375F, 0.0625F, 1.0F, 0.9375F, 0.1875F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.0F, 0.375F, 0.1875F, 0.125F, 0.5625F, 0.4375F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.875F, 0.375F, 0.1875F, 1.0F, 0.5625F, 0.4375F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.0F, 0.75F, 0.1875F, 0.125F, 0.9375F, 0.4375F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
            gate.setBlockBounds(0.875F, 0.75F, 0.1875F, 1.0F, 0.9375F, 0.4375F);
            RenderTools.renderStandardBlock(render, gate, i, j, k);
        }
        gate.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
    }
}
