/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.client.render.carts;

import java.util.HashMap;
import java.util.Map;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelMinecart;
import org.lwjgl.opengl.GL11;
import railcraft.common.api.carts.IMinecartTextureProvider;
import railcraft.common.carts.EntityLocomotiveSteam;
import railcraft.client.render.models.locomotives.ModelLocomotiveSteam;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class CartModelManager {

    private final static ModelBase modelMinecart = new ModelMinecart();
    private final static Map<Class, ModelBase> models = new HashMap<Class, ModelBase>();
    private final static Map<Class, CartModelRenderer> renderers = new HashMap<Class, CartModelRenderer>();

    static {
        models.put(EntityLocomotiveSteam.class, new ModelLocomotiveSteam());
        renderers.put(EntityLocomotiveSteam.class, new LocomotiveRenderer());
    }

    public static ModelBase getModel(Class cartClass) {
        return models.get(cartClass);
    }

    public static void render(RenderCart renderer, EntityMinecart cart, float light) {
        GL11.glScalef(-1F, -1F, 1.0F);

        CartModelRenderer customRenderer = renderers.get(cart.getClass());
        if (customRenderer != null) {
            customRenderer.render(renderer, cart, light);
            return;
        }

        int j = 0xffffff;
        float c1 = (float) (j >> 16 & 0xff) / 255F;
        float c2 = (float) (j >> 8 & 0xff) / 255F;
        float c3 = (float) (j & 0xff) / 255F;

        GL11.glColor4f(c1 * light, c2 * light, c3 * light, 1.0F);

        String texture = null;
        if (cart instanceof IMinecartTextureProvider) {
            texture = ((IMinecartTextureProvider) cart).getTextureFile();
        }

        if (texture == null) {
            texture = "/item/cart.png";
        }

        ModelBase model = models.get(cart.getClass());
        if (model == null) {
            model = modelMinecart;
        }

        renderer.bindTexture(texture);
        model.render(cart, 0.0F, 0.0F, -0.1F, 0.0F, 0.0F, 0.0625F);
    }
}
