package railcraft.client.render.carts;

import java.util.Random;
import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.util.MathHelper;
import net.minecraft.client.renderer.entity.Render;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.util.Vec3;
import net.minecraftforge.client.ForgeHooksClient;
import org.lwjgl.opengl.GL11;
import railcraft.client.render.RenderFakeBlock;
import railcraft.common.api.carts.ICartRenderInterface;
import railcraft.common.api.carts.ICartTextureInterface;
import railcraft.common.api.carts.IExplosiveCart;
import railcraft.common.carts.EntityCartChest;
import railcraft.common.carts.EntityCartGift;
import railcraft.common.carts.EntityCartTank;
import railcraft.common.carts.IDirectionalCart;
import railcraft.common.carts.MaintanceCartBase;
import railcraft.common.util.misc.FakeBlockRenderInfo;

public class RenderCart extends Render {

    private final Random rand = new Random();
    private SubRenderCartTank renderCartTank = new SubRenderCartTank();
    private SubRenderCartTNT renderCartTNT = new SubRenderCartTNT(this);
    private SubRenderCartGift renderCartGift = new SubRenderCartGift(this);
    private SubRenderCartMaintance renderCartMaintance = new SubRenderCartMaintance(this);
    private final FakeBlockRenderInfo fakeBlock = new FakeBlockRenderInfo();

    public RenderCart() {
        shadowSize = 0.5F;
        fakeBlock.texture = new int[6];
    }

    public void renderCart(EntityMinecart cart, double x, double y, double z, float yaw, float time) {
        GL11.glPushMatrix();
        long var10 = (long) cart.entityId * 493286711L;
        var10 = var10 * var10 * 4392167121L + var10 * 98761L;
        float tx = (((float) (var10 >> 16 & 7L) + 0.5F) / 8.0F - 0.5F) * 0.004F;
        float ty = (((float) (var10 >> 20 & 7L) + 0.5F) / 8.0F - 0.5F) * 0.004F;
        float tz = (((float) (var10 >> 24 & 7L) + 0.5F) / 8.0F - 0.5F) * 0.004F;
        GL11.glTranslatef(tx, ty, tz);
        double mx = cart.lastTickPosX + (cart.posX - cart.lastTickPosX) * (double) time;
        double my = cart.lastTickPosY + (cart.posY - cart.lastTickPosY) * (double) time;
        double mz = cart.lastTickPosZ + (cart.posZ - cart.lastTickPosZ) * (double) time;
        double d6 = 0.3;
        Vec3 vec3d = cart.func_70489_a(mx, my, mz);
        float pitch = cart.prevRotationPitch + (cart.rotationPitch - cart.prevRotationPitch) * time;
        if (vec3d != null) {
            Vec3 vec3d1 = cart.func_70495_a(mx, my, mz, d6);
            Vec3 vec3d2 = cart.func_70495_a(mx, my, mz, -d6);
            if (vec3d1 == null) {
                vec3d1 = vec3d;
            }
            if (vec3d2 == null) {
                vec3d2 = vec3d;
            }
            x += vec3d.xCoord - mx;
            y += (vec3d1.yCoord + vec3d2.yCoord) / 2D - my;
            z += vec3d.zCoord - mz;
            Vec3 vec3d3 = vec3d2.addVector(-vec3d1.xCoord, -vec3d1.yCoord, -vec3d1.zCoord);
            if (vec3d3.lengthVector() != 0.0D) {
                vec3d3 = vec3d3.normalize();
                yaw = (float) (Math.atan2(vec3d3.zCoord, vec3d3.xCoord) / Math.PI) * 180;
                pitch = (float) (Math.atan(vec3d3.yCoord) * 73D);
            }
        }
        if (cart instanceof IDirectionalCart) {
            yaw %= 360;
            if (yaw < 0) {
                yaw += 360;
            }
            yaw += 360;

            double serverYaw = cart.rotationYaw;
            serverYaw += 180;
            serverYaw %= 360;
            if (serverYaw < 0) {
                serverYaw += 360;
            }
            serverYaw += 360;

            if (Math.abs(yaw - serverYaw) > 90) {
                yaw += 180;
                pitch = -pitch;
            }

//            EntityLocomotiveSteam loco = (EntityLocomotiveSteam) cart;
//            if (Math.abs(yaw - loco.prevYaw) > 90) {
//                yaw += 180;
//                pitch = -pitch;
//            }
//            yaw %= 360;
//            yaw += 360;
//            loco.prevYaw = yaw;

//            Vec3 heading = Vec3.vec3dPool.getVecFromPool(cart.posX - cart.lastTickPosX, 0, cart.posZ - cart.lastTickPosZ);
//            heading = heading.normalize();
//            float newYaw = (float) (Math.atan2(heading.zCoord, heading.xCoord) / Math.PI * 180D) + 180f;
//            newYaw %= 360;
//            if (newYaw < 0) {
//                newYaw += 360;
//            }
//            newYaw += 360;

//            if (Math.abs(yaw - newYaw) > 90) {
//                yaw += 180;
//                pitch = -pitch;
//            }

            ((IDirectionalCart) cart).setRenderYaw(yaw);
        }
        GL11.glTranslatef((float) x, (float) y, (float) z);
        GL11.glRotatef(180F - yaw, 0.0F, 1.0F, 0.0F);
        GL11.glRotatef(-pitch, 0.0F, 0.0F, 1.0F);

        float f3 = (float) cart.getRollingAmplitude() - time;
        float f4 = (float) cart.getDamage() - time;
        if (f4 < 0.0F) {
            f4 = 0.0F;
        }
        if (f3 > 0.0F) {
            float angle = (MathHelper.sin(f3) * f3 * f4) / 10F;
            angle = Math.min(angle, 0.8f);
            angle = Math.copySign(angle, cart.getRollingDirection());
            GL11.glRotatef(angle, 1.0F, 0.0F, 0.0F);
        }
        float light = cart.getBrightness(time);
        light = light + ((1.0f - light) * 0.4f);

        float f5 = 0.75F;
        GL11.glScalef(f5, f5, f5);
        GL11.glPushAttrib(GL11.GL_LIGHTING_BIT);
        if (cart instanceof EntityCartChest) {
            renderChestCart(light);
        } else if (cart instanceof EntityCartGift) {
            renderCartGift.render(cart, light, time);
        } else if (cart instanceof IExplosiveCart && cart instanceof ICartRenderInterface) {
            renderCartTNT.render(cart, light, time);
        } else if (cart instanceof ICartTextureInterface) {
            renderFakeBlockInCart((ICartTextureInterface) cart, light);
        } else if (cart instanceof ICartRenderInterface) {
            renderBlockInCart((ICartRenderInterface) cart, light);
        } else if (cart instanceof MaintanceCartBase) {
            renderCartMaintance.render(cart, light, time);
        } else if (cart instanceof EntityCartTank) {
            renderCartTank.render(cart, light, time);
        }

        GL11.glScalef(1.0F / f5, 1.0F / f5, 1.0F / f5);
        GL11.glPopAttrib();

        CartModelManager.render(this, cart, light);
        GL11.glPopMatrix();
    }

    @Override
    public void doRender(Entity entity, double x, double y, double d2, float yaw, float time) {
        renderCart((EntityMinecart) entity, x, y, d2, yaw, time);
    }

    public void bindTexture(String texture) {
        loadTexture(texture);
    }

    public RenderBlocks renderBlocks() {
        return renderBlocks;
    }

    public void renderChestCart(float light) {
        loadTexture("/terrain.png");
        GL11.glTranslatef(0.0F, 0.5F, 0.0F);
        renderBlocks.renderBlockAsItem(Block.chest, 0, light);
        GL11.glRotatef(-90F, 0.0F, 1.0F, 0.0F);
        GL11.glTranslatef(0.5F, 0.0F, -0.5F);
    }

    public void renderBlockInCart(ICartRenderInterface cart, float light) {
        GL11.glTranslatef(0.0F, 0.3125F, 0.0F);
        Block block = cart.getBlock();
        ForgeHooksClient.bindTexture(block.getTextureFile(), 0);
        renderBlocks.renderBlockAsItem(block, cart.getBlockMetadata(), light);
        GL11.glRotatef(-90F, 0.0F, 1.0F, 0.0F);
        GL11.glTranslatef(0.0F, -0.3125F, 0.0F);
    }

    public void renderFakeBlockInCart(ICartTextureInterface cart, float light) {
        EntityMinecart entity = (EntityMinecart) cart;
        GL11.glTranslatef(0.0F, 0.3125F, 0.0F);
        GL11.glDisable(2896 /*GL_LIGHTING*/);
        Block block = cart.getBlock();
        ForgeHooksClient.bindTexture(block.getTextureFile(), 0);
        for (int side = 0; side < 6; side++) {
            fakeBlock.texture[side] = cart.getBlockTextureOnSide(side);
        }
        RenderFakeBlock.renderBlockForEntity(fakeBlock, entity.worldObj, (int) entity.posX, (int) entity.posY, (int) entity.posZ, true, true);
//        GL11.glRotatef(-90F, 0.0F, 1.0F, 0.0F);
        GL11.glTranslatef(0.0F, -0.3125F, 0.0F);
    }
}
