/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.client.render.carts;

import net.minecraft.block.Block;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.client.ForgeHooksClient;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.LiquidStack;
import org.lwjgl.opengl.GL11;
import railcraft.client.render.LiquidRenderer;
import railcraft.client.render.RenderFakeBlock;
import railcraft.common.carts.EntityCartTank;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.util.misc.FakeBlockRenderInfo;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class SubRenderCartTank extends SubRenderCart {

    private static final int[] TANK_TEXTURE = new int[]{230, 230, 229, 229, 229, 229};
    private final FakeBlockRenderInfo fakeBlock = new FakeBlockRenderInfo();
    private final FakeBlockRenderInfo fillBlock = new FakeBlockRenderInfo(0.4f, 0.0f, 0.4f, 0.6f, 0.999f, 0.6f);
    private final FakeBlockRenderInfo bucketSign = new FakeBlockRenderInfo();
    private static final float FILTER_SCALE_X = 1.38f;
    private static final float FILTER_SCALE_Y = 0.5f;
    private static final float FILTER_SCALE_Z = 0.5f;

    public SubRenderCartTank() {
        bucketSign.template = Block.glass;
        bucketSign.renderTop = false;
        bucketSign.renderBottom = false;
        bucketSign.renderEast = false;
        bucketSign.renderWest = false;
    }

    @Override
    public void render(EntityMinecart cart, float light, float time) {
        EntityCartTank cartTank = (EntityCartTank) cart;
        GL11.glPushMatrix();
        GL11.glPushAttrib(GL11.GL_ENABLE_BIT);
        GL11.glTranslatef(0.0F, 0.3125F, 0.0F);
        GL11.glRotatef(90F, 0.0F, 1.0F, 0.0F);
        GL11.glDisable(2896 /*GL_LIGHTING*/);

        int x = (int) (Math.floor(cart.posX));
        int y = (int) (Math.floor(cart.posY));
        int z = (int) (Math.floor(cart.posZ));

        ILiquidTank tank = cartTank.getTankManager().getTank(0);
        LiquidStack liquid = tank.getLiquid();
        if (liquid != null && liquid.amount > 0) {
            int[] displayLists = LiquidRenderer.getLiquidDisplayLists(liquid);
            if (displayLists != null) {
                GL11.glPushMatrix();

                String textureFile;

                if (liquid.itemID < Block.blocksList.length && Block.blocksList[liquid.itemID] != null) {
                    textureFile = Block.blocksList[liquid.itemID].getTextureFile();
                } else {
                    textureFile = Item.itemsList[liquid.itemID].getTextureFile();
                }

                ForgeHooksClient.bindTexture(textureFile, 0);


                float cap = tank.getCapacity();
                float level = (float) Math.min(liquid.amount, cap) / cap;

                GL11.glCallList(displayLists[(int) (level * (float) (LiquidRenderer.DISPLAY_STAGES - 1))]);

                if (cartTank.isFilling()) {
                    if (liquid.itemID < Block.blocksList.length && Block.blocksList[liquid.itemID] != null) {
                        fillBlock.texture[0] = Block.blocksList[liquid.itemID].getBlockTextureFromSideAndMetadata(1, liquid.itemMeta);
                    } else {
                        fillBlock.texture[0] = Item.itemsList[liquid.itemID].getIconFromDamage(liquid.itemMeta);
                    }
                    RenderFakeBlock.renderBlockForEntity(fillBlock, cart.worldObj, x, y, z, false, true);
                }

                GL11.glPopMatrix();
            }
        }
        ForgeHooksClient.bindTexture(RailcraftConstants.MAIN_TEXTURE_FILE, 0);

        fakeBlock.texture = TANK_TEXTURE;
        RenderFakeBlock.renderBlockForEntity(fakeBlock, cart.worldObj, x, y, z, true, true);

        ItemStack bucket = cartTank.getFilterItem();

        if (bucket != null && bucket.itemID > 0) {
            ForgeHooksClient.bindTexture(bucket.getItem().getTextureFile(), 0);

            GL11.glPushMatrix();
            GL11.glScalef(FILTER_SCALE_X, FILTER_SCALE_Y, FILTER_SCALE_Z);
            GL11.glTranslatef(0, -0.4f, 0);

            int meta = bucket.getItemDamage();
            for (int pass = 0; pass < bucket.getItem().getRenderPasses(meta); ++pass) {
                int texture = bucket.getItem().getIconFromDamageForRenderPass(meta, pass);
                int color = bucket.getItem().getColorFromItemStack(bucket, pass);

                float c1 = (float) (color >> 16 & 255) / 255.0F;
                float c2 = (float) (color >> 8 & 255) / 255.0F;
                float c3 = (float) (color & 255) / 255.0F;

                float dim = 0.7f;

                GL11.glColor4f(c1 * light * dim, c2 * light * dim, c3 * light * dim, 1.0F);

                Tessellator tess = Tessellator.instance;                
                tess.setBrightness(bucketSign.template.getMixedBrightnessForBlock(cart.worldObj, x, y, z));
                
                bucketSign.texture[0] = texture;
                RenderFakeBlock.renderBlockForEntity(bucketSign, cart.worldObj, x, y, z, false, true);
            }

            GL11.glPopMatrix();
        }

        GL11.glPopAttrib();
        GL11.glPopMatrix();
    }
}
