package railcraft.client.sounds;

import java.net.URL;
import java.util.logging.Level;
import net.minecraft.entity.Entity;
import net.minecraft.util.MathHelper;
import net.minecraft.block.StepSound;
import net.minecraft.client.audio.SoundPool;
import net.minecraft.world.World;
import net.minecraftforge.client.event.sound.PlaySoundEvent;
import net.minecraftforge.client.event.sound.SoundLoadEvent;
import net.minecraftforge.event.ForgeSubscribe;
import net.minecraftforge.event.entity.PlaySoundAtEntityEvent;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.util.misc.Game;
import railcraft.common.util.sounds.SoundRegistry;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class SoundHandler {

    private static SoundHandler instance;

    public static SoundHandler getInstance() {
        if (instance == null) {
            instance = new SoundHandler();
        }
        return instance;
    }

    private SoundHandler() {
    }

    @ForgeSubscribe
    public void loadSounds(SoundLoadEvent event) {
        SoundPool pool = event.manager.soundPoolSounds;

        addSound(pool, "locomotive/steam/whistle1.ogg");
        addSound(pool, "locomotive/steam/whistle2.ogg");
        addSound(pool, "locomotive/steam/whistle3.ogg");

        addSound(pool, "machine/steamburst.ogg");
    }

    private void addSound(SoundPool pool, String path) {
        try {
            URL sound = Railcraft.class.getResource(RailcraftConstants.SOUND_FOLDER + path);
            if (sound != null) {
                pool.addSound(path, sound);
            } else throw new RuntimeException("Resource not found");
        } catch (Exception ex) {
            Game.logError("Failed to load sound: " + path, ex);
        }
    }

    @ForgeSubscribe
    public void onPlaySound(PlaySoundEvent event) {
        String soundName = event.name;
        if (soundName != null && (soundName.equals("dig.railcraft") || soundName.equals("place.railcraft"))) {
            World world = Railcraft.getProxy().getClientWorld();
            if (world != null) {
                int ix = MathHelper.floor_float(event.x);
                int iy = MathHelper.floor_float(event.y);
                int iz = MathHelper.floor_float(event.z);
                int blockID = world.getBlockId(ix, iy, iz);
                int meta = world.getBlockMetadata(ix, iy, iz);
                StepSound sound = SoundRegistry.getSound(blockID, meta);
                if (sound != null) {
                    String newName = soundName.contains("dig") ? sound.getBreakSound() : sound.getPlaceSound();
                    event.manager.playSound(newName, event.x, event.y, event.z, event.volume, event.pitch * sound.getPitch());
                    event.result = null;
                }
            }
        }
    }

    @ForgeSubscribe
    public void onPlaySoundAtEntity(PlaySoundAtEntityEvent event) {
        String soundName = event.name;
        Entity entity = event.entity;
        if (soundName != null && soundName.equals("step.railcraft")) {
            World world = entity.worldObj;
            if (world != null) {
                int ix = MathHelper.floor_double(entity.posX);
                int iy = MathHelper.floor_double(entity.posY - 0.2 - (double) entity.yOffset);
                int iz = MathHelper.floor_double(entity.posZ);
                int blockID = world.getBlockId(ix, iy, iz);
                int meta = world.getBlockMetadata(ix, iy, iz);
                StepSound sound = SoundRegistry.getSound(blockID, meta);
                if (sound != null) {
                    world.playSoundAtEntity(entity, sound.getStepSound(), event.volume, event.pitch * sound.getPitch());
                    event.setCanceled(true);
                }
            }
        }
    }
}
