package railcraft.client.textureFX;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.client.renderer.RenderEngine;
import net.minecraft.util.MathHelper;
import org.lwjgl.opengl.GL11;
import railcraft.common.liquids.LiquidItems;

@SideOnly(Side.CLIENT)
public class TextureSteamFX extends RailcraftTextureFX
{

    private final int redMin, redMax, greenMin, greenMax, blueMin, blueMax;
    protected float[] red = new float[256];
    protected float[] green = new float[256];
    protected float[] blue = new float[256];
    protected float[] alpha = new float[256];

    public TextureSteamFX()
    {
        super(LiquidItems.getSteam().getIconIndex(), LiquidItems.getSteam().getItem().getTextureFile());

        redMin = greenMin = blueMin = 100;
        redMax = greenMax = blueMax = 175;

        setup();
    }

    @Override
    public final void setup()
    {
        super.setup();
        red = new float[tileSizeSquare];
        green = new float[tileSizeSquare];
        blue = new float[tileSizeSquare];
        alpha = new float[tileSizeSquare];
    }

    @Override
    public void bindImage(RenderEngine renderengine)
    {
        GL11.glBindTexture(3553 /*GL_TEXTURE_2D*/, this.tileImage);
    }

    @Override
    public void onTick()
    {
        int j;
        float f1;
        int r;
        int g;
        int b;
        int var8;
        int var9;

        for(int i = 0; i < tileSizeBase; ++i) {
            for(j = 0; j < tileSizeBase; ++j) {
                f1 = 0.0F;
                int var4 = (int)(MathHelper.sin((float)j * (float)Math.PI * 2.0F / 16.0F) * 1.2F);
                r = (int)(MathHelper.sin((float)i * (float)Math.PI * 2.0F / 16.0F) * 1.2F);

                for(g = i - 1; g <= i + 1; ++g) {
                    for(b = j - 1; b <= j + 1; ++b) {
                        var8 = g + var4 & tileSizeMask;
                        var9 = b + r & tileSizeMask;
                        f1 += this.red[var8 + var9 * tileSizeBase];
                    }
                }

                this.green[i + j * tileSizeBase] = f1 / 10.0F + (this.blue[(i + 0 & tileSizeMask) + (j + 0 & tileSizeMask) * tileSizeBase] + this.blue[(i + 1 & tileSizeMask) + (j + 0 & tileSizeMask) * tileSizeBase] + this.blue[(i + 1 & tileSizeMask) + (j + 1 & tileSizeMask) * tileSizeBase] + this.blue[(i + 0 & tileSizeMask) + (j + 1 & tileSizeMask) * tileSizeBase]) / 4.0F * 0.8F;
                this.blue[i + j * tileSizeBase] += this.alpha[i + j * tileSizeBase] * 0.01F;

                if(this.blue[i + j * tileSizeBase] < 0.0F) {
                    this.blue[i + j * tileSizeBase] = 0.0F;
                }

                this.alpha[i + j * tileSizeBase] -= 0.06F;

                if(Math.random() < 0.005D) {
                    this.alpha[i + j * tileSizeBase] = 1.5F;
                }
            }
        }

        float[] var11 = this.green;
        this.green = this.red;
        this.red = var11;

        for(j = 0; j < tileSizeSquare; ++j) {
            f1 = this.red[j] * 2.0F;

            if(f1 > 1.0F) {
                f1 = 1.0F;
            }

            if(f1 < 0.0F) {
                f1 = 0.0F;
            }

            float f2 = f1 * f1;
            r = (int)(redMin + f2 * (redMax - redMin));
            g = (int)(greenMin + f2 * (greenMax - greenMin));
            b = (int)(blueMin + f2 * (blueMax - blueMin));

            if(this.anaglyphEnabled) {
                var8 = (r * 30 + g * 59 + b * 11) / 100;
                var9 = (r * 30 + g * 70) / 100;
                int var10 = (r * 30 + b * 70) / 100;
                r = var8;
                g = var9;
                b = var10;
            }

            this.imageData[j * 4 + 0] = (byte)r;
            this.imageData[j * 4 + 1] = (byte)g;
            this.imageData[j * 4 + 2] = (byte)b;
            this.imageData[j * 4 + 3] = -1;
        }
    }
}
