/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.client.util.effects;

import cpw.mods.fml.client.FMLClientHandler;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Set;
import net.minecraft.client.Minecraft;
import net.minecraft.world.ChunkCoordIntPair;
import net.minecraft.client.particle.EntityFX;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import railcraft.client.core.AuraKeyHandler;
import railcraft.client.particles.EntityChunkLoaderFX;
import railcraft.client.particles.EntitySteamFX;
import railcraft.client.particles.EntityTuningFX;
import railcraft.common.api.signals.SignalTools;
import railcraft.common.items.ItemGoggles;
import railcraft.common.items.ItemGoggles.Aura;
import railcraft.common.util.effects.CommonProxy;
import railcraft.common.util.effects.EffectManager;
import railcraft.common.util.effects.EffectManager.IEffectSource;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.PacketEffect.Effect;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ClientProxy extends CommonProxy {

    public static final short TELEPORT_PARTICLES = 64;

    public ClientProxy() {
        SignalTools.effectManager = this;
    }

    private void doTeleport(DataInputStream data) throws IOException {

        double startX = data.readDouble();
        double startY = data.readDouble();
        double startZ = data.readDouble();
        double destX = data.readDouble();
        double destY = data.readDouble();
        double destZ = data.readDouble();

//        for(int i = 0; i < TELEPORT_PARTICLES / 4; i++) {
//            float vX = (rand.nextFloat() - 0.5F) * 0.2F;
//            float vY = (rand.nextFloat() - 0.5F) * 0.2F;
//            float vZ = (rand.nextFloat() - 0.5F) * 0.2F;
//            Game.getWorld().spawnParticle("portal", startX, startY, startZ, vX, vY, vZ);
//        }

        for (int i = 0; i < TELEPORT_PARTICLES; i++) {
            double travel = (double) i / ((double) TELEPORT_PARTICLES - 1.0D);
            float vX = (rand.nextFloat() - 0.5F) * 0.2F;
            float vY = (rand.nextFloat() - 0.5F) * 0.2F;
            float vZ = (rand.nextFloat() - 0.5F) * 0.2F;
            double pX = startX + (destX - startX) * travel + (rand.nextDouble() - 0.5D) * 2.0D;
            double pY = startY + (destY - startY) * travel + (rand.nextDouble() - 0.5D) * 2.0D;
            double pZ = startZ + (destZ - startZ) * travel + (rand.nextDouble() - 0.5D) * 2.0D;
            Game.getWorld().spawnParticle("portal", pX, pY, pZ, vX, vY, vZ);
        }
    }

    @Override
    public boolean isAnchorAuraActive() {
        if (ItemGoggles.areEnabled()) {
            ItemStack goggles = ItemGoggles.getGoggles(Minecraft.getMinecraft().thePlayer);
            return ItemGoggles.getCurrentAura(goggles) == Aura.ANCHOR;
        }
        return AuraKeyHandler.isAnchorAuraEnabled();
    }

    @Override
    public boolean isTuningAuraActive() {
        if (ItemGoggles.areEnabled()) {
            ItemStack goggles = ItemGoggles.getGoggles(Minecraft.getMinecraft().thePlayer);
            return ItemGoggles.getCurrentAura(goggles) == Aura.TUNING;
        }
        return AuraKeyHandler.isTuningAuraEnabled();
    }

    @Override
    public void tuningEffect(TileEntity start, TileEntity dest) {
        if (rand.nextInt(2) == 0) {
            double px = start.xCoord + 0.5 + rand.nextGaussian() * 0.1;
            double py = start.yCoord + 0.5 + rand.nextGaussian() * 0.1;
            double pz = start.zCoord + 0.5 + rand.nextGaussian() * 0.1;
            EntityFX particle = new EntityTuningFX(start.worldObj, px, py, pz, EffectManager.getEffectSource(dest));
            spawnParticle(particle);
        }
    }

    @Override
    public void handleEffectPacket(DataInputStream data) throws IOException {

        byte effectId = data.readByte();
        if (effectId < 0) {
            return;
        }

        Effect effect = Effect.values()[effectId];
        switch (effect) {
            case TELEPORT:
                doTeleport(data);
                break;
        }
    }

    @Override
    public void chunkLoaderEffect(World world, Object source, Set<ChunkCoordIntPair> chunks) {
        if (!isAnchorAuraActive()) {
            return;
        }
        IEffectSource es = EffectManager.getEffectSource(source);
        if (FMLClientHandler.instance().getClient().thePlayer.getDistanceSq(es.getX(), es.getY(), es.getZ()) > 25600) {
            return;
        }

        for (ChunkCoordIntPair chunk : chunks) {
            int xCorner = chunk.chunkXPos * 16;
            int zCorner = chunk.chunkZPos * 16;
            double yCorner = es.getY() - 8;

//            System.out.println(xCorner + ", " + zCorner);

            if (rand.nextInt(3) == 0) {
                double xParticle = xCorner + rand.nextFloat() * 16;
                double yParticle = yCorner + rand.nextFloat() * 16;
                double zParticle = zCorner + rand.nextFloat() * 16;

                EntityFX particle = new EntityChunkLoaderFX(world, xParticle, yParticle, zParticle, es);
                spawnParticle(particle);
            }
        }
    }

    @Override
    public void steamEffect(World world, Object source, double yOffset) {
        IEffectSource es = EffectManager.getEffectSource(source);
        double vx = rand.nextGaussian() * 0.1;
        double vy = rand.nextDouble() * 0.01;
        double vz = rand.nextGaussian() * 0.1;
        spawnParticle(new EntitySteamFX(world, es.getX(), es.getY() + yOffset, es.getZ(), vx, vy, vz));
    }

    @Override
    protected void spawnParticle(EntityFX particle) {
        Minecraft mc = FMLClientHandler.instance().getClient();
        mc.effectRenderer.addEffect(particle);
    }
}
