package railcraft.common.blocks;

import cpw.mods.fml.common.registry.GameRegistry;
import java.util.Arrays;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecraftForge;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.detector.BlockDetector;
import railcraft.common.blocks.detector.BlockDetector.Detector;
import railcraft.common.blocks.detector.ItemDetector;
import railcraft.common.blocks.machine.BlockMachine;
import railcraft.common.blocks.machine.ItemMachine;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.machine.alpha.MachineProxyAlpha;
import railcraft.common.blocks.machine.beta.EnumMachineBeta;
import railcraft.common.blocks.machine.beta.MachineProxyBeta;
import railcraft.common.blocks.machine.gamma.EnumMachineGamma;
import railcraft.common.blocks.machine.gamma.MachineProxyGamma;
import railcraft.common.blocks.aesthetics.post.BlockPost;
import railcraft.common.blocks.aesthetics.post.BlockPostMetal;
import railcraft.common.blocks.aesthetics.post.EnumPost;
import railcraft.common.blocks.aesthetics.post.ItemPost;
import railcraft.common.blocks.aesthetics.post.ItemPostMetal;
import railcraft.common.blocks.signals.BlockSignal;
import railcraft.common.blocks.signals.ItemSignal;
import railcraft.common.blocks.tracks.BlockTrack;
import railcraft.common.blocks.tracks.BlockTrackElevator;
import railcraft.common.blocks.tracks.ItemTrack;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forestry.ForestryPlugin;

public class RailcraftBlocks {

    private static Block blockDetector;
    private static Block blockMachineAlpha;
    private static Block blockMachineBeta;
    private static Block blockMachineGamma;
    private static Block blockTrack;
    private static Block blockRailElevator;
    private static Block blockSignal;
    private static Block blockPost;
    private static Block blockPostMetal;

    public static void registerBlockTrack() {
        if (blockTrack == null) {
            int id = RailcraftConfig.getBlockId("block.track");

            if (id > 0) {
                int renderId = Railcraft.getProxy().getRenderId();
                blockTrack = new BlockTrack(id, renderId).setBlockName("railcraftTrack");
                GameRegistry.registerBlock(blockTrack, ItemTrack.class, blockTrack.getBlockName());
                MinecraftForge.setBlockHarvestLevel(blockTrack, "crowbar", 0);
                MinecraftForge.setBlockHarvestLevel(blockTrack, "pickaxe", 0);
            }
        }
    }

    public static Block getBlockTrack() {
        return blockTrack;
    }

    public static void registerBlockRailElevator() {
        if (blockRailElevator == null) {
            int id = RailcraftConfig.getBlockId("block.elevator");

            // Define Elevator Rail
            if (id > 0) {
                int renderId = Railcraft.getProxy().getRenderId();
                blockRailElevator = new BlockTrackElevator(id, renderId).setBlockName("elevatorRail");
                GameRegistry.registerBlock(blockRailElevator, blockRailElevator.getBlockName());
                MinecraftForge.setBlockHarvestLevel(blockRailElevator, "crowbar", 0);
                MinecraftForge.setBlockHarvestLevel(blockRailElevator, "pickaxe", 0);
                ItemStack stackElevator = new ItemStack(blockRailElevator, 8);
                GameRegistry.addRecipe(stackElevator, new Object[]{
                            "IRI",
                            "ISI",
                            "IRI",
                            'I', RailcraftPartItems.getRailAdvanced(),
                            'S', RailcraftPartItems.getRailStandard(),
                            'R', Item.redstone,});
                RailcraftLanguage.getInstance().registerItemName(stackElevator, "track.elevator");

                ItemRegistry.registerItem("track.elevator", stackElevator);
            }
        }
    }

    public static Block getBlockElevator() {
        return blockRailElevator;
    }

    public static void registerBlockDetector() {
        if (blockDetector == null) {
            int id = RailcraftConfig.getBlockId("block.detector");

            if (id > 0) {
                blockDetector = new BlockDetector(id).setBlockName("cartDetector");
                GameRegistry.registerBlock(blockDetector, ItemDetector.class, blockDetector.getBlockName());

                for (Detector d : Detector.values()) {
                    ItemStack stack = new ItemStack(blockDetector, 1, d.ordinal());
                    String tag = BlockDetector.getBlockNameFromMetadata(d.ordinal());
                    RailcraftLanguage.getInstance().registerItemName(stack, tag);

                    ItemRegistry.registerItem(tag, stack);
                }
            }
        }
    }

    public static Block getBlockDetector() {
        return blockDetector;
    }

    public static void registerBlockMachineAlpha() {
        if (blockMachineAlpha == null) {
            int id = RailcraftConfig.getBlockId("block.machine.alpha");

            if (id > 0) {
                int[] lightOpacity = new int[16];
                Arrays.fill(lightOpacity, 255);
                blockMachineAlpha = new BlockMachine(id, 0, new MachineProxyAlpha(), true, lightOpacity).setBlockName("rcBlockMachineAlpha");
                GameRegistry.registerBlock(blockMachineAlpha, ItemMachine.class, blockMachineAlpha.getBlockName());

                for (EnumMachineAlpha type : EnumMachineAlpha.values()) {
                    switch (type) {
                        case FEED_STATION:
                        case TANK_WATER:
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "axe", 1);
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "crowbar", 0);
                            break;
                        case WORLD_ANCHOR:
                        case PERSONAL_ANCHOR:
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "pickaxe", 3);
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "crowbar", 0);
                            break;
                        default:
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "pickaxe", 2);
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "crowbar", 0);
                    }
                }
            }
        }
    }

    public static Block getBlockMachineAlpha() {
        return blockMachineAlpha;
    }

    public static void registerBlockMachineBeta() {
        if (blockMachineBeta == null) {
            int id = RailcraftConfig.getBlockId("block.machine.beta");

            if (id > 0) {
                int renderId = Railcraft.getProxy().getRenderId();
                int[] lightOpacity = new int[16];
                Arrays.fill(lightOpacity, 255);
                lightOpacity[EnumMachineBeta.ENGINE_STEAM_HOBBY.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.ENGINE_STEAM_LOW.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.ENGINE_STEAM_HIGH.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.TANK_IRON_WALL.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.TANK_IRON_VALVE.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.TANK_IRON_GAUGE.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.BOILER_TANK_LOW_PRESSURE.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.BOILER_TANK_HIGH_PRESSURE.ordinal()] = 0;
                lightOpacity[EnumMachineBeta.SENTINEL.ordinal()] = 0;
                blockMachineBeta = new BlockMachine(id, renderId, new MachineProxyBeta(), false, lightOpacity).setBlockName("rcBlockMachineBeta");
                GameRegistry.registerBlock(blockMachineBeta, ItemMachine.class, blockMachineBeta.getBlockName());

                for (EnumMachineBeta type : EnumMachineBeta.values()) {
                    switch (type) {
                        case SENTINEL:
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "pickaxe", 3);
                            MinecraftForge.setBlockHarvestLevel(blockMachineAlpha, type.ordinal(), "crowbar", 0);
                            break;
                        default:
                            MinecraftForge.setBlockHarvestLevel(blockMachineBeta, type.ordinal(), "pickaxe", 2);
                            MinecraftForge.setBlockHarvestLevel(blockMachineBeta, type.ordinal(), "crowbar", 0);
                    }
                }
            }
        }
    }

    public static Block getBlockMachineBeta() {
        return blockMachineBeta;
    }

    public static void registerBlockMachineGamma() {
        if (blockMachineGamma == null) {
            int id = RailcraftConfig.getBlockId("block.machine.gamma");

            if (id > 0) {
//                int renderId = ClientProxy.getRenderId(false);  
                int[] lightOpacity = new int[16];
                Arrays.fill(lightOpacity, 255);
                lightOpacity[EnumMachineGamma.LIQUID_LOADER.ordinal()] = 0;
                lightOpacity[EnumMachineGamma.LIQUID_UNLOADER.ordinal()] = 0;
                blockMachineGamma = new BlockMachine(id, 0, new MachineProxyGamma(), false, lightOpacity).setBlockName("rcBlockMachineGamma");
                blockMachineGamma.setCreativeTab(CreativeTabs.tabTransport);
                GameRegistry.registerBlock(blockMachineGamma, ItemMachine.class, blockMachineGamma.getBlockName());

                for (EnumMachineGamma type : EnumMachineGamma.values()) {
                    switch (type) {
                        default:
                            MinecraftForge.setBlockHarvestLevel(blockMachineGamma, type.ordinal(), "pickaxe", 2);
                            MinecraftForge.setBlockHarvestLevel(blockMachineGamma, type.ordinal(), "crowbar", 0);
                    }
                }
            }
        }
    }

    public static Block getBlockMachineGamma() {
        return blockMachineGamma;
    }

    public static void registerBlockSignal() {
        if (blockSignal == null) {
            int id = RailcraftConfig.getBlockId("block.signal");

            if (id > 0) {
                int renderId = Railcraft.getProxy().getRenderId();
                blockSignal = new BlockSignal(id, renderId);
                GameRegistry.registerBlock(blockSignal, ItemSignal.class, blockSignal.getBlockName());
                MinecraftForge.setBlockHarvestLevel(blockSignal, "pickaxe", 2);
                MinecraftForge.setBlockHarvestLevel(blockSignal, "crowbar", 0);
            }
        }
    }

    public static Block getBlockSignal() {
        return blockSignal;
    }

    public static void registerBlockPost() {
        if (blockPost == null) {
            int id = RailcraftConfig.getBlockId("block.post");

            if (id > 0) {
                blockPost = new BlockPost(id, Railcraft.getProxy().getRenderId());
                GameRegistry.registerBlock(blockPost, ItemPost.class, blockPost.getBlockName());

                for (EnumPost post : EnumPost.values()) {
                    ForestryPlugin.addBackpackItem("builder", post.getItem());
                }
            }
        }
    }

    public static Block getBlockPost() {
        return blockPost;
    }

    public static void registerBlockPostMetal() {
        if (blockPostMetal == null) {
            int id = RailcraftConfig.getBlockId("block.post.metal");

            if (id > 0) {
                blockPostMetal = new BlockPostMetal(id, Railcraft.getProxy().getRenderId());
                GameRegistry.registerBlock(blockPostMetal, ItemPostMetal.class, blockPostMetal.getBlockName());

                for (int i = 0; i < 16; i++) {
                    ForestryPlugin.addBackpackItem("builder", new ItemStack(blockPostMetal, 1, i));
                }
            }
        }
    }

    public static Block getBlockPostMetal() {
        return blockPostMetal;
    }
}
