package railcraft.common.blocks;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.block.Block;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.packet.Packet;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.core.INetworkedObject;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.network.PacketBuilder;
import railcraft.common.util.network.PacketTileEntity;
import railcraft.common.util.network.RailcraftPacket;

public abstract class RailcraftTileEntity extends TileEntity implements INetworkedObject {

    private String owner = "[Railcraft]";

    @Override
    public Packet getDescriptionPacket() {
//        System.out.println("Sending Tile Packet");
        RailcraftPacket packet = new PacketTileEntity(this);
        return packet.getPacket();
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
//        data.writeUTF(owner);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
//        owner = data.readUTF();
    }

    public void markBlockForUpdate() {
        worldObj.markBlockForUpdate(xCoord, yCoord, zCoord);
    }

    public void sendUpdateToClient() {
        PacketBuilder.getInstance().sendTileEntityPacket(this);
    }

    public void onBlockPlacedBy(EntityLiving entityliving) {
        if (entityliving instanceof EntityPlayer) {
            String name = ((EntityPlayer) entityliving).username;
            if (name != null) owner = name;
        }
    }

    public boolean isRedstonePowered() {
        for (ForgeDirection side : ForgeDirection.VALID_DIRECTIONS) {
            if (isRedstonePowering(0, side) || isRedstonePowering(-1, side)) {
                return true;
            }
        }
        return false;
    }

    private boolean isRedstonePowering(int yOffset, ForgeDirection side) {
        int id = MiscTools.getBlockIdOnSide(worldObj, xCoord, yCoord + yOffset, zCoord, side);
        if (id == Block.redstoneWire.blockID) {
            int meta = MiscTools.getBlockMetadataOnSide(worldObj, xCoord, yCoord + yOffset, zCoord, side);
            if (meta > 0) {
                return true;
            }
        }
        return false;
    }

    public Block getBlock() {
        if (this.blockType == null) {
            this.blockType = Block.blocksList[this.worldObj.getBlockId(this.xCoord, this.yCoord, this.zCoord)];
        }

        return this.blockType;
    }

    public final int getBlockId() {
        Block block = getBlock();
        if (block == null) {
            return 0;
        }
        return block.blockID;
    }

    public final int getDimension() {
        if (worldObj == null) {
            return 0;
        }
        return worldObj.provider.dimensionId;
    }

    public final String getOwner() {
        return owner;
    }

    public abstract String getInvName();

    public boolean isUseableByPlayer(EntityPlayer player) {
        if (worldObj.getBlockTileEntity(xCoord, yCoord, zCoord) != this) {
            return false;
        }
        if (player.getDistanceSq(xCoord, yCoord, zCoord) > 64) {
            return false;
        }
        return true;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setString("owner", owner);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        owner = data.getString("owner");
    }

    public final int getX() {
        return xCoord;
    }

    public final int getY() {
        return yCoord;
    }

    public final int getZ() {
        return zCoord;
    }

    @Override
    public final World getWorld() {
        return worldObj;
    }

    public abstract short getId();
}
