package railcraft.common.blocks.aesthetics.cube;

import cpw.mods.fml.common.registry.GameRegistry;
import java.util.List;
import java.util.Random;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.block.material.Material;
import net.minecraft.block.StepSound;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import railcraft.client.sounds.RailcraftSound;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.plugins.buildcraft.BuildcraftPlugin;
import railcraft.common.plugins.forestry.ForestryPlugin;
import railcraft.common.util.misc.FakeBlockRenderInfo;

public class BlockCube extends Block {

    private static BlockCube instance;
    private FakeBlockRenderInfo override;

    public static BlockCube getBlock() {
        return instance;
    }

    public static void registerBlock() {
        if (instance == null) {
            int id = RailcraftConfig.getBlockId("block.cube");

            if (id > 0) {
                instance = new BlockCube(id);
                GameRegistry.registerBlock(instance, ItemCube.class, instance.getBlockName());

                MinecraftForge.setBlockHarvestLevel(instance, EnumCube.STEEL_BLOCK.ordinal(), "pickaxe", 2);
                MinecraftForge.setBlockHarvestLevel(instance, EnumCube.CONCRETE_BLOCK.ordinal(), "pickaxe", 1);
                MinecraftForge.setBlockHarvestLevel(instance, EnumCube.CONCRETE_BLOCK.ordinal(), "crowbar", 0);
                MinecraftForge.setBlockHarvestLevel(instance, EnumCube.CRUSHED_OBSIDIAN.ordinal(), "shovel", 3);

                ForestryPlugin.addBackpackItem("builder", EnumCube.CONCRETE_BLOCK.getItem());
                ForestryPlugin.addBackpackItem("builder", EnumCube.INFERNAL_BRICK.getItem());
                ForestryPlugin.addBackpackItem("builder", EnumCube.SANDY_BRICK.getItem());

                BuildcraftPlugin.addFacade(instance, EnumCube.CONCRETE_BLOCK.ordinal());
                BuildcraftPlugin.addFacade(instance, EnumCube.INFERNAL_BRICK.ordinal());
                BuildcraftPlugin.addFacade(instance, EnumCube.SANDY_BRICK.ordinal());
                BuildcraftPlugin.addFacade(instance, EnumCube.STEEL_BLOCK.ordinal());
            }
        }
    }

    public BlockCube(int id) {
        super(id, Material.rock);
        setBlockName("rcCube");
        setResistance(20);
        setHardness(5);
        setStepSound(RailcraftSound.getInstance());

        setCreativeTab(CreativeTabs.tabBlock);
    }

    @Override
    public float getBlockHardness(World world, int i, int j, int k) {
        int meta = world.getBlockMetadata(i, j, k);
        return EnumCube.fromOrdinal(meta).getHardness();
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MAIN_TEXTURE_FILE;
    }

    @Override
    public int damageDropped(int meta) {
        return meta;
    }

    public void setTextureOveride(FakeBlockRenderInfo info) {
        override = info;
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        if (override != null) {
            return override.getBlockTextureFromSide(side);
        }
        return EnumCube.fromOrdinal(meta).getBlock().getBlockTextureFromSide(side);
    }

    public static String getBlockNameFromMetadata(int meta) {
        return EnumCube.fromOrdinal(meta).getTag();
    }

    @Override
    public void updateTick(World world, int i, int j, int k, Random rand) {
        int meta = world.getBlockMetadata(i, j, k);

        EnumCube.fromOrdinal(meta).getBlock().updateTick(world, i, j, k, rand);
    }

    @Override
    public void onPostBlockPlaced(World world, int i, int j, int k, int meta) {
        EnumCube.fromOrdinal(meta).getBlock().onBlockPlaced(world, i, j, k);
    }

    @Override
    public void randomDisplayTick(World world, int i, int j, int k, Random rand) {
        int meta = world.getBlockMetadata(i, j, k);

        EnumCube.fromOrdinal(meta).getBlock().randomDisplayTick(world, i, j, k, rand);
    }

    @Override
    public void onBlockAdded(World world, int i, int j, int k) {
        int meta = world.getBlockMetadata(i, j, k);

        EnumCube.fromOrdinal(meta).getBlock().onBlockAdded(world, i, j, k);
    }

    @Override
    public void onNeighborBlockChange(World world, int i, int j, int k, int id) {
        int meta = world.getBlockMetadata(i, j, k);

        EnumCube.fromOrdinal(meta).getBlock().onNeighborBlockChange(world, i, j, k, id);
    }

    @Override
    public void breakBlock(World world, int i, int j, int k, int id, int meta) {
        EnumCube.fromOrdinal(meta).getBlock().onBlockRemoval(world, i, j, k);
    }

    @Override
    public boolean removeBlockByPlayer(World world, EntityPlayer player, int i, int j, int k) {
        int meta = world.getBlockMetadata(i, j, k);
        return EnumCube.fromOrdinal(meta).getBlock().removeBlockByPlayer(world, player, i, j, k);
    }

    @Override
    public boolean canCreatureSpawn(EnumCreatureType type, World world, int i, int j, int k) {
        int meta = world.getBlockMetadata(i, j, k);
        return EnumCube.fromOrdinal(meta).getBlock().canCreatureSpawn(type, world, i, j, k);
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        for (EnumCube type : EnumCube.getCreativeList()) {
            if (type.isEnabled()) {
                list.add(type.getItem());
            }
        }
    }

    @Override
    public float getExplosionResistance(Entity exploder, World world, int i, int j, int k, double srcX, double srcY, double srcZ) {
        int meta = world.getBlockMetadata(i, j, k);
        return EnumCube.fromOrdinal(meta).getResistance() * 3f / 5f;
    }
}
