package railcraft.common.blocks.aesthetics.cube;

import java.util.Random;
import net.minecraft.block.Block;
import net.minecraft.block.BlockSand;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.block.material.Material;
import net.minecraft.world.World;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class CrushedObsidian extends CubeClass
{

    private static final int[] TEXTURE = {227};

    @Override
    public int[] getTexture()
    {
        return TEXTURE;
    }

    @Override
    public boolean canCreatureSpawn(EnumCreatureType type, World world, int x, int y, int z)
    {
        return false;
    }

    @Override
    public void onBlockAdded(World world, int i, int j, int k)
    {
        world.scheduleBlockUpdate(i, j, k, BlockCube.getBlock().blockID, this.tickRate());
    }

    @Override
    public void onNeighborBlockChange(World world, int i, int j, int k, int id)
    {
        world.scheduleBlockUpdate(i, j, k, BlockCube.getBlock().blockID, this.tickRate());
    }

    @Override
    public void updateTick(World world, int i, int j, int k, Random rand)
    {
        this.tryToFall(world, i, j, k);
    }

    /**
     * How many world ticks before ticking
     */
    public int tickRate()
    {
        return 3;
    }

    /**
     * If there is space to fall below will start this block falling
     */
    private void tryToFall(World par1World, int i, int j, int k)
    {
        if(canFallBelow(par1World, i, j - 1, k) && j >= 0) {
            byte var8 = 32;

            if(!BlockSand.fallInstantly && par1World.checkChunksExist(i - var8, j - var8, k - var8, i + var8, j + var8, k + var8)) {
                if(!par1World.isRemote) {
                    EntityFallingCube entity = new EntityFallingCube(par1World, (double)((float)i + 0.5F), (double)((float)j + 0.5F), (double)((float)k + 0.5F), BlockCube.getBlock().blockID, EnumCube.CRUSHED_OBSIDIAN.ordinal());
                    par1World.spawnEntityInWorld(entity);
                }
            } else {
                par1World.setBlockWithNotify(i, j, k, 0);

                while(canFallBelow(par1World, i, j - 1, k) && j > 0) {
                    --j;
                }

                if(j > 0) {
                    par1World.setBlockAndMetadataWithNotify(i, j, k, BlockCube.getBlock().blockID, EnumCube.CRUSHED_OBSIDIAN.ordinal());
                }
            }
        }
    }

    /**
     * Checks to see if the sand can fall into the block below it
     */
    public static boolean canFallBelow(World par0World, int par1, int par2, int par3)
    {
        int var4 = par0World.getBlockId(par1, par2, par3);

        if(var4 == 0) {
            return true;
        } else if(var4 == Block.fire.blockID) {
            return true;
        } else {
            Material var5 = Block.blocksList[var4].blockMaterial;
            return var5 == Material.water || var5 == Material.lava;
        }
    }
}
