package railcraft.common.blocks.aesthetics.post;

import ic2.api.IPaintableBlock;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.block.StepSound;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.MinecraftForge;
import railcraft.common.api.core.IPostConnection;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.signals.*;
import railcraft.common.core.RailcraftConstants;

public class BlockPost extends Block implements IPaintableBlock {

    public static final Set<Integer> canConnect = new HashSet<Integer>();
    public static final Set<Integer> noConnect = new HashSet<Integer>();

    static {
        canConnect.add(Block.glowStone.blockID);
        canConnect.add(Block.redstoneLampActive.blockID);
        canConnect.add(Block.redstoneLampIdle.blockID);
        canConnect.add(Block.glass.blockID);
        canConnect.add(Block.mobSpawner.blockID);
        canConnect.add(Block.pistonBase.blockID);

        noConnect.add(Block.dirt.blockID);
        noConnect.add(Block.stone.blockID);
        noConnect.add(Block.gravel.blockID);
        noConnect.add(Block.grass.blockID);
        noConnect.add(Block.sand.blockID);
        noConnect.add(Block.bedrock.blockID);
        noConnect.add(Block.oreCoal.blockID);
        noConnect.add(Block.oreDiamond.blockID);
        noConnect.add(Block.oreEmerald.blockID);
        noConnect.add(Block.oreGold.blockID);
        noConnect.add(Block.oreIron.blockID);
        noConnect.add(Block.oreLapis.blockID);
        noConnect.add(Block.oreRedstone.blockID);
        noConnect.add(Block.oreRedstoneGlowing.blockID);
        noConnect.add(Block.blockClay.blockID);
        noConnect.add(Block.blockSnow.blockID);
        noConnect.add(Block.melon.blockID);
        noConnect.add(Block.pumpkin.blockID);
        noConnect.add(Block.tnt.blockID);
        noConnect.add(Block.slowSand.blockID);
        noConnect.add(Block.netherrack.blockID);
        noConnect.add(Block.sponge.blockID);
    }
    private static final float SIZE = 0.15f;
    private static final float SELECT = 0.1f;
    private final int renderType;

    public BlockPost(int id, int renderType) {
        super(id, 0, new MaterialStructure());
        this.renderType = renderType;
        setBlockName("rcPostBlock");
        setStepSound(new StepSound("railcraft", 1, 1));
        setResistance(15);
        setHardness(3);
        setRequiresSelfNotify();

        MinecraftForge.setBlockHarvestLevel(this, "crowbar", 0);
        MinecraftForge.setBlockHarvestLevel(this, EnumPost.WOOD.ordinal(), "axe", 0);
        MinecraftForge.setBlockHarvestLevel(this, EnumPost.STONE.ordinal(), "pickaxe", 1);
        MinecraftForge.setBlockHarvestLevel(this, EnumPost.METAL.ordinal(), "pickaxe", 2);

        setCreativeTab(CreativeTabs.tabDecorations);
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        for (EnumPost post : EnumPost.values()) {
            list.add(post.getItem());
        }
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MAIN_TEXTURE_FILE;
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        return EnumPost.fromId(meta).getTexture();
    }

    @Override
    public int damageDropped(int meta) {
        return meta;
    }

    @Override
    public void setBlockBoundsBasedOnState(IBlockAccess world, int i, int j, int k) {
        setBlockBounds(0.2F, 0.0F, 0.2F, 0.8F, 1.0F, 0.8F);
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
        if (!world.isAirBlock(i, j - 1, k)
                && !(Block.blocksList[world.getBlockId(i, j - 1, k)] instanceof BlockPost)
                && !BlockRail.isRailBlockAt(world, i, j + 1, k)) {
            return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + SIZE, j, k + SIZE, i + 1 - SIZE, j + 1.5, k + 1 - SIZE);
        }
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + SIZE, j, k + SIZE, i + 1 - SIZE, j + 1, k + 1 - SIZE);
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + SELECT, j, k + SELECT, i + 1 - SELECT, j + 1.0F, k + 1 - SELECT);
    }

    @Override
    public boolean getBlocksMovement(IBlockAccess world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean isBlockSolidOnSide(World world, int i, int j, int k, ForgeDirection side) {
        if (side == ForgeDirection.DOWN || side == ForgeDirection.UP) {
            return true;
        }
        return false;
    }

    public static boolean connectPostAt(World world, int i, int j, int k, ForgeDirection side) {
        if (world.isAirBlock(i, j, k)) {
            return false;
        }

        int id = world.getBlockId(i, j, k);
        if (Block.blocksList[id] instanceof BlockPost) {
            return true;
        }

        if (Block.blocksList[id] instanceof IPostConnection) {
            return ((IPostConnection) Block.blocksList[id]).connectsAt(world, i, j, k, side);
        }

        if (Block.blocksList[id].isGenMineableReplaceable(world, i, j, k)) {
            return false;
        }

        if (noConnect.contains(id)) {
            return false;
        }

        if (canConnect.contains(id)) {
            return true;
        }

        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof ISignalTile) {
            return true;
        }

        if (world.isBlockSolidOnSide(i, j, k, side)) {
            return true;
        }

        if (Block.blocksList[id] != null && Block.blocksList[id].getClass().getSimpleName().equals("BlockShapedLamp")) {
            return true;
        }

        return false;
    }

    public boolean connectToBlockAt(World world, int i, int j, int k, ForgeDirection side) {
        return connectPostAt(world, i, j, k, side);
    }

    @Override
    public boolean isOpaqueCube() {
        return false;
    }

    @Override
    public boolean renderAsNormalBlock() {
        return false;
    }

    @Override
    public int getRenderType() {
        return renderType;
    }

    @Override
    public boolean isBlockNormalCube(World world, int i, int j, int k) {
        return false;
    }

    @Override
    public int getFireSpreadSpeed(World world, int x, int y, int z, int metadata, ForgeDirection face) {
        if (metadata == EnumPost.WOOD.ordinal()) {
            return 300;
        }
        return 0;
    }

    @Override
    public int getFlammability(IBlockAccess world, int x, int y, int z, int metadata, ForgeDirection face) {
        if (metadata == EnumPost.WOOD.ordinal()) {
            return 5;
        }
        return 0;
    }

    @Override
    public boolean isFlammable(IBlockAccess world, int x, int y, int z, int metadata, ForgeDirection face) {
        if (metadata == EnumPost.WOOD.ordinal()) {
            return true;
        }
        return false;
    }

    @Override
    public boolean canBeReplacedByLeaves(World world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean canCreatureSpawn(EnumCreatureType type, World world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean colorBlock(World world, int i, int j, int k, int color) {
        int meta = world.getBlockMetadata(i, j, k);
        if (meta != EnumPost.METAL.ordinal()) return false;
        Block block = RailcraftBlocks.getBlockPostMetal();
        if (block != null) {
            world.setBlockAndMetadataWithNotify(i, j, k, block.blockID, color);
            return true;
        }
        return false;
    }
}
