package railcraft.common.blocks.aesthetics.post;

import ic2.api.IPaintableBlock;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.MinecraftForge;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.plugins.redpower.RedPowerPlugin;
import railcraft.common.util.misc.EnumColor;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class BlockPostMetal extends BlockPost implements IPaintableBlock {

    public static int getTexture() {
        return 240;
    }

    public static ItemStack getItem() {
        return getItem(1, 3);
    }

    public static ItemStack getItem(int qty) {
        return getItem(qty, 3);
    }

    public static ItemStack getItem(int qty, int color) {
        Block block = RailcraftBlocks.getBlockPostMetal();
        if (block == null) {
            return null;
        }
        return new ItemStack(block, qty, color);
    }

    public BlockPostMetal(int id, int renderType) {
        super(id, renderType);
        setBlockName("rcMetalPostBlock");
        setStepSound(Block.soundMetalFootstep);
        MinecraftForge.setBlockHarvestLevel(this, "pickaxe", 2);
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        for (EnumColor color : EnumColor.values()) {
            list.add(getItem(1, color.ordinal()));
        }
    }

    @Override
    protected boolean canSilkHarvest() {
        return true;
    }

    @Override
    public boolean canSilkHarvest(World world, EntityPlayer player, int x, int y, int z, int metadata) {
        return true;
    }

    @Override
    public ArrayList<ItemStack> getBlockDropped(World world, int x, int y, int z, int metadata, int fortune) {
        ArrayList<ItemStack> list = new ArrayList<ItemStack>();
        list.add(EnumPost.METAL.getItem());
        return list;
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        return getTexture() + meta;
    }

    @Override
    public boolean colorBlock(World world, int i, int j, int k, int color) {
        world.setBlockMetadata(i, j, k, color);
        return true;
    }

    @Override
    public boolean onBlockActivated(World world, int x, int y, int z, EntityPlayer player, int side, float subHitX, float subHitY, float subHitZ) {
        int paint = RedPowerPlugin.canPaint(player);
        if (paint != -1) {
            world.setBlockMetadata(x, y, z, paint);
            return true;
        }
        return super.onBlockActivated(world, x, y, z, player, side, subHitX, subHitY, subHitZ);
    }

    @Override
    public int getFireSpreadSpeed(World world, int x, int y, int z, int metadata, ForgeDirection face) {
        return 0;
    }

    @Override
    public int getFlammability(IBlockAccess world, int x, int y, int z, int metadata, ForgeDirection face) {

        return 0;
    }

    @Override
    public boolean isFlammable(IBlockAccess world, int x, int y, int z, int metadata, ForgeDirection face) {

        return false;
    }
}
