package railcraft.common.blocks.aesthetics.wall;

import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.block.*;
import java.util.List;
import java.util.Random;
import net.minecraft.block.material.Material;
import net.minecraft.client.particle.EffectRenderer;
import net.minecraft.client.particle.EntityDiggingFX;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import railcraft.client.sounds.RailcraftSound;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.aesthetics.cube.EnumCube;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forestry.ForestryPlugin;
import railcraft.common.util.misc.MiscTools;

public class BlockRailcraftWall extends BlockWall {

    private static BlockRailcraftWall instance;
    public static int currentRenderPass;

    public static BlockRailcraftWall getBlock() {
        return instance;
    }

    public static void registerBlock() {
        if (instance == null) {
            int id = RailcraftConfig.getBlockId("block.wall");

            if (id > 0) {
                int renderId = Railcraft.getProxy().getRenderId();
                instance = new BlockRailcraftWall(id, renderId);
                GameRegistry.registerBlock(instance, ItemWall.class, instance.getBlockName());

                for (EnumWall wall : EnumWall.values()) {
                    switch (wall) {
                        case SNOW:
                            MinecraftForge.setBlockHarvestLevel(instance, wall.ordinal(), "shovel", 0);
                            break;
                        case OBSIDIAN:
                            MinecraftForge.setBlockHarvestLevel(instance, wall.ordinal(), "pickaxe", 3);
                            break;
                        default:
                            MinecraftForge.setBlockHarvestLevel(instance, wall.ordinal(), "pickaxe", 2);
                    }

                    RailcraftLanguage.getInstance().registerItemName(wall.getItem(), wall.getTag());
                    ItemRegistry.registerItem(wall.getTag(), wall.getItem());

                    if (wall != EnumWall.SNOW || wall != EnumWall.ICE) {
                        ForestryPlugin.addBackpackItem("builder", wall.getItem());
                    }
                }
            }
        }
    }
    private final int renderId;

    public BlockRailcraftWall(int id, int renderId) {
        super(id, Block.stoneBrick);
        this.renderId = renderId;
        setBlockName("rc.wall");
        this.setStepSound(RailcraftSound.getInstance());
//        this.setStepSound(soundStoneFootstep);
        this.setCreativeTab(CreativeTabs.tabBlock);
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        return EnumWall.fromMeta(meta).getTexture();
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MAIN_TEXTURE_FILE;
    }

    /**
     * The type of render function that is called for this block
     */
    @Override
    public int getRenderType() {
        return renderId;
    }

    @Override
    public int getRenderBlockPass() {
        return 1;
    }

    @Override
    public boolean canRenderInPass(int pass) {
        currentRenderPass = pass;
        return pass == 0 || pass == 1;
    }

    /**
     * Return whether an adjacent block can connect to a wall.
     */
    @Override
    public boolean canConnectWallTo(IBlockAccess world, int x, int y, int z) {
        int blockId = world.getBlockId(x, y, z);

        if (blockId != this.blockID && blockId != Block.fenceGate.blockID) {
            Block block = Block.blocksList[blockId];
            return block != null && block.blockMaterial.isOpaque() && block.renderAsNormalBlock() ? block.blockMaterial != Material.pumpkin : false;
        } else {
            return true;
        }
    }

    /**
     * Determines if a torch can be placed on the top surface of this block.
     * Useful for creating your own block that torches can be on, such as
     * fences.
     *
     * @param world The current world
     * @param x X Position
     * @param y Y Position
     * @param z Z Position
     * @return True to allow the torch to be placed
     */
    @Override
    public boolean canPlaceTorchOnTop(World world, int x, int y, int z) {
        return true;
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        for (EnumWall wall : EnumWall.VALUES) {
            if (wall.isEnabled()) {
                list.add(wall.getItem());
            }
        }
    }

    /**
     * Spawn a digging particle effect in the world, this is a wrapper around
     * EffectRenderer.addBlockHitEffects to allow the block more control over
     * the particles. Useful when you have entirely different texture sheets for
     * different sides/locations in the world.
     *
     * @param world The current world
     * @param target The target the player is looking at {x/y/z/side/sub}
     * @param effectRenderer A reference to the current effect renderer.
     * @return True to prevent vanilla digging particles form spawning.
     */
    @SideOnly(Side.CLIENT)
    @Override
    public boolean addBlockHitEffects(World worldObj, MovingObjectPosition target, EffectRenderer effectRenderer) {
        int x = target.blockX;
        int y = target.blockY;
        int z = target.blockZ;

        int sideHit = target.sideHit;

        int meta = worldObj.getBlockMetadata(x, y, z);

        Random rand = MiscTools.getRand();
        Block block = instance;
        float b = 0.1F;
        double px = x + rand.nextDouble() * (block.getBlockBoundsMaxX() - block.getBlockBoundsMinX() - (b * 2.0F)) + b + block.getBlockBoundsMinX();
        double py = y + rand.nextDouble() * (block.getBlockBoundsMaxY() - block.getBlockBoundsMinY() - (b * 2.0F)) + b + block.getBlockBoundsMinY();
        double pz = z + rand.nextDouble() * (block.getBlockBoundsMaxZ() - block.getBlockBoundsMinZ() - (b * 2.0F)) + b + block.getBlockBoundsMinZ();

        if (sideHit == 0) {
            py = (double) y + block.getBlockBoundsMinY() - (double) b;
        }

        if (sideHit == 1) {
            py = (double) y + block.getBlockBoundsMaxY() + (double) b;
        }

        if (sideHit == 2) {
            pz = (double) z + block.getBlockBoundsMinZ() - (double) b;
        }

        if (sideHit == 3) {
            pz = (double) z + block.getBlockBoundsMaxZ() + (double) b;
        }

        if (sideHit == 4) {
            px = (double) x + block.getBlockBoundsMinX() - (double) b;
        }

        if (sideHit == 5) {
            px = (double) x + block.getBlockBoundsMaxX() + (double) b;
        }

        EntityDiggingFX fx = new EntityDiggingFX(worldObj, px, py, pz, 0.0D, 0.0D, 0.0D, block, sideHit, worldObj.getBlockMetadata(x, y, z));
        if (EnumWall.fromMeta(meta).useBlockTexture()) {
            effectRenderer.addEffect(fx.func_70596_a(x, y, z).multiplyVelocity(0.2F).multipleParticleScaleBy(0.6F), block);
        } else {
            effectRenderer.addEffect(fx.func_70596_a(x, y, z).multiplyVelocity(0.2F).multipleParticleScaleBy(0.6F));
        }

        return true;
    }

    /**
     * Spawn particles for when the block is destroyed. Due to the nature of how
     * this is invoked, the x/y/z locations are not always guaranteed to host
     * your block. So be sure to do proper sanity checks before assuming that
     * the location is this block.
     *
     * @param world The current world
     * @param x X position to spawn the particle
     * @param y Y position to spawn the particle
     * @param z Z position to spawn the particle
     * @param meta The metadata for the block before it was destroyed.
     * @param effectRenderer A reference to the current effect renderer.
     * @return True to prevent vanilla break particles from spawning.
     */
    @SideOnly(Side.CLIENT)
    @Override
    public boolean addBlockDestroyEffects(World world, int x, int y, int z, int meta, EffectRenderer effectRenderer) {
        byte its = 4;
        for (int i = 0; i < its; ++i) {
            for (int j = 0; j < its; ++j) {
                for (int k = 0; k < its; ++k) {
                    double px = x + (i + 0.5D) / (double) its;
                    double py = y + (j + 0.5D) / (double) its;
                    double pz = z + (k + 0.5D) / (double) its;
                    int rand = MiscTools.getRand().nextInt(6);
                    EntityDiggingFX fx = new EntityDiggingFX(world, px, py, pz, px - x - 0.5D, py - y - 0.5D, pz - z - 0.5D, instance, rand, meta);
                    if (EnumWall.fromMeta(meta).useBlockTexture()) {
                        effectRenderer.addEffect(fx.func_70596_a(x, y, z), instance);
                    } else {
                        effectRenderer.addEffect(fx.func_70596_a(x, y, z));
                    }

                }
            }
        }
        return true;
    }

    @Override
    public float getBlockHardness(World world, int x, int y, int z) {
        int meta = world.getBlockMetadata(x, y, z);
        EnumWall wall = EnumWall.fromMeta(meta);
        switch (wall) {
            case INFERNAL:
                return EnumCube.INFERNAL_BRICK.getHardness();
            case SANDY:
                return EnumCube.SANDY_BRICK.getHardness();
            case CONCRETE:
                return EnumCube.CONCRETE_BLOCK.getHardness();
            default:
                int id = wall.getSourceId();
                Block block = Block.blocksList[id];
                if (id <= 0 || block == null) {
                    return Block.brick.getBlockHardness(world, x, y, z);
                }
                return block.getBlockHardness(world, x, y, z);
        }
    }

    @Override
    public float getExplosionResistance(Entity entity, World world, int x, int y, int z, double explosionX, double explosionY, double explosionZ) {
        int meta = world.getBlockMetadata(x, y, z);
        EnumWall wall = EnumWall.fromMeta(meta);
        switch (wall) {
            case INFERNAL:
                return EnumCube.INFERNAL_BRICK.getResistance() * 3f / 5f;
            case SANDY:
                return EnumCube.SANDY_BRICK.getResistance() * 3f / 5f;
            case CONCRETE:
                return EnumCube.CONCRETE_BLOCK.getResistance() * 3f / 5f;
            default:
                int id = wall.getSourceId();
                Block block = Block.blocksList[id];
                if (id <= 0 || block == null) {
                    return Block.brick.getExplosionResistance(entity);
                }
                return block.getExplosionResistance(entity);
        }

    }
}
