/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.blocks.aesthetics.wall;

import net.minecraft.block.Block;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.aesthetics.cube.EnumCube;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.modules.ModuleManager;
import railcraft.common.modules.ModuleManager.Module;
import railcraft.common.plugins.forge.CraftingPlugin;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public enum EnumWall {

    INFERNAL(215, true),
    SANDY(214, true),
    CONCRETE(103, true),
    SNOW(66, false),
    ICE(67, false),
    STONE_BRICK(54, false),
    STONE_BRICK_MOSSY(100, false),
    STONE_BRICK_CRACKED(101, false),
    STONE_BRICK_CHISELED(213, false),
    NETHER_BRICK(224, false),
    BRICK(7, false),
    SANDSTONE(192, false),
    SANDSTONE_CHISELED(229, false),
    SANDSTONE_SMOOTH(230, false),
    OBSIDIAN(37, false);
    public static final EnumWall[] VALUES = values();
    private final int texture;
    private final boolean useBlockTexture;
    private ItemStack source;

    public static void registerRecipes() {
        INFERNAL.source = EnumCube.INFERNAL_BRICK.getItem();
        SANDY.source = EnumCube.SANDY_BRICK.getItem();
        CONCRETE.source = EnumCube.CONCRETE_BLOCK.getItem();

        SNOW.source = new ItemStack(Block.blockSnow);
        ICE.source = new ItemStack(Block.ice);

        STONE_BRICK.source = new ItemStack(Block.stoneBrick, 1, 0);
        STONE_BRICK_MOSSY.source = new ItemStack(Block.stoneBrick, 1, 1);
        STONE_BRICK_CRACKED.source = new ItemStack(Block.stoneBrick, 1, 2);
        STONE_BRICK_CHISELED.source = new ItemStack(Block.stoneBrick, 1, 3);

        NETHER_BRICK.source = new ItemStack(Block.netherBrick);

        BRICK.source = new ItemStack(Block.brick);

        SANDSTONE.source = new ItemStack(Block.sandStone, 1, 0);
        SANDSTONE_CHISELED.source = new ItemStack(Block.sandStone, 1, 1);
        SANDSTONE_SMOOTH.source = new ItemStack(Block.sandStone, 1, 2);

        OBSIDIAN.source = new ItemStack(Block.obsidian);

        for (EnumWall wall : VALUES) {
            if (wall.isEnabled()) {
                if (wall == NETHER_BRICK) {                    
                    CraftingPlugin.addShapedRecipe(wall.getItem(5), "S S", "SSS", 'S', wall.source);
                } else {
                    CraftingPlugin.addShapedRecipe(wall.getItem(6), "SSS", "SSS", 'S', wall.source);
                }
            }
        }
    }

    private EnumWall(int texture, boolean useBlockTexture) {
        this.texture = texture;
        this.useBlockTexture = useBlockTexture;
    }

    public static EnumWall fromMeta(int id) {
        if (id < 0 || id >= VALUES.length) {
            return VALUES[0];
        }
        return VALUES[id];
    }

    public boolean useBlockTexture() {
        return useBlockTexture;
    }

    public int getTexture() {
        return texture;
    }

    public int getSourceId() {
        if (source == null) return 0;
        return source.itemID;
    }

    public ItemStack getItem() {
        return getItem(1);
    }

    public ItemStack getItem(int qty) {
        Block block = BlockRailcraftWall.getBlock();
        if (block == null) return null;
        return new ItemStack(block, qty, ordinal());
    }

    public String getTag() {
        return "rc.wall." + name().replace("_", ".").toLowerCase();
    }

    public boolean isEnabled() {
        return ModuleManager.isModuleLoaded(Module.STRUCTURES) && RailcraftConfig.isSubBlockEnabled(getTag()) && BlockRailcraftWall.getBlock() != null;
    }
}
