package railcraft.common.blocks.detector;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.List;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.passive.EntityAnimal;
import net.minecraft.entity.passive.EntityChicken;
import net.minecraft.entity.passive.EntityCow;
import net.minecraft.entity.passive.EntityMooshroom;
import net.minecraft.entity.passive.EntityPig;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.entity.passive.EntityWolf;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.api.carts.CartTools;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.network.IGuiReturnHandler;

// Referenced classes of package net.minecraft.src:
//            TileEntity, NBTTagCompound, World
public class TileDetectorAnimal extends TileDetector implements IGuiReturnHandler {

    public boolean chicken = true;
    public boolean cow = true;
    public boolean pig = true;
    public boolean sheep = true;
    public boolean mooshroom = true;
    public boolean wolf = true;
    public boolean other = true;

    public TileDetectorAnimal() {
    }

    @Override
    public boolean testForCarts() {
        List<EntityMinecart> carts = CartTools.getMinecartsOnAllSides(worldObj, xCoord, yCoord, zCoord, SENSITIVITY);
        for (EntityMinecart cart : carts) {
            if (cart.riddenByEntity instanceof EntityChicken) {
                if (chicken) {
                    return true;
                }
            } else if (cart.riddenByEntity instanceof EntityCow) {
                if (cow) {
                    return true;
                }
            } else if (cart.riddenByEntity instanceof EntityPig) {
                if (pig) {
                    return true;
                }
            } else if (cart.riddenByEntity instanceof EntitySheep) {
                if (sheep) {
                    return true;
                }
            } else if (cart.riddenByEntity instanceof EntityMooshroom) {
                if (mooshroom) {
                    return true;
                }
            } else if (cart.riddenByEntity instanceof EntityWolf) {
                if (wolf) {
                    return true;
                }
            } else if (cart.riddenByEntity instanceof EntityAnimal) {
                if (other) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public boolean blockActivated(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.DETECTOR_ANIMAL, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setBoolean("chicken", chicken);
        data.setBoolean("cow", cow);
        data.setBoolean("pig", pig);
        data.setBoolean("sheep", sheep);
        data.setBoolean("mooshroom", mooshroom);
        data.setBoolean("wolf", wolf);
        data.setBoolean("other", other);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        chicken = data.getBoolean("chicken");
        cow = data.getBoolean("cow");
        pig = data.getBoolean("pig");
        sheep = data.getBoolean("sheep");
        mooshroom = data.getBoolean("mooshroom");
        wolf = data.getBoolean("wolf");
        other = data.getBoolean("other");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        writeGuiData(data);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        readGuiData(data);
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        byte bits = 0;
        bits |= chicken ? 1 : 0;
        bits |= cow ? 2 : 0;
        bits |= pig ? 4 : 0;
        bits |= sheep ? 8 : 0;
        bits |= mooshroom ? 16 : 0;
        bits |= wolf ? 32 : 0;
        bits |= other ? 64 : 0;

        data.writeByte(bits);
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        byte bits = data.readByte();
        chicken = (bits & 1) != 0;
        cow = (bits & 2) != 0;
        pig = (bits & 4) != 0;
        sheep = (bits & 8) != 0;
        mooshroom = (bits & 16) != 0;
        wolf = (bits & 32) != 0;
        other = (bits & 64) != 0;
    }
}
