package railcraft.common.blocks.detector;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.List;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.carts.ILinkageManager;
import railcraft.common.carts.LinkageManager;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.network.IGuiReturnHandler;

public class TileDetectorTrain extends TileDetector implements IGuiReturnHandler {

    private short trainSize = 5;

    public TileDetectorTrain() {
    }

    @Override
    public boolean testForCarts() {
        ILinkageManager lm = LinkageManager.getInstance(worldObj);
        List<EntityMinecart> carts = CartTools.getMinecartsOnAllSides(worldObj, xCoord, yCoord, zCoord, SENSITIVITY);
        for (EntityMinecart cart : carts) {
            if (lm.countCartsInTrain(cart) >= getTrainSize()) {
                return true;
            }
        }
        return false;
    }

    @Override
    protected short updateInterval() {
        return 4;
    }

    @Override
    public boolean blockActivated(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.DETECTOR_TRAIN, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setShort("size", getTrainSize());
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        setTrainSize(data.getShort("size"));
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeShort(getTrainSize());
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        setTrainSize(data.readShort());
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeShort(getTrainSize());
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        setTrainSize(data.readShort());
    }

    public short getTrainSize() {
        return trainSize;
    }

    public void setTrainSize(short trainSize) {
        this.trainSize = trainSize;
    }
}
