package railcraft.common.blocks.detector;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.network.IGuiReturnHandler;

public class TileDetectorVillager extends TileDetector implements IGuiReturnHandler {

    public static enum Mode {

        ANY, NONE, NOT, EQUALS
    }
    private int profession;
    private Mode mode = Mode.ANY;

    public TileDetectorVillager() {
    }

    private boolean cartHasVillager() {
        for (EntityMinecart cart : getCarts()) {
            if (cart.riddenByEntity instanceof EntityVillager) {
                return true;
            }
        }
        return false;
    }

    private boolean cartHasProfesion() {
        for (EntityMinecart cart : getCarts()) {
            if (cart.riddenByEntity instanceof EntityVillager) {
                EntityVillager villager = (EntityVillager) cart.riddenByEntity;
                if (villager.getProfession() == profession) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public boolean testForCarts() {
        switch (mode) {
            case ANY:
                return cartHasVillager();
            case NONE:
                return !cartHasVillager();
            case EQUALS:
                return cartHasProfesion();
            case NOT:
                return !cartHasProfesion();
            default:
                return false;
        }
    }

    @Override
    public boolean blockActivated(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.DETECTOR_VILLAGER, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    public void setMode(Mode mode) {
        this.mode = mode;
    }

    public Mode getMode() {
        return mode;
    }

    public void setProfession(int profession) {
        this.profession = profession;
    }

    public int getProfession() {
        return profession;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setInteger("profession", profession);
        data.setByte("mode", (byte) mode.ordinal());
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        profession = data.getInteger("profession");
        mode = Mode.values()[data.getByte("mode")];
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeInt(profession);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        profession = data.readInt();
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeInt(profession);
        data.writeByte(mode.ordinal());
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        profession = data.readInt();
        mode = Mode.values()[data.readByte()];
    }
}
