package railcraft.common.blocks.machine;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import net.minecraft.block.BlockContainer;
import net.minecraft.block.material.Material;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.plugins.forge.CreativePlugin;
import railcraft.common.util.misc.Game;

public class BlockMachine extends BlockContainer {

    private final IMachineProxy proxy;
    private final boolean opaque;
    private final int renderId;
    private final int[] metaOpacity;

    public BlockMachine(int blockID, int renderId, IMachineProxy proxy, boolean opaque, int[] metaOpacity) {
        super(blockID, 45, Material.rock);
        setResistance(4.5F);
        setHardness(2.0F);
        setStepSound(soundStoneFootstep);
        setRequiresSelfNotify();
        setTickRandomly(true);
        this.proxy = proxy;
        this.opaque = opaque;
        this.renderId = renderId;
        this.metaOpacity = metaOpacity;

        setCreativeTab(CreativePlugin.TAB);

        opaqueCubeLookup[blockID] = opaque;
        lightOpacity[blockID] = opaque ? 255 : 0;
    }

    @Override
    public int getRenderType() {
        return renderId;
    }

    /**
     * Returns the default ambient occlusion value based on block opacity
     */
    @SideOnly(Side.CLIENT)
    @Override
    public float getAmbientOcclusionLightValue(IBlockAccess world, int x, int y, int z) {
        return 1;
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MACHINE_TEXTURE_FILE;
    }

    public IMachineProxy getMachineProxy() {
        return proxy;
    }

    @Override
    public int getBlockTexture(IBlockAccess world, int i, int j, int k, int side) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).getBlockTexture(side);
        }
        int meta = world.getBlockMetadata(i, j, k);
        return getBlockTextureFromSideAndMetadata(side, meta);
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        return proxy.getTexture(meta, side);
    }

    @Override
    public int idDropped(int meta, Random random, int j) {
        return blockID;
    }

    @Override
    public boolean onBlockActivated(World world, int i, int j, int k, EntityPlayer player, int side, float u1, float u2, float u3) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).blockActivated(player, side);
        }
        return false;
    }

    @SideOnly(Side.CLIENT)
    @Override
    public void randomDisplayTick(World world, int i, int j, int k, Random random) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            ((TileMachineBase) tile).randomDisplayTick(random);
        }
    }

    @Override
    public boolean isBlockNormalCube(World world, int i, int j, int k) {
        return false;
    }

    @Override
    public boolean isBlockSolidOnSide(World world, int x, int y, int z, ForgeDirection side) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).isBlockSolidOnSide(side);
        }
        return true;
    }

    @Override
    public void harvestBlock(World world, EntityPlayer entityplayer, int i, int j, int k, int l) {
    }

    @Override
    public boolean removeBlockByPlayer(World world, EntityPlayer player, int i, int j, int k) {
        player.addStat(StatList.mineBlockStatArray[blockID], 1);
        player.addExhaustion(0.025F);
        if (Game.isHost(world) && !player.capabilities.isCreativeMode) {
            dropBlockAsItem(world, i, j, k, 0, 0);
        }
        return world.setBlockWithNotify(i, j, k, 0);
    }

    @Override
    public ArrayList<ItemStack> getBlockDropped(World world, int x, int y, int z, int metadata, int fortune) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).getBlockDropped(fortune);
        }
        return super.getBlockDropped(world, x, y, z, metadata, fortune);
    }

    @Override
    public int damageDropped(int meta) {
        return meta;
    }

    @Override
    public boolean canProvidePower() {
        return true;
    }

    @Override
    public boolean isProvidingWeakPower(IBlockAccess world, int i, int j, int k, int side) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).isPoweringTo(side);
        }
        return false;
    }

    @Override
    public boolean isProvidingStrongPower(IBlockAccess world, int i, int j, int k, int side) {
        return isProvidingWeakPower(world, i, j, k, side);
    }

    @Override
    public boolean canConnectRedstone(IBlockAccess world, int i, int j, int k, int dir) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            ((TileMachineBase) tile).canConnectRedstone(dir);
        }
        return false;
    }

    public void initFromItem(World world, int i, int j, int k, ItemStack stack) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            ((TileMachineBase) tile).initFromItem(stack);
        }
    }

    @Override
    public void onBlockPlacedBy(World world, int i, int j, int k, EntityLiving entityliving) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            ((TileMachineBase) tile).onBlockPlacedBy(entityliving);
        }
    }

    @Override
    public void onNeighborBlockChange(World world, int i, int j, int k, int id) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            ((TileMachineBase) tile).onNeighborBlockChange(id);
        }
    }

    @Override
    public void onBlockAdded(World world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            ((TileMachineBase) tile).onBlockAdded();
        }
    }

    @Override
    public void breakBlock(World world, int x, int y, int z, int id, int meta) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileMachineBase) {
            ((TileMachineBase) tile).onBlockRemoval();
        }
//        world.notifyBlocksOfNeighborChange(x + 1, y, z, blockID);
//        world.notifyBlocksOfNeighborChange(x - 1, y, z, blockID);
//        world.notifyBlocksOfNeighborChange(x, y, z + 1, blockID);
//        world.notifyBlocksOfNeighborChange(x, y, z - 1, blockID);
//        world.notifyBlocksOfNeighborChange(x, y - 1, z, blockID);
//        world.notifyBlocksOfNeighborChange(x, y + 1, z, blockID);
        super.breakBlock(world, x, y, z, id, meta);
    }

    @Override
    public TileEntity createNewTileEntity(World world, int metadata) {
        return proxy.getTileEntity(metadata);
    }

    @Override
    public TileEntity createNewTileEntity(World var1) {
        return null;
    }

    @Override
    public int getLightValue(IBlockAccess world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).getLightValue();
        }
        return 0;
    }

    @Override
    public boolean hasTileEntity(int metadata) {
        return true;
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        for (IEnumMachine type : proxy.getCreativeList()) {
            if (type.isEnabled()) {
                list.add(type.getItem());
            }
        }
    }

    @Override
    public final boolean isOpaqueCube() {
        return opaque;
    }

    @Override
    public int getLightOpacity(World world, int x, int y, int z) {
        int meta = world.getBlockMetadata(x, y, z);
        return metaOpacity[meta];
    }

    @Override
    public float getExplosionResistance(Entity exploder, World world, int x, int y, int z, double srcX, double srcY, double srcZ) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).getResistance(exploder) * 3f / 5f;
        }
        return super.getExplosionResistance(exploder, world, x, y, z, minX, minY, minZ);
    }

    @Override
    public boolean canBeReplacedByLeaves(World world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean canCreatureSpawn(EnumCreatureType type, World world, int x, int y, int z) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).canCreatureSpawn(type);
        }
        return super.canCreatureSpawn(type, world, x, y, z);
    }

    @Override
    public float getBlockHardness(World world, int x, int y, int z) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileMachineBase) {
            return ((TileMachineBase) tile).getHardness();
        }
        return super.getBlockHardness(world, x, y, z);
    }
}
