package railcraft.common.blocks.machine;

import net.minecraft.util.AxisAlignedBB;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class MultiBlockPattern {

    public final char[][][] pattern;
    private final int offsetX;
    private final int offsetY;
    private final int offsetZ;
    private final AxisAlignedBB entityCheckBounds;

    public MultiBlockPattern(char[][][] pattern) {
        this(pattern, 1, 1, 1);
    }

    public MultiBlockPattern(char[][][] pattern, int offsetX, int offsetY, int offsetZ) {
        this(pattern, offsetX, offsetY, offsetZ, null);
    }

    public MultiBlockPattern(char[][][] pattern, int offsetX, int offsetY, int offsetZ, AxisAlignedBB entityCheckBounds) {
        this.pattern = pattern;
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.offsetZ = offsetZ;
        this.entityCheckBounds = entityCheckBounds;
    }

    public AxisAlignedBB getEntityCheckBounds(int masterX, int masterY, int masterZ) {
        if (entityCheckBounds == null) {
            return null;
        }
        return entityCheckBounds.copy().offset(masterX, masterY, masterZ);
    }

    public char getPatternMarkerChecked(int x, int y, int z) {
        if (x < 0 || y < 0 || z < 0) {
            return 'O';
        }
        if (x >= getPatternWidthX() || y >= getPatternHeight() || z >= getPatternWidthZ()) {
            return 'O';
        }
        return getPatternMarker(x, y, z);
    }

    public char getPatternMarker(int x, int y, int z) {
        return pattern[y][x][z];
    }

    public int getMasterOffsetX() {
        return offsetX;
    }

    public int getMasterOffsetY() {
        return offsetY;
    }

    public int getMasterOffsetZ() {
        return offsetZ;
    }

    public int getPatternHeight() {
        return pattern.length;
    }

    public int getPatternWidthX() {
        return pattern[0].length;
    }

    public int getPatternWidthZ() {
        return pattern[0][0].length;
    }

    public int getMasterRelativeX(int posX, int patternX) {
        return (offsetX - patternX) + posX;
    }

    public int getMasterRelativeY(int posY, int patternY) {
        return (offsetY - patternY) + posY;
    }

    public int getMasterRelativeZ(int posZ, int patternZ) {
        return (offsetZ - patternZ) + posZ;
    }
}
