package railcraft.common.blocks.machine;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Level;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.core.items.ITrackItem;
import railcraft.common.blocks.RailcraftTileEntity;
import railcraft.common.items.IActivationBlockingItem;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.Game;

public abstract class TileMachineBase extends RailcraftTileEntity {

    private boolean checkedBlock = false;

    public abstract IEnumMachine getMachineType();

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(getMachineType().getTag());
    }

    @Override
    public final short getId() {
        return (short) getMachineType().ordinal();
    }

    public boolean canCreatureSpawn(EnumCreatureType type) {
        return true;
    }

    public ArrayList<ItemStack> getBlockDropped(int fortune) {
        ArrayList<ItemStack> items = new ArrayList<ItemStack>();
        items.add(getMachineType().getItem());
        return items;
    }

    public void initFromItem(ItemStack stack) {
    }

    public void onBlockAdded() {
    }

    public void onBlockRemoval() {
        if (this instanceof IInventory) {
            InvTools.dropInventory((IInventory) this, worldObj, xCoord, yCoord, zCoord);
        }
    }

    public boolean blockActivated(EntityPlayer player, int side) {
        if (player.isSneaking()) {
            return false;
        }
        ItemStack stack = player.getCurrentEquippedItem();
        if (stack != null) {
            if (stack.getItem() instanceof IActivationBlockingItem) {
                return false;
            }
            if (stack.getItem() instanceof ITrackItem) {
                return false;
            }
            if (stack.itemID < Block.blocksList.length && BlockRail.isRailBlock(stack.itemID)) {
                return false;
            }
        }
        return openGui(player);
    }

    public boolean isBlockSolidOnSide(ForgeDirection side) {
        return true;
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isNotHost(worldObj)) {
            return;
        }

        // Check and fix invalid block ids
        if (!checkedBlock) {
            checkedBlock = true;
            
            if(!getMachineType().isEnabled()){
                worldObj.setBlockWithNotify(xCoord, yCoord, zCoord, 0);
                return;
            }

            if (getBlock() != getMachineType().getBlock()) {
                Game.log(Level.INFO, "Updating Machine Tile Block Id: {0}, [{1}, {2}, {3}]", getClass().getSimpleName(), xCoord, yCoord, zCoord);
                worldObj.setBlockAndMetadata(xCoord, yCoord, zCoord, getMachineType().getBlockId(), getMachineType().ordinal());
                validate();
                worldObj.setBlockTileEntity(xCoord, yCoord, zCoord, this);
                updateContainingBlockInfo();
            }
        
            if (getBlock() != null && getClass() != ((BlockMachine) getBlock()).getMachineProxy().getTileClass(worldObj.getBlockMetadata(xCoord, yCoord, zCoord))) {
                worldObj.setBlockMetadata(xCoord, yCoord, zCoord, getId());
                Game.log(Level.INFO, "Updating Machine Tile Metadata: {0}, [{1}, {2}, {3}]", getClass().getSimpleName(), xCoord, yCoord, zCoord);
                updateContainingBlockInfo();
            }
        }
    }

    public boolean openGui(EntityPlayer player) {
        return false;
    }

    public int getBlockTexture(int side) {
        return 0;
    }

    public int getLightValue() {
        return 0;
    }

    public float getResistance(Entity exploder) {
        return 4.5f;
    }

    public float getHardness() {
        return 2.0f;
    }

    public boolean isPoweringTo(int side) {
        return false;
    }

    public boolean canConnectRedstone(int dir) {
        return false;
    }

    public void onNeighborBlockChange(int id) {
    }

    @SideOnly(Side.CLIENT)
    public void randomDisplayTick(Random rand) {
    }
}
