package railcraft.common.blocks.machine;

import java.util.List;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.StandaloneInventory;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileMultiBlockInventory extends TileMultiBlock implements IInventory
{

    private final StandaloneInventory inv;
    private final String guiTag;

    public TileMultiBlockInventory(String guiTag, int invSize, List<? extends MultiBlockPattern> patterns) {
        super(patterns);
        inv = new StandaloneInventory(invSize, (IInventory)this);
        this.guiTag = guiTag;
    }

    @Override
    protected void onMasterReset() {
        super.onMasterReset();
        InvTools.dropInventory(inv, worldObj, xCoord, yCoord, zCoord);
    }

    protected void dropItem(ItemStack stack) {
        InvTools.dropItem(stack, worldObj, xCoord, yCoord, zCoord);
    }

    @Override
    public ItemStack decrStackSize(int i, int j) {
        TileMultiBlockInventory mBlock = (TileMultiBlockInventory)getMasterBlock();
        if(mBlock != null) {
            return mBlock.inv.decrStackSize(i, j);
        }
        return null;
    }

    @Override
    public ItemStack getStackInSlot(int i) {
        TileMultiBlockInventory mBlock = (TileMultiBlockInventory)getMasterBlock();
        if(mBlock != null) {
            return mBlock.inv.getStackInSlot(i);
        }
        return null;
    }

    @Override
    public void setInventorySlotContents(int i, ItemStack itemstack) {
        TileMultiBlockInventory mBlock = (TileMultiBlockInventory)getMasterBlock();
        if(mBlock != null) {
            mBlock.inv.setInventorySlotContents(i, itemstack);
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        inv.writeToNBT("invStructure", data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        inv.readFromNBT("invStructure", data);
    }

    @Override
    public int getSizeInventory() {
        return inv.getSizeInventory();
    }

    @Override
    public ItemStack getStackInSlotOnClosing(int var1) {
        return null;
    }

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(guiTag);
    }

    @Override
    public int getInventoryStackLimit() {
        return 64;
    }

    @Override
    public void openChest() {
    }

    @Override
    public void closeChest() {
    }
}
