package railcraft.common.blocks.machine;

import buildcraft.api.transport.IPipe;
import buildcraft.api.transport.IPipeTile;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.List;
import net.minecraft.inventory.IInventory;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.inventory.Slot;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.gui.slots.SlotLiquidContainer;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.FakeTank;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileTank extends TileMultiBlockInventory implements ITankContainer, ISidedInventory, ITankTile {

    protected final TankManager tankManager = new TankManager();
    private boolean testOutput = true;

    public TileTank(String name, int invNum, List<? extends MultiBlockPattern> patterns) {
        super(name, invNum, patterns);
    }

//    @Override
//    public void updateEntity()
//    {
//        super.updateEntity();
//
//        if(isMaster) {
//            if(Game.isHost(getWorld()) && update % LiquidManager.NETWORK_UPDATE_INTERVAL == 0) {
//                markBlockNeedsUpdate();
//            }
//        }
//    }
    @Override
    public IInventory getInventory() {
        return this;
    }

    @Override
    public TankManager getTankManager() {
        TileTank mBlock = (TileTank) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager;
        }
        return null;
    }

    @Override
    public ILiquidTank getTank() {
        TileTank mBlock = (TileTank) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager.getTank(0);
        }
        return null;
    }

    @Override
    public Slot getInputSlot(IInventory inv, int id, int x, int y) {
        return new SlotLiquidContainer(inv, id, x, y);
    }

    /**
     * Drains liquid out of internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is drained to.
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return drain(0, maxDrain, doDrain);
    }

    /**
     * Drains liquid out of the specified internal tank.
     *
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.drain(tankIndex, maxDrain, doDrain);
        }
        return null;
    }

    /**
     * Fills liquid into internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return fill(0, resource, doFill);
    }

    /**
     * Fills liquid into the specified internal tank.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.fill(tankIndex, resource, doFill);
        }
        return 0;
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection side) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.getTanks();
        }
        return FakeTank.ARRAY;
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.getTank(0);
        }
        return FakeTank.INSTANCE;
    }

    public ITankContainer getOutputOnSide(ForgeDirection side) {
        TileEntity tile = MiscTools.getBlockTileEntityOnSide(worldObj, xCoord, yCoord, zCoord, side);
        if (tile instanceof TileTank) {
            return null;
        } else if (tile instanceof ITankContainer) {
            return (ITankContainer) tile;
        } else if (tile instanceof IPipeTile) {
            IPipe pipe = ((IPipeTile) tile).getPipe();
            if (pipe instanceof ITankContainer) {
                return (ITankContainer) pipe;
            }
        }
        return null;
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 1;
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        if (side == ForgeDirection.UP) {
            return 0;
        }
        return 1;
    }

    @Override
    protected void onMasterReset() {
        super.onMasterReset();
//        tank.setLiquid(null);
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        tankManager.writeTanksToNBT(data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        tankManager.readTanksFromNBT(data);
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);
        tankManager.writePacketData(data);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);
        tankManager.readPacketData(data);
    }
}
