package railcraft.common.blocks.machine.alpha;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.TileMachineBase;
import railcraft.common.blocks.machine.gamma.TileDispenserCart;
import railcraft.common.blocks.machine.gamma.TileDispenserTrain;
import railcraft.common.blocks.machine.gamma.TileEnergyLoader;
import railcraft.common.blocks.machine.gamma.TileEnergyUnloader;
import railcraft.common.blocks.machine.gamma.TileItemUnloaderAdvanced;
import railcraft.common.blocks.machine.gamma.TileLiquidUnloader;
import railcraft.common.carts.ItemCartAnchor;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.modules.ModuleManager;
import railcraft.common.modules.ModuleManager.Module;

/**
 *
 * @author CovertJaguar
 */
public enum EnumMachineAlpha implements IEnumMachine {

    WORLD_ANCHOR(Module.CHUNK_LOADING, "world.anchor", TileAnchorWorld.class, 185, 185, 201, 201, 201, 201, 169),
    TURBINE(Module.ENERGY, "turbine", TileSteamTurbine.class, 2, 2, 2, 2, 32, 2, 0, 1, 16, 17, 18, 32, 33, 34),
    PERSONAL_ANCHOR(Module.CHUNK_LOADING, "personal.anchor", TileAnchorPersonal.class, 183, 183, 199, 199, 199, 199, 167),
    STEAM_OVEN(Module.FACTORY, "steam.oven", TileSteamOven.class, 226, 226, 243, 243, 242, 243, 224, 225, 240, 241),
    @Deprecated
    LIQUID_UNLOADER(null, "unloader.liquid", TileLiquidUnloader.class, 26, 28, 27, 27, 27, 27),
    @Deprecated
    ITEM_UNLOADER_ADVANCED(null, "unloader.item.advanced", TileItemUnloaderAdvanced.class, 138, 138, 139, 140, 139, 139),
    @Deprecated
    DISPENSER_CART(null, "dispenser.cart", TileDispenserCart.class, 106, 106, 107, 108, 107, 107),
    COKE_OVEN(Module.FACTORY, "coke.oven", TileCokeOven.class, 170, 170, 170, 170, 171, 170, 171, 172),
    ROLLING_MACHINE(Module.FACTORY, "rolling.machine", TileRollingMachine.class, 186, 188, 187, 187, 187, 187),
    @Deprecated
    ENERGY_LOADER(null, "loader.energy", TileEnergyLoader.class, 202, 202, 203, 204, 203, 203),
    @Deprecated
    ENERGY_UNLOADER(null, "unloader.energy", TileEnergyUnloader.class, 218, 218, 219, 220, 219, 219),
    FEED_STATION(Module.AUTOMATION, "feed.station", TileFeedStation.class, 234, 236, 235, 235, 235, 235),
    BLAST_FURNACE(Module.FACTORY, "blast.furnace", TileBlastFurnace.class, 215, 215, 215, 215, 216, 215, 216, 217),
    @Deprecated
    DISPENSER_TRAIN(null, "dispenser.train", TileDispenserTrain.class, 231, 231, 232, 233, 232, 232),
    TANK_WATER(Module.TRANSPORT, "tank.water", TileTankWater.class, 214, 214, 213, 213, 213, 213),
    ROCK_CRUSHER(Module.FACTORY, "rock.crusher", TileRockCrusher.class, 3, 5, 4, 4, 4, 4);
    private final Module module;
    private final String tag;
    private final Class<? extends TileMachineBase> tile;
    private final int[] texture;
    private static final List<EnumMachineAlpha> creativeList = new ArrayList<EnumMachineAlpha>();
    private static final EnumMachineAlpha[] VALUES = values();

    static {
        creativeList.add(COKE_OVEN);
        creativeList.add(BLAST_FURNACE);
        creativeList.add(STEAM_OVEN);
        creativeList.add(TANK_WATER);
        creativeList.add(ROLLING_MACHINE);
        creativeList.add(ROCK_CRUSHER);
        creativeList.add(FEED_STATION);
        creativeList.add(WORLD_ANCHOR);
        creativeList.add(PERSONAL_ANCHOR);
        creativeList.add(TURBINE);
    }

    private EnumMachineAlpha(Module module, String tag, Class<? extends TileMachineBase> tile, int... texture) {
        this.module = module;
        this.tile = tile;
        this.tag = tag;
        this.texture = texture;
    }

    @Override
    public boolean isDepreciated() {
        return module == null;
    }

    @Override
    public int getTexture(int index) {
        if (index < 0 || index >= texture.length) {
            index = 0;
        }
        return texture[index];
    }

    public static EnumMachineAlpha fromId(int id) {
        if (id < 0 || id >= VALUES.length) {
            id = 0;
        }
        return VALUES[id];
    }

    public static List<EnumMachineAlpha> getCreativeList() {
        return creativeList;
    }

    @Override
    public String getTag() {
        return "rc.machine.alpha." + tag;
    }

    @Override
    public Class getTileClass() {
        return tile;
    }

    public TileMachineBase getTileEntity() {
        try {
            return tile.newInstance();
        } catch (Exception ex) {
        }
        return null;
    }

    @Override
    public ItemStack getItem() {
        return getItem(1);
    }

    @Override
    public ItemStack getItem(int qty) {
        Block block = getBlock();
        if (block == null) {
            return null;
        }
        return new ItemStack(block, qty, ordinal());
    }

    public Module getModule() {
        return module;
    }

    @Override
    public Block getBlock() {
        return RailcraftBlocks.getBlockMachineAlpha();
    }

    @Override
    public int getBlockId() {
        Block block = getBlock();
        if (block != null) {
            return block.blockID;
        }
        return 0;
    }

    @Override
    public boolean isEnabled() {
        return ModuleManager.isModuleLoaded(getModule()) && getBlock() != null && RailcraftConfig.isSubBlockEnabled(getTag());
    }

    public void addItemInfo(ItemStack stack, EntityPlayer player, List<String> info, boolean adv) {
        switch (this) {
            case WORLD_ANCHOR:
                if (RailcraftConfig.anchorRefuel() > 0) {
                    addAnchorInfo(stack, info);
                }
                break;
            case PERSONAL_ANCHOR:
                if (RailcraftConfig.anchorRefuelPersonal() > 0) {
                    addAnchorInfo(stack, info);
                }
                break;
            default:
                String tipTag = "gui.tip." + tag;
                if (RailcraftLanguage.hasTag(tipTag)) {
                    info.add(RailcraftLanguage.translate(tipTag));
                }
                break;
        }
    }

    private void addAnchorInfo(ItemStack stack, List<String> info) {
        long fuel = ItemCartAnchor.getFuel(stack);
        double hours = (double) fuel / RailcraftConstants.TICKS_PER_HOUR;
        String format = RailcraftLanguage.translate("gui.anchor.fuel.remaining");
        info.add(String.format(format, hours));
    }
}
