package railcraft.common.blocks.machine.alpha;

import buildcraft.api.inventory.ISpecialInventory;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.block.material.Material;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import railcraft.common.api.crafting.IBlastFurnaceRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.TileMultiBlockOven;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.items.RailcraftToolItems;
import railcraft.common.plugins.buildcraft.triggers.INeedsFuel;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

public class TileBlastFurnace extends TileMultiBlockOven implements ISidedInventory, ISpecialInventory, INeedsFuel {

    private static final int FUEL_PER_TICK = 5;
    private static final int COOK_STEP_LENGTH = 8;
    private static final int SLOT_INPUT = 0;
    private static final int SLOT_FUEL = 1;
    private static final int SLOT_OUTPUT = 2;
    private final static List<MultiBlockPattern> patterns = new ArrayList<MultiBlockPattern>();
    /**
     * The number of ticks that the furnace will keep burning
     */
    public int burnTime = 0;
    /**
     * The number of ticks that a fresh copy of the currently-burning item would
     * keep the furnace burning for
     */
    public int currentItemBurnTime = 0;
    private int finishedAt;
    private final IInventory invFuel = new InventoryMapper(this, SLOT_FUEL, 1);
    private final IInventory invInput = new InventoryMapper(this, SLOT_INPUT, 1);
    private final IInventory invOutput = new InventoryMapper(this, SLOT_OUTPUT, 1);

    static {
        char[][][] map = {
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'W', 'B', 'O'},
                {'O', 'W', 'B', 'W', 'O'},
                {'O', 'B', 'W', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'A', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'A', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            }
        };
        patterns.add(new MultiBlockPattern(map));
    }

    public TileBlastFurnace() {
        super("gui.blast.furnace", 3, patterns);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineAlpha.BLAST_FURNACE;
    }

    @Override
    public int getBlockTexture(int side) {
        if (getPatternMarker() == 'W' && isStructureValid()) {
            if (isBurning()) {
                return getMachineType().getTexture(7);
            }
            return getMachineType().getTexture(6);
        }
        return getMachineType().getTexture(0);
    }

    @Override
    protected boolean isMapPositionValid(int i, int j, int k, char mapPos) {
        int id = worldObj.getBlockId(i, j, k);
        switch (mapPos) {
            case 'O':
                if (id != RailcraftBlocks.getBlockMachineAlpha().blockID || worldObj.getBlockMetadata(i, j, k) != getBlockMetadata()) {
                    return true;
                }
                break;
            case 'B':
            case 'W':
                if (id == RailcraftBlocks.getBlockMachineAlpha().blockID && worldObj.getBlockMetadata(i, j, k) == getBlockMetadata()) {
                    return true;
                }
                break;
            case 'A':
                if (worldObj.isAirBlock(i, j, k) || worldObj.getBlockMaterial(i, j, k) == Material.lava) {
                    return true;
                }
                break;
        }
        return false;
    }

    @Override
    protected void onMasterReset() {
        super.onMasterReset();
        cookTime = 0;
        burnTime = 0;
        currentItemBurnTime = 0;
    }

    @Override
    public int getTotalCookTime() {
        ItemStack input = getStackInSlot(SLOT_INPUT);
        IBlastFurnaceRecipe recipe = RailcraftCraftingManager.blastFurnace.getRecipe(input);
        if (recipe != null) {
            return recipe.getCookTime();
        }
        return 1;
    }

    @Override
    public int getBurnProgressScaled(int i) {
        if (burnTime <= 0 || currentItemBurnTime <= 0) {
            return 0;
        }
        int scale = burnTime * i / currentItemBurnTime;
        scale = Math.min(scale, i);
        scale = Math.max(scale, 0);
        return scale;
    }

    private void setLavaIdle() {
        int xLava = xCoord + 1;
        int yLava = yCoord + 1;
        int zLava = zCoord + 1;
        if (worldObj.getBlockId(xLava, yLava, zLava) == 0) {
            worldObj.setBlockAndMetadata(xLava, yLava, zLava, Block.lavaStill.blockID, 7);
        }
    }

    private void setLavaBurn() {
        int xLava = xCoord + 1;
        int yLava = yCoord + 1;
        int zLava = zCoord + 1;
        if (worldObj.getBlockId(xLava, yLava, zLava) == 0) {
            worldObj.setBlockAndMetadata(xLava, yLava, zLava, Block.lavaMoving.blockID, 1);
        }
        yLava += 1;
        if (worldObj.getBlockId(xLava, yLava, zLava) == 0) {
            worldObj.setBlockAndMetadata(xLava, yLava, zLava, Block.lavaMoving.blockID, 1);
        }
    }

    private void destroyLava() {
        int xLava = xCoord + 1;
        int yLava = yCoord + 2;
        int zLava = zCoord + 1;
        if (worldObj.getBlockMaterial(xLava, yLava, zLava) == Material.lava) {
            worldObj.setBlock(xLava, yLava, zLava, 0);
        }
        yLava -= 1;
        if (worldObj.getBlockMaterial(xLava, yLava, zLava) == Material.lava) {
            worldObj.setBlock(xLava, yLava, zLava, 0);
        }
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isHost(getWorld())) {
            if (isMaster()) {
                int prevBurnTime = burnTime;
                if (update > finishedAt + COOK_STEP_LENGTH + 5) {
                    if (cookTime <= 0) {
                        setCooking(false);
                    }
                }

                if (burnTime >= FUEL_PER_TICK) {
                    burnTime -= FUEL_PER_TICK;
                } else {
                    burnTime = 0;
                }

                if (isBurning()) {
                    setLavaBurn();
                } else {
                    setLavaIdle();
                }

                ItemStack input = getStackInSlot(SLOT_INPUT);
                if (input != null && input.stackSize > 0) {

                    ItemStack output = getStackInSlot(SLOT_OUTPUT);
                    IBlastFurnaceRecipe recipe = RailcraftCraftingManager.blastFurnace.getRecipe(input);

                    if (recipe != null && recipe.isRoomForOutput(output)) {

                        if (burnTime <= FUEL_PER_TICK * 2) {
                            ItemStack fuel = getStackInSlot(SLOT_FUEL);
                            if (fuel != null && ((fuel.getItem() == Item.coal && fuel.getItemDamage() == 1) || InvTools.isItemEqual(fuel, RailcraftToolItems.getCoalCoke()))) {
                                int itemBurnTime = MiscTools.getItemBurnTime(fuel);
                                if (itemBurnTime > 0) {
                                    currentItemBurnTime = itemBurnTime + burnTime;
                                    burnTime = currentItemBurnTime;
                                    decrStackSize(SLOT_FUEL, 1);
                                }
                            }
                        }

                        if (update % COOK_STEP_LENGTH == 0 && isBurning()) {
                            cookTime += COOK_STEP_LENGTH;
                            setCooking(true);

                            if (cookTime >= recipe.getCookTime()) {
                                cookTime = 0;
                                finishedAt = update;
                                if (output == null) {
                                    setInventorySlotContents(SLOT_OUTPUT, recipe.getOutput());
                                } else {
                                    output.stackSize += recipe.getOutputStackSize();
                                }
                                decrStackSize(SLOT_INPUT, 1);
                            }
                            sendUpdateToClient();
                        }
                    } else {
                        cookTime = 0;
                        setCooking(false);
                    }
                } else {
                    cookTime = 0;
                    setCooking(false);
                }

                if (prevBurnTime == 0 ^ burnTime == 0) {
                    sendUpdateToClient();
                }
            }
        }
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        if (side == ForgeDirection.DOWN) {
            return SLOT_FUEL;
        }
        if (side == ForgeDirection.UP) {
            return SLOT_INPUT;
        }
        return SLOT_OUTPUT;
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 1;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock masterBlock = getMasterBlock();
        if (masterBlock != null) {
            GuiHandler.openGui(EnumGui.BLAST_FURNACE, player, worldObj, masterBlock.xCoord, masterBlock.yCoord, masterBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setInteger("burnTime", burnTime);
        data.setInteger("currentItemBurnTime", currentItemBurnTime);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        burnTime = data.getInteger("burnTime");
        currentItemBurnTime = data.getInteger("currentItemBurnTime");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeInt(burnTime);
        data.writeInt(currentItemBurnTime);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        burnTime = data.readInt();
        currentItemBurnTime = data.readInt();
    }

    @Override
    public boolean needsFuel() {
        ItemStack fuel = getStackInSlot(SLOT_FUEL);
        return fuel == null || fuel.stackSize < 8;
    }

    @Override
    public boolean isBurning() {
        TileBlastFurnace mBlock = (TileBlastFurnace) getMasterBlock();
        if (mBlock != null) {
            return mBlock.burnTime > 0;
        }
        return false;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (!isStructureValid()) {
            return 0;
        }
        if (stack == null) {
            return 0;
        }
        if ((stack.itemID == Item.coal.itemID && stack.getItemDamage() == 1)
                || InvTools.isItemEqual(stack, RailcraftToolItems.getCoalCoke())) {
            IInventory inv = invFuel;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remaining = InvTools.moveItemStack(stack, inv);
            if (remaining == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remaining.stackSize;
        }

        if (RailcraftCraftingManager.blastFurnace.getRecipe(stack) != null) {
            IInventory inv = invInput;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remaining = InvTools.moveItemStack(stack, inv);
            if (remaining == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remaining.stackSize;
        }

        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        ItemStack output = getStackInSlot(SLOT_OUTPUT);

        ItemStack newStack = null;
        if (output != null) {
            newStack = output.copy();
            int stackSize = Math.min(output.stackSize, maxItemCount);
            newStack.stackSize = stackSize;

            if (doRemove) {
                decrStackSize(SLOT_OUTPUT, stackSize);
            }
        }

        if (newStack == null) {
            return new ItemStack[0];
        }

        return new ItemStack[]{newStack};
    }
}
