package railcraft.common.blocks.machine.alpha;

import buildcraft.api.inventory.ISpecialInventory;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.api.crafting.ICokeOvenRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.TileMultiBlockOven;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.plugins.buildcraft.triggers.INeedsFuel;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.liquids.tanks.FakeTank;
import railcraft.common.util.misc.Game;

public class TileCokeOven extends TileMultiBlockOven implements ITankContainer, ISpecialInventory, ISidedInventory, INeedsFuel {

    private static final int COOK_STEP_LENGTH = 50;
    private static final int SLOT_INPUT = 0;
    private static final int SLOT_OUTPUT = 1;
    private static final int SLOT_LIQUID_OUTPUT = 2;
    private static final int SLOT_LIQUID_INPUT = 3;
    private static final int TANK_CAPACITY = 64 * LiquidManager.BUCKET_VOLUME;
    private final static List<MultiBlockPattern> patterns = new ArrayList<MultiBlockPattern>();
    private int finishedAt;
    private int cookTimeTotal = 3600;
    private final TankManager tankManager = new TankManager();
    private final StandardTank tank;
    private IInventory invInput = new InventoryMapper(this, SLOT_INPUT, 1);
    private IInventory invOutput = new InventoryMapper(this, SLOT_OUTPUT, 2);

    static {
        char[][][] map = {
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'W', 'B', 'O'},
                {'O', 'W', 'A', 'W', 'O'},
                {'O', 'B', 'W', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },};
        patterns.add(new MultiBlockPattern(map));
    }

    public TileCokeOven() {
        super("gui.coke.oven", 4, patterns);
        tank = new StandardTank(TANK_CAPACITY, this);
        tankManager.addTank(tank);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineAlpha.COKE_OVEN;
    }

    public TankManager getTankManager() {
        TileCokeOven mBlock = (TileCokeOven) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager;
        }
        return null;
    }

    @Override
    public int getBlockTexture(int side) {
        if (getPatternMarker() == 'W' && isStructureValid()) {
            if (isBurning()) {
                return getMachineType().getTexture(7);
            }
            return getMachineType().getTexture(6);
        }
        return getMachineType().getTexture(0);
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        if (isStructureValid() && LiquidManager.getInstance().handleRightClick(this, 0, player, false, true)) {
            return true;
        } else if (LiquidManager.getInstance().isContainer(player.inventory.getCurrentItem())) {
            return true;
        }
        return super.blockActivated(player, side);
    }

    @Override
    public int getTotalCookTime() {
        TileCokeOven mBlock = (TileCokeOven) getMasterBlock();
        if (mBlock != null) {
            return mBlock.cookTimeTotal;
        }
        return 3600;
    }

    @Override
    public int getBurnProgressScaled(int i) {
        return ((getTotalCookTime() - getCookTime()) * i) / getTotalCookTime();
    }

    @Override
    public boolean needsFuel() {
        ItemStack fuel = getStackInSlot(SLOT_INPUT);
        return fuel == null || fuel.stackSize < 8;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isHost(getWorld())) {
            if (isMaster()) {
                if (update > finishedAt + COOK_STEP_LENGTH + 5) {
                    if (cookTime <= 0) {
                        setCooking(false);
                    }
                }

                ItemStack input = getStackInSlot(SLOT_INPUT);
                if (input != null && input.stackSize > 0 && !InvTools.isSynthetic(input)) {
                    if (update % COOK_STEP_LENGTH == 0) {
                        ItemStack output = getStackInSlot(SLOT_OUTPUT);
                        ICokeOvenRecipe recipe = RailcraftCraftingManager.cokeOven.getRecipe(input);

                        if (recipe != null) {
                            if ((output == null || (output.isItemEqual(recipe.getOutput()) && output.stackSize + recipe.getOutput().stackSize <= output.getMaxStackSize()))
                                    && tank.fill(recipe.getLiquidOutput(), false) >= recipe.getLiquidOutput().amount) {
                                cookTimeTotal = recipe.getCookTime();
                                cookTime += COOK_STEP_LENGTH;
                                setCooking(true);

                                if (cookTime >= recipe.getCookTime()) {
                                    cookTime = 0;
                                    finishedAt = update;
                                    decrStackSize(SLOT_INPUT, 1);
                                    if (output == null) {
                                        setInventorySlotContents(SLOT_OUTPUT, recipe.getOutput());
                                    } else {
                                        output.stackSize += recipe.getOutput().stackSize;
                                    }
                                    tank.fill(recipe.getLiquidOutput(), true);
                                    sendUpdateToClient();
                                }

                            } else {
                                cookTime = 0;
                                setCooking(false);
                            }
                        } else {
                            cookTime = 0;
                            setCooking(false);
                            setInventorySlotContents(SLOT_INPUT, null);
                            dropItem(input);
                        }
                    }
                } else {
                    cookTime = 0;
                    setCooking(false);
                }

                ItemStack topSlot = getStackInSlot(SLOT_LIQUID_INPUT);
                if (topSlot != null && !LiquidManager.getInstance().isContainer(topSlot)) {
                    setInventorySlotContents(SLOT_LIQUID_INPUT, null);
                    dropItem(topSlot);
                }

                ItemStack bottomSlot = getStackInSlot(SLOT_LIQUID_OUTPUT);
                if (bottomSlot != null && !LiquidManager.getInstance().isContainer(bottomSlot)) {
                    setInventorySlotContents(SLOT_LIQUID_OUTPUT, null);
                    dropItem(bottomSlot);
                }

                if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
                    LiquidManager.getInstance().processContainers(tank, this, SLOT_LIQUID_INPUT, SLOT_LIQUID_OUTPUT, false, true);
                }
            }
        }
    }

//    @Override
//    protected void onMasterReset()
//    {
//        super.onMasterReset();
//        tank.setLiquid(null);
//    }
    @Override
    public int getStartInventorySide(ForgeDirection side) {
        switch (side) {
            case UP:
                return SLOT_INPUT;
            case DOWN:
                return SLOT_LIQUID_INPUT;
            default:
                return SLOT_OUTPUT;
        }
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        switch (side) {
            case UP:
                return 1;
            case DOWN:
                return 1;
            default:
                return 2;
        }
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock masterBlock = getMasterBlock();
        if (masterBlock != null && isStructureValid()) {
            GuiHandler.openGui(EnumGui.COKE_OVEN, player, worldObj, masterBlock.xCoord, masterBlock.yCoord, masterBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        // old stuff
        if (data.hasKey("tank")) {
            LiquidStack liquid = new LiquidStack(0, 0, 0);
            liquid.readFromNBT(data.getCompoundTag("tank"));
            tank.setLiquid(liquid);
        } else {
            tankManager.readTanksFromNBT(data);
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        tankManager.writeTanksToNBT(data);
    }

    /**
     * Fills liquid into internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return fill(0, resource, doFill);
    }

    /**
     * Fills liquid into the specified internal tank.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        return 0;
    }

    /**
     * Drains liquid out of internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is drained to.
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return drain(0, maxDrain, doDrain);
    }

    /**
     * Drains liquid out of the specified internal tank.
     *
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        if (tankIndex != 0) {
            return null;
        }
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.drain(tankIndex, maxDrain, doDrain);
        }
        return null;
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection dir) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.getTanks();
        }
        return FakeTank.ARRAY;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        TileCokeOven mBlock = (TileCokeOven) getMasterBlock();
        if (mBlock != null) {
            if (stack == null) {
                return 0;
            }
            if (LiquidManager.getInstance().isEmptyContainer(stack)) {
                IInventory slot = new InventoryMapper(mBlock, SLOT_LIQUID_INPUT, 1);
                if (!doAdd) {
                    slot = new InventoryCopy(slot);
                }
                ItemStack ret = InvTools.moveItemStack(stack.copy(), slot);
                if (ret != null && stack.stackSize != ret.stackSize) {
                    return stack.stackSize - ret.stackSize;
                } else if (ret == null) {
                    return stack.stackSize;
                }
            }
            if (InvTools.isSynthetic(stack)) {
                return 0;
            }
            if (RailcraftCraftingManager.cokeOven.getRecipe(stack) != null) {
                IInventory inv = mBlock.invInput;
                if (!doAdd) {
                    inv = new InventoryCopy(inv);
                }
                ItemStack remainder = InvTools.moveItemStack(stack, inv);
                if (remainder == null) {
                    return stack.stackSize;
                }
                return stack.stackSize - remainder.stackSize;
            }
//            System.out.println("no recipe");
            return 0;
        }

        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        TileCokeOven mBlock = (TileCokeOven) getMasterBlock();
        if (mBlock != null) {
            IInventory inv = mBlock.invOutput;
            if (!doRemove) {
                inv = new InventoryCopy(inv);
            }
            return InvTools.removeItems(inv, maxItemCount);
        }
        return new ItemStack[0];
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.getTank(0);
        }
        return FakeTank.INSTANCE;
    }
}
