package railcraft.common.blocks.machine.alpha;

import buildcraft.api.gates.IAction;
import buildcraft.api.inventory.ISpecialInventory;
import buildcraft.api.power.IPowerProvider;
import buildcraft.api.power.IPowerReceptor;
import buildcraft.api.power.PowerFramework;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import railcraft.common.api.crafting.IRockCrusherRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.TileMultiBlockInventory;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.gui.indicator.IIndicatorController;
import railcraft.common.gui.util.ToolTip;
import railcraft.common.plugins.buildcraft.actions.EnumTriggerActions;
import railcraft.common.plugins.buildcraft.triggers.IHasWork;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.util.misc.Game;
import railcraft.common.util.sounds.SoundHelper;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TileRockCrusher extends TileMultiBlockInventory implements IPowerReceptor, ISidedInventory, IHasWork, ISpecialInventory {

    private final static int PROCESS_TIME = 100;
    private final static int POWER = 16;
    private final static int MAX_RECEIVE = 500;
    private final static int MAX_ENERGY = POWER * PROCESS_TIME;
    private final static List<MultiBlockPattern> patterns = new ArrayList<MultiBlockPattern>();
    private int processTime;
    private IInventory invInput = new InventoryMapper(this, 0, 9);
    private IInventory invOutput = new InventoryMapper(this, 9, 9);
    private IPowerProvider provider;
    private boolean isWorking = false;
    private boolean paused = false;
    public int guiEnergy;
    private final Set<IAction> actions = new HashSet<IAction>();
    private IIndicatorController energyIndicator = new EnergyIndicator();

    private class EnergyIndicator implements IIndicatorController {

        List<ToolTip> tips = new ArrayList<ToolTip>();
        ToolTip tip = new ToolTip();

        private EnergyIndicator() {
            tips.add(tip);
        }

        @Override
        public List<ToolTip> getToolTip() {
            tip.text = String.format("%d MJ", guiEnergy);
            return tips;
        }

        @Override
        public int getScaledLevel(int size) {
            float e = Math.min(guiEnergy, MAX_ENERGY);
            return (int) (e * size / MAX_ENERGY);
        }
    };

    public IIndicatorController getEnergyIndicator() {
//        if(isMaster) {
//            return energyIndicator;
//        }
//        TileRockCrusher mBlock = (TileRockCrusher)getMasterBlock();
//        if(mBlock != null) {
//            return mBlock.energyIndicator;
//        }
        return energyIndicator;
    }

    static {
        char[][][] map1 = {
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'D', 'B', 'O'},
                {'O', 'B', 'D', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            }
        };
        patterns.add(new MultiBlockPattern(map1));


        char[][][] map2 = {
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'D', 'D', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'}
            }
        };
        patterns.add(new MultiBlockPattern(map2));
    }

    public TileRockCrusher() {
        super(EnumMachineAlpha.ROCK_CRUSHER.getTag(), 18, patterns);

        if (PowerFramework.currentFramework != null) {
            provider = PowerFramework.currentFramework.createPowerProvider();
        }
        initPowerProvider();
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineAlpha.ROCK_CRUSHER;
    }

    private void initPowerProvider() {
        if (PowerFramework.currentFramework != null) {
//            powerProvider = PowerFramework.currentFramework.createPowerProvider();
            provider.configure(0, 1, MAX_RECEIVE, POWER, MAX_ENERGY);
            provider.configurePowerPerdition(1, 10);
        }
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isHost(getWorld())) {

            if (PowerFramework.currentFramework != null) {
                provider.update(this);
            }

            if (isMaster()) {
                if (update % 16 == 0) {
                    processActions();
                }

                if (paused) {
                    return;
                }

                ItemStack input = null;
                IRockCrusherRecipe recipe = null;
                for (int i = 0; i < 9; i++) {
                    input = invInput.getStackInSlot(i);
                    if (input != null) {
                        recipe = RailcraftCraftingManager.rockCrusher.getRecipe(input);
                        if (recipe != null) {
                            break;
                        }
                    }
                }

                if (recipe != null) {
                    if (processTime >= PROCESS_TIME) {
                        isWorking = false;
                        IInventory tempInv = new InventoryCopy(invOutput);
                        boolean hasRoom = true;
                        List<ItemStack> outputs = recipe.getRandomizedOuput();
                        for (ItemStack output : outputs) {
                            output = InvTools.moveItemStack(output, tempInv);
                            if (output != null) {
                                hasRoom = false;
                                break;
                            }
                        }

                        if (hasRoom) {
                            for (ItemStack output : outputs) {
                                InvTools.moveItemStack(output, invOutput);
                            }

                            InvTools.removeOneItem(invInput, input);

                            SoundHelper.playSound(worldObj, xCoord, yCoord, zCoord, "mob.irongolem.death", 1.5f, worldObj.rand.nextFloat() * 0.25F + 0.7F);

                            processTime = 0;
                        }

                    } else {
                        isWorking = true;
                        if (PowerFramework.currentFramework != null) {
                            float energy = getPowerProvider().useEnergy(POWER, POWER, true);
                            if (energy >= POWER) {
                                processTime++;
                            }
                        } else {
                            processTime++;
                        }
                    }
                } else {
                    processTime = 0;
                    isWorking = false;
                }
            }
        }
    }

    @Override
    protected void onMasterReset() {
        super.onMasterReset();
        processTime = 0;
        if (PowerFramework.currentFramework != null) {
            provider = PowerFramework.currentFramework.createPowerProvider();
        }
        initPowerProvider();
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock mBlock = getMasterBlock();
        if (mBlock != null) {
            GuiHandler.openGui(EnumGui.ROCK_CRUSHER, player, worldObj, mBlock.xCoord, mBlock.yCoord, mBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    public int getBlockTexture(int side) {
        return EnumMachineAlpha.ROCK_CRUSHER.getTexture(side);
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        if (side == ForgeDirection.UP) {
            return 0;
        }
        return 9;
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 9;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setInteger("processTime", processTime);

        if (provider != null) {
            PowerFramework.currentFramework.savePowerProvider(this, data);
        }
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        processTime = data.getInteger("processTime");

        if (PowerFramework.currentFramework != null) {
            PowerFramework.currentFramework.loadPowerProvider(this, data);
            initPowerProvider();
        }
    }

    public int getProcessTime() {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null) {
            return mBlock.processTime;
        }
        return -1;
    }

    public void setProcessTime(int processTime) {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null) {
            mBlock.processTime = processTime;
        }
    }

    public int getProgressScaled(int i) {
        return (getProcessTime() * i) / PROCESS_TIME;
    }

    @Override
    public void setPowerProvider(IPowerProvider provider) {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null) {
            mBlock.provider = provider;
        }
        this.provider = provider;
    }

    @Override
    public IPowerProvider getPowerProvider() {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null) {
            return mBlock.provider;
        }
        return provider;
    }

    @Override
    public int powerRequest() {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null && mBlock.provider != null) {
            return (int) Math.ceil(Math.min(mBlock.provider.getMaxEnergyReceived(), mBlock.provider.getMaxEnergyStored() - mBlock.provider.getEnergyStored()));
        }
        return 0;
    }

    @Override
    public void doWork() {
    }

    @Override
    public boolean hasWork() {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null) {
            return mBlock.isWorking;
        }
        return false;
    }

    public void setPaused(boolean p) {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null) {
            mBlock.paused = p;
        }
    }

    private void processActions() {
        paused = false;
        for (IAction action : actions) {
            if (action == EnumTriggerActions.PAUSE.getAction()) {
                paused = true;
            }
        }
        actions.clear();
    }

    @Override
    public void actionActivated(IAction action) {
        TileRockCrusher mBlock = (TileRockCrusher) getMasterBlock();
        if (mBlock != null) {
            mBlock.actions.add(action);
        }
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (stack == null || from != ForgeDirection.UP) {
            return 0;
        }

        LiquidManager lm = LiquidManager.getInstance();

        if (RailcraftCraftingManager.rockCrusher.getRecipe(stack) != null) {
            IInventory inv = invInput;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remainder = InvTools.moveItemStack(stack, inv);
            if (remainder == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remainder.stackSize;
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        if (from == ForgeDirection.UP) {
            return new ItemStack[0];
        }

        IInventory inv = invOutput;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return InvTools.removeItems(inv, maxItemCount);
    }
}
