package railcraft.common.blocks.machine.alpha;

import buildcraft.api.gates.IAction;
import buildcraft.api.inventory.ISpecialInventory;
import buildcraft.api.power.IPowerProvider;
import buildcraft.api.power.IPowerReceptor;
import buildcraft.api.power.PowerFramework;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.minecraft.inventory.Container;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntityChest;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.TileMachineItem;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.gui.indicator.IIndicatorController;
import railcraft.common.gui.util.ToolTip;
import railcraft.common.plugins.buildcraft.actions.EnumTriggerActions;
import railcraft.common.plugins.buildcraft.triggers.IHasWork;
import railcraft.common.util.crafting.RollingMachineCraftingManager;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.inventory.ItemStackSizeSorter;
import railcraft.common.util.misc.Game;

public class TileRollingMachine extends TileMachineItem implements ISpecialInventory, IPowerReceptor, ISidedInventory, IHasWork {

    private final static int PROCESS_TIME = 100;
    private final static int ACTIVATION_POWER = 5;
    private final static int MAX_RECEIVE = 100;
    private final static int MAX_ENERGY = ACTIVATION_POWER * PROCESS_TIME;
    private InventoryCrafting craftMatrix = new InventoryCrafting(new RollingContainer(), 3, 3);
    private IInventory invResult = new InventoryMapper(this, 0, 1);
    private IInventory invBuffer = new InventoryMapper(this, 1, 4);
    private IPowerProvider provider;
    public boolean useLast;
    private boolean isWorking, paused;
    private int progress, update;
    public int guiEnergy;
    private final Set<IAction> actions = new HashSet<IAction>();
    private IIndicatorController energyIndicator = new EnergyIndicator();

    private class EnergyIndicator implements IIndicatorController {

        List<ToolTip> tips = new ArrayList<ToolTip>();
        ToolTip tip = new ToolTip();

        private EnergyIndicator() {
            tips.add(tip);
        }

        @Override
        public List<ToolTip> getToolTip() {
            tip.text = String.format("%d MJ", guiEnergy);
            return tips;
        }

        @Override
        public int getScaledLevel(int size) {
            float e = Math.min(guiEnergy, MAX_ENERGY);
            return (int) (e * size / MAX_ENERGY);
        }
    };

    public IIndicatorController getEnergyIndicator() {
        return energyIndicator;
    }

    private static class RollingContainer extends Container {

        @Override
        public boolean canInteractWith(EntityPlayer entityplayer) {
            return true;
        }
    }

    public TileRollingMachine() {
        super(5);
        if (PowerFramework.currentFramework != null) {
            provider = PowerFramework.currentFramework.createPowerProvider();
            initPowerProvider();
        }
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineAlpha.ROLLING_MACHINE;
    }

    private void initPowerProvider() {
        if (PowerFramework.currentFramework != null) {
            provider.configure(0, 1, MAX_RECEIVE, ACTIVATION_POWER, MAX_ENERGY);
            provider.configurePowerPerdition(1, 40);
        }
    }

    @Override
    public void setPowerProvider(IPowerProvider provider) {
        this.provider = provider;
    }

    @Override
    public IPowerProvider getPowerProvider() {
        return provider;
    }

    @Override
    public void doWork() {
    }

    @Override
    public int powerRequest() {
        if (provider != null) {
            return (int) Math.ceil(Math.min(provider.getMaxEnergyReceived(), provider.getMaxEnergyStored() - provider.getEnergyStored()));
        }
        return 0;
    }

    @Override
    public int getBlockTexture(int side) {
        return EnumMachineAlpha.ROLLING_MACHINE.getTexture(side);
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setInteger("progress", progress);

        if (provider != null) {
            PowerFramework.currentFramework.savePowerProvider(this, data);
        }

        NBTTagList nbttaglist = new NBTTagList();
        for (int i = 0; i < craftMatrix.getSizeInventory(); i++) {
            if (craftMatrix.getStackInSlot(i) != null) {
                NBTTagCompound nbttagcompound1 = new NBTTagCompound();
                nbttagcompound1.setByte("Slot", (byte) i);
                craftMatrix.getStackInSlot(i).writeToNBT(nbttagcompound1);
                nbttaglist.appendTag(nbttagcompound1);
            }
        }
        data.setTag("Crafting", nbttaglist);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        progress = data.getInteger("progress");

        if (PowerFramework.currentFramework != null) {
            PowerFramework.currentFramework.loadPowerProvider(this, data);
            initPowerProvider();
        }

        NBTTagList nbttaglist = data.getTagList("Crafting");
        for (int i = 0; i < craftMatrix.getSizeInventory(); i++) {
            craftMatrix.setInventorySlotContents(i, null);
        }
        for (int i = 0; i < nbttaglist.tagCount(); i++) {
            NBTTagCompound nbttagcompound1 = (NBTTagCompound) nbttaglist.tagAt(i);
            int j = nbttagcompound1.getByte("Slot") & 255;
            if (j >= 0 && j < craftMatrix.getSizeInventory()) {
                craftMatrix.setInventorySlotContents(j, ItemStack.loadItemStackFromNBT(nbttagcompound1));
            }
        }
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        if (player.getDistanceSq(xCoord + 0.5, yCoord + 0.5, zCoord + 0.5) > 64D) {
            return false;
        }
        GuiHandler.openGui(EnumGui.ROLLING_MACHINE, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        switch (side) {
            case UP:
                return 1;
            default:
                return 0;
        }
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        switch (side) {
            case UP:
                return 4;
            default:
                return 1;
        }
    }

    @Override
    public void onInventoryChanged() {
        craftMatrix.onInventoryChanged();
    }

    @Override
    public void onBlockRemoval() {
        super.onBlockRemoval();
        InvTools.dropInventory(craftMatrix, worldObj, xCoord, yCoord, zCoord);
    }

    public void setProgress(int progress) {
        this.progress = progress;
    }

    public int getProgress() {
        return progress;
    }

    public int getProgressScaled(int i) {
        return (progress * i) / PROCESS_TIME;
    }

    public InventoryCrafting getCraftMatrix() {
        return craftMatrix;
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();
        update++;

        if (Game.isNotHost(worldObj)) {
            return;
        }

        if (PowerFramework.currentFramework != null) {
            provider.update(this);
        }

        for (int inputSlot = 0; inputSlot < invBuffer.getSizeInventory(); inputSlot++) {
            ItemStack inputStack = invBuffer.getStackInSlot(inputSlot);
            if (inputStack == null) {
                continue;
            }

            if (inputStack.stackSize <= 0) {
                invBuffer.setInventorySlotContents(inputSlot, null);
                continue;
            }

            if (!inputStack.isStackable()) {
                invBuffer.setInventorySlotContents(inputSlot, null);
                InvTools.dropItem(inputStack, worldObj, xCoord, yCoord, zCoord);
                continue;
            }

            ItemStack outputStack = null;
            for (int recipeSlot = 0; recipeSlot < craftMatrix.getSizeInventory(); recipeSlot++) {
                ItemStack craftingStack = craftMatrix.getStackInSlot(recipeSlot);
                if (InvTools.isItemEqual(inputStack, craftingStack) && (outputStack == null || craftingStack.stackSize < outputStack.stackSize)) {
                    outputStack = craftingStack;
                }
            }

            if (outputStack != null && outputStack.stackSize < outputStack.getMaxStackSize()) {
                invBuffer.decrStackSize(inputSlot, 1);
                outputStack.stackSize++;
                break;
            } else if (outputStack == null) {
                invBuffer.setInventorySlotContents(inputSlot, null);
                InvTools.dropItem(inputStack, worldObj, xCoord, yCoord, zCoord);
            }
        }

        List<IInventory> chests = InvTools.getAdjacentInventories(worldObj, xCoord, yCoord, zCoord, TileEntityChest.class);
        for (int slot = 0; slot < craftMatrix.getSizeInventory(); slot++) {
            ItemStack stack = craftMatrix.getStackInSlot(slot);
            if (stack != null && stack.isStackable() && stack.stackSize == 1) {
                for (IInventory inv : chests) {
                    ItemStack request = InvTools.removeOneItem(inv, stack);
                    if (request != null) {
                        stack.stackSize++;
                        break;
                    }
                }
                if (stack.stackSize > 1) {
                    break;
                }
            }
        }

        if (update % 16 == 0) {
            processActions();
        }

        if (paused) {
            return;
        }

        ItemStack recipe = RollingMachineCraftingManager.getInstance().findMatchingRecipe(craftMatrix, worldObj);
        if (recipe != null && canMakeMore()) {
            if (progress >= PROCESS_TIME) {
                isWorking = false;
                if (InvTools.isRoomForStack(recipe, invResult)) {
                    for (int i = 0; i < craftMatrix.getSizeInventory(); i++) {
                        craftMatrix.decrStackSize(i, 1);
                    }
                    InvTools.moveItemStack(recipe, invResult);
                    useLast = false;
                    progress = 0;
                }
            } else {
                isWorking = true;
                if (PowerFramework.currentFramework != null) {
                    float energy = getPowerProvider().useEnergy(ACTIVATION_POWER, ACTIVATION_POWER, true);
                    if (energy >= ACTIVATION_POWER) {
                        progress++;
                    }
                } else {
                    progress++;
                }
            }
        } else {
            progress = 0;
            isWorking = false;
        }
    }

    @Override
    public boolean hasWork() {
        return isWorking;
    }

    public void setPaused(boolean p) {
        paused = p;
    }

    private void processActions() {
        paused = false;
        for (IAction action : actions) {
            if (action == EnumTriggerActions.PAUSE.getAction()) {
                paused = true;
            }
        }
        actions.clear();
    }

    @Override
    public void actionActivated(IAction action) {
        actions.add(action);
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (stack == null || !stack.isStackable() || stack.stackSize <= 0) {
            return 0;
        }
        stack = stack.copy();
        IInventory inv = craftMatrix;
        if (!doAdd) {
            inv = new InventoryCopy(inv);
        }
        List<ItemStack> slots = new ArrayList<ItemStack>();
        for (int i = 0; i < inv.getSizeInventory(); i++) {
            ItemStack slot = inv.getStackInSlot(i);
            if (InvTools.isItemEqual(slot, stack)) {
                slots.add(slot);
            }
        }
        if (!slots.isEmpty()) {
            int used = 0;
            while (stack.stackSize > 0) {
                ItemStackSizeSorter.sort(slots);
                ItemStack slot = slots.get(0);
                if (slot.stackSize < slot.getMaxStackSize()) {
                    slot.stackSize++;
                    stack.stackSize--;
                    used++;
                } else {
                    break;
                }
            }
            return used;
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int max) {
        IInventory inv = invResult;
        if (!doRemove) {
            inv = new InventoryCopy(invResult);
        }
        return InvTools.removeItems(inv, max);
    }

    public boolean canMakeMore() {
        if (RollingMachineCraftingManager.getInstance().findMatchingRecipe(craftMatrix, worldObj) == null) {
            return false;
        }
        if (useLast) {
            return true;
        }
        for (int i = 0; i < craftMatrix.getSizeInventory(); i++) {
            ItemStack slot = craftMatrix.getStackInSlot(i);
            if (slot != null && slot.stackSize <= 1) {
                return false;
            }
        }
        return true;
    }
}
