package railcraft.common.blocks.machine.alpha;

import buildcraft.api.inventory.ISpecialInventory;
import buildcraft.api.tools.IToolWrench;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.FurnaceRecipes;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.TileMultiBlockInventory;
import railcraft.common.blocks.machine.beta.ISteamUser;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.liquids.LiquidFilter;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.liquids.tanks.FakeTank;
import railcraft.common.liquids.tanks.FilteredTank;
import railcraft.common.util.effects.EffectManager;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.sounds.SoundHelper;

public class TileSteamOven extends TileMultiBlockInventory implements ITankContainer, ISpecialInventory, ISidedInventory, ISteamUser {

    private static final int STEAM_PER_BATCH = 8000;
    private static final int TOTAL_COOK_TIME = 256;
    private static final int COOK_STEP = 16;
    private static final int ITEMS_SMELTED = 9;
    private static final int SLOT_INPUT = 0;
    private static final int SLOT_OUTPUT = 9;
    private static final int TANK_CAPACITY = 8 * LiquidManager.BUCKET_VOLUME;
    private final static List<MultiBlockPattern> patterns = new ArrayList<MultiBlockPattern>();
    private ForgeDirection facing = ForgeDirection.NORTH;
    public int cookTime;
    public boolean finishedCycle = false;
    private final TankManager tankManager = new TankManager();
    private final StandardTank tank;
    private IInventory invInput = new InventoryMapper(this, SLOT_INPUT, 9);
    private IInventory invOutput = new InventoryMapper(this, SLOT_OUTPUT, 9);

    static {
        char[][][] map = {
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},},
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},},};
        patterns.add(new MultiBlockPattern(map));
    }

    public TileSteamOven() {
        super("gui.steam.oven", 18, patterns);
        tank = new FilteredTank(TANK_CAPACITY, LiquidFilter.STEAM);
        tankManager.addTank(tank);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineAlpha.STEAM_OVEN;
    }

    public TankManager getTankManager() {
        TileSteamOven mBlock = (TileSteamOven) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager;
        }
        return null;
    }

    @Override
    public int getBlockTexture(int side) {
        if (isStructureValid() && side == getFacing().ordinal()) {
            switch (side) {
                case 2:
                    if (getPatternPositionY() == 2) {
                        if (getPatternPositionX() == 2) {
                            return getMachineType().getTexture(6);
                        }
                        return getMachineType().getTexture(7);
                    }
                    if (getPatternPositionX() == 2) {
                        return getMachineType().getTexture(8);
                    }
                    return getMachineType().getTexture(9);
                case 3:
                    if (getPatternPositionY() == 2) {
                        if (getPatternPositionX() == 1) {
                            return getMachineType().getTexture(6);
                        }
                        return getMachineType().getTexture(7);
                    }
                    if (getPatternPositionX() == 1) {
                        return getMachineType().getTexture(8);
                    }
                    return getMachineType().getTexture(9);
                case 4:
                    if (getPatternPositionY() == 2) {
                        if (getPatternPositionZ() == 1) {
                            return getMachineType().getTexture(6);
                        }
                        return getMachineType().getTexture(7);
                    }
                    if (getPatternPositionZ() == 1) {
                        return getMachineType().getTexture(8);
                    }
                    return getMachineType().getTexture(9);
                case 5:
                    if (getPatternPositionY() == 2) {
                        if (getPatternPositionZ() == 2) {
                            return getMachineType().getTexture(6);
                        }
                        return getMachineType().getTexture(7);
                    }
                    if (getPatternPositionZ() == 2) {
                        return getMachineType().getTexture(8);
                    }
                    return getMachineType().getTexture(9);
            }
        }
        if (side > 1) {
            return getMachineType().getTexture(2);
        }
        return getMachineType().getTexture(0);
    }

    public int getCookProgressScaled(int i) {
        int scale = (getCookTime() * i) / TOTAL_COOK_TIME;
        scale = Math.min(scale, i);
        scale = Math.max(scale, 0);
        return scale;
    }

    public int getCookTime() {
        TileSteamOven masterOven = (TileSteamOven) getMasterBlock();
        if (masterOven != null) {
            return masterOven.cookTime;
        }
        return -1;
    }

    public ForgeDirection getFacing() {
        TileSteamOven masterOven = (TileSteamOven) getMasterBlock();
        if (masterOven != null) {
            return masterOven.facing;
        }
        return facing;
    }

    public boolean hasFinishedCycle() {
        TileSteamOven mBlock = (TileSteamOven) getMasterBlock();
        return mBlock != null && mBlock.finishedCycle;
    }

    public void setHasFinishedCycle(boolean finished) {
        if (finishedCycle != finished) {
            finishedCycle = finished;
            sendUpdateToClient();
        }
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isNotHost(getWorld())) {
            if (hasFinishedCycle()) {
                EffectManager.instance.steamEffect(worldObj, this, +0.25);
            }
            return;
        }

        if (isMaster()) {
            if (update % COOK_STEP == 0) {
                setHasFinishedCycle(false);
                if (cookTime <= 0 && hasRecipe() && drainSteam()) {
                    cookTime = 1;
                } else if (cookTime > 0 && hasRecipe()) {
                    cookTime += COOK_STEP;
                    if (cookTime >= TOTAL_COOK_TIME) {
                        if (smeltItems()) {
                            cookTime = 0;
                            setHasFinishedCycle(true);
                            SoundHelper.playSound(worldObj, xCoord, yCoord, zCoord, "machine.steamburst", 1, (float) (1 + MiscTools.getRand().nextGaussian() * 0.1));
                        }
                    }
                } else {
                    cookTime = 0;
                }
            }
        }
    }

    private boolean drainSteam() {
        LiquidStack steam = tank.drain(STEAM_PER_BATCH, false);
        if (steam != null && steam.amount >= STEAM_PER_BATCH) {
            tank.drain(STEAM_PER_BATCH, true);
            return true;
        }
        return false;
    }

    private boolean hasRecipe() {
        for (int slot = 0; slot < 9; slot++) {
            ItemStack stack = invInput.getStackInSlot(slot);
            if (stack != null && FurnaceRecipes.smelting().getSmeltingResult(stack) != null) {
                return true;
            }
        }
        return false;
    }

    private boolean smeltItems() {
        int count = 0;
        boolean changed = true;
        boolean smelted = false;
        while (count < ITEMS_SMELTED && changed) {
            changed = false;
            for (int slot = 0; slot < 9; slot++) {
                ItemStack stack = invInput.getStackInSlot(slot);
                if (stack != null) {
                    ItemStack output = FurnaceRecipes.smelting().getSmeltingResult(stack);
                    if (output != null && InvTools.isRoomForStack(output, invOutput)) {
                        ItemStack remainder = InvTools.moveItemStack(output.copy(), invOutput);
                        if (remainder == null) {
                            invInput.decrStackSize(slot, 1);
                            changed = true;
                            count++;
                        }
                    }
                }
            }
            smelted |= changed;
        }
        return smelted;
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        switch (side) {
            case UP:
                return SLOT_INPUT;
            default:
                return SLOT_OUTPUT;
        }
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 9;
    }

    @Override
    public void onBlockPlacedBy(EntityLiving player) {
        super.onBlockPlacedBy(player);
        facing = MiscTools.getHorizontalSideClosestToPlayer(worldObj, xCoord, yCoord, zCoord, player);
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        if (side > 1 && Game.isHost(worldObj)) {
            ItemStack current = player.getCurrentEquippedItem();
            if (current != null && current.getItem() instanceof IToolWrench) {
                IToolWrench wrench = (IToolWrench) current.getItem();
                if (wrench.canWrench(player, xCoord, yCoord, zCoord)) {
                    TileSteamOven master = (TileSteamOven) getMasterBlock();
                    if (master != null) {
                        if (master.facing.ordinal() == side) {
                            master.facing = MiscTools.getOppositeSide(side);
                        } else {
                            master.facing = ForgeDirection.getOrientation(side);
                        }
                        wrench.wrenchUsed(player, xCoord, yCoord, zCoord);
                        master.retestMasterBlock();
                        return true;
                    }
                }
            }
        }
        return super.blockActivated(player, side);
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock masterBlock = getMasterBlock();
        if (masterBlock != null) {
            GuiHandler.openGui(EnumGui.STEAN_OVEN, player, worldObj, masterBlock.xCoord, masterBlock.yCoord, masterBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        tankManager.writeTanksToNBT(data);
        data.setInteger("cookTime", cookTime);
        data.setByte("facing", (byte) facing.ordinal());
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        tankManager.readTanksFromNBT(data);
        cookTime = data.getInteger("cookTime");
        facing = ForgeDirection.getOrientation(data.getByte("facing"));
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);
        data.writeByte(facing.ordinal());
        data.writeBoolean(finishedCycle);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);
        byte f = data.readByte();
        finishedCycle = data.readBoolean();
        if (f != facing.ordinal()) {
            facing = ForgeDirection.getOrientation(f);
            markBlockForUpdate();
        }
    }

    /**
     * Fills liquid into internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return fill(0, resource, doFill);
    }

    /**
     * Fills liquid into the specified internal tank.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        if (resource == null) return 0;
        TankManager tMan = getTankManager();
        if (tMan == null) {
            return 0;
        }
        return tMan.fill(tankIndex, resource, doFill);
    }

    /**
     * Drains liquid out of internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is drained to.
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return null;
    }

    /**
     * Drains liquid out of the specified internal tank.
     *
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection dir) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.getTanks();
        }
        return FakeTank.ARRAY;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        TileSteamOven mBlock = (TileSteamOven) getMasterBlock();
        if (mBlock != null) {
            if (stack == null) {
                return 0;
            }
            if (FurnaceRecipes.smelting().getSmeltingResult(stack) != null) {
                IInventory inv = mBlock.invInput;
                if (!doAdd) {
                    inv = new InventoryCopy(inv);
                }
                ItemStack remainder = InvTools.moveItemStack(stack, inv);
                if (remainder == null) {
                    return stack.stackSize;
                }
                return stack.stackSize - remainder.stackSize;
            }
//            System.out.println("no recipe");
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        TileSteamOven mBlock = (TileSteamOven) getMasterBlock();
        if (mBlock != null) {
            IInventory inv = mBlock.invOutput;
            if (!doRemove) {
                inv = new InventoryCopy(inv);
            }
            return InvTools.removeItems(inv, maxItemCount);
        }
        return new ItemStack[0];
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return tMan.getTank(0);
        }
        return FakeTank.INSTANCE;
    }
}
