package railcraft.common.blocks.machine.alpha;

import ic2.api.Direction;
import ic2.api.energy.tile.IEnergySource;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import net.minecraftforge.liquids.LiquidTank;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.beta.ISteamUser;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.plugins.buildcraft.triggers.INeedsMaintenance;
import railcraft.common.plugins.ic2.IC2Plugin;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.StandaloneInventory;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.FilteredTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import universalelectricity.core.electricity.ElectricityConnections;
import universalelectricity.core.electricity.ElectricityNetwork;
import universalelectricity.core.electricity.ElectricityPack;
import universalelectricity.core.implement.IConductor;
import universalelectricity.core.implement.IDisableable;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TileSteamTurbine extends TileMultiBlock implements IEnergySource, ITankContainer, IDisableable, INeedsMaintenance, ISteamUser {

    private static final int DAMAGE_CHANCE = 200;
    private static final int IC2_OUTPUT = 100;
    private static final float UE_WATTS = 100000 / 20;
    private static final float UE_VOLTAGE = 120;
    private static final int BC_OUTPUT = 64;
    private static final int STEAM_USAGE = 320;
    private static final List<MultiBlockPattern> patterns = new ArrayList<MultiBlockPattern>();
    private static final ItemStack sampleRotor = RailcraftPartItems.getTurbineRotor();
    private final StandaloneInventory inv = new StandaloneInventory(1, this);
    public float output;
    private final LiquidStack waterFilter = new LiquidStack(Block.waterStill, 2);
    private final LiquidStack steamFilter = LiquidItems.getSteamLiquid();
    protected final TankManager tankManager = new TankManager();
    public static final int TANK_STEAM = 0;
    public static final int TANK_WATER = 1;
    private boolean needsInit = true;
    private int disabled;
    private byte gaugeState = 0;
    private float energy;
    private IConductor[] conductors = new IConductor[6];
    private final ElectricityPack uePack = new ElectricityPack(UE_WATTS / UE_VOLTAGE, UE_VOLTAGE);

    static {
        char[][][] map1 = {
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            }
        };
        patterns.add(new MultiBlockPattern(map1));


        char[][][] map2 = {
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'W', 'W', 'O'},
                {'O', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O'}
            }
        };
        patterns.add(new MultiBlockPattern(map2));
    }

    public TileSteamTurbine() {
        super(patterns);
        tankManager.addTank(new FilteredTank(LiquidManager.BUCKET_VOLUME * 4, steamFilter)); // Steam

        FilteredTank tank = new FilteredTank(LiquidManager.BUCKET_VOLUME * 4, waterFilter); // Water
        tank.setTankPressure(1);
        tankManager.addTank(tank);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineAlpha.TURBINE;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isHost(getWorld())) {

            if (needsInit) {
                IC2Plugin.addTileToNet(this);
                grabConductors();
                needsInit = false;
            }

            if (isMaster()) {
                boolean addedEnergy = false;
                if (energy < BC_OUTPUT) {
                    LiquidStack steam = tankManager.drain(TANK_STEAM, STEAM_USAGE, false);
//                if(steam != null) System.out.println("steam=" + steam.amount);
                    if (steam != null && steam.amount >= STEAM_USAGE) {
                        ItemStack rotor = inv.getStackInSlot(0);
                        if (InvTools.isItemEqual(rotor, sampleRotor)) {
                            addedEnergy = true;
                            energy += BC_OUTPUT;
                            tankManager.drain(TANK_STEAM, STEAM_USAGE, true);

                            if (worldObj.rand.nextInt(4) != 0) {
                                tankManager.fill(TANK_WATER, waterFilter, true);
                            }

                            if (worldObj.rand.nextInt(DAMAGE_CHANCE) == 0) {
                                inv.setInventorySlotContents(0, InvTools.damageItem(rotor, 1));
                            }
                        }
                    }
                }

                output = (float) ((output * 49D + (addedEnergy ? 100D : 0D)) / 50D);
//                System.out.println("output=" + output);
//                System.out.println("addedEnergy=" + addedEnergy);
            }

            if (getPatternMarker() == 'W') {
                byte prevGaugeState = gaugeState;
                float out = getOutput();
                if (out < 30) {
                    gaugeState = 0;
                } else if (out < 70) {
                    gaugeState = 1;
                } else {
                    gaugeState = 2;
                }
                if (prevGaugeState != gaugeState) {
                    sendUpdateToClient();
                }
            }

            if (IC2Plugin.isIC2Installed() && hasEnergy()) {
                float extra = IC2Plugin.emitEnergy(this, IC2_OUTPUT);
                removeEnergy(((IC2_OUTPUT - extra) / IC2_OUTPUT) * BC_OUTPUT);
            }

            for (int side = 0; side < 6; side++) {
                if (conductors[side] != null) {
                    if (hasEnergy()) {
                        ElectricityNetwork network = conductors[side].getNetwork();
                        ElectricityPack request = network.getRequest();
                        if (request.getWatts() >= uePack.getWatts()) {
                            network.startProducing(this, uePack);
                            removeEnergy(BC_OUTPUT);
                        } else {
                            double ratio = request.getWatts() / uePack.getWatts();
                            network.startProducing(this, new ElectricityPack(UE_WATTS / UE_VOLTAGE * ratio, UE_VOLTAGE));
                            removeEnergy((float) (BC_OUTPUT * ratio));
                        }
                    } else {
                        conductors[side].getNetwork().stopProducing(this);
                    }
                }
            }
        }
    }

    private void onRemove() {
        ElectricityConnections.unregisterConnector(this);
        if (!needsInit) {
            IC2Plugin.removeTileFromNet(this);
            needsInit = true;
        }
    }

    @Override
    public void onChunkUnload() {
        super.onChunkUnload();
        onRemove();
    }

    @Override
    public void validate() {
        super.validate();
        ElectricityConnections.registerConnector(this, EnumSet.allOf(ForgeDirection.class));
    }

    @Override
    public void invalidate() {
        super.invalidate();
        onRemove();
    }

    private void grabConductors() {
        for (int side = 0; side < 6; side++) {
            TileEntity tile = MiscTools.getBlockTileEntityOnSide(worldObj, xCoord, yCoord, zCoord, ForgeDirection.getOrientation(side));
            if (tile instanceof IConductor) {
                conductors[side] = (IConductor) tile;
            } else {
                conductors[side] = null;
            }
        }
    }

    @Override
    public void onNeighborBlockChange(int id) {
        super.onNeighborBlockChange(id);
        grabConductors();
    }

    @Override
    protected void onMasterReset() {
        super.onMasterReset();
        tankManager.getTank(TANK_STEAM).setLiquid(null);
        tankManager.getTank(TANK_WATER).setLiquid(null);
        InvTools.dropInventory(inv, worldObj, xCoord, yCoord, zCoord);
    }

    @Override
    public void onBlockRemoval() {
        super.onBlockRemoval();
        InvTools.dropInventory(inv, worldObj, xCoord, yCoord, zCoord);
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock mBlock = getMasterBlock();
        if (mBlock != null) {
            GuiHandler.openGui(EnumGui.TURBINE, player, worldObj, mBlock.xCoord, mBlock.yCoord, mBlock.zCoord);
            return true;
        }
        return false;
    }

    public boolean hasEnergy() {
        return getEnergy() >= BC_OUTPUT;
    }

    public void removeEnergy(float amount) {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock != null) {
            mBlock.energy -= amount;
            if (mBlock.energy < 0) mBlock.energy = 0;
        }
    }

    public float getEnergy() {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock == null) {
            return 0;
        }
        return mBlock.energy;
    }

    public float getOutput() {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock == null) {
            return 0;
        }
        return mBlock.output;
    }

    @Override
    public int getBlockTexture(int side) {
        if (isStructureValid()) {
            int patIndex = patterns.indexOf(getPattern());
            if (side == 0 || side == 1) {
                if (patIndex == 0)
                    return EnumMachineAlpha.TURBINE.getTexture(10);
                return EnumMachineAlpha.TURBINE.getTexture(0);
            }

            if (getPatternMarker() == 'W') {
                return EnumMachineAlpha.TURBINE.getTexture(11 + gaugeState);
            }
            if (patIndex == 0) {
                switch (side) {
                    case 2:
                        if (getPatternPositionY() == 2) {
                            if (getPatternPositionX() == 2) {
                                return EnumMachineAlpha.TURBINE.getTexture(6);
                            }
                            return EnumMachineAlpha.TURBINE.getTexture(7);
                        }
                        if (getPatternPositionX() == 2) {
                            return EnumMachineAlpha.TURBINE.getTexture(8);
                        }
                        return EnumMachineAlpha.TURBINE.getTexture(9);
                    case 3:
                        if (getPatternPositionY() == 2) {
                            if (getPatternPositionX() == 1) {
                                return EnumMachineAlpha.TURBINE.getTexture(6);
                            }
                            return EnumMachineAlpha.TURBINE.getTexture(7);
                        }
                        if (getPatternPositionX() == 1) {
                            return EnumMachineAlpha.TURBINE.getTexture(8);
                        }
                        return EnumMachineAlpha.TURBINE.getTexture(9);
                }
            } else {
                switch (side) {
                    case 4:
                        if (getPatternPositionY() == 2) {
                            if (getPatternPositionZ() == 1) {
                                return EnumMachineAlpha.TURBINE.getTexture(6);
                            }
                            return EnumMachineAlpha.TURBINE.getTexture(7);
                        }
                        if (getPatternPositionZ() == 1) {
                            return EnumMachineAlpha.TURBINE.getTexture(8);
                        }
                        return EnumMachineAlpha.TURBINE.getTexture(9);
                    case 5:
                        if (getPatternPositionY() == 2) {
                            if (getPatternPositionZ() == 2) {
                                return EnumMachineAlpha.TURBINE.getTexture(6);
                            }
                            return EnumMachineAlpha.TURBINE.getTexture(7);
                        }
                        if (getPatternPositionZ() == 2) {
                            return EnumMachineAlpha.TURBINE.getTexture(8);
                        }
                        return EnumMachineAlpha.TURBINE.getTexture(9);
                }
            }

        }
        return EnumMachineAlpha.TURBINE.getTexture(0);
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        inv.writeToNBT("rotor", data);
        tankManager.writeTanksToNBT(data);
        data.setFloat("energy", energy);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        inv.readFromNBT("rotor", data);
        tankManager.readTanksFromNBT(data);
        energy = data.getFloat("energy");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);
        data.writeByte(gaugeState);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);
        byte gs = data.readByte();
        if (gs != gaugeState) {
            gaugeState = gs;
            markBlockForUpdate();
        }
    }

    @Override
    public int getMaxEnergyOutput() {
        return 128;
    }

    @Override
    public boolean emitsEnergyTo(TileEntity receiver, Direction direction) {
        return true;
    }

    @Override
    public boolean isAddedToEnergyNet() {
        return !needsInit;
    }

    public StandaloneInventory getInventory() {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock != null) {
            return mBlock.inv;
        }
        return inv;
    }

    public TankManager getTankManager() {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager;
        }
        return null;
    }

    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return drain(TANK_WATER, maxDrain, doDrain);
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        TankManager tMan = getTankManager();
        if (tMan != null) return tMan.drain(tankIndex, maxDrain, doDrain);
        return null;
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return fill(TANK_STEAM, resource, doFill);
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        TankManager tMan = getTankManager();
        if (tMan != null) return tMan.fill(tankIndex, resource, doFill);
        return 0;
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection side) {
        TankManager tMan = getTankManager();
        if (tMan != null) return tMan.getTanks();
        return new ILiquidTank[]{new LiquidTank(0)};
    }

    @Override
    public ILiquidTank getTank(ForgeDirection side, LiquidStack type) {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        TankManager tMan = tankManager;
        if (mBlock != null) {
            tMan = mBlock.tankManager;
        }
        switch (side) {
            case DOWN:
                return tMan.getTank(TANK_WATER);
            default:
                return tMan.getTank(TANK_STEAM);
        }
    }

    @Override
    public void onDisable(int duration) {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock != null) {
            mBlock.disabled = duration;
        }
    }

    @Override
    public boolean isDisabled() {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock != null) {
            return mBlock.disabled <= 0;
        }
        return true;
    }

    @Override
    public boolean needsMaintenance() {
        TileSteamTurbine mBlock = (TileSteamTurbine) getMasterBlock();
        if (mBlock != null) {
            ItemStack rotor = mBlock.inv.getStackInSlot(0);
            if (rotor == null) {
                return true;
            }
            if (!InvTools.isItemEqual(rotor, sampleRotor)) {
                return true;
            }
            if (rotor.getItemDamage() / (double) rotor.getMaxDamage() > 0.75f) {
                return true;
            }
        }
        return false;
    }
}
