package railcraft.common.blocks.machine.alpha;

import buildcraft.api.inventory.ISpecialInventory;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.util.MathHelper;
import net.minecraft.inventory.Slot;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.blocks.machine.TileTank;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.gui.slots.SlotWaterOrEmpty;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.liquids.LiquidFilter;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.FilteredTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TileTankWater extends TileTank implements ISpecialInventory {

    private final static int OUTPUT_RATE = 40;
    private final static int TANK_CAPACITY = LiquidManager.BUCKET_VOLUME * 400;
    private final static int REFILL_INTERVAL = 8;
    private final static float REFILL_RATE = 10f;
    private final static float REFILL_PENALTY_INSIDE = 0.5f;
    private final static float REFILL_PENALTY_SNOW = 0.5f;
    private final static float REFILL_BOOST_RAIN = 3.0f;
    private final static byte REFILL_RATE_MIN = 1;
    private final static int SLOT_INPUT = 0;
    private final static int SLOT_OUTPUT = 1;
    private final static List<MultiBlockPattern> patterns = new ArrayList<MultiBlockPattern>();
    private IInventory invInput = new InventoryMapper(this, SLOT_INPUT, 1);
    private IInventory invOutput = new InventoryMapper(this, SLOT_OUTPUT, 1);
    private boolean testOutputs = true;
    private ITankContainer[] outputs = new ITankContainer[6];

    static {
        char[][][] map = {
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'A', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },
            {
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O'}
            },};
        patterns.add(new MultiBlockPattern(map));
    }

    public TileTankWater() {
        super("gui.tank.water", 2, patterns);
        FilteredTank tank = new FilteredTank(TANK_CAPACITY, new LiquidStack(Block.waterStill, 0));
        tank.setTankPressure(1);
        tankManager.addTank(tank);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineAlpha.TANK_WATER;
    }

    @Override
    public Slot getInputSlot(IInventory inv, int id, int x, int y) {
        return new SlotWaterOrEmpty(inv, id, x, y);
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        if (Game.isHost(worldObj)) {
            if (isStructureValid() && LiquidManager.getInstance().handleRightClick(getTankManager(), 0, player, true, true)) {
                return true;
            }
        } else {
            if (LiquidManager.getInstance().isContainer(player.inventory.getCurrentItem())) {
                return true;
            }
        }
        return super.blockActivated(player, side);
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isHost(getWorld())) {
            if (isMaster()) {
                if (worldObj.provider.dimensionId != -1 && update % REFILL_INTERVAL == 0) {
                    float rate = REFILL_RATE;
                    BiomeGenBase biome = worldObj.getBiomeGenForCoords(xCoord, zCoord);
                    float humidity = biome.rainfall;
                    rate *= humidity;
//                    String debug = "Biome=" + biome.biomeName + ", Humidity=" + humidity;

                    boolean outside = false;
                    for (int x = xCoord; x < xCoord + 3; x++) {
                        for (int z = zCoord; z < zCoord + 3; z++) {
                            outside = worldObj.canBlockSeeTheSky(x, yCoord + 3, z);
//                            System.out.println(x + ", " + (yCoord + 3) + ", " + z);
                            if (outside) {
                                break;
                            }
                        }
                    }

//                    debug += ", Outside=" + outside;

                    if (!outside) {
                        rate *= REFILL_PENALTY_INSIDE;
                    } else if (worldObj.isRaining()) {
                        if (biome.getEnableSnow()) {
                            rate *= REFILL_PENALTY_SNOW;
//                            debug += ", Snow=true";
                        } else {
                            rate *= REFILL_BOOST_RAIN;
//                            debug += ", Rain=true";
                        }
                    }
                    int rateFinal = MathHelper.floor_float(rate);
                    if (rateFinal < REFILL_RATE_MIN) {
                        rateFinal = REFILL_RATE_MIN;
                    }
//                    debug += ", Refill=" + rateFinal;
//                    System.out.println(debug);

                    LiquidStack fillStack = new LiquidStack(Block.waterStill, rateFinal);
                    fill(0, fillStack, true);
                }

                if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
                    LiquidManager.getInstance().processContainers(tankManager.getTank(0), this, SLOT_INPUT, SLOT_OUTPUT);
                }
            }

            TankManager tMan = getTankManager();
            if (tMan != null) {
                findOutputs();
                tMan.outputLiquid(outputs, 0, OUTPUT_RATE);
            }
        }
    }

    @Override
    public ITankContainer getOutputOnSide(ForgeDirection side) {
        if (side == ForgeDirection.UP) {
            return null;
        }

        return super.getOutputOnSide(side);
    }

    private void findOutputs() {
        if (testOutputs) {
            testOutputs = false;
            for (int side = 0; side < 6; side++) {
                ForgeDirection dir = ForgeDirection.getOrientation(side);
                if (!MiscTools.blockExistsOnSide(worldObj, xCoord, yCoord, zCoord, dir)) {
                    testOutputs = true;
                    outputs[side] = null;
                    continue;
                }
                outputs[side] = getOutputOnSide(dir);
            }
        }
    }

    @Override
    public void onNeighborBlockChange(int id) {
        super.onNeighborBlockChange(id);
        testOutputs = true;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock mBlock = getMasterBlock();
        if (mBlock != null) {
            GuiHandler.openGui(EnumGui.TANK, player, worldObj, mBlock.xCoord, mBlock.yCoord, mBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    public int getBlockTexture(int side) {
        return EnumMachineAlpha.TANK_WATER.getTexture(side);
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        if (from != ForgeDirection.UP) {
            return 0;
        }
        return fill(0, resource, doFill);
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        if (tankIndex != 0 || resource == null || resource.itemID != Block.waterStill.blockID) {
            return 0;
        }
        return super.fill(tankIndex, resource, doFill);
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (stack == null) {
            return 0;
        }
        
        LiquidManager lm = LiquidManager.getInstance();

        if (lm.isContainer(stack) && (lm.isEmptyContainer(stack) || lm.containsLiquid(stack, LiquidFilter.WATER))) {
            IInventory inv = invInput;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remainder = InvTools.moveItemStack(stack, inv);
            if (remainder == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remainder.stackSize;
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory inv = invOutput;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return InvTools.removeItems(inv, maxItemCount);
    }
}
