package railcraft.common.blocks.machine.beta;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.TileMachineBase;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.modules.ModuleManager;
import railcraft.common.modules.ModuleManager.Module;

/**
 *
 * @author CovertJaguar
 */
public enum EnumMachineBeta implements IEnumMachine {

    TANK_IRON_WALL(Module.TRANSPORT, "tank.iron.wall", TileTankIronWall.class, new int[]{19, 19, 20, 20, 20, 20}),
    TANK_IRON_GAUGE(Module.TRANSPORT, "tank.iron.gauge", TileTankIronGauge.class, new int[]{36, 36, 36, 36, 36, 36, 52, 53, 37, 21}),
    TANK_IRON_VALVE(Module.TRANSPORT, "tank.iron.valve", TileTankIronValve.class, new int[]{35, 35, 51, 51, 51, 51, 19, 20}),
    BOILER_TANK_LOW_PRESSURE(Module.ENERGY, "boiler.tank.pressure.low", TileBoilerTankLow.class, new int[]{97, 97, 98, 98, 98, 98}),
    BOILER_TANK_HIGH_PRESSURE(Module.ENERGY, "boiler.tank.pressure.high", TileBoilerTankHigh.class, new int[]{113, 113, 114, 114, 114, 114}),
    BOILER_FIREBOX_SOLID(Module.ENERGY, "boiler.firebox.solid", TileBoilerFireboxSolid.class, new int[]{99, 99, 100, 100, 100, 100}),
    BOILER_FIREBOX_LIQUID(Module.ENERGY, "boiler.firebox.liquid", TileBoilerFireboxLiquid.class, new int[]{115, 115, 116, 116, 116, 116}),
    ENGINE_STEAM_HOBBY(Module.ENERGY, "engine.steam.hobby", TileEngineSteamHobby.class, new int[]{83}),
    ENGINE_STEAM_LOW(Module.ENERGY, "engine.steam.low", TileEngineSteamLow.class, new int[]{84}),
    ENGINE_STEAM_HIGH(Module.ENERGY, "engine.steam.high", TileEngineSteamHigh.class, new int[]{85}),
    SENTINEL(Module.CHUNK_LOADING, "anchor.sentinel", TileSentinel.class, new int[]{200, 200, 184, 184, 184, 184}),;
    private final Module module;
    private final String tag;
    private final Class<? extends TileMachineBase> tile;
    private final int[] texture;
    private static final List<EnumMachineBeta> creativeList = new ArrayList<EnumMachineBeta>();
    private static final EnumMachineBeta[] VALUES = values();

    static {
        creativeList.add(TANK_IRON_WALL);
        creativeList.add(TANK_IRON_GAUGE);
        creativeList.add(TANK_IRON_VALVE);
        creativeList.add(BOILER_FIREBOX_SOLID);
        creativeList.add(BOILER_FIREBOX_LIQUID);
        creativeList.add(BOILER_TANK_LOW_PRESSURE);
        creativeList.add(BOILER_TANK_HIGH_PRESSURE);
        creativeList.add(ENGINE_STEAM_HOBBY);
        creativeList.add(ENGINE_STEAM_LOW);
        creativeList.add(ENGINE_STEAM_HIGH);
        creativeList.add(SENTINEL);
    }

    private EnumMachineBeta(Module module, String tag, Class<? extends TileMachineBase> tile, int[] texture) {
        this.module = module;
        this.tile = tile;
        this.tag = tag;
        this.texture = texture;
    }

    @Override
    public boolean isDepreciated() {
        return module == null;
    }

    public int[] getTexture() {
        return texture;
    }

    @Override
    public int getTexture(int index) {
        if (index < 0 || index >= texture.length) {
            index = 0;
        }
        return texture[index];
    }

    public static EnumMachineBeta fromId(int id) {
        if (id < 0 || id >= VALUES.length) {
            id = 0;
        }
        return VALUES[id];
    }

    public static List<EnumMachineBeta> getCreativeList() {
        return creativeList;
    }

    @Override
    public String getTag() {
        return "rc.machine.beta." + tag;
    }

    @Override
    public Class getTileClass() {
        return tile;
    }

    public TileMachineBase getTileEntity() {
        try {
            return tile.newInstance();
        } catch (Exception ex) {
        }
        return null;
    }

    @Override
    public ItemStack getItem() {
        return getItem(1);
    }

    @Override
    public ItemStack getItem(int qty) {
        Block block = getBlock();
        if (block == null) {
            return null;
        }
        return new ItemStack(block, qty, ordinal());
    }

    public Module getModule() {
        return module;
    }

    @Override
    public Block getBlock() {
        return RailcraftBlocks.getBlockMachineBeta();
    }

    @Override
    public int getBlockId() {
        Block block = getBlock();
        if (block != null) {
            return block.blockID;
        }
        return 0;
    }

    @Override
    public boolean isEnabled() {
        return ModuleManager.isModuleLoaded(getModule()) && getBlock() != null && RailcraftConfig.isSubBlockEnabled(getTag());
    }

    public void addItemInfo(ItemStack stack, EntityPlayer player, List<String> info, boolean adv) {
        switch (this) {
            default:
                String tipTag = "gui.tip." + tag;
                if (RailcraftLanguage.hasTag(tipTag)) {
                    info.add(RailcraftLanguage.translate(tipTag));
                }
                break;
        }
    }
}
