package railcraft.common.blocks.machine.beta;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.liquids.LiquidFilter;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.FakeTank;
import railcraft.common.liquids.tanks.FilteredTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.steam.Steam;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileBoiler extends TileMultiBlock implements ITankContainer {

    private static final Set<Integer> boilerBlocks = new HashSet<Integer>();
    private static final Set<Integer> fireboxBlocks = new HashSet<Integer>();
    private static final List<MultiBlockPattern> patterns = new ArrayList<MultiBlockPattern>();
    public static final int TANK_WATER = 0;
    public static final int TANK_STEAM = 1;
    public final static int TRANSFER_RATE = 80;
    public final static int TICKS_LOW = 16;
    public final static int TICKS_HIGH = 8;
    public final static int STEAM_LOW = 16;
    public final static int STEAM_HIGH = 32;
    public final static float HEAT_LOW = Steam.MAX_HEAT;
    public final static float HEAT_HIGH = Steam.MAX_HEAT * 2;
    protected final TankManager tankManager = new TankManager();
    private boolean testOutputs = true;
    private ITankContainer[] outputs = new ITankContainer[6];

    static {
        fireboxBlocks.add(EnumMachineBeta.BOILER_FIREBOX_SOLID.ordinal());
        fireboxBlocks.add(EnumMachineBeta.BOILER_FIREBOX_LIQUID.ordinal());

        boilerBlocks.addAll(fireboxBlocks);
        boilerBlocks.add(EnumMachineBeta.BOILER_TANK_LOW_PRESSURE.ordinal());
        boilerBlocks.add(EnumMachineBeta.BOILER_TANK_HIGH_PRESSURE.ordinal());

        patterns.add(buildMap(3, 4, 'H', TICKS_HIGH, HEAT_HIGH, STEAM_HIGH));
        patterns.add(buildMap(3, 3, 'H', TICKS_HIGH, HEAT_HIGH, STEAM_HIGH));
        patterns.add(buildMap(3, 2, 'H', TICKS_HIGH, HEAT_HIGH, STEAM_HIGH));

        patterns.add(buildMap(2, 3, 'H', TICKS_HIGH, HEAT_HIGH, STEAM_HIGH));
        patterns.add(buildMap(2, 2, 'H', TICKS_HIGH, HEAT_HIGH, STEAM_HIGH));

        patterns.add(buildMap(1, 1, 'H', TICKS_HIGH, HEAT_HIGH, STEAM_HIGH));

        patterns.add(buildMap(3, 4, 'L', TICKS_LOW, HEAT_LOW, STEAM_LOW));
        patterns.add(buildMap(3, 3, 'L', TICKS_LOW, HEAT_LOW, STEAM_LOW));
        patterns.add(buildMap(3, 2, 'L', TICKS_LOW, HEAT_LOW, STEAM_LOW));

        patterns.add(buildMap(2, 3, 'L', TICKS_LOW, HEAT_LOW, STEAM_LOW));
        patterns.add(buildMap(2, 2, 'L', TICKS_LOW, HEAT_LOW, STEAM_LOW));

        patterns.add(buildMap(1, 1, 'L', TICKS_LOW, HEAT_LOW, STEAM_LOW));
    }

    private static MultiBlockPattern buildMap(int width, int tankHeight, char tank, int ticks, float heat, int capacity) {
        char[][][] map = new char[tankHeight + 3][width + 2][width + 2];

        for (int x = 0; x < width + 2; x++) {
            for (int z = 0; z < width + 2; z++) {
                map[0][x][z] = 'O';
            }
        }

        for (int x = 0; x < width + 2; x++) {
            for (int z = 0; z < width + 2; z++) {
                char m = x == 0 || z == 0 || x == width + 1 || z == width + 1 ? 'O' : 'F';
                map[1][x][z] = m;
            }
        }

        for (int y = 2; y < tankHeight + 2; y++) {
            for (int x = 0; x < width + 2; x++) {
                for (int z = 0; z < width + 2; z++) {
                    char m = x == 0 || z == 0 || x == width + 1 || z == width + 1 ? 'O' : tank;
                    map[y][x][z] = m;
                }
            }
        }

        for (int x = 0; x < width + 2; x++) {
            for (int z = 0; z < width + 2; z++) {
                map[tankHeight + 2][x][z] = 'O';
            }
        }

        return new BoilerPattern(map, width * width * tankHeight, ticks, heat, capacity);
    }

    public static class BoilerPattern extends MultiBlockPattern {

        public final int numTanks;
        public final int ticksPerConversion;
        public final float maxHeat;
        public final int steamCapacity;

        public BoilerPattern(char[][][] pattern, int tanks, int ticks, float heat, int capacity) {
            super(pattern, 1, 1, 1);
            numTanks = tanks;
            ticksPerConversion = ticks;
            this.maxHeat = heat;
            this.steamCapacity = capacity;
        }
    }

    protected TileBoiler() {
        super(patterns);

        FilteredTank waterTank = new FilteredTank(4 * LiquidManager.BUCKET_VOLUME, LiquidFilter.WATER);
        tankManager.addTank(waterTank);

        FilteredTank steamTank = new FilteredTank(16 * LiquidManager.BUCKET_VOLUME, LiquidFilter.STEAM);
        steamTank.setTankPressure(1);
        tankManager.addTank(steamTank);
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null && current.itemID != Item.bucketEmpty.itemID) {
            if (Game.isHost(worldObj)) {
                TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
                if (mBlock != null) {
                    if (mBlock.handleClick(player)) {
                        return true;
                    }
                }
            } else {
                if (LiquidManager.getInstance().isContainer(current)) {
                    return true;
                }
            }
        }
        return super.blockActivated(player, side);
    }

    public void explode() {
        if (Game.isHost(worldObj)) {
            worldObj.createExplosion(null, xCoord, yCoord, zCoord, 5f + 0.1f * getNumTanks(), true);
        }
    }

    public int getNumTanks() {
        MultiBlockPattern pattern = getPattern();
        return ((BoilerPattern) pattern).numTanks;
    }

    public float getMaxHeat() {
        MultiBlockPattern pattern = getPattern();
        return ((BoilerPattern) pattern).maxHeat;
    }

    public int getTicksPerConversion() {
        MultiBlockPattern pattern = getPattern();
        return ((BoilerPattern) pattern).ticksPerConversion;
    }

    public int getSteamCapacityPerTank() {
        MultiBlockPattern pattern = getPattern();
        return ((BoilerPattern) pattern).steamCapacity;
    }

    public TankManager getTankManager() {
        TileBoiler mBlock = (TileBoiler) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager;
        }
        return null;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();
        if (Game.isHost(worldObj)) {
            TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
            if (mBlock != null) {
                ILiquidTank tank = mBlock.tankManager.getTank(TANK_STEAM);
                LiquidStack steam = tank.getLiquid();
                if (steam != null && (mBlock.heat < Steam.BOILING_POINT || steam.amount >= tank.getCapacity() / 2)) {
                    findOutputs();
                    mBlock.tankManager.outputLiquid(outputs, TANK_STEAM, TRANSFER_RATE);
                }
            }
        }
    }

    private void findOutputs() {
        if (testOutputs) {
            testOutputs = false;
            for (int side = 0; side < 6; side++) {
                ForgeDirection dir = ForgeDirection.getOrientation(side);
                if (!MiscTools.blockExistsOnSide(worldObj, xCoord, yCoord, zCoord, dir)) {
                    testOutputs = true;
                }
                outputs[side] = getOutputOnSide(dir);
            }
        }
    }

    public abstract ITankContainer getOutputOnSide(ForgeDirection side);

    @Override
    public void onNeighborBlockChange(int id) {
        super.onNeighborBlockChange(id);
        testOutputs = true;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock mBlock = getMasterBlock();
        if (mBlock != null) {
            return mBlock.openGui(player);
        }
        return false;
    }

    @Override
    protected int getMaxRecursionDepth() {
        return 20;
    }

    @Override
    protected boolean isMapPositionValid(int i, int j, int k, char mapPos) {
        int id = worldObj.getBlockId(i, j, k);
        int meta = worldObj.getBlockMetadata(i, j, k);

        switch (mapPos) {
            case 'O': // Other
                if (id == getBlockId() && boilerBlocks.contains(meta)) {
                    return false;
                }
                break;
            case 'L': // Tank
                if (id != getBlockId() || meta != EnumMachineBeta.BOILER_TANK_LOW_PRESSURE.ordinal()) {
                    return false;
                }
                break;
            case 'H': // Tank
                if (id != getBlockId() || meta != EnumMachineBeta.BOILER_TANK_HIGH_PRESSURE.ordinal()) {
                    return false;
                }
                break;
            case 'F': // Firebox
                if (id != getBlockId() || meta != getBlockMetadata() || !fireboxBlocks.contains(meta)) {
                    return false;
                }
                break;
            case 'A': // Air
                if (!worldObj.isAirBlock(i, j, k)) {
                    return false;
                }
                break;
        }
        return true;
    }

    @Override
    protected boolean isStructureTile(TileEntity tile) {
        return tile instanceof TileBoiler;
    }

    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return drain(TANK_STEAM, maxDrain, doDrain);
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock == null) {
            return null;
        }
        return mBlock.tankManager.drain(tankIndex, maxDrain, doDrain);
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return 0;
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        if (tankIndex == TANK_STEAM) {
            return 0;
        }
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock == null) {
            return 0;
        }
        if (doFill && tankIndex == TANK_WATER && LiquidFilter.WATER.isLiquidEqual(resource)) {
            if (mBlock.heat >= Steam.SUPER_HEATED) {
                LiquidStack water = mBlock.tankManager.getTank(TANK_WATER).getLiquid();
                if (water == null || water.amount <= 0) {
                    explode();
                    return 0;
                }
            }
        }
        return mBlock.tankManager.fill(tankIndex, resource, doFill);
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection dir) {
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager.getTanks();
        }
        return FakeTank.ARRAY;
    }
}
