package railcraft.common.blocks.machine.beta;

import buildcraft.api.inventory.ISpecialInventory;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.alpha.TileSteamTurbine;
import railcraft.common.gui.indicator.IIndicatorController;
import railcraft.common.gui.util.ToolTip;
import railcraft.common.plugins.buildcraft.triggers.ITemperature;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.inventory.StandaloneInventory;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.TankWrapper;
import railcraft.common.liquids.tanks.FakeTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.steam.Steam;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileBoilerFirebox extends TileBoiler implements IInventory, ISpecialInventory, ITemperature {

    public static final int SLOT_LIQUID_INPUT = 0;
    public static final int SLOT_LIQUID_OUTPUT = 1;
    public float heat = Steam.COLD_TEMP;
    private byte burnCycle;
    public float burnTime;
    private boolean burning = false;
    public float currentItemBurnTime;
    private boolean wasLit;
    private byte prevPattern;
    protected final StandaloneInventory inventory;
    protected IInventory invWaterInput = new InventoryMapper(this, SLOT_LIQUID_INPUT, 1);
    protected IInventory invWaterOutput = new InventoryMapper(this, SLOT_LIQUID_OUTPUT, 1);
    private IIndicatorController heatIndicator = new HeatIndicator();

    private class HeatIndicator implements IIndicatorController {

        List<ToolTip> tips = new ArrayList<ToolTip>();
        ToolTip tip = new ToolTip();

        private HeatIndicator() {
            tips.add(tip);
        }

        @Override
        public List<ToolTip> getToolTip() {
            tip.text = String.format("%.0f\u00B0C", heat);
            return tips;
        }

        @Override
        public int getScaledLevel(int size) {
            return (int) ((heat - Steam.COLD_TEMP) * size / (getMaxHeat() - Steam.COLD_TEMP));
        }
    };

    public IIndicatorController getHeatIndicator() {
        return heatIndicator;
    }

    protected TileBoilerFirebox(int invSize) {
        super();
        inventory = new StandaloneInventory(invSize, (IInventory) this);
    }

    @Override
    public int getBlockTexture(int side) {
        int texture = getMachineType().getTexture(side);
        if (side != 0 && side != 1) {
            if (isBurning()) {
                texture++;
            }
        }
        return texture;
    }

    @Override
    protected void onPatternLock(MultiBlockPattern pattern) {
        int capacity = getNumTanks() * LiquidManager.BUCKET_VOLUME;
        tankManager.setCapacity(TANK_STEAM, capacity * getSteamCapacityPerTank());
        tankManager.setCapacity(TANK_WATER, capacity * 4);

        byte newPattern = getPatternIndex();
        if (newPattern != prevPattern) {
            heat = 0;
        }
        prevPattern = newPattern;
    }

    public int getBurnProgressScaled(int i) {
        if (!isBurning()) {
            return 0;
        }
        int scale = (int) (((float) burnTime / (float) currentItemBurnTime) * i);
        scale = Math.max(0, scale);
        scale = Math.min(i, scale);
        return scale;
    }

    public boolean isBurning() {
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock == null) {
            return false;
        }
        return mBlock.burning;
    }

    protected void updateLighting() {
        boolean b = isBurning();
        if (wasLit != b) {
            wasLit = b;
            worldObj.updateLightByType(EnumSkyBlock.Block, xCoord, yCoord, zCoord);
            markBlockForUpdate();
        }
    }

    @Override
    public int getLightValue() {
        if (isStructureValid() && isBurning()) {
            return 13;
        }
        return 0;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void randomDisplayTick(Random random) {
        if (isStructureValid()) {
            updateLighting();
            if (isBurning() && random.nextInt(100) < 20) {
                float f = (float) xCoord + 0.5F;
                float f1 = (float) yCoord + 0.4375F + (random.nextFloat() * 3F / 16F);
                float f2 = (float) zCoord + 0.5F;
                float f3 = 0.52F;
                float f4 = random.nextFloat() * 0.6F - 0.3F;
                worldObj.spawnParticle("flame", f - f3, f1, f2 + f4, 0.0D, 0.0D, 0.0D);
                worldObj.spawnParticle("flame", f + f3, f1, f2 + f4, 0.0D, 0.0D, 0.0D);
                worldObj.spawnParticle("flame", f + f4, f1, f2 - f3, 0.0D, 0.0D, 0.0D);
                worldObj.spawnParticle("flame", f + f4, f1, f2 + f3, 0.0D, 0.0D, 0.0D);
            }
        }
    }

    @Override
    public ITankContainer getOutputOnSide(ForgeDirection side) {
        TileEntity tile = MiscTools.getBlockTileEntityOnSide(worldObj, xCoord, yCoord, zCoord, side);
        if (tile instanceof ISteamUser) {
            return (ITankContainer) tile;
        }
        return null;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isNotHost(getWorld())) {
            return;
        }

        if (isMaster) {
            process();

            int numTanks = getNumTanks();

            burnCycle++;
            int ticks = getTicksPerConversion();
            if (burnCycle >= ticks) {
                burnCycle = 0;
                float fuelNeeded = getFuelPerConversion(numTanks) * numTanks;
                while (burnTime < fuelNeeded) {
                    boolean addedFuel = addFuel();
                    if (!addedFuel) {
                        break;
                    }
                }
                boolean wasBurning = burning;
                if (burnTime < fuelNeeded) {
                    burning = false;
                } else {
                    burnTime -= fuelNeeded;
                    burning = true;
                }
                if (burning != wasBurning) {
                    sendUpdateToClient();
                }
                if (heat >= Steam.BOILING_POINT) {
                    int waterCost = numTanks;
                    LiquidStack water = tankManager.drain(TANK_WATER, waterCost, false);
                    if (water != null) {
                        waterCost = Math.min(waterCost, water.amount);
                        tankManager.drain(TANK_WATER, waterCost, true);
                        LiquidStack steam = LiquidItems.getSteamLiquid(Steam.STEAM_PER_UNIT_WATER * waterCost);
                        tankManager.fill(TANK_STEAM, steam, true);
                    }
                }
            }

            if (isBurning()) {
                increaseHeat(numTanks);
            } else {
                reduceHeat(numTanks);
            }

            if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
                processBuckets();
            }
        }
    }

    private boolean addFuel() {
        float fuel = getMoreFuel();
        burnTime += fuel;
        currentItemBurnTime = burnTime;
        return fuel > 0;
    }

    protected abstract void process();

    protected abstract float getMoreFuel();

    protected void processBuckets() {
        LiquidManager.getInstance().processContainers(this, TANK_WATER, inventory, SLOT_LIQUID_INPUT, SLOT_LIQUID_OUTPUT, true, false);
    }

    public float getFuelPerConversion(int numTanks) {
        float fuel = Steam.FUEL_PER_STEAM_CONVERSION - numTanks * 0.08f;
        fuel = fuel + fuel * (Steam.INEFFICENCY_MULTIPLIER - Steam.INEFFICENCY_MULTIPLIER * getHeatLevel());
        return fuel;
    }

    public float getHeatLevel() {
        return heat / getMaxHeat();
    }

    private void increaseHeat(int numTanks) {
        float heatLevel = getHeatLevel();
        float change = Steam.HEAT_STEP;
        if (heatLevel < 0.25f) {
            change += Steam.HEAT_STEP;
        }
        if (heatLevel < 0.5f) {
            change += Steam.HEAT_STEP;
        }
        if (heatLevel < 0.75f) {
            change += Steam.HEAT_STEP;
        }
        change /= numTanks;
        heat += change;
        heat = Math.min(heat, getMaxHeat());
    }

    private void reduceHeat(int numTanks) {
        float heatLevel = getHeatLevel();
        float change = Steam.HEAT_STEP;
        if (heatLevel > 0.25f) {
            change += Steam.HEAT_STEP;
        }
        if (heatLevel > 0.5f) {
            change += Steam.HEAT_STEP;
        }
        if (heatLevel > 0.75f) {
            change += Steam.HEAT_STEP;
        }
        change /= numTanks;
        heat -= change;
        heat = Math.max(heat, Steam.COLD_TEMP);
    }

    @Override
    protected void onMasterReset() {
        super.onMasterReset();
        InvTools.dropInventory(inventory, worldObj, xCoord, yCoord, zCoord);
        tankManager.getTank(TANK_STEAM).setLiquid(null);
//        heat = Steam.COLD_TEMP;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        tankManager.writeTanksToNBT(data);
        inventory.writeToNBT("inv", data);

        data.setFloat("heat", heat);
        data.setFloat("burnTime", burnTime);
        data.setFloat("currentItemBurnTime", currentItemBurnTime);

        data.setByte("prevPattern", prevPattern);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        tankManager.readTanksFromNBT(data);
        inventory.readFromNBT("inv", data);

        heat = data.getFloat("heat");
        burnTime = data.getFloat("burnTime");
        currentItemBurnTime = data.getFloat("currentItemBurnTime");

        prevPattern = data.getByte("prevPattern");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);
        tankManager.writePacketData(data);
        data.writeBoolean(burning);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);
        tankManager.readPacketData(data);
        burning = data.readBoolean();
    }

    @Override
    public ItemStack decrStackSize(int i, int j) {
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock != null) {
            return mBlock.inventory.decrStackSize(i, j);
        }
        return null;
    }

    @Override
    public ItemStack getStackInSlot(int i) {
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock != null) {
            return mBlock.inventory.getStackInSlot(i);
        }
        return null;
    }

    @Override
    public void setInventorySlotContents(int i, ItemStack itemstack) {
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock != null) {
            mBlock.inventory.setInventorySlotContents(i, itemstack);
        }
    }

    @Override
    public void openChest() {
    }

    @Override
    public void closeChest() {
    }

    @Override
    public int getInventoryStackLimit() {
        return 64;
    }

    @Override
    public int getSizeInventory() {
        return inventory.getSizeInventory();
    }

    @Override
    public ItemStack getStackInSlotOnClosing(int var1) {
        return null;
    }

    protected boolean handleClick(EntityPlayer player) {
        if (LiquidManager.getInstance().handleRightClick(this, TANK_WATER, player, true, false)) {
            return true;
        }
        return false;
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return fill(TANK_WATER, resource, doFill);
    }

    @Override
    public float getTemperature() {
        TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
        if (mBlock != null) {
            return mBlock.heat;
        }
        return Steam.COLD_TEMP;
    }

    @Override
    public ILiquidTank getTank(ForgeDirection side, LiquidStack type) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            return new WaterTankWrapper(tMan.getTank(TANK_WATER));
        }
        return FakeTank.INSTANCE;
    }

    protected class WaterTankWrapper extends TankWrapper {

        public WaterTankWrapper(ILiquidTank tank) {
            super(tank);
        }

        @Override
        public int fill(LiquidStack resource, boolean doFill) {
            if (resource == null) return 0;

            TileBoilerFirebox mBlock = (TileBoilerFirebox) getMasterBlock();
            if (mBlock != null && mBlock.heat >= Steam.SUPER_HEATED) {
                LiquidStack water = tank.getLiquid();
                if (water == null || water.amount <= 0) {
                    explode();
                }
            }

            return tank.fill(resource, doFill);
        }
    }
}
