package railcraft.common.blocks.machine.beta;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.api.fuel.FuelManager;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.liquids.LiquidFilter;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.BoilerFuelTank;
import railcraft.common.liquids.tanks.FakeTank;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TileBoilerFireboxLiquid extends TileBoilerFirebox {

    private static final int TANK_FUEL = 2;

    public TileBoilerFireboxLiquid() {
        super(2);
        tankManager.addTank(new BoilerFuelTank(LiquidManager.BUCKET_VOLUME * 16));
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineBeta.BOILER_FIREBOX_LIQUID;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
//        ItemStack current = player.getCurrentEquippedItem();
//        if (current != null && current.itemID == getBlockId()) {
//            return false;
//        }
        TileMultiBlock mBlock = getMasterBlock();
        if (mBlock != null) {
            GuiHandler.openGui(EnumGui.BOILER_LIQUID, player, worldObj, mBlock.xCoord, mBlock.yCoord, mBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    protected boolean handleClick(EntityPlayer player) {
        if (LiquidManager.getInstance().handleRightClick(this, TANK_FUEL, player, true, false)) {
            return true;
        }
        return super.handleClick(player);
    }

    @Override
    protected void process() {
    }

    @Override
    protected void processBuckets() {
        super.processBuckets();

        LiquidManager.getInstance().processContainers(this, TANK_FUEL, inventory, SLOT_LIQUID_INPUT, SLOT_LIQUID_OUTPUT, true, false);
    }

    @Override
    protected float getMoreFuel() {
        LiquidStack fuel = tankManager.drain(TANK_FUEL, LiquidManager.BUCKET_VOLUME, false);
        if (fuel == null) {
            return 0;
        }

        float heatValue = FuelManager.getBoilerFuelValue(fuel);
        if (heatValue > 0) {
            tankManager.drain(TANK_FUEL, LiquidManager.BUCKET_VOLUME, true);
            if (fuel.amount < LiquidManager.BUCKET_VOLUME) {
                heatValue *= (float) fuel.amount / LiquidManager.BUCKET_VOLUME;
            }
        }
        return heatValue;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        LiquidStack liquid = LiquidManager.getInstance().getLiquidInContainer(stack);
        if (liquid == null) {
            return 0;
        }
        if (liquid.isLiquidEqual(LiquidFilter.WATER) || FuelManager.getBoilerFuelValue(liquid) > 0) {
            IInventory inv = invWaterInput;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remainder = InvTools.moveItemStack(stack, inv);
            if (remainder == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remainder.stackSize;
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory inv = invWaterOutput;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return InvTools.removeItems(inv, maxItemCount);
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        if (LiquidFilter.WATER.isLiquidEqual(resource)) {
            return fill(TANK_WATER, resource, doFill);
        }
        return fill(TANK_FUEL, resource, doFill);
    }

    @Override
    public ILiquidTank getTank(ForgeDirection side, LiquidStack type) {
        TankManager tMan = getTankManager();
        if (tMan != null) {
            switch (side) {
                case DOWN:
                    return new WaterTankWrapper(tMan.getTank(TANK_WATER));
                default:
                    return tMan.getTank(TANK_FUEL);
            }
        }
        return FakeTank.INSTANCE;
    }
}
