package railcraft.common.blocks.machine.beta;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.plugins.buildcraft.triggers.INeedsFuel;
import railcraft.common.util.inventory.EnumItemType;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.liquids.LiquidFilter;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TileBoilerFireboxSolid extends TileBoilerFirebox implements ISidedInventory, INeedsFuel {

    private static final int SLOT_BURN = 2;
    private static final int SLOT_FUEL_A = 3;
    private static final int SLOT_FUEL_B = 4;
    private static final int SLOT_FUEL_C = 5;
    private IInventory invBurn = new InventoryMapper(this, SLOT_BURN, 1);
    private IInventory invStock = new InventoryMapper(this, SLOT_FUEL_A, 3);
    private IInventory invFuel = new InventoryMapper(this, SLOT_BURN, 4);
    private ItemStack bucket = new ItemStack(Item.bucketEmpty);
    private boolean testNearbyChests = true;
    private List<IInventory> nearbyChests = new ArrayList<IInventory>();

    public TileBoilerFireboxSolid() {
        super(6);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineBeta.BOILER_FIREBOX_SOLID;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
//        ItemStack current = player.getCurrentEquippedItem();
//        if (current != null && current.itemID == getBlockId()) {
//            return false;
//        }
        TileMultiBlock mBlock = getMasterBlock();
        if (mBlock != null) {
            GuiHandler.openGui(EnumGui.BOILER_SOLID, player, worldObj, mBlock.xCoord, mBlock.yCoord, mBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    protected void process() {
        InvTools.moveOneItem(invStock, invBurn);
        InvTools.moveOneItem(invBurn, invWaterOutput, bucket);
    }

    @Override
    public void updateEntity() {
        super.updateEntity();        
        
        if (Game.isNotHost(getWorld())) {
            return;
        }

        TileBoilerFireboxSolid mBlock = (TileBoilerFireboxSolid) getMasterBlock();

        if (mBlock != null) {
            for (IInventory nearby : getNearbyChests()) {
                if (InvTools.moveOneItem(nearby, mBlock.invStock, EnumItemType.FUEL) != null) {
                    break;
                }
            }
        }
    }

    private List<IInventory> getNearbyChests() {
        if (testNearbyChests) {
            testNearbyChests = false;
            nearbyChests.clear();
            for (int side = 2; side < 6; side++) {
                ForgeDirection dir = ForgeDirection.getOrientation(side);
                IInventory inv = InvTools.getInventoryFromSide(worldObj, xCoord, yCoord, zCoord, dir, null, TileBoilerFirebox.class);
                if (inv != null) {
                    nearbyChests.add(inv);
                }
            }
        }
        return nearbyChests;
    }

    @Override
    public void onNeighborBlockChange(int id) {
        super.onNeighborBlockChange(id);
        testNearbyChests = true;
    }

    @Override
    protected float getMoreFuel() {
        ItemStack fuel = getStackInSlot(SLOT_BURN);
        int burn = MiscTools.getItemBurnTime(fuel);

        if (burn > 0) {
            setInventorySlotContents(SLOT_BURN, InvTools.depleteItem(fuel));
        }
        return burn;
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        switch (side) {
            case DOWN:
                return 0;
            default:
                return SLOT_BURN;
        }
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        switch (side) {
            case DOWN:
                return 2;
            default:
                return 4;
        }
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (stack == null) {
            return 0;
        }
        if (MiscTools.getItemBurnTime(stack) > 0) {
            IInventory inv = invFuel;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remainder = InvTools.moveItemStack(stack, inv);
            if (remainder == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remainder.stackSize;
        }

        if (LiquidManager.getInstance().isFilledContainer(stack) && LiquidManager.getInstance().containsLiquid(stack, LiquidFilter.WATER)) {
            IInventory inv = invWaterInput;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remainder = InvTools.moveItemStack(stack, inv);
            if (remainder == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remainder.stackSize;
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory inv = invWaterOutput;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return InvTools.removeItems(inv, maxItemCount);
    }

    @Override
    public boolean needsFuel() {
        TileBoilerFireboxSolid mBlock = (TileBoilerFireboxSolid) getMasterBlock();
        if (mBlock != null) {
            int amount = InvTools.countItems(mBlock.invFuel);
            return amount < 64;
        }
        return false;
    }
}
