package railcraft.common.blocks.machine.beta;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import java.util.Random;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.effects.EffectManager;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.FilteredTank;
import railcraft.common.util.steam.Steam;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileEngineSteam extends TileEngine implements ITankContainer, ISteamUser {

    private final static int TANK_CAPACITY = 8 * LiquidManager.BUCKET_VOLUME;
    public final static int TANK_STEAM = 0;
    private final FilteredTank steamTank;
    private TankManager tankManager = new TankManager();
    private int steamUsed;

    public TileEngineSteam() {
        steamTank = new FilteredTank(TANK_CAPACITY, LiquidItems.getSteamLiquid());
        tankManager.addTank(steamTank);
    }

    private int getParticleRate() {
        switch (getEnergyStage()) {
            case BLUE:
                return 1;
            case GREEN:
                return 2;
            case YELLOW:
                return 3;
            case ORANGE:
                return 4;
            case RED:
                return 5;
            case OVERHEAT:
                return 8;
            default:
                return 0;
        }
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.ENGINE_STEAM, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void randomDisplayTick(Random rand) {
        if (isActive() || getEnergyStage() == EnergyStage.OVERHEAT) {
            int steamRate = getParticleRate();
            for (int i = 0; i < steamRate; i++) {
                EffectManager.instance.steamEffect(worldObj, this, 0);
            }
        }
    }

    @Override
    public void burn() {
        float output = 0;

        if (getEnergyStage() != EnergyStage.OVERHEAT) {
            if (isPowered()) {
                LiquidStack steam = steamTank.getLiquid();
                if (steam != null && steam.amount >= steamTank.getCapacity() / 2 - Steam.STEAM_PER_UNIT_WATER) {
                    steam = tankManager.drain(0, steamUsedPerTick() - 1, true);
                    if (steam != null) {
                        steamUsed += steam.amount;
                    }
                }
            }
            LiquidStack steam = tankManager.drain(0, 1, true);
            if (steam != null) {
                steamUsed += steam.amount;
            }

            if (isPowered()) {
                if (steamUsed >= steamUsedPerTick()) {
                    steamUsed -= steamUsedPerTick();
                    output = getMaxOutputMJ();
                    addEnergy(getMaxOutputMJ());
                }
            } else {
                steamUsed = 0;
                ventSteam();
            }
        }

        currentOutput = (currentOutput * 74 + output) / 75f;
    }

    @Override
    protected void overheat() {
        super.overheat();
        ventSteam();
    }

    protected void ventSteam() {
        getTankManager().drain(TANK_STEAM, 5, true);
    }

    public abstract float getMaxOutputMJ();

    public abstract int steamUsedPerTick();

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        tankManager.writeTanksToNBT(data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        tankManager.readTanksFromNBT(data);
    }

    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        if (!isPowered()) {
            return 0;
        }
        return fill(0, resource, doFill);
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        return tankManager.fill(tankIndex, resource, doFill);
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection dir) {
        return tankManager.getTanks();
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type) {
        return tankManager.getTank(TANK_STEAM);
    }

    public TankManager getTankManager() {
        return tankManager;
    }
}
