package railcraft.common.blocks.machine.beta;

import buildcraft.api.inventory.ISpecialInventory;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.gui.indicator.IIndicatorController;
import railcraft.common.gui.util.ToolTip;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.buildcraft.triggers.INeedsFuel;
import railcraft.common.plugins.buildcraft.triggers.ITemperature;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.inventory.StandaloneInventory;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankWrapper;
import railcraft.common.liquids.tanks.FilteredTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.steam.Steam;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TileEngineSteamHobby extends TileEngineSteam implements IInventory, ISidedInventory, ISpecialInventory, INeedsFuel, ITemperature {

    private static final float OUTPUT_MJ = 2f;
    private static final byte FUEL_PER_CONVERSION = 8;
    private static final byte TICKS_PER_CONVERSION = 20;
    private static final byte TANK_WATER = 1;
    private static final byte SLOT_FUEL = 0;
    private static final byte SLOT_LIQUID_INPUT = 1;
    private static final byte SLOT_LIQUID_OUTPUT = 2;
    public float burnTime;
    public float currentItemBurnTime;
    private int burnCycle;
    public float heat = Steam.COLD_TEMP;
    private int update;
    private final LiquidStack waterFilter = new LiquidStack(Block.waterStill, 0);
    private final LiquidStack steamFilter = LiquidItems.getSteamLiquid();
    private StandaloneInventory inv = new StandaloneInventory(3, (IInventory) this);
    private IIndicatorController heatIndicator = new HeatIndicator();

    private class HeatIndicator implements IIndicatorController {

        List<ToolTip> tips = new ArrayList<ToolTip>();
        ToolTip tip = new ToolTip();

        private HeatIndicator() {
            tips.add(tip);
        }

        @Override
        public List<ToolTip> getToolTip() {
            tip.text = String.format("%.0f\u00B0C", heat);
            return tips;
        }

        @Override
        public int getScaledLevel(int size) {
            return (int) ((heat - Steam.COLD_TEMP) * size / (Steam.MAX_HEAT - Steam.COLD_TEMP));
        }
    };

    public TileEngineSteamHobby() {
        getTankManager().addTank(new FilteredTank(4 * LiquidManager.BUCKET_VOLUME, waterFilter));
        getTankManager().getTank(TANK_STEAM).setCapacity(4 * LiquidManager.BUCKET_VOLUME);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineBeta.ENGINE_STEAM_HOBBY;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.ENGINE_HOBBY, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null && current.itemID != Item.bucketEmpty.itemID) {
            if (Game.isHost(worldObj)) {
                if (LiquidManager.getInstance().handleRightClick(this, TANK_WATER, player, true, false)) {
                    return true;
                }
            } else {
                if (LiquidManager.getInstance().isContainer(current)) {
                    return true;
                }
            }
        }
        return super.blockActivated(player, side);
    }

    public IIndicatorController getHeatIndicator() {
        return heatIndicator;
    }

    public boolean isBurning() {
        return burnTime > 0;
    }

    @Override
    public float getMaxOutputMJ() {
        return OUTPUT_MJ;
    }

    @Override
    public int steamUsedPerTick() {
        return 10;
    }

    @Override
    public void burn() {
        super.burn();
        update++;

        if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
            LiquidManager.getInstance().processContainers(this, 1, inv, SLOT_LIQUID_INPUT, SLOT_LIQUID_OUTPUT, true, false);
        }

        burnCycle++;
        if (burnCycle >= TICKS_PER_CONVERSION) {
            burnCycle = 0;
            float fuelNeeded = getFuelPerConversion();
            while (burnTime < fuelNeeded) {
                boolean addedFuel = addFuel();
                if (!addedFuel) {
                    break;
                }
            }
            burnTime -= fuelNeeded;
            if (burnTime <= 0) {
                burnTime = 0;
            }
            if (heat >= Steam.BOILING_POINT) {
                LiquidStack water = getTankManager().drain(TANK_WATER, 1, false);
                if (water != null) {
                    getTankManager().drain(TANK_WATER, 1, true);
                    LiquidStack steam = LiquidItems.getSteamLiquid(Steam.STEAM_PER_UNIT_WATER);
                    getTankManager().fill(TANK_STEAM, steam, true);
                }
            }
        }

        if (isBurning()) {
            increaseHeat();
        } else {
            reduceHeat();
        }
    }

    private boolean addFuel() {
        if (!isPowered()) return false;
        if (getEnergyStage() == EnergyStage.OVERHEAT) return false;
        int fuel = getMoreFuel();
        burnTime += fuel;
        currentItemBurnTime = burnTime;
        return fuel > 0;
    }

    protected int getMoreFuel() {
        ItemStack fuel = getStackInSlot(SLOT_FUEL);
        int burn = MiscTools.getItemBurnTime(fuel);

        if (burn > 0) {
            setInventorySlotContents(SLOT_FUEL, InvTools.depleteItem(fuel));
        }
        return burn;
    }

    public int getFuelPerConversion() {
        float fuel = FUEL_PER_CONVERSION;
        fuel = fuel + fuel * (Steam.INEFFICENCY_MULTIPLIER - Steam.INEFFICENCY_MULTIPLIER * getHeatLevel());
        return Math.round(fuel);
    }

    public float getHeatLevel() {
        return heat / Steam.MAX_HEAT;
    }

    private void increaseHeat() {
        float heatLevel = getHeatLevel();
        heat += Steam.HEAT_STEP;
        if (heatLevel < 0.25f) {
            heat += Steam.HEAT_STEP;
        }
        if (heatLevel < 0.5f) {
            heat += Steam.HEAT_STEP;
        }
        if (heatLevel < 0.75f) {
            heat += Steam.HEAT_STEP;
        }
        heat = Math.min(heat, Steam.MAX_HEAT);
    }

    private void reduceHeat() {
        float heatLevel = getHeatLevel();
        heat -= Steam.HEAT_STEP;
        if (heatLevel > 0.25f) {
            heat -= Steam.HEAT_STEP;
        }
        if (heatLevel > 0.5f) {
            heat -= Steam.HEAT_STEP;
        }
        if (heatLevel > 0.75f) {
            heat -= Steam.HEAT_STEP;
        }
        heat = Math.max(heat, Steam.COLD_TEMP);
    }

    public int getBurnProgressScaled(int i) {
        return (int) (((float) burnTime / (float) currentItemBurnTime) * i);
    }

    @Override
    public int getSizeInventory() {
        return inv.getSizeInventory();
    }

    @Override
    public ItemStack getStackInSlot(int slot) {
        return inv.getStackInSlot(slot);
    }

    @Override
    public ItemStack decrStackSize(int slot, int amount) {
        return inv.decrStackSize(slot, amount);
    }

    @Override
    public ItemStack getStackInSlotOnClosing(int var1) {
        return null;
    }

    @Override
    public void setInventorySlotContents(int slot, ItemStack stack) {
        inv.setInventorySlotContents(slot, stack);
    }

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(EnumMachineBeta.ENGINE_STEAM_HOBBY.getTag());
    }

    @Override
    public int getInventoryStackLimit() {
        return 64;
    }

    @Override
    public void openChest() {
    }

    @Override
    public void closeChest() {
    }

    @Override
    public int maxEnergy() {
        return 10000;
    }

    @Override
    public int maxEnergyReceived() {
        return 300;
    }

    @Override
    public int maxEnergyExtracted() {
        return 40;
    }

    @Override
    public boolean needsFuel() {
        ItemStack fuel = inv.getStackInSlot(SLOT_FUEL);
        return fuel == null || fuel.stackSize < 8;
    }

    @Override
    public float getTemperature() {
        return heat;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        inv.writeToNBT("Items", data);

        data.setFloat("heat", heat);
        data.setFloat("burnTime", burnTime);
        data.setFloat("currentItemBurnTime", currentItemBurnTime);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        inv.readFromNBT("Items", data);

        heat = data.getFloat("heat");
        burnTime = data.getFloat("burnTime");
        currentItemBurnTime = data.getFloat("currentItemBurnTime");
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        switch (side) {
            case UP:
                return SLOT_LIQUID_INPUT;
            case DOWN:
                return SLOT_LIQUID_OUTPUT;
            default:
                return SLOT_FUEL;
        }
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 1;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (LiquidManager.getInstance().isFilledContainer(stack)) {
            IInventory slot = new InventoryMapper(this, SLOT_LIQUID_INPUT, 1);
            if (!doAdd) {
                slot = new InventoryCopy(slot);
            }
            ItemStack ret = InvTools.moveItemStack(stack.copy(), slot);
            if (ret != null && stack.stackSize != ret.stackSize) {
                return stack.stackSize - ret.stackSize;
            } else if (ret == null) {
                return stack.stackSize;
            }
        }
        if (MiscTools.getItemBurnTime(stack) > 0) {
            IInventory slot = new InventoryMapper(this, SLOT_FUEL, 1);
            if (!doAdd) {
                slot = new InventoryCopy(slot);
            }
            ItemStack ret = InvTools.moveItemStack(stack.copy(), slot);
            if (ret != null && stack.stackSize != ret.stackSize) {
                return stack.stackSize - ret.stackSize;
            } else if (ret == null) {
                return stack.stackSize;
            }
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory outputInv = this;
        if (!doRemove) {
            outputInv = new InventoryCopy(outputInv);
        }
        return new ItemStack[]{outputInv.decrStackSize(SLOT_LIQUID_OUTPUT, maxItemCount)};
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        if (isPowered() && steamFilter.isLiquidEqual(resource)) {
            return fill(0, resource, doFill);
        }
        if (waterFilter.isLiquidEqual(resource)) {
            return fill(1, resource, doFill);
        }
        return 0;
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        if (tankIndex == 1) {
            if (heat >= Steam.SUPER_HEATED) {
                LiquidStack water = getTankManager().getTank(TANK_WATER).getLiquid();
                if (water == null || water.amount <= 0) {
                    explode();
                }
            }
        }
        return super.fill(tankIndex, resource, doFill);
    }

    @Override
    public ILiquidTank getTank(ForgeDirection side, LiquidStack type) {
        switch (side) {
            case DOWN:
            case UP:
                return getTankManager().getTank(TANK_STEAM);
            default:
                return new WaterTankWrapper(getTankManager().getTank(TANK_WATER));
        }
    }

    public void explode() {
        if (Game.isHost(worldObj)) {
            worldObj.createExplosion(null, xCoord, yCoord, zCoord, 2, true);
        }
    }

    protected class WaterTankWrapper extends TankWrapper {

        public WaterTankWrapper(ILiquidTank tank) {
            super(tank);
        }

        @Override
        public int fill(LiquidStack resource, boolean doFill) {
            if (resource == null) return 0;

            if (heat >= Steam.SUPER_HEATED) {
                LiquidStack water = tank.getLiquid();
                if (water == null || water.amount <= 0) {
                    explode();
                }
            }

            return tank.fill(resource, doFill);
        }
    }
}
