package railcraft.common.blocks.machine.beta;

import cpw.mods.fml.common.FMLCommonHandler;
import cpw.mods.fml.relauncher.Side;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraftforge.liquids.ILiquidTank;
import railcraft.common.blocks.machine.ITankTile;
import railcraft.common.blocks.machine.MultiBlockPattern;
import railcraft.common.blocks.machine.TileMultiBlock;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.gui.slots.SlotLiquidContainer;
import railcraft.common.util.inventory.StandaloneInventory;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileTankIron extends TileMultiBlock implements ITankTile {

    private final static int CAPACITY_PER_BLOCK = 16 * LiquidManager.BUCKET_VOLUME;
    protected final static int SLOT_INPUT = 0;
    protected final static int SLOT_OUTPUT = 1;
    protected final TankManager tankManager = new TankManager();
    private final static List<MultiBlockPattern> patterns = buildPatterns();
    private final static Set<Integer> tankBlocks = new HashSet<Integer>();
    private final StandaloneInventory inv;

    static {
        tankBlocks.add(EnumMachineBeta.TANK_IRON_WALL.ordinal());
        tankBlocks.add(EnumMachineBeta.TANK_IRON_VALVE.ordinal());
        tankBlocks.add(EnumMachineBeta.TANK_IRON_GAUGE.ordinal());
    }

    protected TileTankIron() {
        super(patterns);
        inv = new StandaloneInventory(2, "gui.tank.iron", this);
        StandardTank tank = new StandardTank(64 * LiquidManager.BUCKET_VOLUME, this);
        tankManager.addTank(tank);
    }

    @Override
    public IInventory getInventory() {
        return inv;
    }

    @Override
    public Slot getInputSlot(IInventory inv, int slotNum, int x, int y) {
        return new SlotLiquidContainer(inv, slotNum, x, y);
    }

    @Override
    public float getResistance(Entity exploder) {
        return 15f;
    }

    @Override
    protected int getMaxRecursionDepth() {
        return 500;
    }

    @Override
    protected boolean isStructureTile(TileEntity tile) {
        return tile instanceof TileTankIron;
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        ItemStack current = player.getCurrentEquippedItem();
        if (Game.isHost(worldObj)) {
            if (isStructureValid() && LiquidManager.getInstance().handleRightClick(getTankManager(), 0, player, true, true)) {
                return true;
            }
        } else {
            if (LiquidManager.getInstance().isContainer(current)) {
                return true;
            }
        }

        // Prevents players from getting inside tanks using boats
        if (current != null && current.itemID == Item.boat.itemID) {
            return true;
        }
        return super.blockActivated(player, side);
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        TileMultiBlock mBlock = getMasterBlock();
        if (mBlock != null) {
            GuiHandler.openGui(EnumGui.TANK, player, worldObj, mBlock.xCoord, mBlock.yCoord, mBlock.zCoord);
            return true;
        }
        return false;
    }

    @Override
    public TankManager getTankManager() {
        TileTankIron mBlock = (TileTankIron) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager;
        }
        return null;
    }

    @Override
    public ILiquidTank getTank() {
        TileTankIron mBlock = (TileTankIron) getMasterBlock();
        if (mBlock != null) {
            return mBlock.tankManager.getTank(0);
        }
        return null;
    }

    @Override
    protected void onPatternLock(MultiBlockPattern pattern) {
        if (isMaster) {
            int capacity = (pattern.getPatternWidthX() - 2) * (pattern.getPatternHeight() - (pattern.getMasterOffsetY() * 2)) * (pattern.getPatternWidthZ() - 2) * CAPACITY_PER_BLOCK;
            tankManager.setCapacity(0, capacity);
        } else {
            tankManager.getTank(0).setLiquid(null);
        }
    }

    @Override
    protected boolean isMapPositionValid(int x, int y, int z, char mapPos) {
        switch (mapPos) {
            case 'O': // Other
            {
                int id = worldObj.getBlockId(x, y, z);
                if (id == getBlockId()) {
                    int meta = worldObj.getBlockMetadata(x, y, z);
                    if (tankBlocks.contains(meta)) {
                        return false;
                    }
                }
                return true;
            }
            case 'W': // Gauge or Valve
            {
                int id = worldObj.getBlockId(x, y, z);
                if (id != getBlockId()) {
                    return false;
                }
                int meta = worldObj.getBlockMetadata(x, y, z);
                if (!tankBlocks.contains(meta)) {
                    return false;
                }
                return true;
            }
            case 'B': // Block
            {
                int id = worldObj.getBlockId(x, y, z);
                if (id != getBlockId()) {
                    return false;
                }
                int meta = worldObj.getBlockMetadata(x, y, z);
                if (meta != EnumMachineBeta.TANK_IRON_WALL.ordinal()) {
                    return false;
                }
                return true;
            }
            case 'M': // Master
            case 'T': // Top Block
            {
                int id = worldObj.getBlockId(x, y, z);
                if (id != getBlockId()) {
                    return false;
                }
                int meta = worldObj.getBlockMetadata(x, y, z);
                if (!tankBlocks.contains(meta)) {
                    return false;
                }
                TileEntity tile = worldObj.getBlockTileEntity(x, y, z);
                if (!(tile instanceof TileMultiBlock)) {
                    worldObj.removeBlockTileEntity(x, y, z);
                    return true;
                }
                if (((TileMultiBlock) tile).isStructureValid()) {
                    return false;
                }
                return true;
            }
            case 'A': // Air
            {
                if (!worldObj.isAirBlock(x, y, z)) {
                    return false;
                }
                return true;
            }
        }
        return true;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isHost(worldObj)) {
            if (isMaster) {

                if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
                    LiquidManager.getInstance().processContainers(tankManager.getTank(0), inv, SLOT_INPUT, SLOT_OUTPUT);
                }

                if (update % LiquidManager.NETWORK_UPDATE_INTERVAL == 0) {
                    sendUpdateToClient();
                }
            }
        }
    }

    @Override
    public void invalidate() {
        super.invalidate();
        tankManager.getTank(0).setLiquid(null);
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        tankManager.writeTanksToNBT(data);
        inv.writeToNBT("inv", data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        tankManager.readTanksFromNBT(data);
        inv.readFromNBT("inv", data);
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);
        tankManager.writePacketData(data);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);
        tankManager.readPacketData(data);
    }

    private static List<MultiBlockPattern> buildPatterns() {
        List<MultiBlockPattern> pats = new ArrayList<MultiBlockPattern>();
        boolean client = FMLCommonHandler.instance().getEffectiveSide() == Side.CLIENT;

        // 3x3
        int xOffset = 2;
        int yOffset = 0;
        int zOffset = 2;

        char[][] bottom = new char[][]{
            {'O', 'O', 'O', 'O', 'O'},
            {'O', 'B', 'B', 'B', 'O'},
            {'O', 'B', 'M', 'B', 'O'},
            {'O', 'B', 'B', 'B', 'O'},
            {'O', 'O', 'O', 'O', 'O'}
        };

        char[][] middle = new char[][]{
            {'O', 'O', 'O', 'O', 'O'},
            {'O', 'B', 'W', 'B', 'O'},
            {'O', 'W', 'A', 'W', 'O'},
            {'O', 'B', 'W', 'B', 'O'},
            {'O', 'O', 'O', 'O', 'O'}
        };

        char[][] top = new char[][]{
            {'O', 'O', 'O', 'O', 'O'},
            {'O', 'B', 'B', 'B', 'O'},
            {'O', 'B', 'T', 'B', 'O'},
            {'O', 'B', 'B', 'B', 'O'},
            {'O', 'O', 'O', 'O', 'O'}
        };

        char[][] border = new char[][]{
            {'O', 'O', 'O', 'O', 'O'},
            {'O', 'O', 'O', 'O', 'O'},
            {'O', 'O', 'O', 'O', 'O'},
            {'O', 'O', 'O', 'O', 'O'},
            {'O', 'O', 'O', 'O', 'O'}
        };

        for (int i = 4; i <= 8; i++) {
            char[][][] map = buildMap(i, bottom, middle, top, border);
            AxisAlignedBB entityCheck = AxisAlignedBB.getBoundingBox(0, 1, 0, 1, i - 1, 1);
            pats.add(buildPattern(map, xOffset, yOffset, zOffset, entityCheck));
        }

        // 5x5
        if (client || RailcraftConfig.getMaxTankSize() >= 5) {
            xOffset = zOffset = 3;

            bottom = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'M', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            middle = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            top = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'T', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            border = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            for (int i = 4; i <= 8; i++) {
                char[][][] map = buildMap(i, bottom, middle, top, border);
                AxisAlignedBB entityCheck = AxisAlignedBB.getBoundingBox(-1, 1, -1, 2, i - 1, 2);
                pats.add(buildPattern(map, xOffset, yOffset, zOffset, entityCheck));
            }
        }


        // 7x7
        if (client || RailcraftConfig.getMaxTankSize() >= 7) {
            xOffset = zOffset = 4;

            bottom = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'M', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            middle = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            top = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'T', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            border = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            for (int i = 4; i <= 8; i++) {
                char[][][] map = buildMap(i, bottom, middle, top, border);
                AxisAlignedBB entityCheck = AxisAlignedBB.getBoundingBox(-2, 1, -2, 3, i - 1, 3);
                pats.add(buildPattern(map, xOffset, yOffset, zOffset, entityCheck));
            }
        }

        // 9x9
        if (client || RailcraftConfig.getMaxTankSize() >= 9) {
            xOffset = zOffset = 5;

            bottom = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'M', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            middle = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'W', 'A', 'A', 'A', 'A', 'A', 'A', 'A', 'W', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            top = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'T', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'W', 'W', 'W', 'W', 'W', 'W', 'W', 'B', 'O'},
                {'O', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'B', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            border = new char[][]{
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'},
                {'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O', 'O'}
            };

            for (int i = 4; i <= 8; i++) {
                char[][][] map = buildMap(i, bottom, middle, top, border);
                AxisAlignedBB entityCheck = AxisAlignedBB.getBoundingBox(-3, 1, -3, 4, i - 1, 4);
                pats.add(buildPattern(map, xOffset, yOffset, zOffset, entityCheck));
            }
        }

        return pats;
    }

    private static MultiBlockPattern buildPattern(char[][][] map, int xOffset, int yOffset, int zOffset, AxisAlignedBB entityCheck) {
        if (!RailcraftConfig.allowTankStacking()) {
            entityCheck.offset(0, 1, 0);
            yOffset = 1;
        }
        return new MultiBlockPattern(map, xOffset, yOffset, zOffset, entityCheck);
    }

    private static char[][][] buildMap(int height, char[][] bottom, char[][] mid, char[][] top, char[][] border) {
        char[][][] map;
        if (RailcraftConfig.allowTankStacking()) {
            map = new char[height][][];

            map[0] = bottom;
            map[height - 1] = top;

            for (int i = 1; i < height - 1; i++) {
                map[i] = mid;
            }
        } else {
            map = new char[height + 2][][];

            map[0] = border;
            map[1] = bottom;
            map[height] = top;
            map[height + 1] = border;

            for (int i = 2; i < height; i++) {
                map[i] = mid;
            }
        }

        return map;
    }
}
