package railcraft.common.blocks.machine.gamma;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.TileMachineBase;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.modules.ModuleManager;
import railcraft.common.modules.ModuleManager.Module;

/**
 *
 * @author CovertJaguar
 */
public enum EnumMachineGamma implements IEnumMachine {

    ITEM_LOADER(Module.TRANSPORT, "loader.item", TileItemLoader.class, new int[]{12, 10, 11, 11, 11, 11}),
    ITEM_UNLOADER(Module.TRANSPORT, "unloader.item", TileItemUnloader.class, new int[]{154, 156, 155, 155, 155, 155}),
    ITEM_LOADER_ADVANCED(Module.TRANSPORT, "loader.item.advanced", TileItemLoaderAdvanced.class, new int[]{122, 122, 123, 124, 123, 123}),
    ITEM_UNLOADER_ADVANCED(Module.TRANSPORT, "unloader.item.advanced", TileItemUnloaderAdvanced.class, new int[]{138, 138, 139, 140, 139, 139}),
    LIQUID_LOADER(Module.TRANSPORT, "loader.liquid", TileLiquidLoader.class, new int[]{28, 26, 27, 27, 27, 27}),
    LIQUID_UNLOADER(Module.TRANSPORT, "unloader.liquid", TileLiquidUnloader.class, new int[]{26, 28, 27, 27, 27, 27}),
    ENERGY_LOADER(Module.IC2, "loader.energy", TileEnergyLoader.class, new int[]{202, 202, 203, 204, 203, 203}),
    ENERGY_UNLOADER(Module.IC2, "unloader.energy", TileEnergyUnloader.class, new int[]{218, 218, 219, 220, 219, 219}),
    DISPENSER_CART(Module.AUTOMATION, "dispenser.cart", TileDispenserCart.class, new int[]{106, 106, 107, 108, 107, 107}),
    DISPENSER_TRAIN(Module.TRAIN, "dispenser.train", TileDispenserTrain.class, new int[]{231, 231, 232, 233, 232, 232}),;
    private final Module module;
    private final String tag;
    private final Class<? extends TileMachineBase> tile;
    private final int[] texture;
    private static final List<EnumMachineGamma> creativeList = new ArrayList<EnumMachineGamma>();
    private static final EnumMachineGamma[] VALUES = values();

    static {
        creativeList.add(ITEM_LOADER);
        creativeList.add(ITEM_UNLOADER);
        creativeList.add(ITEM_LOADER_ADVANCED);
        creativeList.add(ITEM_UNLOADER_ADVANCED);
        creativeList.add(LIQUID_LOADER);
        creativeList.add(LIQUID_UNLOADER);
        creativeList.add(ENERGY_LOADER);
        creativeList.add(ENERGY_UNLOADER);
        creativeList.add(DISPENSER_CART);
        creativeList.add(DISPENSER_TRAIN);
    }

    private EnumMachineGamma(Module module, String tag, Class<? extends TileMachineBase> tile, int[] texture) {
        this.module = module;
        this.tile = tile;
        this.tag = tag;
        this.texture = texture;
    }

    @Override
    public boolean isDepreciated() {
        return module == null;
    }

    @Override
    public int getTexture(int index) {
        if (index < 0 || index >= texture.length) {
            index = 0;
        }
        return texture[index];
    }

    public static EnumMachineGamma fromId(int id) {
        if (id < 0 || id >= VALUES.length) {
            id = 0;
        }
        return VALUES[id];
    }

    public static List<EnumMachineGamma> getCreativeList() {
        return creativeList;
    }

    @Override
    public String getTag() {
        return "rc.machine.gamma." + tag;
    }

    @Override
    public Class getTileClass() {
        return tile;
    }

    public TileMachineBase getTileEntity() {
        try {
            return tile.newInstance();
        } catch (Exception ex) {
        }
        return null;
    }

    @Override
    public ItemStack getItem() {
        return getItem(1);
    }

    @Override
    public ItemStack getItem(int qty) {
        Block block = getBlock();
        if (block == null) {
            return null;
        }
        return new ItemStack(block, qty, ordinal());
    }

    public Module getModule() {
        return module;
    }

    @Override
    public Block getBlock() {
        return RailcraftBlocks.getBlockMachineGamma();
    }

    @Override
    public int getBlockId() {
        Block block = getBlock();
        if (block != null) {
            return block.blockID;
        }
        return 0;
    }

    @Override
    public boolean isEnabled() {
        return ModuleManager.isModuleLoaded(getModule()) && getBlock() != null && RailcraftConfig.isSubBlockEnabled(getTag());
    }
}
