package railcraft.common.blocks.machine.gamma;

import buildcraft.api.tools.IToolWrench;
import ic2.api.IWrenchable;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemMinecart;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.core.items.IMinecartItem;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.machine.TileMachineItem;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

public class TileDispenserCart extends TileMachineItem implements IWrenchable, ISidedInventory {

    protected ForgeDirection direction = ForgeDirection.NORTH;
    protected boolean powered;
    protected int timeSinceLastSpawn;

    public TileDispenserCart() {
        super(3);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineGamma.DISPENSER_CART;
    }

    @Override
    public int getBlockTexture(int side) {
        if (direction.ordinal() == side) {
            return getMachineType().getTexture(3);
        }
        if (side != 0 && side != 1) {
            return getMachineType().getTexture(2);
        }
        return getMachineType().getTexture(1);
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null && current.getItem() instanceof IToolWrench) {
            IToolWrench wrench = (IToolWrench) current.getItem();
            if (wrench.canWrench(player, xCoord, yCoord, zCoord)) {
                if (direction.ordinal() == side) {
                    direction = MiscTools.getOppositeSide(side);
                } else {
                    direction = ForgeDirection.getOrientation(side);
                }
                wrench.wrenchUsed(player, xCoord, yCoord, zCoord);
                markBlockForUpdate();
                return true;
            }
        }
        return super.blockActivated(player, side);
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.CART_DISPENSER, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public void onBlockPlacedBy(EntityLiving entityliving) {
        super.onBlockPlacedBy(entityliving);
        direction = MiscTools.getSideClosestToPlayer(worldObj, xCoord, yCoord, zCoord, entityliving);
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();
        if (timeSinceLastSpawn < Integer.MAX_VALUE) {
            timeSinceLastSpawn++;
        }
    }

    @Override
    public void onNeighborBlockChange(int id) {
        if (Game.isNotHost(getWorld())) {
            return;
        }
        boolean newPower = worldObj.isBlockGettingPowered(xCoord, yCoord, zCoord) || worldObj.isBlockIndirectlyGettingPowered(xCoord, yCoord, zCoord);
        if (!powered && newPower) {
            powered = newPower;
            EntityMinecart cart = CartTools.getMinecartOnSide(worldObj, xCoord, yCoord, zCoord, 0, direction);
            if (cart == null) {
                if (timeSinceLastSpawn > RailcraftConfig.getCartDispenserMinDelay() * 20) {
                    for (int ii = 0; ii < getSizeInventory(); ii++) {
                        ItemStack cartStack = getStackInSlot(ii);
                        if (cartStack != null) {
                            int x = MiscTools.getXOnSide(xCoord, direction);
                            int y = MiscTools.getYOnSide(yCoord, direction);
                            int z = MiscTools.getZOnSide(zCoord, direction);
                            boolean minecartItem = cartStack.getItem() instanceof IMinecartItem;
                            if (cartStack.getItem() instanceof ItemMinecart || minecartItem) {
                                boolean canPlace = true;
                                if (minecartItem) {
                                    canPlace = ((IMinecartItem) cartStack.getItem()).canBePlacedByNonPlayer(cartStack);
                                }
                                if (canPlace) {
                                    ItemStack placedStack = cartStack.copy();
                                    EntityMinecart placedCart = CartTools.placeCart(getOwner(), placedStack, worldObj, x, y, z);
                                    if (placedCart != null) {
                                        decrStackSize(ii, 1);
                                        timeSinceLastSpawn = 0;
                                        break;
                                    }
                                }
                            } else {
                                float rx = MiscTools.getRand().nextFloat() * 0.8F + 0.1F;
                                float ry = MiscTools.getRand().nextFloat() * 0.8F + 0.1F;
                                float rz = MiscTools.getRand().nextFloat() * 0.8F + 0.1F;
                                EntityItem item = new EntityItem(worldObj, x + rx, y + ry, z + rz, cartStack);
                                float factor = 0.05F;
                                item.motionX = (float) MiscTools.getRand().nextGaussian() * factor;
                                item.motionY = (float) MiscTools.getRand().nextGaussian() * factor + 0.2F;
                                item.motionZ = (float) MiscTools.getRand().nextGaussian() * factor;
                                if (worldObj.spawnEntityInWorld(item)) {
                                    setInventorySlotContents(ii, null);
                                }
                            }
                        }
                    }
                }
            } else if (!cart.isDead && cart.getCartItem() != null) {
                IInventory testInv = new InventoryCopy(this);
                ItemStack cartStack = InvTools.moveItemStack(cart.getCartItem(), testInv);
                if (cartStack == null) {
                    InvTools.moveItemStack(cart.getCartItem(), this);
                    if (cart.riddenByEntity != null) {
                        cart.riddenByEntity.mountEntity(null);
                    }
                    cart.setDead();
                }
            }
        } else if (powered ^ newPower) {
            powered = newPower;
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setBoolean("powered", powered);
        data.setByte("direction", (byte) direction.ordinal());

        data.setInteger("time", timeSinceLastSpawn);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        powered = data.getBoolean("powered");
        direction = ForgeDirection.getOrientation(data.getByte("direction"));

        timeSinceLastSpawn = data.getInteger("time");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeByte(direction.ordinal());
//        data.writeBoolean(powered);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        direction = ForgeDirection.getOrientation(data.readByte());
//        powered = data.readBoolean();

        markBlockForUpdate();
    }

    public boolean getPowered() {
        return powered;
    }

    public void setPowered(boolean power) {
        powered = power;
    }

    @Override
    public boolean wrenchCanSetFacing(EntityPlayer player, int side) {
        if (direction.ordinal() == side) {
            return false;
        }
        return true;
    }

    @Override
    public void setFacing(short facing) {
        direction = ForgeDirection.getOrientation(facing);
        sendUpdateToClient();
    }

    @Override
    public short getFacing() {
        return (short) direction.ordinal();
    }

    @Override
    public boolean wrenchCanRemove(EntityPlayer entityPlayer) {
        return true;
    }

    @Override
    public float getWrenchDropRate() {
        return 1;
    }

    @Override
    public ItemStack getWrenchDrop(EntityPlayer entityPlayer) {
        return getMachineType().getItem();
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        return 0;
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return getSizeInventory();
    }

    @Override
    public int getInventoryStackLimit() {
        return 64;
    }
}
