package railcraft.common.blocks.machine.gamma;

import ic2.api.Direction;
import ic2.api.energy.EnergyNet;
import ic2.api.energy.tile.IEnergySource;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.carts.IEnergyTransfer;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.plugins.ic2.IC2Plugin;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.IGuiReturnHandler;

public class TileEnergyUnloader extends TileLoaderEnergyBase implements IEnergySource, IGuiReturnHandler
{

    private static final int[] OUTPUT_LEVELS = {512, 2048};
    private boolean transferedEnergy;
    private boolean waitTillEmpty = true;

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineGamma.ENERGY_UNLOADER;
    }

    @Override
    public int getBlockTexture(int side) {
        if(getFacing() == side) {
            return getMachineType().getTexture(3);
        }
        if(side != 0 && side != 1) {
            return getMachineType().getTexture(2);
        }
        return getMachineType().getTexture(1);
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.UNLOADER_ENERGY, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if(Game.isNotHost(getWorld())) {
            return;
        }

        int emit = transformerUpgrades > 0 ? OUTPUT_LEVELS[1] : OUTPUT_LEVELS[0];
        if(energy > emit) {
            int extra = IC2Plugin.emitEnergy(this, emit);
            energy -= emit - extra;
        }

        transferedEnergy = false;
        transferRate = 0;

        EntityMinecart cart = CartTools.getMinecartOnSide(worldObj, xCoord, yCoord, zCoord, 0.1f, direction);

        if(cart != currentCart) {
            setPowered(false);
            currentCart = cart;
            cartSent();
        }

        if(cart == null) {
            return;
        }

        if(!(cart instanceof IEnergyTransfer)) {
            if(CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                setPowered(true);
            }
            return;
        }

        if(isSendCart()) {
            if(CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                setPowered(true);
            }
            return;
        }

        IEnergyTransfer energyCart = (IEnergyTransfer)cart;

        if(!energyCart.canExtractEnergy() || energyCart.getTier() > getTier()) {
            if(CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                setPowered(true);
            }
            return;
        }

        if(energy < getCapacity() && energyCart.getEnergy() > 0) {
            int usage = (int)(energyCart.getTransferLimit() * Math.pow(1.5, overclockerUpgrades));
            int injection = (int)(energyCart.getTransferLimit() * Math.pow(1.3, overclockerUpgrades));

            int room = getCapacity() - getEnergy();
            if(room < injection) {
                double ratio = (double)room / (double)injection;
                injection = room;
                usage = (int)(usage * ratio);
            }

            int extract = energyCart.extractEnergy(this, usage, getTier(), true, false, false);

            if(extract < usage) {
                double ratio = (double)extract / (double)usage;
                usage = extract;
                injection = (int)(injection * ratio);
            }

            transferRate = injection;
            energy += injection;
            transferedEnergy = extract > 0;
        }

        if(!transferedEnergy && !isPowered() && CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
            if(!waitTillEmpty) {
                setPowered(true);
            } else if(energyCart.getEnergy() == 0) {
                setPowered(true);
            }
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound nbttagcompound) {
        super.writeToNBT(nbttagcompound);
        nbttagcompound.setBoolean("WaitTillEmpty", waitTillEmpty());
    }

    @Override
    public void readFromNBT(NBTTagCompound nbttagcompound) {
        super.readFromNBT(nbttagcompound);
        setWaitTillEmpty(nbttagcompound.getBoolean("WaitTillEmpty"));
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeBoolean(waitTillEmpty);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        waitTillEmpty = data.readBoolean();
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeBoolean(waitTillEmpty);
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        waitTillEmpty = data.readBoolean();
    }

    public boolean waitTillEmpty() {
        return waitTillEmpty;
    }

    public void setWaitTillEmpty(boolean wait) {
        waitTillEmpty = wait;
    }

    @Override
    public int getMaxEnergyOutput() {
        return OUTPUT_LEVELS[1];
    }

    @Override
    public boolean emitsEnergyTo(TileEntity receiver, Direction direction) {
        if(getFacing() == direction.toSideValue()) {
            return false;
        }
        return true;
    }
}
