package railcraft.common.blocks.machine.gamma;

import ic2.api.IWrenchable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.inventory.Slot;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.carts.CartTools;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.ItemStackMap;
import railcraft.common.util.inventory.ItemStackSet;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

public class TileItemLoader extends TileLoaderItemBase implements IWrenchable {

    private final Map<ItemStack, Short> transferedItems = new ItemStackMap<Short>();
    private final Set<ItemStack> checkedItems = new ItemStackSet();

    public TileItemLoader() {
        super();
        setInventorySize(9);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineGamma.ITEM_LOADER;
    }

    @Override
    public int getBlockTexture(int side) {
        return getMachineType().getTexture(side);
    }

    @Override
    public Slot getBufferSlot(int id, int x, int y) {
        return new Slot(this, id, x, y);
    }

    @Override
    public short getFacing() {
        return (short) getOrientation().ordinal();
    }

    public ForgeDirection getOrientation() {
        return ForgeDirection.DOWN;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();

        if (Game.isNotHost(getWorld())) {
            return;
        }

        boolean movedItemCart = false;

        EntityMinecart cart = CartTools.getMinecartOnSide(worldObj, xCoord, yCoord, zCoord, 0.1f, getOrientation());

        if (cart != currentCart) {
            setPowered(false);
            currentCart = cart;
            transferedItems.clear();
            cartSent();
        }

        if (cart == null) {
            return;
        }

        if (cart.getSizeInventory() == 0) {
            if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY) || cart.isPoweredCart()) {
                setPowered(true);
            }
            return;
        }

        if (isSendCart()) {
            if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                setPowered(true);
            }
            return;
        }

        ItemStack minecartSlot1 = getCartFilters().getStackInSlot(0);
        ItemStack minecartSlot2 = getCartFilters().getStackInSlot(1);
        if (minecartSlot1 != null || minecartSlot2 != null) {
            if (!CartTools.doesCartMatchFilter(minecartSlot1, cart) && !CartTools.doesCartMatchFilter(minecartSlot2, cart)) {
                if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                    setPowered(true);
                }
                return;
            }
        }

        List<IInventory> chests = InvTools.getAdjacentInventories(worldObj, xCoord, yCoord, zCoord);

        Iterator<IInventory> it = chests.iterator();
        while (it.hasNext()) {
            IInventory inv = it.next();
//            if(inv instanceof InventoryMapper) {
//                inv = ((InventoryMapper)inv).getBaseInventory();
//            }
            if (inv instanceof TileItemLoader) {
                it.remove();
            }
        }

        chests.add(0, this);

        checkedItems.clear();

        switch (getMode()) {
            case TRANSFER: {
                boolean hasFilter = false;
                for (ItemStack filter : getItemFilters().getContents()) {
                    if (filter == null) {
                        continue;
                    }
                    if (!checkedItems.add(filter)) {
                        continue;
                    }
                    hasFilter = true;
                    Short numMoved = transferedItems.get(filter);
                    if (numMoved == null) {
                        numMoved = 0;
                    }
                    if (numMoved < InvTools.countItems(getItemFilters(), filter)) {
                        ItemStack moved = InvTools.moveOneItem(chests, cart, filter);
                        if (moved != null) {
                            movedItemCart = true;
                            numMoved++;
                            transferedItems.put(moved, numMoved);
                            break;
                        }
                    }
                }
                if (!hasFilter) {
                    ItemStack moved = InvTools.moveOneItem(chests, cart);
                    if (moved != null) {
                        movedItemCart = true;
                        break;
                    }
                }
                break;
            }
            case STOCK: {
                for (ItemStack filter : getItemFilters().getContents()) {
                    if (filter == null) {
                        continue;
                    }
                    if (!checkedItems.add(filter)) {
                        continue;
                    }
                    int stocked = InvTools.countItems(cart, filter);
                    if (stocked < InvTools.countItems(getItemFilters(), filter)) {
                        ItemStack moved = InvTools.moveOneItem(chests, cart, filter);
                        if (moved != null) {
                            movedItemCart = true;
                            break;
                        }
                    }
                }
                break;
            }
            case EXCESS: {
                for (ItemStack filter : getItemFilters().getContents()) {
                    if (filter == null) {
                        continue;
                    }
                    if (!checkedItems.add(filter)) {
                        continue;
                    }
                    int stocked = InvTools.countItems(chests, filter);
                    if (stocked > InvTools.countItems(getItemFilters(), filter)) {
                        ItemStack moved = InvTools.moveOneItem(chests, cart, filter);
                        if (moved != null) {
                            movedItemCart = true;
                            break;
                        }
                    }
                }
                if (!movedItemCart) {
                    movedItemCart = InvTools.moveOneItemExcept(chests, cart, getItemFilters().getContents()) != null;
                }
                break;
            }
        }

        if (movedItemCart) {
            setPowered(false);
        }

        if (!isPowered() && (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY) || cart.isPoweredCart())) {
            if (!movedItemCart && !cartShouldWait()) {
                setPowered(true);
            } else if (getMode() == EnumItemMode.TRANSFER && isTranferComplete(getItemFilters().getContents())) {
                setPowered(true);
            } else if (getMode() == EnumItemMode.STOCK && isStockComplete(cart, getItemFilters().getContents())) {
                setPowered(true);
            } else if (getMode() == EnumItemMode.EXCESS && isExcessComplete(chests, getItemFilters().getContents())) {
                setPowered(true);
            } else if (!movedItemCart && InvTools.isInventoryFull(cart, MiscTools.getOppositeSide(getFacing()))) {
                setPowered(true);
            }
        }
    }

    private boolean isTranferComplete(ItemStack[] filters) {
        checkedItems.clear();
        boolean hasFilter = false;
        for (ItemStack filter : filters) {
            if (filter == null) {
                continue;
            }
            if (!checkedItems.add(filter)) {
                continue;
            }
            hasFilter = true;
            Short numMoved = transferedItems.get(filter);
            if (numMoved == null || numMoved < InvTools.countItems(getItemFilters(), filter)) {
                return false;
            }
        }
        return hasFilter;
    }

    private boolean isStockComplete(EntityMinecart cart, ItemStack[] filters) {
        checkedItems.clear();
        for (ItemStack filter : filters) {
            if (filter == null) {
                continue;
            }
            if (!checkedItems.add(filter)) {
                continue;
            }
            int stocked = InvTools.countItems(cart, filter);
            if (stocked < InvTools.countItems(getItemFilters(), filter)) {
                return false;
            }
        }
        return true;
    }

    private boolean isExcessComplete(List<IInventory> chests, ItemStack[] filters) {
        checkedItems.clear();
        int max = 0;
        for (ItemStack filter : filters) {
            if (filter == null) {
                continue;
            }
            if (!checkedItems.add(filter)) {
                continue;
            }
            int stocked = InvTools.countItems(chests, filter);
            max += filter.stackSize;
            if (stocked > InvTools.countItems(getItemFilters(), filter)) {
                return false;
            }
        }
        if (InvTools.countItems(chests) > max) {
            return false;
        }
        return true;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.LOADER_ITEM, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }
}
