package railcraft.common.blocks.machine.gamma;

import buildcraft.api.tools.IToolWrench;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.util.misc.MiscTools;

public class TileItemUnloaderAdvanced extends TileItemUnloader {

    private ForgeDirection direction = ForgeDirection.NORTH;

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineGamma.ITEM_UNLOADER_ADVANCED;
    }

    @Override
    public void writeToNBT(NBTTagCompound nbttagcompound) {
        super.writeToNBT(nbttagcompound);

        nbttagcompound.setByte("direction", (byte) direction.ordinal());
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        direction = ForgeDirection.getOrientation(data.getByte("direction"));
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeByte(direction.ordinal());
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        direction = ForgeDirection.getOrientation(data.readByte());
    }

    @Override
    public int getBlockTexture(int side) {
        if (direction.ordinal() == side) {
            return getMachineType().getTexture(3);
        }
        if (side != 0 && side != 1) {
            return getMachineType().getTexture(2);
        }
        return getMachineType().getTexture(1);
    }

    @Override
    public void onBlockPlacedBy(EntityLiving entityliving) {
        super.onBlockPlacedBy(entityliving);
        direction = MiscTools.getSideClosestToPlayer(worldObj, xCoord, yCoord, zCoord, entityliving);
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null && current.getItem() instanceof IToolWrench) {
            IToolWrench wrench = (IToolWrench) current.getItem();
            if (wrench.canWrench(player, xCoord, yCoord, zCoord)) {
                if (direction.ordinal() == side) {
                    direction = MiscTools.getOppositeSide(side);
                } else {
                    direction = ForgeDirection.getOrientation(side);
                }
                wrench.wrenchUsed(player, xCoord, yCoord, zCoord);
                markBlockForUpdate();
                return true;
            }
        }
        return super.blockActivated(player, side);
    }

    @Override
    public boolean wrenchCanSetFacing(EntityPlayer player, int side) {
        if (direction.ordinal() == side) {
            return false;
        }
        return true;
    }

    @Override
    public void setFacing(short facing) {
        direction = ForgeDirection.getOrientation(facing);
        sendUpdateToClient();
    }

    @Override
    public short getFacing() {
        return (short) direction.ordinal();
    }

    @Override
    public ForgeDirection getOrientation() {
        return direction;
    }
}
