package railcraft.common.blocks.machine.gamma;

import buildcraft.api.inventory.ISpecialInventory;
import buildcraft.api.transport.IPipe;
import buildcraft.api.transport.PipeManager;
import ic2.api.IWrenchable;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.carts.ILiquidTransfer;
import railcraft.common.api.tracks.ITrackInstance;
import railcraft.common.api.tracks.ITrackLockdown;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.blocks.tracks.TileTrack;
import railcraft.common.carts.EntityCartTank;
import railcraft.common.carts.EntityLocomotiveSteam;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankToolkit;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.misc.SafeNBTWrapper;
import railcraft.common.util.network.IGuiReturnHandler;

public class TileLiquidLoader extends TileLoaderLiquidBase implements IWrenchable, ISidedInventory, ISpecialInventory, IGuiReturnHandler {

    private static final int TRANSFER_RATE = 20;
    private static final int CAPACITY = LiquidManager.BUCKET_VOLUME * 8;
    private static final float MAX_PIPE_LENGTH = 16 * 0.0625f;
    private static final float PIPE_INCREMENT = 0.01f;
    private static final int SLOT_INPUT = 0;
    private static final int SLOT_OUTPUT = 1;
    private static final int RESET_TICKS = 10;
    private boolean waitTillFull = true;
    private boolean waitIfEmpty = true;
    private boolean waitForReset = false;
    private float pipeLenght = 0;
    private final IInventory invInput = new InventoryMapper(this, SLOT_INPUT, 1);
    private StandardTank tank;

    public TileLiquidLoader() {
        super();
        tank = new StandardTank(CAPACITY, this);
        tank.setTankPressure(-1);
        tankManager.addTank(tank);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineGamma.LIQUID_LOADER;
    }

    public IInventory getInputInventory() {
        return invInput;
    }

    private void resetPipe() {
        pipeLenght = 0;

    }

    public float getPipeLenght() {
        return pipeLenght;
    }

    private void setPipeLength(float y) {
        pipeLenght = y;
        sendUpdateToClient();
    }

    private void extendPipe() {
        float y = pipeLenght + PIPE_INCREMENT;
        if (pipeIsExtended()) {
            y = MAX_PIPE_LENGTH;
        }
        setPipeLength(y);
    }

    private void retractPipe() {
        float y = pipeLenght - PIPE_INCREMENT;
        if (pipeIsRetracted()) {
            y = 0;
        }
        setPipeLength(y);
    }

    private boolean pipeIsExtended() {
        if (pipeLenght >= MAX_PIPE_LENGTH) {
            return true;
        }
        return false;
    }

    private boolean pipeIsRetracted() {
        if (pipeLenght <= 0) {
            return true;
        }
        return false;
    }

    @Override
    public int getBlockTexture(int side) {
        if (side > 1) {
            return 24;
        }
        return getMachineType().getTexture(side);
    }

    @Override
    public void updateEntity() {
        super.updateEntity();
        if (Game.isNotHost(getWorld())) {
            return;
        }

        ItemStack topSlot = getStackInSlot(SLOT_INPUT);
        if (topSlot != null && !LiquidManager.getInstance().isContainer(topSlot)) {
            setInventorySlotContents(SLOT_INPUT, null);
            dropItem(topSlot);
        }

        ItemStack bottomSlot = getStackInSlot(SLOT_OUTPUT);
        if (bottomSlot != null && !LiquidManager.getInstance().isContainer(bottomSlot)) {
            setInventorySlotContents(SLOT_OUTPUT, null);
            dropItem(bottomSlot);
        }

        if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
            LiquidManager.getInstance().processContainers(tankManager.getTank(0), this, SLOT_INPUT, SLOT_OUTPUT, true, false);
        }

        for (ForgeDirection side : ForgeDirection.values()) {
            if (side == ForgeDirection.UNKNOWN) {
                continue;
            }
            TileEntity tile = MiscTools.getBlockTileEntityOnSide(worldObj, xCoord, yCoord, zCoord, side);
            if (tile instanceof ITankContainer) {
                ITankContainer nearbyTank = (ITankContainer) tile;
                if (PipeManager.canExtractLiquids((IPipe) null, worldObj, MiscTools.getXOnSide(xCoord, side), MiscTools.getYOnSide(yCoord, side), MiscTools.getZOnSide(zCoord, side))) {
                    side = side.getOpposite();
                    LiquidStack drained = nearbyTank.drain(side, TRANSFER_RATE, false);
                    int used = tank.fill(drained, true);
                    nearbyTank.drain(side, used, true);
                }
            }
        }

        boolean needsPipe = false;

        EntityMinecart cart = CartTools.getMinecartOnSide(worldObj, xCoord, yCoord, zCoord, 0.2f, ForgeDirection.DOWN);
        if (cart == null) {
            cart = CartTools.getMinecartOnSide(worldObj, xCoord, yCoord - 1, zCoord, 0.2f, ForgeDirection.DOWN);
            needsPipe = true;
        }


        if (cart != currentCart) {
            if (currentCart instanceof ILiquidTransfer) {
                ((EntityCartTank) currentCart).setFilling(false);
            }
            setPowered(false);
            currentCart = cart;
            cartSent();
            waitForReset = false;
        }

        if (waitForReset) {
            if (pipeIsRetracted()) {
                sendCart(cart);
            } else {
                retractPipe();
            }
            return;
        }

        if (cart == null) {
            if (!pipeIsRetracted()) {
                retractPipe();
            }
            return;
        }

        if (isSendCart()) {
            sendCart(cart);
            return;
        }

        if (!(cart instanceof ITankContainer)) {
            sendCart(cart);
            return;
        }

        ItemStack minecartSlot1 = getCartFilters().getStackInSlot(0);
        ItemStack minecartSlot2 = getCartFilters().getStackInSlot(1);
        if (minecartSlot1 != null || minecartSlot2 != null) {
            if (!CartTools.doesCartMatchFilter(minecartSlot1, cart) && !CartTools.doesCartMatchFilter(minecartSlot2, cart)) {
                sendCart(cart);
                return;
            }
        }

        if (cart instanceof EntityLocomotiveSteam) {
            EntityLocomotiveSteam loco = (EntityLocomotiveSteam) cart;
            if (!loco.isSafeToFill()) {
                retractPipe();
                return;
            }
        }

        TankToolkit tankCart = new TankToolkit((ITankContainer) cart);
        boolean cartNeedsFilling = cartNeedsFilling(tankCart);

        if (cartNeedsFilling && needsPipe) {
            extendPipe();
        } else {
            retractPipe();
        }

        int flow = 0;
        if (cartNeedsFilling && (!needsPipe || pipeIsExtended())) {
            LiquidStack drained = tankManager.drain(0, RailcraftConfig.getTankCartFillRate(), false);
            if (drained != null) {
                flow = tankCart.fill(ForgeDirection.UP, drained, true);
                tankManager.drain(0, flow, true);
            }
        }

        if (flow > 0) {
            if (cart instanceof ILiquidTransfer) {
                ((EntityCartTank) cart).setFilling(true);
            }
            setPowered(false);
        } else {
            if (cart instanceof ILiquidTransfer) {
                ((EntityCartTank) cart).setFilling(false);
            }
        }

        if (tankCart.isTankFull(tank.getLiquid())) {
            waitForReset = true;
        }

        if (pipeIsRetracted() && flow <= 0 && shouldCartLeave(cart, tankCart)) {
            sendCart(cart);
        }
    }

    private void sendCart(EntityMinecart cart) {
        if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
            setPowered(true);
        }
    }

    private boolean cartNeedsFilling(TankToolkit tankCart) {
        LiquidStack loaderLiquid = tank.getLiquid();
        return loaderLiquid != null && loaderLiquid.amount > 0 && tankCart.canPutLiquid(ForgeDirection.UP, loaderLiquid);
    }

    private boolean shouldCartLeave(EntityMinecart cart, TankToolkit tankCart) {
        boolean leave = false;
        if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
            if (tank.getLiquid() != null && tank.getLiquid().amount > 0 && !tankCart.canPutLiquid(ForgeDirection.UP, tank.getLiquid())) {
                leave = true;
            } else if (!waitTillFull && !tankCart.isTankEmpty(tank.getLiquid())) {
                leave = true;
            } else if (!waitIfEmpty && !waitTillFull && tankCart.isTankEmpty(tank.getLiquid())) {
                leave = true;
            } else if (tankCart.isTankFull(tank.getLiquid())) {
                leave = true;
            }
        }
        return leave;
    }

    @Override
    protected void setPowered(boolean p) {
        if (p) {
            resetPipe();
            if (worldObj != null) {
                TileEntity tile = worldObj.getBlockTileEntity(xCoord, yCoord - 2, zCoord);
                if (tile instanceof TileTrack) {
                    TileTrack trackTile = (TileTrack) tile;
                    ITrackInstance track = trackTile.getTrackInstance();
                    if (track instanceof ITrackLockdown) {
                        ((ITrackLockdown) track).releaseCart();
                    }
                }
            }
        }
        super.setPowered(p);
    }

    @Override
    public void invalidate() {
        super.invalidate();
        resetPipe();
    }

    @Override
    public void onBlockRemoval() {
        super.onBlockRemoval();
        resetPipe();
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setBoolean("WaitIfEmpty", waitIfEmpty);
        data.setBoolean("WaitTillFull", waitTillFull);

        data.setFloat("pipeLenght", pipeLenght);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        waitIfEmpty = data.getBoolean("WaitIfEmpty");
        waitTillFull = data.getBoolean("WaitTillFull");

        SafeNBTWrapper safe = new SafeNBTWrapper(data);
        pipeLenght = safe.getFloat("pipeLenght");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        byte bits = 0;
        bits |= waitIfEmpty ? 1 : 0;
        bits |= waitTillFull ? 2 : 0;
        data.writeByte(bits);

        data.writeFloat(pipeLenght);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        byte bits = data.readByte();
        waitIfEmpty = (bits & 1) != 0;
        waitTillFull = (bits & 2) != 0;

        setPipeLength(data.readFloat());
    }

    public boolean waitTillFull() {
        return waitTillFull;
    }

    public void setWaitTillFull(boolean waitTillFull) {
        this.waitTillFull = waitTillFull;
    }

    public boolean waitIfEmpty() {
        return waitIfEmpty;
    }

    public void setWaitIfEmpty(boolean waitIfEmpty) {
        this.waitIfEmpty = waitIfEmpty;
    }

    @Override
    public short getFacing() {
        return (short) ForgeDirection.DOWN.ordinal();
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.LOADER_LIQUID, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        if (side == ForgeDirection.UP) {
            return SLOT_INPUT;
        }
        return SLOT_OUTPUT;
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 1;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (LiquidManager.getInstance().isFilledContainer(stack)) {
            IInventory slot = new InventoryMapper(this, SLOT_INPUT, 1);
            if (!doAdd) {
                slot = new InventoryCopy(slot);
            }
            ItemStack ret = InvTools.moveItemStack(stack.copy(), slot);
            if (ret != null && stack.stackSize != ret.stackSize) {
                return stack.stackSize - ret.stackSize;
            } else if (ret == null) {
                return stack.stackSize;
            }
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory inv = this;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return new ItemStack[]{inv.decrStackSize(SLOT_OUTPUT, maxItemCount)};
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeBoolean(waitIfEmpty);
        data.writeBoolean(waitTillFull);
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        waitIfEmpty = data.readBoolean();
        waitTillFull = data.readBoolean();
    }
}
