package railcraft.common.blocks.machine.gamma;

import buildcraft.api.inventory.ISpecialInventory;
import buildcraft.api.transport.IPipe;
import buildcraft.api.transport.IPipeTile;
import ic2.api.IWrenchable;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.api.carts.CartTools;
import railcraft.common.blocks.machine.IEnumMachine;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.inventory.PhantomInventory;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankToolkit;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.network.IGuiReturnHandler;

public class TileLiquidUnloader extends TileLoaderLiquidBase implements IWrenchable, ISidedInventory, ISpecialInventory, ITankContainer, IGuiReturnHandler {

    private static final int CAPACITY = LiquidManager.BUCKET_VOLUME * 8;
    private static final int TRANSFER_RATE = 80;
    private static final int SLOT_INPUT = 0;
    private static final int SLOT_OUTPUT = 1;
    private boolean waitTillEmpty = true;
    private final PhantomInventory invFilter = new PhantomInventory(1);

    public TileLiquidUnloader() {
        super();
        StandardTank tank = new StandardTank(CAPACITY, this);
        tank.setTankPressure(1);
        tankManager.addTank(tank);
    }

    @Override
    public IEnumMachine getMachineType() {
        return EnumMachineGamma.LIQUID_UNLOADER;
    }

    public PhantomInventory getLiquidFilter() {
        return invFilter;
    }

    @Override
    public int getBlockTexture(int side) {
        if (side > 1) {
            return 24;
        }
        return getMachineType().getTexture(side);
    }

    public LiquidStack getFilterLiquid() {
        if (invFilter.getStackInSlot(0) != null) {
            return LiquidManager.getInstance().getLiquidInContainer(invFilter.getStackInSlot(0));
        }
        return null;
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        
        return super.blockActivated(player, side);
    }

    @Override
    public void updateEntity() {
        super.updateEntity();
        if (Game.isNotHost(getWorld())) {
            return;
        }

        ItemStack topSlot = getStackInSlot(SLOT_INPUT);
        if (topSlot != null && !LiquidManager.getInstance().isContainer(topSlot)) {
            setInventorySlotContents(SLOT_INPUT, null);
            dropItem(topSlot);
        }

        ItemStack bottomSlot = getStackInSlot(SLOT_OUTPUT);
        if (bottomSlot != null && !LiquidManager.getInstance().isContainer(bottomSlot)) {
            setInventorySlotContents(SLOT_OUTPUT, null);
            dropItem(bottomSlot);
        }

        if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
            LiquidManager.getInstance().processContainers(tankManager.getTank(0), this, SLOT_INPUT, SLOT_OUTPUT, false, true);
        }


        for (ForgeDirection side : ForgeDirection.VALID_DIRECTIONS) {
            if (ForgeDirection.UP == side) {
                continue;
            }
            TileEntity tile = MiscTools.getBlockTileEntityOnSide(worldObj, xCoord, yCoord, zCoord, side);
            ITankContainer nearbyTank = null;
            side = side.getOpposite();
            if (tile instanceof ITankContainer) {
                nearbyTank = (ITankContainer) tile;
            } else if (tile instanceof IPipeTile) {
                IPipe pipe = ((IPipeTile) tile).getPipe();
                if (pipe instanceof ITankContainer) {
                    nearbyTank = (ITankContainer) pipe;
                }
            }
            if (nearbyTank != null) {
                LiquidStack available = tankManager.getTank(0).drain(TRANSFER_RATE, false);
                if (available != null) {
                    int used = nearbyTank.fill(side, available, true);
                    tankManager.getTank(0).drain(used, true);
                }
            }
        }

        EntityMinecart cart = CartTools.getMinecartOnSide(worldObj, xCoord, yCoord, zCoord, 0.1f, ForgeDirection.UP);

        if (cart != currentCart) {
            setPowered(false);
            currentCart = cart;
            cartSent();
        }

        if (cart == null) {
            return;
        }

        if (!(cart instanceof ITankContainer)) {
            if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                setPowered(true);
            }
            return;
        }

        if (isSendCart()) {
            if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                setPowered(true);
            }
            return;
        }

        ItemStack minecartSlot1 = getCartFilters().getStackInSlot(0);
        ItemStack minecartSlot2 = getCartFilters().getStackInSlot(1);
        if (minecartSlot1 != null || minecartSlot2 != null) {
            if (!CartTools.doesCartMatchFilter(minecartSlot1, cart) && !CartTools.doesCartMatchFilter(minecartSlot2, cart)) {
                if (CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
                    setPowered(true);
                }
                return;
            }
        }

        TankToolkit tankCart = new TankToolkit((ITankContainer) cart);

        int flow = 0;
        LiquidStack drained = tankCart.drain(ForgeDirection.DOWN, RailcraftConfig.getTankCartFillRate(), false);
        if (getFilterLiquid() == null || getFilterLiquid().isLiquidEqual(drained)) {
            flow = tankManager.getTank(0).fill(drained, true);
            tankCart.drain(ForgeDirection.DOWN, flow, true);
        }

        if (flow > 0) {
            setPowered(false);
        }

        if (flow <= 0 && !isPowered() && CartTools.cartVelocityIsLessThan(cart, STOP_VELOCITY)) {
            if (!waitTillEmpty()) {
                setPowered(true);
            } else if (getFilterLiquid() != null && tankCart.isTankEmpty(getFilterLiquid())) {
                setPowered(true);
            } else if (tankCart.areTanksEmpty()) {
                setPowered(true);
            }
        }
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        return 0;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setBoolean("waitTillEmpty", waitTillEmpty);

        getLiquidFilter().writeToNBT("invFilter", data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        waitTillEmpty = data.getBoolean("waitTillEmpty");

        if (data.hasKey("filter")) {
            NBTTagCompound filter = data.getCompoundTag("filter");
            getLiquidFilter().readFromNBT("Items", filter);
        } else {
            getLiquidFilter().readFromNBT("invFilter", data);
        }
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeBoolean(waitTillEmpty);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        waitTillEmpty = data.readBoolean();
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeBoolean(waitTillEmpty);
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        waitTillEmpty = data.readBoolean();
    }

    public boolean waitTillEmpty() {
        return waitTillEmpty;
    }

    public void setWaitTillEmpty(boolean wait) {
        waitTillEmpty = wait;
    }

    @Override
    public short getFacing() {
        return (short) ForgeDirection.UP.ordinal();
    }

    public ForgeDirection getOrientation() {
        return ForgeDirection.UP;
    }

    @Override
    public boolean openGui(EntityPlayer player) {
        GuiHandler.openGui(EnumGui.UNLOADER_LIQUID, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        if (side == ForgeDirection.DOWN) {
            return SLOT_OUTPUT;
        }
        return SLOT_INPUT;
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 1;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (LiquidManager.getInstance().isEmptyContainer(stack)) {
            IInventory slot = new InventoryMapper(this, SLOT_INPUT, 1);
            if (!doAdd) {
                slot = new InventoryCopy(slot);
            }
            ItemStack ret = InvTools.moveItemStack(stack.copy(), slot);
            if (ret != null && stack.stackSize != ret.stackSize) {
                return stack.stackSize - ret.stackSize;
            } else if (ret == null) {
                return stack.stackSize;
            }
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory inv = this;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return new ItemStack[]{inv.decrStackSize(SLOT_OUTPUT, maxItemCount)};
    }
}
