package railcraft.common.blocks.machine.gamma;

import buildcraft.api.gates.IAction;
import ic2.api.IWrenchable;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.blocks.machine.TileMachineItem;
import railcraft.common.plugins.buildcraft.actions.EnumTriggerActions;
import railcraft.common.plugins.buildcraft.triggers.IHasCart;
import railcraft.common.util.inventory.PhantomInventory;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileLoaderBase extends TileMachineItem implements IHasCart, IWrenchable {

    public static final float STOP_VELOCITY = 0.02f;
    private boolean powered;
    protected EntityMinecart currentCart;
    private boolean sendCart = false;
    private final PhantomInventory invCarts = new PhantomInventory(2, this);

    @Override
    public boolean hasMinecart() {
        return currentCart != null;
    }

    public final PhantomInventory getCartFilters() {
        return invCarts;
    }

    @Override
    public void actionActivated(IAction action) {
        if (action == EnumTriggerActions.SEND_CART.getAction()) {
            sendCart = true;
        }
    }

    public boolean isSendCart() {
        return sendCart;
    }

    public void cartSent() {
        sendCart = false;
    }

    @Override
    public final boolean canUpdate() {
        return true;
    }

    @Override
    public final boolean canConnectRedstone(int dir) {
        return true;
    }

    @Override
    public final boolean isPoweringTo(int side) {
        if (!isPowered()) {
            return false;
        }
        ForgeDirection opSide = MiscTools.getOppositeSide(side);
        int id = MiscTools.getBlockIdOnSide(worldObj, xCoord, yCoord, zCoord, opSide);
        if (BlockRail.isRailBlock(id) || id == Block.redstoneWire.blockID || id == Block.redstoneRepeaterActive.blockID || id == Block.redstoneRepeaterIdle.blockID) {
            return true;
        }

        return false;
    }

    public final boolean isPowered() {
        return powered;
    }

    protected void setPowered(boolean p) {
        if (powered != p) {
            powered = p;
            int i = xCoord;
            int j = yCoord;
            int k = zCoord;
            if (worldObj != null) {
                int blockID = getBlockId();
                worldObj.notifyBlocksOfNeighborChange(i, j, k, blockID);
//                worldObj.notifyBlocksOfNeighborChange(i + 1, j, k, blockID);
//                worldObj.notifyBlocksOfNeighborChange(i - 1, j, k, blockID);
//                worldObj.notifyBlocksOfNeighborChange(i, j, k + 1, blockID);
//                worldObj.notifyBlocksOfNeighborChange(i, j, k - 1, blockID);
//                worldObj.notifyBlocksOfNeighborChange(i, j - 1, k, blockID);
//                worldObj.notifyBlocksOfNeighborChange(i, j + 1, k, blockID);
            }
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setBoolean("powered", powered);

        getCartFilters().writeToNBT("invCarts", data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        setPowered(data.getBoolean("powered"));

        getCartFilters().readFromNBT("invCarts", data);
    }

    @Override
    public boolean wrenchCanSetFacing(EntityPlayer entityPlayer, int side) {
        return false;
    }

    @Override
    public void setFacing(short facing) {
    }

    @Override
    public boolean wrenchCanRemove(EntityPlayer entityPlayer) {
        return true;
    }

    @Override
    public float getWrenchDropRate() {
        return 1;
    }

    @Override
    public ItemStack getWrenchDrop(EntityPlayer entityPlayer) {
        return getMachineType().getItem();
    }
}
