package railcraft.common.blocks.machine.gamma;

import buildcraft.api.tools.IToolWrench;
import ic2.api.ElectricItem;
import ic2.api.IElectricItem;
import ic2.api.IWrenchable;
import ic2.api.energy.tile.IEnergyTile;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import railcraft.common.modules.ModuleIC2;
import railcraft.common.plugins.ic2.IC2Plugin;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.IEnergyDevice;
import railcraft.common.util.misc.MiscTools;

public abstract class TileLoaderEnergyBase extends TileLoaderBase implements IWrenchable, IEnergyDevice, ISidedInventory, IEnergyTile {

    private static final int SLOT_CHARGE = 0;
    private static final int SLOT_BATTERY = 1;
    private static final int TIER = 2;
    private static final int CAPACITY = 100000;
    private static final int MAX_OVERCLOCKS = 10;
    private static final int MAX_LAPOTRON = 9;
    protected int energy;
    private int prevEnergy;
    protected int transferRate;
    protected short storageUpgrades;
    protected short lapotronUpgrades;
    protected short transformerUpgrades;
    protected short overclockerUpgrades;
    protected ForgeDirection direction = ForgeDirection.NORTH;
    protected int update;
    private boolean addedToIC2EnergyNet;

    public TileLoaderEnergyBase() {
        super();
        setInventorySize(6);
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 1;
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        if (side == ForgeDirection.UP) {
            return 0;
        }
        return 1;
    }

    @Override
    public void onBlockPlacedBy(EntityLiving entityliving) {
        super.onBlockPlacedBy(entityliving);
        if (Game.isNotHost(getWorld())) {
            return;
        }
        direction = MiscTools.getSideClosestToPlayer(worldObj, xCoord, yCoord, zCoord, entityliving);
    }

    protected void countUpgrades() {
        ItemStack storage = IC2Plugin.getItem("energyStorageUpgrade");
        ItemStack overclocker = IC2Plugin.getItem("overclockerUpgrade");
        ItemStack transformer = IC2Plugin.getItem("transformerUpgrade");
        Item lapotron = ModuleIC2.lapotronUpgrade;

        storageUpgrades = 0;
        overclockerUpgrades = 0;
        transformerUpgrades = 0;
        lapotronUpgrades = 0;

        for (int i = 2; i < 6; i++) {
            ItemStack stack = getStackInSlot(i);
            if (stack != null) {
                if (storage != null && stack.isItemEqual(storage)) {
                    storageUpgrades += stack.stackSize;
                } else if (overclocker != null && stack.isItemEqual(overclocker)) {
                    overclockerUpgrades += stack.stackSize;
                } else if (transformer != null && stack.isItemEqual(transformer)) {
                    transformerUpgrades += stack.stackSize;
                } else if (lapotron != null && stack.itemID == lapotron.itemID) {
                    lapotronUpgrades += stack.stackSize;
                }
            }
        }
        if (overclockerUpgrades > MAX_OVERCLOCKS) {
            overclockerUpgrades = MAX_OVERCLOCKS;
        }
        if (lapotronUpgrades > MAX_LAPOTRON) {
            lapotronUpgrades = MAX_LAPOTRON;
        }
    }

    @Override
    public void onInventoryChanged() {
        super.onInventoryChanged();
        countUpgrades();
    }

    @Override
    public void updateEntity() {
        super.updateEntity();
        update++;

        if (Game.isNotHost(getWorld())) {
            return;
        }

        if (!addedToIC2EnergyNet) {
            IC2Plugin.addTileToNet(this);
            addedToIC2EnergyNet = true;
        }

        int capacity = getCapacity();
        if (energy > capacity) {
            energy = capacity;
        }

        if (Game.isHost(getWorld()) && prevEnergy != energy && update % 4 == 0) {
            prevEnergy = energy;
            markBlockForUpdate();
        }

        ItemStack charge = getStackInSlot(SLOT_CHARGE);
        if (charge != null && charge.getItem() instanceof IElectricItem && energy > 0) {
            energy -= ElectricItem.charge(charge, energy, getTier(), false, false);
        }

        ItemStack battery = getStackInSlot(SLOT_BATTERY);
        if (battery != null && battery.getItem() instanceof IElectricItem && ((IElectricItem) battery.getItem()).canProvideEnergy() && energy < capacity) {
            energy += ElectricItem.discharge(battery, capacity - energy, getTier(), false, false);
        }

        if (update % 64 == 0) {
            countUpgrades();
        }
    }

    @Override
    public void onChunkUnload() {
        super.onChunkUnload();

        if (addedToIC2EnergyNet) {
            IC2Plugin.removeTileFromNet(this);
        }
    }

    @Override
    public boolean isAddedToEnergyNet() {
        return addedToIC2EnergyNet;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setInteger("energy", energy);
        data.setByte("direction", (byte) direction.ordinal());
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        energy = data.getInteger("energy");
        direction = ForgeDirection.getOrientation(data.getByte("direction"));

        countUpgrades();

        update = MiscTools.getRand().nextInt();
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeByte(getFacing());
        data.writeShort(storageUpgrades);
        data.writeShort(lapotronUpgrades);
        data.writeInt(energy);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        setFacing(data.readByte());
        storageUpgrades = data.readShort();
        lapotronUpgrades = data.readShort();
        energy = data.readInt();
    }

    @Override
    public boolean blockActivated(EntityPlayer player, int side) {
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null && current.getItem() instanceof IToolWrench) {
            IToolWrench wrench = (IToolWrench) current.getItem();
            if (wrench.canWrench(player, xCoord, yCoord, zCoord)) {
                if (direction.ordinal() == side) {
                    direction = MiscTools.getOppositeSide(side);
                } else {
                    direction = ForgeDirection.getOrientation(side);
                }
                wrench.wrenchUsed(player, xCoord, yCoord, zCoord);
                markBlockForUpdate();
                return true;
            }
        }
        return super.blockActivated(player, side);
    }

    @Override
    public boolean wrenchCanSetFacing(EntityPlayer entityPlayer, int side) {
        if (direction.ordinal() == side) {
            return false;
        }
        return true;
    }

    @Override
    public short getFacing() {
        return (short) direction.ordinal();
    }

    @Override
    public void setFacing(short side) {
        if (direction.ordinal() != side) {
            direction = ForgeDirection.getOrientation(side);
            sendUpdateToClient();
        }
    }

    @Override
    public boolean wrenchCanRemove(EntityPlayer entityPlayer) {
        return true;
    }

    @Override
    public float getWrenchDropRate() {
        return 1;
    }

    @Override
    public ItemStack getWrenchDrop(EntityPlayer entityPlayer) {
        return getMachineType().getItem();
    }

    @Override
    public int getEnergy() {
        return energy;
    }

    @Override
    public int getCapacity() {
        int capacity = CAPACITY;
        capacity += storageUpgrades * 10000;
        capacity += lapotronUpgrades * 1000000;
        return capacity;
    }

    public int getTier() {
        return TIER + transformerUpgrades;
    }

    public int getTransferRate() {
        return transferRate;
    }

    @Override
    public int getEnergyBarScaled(int scale) {
        return (getEnergy() * scale) / getCapacity();
    }
}
