package railcraft.common.blocks.machine.gamma;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.inventory.Slot;
import railcraft.common.util.inventory.PhantomInventory;
import railcraft.common.util.misc.SafeNBTWrapper;
import railcraft.common.util.network.IGuiReturnHandler;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TileLoaderItemBase extends TileLoaderBase implements IGuiReturnHandler
{

    private EnumItemMode mode = EnumItemMode.TRANSFER;
    private final PhantomInventory invFilters = new PhantomInventory(9, this);
    private boolean cartShouldWait = false;

    public enum EnumItemMode
    {

        TRANSFER, STOCK, EXCESS
    }

    public final PhantomInventory getItemFilters() {
        return invFilters;
    }

    public abstract Slot getBufferSlot(int id, int x, int y);

    public final boolean cartShouldWait() {
        return cartShouldWait;
    }

    public final void setCartShouldWait(boolean wait) {
        this.cartShouldWait = wait;
    }

    public final EnumItemMode getMode() {
        return mode;
    }

    public final void setMode(EnumItemMode mode) {
        this.mode = mode;
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeBoolean(cartShouldWait);
        data.writeByte(mode.ordinal());
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        cartShouldWait = data.readBoolean();
        mode = EnumItemMode.values()[data.readByte()];
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeBoolean(cartShouldWait);
        data.writeByte(mode.ordinal());
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        cartShouldWait = data.readBoolean();
        mode = EnumItemMode.values()[data.readByte()];
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setBoolean("waitTillComplete", cartShouldWait);
        data.setByte("mode", (byte)mode.ordinal());

        getItemFilters().writeToNBT("invFilters", data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        SafeNBTWrapper safe = new SafeNBTWrapper(data);
        cartShouldWait = data.getBoolean("waitTillComplete");
        mode = EnumItemMode.values()[safe.getByte("mode")];

        if(data.hasKey("filters")) {
            NBTTagCompound filters = data.getCompoundTag("filters");
            getItemFilters().readFromNBT("Items", filters);
        } else {
            getItemFilters().readFromNBT("invFilters", data);
        }
    }
}
