package railcraft.common.blocks.ore;

import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.ItemStack;
import net.minecraft.block.material.Material;
import net.minecraft.client.particle.EffectRenderer;
import net.minecraft.client.particle.EntityDiggingFX;
import net.minecraft.util.MathHelper;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.oredict.OreDictionary;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forestry.ForestryPlugin;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class BlockOre extends Block {

    public static int renderPass;
    private final int renderType;
    private final Random rand = new Random();
    private static BlockOre instance;

    public static BlockOre getBlock() {
        return instance;
    }

    public static void registerBlock() {
        if (instance == null) {
            int id = RailcraftConfig.getBlockId("block.ore");

            if (id > 0) {
                int renderId = Railcraft.getProxy().getRenderId();
                instance = new BlockOre(id, renderId);
                GameRegistry.registerBlock(instance, ItemOre.class, instance.getBlockName());

                for (EnumOre ore : EnumOre.values()) {
                    RailcraftLanguage.getInstance().registerItemName(ore.getItem(), ore.getTag());
                    ForestryPlugin.addBackpackItem("miner", ore.getItem());
                }

                OreDictionary.registerOre("oreSulfur", EnumOre.SULFUR.getItem());
                OreDictionary.registerOre("oreSaltpeter", EnumOre.SALTPETER.getItem());
            }
        }
    }

    public BlockOre(int id, int renderId) {
        super(id, Material.rock);
        renderType = renderId;
        setBlockName("rcOres");
        setResistance(5);
        setHardness(3);
        setTextureFile(RailcraftConstants.MAIN_TEXTURE_FILE);
        setStepSound(Block.soundStoneFootstep);
        setCreativeTab(CreativeTabs.tabBlock);

        MinecraftForge.setBlockHarvestLevel(this, "pickaxe", 2);
    }

    @Override
    public int getRenderType() {
        return renderType;
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        for (EnumOre ore : EnumOre.values()) {
            list.add(ore.getItem());
        }
    }

    @Override
    public ItemStack getPickBlock(MovingObjectPosition target, World world, int x, int y, int z) {
        int meta = world.getBlockMetadata(x, y, z);
        return EnumOre.fromMeta(meta).getItem();
    }

    @Override
    public void dropBlockAsItemWithChance(World world, int x, int y, int z, int par5, float par6, int par7) {
        super.dropBlockAsItemWithChance(world, x, y, z, par5, par6, par7);

        int xp = MathHelper.getRandomIntegerInRange(world.rand, 2, 5);
        this.dropXpOnBlockBreak(world, x, y, z, xp);
    }

    @Override
    public ArrayList<ItemStack> getBlockDropped(World world, int x, int y, int z, int meta, int fortune) {
        switch (EnumOre.fromMeta(meta)) {
            case SULFUR: {
                ArrayList<ItemStack> list = new ArrayList<ItemStack>();
                int qty = 2 + rand.nextInt(4) + rand.nextInt(fortune + 1);
                list.add(RailcraftPartItems.getDustSulfur(qty));
                return list;
            }
            case SALTPETER: {
                ArrayList<ItemStack> list = new ArrayList<ItemStack>();
                int qty = 1 + rand.nextInt(2) + rand.nextInt(fortune + 1);
                list.add(RailcraftPartItems.getDustSaltpeter(qty));
                return list;
            }
            default:
                return super.getBlockDropped(world, x, y, z, meta, fortune);
        }
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        if (renderPass == 0) {
            if (meta == EnumOre.SALTPETER.ordinal()) {
                return Block.sandStone.getBlockTextureFromSideAndMetadata(ForgeDirection.DOWN.ordinal(), 0);
            } else {
                return Block.stone.getBlockTextureFromSideAndMetadata(side, 0);
            }
        }
        return EnumOre.fromMeta(meta).getTexture(side);
    }

    @Override
    public void setBlockBoundsForItemRender() {
        setBlockBounds(0, 0, 0, 1, 1, 1);
    }

    /**
     * Spawn a digging particle effect in the world, this is a wrapper around
     * EffectRenderer.addBlockHitEffects to allow the block more control over
     * the particles. Useful when you have entirely different texture sheets for
     * different sides/locations in the world.
     *
     * @param world The current world
     * @param target The target the player is looking at {x/y/z/side/sub}
     * @param effectRenderer A reference to the current effect renderer.
     * @return True to prevent vanilla digging particles form spawning.
     */
    @SideOnly(Side.CLIENT)
    @Override
    public boolean addBlockHitEffects(World worldObj, MovingObjectPosition target, EffectRenderer effectRenderer) {
        int x = target.blockX;
        int y = target.blockY;
        int z = target.blockZ;

        int sideHit = target.sideHit;

        int meta = worldObj.getBlockMetadata(x, y, z);

        Block block = instance;
        float b = 0.1F;
        double px = x + rand.nextDouble() * (block.getBlockBoundsMaxX() - block.getBlockBoundsMinX() - (b * 2.0F)) + b + block.getBlockBoundsMinX();
        double py = y + rand.nextDouble() * (block.getBlockBoundsMaxY() - block.getBlockBoundsMinY() - (b * 2.0F)) + b + block.getBlockBoundsMinY();
        double pz = z + rand.nextDouble() * (block.getBlockBoundsMaxZ() - block.getBlockBoundsMinZ() - (b * 2.0F)) + b + block.getBlockBoundsMinZ();

        if (sideHit == 0) {
            py = (double) y + block.getBlockBoundsMinY() - (double) b;
        }

        if (sideHit == 1) {
            py = (double) y + block.getBlockBoundsMaxY() + (double) b;
        }

        if (sideHit == 2) {
            pz = (double) z + block.getBlockBoundsMinZ() - (double) b;
        }

        if (sideHit == 3) {
            pz = (double) z + block.getBlockBoundsMaxZ() + (double) b;
        }

        if (sideHit == 4) {
            px = (double) x + block.getBlockBoundsMinX() - (double) b;
        }

        if (sideHit == 5) {
            px = (double) x + block.getBlockBoundsMaxX() + (double) b;
        }

        renderPass = 0;
        EntityDiggingFX fx = new EntityDiggingFX(worldObj, px, py, pz, 0.0D, 0.0D, 0.0D, block, sideHit, worldObj.getBlockMetadata(x, y, z));
        effectRenderer.addEffect(fx.func_70596_a(x, y, z).multiplyVelocity(0.2F).multipleParticleScaleBy(0.6F));

        return true;
    }

    /**
     * Spawn particles for when the block is destroyed. Due to the nature of how
     * this is invoked, the x/y/z locations are not always guaranteed to host
     * your block. So be sure to do proper sanity checks before assuming that
     * the location is this block.
     *
     * @param world The current world
     * @param x X position to spawn the particle
     * @param y Y position to spawn the particle
     * @param z Z position to spawn the particle
     * @param meta The metadata for the block before it was destroyed.
     * @param effectRenderer A reference to the current effect renderer.
     * @return True to prevent vanilla break particles from spawning.
     */
    @SideOnly(Side.CLIENT)
    @Override
    public boolean addBlockDestroyEffects(World world, int x, int y, int z, int meta, EffectRenderer effectRenderer) {
        byte its = 4;
        for (int i = 0; i < its; ++i) {
            for (int j = 0; j < its; ++j) {
                for (int k = 0; k < its; ++k) {
                    double px = x + (i + 0.5D) / (double) its;
                    double py = y + (j + 0.5D) / (double) its;
                    double pz = z + (k + 0.5D) / (double) its;
                    int random = MiscTools.getRand().nextInt(6);
                    renderPass = 0;
                    EntityDiggingFX fx = new EntityDiggingFX(world, px, py, pz, px - x - 0.5D, py - y - 0.5D, pz - z - 0.5D, instance, random, meta);
                    effectRenderer.addEffect(fx.func_70596_a(x, y, z));
                }
            }
        }
        return true;
    }
}
