/*
 * Copyright (c) CovertJaguar, 2011 http://railcraft.info
 * 
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.blocks.ore;

import cpw.mods.fml.common.registry.GameRegistry;
import java.util.EnumSet;
import java.util.Random;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CreativePlugin;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class BlockWorldLogic extends Block {

    private static BlockWorldLogic instance;

    public static BlockWorldLogic getBlock() {
        return instance;
    }

    public static void registerBlock() {
        if (instance == null) {
            int id = RailcraftConfig.getBlockId("block.worldlogic");

            if (id > 0) {
                instance = new BlockWorldLogic(id);
                GameRegistry.registerBlock(instance, instance.getBlockName());

                RailcraftLanguage.getInstance().registerItemName(new ItemStack(instance), "ore.saltpeter.gen");
            }
        }
    }

    public BlockWorldLogic(int id) {
        super(id, Material.rock);
        blockIndexInTexture = Block.bedrock.blockIndexInTexture;
        setBlockName("rc.block.worldlogic");
        setResistance(6000000.0F);
        setBlockUnbreakable();
        setStepSound(Block.soundStoneFootstep);
        setCreativeTab(CreativePlugin.TAB);
        disableStats();

        setTickRandomly(true);
    }

    @Override
    public void onBlockAdded(World world, int x, int y, int z) {
        super.onBlockAdded(world, x, y, z);
        world.scheduleBlockUpdate(x, y, z, blockID, tickRate());
    }

    @Override
    public void updateTick(World world, int x, int y, int z, Random rand) {
        world.scheduleBlockUpdate(x, y, z, blockID, tickRate());
        if (MiscTools.getRand().nextInt(32) != 0) {
            return;
        }
        BlockOre blockOre = BlockOre.getBlock();
        if (blockOre == null || !EnumOre.SALTPETER.isEnabled() || !RailcraftConfig.isWorldGenEnabled("saltpeter")) {
            return;
        }
        int surfaceY = world.getTopSolidOrLiquidBlock(x, z) - 2;

        if (surfaceY < 50 || surfaceY > 100) {
            return;
        }

        int blockId = world.getBlockId(x, surfaceY, z);
        if (blockId != Block.sand.blockID) {
            return;
        }

        int above = world.getBlockId(x, surfaceY + 1, z);
        if (above != Block.sand.blockID) {
            return;
        }

        int below = world.getBlockId(x, surfaceY - 1, z);
        if (below != Block.sand.blockID && below != Block.sandStone.blockID) {
            return;
        }

        int airCount = 0;
        Block ore = BlockOre.getBlock();
        for (ForgeDirection side : EnumSet.of(ForgeDirection.NORTH, ForgeDirection.SOUTH, ForgeDirection.EAST, ForgeDirection.WEST)) {
            int id = MiscTools.getBlockIdOnSide(world, x, surfaceY, z, side);
            if (id == 0) {
                airCount++;
            }
            if (airCount > 1) {
                return;
            }
            if (id != 0 && id != Block.sand.blockID && id != Block.sandStone.blockID && id != ore.blockID) {
                return;
            }
        }

        world.setBlockAndMetadata(x, surfaceY, z, ore.blockID, EnumOre.SALTPETER.ordinal());
//        System.out.println("saltpeter spawned");
    }

    @Override
    public int tickRate() {
        return 6000;
    }
}
