package railcraft.common.blocks.signals;

import cpw.mods.fml.common.registry.GameRegistry;
import ic2.api.IPaintableBlock;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.block.BlockContainer;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.core.IPostConnection;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.items.IActivationBlockingItem;

public class BlockSignal extends BlockContainer implements IPaintableBlock, IPostConnection {

    private final int renderType;

    public BlockSignal(int id, int renderType) {
        super(id, 0, new MaterialStructure());
        this.renderType = renderType;
        setBlockName("rc.signal");
        setStepSound(Block.soundMetalFootstep);
        setResistance(15);

        setCreativeTab(CreativeTabs.tabTransport);

        GameRegistry.registerTileEntity(TileBoxController.class, "RCTileStructureControllerBox");
        GameRegistry.registerTileEntity(TileBoxReceiver.class, "RCTileStructureReceiverBox");
        GameRegistry.registerTileEntity(TileBoxCapacitor.class, "RCTileStructureCapacitorBox");
        GameRegistry.registerTileEntity(TileBoxBlockRelay.class, "RCTileStructureSignalBox");
        GameRegistry.registerTileEntity(TileSwitchMotor.class, "RCTileStructureSwitchMotor");
        GameRegistry.registerTileEntity(TileSwitchLever.class, "RCTileStructureSwitchLever");
        GameRegistry.registerTileEntity(TileSignalDistantSignal.class, "RCTileStructureDistantSignal");
        GameRegistry.registerTileEntity(TileSignalDualHeadBlockSignal.class, "RCTileStructureDualHeadBlockSignal");
        GameRegistry.registerTileEntity(TileSignalBlockSignal.class, "RCTileStructureBlockSignal");
        GameRegistry.registerTileEntity(TileSignalDualHeadDistantSignal.class, "RCTileStructureDualHeadDistantSignal");
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        for (EnumSignal type : EnumSignal.getCreativeList()) {
            if (type.isEnabled()) {
                list.add(type.getItem());
            }
        }
    }

    @Override
    public int damageDropped(int meta) {
        return meta;
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MAIN_TEXTURE_FILE;
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        return EnumSignal.fromId(meta).getBlockTextureFromSide(side);
    }

    @Override
    public boolean onBlockActivated(World world, int i, int j, int k, EntityPlayer player, int side, float u1, float u2, float u3) {
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null) {
            if (current.getItem() instanceof IActivationBlockingItem) {
                return false;
            }
        }
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            return ((TileSignalFoundation) tile).blockActivated(side, player);
        }
        return false;
    }

    @Override
    public void onPostBlockPlaced(World world, int i, int j, int k, int meta) {
        super.onPostBlockPlaced(world, i, j, k, meta);
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            ((TileSignalFoundation) tile).onBlockPlaced();
        }
    }

    @Override
    public void onBlockPlacedBy(World world, int i, int j, int k, EntityLiving entityliving) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            ((TileSignalFoundation) tile).onBlockPlacedBy(entityliving);
        }
    }

    @Override
    public void onNeighborBlockChange(World world, int x, int y, int z, int side) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileSignalFoundation) {
            TileSignalFoundation structure = (TileSignalFoundation) tile;
            if (structure.getSignalType().needsSupport() && !world.isBlockSolidOnSide(x, y - 1, z, ForgeDirection.UP)) {
                world.setBlockWithNotify(x, y, z, 0);
                dropBlockAsItem(world, x, y, z, structure.getSignalType().ordinal(), 0);
            } else {
                structure.onNeighborBlockChange(side);
            }
        }
    }

    @Override
    public void breakBlock(World world, int i, int j, int k, int id, int meta) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            ((TileSignalFoundation) tile).onBlockRemoval(world, i, j, k);
        }
        super.breakBlock(world, i, j, k, id, meta);
    }

    @Override
    public void setBlockBoundsBasedOnState(IBlockAccess world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            ((TileSignalFoundation) tile).setBlockBoundsBasedOnState(world, i, j, k);
        } else {
            setBlockBounds(0, 0, 0, 1, 1, 1);
        }
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            return ((TileSignalFoundation) tile).getCollisionBoundingBoxFromPool(world, i, j, k);
        }
        setBlockBounds(0, 0, 0, 1, 1, 1);
        return super.getCollisionBoundingBoxFromPool(world, i, j, k);
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            return ((TileSignalFoundation) tile).getSelectedBoundingBoxFromPool(world, i, j, k);
        }
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool((double) i + minX, (double) j + minY, (double) k + minZ, (double) i + maxX, (double) j + maxY, (double) k + maxZ);
    }

    @Override
    public int getLightValue(IBlockAccess world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof ISignalTile) {
            return ((ISignalTile) tile).getLightValue();
        }
        return 0;
    }

    @Override
    public float getBlockHardness(World world, int par2, int par3, int par4) {
        int md = world.getBlockMetadata(par4, par2, par3);
        return EnumSignal.fromId(md).getHardness();
    }

    @Override
    public boolean isBlockSolidOnSide(World world, int i, int j, int k, ForgeDirection side) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            return ((TileSignalFoundation) tile).isBlockSolidOnSide(world, i, j, k, side);
        }
        return false;
    }

//    @Override
//    public boolean canPlaceBlockOnSide(World world, int x, int y, int z, int side) {
//        int md = world.getBlockMetadata(x, y, z);
//        EnumSignal type = EnumSignal.fromId(md);
//        return super.canPlaceBlockOnSide(world, x, y, z, side) && (!type.needsSupport() || world.isBlockSolidOnSide(x, y - 1, z, ForgeDirection.UP));
//    }

    @Override
    public boolean isOpaqueCube() {
        return false;
    }

    @Override
    public boolean renderAsNormalBlock() {
        return false;
    }

    @Override
    public int getRenderType() {
        return renderType;
    }

    @Override
    public TileEntity createNewTileEntity(World var1) {
        return null;
    }

    @Override
    public TileEntity createNewTileEntity(World world, int metadata) {
        return EnumSignal.fromId(metadata).getBlockEntity();
    }

    @Override
    public boolean canProvidePower() {
        return true;
    }

    @Override
    public boolean canConnectRedstone(IBlockAccess world, int i, int j, int k, int dir) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            return ((TileSignalFoundation) tile).canConnectRedstone(dir);
        }
        return false;
    }

    @Override
    public boolean isBlockNormalCube(World world, int i, int j, int k) {
        return false;
    }

    @Override
    public boolean isProvidingWeakPower(IBlockAccess world, int i, int j, int k, int side) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileSignalFoundation) {
            return ((TileSignalFoundation) tile).isPoweringTo(world, i, j, k, side);
        }
        return false;
    }

    @Override
    public boolean isProvidingStrongPower(IBlockAccess world, int i, int j, int k, int oppositeSide) {
        return isProvidingWeakPower(world, i, j, k, oppositeSide);
    }

    @Override
    public boolean colorBlock(World world, int i, int j, int k, int color) {
        TileEntity t = world.getBlockTileEntity(i, j, k);
        if (t instanceof IPaintableBlock) {
            IPaintableBlock tile = (IPaintableBlock) t;
            return tile.colorBlock(world, i, j, k, color);
        }
        return false;
    }

    @Override
    public boolean hasTileEntity(int metadata) {
        return true;
    }

    @Override
    public boolean canBeReplacedByLeaves(World world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean canCreatureSpawn(EnumCreatureType type, World world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean connectsAt(World world, int i, int j, int k, ForgeDirection side) {
        TileEntity t = world.getBlockTileEntity(i, j, k);
        if (t instanceof ISignalTile) {
            return true;
        }
        return false;
    }
}
