package railcraft.common.blocks.signals;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.modules.ModuleManager;
import railcraft.common.modules.ModuleManager.Module;

public enum EnumSignal {

    // Name (texture, hardness, light, needsSupport, connectToPost, isPost, tile)
    @Deprecated
    METAL_POST(null, 169, 240, 3, false, true, "post.metal", TileStructureMetalPost.class),
    DUAL_HEAD_BLOCK_SIGNAL(Module.SIGNALS, 56, 56, 8, false, false, "block.signal.dual", TileSignalDualHeadBlockSignal.class),
    SWITCH_MOTOR(Module.SIGNALS, 42, 42, 8, true, false, "switch.motor", TileSwitchMotor.class),
    BLOCK_SIGNAL(Module.SIGNALS, 55, 55, 8, false, false, "block.signal", TileSignalBlockSignal.class),
    SWITCH_LEVER(Module.SIGNALS, 42, 42, 8, true, false, "switch.lever", TileSwitchLever.class),
    @Deprecated
    WOOD_POST(null, 87, 87, 3, false, true, "post.wood", TileStructureWoodPost.class),
    @Deprecated
    STONE_POST(null, 103, 103, 3, false, true, "post.stone", TileStructureStonePost.class),
    BOX_CAPACITOR(Module.SIGNALS, 88, 101, 3, true, false, "box.capacitor", TileBoxCapacitor.class),
    BOX_RECEIVER(Module.SIGNALS, 88, 120, 3, true, false, "box.receiver", TileBoxReceiver.class),
    BOX_CONTROLLER(Module.SIGNALS, 88, 119, 3, true, false, "box.controller", TileBoxController.class),
    DISTANT_SIGNAL(Module.SIGNALS, 55, 55, 8, false, false, "distant", TileSignalDistantSignal.class),
    DUAL_HEAD_DISTANT_SIGNAL(Module.SIGNALS, 56, 56, 8, false, false, "distant.dual", TileSignalDualHeadDistantSignal.class),
    BOX_BLOCK_RELAY(Module.SIGNALS, 88, 117, 3, true, false, "box.block.relay", TileBoxBlockRelay.class),;
    private final Module module;
    private final int primaryTexture;
    private final int secondaryTexture;
    private final float hardness;
    private final boolean needsSupport;
    private final boolean isPost;
    private final String tag;
    private int altTextureOffset = 0;
    private final Class<? extends TileSignalFoundation> tile;
    private boolean useAltTexture;
    private int[] textures = new int[6];
    private static Map<Integer, EnumSignal> structures = new HashMap<Integer, EnumSignal>();
    private static final List<EnumSignal> creativeList = new ArrayList<EnumSignal>();

    static {
        creativeList.add(SWITCH_LEVER);
        creativeList.add(SWITCH_MOTOR);
        creativeList.add(BLOCK_SIGNAL);
        creativeList.add(DISTANT_SIGNAL);
        creativeList.add(DUAL_HEAD_BLOCK_SIGNAL);
        creativeList.add(DUAL_HEAD_DISTANT_SIGNAL);
        creativeList.add(BOX_BLOCK_RELAY);
        creativeList.add(BOX_CONTROLLER);
        creativeList.add(BOX_RECEIVER);
        creativeList.add(BOX_CAPACITOR);
    }

    private EnumSignal(Module module, int primaryTexture, int secondaryTexture, float hardness, boolean needsSupport, boolean isPost, String tag, Class<? extends TileSignalFoundation> tile) {
        this.module = module;
        this.primaryTexture = primaryTexture;
        this.secondaryTexture = secondaryTexture;
        this.hardness = hardness;
        this.needsSupport = needsSupport;
        this.isPost = isPost;
        this.tile = tile;
        this.tag = tag;
        setUseAltTextures(false);
    }

    public ItemStack getItem() {
        return getItem(1);
    }

    public ItemStack getItem(int qty) {
        return new ItemStack(RailcraftBlocks.getBlockSignal().blockID, qty, ordinal());
    }

    public String getTag() {
        return "rc.signal." + tag;
    }

    public Module getModule() {
        return module;
    }

    public Class<? extends TileSignalFoundation> getTileClass() {
        return tile;
    }

    public TileSignalFoundation getBlockEntity() {
        if (tile == null) {
            return null;
        }
        try {
            return tile.newInstance();
        } catch (Exception ex) {
        }
        return null;
    }

    public float getHardness() {
        return hardness;
    }

    public int getBlockTexture() {
        if (useAltTexture) {
            return textures[0] + altTextureOffset;
        }
        return primaryTexture;
    }

    public int getBlockTextureFromSide(int side) {
        if (useAltTexture) {
            return textures[side] + altTextureOffset;
        }
        return textures[side];
    }

    public int getOffsetTexture(int offset) {
        return primaryTexture + offset;
    }

    public int getPrimaryTexture() {
        return primaryTexture;
    }

    public int getSecondaryTexture() {
        return secondaryTexture;
    }

    public static EnumSignal fromId(int id) {
        EnumSignal s = structures.get(id);
        if (s != null) {
            return s;
        }
        for (EnumSignal t : EnumSignal.values()) {
            if (t.ordinal() == id) {
                structures.put(id, t);
                return t;
            }
        }
        return SWITCH_LEVER;
    }

    public static List<EnumSignal> getCreativeList() {
        return creativeList;
    }

    public boolean useAltTexture() {
        return useAltTexture;
    }

    public void setUseAltTextures(boolean use) {
        for (int i = 0; i < textures.length; i++) {
            textures[i] = primaryTexture;
        }
        altTextureOffset = 0;
        useAltTexture = use;
    }

    public void setUseAltTextureOnSide(int side, int texture) {
        textures[side] = texture;
    }

    public void setUseOffsetTextureOnSide(int side, int offset) {
        textures[side] = primaryTexture + offset;
    }

    public void setAltTextureIndex(int index) {
        for (int i = 0; i < textures.length; i++) {
            textures[i] = index;
        }
    }

    public void setAltTextureOffset(int altTextureOffset) {
        this.altTextureOffset = altTextureOffset;
    }

    public boolean isPost() {
        return isPost;
    }

    public boolean needsSupport() {
        return needsSupport;
    }

    public boolean isEnabled() {
        if (module == null) return false;
        return ModuleManager.isModuleLoaded(getModule()) && RailcraftBlocks.getBlockSignal() != null && RailcraftConfig.isSubBlockEnabled(getTag());
    }
}
