package railcraft.common.blocks.signals;

import cpw.mods.fml.common.registry.GameRegistry;
import ic2.api.IBoxable;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import railcraft.common.api.core.WorldCoordinate;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.Game;

public class ItemSignalBlockSurveyor extends Item implements IBoxable {

    private static Item item;

    public static void registerItem() {
        if (item == null) {
            String tag = "rc.tool.surveyor";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemSignalBlockSurveyor(id);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedRecipe(new ItemStack(item),
                        " C ",
                        "BGB",
                        " R ",
                        'G', Block.thinGlass,
                        'C', Item.compass,
                        'B', Block.stoneButton,
                        'R', Item.redstone);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getItem() {
        if (item == null) {
            return null;
        }
        return new ItemStack(item);
    }

    private ItemSignalBlockSurveyor(int id) {
        super(id);
        setMaxDamage(0);
        setHasSubtypes(true);
        setMaxStackSize(1);
        setIconIndex(86);
        setItemName("rc.tool.surveyor");

        setCreativeTab(CreativeTabs.tabTools);
    }

    @Override
    public void getSubItems(int id, CreativeTabs tag, List list) {
        list.add(new ItemStack(this));
    }

    @Override
    public boolean onItemUse(ItemStack item, EntityPlayer player, World world, int x, int y, int z, int side, float par8, float par9, float par10) {
//        System.out.println("click");
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile != null && tile instanceof ISignalBlockTile) {
//            System.out.println("target found");
            if (Game.isHost(world)) {
                ISignalBlockTile signalTile = (ISignalBlockTile) tile;
                SignalBlock signalBlock = signalTile.getSignalBlock();
                WorldCoordinate pos = null;
                NBTTagCompound data = item.getTagCompound();
                if (data != null) {
                    int sDim = data.getInteger("signalDim");
                    int sx = data.getInteger("signalX");
                    int sy = data.getInteger("signalY");
                    int sz = data.getInteger("signalZ");
                    pos = new WorldCoordinate(sDim, sx, sy, sz);
                }
                WorldCoordinate track = signalBlock.getTrackLocation();
                if (track == null) {
                    player.addChatMessage("No rail found near " + signalTile.getInvName());
                } else if (pos == null) {
                    player.addChatMessage("Beginning Signal Block Survey");
                    setSignalData(item, tile);
                    signalBlock.startPairing();
                } else if (x != pos.x || y != pos.y || z != pos.z) {
//                System.out.println("attempt pairing");
                    tile = world.getBlockTileEntity(pos.x, pos.y, pos.z);
                    if (tile != null && tile instanceof ISignalBlockTile) {
                        ISignalBlockTile otherTile = (ISignalBlockTile) tile;
                        SignalBlock otherSignal = otherTile.getSignalBlock();
                        if (otherSignal != signalBlock && signalBlock.createSignalBlock(otherSignal)) {
                            player.addChatMessage("Successfully defined Signal Block");
                            item.setTagCompound(null);
                        } else {
                            player.addChatMessage("Signal Block is not valid");
                        }
                    } else if (world.blockExists(pos.x, pos.y, pos.z)) {
                        player.addChatMessage("First Signal no longer exists, abandoning Survey");
                        signalBlock.endPairing();
                        item.setTagCompound(null);
                    } else {
                        player.addChatMessage("First Signal is not in a loaded Chunk");
                    }
                } else {
                    player.addChatMessage("Signal Block Survey abandoned");
                    signalBlock.endPairing();
                    item.setTagCompound(null);
                }
            }
            return true;
        }
        return false;
    }

    private void setSignalData(ItemStack item, TileEntity tile) {
        NBTTagCompound data = new NBTTagCompound("tag");
        data.setInteger("signalDim", tile.worldObj.provider.dimensionId);
        data.setInteger("signalX", tile.xCoord);
        data.setInteger("signalY", tile.yCoord);
        data.setInteger("signalZ", tile.zCoord);
        item.setTagCompound(data);
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MAIN_TEXTURE_FILE;
    }

    @Override
    public boolean canBeStoredInToolbox(ItemStack itemstack) {
        return true;
    }
}
