package railcraft.common.blocks.signals;

import ic2.api.IBoxable;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import railcraft.common.api.core.WorldCoordinate;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.api.signals.IControllerTile;
import railcraft.common.api.signals.IReceiverTile;
import railcraft.common.api.signals.SignalController;
import railcraft.common.api.signals.SignalReceiver;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.items.ItemCircuit;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.Game;

public class ItemSignalTuner extends Item implements IBoxable {

    private static Item item;

    public static void registerItem() {
        if (item == null) {
            String tag = "rc.tool.signal.tuner";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemSignalTuner(id);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedRecipe(new ItemStack(item),
                        " T ",
                        "BRB",
                        "   ",
                        'B', Block.stoneButton,
                        'R', ItemCircuit.getReceiverCircuit(),
                        'T', Block.torchRedstoneActive);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getItem() {
        if (item == null) {
            return null;
        }
        return new ItemStack(item);
    }

    private ItemSignalTuner(int id) {
        super(id);
        setMaxDamage(0);
        setHasSubtypes(true);
        setMaxStackSize(1);
        setIconIndex(118);
        setItemName("rc.tool.signal.tuner");

        setCreativeTab(CreativeTabs.tabTools);
    }

    @Override
    public void getSubItems(int id, CreativeTabs tag, List list) {
        list.add(new ItemStack(this));
    }

    @Override
    public boolean onItemUse(ItemStack item, EntityPlayer player, World world, int i, int j, int k, int side, float par8, float par9, float par10) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile != null) {
            WorldCoordinate cPos = null;
            NBTTagCompound data = item.getTagCompound();
            if (data != null) {
                int cDim = data.getInteger("controllerDim");
                int cx = data.getInteger("controllerX");
                int cy = data.getInteger("controllerY");
                int cz = data.getInteger("controllerZ");
                cPos = new WorldCoordinate(cDim, cx, cy, cz);
            }
            if (tile instanceof IReceiverTile && cPos != null) {
                if (Game.isHost(world)) {
                    SignalReceiver receiver = ((IReceiverTile) tile).getReceiver();
                    if (i != cPos.x || j != cPos.y || k != cPos.z) {
                        tile = world.getBlockTileEntity(cPos.x, cPos.y, cPos.z);
                        if (tile != null && tile instanceof IControllerTile) {
                            SignalController controller = ((IControllerTile) tile).getController();
                            if (receiver.getTile() != controller.getTile()) {
                                controller.registerReceiver(receiver);
                                controller.endPairing();
                                String message = RailcraftLanguage.translate("gui.tuner.success");
                                player.addChatMessage(String.format(message, controller.getName(), receiver.getName()));
                                item.setTagCompound(null);
                                return true;
                            }
                        } else if (world.blockExists(cPos.x, cPos.y, cPos.z)) {
                            player.addChatMessage(RailcraftLanguage.translate("gui.tuner.abandon.gone"));
                            item.setTagCompound(null);
                        } else {
                            player.addChatMessage(RailcraftLanguage.translate("gui.tuner.abandon.chunk"));
                            item.setTagCompound(null);
                        }
                    }
                }
            } else if (tile instanceof IControllerTile) {
                if (Game.isHost(world)) {
                    SignalController controller = ((IControllerTile) tile).getController();
                    if (cPos == null || (i != cPos.x || j != cPos.y || k != cPos.z)) {
                        String message = RailcraftLanguage.translate("gui.tuner.start");
                        player.addChatMessage(String.format(message, controller.getName()));
                        setControllerData(item, tile);
                        controller.startPairing();
                    } else {
                        String message = RailcraftLanguage.translate("gui.tuner.stop");
                        player.addChatMessage(String.format(message, controller.getName()));
                        controller.endPairing();
                        item.setTagCompound(null);
                    }
                }
            } else {
                return false;
            }
            return true;
        }
        return false;
    }

    private void setControllerData(ItemStack item, TileEntity tile) {
        NBTTagCompound data = new NBTTagCompound("tag");
        data.setInteger("controllerDim", tile.worldObj.provider.dimensionId);
        data.setInteger("controllerX", tile.xCoord);
        data.setInteger("controllerY", tile.yCoord);
        data.setInteger("controllerZ", tile.zCoord);
        item.setTagCompound(data);
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MAIN_TEXTURE_FILE;
    }

    @Override
    public boolean canBeStoredInToolbox(ItemStack itemstack) {
        return true;
    }
}
