package railcraft.common.blocks.signals;

import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.signals.SignalAspect;

public abstract class TileBoxBase extends TileSignalFoundation {

    private static final float BOUND = 0.1f;

    @Override
    public void setBlockBoundsBasedOnState(IBlockAccess world, int i, int j, int k) {
        getBlock().setBlockBounds(BOUND, 0, BOUND, 1 - BOUND, 1 - BOUND / 2, 1 - BOUND);
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + BOUND, j, k + BOUND, i + 1 - BOUND, j + 1 - BOUND / 2, k + 1 - BOUND);
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + BOUND, j, k + BOUND, i + 1 - BOUND, j + 1 - BOUND / 2, k + 1 - BOUND);
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    public abstract boolean connectToBoxAt(int i, int j, int k, int side);

    public abstract SignalAspect getBoxSignalAspect();

    public boolean canTransferAspect() {
        return false;
    }

    public boolean canReceiveAspect() {
        return false;
    }

    public void onNeighborStateChange(TileBoxBase neighbor) {
    }

    public boolean isEmitingRedstone() {
        return false;
    }

    @Override
    public boolean isBlockSolidOnSide(World world, int i, int j, int k, ForgeDirection side) {
        if (side == ForgeDirection.UP) {
            return true;
        }
        return false;
    }

    @Override
    public boolean canConnectRedstone(int dir) {
        return true;
    }
}
