package railcraft.common.blocks.signals;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import railcraft.common.api.signals.SignalAspect;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.network.IGuiReturnHandler;

public class TileBoxCapacitor extends TileBoxBase implements IGuiReturnHandler {

    protected int update = MiscTools.getRand().nextInt();
    private short ticksPowered;
    public short ticksToPower = 200;

    public TileBoxCapacitor() {
    }

    @Override
    public EnumSignal getSignalType() {
        return EnumSignal.BOX_CAPACITOR;
    }

    @Override
    public boolean blockActivated(int side, EntityPlayer player) {
        if (player.isSneaking()) {
            return false;
        }
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null && current.getItem() instanceof ItemSignalTuner) {
            return super.blockActivated(side, player);
        }
        GuiHandler.openGui(EnumGui.BOX_CAPACITOR, player, worldObj, xCoord, yCoord, zCoord);
        return true;
    }

    @Override
    public void updateEntity() {
        update++;
        if (Game.isNotHost(worldObj)) {
            return;
        }
        if (ticksPowered > 0) {
            ticksPowered--;
            if (ticksPowered <= 0) {
                updateNeighbors();
            }
        }
    }

    @Override
    public void onNeighborStateChange(TileBoxBase neighbor) {
        if (neighbor.isEmitingRedstone()) {
            ticksPowered = ticksToPower;
            updateNeighbors();
        }
    }

    private void updateNeighbors() {
        sendUpdateToClient();
        int blockID = getBlockId();
        worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord, zCoord, blockID);
//        worldObj.notifyBlocksOfNeighborChange(xCoord + 1, yCoord, zCoord, blockID);
//        worldObj.notifyBlocksOfNeighborChange(xCoord - 1, yCoord, zCoord, blockID);
//        worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord, zCoord + 1, blockID);
//        worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord, zCoord - 1, blockID);
//        worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord - 1, zCoord, blockID);
//        worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord + 1, zCoord, blockID);
    }

    @Override
    public boolean isPoweringTo(IBlockAccess world, int i, int j, int k, int side) {
        TileEntity tile = MiscTools.getBlockTileEntityOnSide(worldObj, i, j, k, MiscTools.getOppositeSide(side));
        if (tile instanceof TileBoxBase) {
            return false;
        }
        return ticksPowered > 0;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setShort("ticksPowered", ticksPowered);
        data.setShort("ticksToPower", ticksToPower);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        ticksPowered = data.getShort("ticksPowered");
        ticksToPower = data.getShort("ticksToPower");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeBoolean(ticksPowered > 0);
        data.writeShort(ticksToPower);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        ticksPowered = (short) (data.readBoolean() ? 1 : 0);
        ticksToPower = data.readShort();

        markBlockForUpdate();
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeShort(ticksToPower);
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        ticksToPower = data.readShort();
    }

    @Override
    public boolean connectToBoxAt(int i, int j, int k, int side) {
        TileEntity tile = worldObj.getBlockTileEntity(i, j, k);
        if (tile instanceof TileBoxBase) {
            return ((TileBoxBase) tile).canTransferAspect();
        }
        return false;
    }

    @Override
    public SignalAspect getBoxSignalAspect() {
        return ticksPowered > 0 ? SignalAspect.RED : SignalAspect.OFF;
    }

    @Override
    public boolean canReceiveAspect() {
        return true;
    }
}
