package railcraft.common.blocks.signals;

import buildcraft.api.tools.IToolWrench;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.signals.SignalAspect;
import railcraft.common.util.misc.MiscTools;

public abstract class TileSignalBase extends TileSignalFoundation implements ISignalTile {

    protected static final float BOUNDS = 0.15f;
    private ForgeDirection facing = ForgeDirection.NORTH;
    protected int update = MiscTools.getRand().nextInt();

    @Override
    public boolean blockActivated(int side, EntityPlayer player) {
        ItemStack current = player.getCurrentEquippedItem();
        if (current != null && current.getItem() instanceof IToolWrench
                && side != ForgeDirection.UP.ordinal() && side != ForgeDirection.DOWN.ordinal()) {
            IToolWrench wrench = (IToolWrench) current.getItem();
            if (wrench.canWrench(player, xCoord, yCoord, zCoord)) {
                if (facing.ordinal() == side) {
                    facing = MiscTools.getOppositeSide(side);
                } else {
                    facing = ForgeDirection.getOrientation(side);
                }
                wrench.wrenchUsed(player, xCoord, yCoord, zCoord);
                markBlockForUpdate();
                return true;
            }
        }
        return super.blockActivated(side, player);
    }

    @Override
    public void setBlockBoundsBasedOnState(IBlockAccess world, int i, int j, int k) {
        getBlock().setBlockBounds(BOUNDS, 0.35f, BOUNDS, 1 - BOUNDS, 1f, 1 - BOUNDS);
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + BOUNDS, j + 0.35f, k + BOUNDS, i + 1 - BOUNDS, j + 1, k + 1 - BOUNDS);
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + BOUNDS, j + 0.35f, k + BOUNDS, i + 1 - BOUNDS, j + 1, k + 1 - BOUNDS);
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    @Override
    public void updateEntity() {
    }

    public void setFacing(ForgeDirection facing) {
        this.facing = facing;
    }

    public ForgeDirection getFacing() {
        return facing;
    }

    @Override
    public void onBlockPlacedBy(EntityLiving entityliving) {
        super.onBlockPlacedBy(entityliving);
        facing = MiscTools.getHorizontalSideClosestToPlayer(worldObj, xCoord, yCoord, zCoord, entityliving);
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setByte("Facing", (byte) facing.ordinal());
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        facing = ForgeDirection.getOrientation(data.getByte("Facing"));
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeByte((byte) facing.ordinal());
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        facing = ForgeDirection.getOrientation(data.readByte());

        markBlockForUpdate();
    }

    public abstract SignalAspect getSignalAspect();

    @Override
    public int getLightValue() {
        if (Signals.isAspectLit(getSignalAspect())) {
            return 5;
        }
        return 0;
    }
}
