package railcraft.common.blocks.signals;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import railcraft.common.api.signals.DualSignalReceiver;
import railcraft.common.api.signals.IReceiverTile;
import railcraft.common.api.signals.SignalAspect;
import railcraft.common.api.signals.SignalController;
import railcraft.common.util.misc.Game;

public class TileSignalDualHeadDistantSignal extends TileSignalBase implements IReceiverTile, IDualHeadSignal {

    private static final float SIZE = 0.15f;
    private boolean prevLightState;
    private final DualSignalReceiver receiver = new DualSignalReceiver(getInvName(), this);

    @Override
    public EnumSignal getSignalType() {
        return EnumSignal.DUAL_HEAD_DISTANT_SIGNAL;
    }

    @Override
    public void updateEntity() {
        super.updateEntity();
        if (Game.isNotHost(worldObj)) {
            receiver.tickClient();
            boolean lightState = Signals.isAspectLit(getTopAspect()) || Signals.isAspectLit(getBottomAspect());
            if (update % 4 == 0 && lightState != prevLightState) {
                prevLightState = lightState;
                worldObj.updateLightByType(EnumSkyBlock.Block, getX(), getY(), getZ());
                markBlockForUpdate();
            }
            return;
        }

        receiver.tickServer();
        int numPairs = receiver.getNumPairs();
        boolean changed = false;
        switch (numPairs) {
            case 0:
                changed |= receiver.setTopAspect(SignalAspect.BLINK_RED);
            case 1:
                changed |= receiver.setBottomAspect(SignalAspect.BLINK_RED);
        }
        if (changed) {
            sendUpdateToClient();
        }
    }

    @Override
    public void onControllerAspectChange(SignalController con, SignalAspect aspect) {
        sendUpdateToClient();
    }

    @Override
    public void setBlockBoundsBasedOnState(IBlockAccess world, int i, int j, int k) {
        getBlock().setBlockBounds(0.15f, 0f, 0.15f, 0.85f, 1f, 0.85f);
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + 0.15f, j, k + 0.15f, i + 0.85f, j + 1f, k + 0.85f);
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + SIZE, j, k + SIZE, i + 1 - SIZE, j + 1, k + 1 - SIZE);
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        receiver.writeToNBT(data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        receiver.readFromNBT(data);

    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        receiver.writePacketData(data);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        receiver.readPacketData(data);
    }

    @Override
    public int getLightValue() {
        if (Signals.isAspectLit(getTopAspect()) || Signals.isAspectLit(getBottomAspect())) {
            return 5;
        }
        return 0;
    }

//    @Override
//    public String getDescription() {
//        return "Dual-Head Distant Signal";
//    }
    @Override
    public DualSignalReceiver getReceiver() {
        return receiver;
    }

    @Override
    public SignalAspect getTopAspect() {
        return receiver.getTopAspect();
    }

    @Override
    public SignalAspect getBottomAspect() {
        return receiver.getBottomAspect();
    }

    @Override
    public SignalAspect getSignalAspect() {
        return receiver.getTopAspect();
    }
}
