package railcraft.common.blocks.signals;

import java.util.HashSet;
import java.util.Set;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.core.IPostConnection;

public abstract class TileStructurePost extends TileSignalFoundation {

    public static final Set<Integer> canConnect = new HashSet<Integer>();
    public static final Set<Integer> noConnect = new HashSet<Integer>();

    static {
        canConnect.add(Block.glowStone.blockID);
        canConnect.add(Block.redstoneLampActive.blockID);
        canConnect.add(Block.redstoneLampIdle.blockID);
        canConnect.add(Block.glass.blockID);
        canConnect.add(Block.mobSpawner.blockID);
        canConnect.add(Block.pistonBase.blockID);

        noConnect.add(Block.dirt.blockID);
        noConnect.add(Block.stone.blockID);
        noConnect.add(Block.gravel.blockID);
        noConnect.add(Block.grass.blockID);
        noConnect.add(Block.sand.blockID);
        noConnect.add(Block.bedrock.blockID);
        noConnect.add(Block.oreCoal.blockID);
        noConnect.add(Block.oreDiamond.blockID);
        noConnect.add(Block.oreEmerald.blockID);
        noConnect.add(Block.oreGold.blockID);
        noConnect.add(Block.oreIron.blockID);
        noConnect.add(Block.oreLapis.blockID);
        noConnect.add(Block.oreRedstone.blockID);
        noConnect.add(Block.oreRedstoneGlowing.blockID);
        noConnect.add(Block.blockClay.blockID);
        noConnect.add(Block.blockSnow.blockID);
        noConnect.add(Block.melon.blockID);
        noConnect.add(Block.pumpkin.blockID);
        noConnect.add(Block.tnt.blockID);
        noConnect.add(Block.slowSand.blockID);
        noConnect.add(Block.netherrack.blockID);
        noConnect.add(Block.sponge.blockID);
    }
    private static final float SIZE = 0.15f;
    private static final float SELECT = 0.1f;

    @Override
    public void setBlockBoundsBasedOnState(IBlockAccess world, int i, int j, int k) {
        getBlock().setBlockBounds(0.2F, 0.0F, 0.2F, 0.8F, 1.0F, 0.8F);
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
        if (!world.isAirBlock(i, j - 1, k)
                && !(world.getBlockTileEntity(i, j - 1, k) instanceof TileStructurePost)
                && !BlockRail.isRailBlockAt(world, i, j + 1, k)) {
            return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + SIZE, j, k + SIZE, i + 1 - SIZE, j + 1.5, k + 1 - SIZE);
        }
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + SIZE, j, k + SIZE, i + 1 - SIZE, j + 1, k + 1 - SIZE);
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i + SELECT, j, k + SELECT, i + 1 - SELECT, j + 1.0F, k + 1 - SELECT);
    }

    @Override
    public boolean isBlockSolidOnSide(World world, int i, int j, int k, ForgeDirection side) {
        if (side == ForgeDirection.DOWN || side == ForgeDirection.UP) {
            return true;
        }
        return false;
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    public boolean connectToBlockAt(int i, int j, int k, ForgeDirection side) {
        int id = worldObj.getBlockId(i, j, k);
        if (Block.blocksList[id] instanceof IPostConnection) {
            return ((IPostConnection) Block.blocksList[id]).connectsAt(worldObj, i, j, k, side);
        }

        if (worldObj.isAirBlock(i, j, k)) {
            return false;
        }

        if (Block.blocksList[id].isGenMineableReplaceable(worldObj, i, j, k)) {
            return false;
        }

        if (noConnect.contains(id)) {
            return false;
        }

        if (canConnect.contains(id)) {
            return true;
        }

        if (worldObj.isBlockSolidOnSide(i, j, k, side)) {
            return true;
        }

        return false;
    }
}
