package railcraft.common.blocks.tracks;

import cpw.mods.fml.common.registry.GameRegistry;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;
import java.util.logging.Level;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.world.IBlockAccess;
import net.minecraft.item.ItemStack;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.stats.StatList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.core.IPostConnection;
import railcraft.common.api.tracks.ITrackBlocksMovement;
import railcraft.common.api.tracks.ITrackCustomShape;
import railcraft.common.api.tracks.ITrackEmitter;
import railcraft.common.api.tracks.ITrackInstance;
import railcraft.common.api.tracks.TrackRegistry;
import railcraft.common.api.tracks.TrackSpec;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.util.misc.Game;

public class BlockTrack extends BlockRail implements IPostConnection {

    protected final int renderType;

    public BlockTrack(int blockID, int modelID) {
        super(blockID, 255, false);
        TrackSpec.blockID = blockID;
        renderType = modelID;
        setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 0.125F, 1.0F);
        setResistance(3.5F);
        setHardness(1.05F);
        setStepSound(soundMetalFootstep);
        setRequiresSelfNotify();
        setCreativeTab(CreativeTabs.tabTransport);
        GameRegistry.registerTileEntity(TileTrack.class, "RailcraftTrackTile");
    }

    @Override
    public void getSubBlocks(int id, CreativeTabs tab, List list) {
        Set<TrackSpec> railcraftSpecs = new HashSet<TrackSpec>();
        for (EnumTrack t : EnumTrack.getCreativeList()) {
            railcraftSpecs.add(TrackRegistry.getTrackSpec(t.ordinal()));
            if (t.isEnabled()) {
                list.add(t.getItem());
            }
        }

        Set<TrackSpec> specs = new HashSet<TrackSpec>(TrackRegistry.getTrackSpecs().values());
        specs.removeAll(railcraftSpecs);
        specs.remove(TrackRegistry.getTrackSpec(-1));
        for (TrackSpec spec : specs) {
            list.add(new ItemStack(this, 1, spec.getTrackId()));
        }
    }

    @Override
    public ItemStack getPickBlock(MovingObjectPosition target, World world, int x, int y, int z) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            return new ItemStack(this, 1, track.getTrackSpec().getTrackId());
        }
        return null;
    }

    @Override
    public int getRenderType() {
        return renderType;
    }

    @Override
    public boolean hasTileEntity(int metadata) {
        return true;
    }

    @Override
    public int getMobilityFlag() {
        return 0;
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            if (track instanceof ITrackCustomShape) {
                return ((ITrackCustomShape) track).getCollisionBoundingBoxFromPool();
            }
        }
        return null;
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            if (track instanceof ITrackCustomShape) {
                return ((ITrackCustomShape) track).getSelectedBoundingBoxFromPool();
            }
        }
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool((double) i + minX, (double) j + minY, (double) k + minZ, (double) i + maxX, (double) j + maxY, (double) k + maxZ);
    }

    @Override
    public boolean isOpaqueCube() {
        return false;
    }

    @Override
    public MovingObjectPosition collisionRayTrace(World world, int i, int j, int k, Vec3 vec3d, Vec3 vec3d1) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            if (track instanceof ITrackCustomShape) {
                return ((ITrackCustomShape) track).collisionRayTrace(vec3d, vec3d1);
            }
        }
        return super.collisionRayTrace(world, i, j, k, vec3d, vec3d1);
    }

    @Override
    public void setBlockBoundsBasedOnState(IBlockAccess iblockaccess, int i, int j, int k) {
        int l = iblockaccess.getBlockMetadata(i, j, k);
        if (l >= 2 && l <= 5) {
            setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 0.625F, 1.0F);
        } else {
            setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 0.125F, 1.0F);
        }
    }

    @Override
    public boolean getBlocksMovement(IBlockAccess world, int x, int y, int z) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            if (track instanceof ITrackBlocksMovement) {
                return !((ITrackBlocksMovement) track).blocksMovement();
            }
        }
        return super.getBlocksMovement(world, x, y, z);
    }

    @Override
    public boolean canPlaceBlockAt(World world, int i, int j, int k) {
        return !BlockRail.isRailBlockAt(world, i, j + 1, k);
    }

    @Override
    public boolean renderAsNormalBlock() {
        return false;
    }

    @Override
    public boolean canProvidePower() {
        return true;
    }

    @Override
    public boolean canConnectRedstone(IBlockAccess world, int x, int y, int z, int side) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            return track instanceof ITrackEmitter;
        }
        return false;
    }

    @Override
    public boolean isProvidingWeakPower(IBlockAccess world, int x, int y, int z, int side) {
        TileEntity tile = world.getBlockTileEntity(x, y, z);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            return track instanceof ITrackEmitter && ((ITrackEmitter) track).isTrackPowering();
        }
        return false;
    }

    @Override
    public boolean isProvidingStrongPower(IBlockAccess world, int x, int y, int z, int side) {
        return isProvidingWeakPower(world, x, y, z, side);
    }

    @Override
    public void onMinecartPass(World world, EntityMinecart cart, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            ((TileTrack) tile).getTrackInstance().onMinecartPass(cart);
        }
    }

    @Override
    public int getBasicRailMetadata(IBlockAccess world, EntityMinecart cart, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            return ((TileTrack) tile).getTrackInstance().getBasicRailMetadata(cart);
        }
        return world.getBlockMetadata(i, j, k);
    }

    @Override
    public float getRailMaxSpeed(World world, EntityMinecart cart, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            return ((TileTrack) tile).getTrackInstance().getRailMaxSpeed(cart);
        }
        return 0.4f;
    }

    @Override
    public boolean onBlockActivated(World world, int i, int j, int k, EntityPlayer player, int side, float u1, float u2, float u3) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            return ((TileTrack) tile).getTrackInstance().blockActivated(player);
        }
        return false;
    }

    @Override
    public boolean isFlexibleRail(World world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            return ((TileTrack) tile).getTrackInstance().isFlexibleRail();
        }
        return false;
    }

    @Override
    public boolean canMakeSlopes(World world, int i, int j, int k) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            return ((TileTrack) tile).getTrackInstance().canMakeSlopes();
        }
        return true;
    }

    @Override
    public boolean hasPowerBit(World world, int i, int j, int k) {
        return false;
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.TRACK_TEXTURE_FILE;
    }

    @Override
    public int getBlockTextureFromSideAndMetadata(int side, int meta) {
        return blockIndexInTexture;
    }

    @Override
    public int getBlockTexture(IBlockAccess world, int i, int j, int k, int side) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            return ((TileTrack) tile).getTrackInstance().getTextureIndex();

        }
        return -1;
    }

    @Override
    public ArrayList<ItemStack> getBlockDropped(World world, int i, int j, int k, int md, int fortune) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        ArrayList<ItemStack> items = new ArrayList<ItemStack>();
        if (tile instanceof TileTrack) {
            items.add(new ItemStack(this, 1, ((TileTrack) tile).getTrackInstance().getTrackSpec().getTrackId()));
        } else {
            Game.log(Level.WARNING, "Rail Tile was invalid when harvesting rail");
            items.add(new ItemStack(Block.rail));
        }
        return items;
    }

    @Override
    public int quantityDropped(int meta, int fortune, Random random) {
        return 1;
    }

    @Override
    public int idDropped(int i, Random random, int j) {
        Game.log(Level.WARNING, "Wrong function called when harvesting rail");
        return Block.rail.idDropped(i, random, j);
    }

    public TileEntity getBlockEntity(int md) {
        return null;
    }

    // Determine direction here
    @Override
    public void onBlockPlacedBy(World world, int i, int j, int k, EntityLiving entityliving) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            ((TileTrack) tile).onBlockPlacedBy(entityliving);
            ((TileTrack) tile).getTrackInstance().onBlockPlacedBy(entityliving);
        }
    }

    @Override
    public void onPostBlockPlaced(World world, int i, int j, int k, int meta) {
//        if(Game.isNotHost(world)) {
//            return;
//        }
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            ((TileTrack) tile).getTrackInstance().onBlockPlaced();
        }
    }

    @Override
    public void harvestBlock(World world, EntityPlayer entityplayer, int i, int j, int k, int l) {
    }

    @Override
    public boolean removeBlockByPlayer(World world, EntityPlayer player, int i, int j, int k) {
        player.addStat(StatList.mineBlockStatArray[blockID], 1);
        player.addExhaustion(0.025F);
        if (Game.isHost(world) && !player.capabilities.isCreativeMode) {
            dropBlockAsItem(world, i, j, k, 0, 0);
        }
        return world.setBlockWithNotify(i, j, k, 0);
    }

    @Override
    public void breakBlock(World world, int i, int j, int k, int id, int meta) {
        super.breakBlock(world, i, j, k, id, meta);

        try {
            TileEntity tile = world.getBlockTileEntity(i, j, k);
            if (tile instanceof TileTrack) {
                ((TileTrack) tile).getTrackInstance().onBlockRemoved();
            }
        } catch (Throwable error) {
            Game.logErrorAPI("Railcraft", error);
        }

        world.removeBlockTileEntity(i, j, k);
    }

    @Override
    public void onNeighborBlockChange(World world, int i, int j, int k, int id) {
        if (Game.isNotHost(world)) {
            return;
        }
        TileEntity t = world.getBlockTileEntity(i, j, k);
        if (t instanceof TileTrack) {
            TileTrack tile = (TileTrack) t;
            tile.getTrackInstance().onNeighborBlockChange(id);
        }
    }

    @Override
    public float getExplosionResistance(Entity exploder, World world, int i, int j, int k, double srcX, double srcY, double srcZ) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            return ((TileTrack) tile).getTrackInstance().getExplosionResistance(srcX, srcY, srcZ, exploder) * 3f / 5f;
        }
        return getExplosionResistance(exploder);
    }

    @Override
    public boolean canBeReplacedByLeaves(World world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean canCreatureSpawn(EnumCreatureType type, World world, int x, int y, int z) {
        return false;
    }

    @Override
    public boolean connectsAt(World world, int i, int j, int k, ForgeDirection side) {
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        if (tile instanceof TileTrack) {
            ITrackInstance track = ((TileTrack) tile).getTrackInstance();
            if (track instanceof IPostConnection) {
                return ((IPostConnection) track).connectsAt(world, i, j, k, side);
            }
        }
        return false;
    }
}
