package railcraft.common.blocks.tracks;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import railcraft.common.api.tracks.TrackRegistry;
import railcraft.common.api.tracks.TrackSpec;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.tracks.old.*;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.items.RailcraftToolItems;
import railcraft.common.modules.ModuleManager;
import railcraft.common.modules.ModuleManager.Module;
import railcraft.common.plugins.forge.CraftingPlugin;

public enum EnumTrack {

    BOARDING(Module.TRACKS, 0, "boarding", "RCBoardingRailTile", 8, TileRailBoarding.class, TrackBoarding.class),
    HOLDING(Module.TRACKS, 4, "holding", "RCHoldingRailTile", 8, TileRailHolding.class, TrackHolding.class),
    ONEWAY(Module.TRACKS, 1 * 16, "oneway", "RCOneWayRailTile", 8, TileRailOneway.class, TrackOneWay.class),
    CONTROL(Module.TRACKS, 100, "control", "RCControlRailTile", 16, TileRailControl.class, TrackControl.class),
    LAUNCHER(Module.EXTRAS, 36, "launcher", "RCLauncherRailTile", 1, TileRailLauncher.class, TrackLauncher.class),
    PRIMING(Module.EXTRAS, 68, "priming", "RCPrimingRailTile", 8, TileRailPriming.class, TrackPriming.class),
    JUNCTION(Module.TRACKS, 7 + 32, "junction", "RCJunctionRailTile", 8, TileRailJunction.class, TrackJunction.class),
    SWITCH(Module.SIGNALS, 3 * 16, "switch", "RCSwitchRailTile", 8, TileRailSwitch.class, TrackSwitch.class),
    DISEMBARK(Module.TRACKS, 116, "disembarking", "RCDisembarkingRailTile", 8, TileRailDisembark.class, TrackDisembark.class),
    SUSPENDED(Module.EXTRAS, 253, "suspended", "RCSuspendedRailTile", 8, TileRailSuspended.class, TrackSuspended.class),
    GATED_ONEWAY(Module.TRACKS, 132, "gated.oneway", "RCGatedOneWayRailTile", 4, TileRailGatedOneWay.class, TrackGatedOneWay.class),
    GATED(Module.TRACKS, 254, "gated", "RCGatedRailTile", 4, TileRailGated.class, TrackGated.class),
    SLOW(Module.TRACKS_WOOD, 6 + 16, "slow", "RCSlowRailTile", 16, TileRailSlow.class, TrackSlow.class),
    SLOW_BOOSTER(Module.TRACKS_WOOD, 52, "slow.boost", "RCSlowBoosterRailTile", 8, TileRailSlowBooster.class, TrackSlowBooster.class),
    SLOW_JUNCTION(Module.TRACKS_WOOD, 6 + 32, "slow.junction", "RCSlowJunctionRailTile", 8, TileRailSlowJunction.class, TrackSlowJunction.class),
    SLOW_SWITCH(Module.SIGNALS, 4 * 16, "slow.switch", "RCSlowSwitchRailTile", 8, TileRailSlowSwitch.class, TrackSlowSwitch.class),
    SPEED(Module.TRACKS_HIGHSPEED, 7 + 16, "speed", "RCSpeedRailTile", 16, TileRailSpeed.class, TrackSpeed.class),
    SPEED_BOOST(Module.TRACKS_HIGHSPEED, 20, "speed.boost", "RCSpeedBoostRailTile", 8, TileRailSpeedBoost.class, TrackSpeedBoost.class),
    SPEED_TRANSITION(Module.TRACKS_HIGHSPEED, 2 * 16, "speed.transition", "RCSpeedTransitionRailTile", 8, TileRailSpeedTransition.class, TrackSpeedTransition.class),
    SPEED_SWITCH(Module.SIGNALS, 5 * 16, "speed.switch", "RCSpeedSwitchRailTile", 8, TileRailSpeedSwitch.class, TrackSpeedSwitch.class),
    BOARDING_TRAIN(Module.TRAIN, 6 * 16, "boarding.train", "RCBoardingRailTrainTile", 8, TileRailBoardingTrain.class, TrackBoardingTrain.class),
    HOLDING_TRAIN(Module.TRAIN, 148, "holding.train", "RCHoldingRailTrainTile", 8, TileRailHoldingTrain.class, TrackHoldingTrain.class),
    COUPLER(Module.TRACKS, 7 * 16, "coupler", "RCCouplerTrackTile", 8, TileRailCoupler.class, TrackCoupler.class),
    DECOUPLER(Module.TRACKS, 7 * 16 + 2, "decoupler", "RCDecouplerTrackTile", 8, TileRailDecoupler.class, TrackDecoupler.class),
    REINFORCED(Module.TRACKS_REINFORCED, 8 + 16, "reinforced", "", 16, null, TrackReinforced.class),
    REINFORCED_BOOSTER(Module.TRACKS_REINFORCED, 164, "reinforced.boost", "", 8, null, TrackReinforcedBooster.class),
    REINFORCED_JUNCTION(Module.TRACKS_REINFORCED, 8 + 32, "reinforced.junction", "", 8, null, TrackReinforcedJunction.class),
    REINFORCED_SWITCH(Module.TRACKS_REINFORCED, 8 * 16, "reinforced.switch", "", 8, null, TrackReinforcedSwitch.class),
    BUFFER_STOP(Module.TRACKS, 180, "buffer.stop", "", 8, null, TrackBufferStop.class),
    DISPOSAL(Module.TRACKS, 252, "disposal", "", 8, null, TrackDisposal.class),
    DETECTOR_DIRECTION(Module.TRACKS, 9 * 16, "detector.direction", "", 8, null, TrackDetectorDirection.class),
    EMBARKING(Module.TRACKS, 196, "embarking", "", 8, null, TrackEmbarking.class),
    WYE(Module.TRACKS, 10 * 16, "wye", "", 8, null, TrackWye.class),
    SLOW_WYE(Module.TRACKS, 162, "slow.wye", "", 8, null, TrackSlowWye.class),
    REINFORCED_WYE(Module.TRACKS, 11 * 16, "reinforced.wye", "", 8, null, TrackReinforcedWye.class),
    SPEED_WYE(Module.TRACKS, 178, "speed.wye", "", 8, null, TrackSpeedWye.class),
    LOCKDOWN(Module.TRACKS, 228, "lockdown", "", 8, null, TrackLockdown.class),
    LOCKDOWN_TRAIN(Module.TRAIN, 244, "lockdown.train", "", 8, null, TrackLockdownTrain.class),
    WHISTLE(Module.LOCOMOTIVES, 54, "whistle", "", 8, null, TrackWhistle.class),
    LOCOMOTIVE(Module.LOCOMOTIVES, 16 * 12, "locomotive", "", 8, null, TrackLocomotive.class),
    LIMITER(Module.LOCOMOTIVES, 16 * 14, "limiter", "", 8, null, TrackLimiter.class),;
    private final Module module;
    private final String tag;
    private final String tileTag;
    private final int recipeOutput;
    private final Class<? extends TileEntity> tile;
    private final int textureIndex;
    private final TrackSpec trackSpec;
    private static final List<EnumTrack> creativeList = new ArrayList<EnumTrack>(50);

    static {
        TrackRegistry.registerTrackSpec(new TrackSpec((short) -1, "track.default", "/terrain.png", 8 * 16, TrackDefault.class));

        creativeList.add(SWITCH);
        creativeList.add(WYE);
        creativeList.add(JUNCTION);
        creativeList.add(CONTROL);
        creativeList.add(BOARDING);
        creativeList.add(BOARDING_TRAIN);
        creativeList.add(HOLDING);
        creativeList.add(HOLDING_TRAIN);
        creativeList.add(LOCKDOWN);
        creativeList.add(LOCKDOWN_TRAIN);
        creativeList.add(DISEMBARK);
        creativeList.add(EMBARKING);
        creativeList.add(COUPLER);
        creativeList.add(DECOUPLER);
        creativeList.add(WHISTLE);
        creativeList.add(LOCOMOTIVE);
        creativeList.add(LIMITER);
        creativeList.add(BUFFER_STOP);
        creativeList.add(ONEWAY);
        creativeList.add(DETECTOR_DIRECTION);
        creativeList.add(GATED_ONEWAY);
        creativeList.add(GATED);
        creativeList.add(SUSPENDED);
        creativeList.add(DISPOSAL);
        creativeList.add(SLOW);
        creativeList.add(SLOW_BOOSTER);
        creativeList.add(SLOW_JUNCTION);
        creativeList.add(SLOW_SWITCH);
        creativeList.add(SLOW_WYE);
        creativeList.add(REINFORCED);
        creativeList.add(REINFORCED_BOOSTER);
        creativeList.add(REINFORCED_JUNCTION);
        creativeList.add(REINFORCED_SWITCH);
        creativeList.add(REINFORCED_WYE);
        creativeList.add(SPEED);
        creativeList.add(SPEED_BOOST);
        creativeList.add(SPEED_TRANSITION);
        creativeList.add(SPEED_SWITCH);
        creativeList.add(SPEED_WYE);
        creativeList.add(PRIMING);
        creativeList.add(LAUNCHER);
    }

    private EnumTrack(Module module, int textureIndex, String tag, String tileTag, int recipeOutput, Class<? extends TileEntity> tile, Class<? extends TrackBaseRailcraft> track) {
        this.module = module;
        this.textureIndex = textureIndex;
        this.tag = tag;
        this.tileTag = tileTag;
        this.recipeOutput = recipeOutput;
        this.tile = tile;

        String textureFile = RailcraftConstants.TRACK_TEXTURE_FILE;
        trackSpec = new TrackSpec((short) ordinal(), getTag(), textureFile, textureIndex, track);
        TrackRegistry.registerTrackSpec(trackSpec);
    }

    public boolean isEnabled() {
        return ModuleManager.isModuleLoaded(module) && RailcraftConfig.isSubBlockEnabled(getTag());
    }

    public ItemStack getItem() {
        return getItem(1);
    }

    public ItemStack getItem(int qty) {
        return new ItemStack(RailcraftBlocks.getBlockTrack(), qty, ordinal());
    }

    public TrackSpec getTrackSpec() {
        return trackSpec;
    }

    public String getTag() {
        return "rc.track." + tag;
    }

    public String getTileTag() {
        return tileTag;
    }

    public Class<? extends TileEntity> getTileClass() {
        return tile;
    }

    public TileEntity getBlockEntity() {
        try {
            return tile.newInstance();
        } catch (InstantiationException ex) {
        } catch (IllegalAccessException ex) {
        }
        return null;
    }

    public int getTextureIndex() {
        return textureIndex;
    }

    public static EnumTrack fromId(int id) {
        if (id < 0 || id >= EnumTrack.values().length) {
            id = 0;
        }
        return EnumTrack.values()[id];
    }

    public static List<EnumTrack> getCreativeList() {
        return creativeList;
    }

    public ItemStack registerRecipe() {
        if (getItem() == null) {
            return null;
        }
        ItemStack output = getItem(recipeOutput * 2);
        ItemStack railWood = RailcraftPartItems.getRailWood();
        ItemStack railStandard = RailcraftPartItems.getRailStandard();
        ItemStack railAdvanced = RailcraftPartItems.getRailAdvanced();
        ItemStack railSpeed = RailcraftPartItems.getRailSpeed();
        ItemStack railReinforced = RailcraftPartItems.getRailReinforced();
        ItemStack woodTie = RailcraftPartItems.getTieWood();
        ItemStack woodRailbed = RailcraftPartItems.getRailbedWood();
        ItemStack stoneRailbed = RailcraftPartItems.getRailbedStone();
        ItemStack reinforcedRailbed = RailcraftConfig.useOldRecipes() ? new ItemStack(Block.obsidian) : RailcraftPartItems.getRailbedStone();

        ItemStack crowbar = RailcraftToolItems.getCrowbar();
        crowbar.setItemDamage(-1);

        switch (this) {
            case BOARDING:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IrI",
                            "IbI",
                            "IsI",
                            'I', railAdvanced,
                            's', woodRailbed,
                            'r', Item.redstone,
                            'b', Block.pressurePlateStone,});
                break;
            case BOARDING_TRAIN:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IrI",
                            "IbI",
                            "IsI",
                            'I', railAdvanced,
                            's', woodRailbed,
                            'r', Item.redstoneRepeater,
                            'b', Block.pressurePlateStone,});
                break;
            case ONEWAY:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IbI",
                            "IsI",
                            "IpI",
                            'I', railStandard,
                            's', woodRailbed,
                            'b', Block.pressurePlateStone,
                            'p', Block.pistonBase,});
                break;
            case CONTROL:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IrI",
                            "GsG",
                            "IrI",
                            'I', railStandard,
                            'G', railAdvanced,
                            's', woodRailbed,
                            'r', Item.redstone,});
                break;
            case HOLDING:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IsI",
                            "IbI",
                            "IrI",
                            'I', railAdvanced,
                            's', woodRailbed,
                            'r', Item.redstone,
                            'b', Block.pressurePlateStone,});
                break;
            case HOLDING_TRAIN:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IsI",
                            "IbI",
                            "IrI",
                            'I', railAdvanced,
                            's', woodRailbed,
                            'r', Item.redstoneRepeater,
                            'b', Block.pressurePlateStone,});
                break;
            case LOCKDOWN:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IbI",
                            "IrI",
                            "IsI",
                            'I', railAdvanced,
                            's', woodRailbed,
                            'r', Item.redstone,
                            'b', Block.pressurePlateStone,});
                break;
            case LOCKDOWN_TRAIN:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IbI",
                            "IrI",
                            "IsI",
                            'I', railAdvanced,
                            's', woodRailbed,
                            'r', Item.redstoneRepeater,
                            'b', Block.pressurePlateStone,});
                break;
            case SPEED:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I I",
                            "IsI",
                            "I I",
                            'I', railSpeed,
                            's', stoneRailbed,});
                break;
            case SPEED_BOOST:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IrI",
                            "IsI",
                            "IrI",
                            'I', railSpeed,
                            's', stoneRailbed,
                            'r', Item.redstone,});
                break;
            case SPEED_TRANSITION:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IrI",
                            "IrI",
                            "IsI",
                            'I', railSpeed,
                            's', stoneRailbed,
                            'r', Item.redstone,});
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IsI",
                            "IrI",
                            "IrI",
                            'I', railSpeed,
                            's', stoneRailbed,
                            'r', Item.redstone,});
                break;
            case SPEED_SWITCH:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IsI",
                            "III",
                            "III",
                            'I', railSpeed,
                            's', stoneRailbed,});
                break;
            case LAUNCHER:
                CraftingPlugin.addShapedOreRecipe(output, new Object[]{
                            "IsI",
                            "BPB",
                            "IsI",
                            'I', railReinforced,
                            'B', "blockSteel",
                            's', stoneRailbed,
                            'P', Block.pistonBase,});
                break;
            case PRIMING:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IpI",
                            "IsI",
                            "IfI",
                            'I', railReinforced,
                            's', stoneRailbed,
                            'p', Block.pressurePlateStone,
                            'f', Item.flintAndSteel,});
                break;
            case JUNCTION:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "III",
                            "I#I",
                            "III",
                            'I', railStandard,
                            '#', woodRailbed,});
                break;
            case SLOW:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I I",
                            "I#I",
                            "I I",
                            'I', railWood,
                            '#', woodRailbed,});
                break;
            case SLOW_BOOSTER:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I I",
                            "G#G",
                            "IrI",
                            'G', Item.ingotGold,
                            'I', railWood,
                            '#', woodRailbed,
                            'r', Item.redstone,});
                break;
            case SLOW_SWITCH:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I#I",
                            "III",
                            "III",
                            'I', railWood,
                            '#', woodRailbed,});
                break;
            case SLOW_JUNCTION:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "III",
                            "I#I",
                            "III",
                            'I', railWood,
                            '#', woodRailbed,});
                break;
            case SWITCH:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I#I",
                            "III",
                            "III",
                            'I', railStandard,
                            '#', woodRailbed,});
                break;
            case WYE:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "III",
                            "II#",
                            "III",
                            'I', railStandard,
                            '#', woodRailbed,});
                break;
            case SLOW_WYE:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "III",
                            "II#",
                            "III",
                            'I', railWood,
                            '#', woodRailbed,});
                break;
            case REINFORCED_WYE:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "III",
                            "II#",
                            "III",
                            'I', railReinforced,
                            '#', stoneRailbed,});
                break;
            case SPEED_WYE:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "III",
                            "II#",
                            "III",
                            'I', railSpeed,
                            '#', stoneRailbed,});
                break;
            case DISEMBARK:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IpI",
                            "I#I",
                            "IrI",
                            'I', railAdvanced,
                            '#', woodRailbed,
                            'r', Item.redstone,
                            'p', Block.pressurePlateStone,});
                break;
            case EMBARKING:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IpI",
                            "I#I",
                            "IpI",
                            'I', railAdvanced,
                            '#', woodRailbed,
                            'p', Item.enderPearl,});
                break;
            case SUSPENDED:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "ItI",
                            "ItI",
                            "ItI",
                            'I', railStandard,
                            't', woodTie,});
                break;
            case DISPOSAL:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "ItI",
                            "IPI",
                            "ItI",
                            'I', railStandard,
                            'P', RailcraftPartItems.getPlateSteel(),
                            't', woodTie,});
                break;
            case BUFFER_STOP:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I I",
                            "I#I",
                            "IbI",
                            'I', railStandard,
                            '#', woodRailbed,
                            'b', Block.blockSteel,});
                break;
            case DETECTOR_DIRECTION:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IrI",
                            "I#I",
                            "IsI",
                            'I', railStandard,
                            '#', woodRailbed,
                            'r', Item.redstone,
                            's', Block.pressurePlateStone,});
                break;
            case GATED:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IgI",
                            "I#I",
                            "IgI",
                            'I', railStandard,
                            '#', woodRailbed,
                            'g', Block.fenceGate,});
                break;
            case GATED_ONEWAY:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IgI",
                            "G#G",
                            "IgI",
                            'I', railStandard,
                            '#', woodRailbed,
                            'g', Block.fenceGate,
                            'G', railAdvanced,});
                break;
            case COUPLER:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I I",
                            "I#I",
                            "IcI",
                            'I', railAdvanced,
                            '#', woodRailbed,
                            'c', crowbar});
                break;
            case DECOUPLER:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "IcI",
                            "I#I",
                            "I I",
                            'I', railAdvanced,
                            '#', woodRailbed,
                            'c', crowbar});
                break;
            case WHISTLE:
                CraftingPlugin.addShapedOreRecipe(output,
                        "IyI",
                        "I#I",
                        "IbI",
                        'I', railStandard,
                        '#', woodRailbed,
                        'y', "dyeYellow",
                        'b', "dyeBlack");
                break;
            case LOCOMOTIVE:
                CraftingPlugin.addShapedRecipe(output,
                        "IlI",
                        "I#I",
                        "IlI",
                        'I', railStandard,
                        '#', woodRailbed,
                        'l', RailcraftPartItems.getSignalLamp());
                break;
            case LIMITER:
                CraftingPlugin.addShapedRecipe(output,
                        "IlI",
                        "I#I",
                        "IlI",
                        'I', railStandard,
                        '#', woodRailbed,
                        'l', Item.redstoneRepeater);
                break;
            case REINFORCED:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I I",
                            "I#I",
                            "I I",
                            'I', railReinforced,
                            '#', reinforcedRailbed});
                break;
            case REINFORCED_BOOSTER:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I I",
                            "I#I",
                            "IrI",
                            'I', railReinforced,
                            'r', new ItemStack(Item.redstone),
                            '#', reinforcedRailbed});
                break;
            case REINFORCED_JUNCTION:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "III",
                            "I#I",
                            "III",
                            'I', railReinforced,
                            '#', reinforcedRailbed});
                break;
            case REINFORCED_SWITCH:
                CraftingPlugin.addShapedRecipe(output, new Object[]{
                            "I#I",
                            "III",
                            "III",
                            'I', railReinforced,
                            '#', reinforcedRailbed});
                break;
        }
        return output;
    }
}
