package railcraft.common.blocks.tracks;

import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.core.items.ITrackItem;
import railcraft.common.api.tracks.ITrackCustomPlaced;
import railcraft.common.api.tracks.ITrackInstance;
import railcraft.common.api.tracks.TrackRegistry;
import railcraft.common.api.tracks.TrackSpec;
import railcraft.common.blocks.RailcraftBlocks;

public class ItemTrack extends ItemBlock implements ITrackItem {

    private String textureFile = "";

    public ItemTrack(int id) {
        super(id);
        setIconIndex(9);
        setMaxDamage(0);
        setHasSubtypes(true);
        setItemName("rc.track");
    }

    @Override
    public int getIconFromDamage(int damage) {
        TrackSpec trackSpec = TrackRegistry.getTrackSpec(damage);
        textureFile = trackSpec.getTextureFile();
        return trackSpec.getTextureIndex();
    }

    @Override
    public int getMetadata(int i) {
        return 0;
    }

    @Override
    public String getItemNameIS(ItemStack stack) {
        return TrackRegistry.getTrackSpec(stack.getItemDamage()).getTrackTag();
    }

    @Override
    public String getTextureFile() {
        return textureFile;
    }

    @Override
    public int getPlacedBlockId() {
        return RailcraftBlocks.getBlockTrack().blockID;
    }

    @Override
    public boolean isPlacedTileEntity(ItemStack stack, TileEntity tile) {
        if (tile instanceof TileTrack) {
            TileTrack track = (TileTrack) tile;
            if (track.getTrackInstance().getTrackSpec().getTrackId() == stack.getItemDamage()) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean placeTrack(ItemStack stack, World world, int i, int j, int k) {
        return placeTrack(stack, world, i, j, k, 1);
    }

    private boolean placeTrack(ItemStack stack, World world, int i, int j, int k, int side) {
        Block blockTrack = RailcraftBlocks.getBlockTrack();
        if (blockTrack == null) {
            return false;
        }
        if (j >= world.getHeight() - 1) {
            return false;
        }
        if (stack == null || !(stack.getItem() instanceof ItemTrack)) {
            return false;
        }
        TrackSpec spec = TrackRegistry.getTrackSpec(stack.getItemDamage());
        ITrackInstance track = spec.createInstanceFromSpec();
        boolean canPlace = world.canPlaceEntityOnSide(blockTrack.blockID, i, j, k, true, side, null);
        if (track instanceof ITrackCustomPlaced) {
            canPlace &= ((ITrackCustomPlaced) track).canPlaceRailAt(world, i, j, k);
        } else {
            canPlace &= world.isBlockSolidOnSide(i, j - 1, k, ForgeDirection.UP);
        }
        if (canPlace) {
            boolean placed = world.setBlockWithNotify(i, j, k, blockTrack.blockID);
            // System.out.println("Block placement attempted");
            if (placed) {
                if (world.getBlockId(i, j, k) == blockTrack.blockID) {
                    TileTrack tile = new TileTrack(track);
                    world.setBlockTileEntity(i, j, k, tile);
                    blockTrack.onPostBlockPlaced(world, i, j, k, 0);
                    world.markBlockForUpdate(i, j, k);
                }
                world.playSoundEffect((float) i + 0.5F, (float) j + 0.5F, (float) k + 0.5F, blockTrack.stepSound.getStepSound(), (blockTrack.stepSound.getVolume() + 1.0F) / 2.0F, blockTrack.stepSound.getPitch() * 0.8F);
            }
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean onItemUse(ItemStack stack, EntityPlayer player, World world, int x, int y, int z, int side, float par8, float par9, float par10) {
        if (stack.stackSize <= 0) {
            return false;
        }

        int id = world.getBlockId(x, y, z);
        if (id == Block.snow.blockID) {
            side = 1;
        } else if (id != Block.vine.blockID && id != Block.tallGrass.blockID && id != Block.deadBush.blockID
                && (Block.blocksList[id] == null || !Block.blocksList[id].isBlockReplaceable(world, x, y, z))) {
            if (side == 0) {
                y--;
            }
            if (side == 1) {
                y++;
            }
            if (side == 2) {
                z--;
            }
            if (side == 3) {
                z++;
            }
            if (side == 4) {
                x--;
            }
            if (side == 5) {
                x++;
            }
        }

        if (player != null && !player.canPlayerEdit(x, y, z, side, stack)) {
            return false;
        }

        boolean success = placeTrack(stack, world, x, y, z, side);
        if (success) {
            Block blockTrack = RailcraftBlocks.getBlockTrack();
            if (player != null) {
                blockTrack.onBlockPlacedBy(world, x, y, z, player);
            }
            stack.stackSize--;
        }
        return success;
    }
}
