package railcraft.common.blocks.tracks;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.api.tracks.ITrackInstance;
import railcraft.common.api.tracks.ITrackTile;
import railcraft.common.api.tracks.TrackRegistry;
import railcraft.common.api.tracks.TrackSpec;
import railcraft.common.blocks.RailcraftTileEntity;
import railcraft.common.core.Railcraft;
import railcraft.common.util.network.IGuiReturnHandler;

public class TileTrack extends RailcraftTileEntity implements ITrackTile, IGuiReturnHandler
{

    public ITrackInstance track;

    public TileTrack() {
    }

    public TileTrack(ITrackInstance t) {
        track = t;
        track.setTile(this);
    }

    @Override
    public String getInvName() {
        return Railcraft.getProxy().getItemDisplayName(getTrackInstance().getTrackSpec().getItem());
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);

        data.setInteger("trackId", getId());

        track.writeToNBT(data);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);

        TrackSpec spec = TrackRegistry.getTrackSpec(data.getInteger("trackId"));
        track = spec.createInstanceFromSpec();
        track.setTile(this);

        track.readFromNBT(data);
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        if(track != null) {
            track.writePacketData(data);
        }
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        if(track != null) {
            track.readPacketData(data);
        }
    }

    @Override
    public boolean canUpdate() {
        return track.canUpdate();
    }

    @Override
    public void updateEntity() {
        track.updateEntity();
    }

    @Override
    public short getId() {
        return (short)track.getTrackSpec().getTrackId();
    }

    @Override
    public ITrackInstance getTrackInstance() {
        track.setTile(this);
        return track;
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        if(track instanceof IGuiReturnHandler) {
            ((IGuiReturnHandler)track).writeGuiData(data);
        }
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        if(track instanceof IGuiReturnHandler) {
            ((IGuiReturnHandler)track).readGuiData(data);
        }
    }
}
