package railcraft.common.blocks.tracks;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.tracks.ITrackLockdown;
import railcraft.common.api.tracks.ITrackPowered;
import railcraft.common.api.tracks.ITrackReversable;
import railcraft.common.util.misc.Game;

public class TrackBoarding extends TrackLockingBase implements ITrackPowered, ITrackReversable, ITrackLockdown {

    protected boolean reversed = false;

    @Override
    public EnumTrack getTrackType() {
        return EnumTrack.BOARDING;
    }

    @Override
    public int getTextureIndex() {
        if (isPowered() || delay > 0) {
            if (isReversed()) {
                return getTrackType().getTextureIndex() + 2;
            }
            return getTrackType().getTextureIndex();
        }
        if (isReversed()) {
            return getTrackType().getTextureIndex() + 3;
        }
        return getTrackType().getTextureIndex() + 1;
    }

    @Override
    public void updateEntity() {
        if (getCurrentCart() != null && getCurrentCart().isDead) {
            setCurrentCart(null);
        }
        if (Game.isHost(getWorld())) {
            if (isPowered()) {
                delay = getDelayTime();
            } else if (delay > 0) {
                delay--;
                if (delay == 0) {
                    setCurrentCart(null);
                }
            }
        }

        if (prevDelay == 0 ^ delay == 0) {
            sendUpdateToClient();
        }
        prevDelay = delay;
    }

    @Override
    public void onMinecartPass(EntityMinecart cart) {
        checkCart(cart);
        int meta = tileEntity.getBlockMetadata();
        if (isPowered() || delay > 0) {
            releaseCart(cart);
            double speed = CartTools.getCartSpeedUncapped(cart);
            double boostX = START_BOOST;
            double boostZ = START_BOOST;
            if (speed > 0.005D) {
                boostX = (Math.abs(cart.motionX) / speed) * BOOST_FACTOR;
                boostZ = (Math.abs(cart.motionZ) / speed) * BOOST_FACTOR;
            }
            if (meta == 0 || meta == 4 || meta == 5) {
                if (isReversed()) {
                    cart.motionZ += boostZ;
                } else {
                    cart.motionZ -= boostZ;
                }
            } else if (meta == 1 || meta == 2 || meta == 3) {
                if (isReversed()) {
                    cart.motionX -= boostX;
                } else {
                    cart.motionX += boostX;
                }
            }
        } else {
            if (getCurrentCart() == null) {
                setCurrentCart(cart);
            }
            if (getCurrentCart() == cart) {
                lockCart(cart);
                cart.motionX = 0.0D;
                cart.motionZ = 0.0D;
                if (meta == 0 || meta == 4 || meta == 5) {
                    cart.posZ = tileEntity.zCoord + 0.5D;
                } else {
                    cart.posX = tileEntity.xCoord + 0.5D;
                }
            } else {
//                double distX = cart.posX - (i + 0.5D);
//                double distZ = cart.posZ - (k + 0.5D);
//                if(Math.abs(distX) < 1.0 && Math.abs(distX) > 0.0) {
//                    cart.motionX += (1.0 / distX) * 0.01;
//                }
//                if(Math.abs(distZ) < 1.0 && Math.abs(distZ) > 0.0) {
//                    cart.motionZ += (1.0 / distZ) * 0.01;
//                }
            }
        }
    }

    @Override
    public boolean isReversed() {
        return reversed;
    }

    @Override
    public void setReversed(boolean reversed) {
        this.reversed = reversed;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setBoolean("direction", reversed);
//        data.setByte("delay", delay);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        reversed = data.getBoolean("direction");
//        delay = data.getByte("delay");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeBoolean(reversed);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        reversed = data.readBoolean();

        markBlockNeedsUpdate();
    }
}
