package railcraft.common.blocks.tracks;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import railcraft.common.api.tracks.ITrackCustomShape;
import railcraft.common.api.tracks.ITrackReversable;
import railcraft.common.util.misc.MiscTools;

public class TrackBufferStop extends TrackBaseRailcraft implements ITrackReversable, ITrackCustomShape
{

    private static final float CBOX = 0.0625f;
    private static final float SBOX = 0.0625f * 3;
    private static final float SBOXY = 0.0625f * 5;
    private boolean reversed = false;

    @Override
    public EnumTrack getTrackType()
    {
        return EnumTrack.BUFFER_STOP;
    }

    @Override
    public int getTextureIndex()
    {
        if(reversed) {
            return getTrackType().getTextureIndex() + 1;
        }
        return getTrackType().getTextureIndex();
    }

    @Override
    public AxisAlignedBB getSelectedBoundingBoxFromPool()
    {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(tileEntity.xCoord + SBOX, tileEntity.yCoord, tileEntity.zCoord + SBOX, tileEntity.xCoord + 1 - SBOX, tileEntity.yCoord + 1 - SBOXY, tileEntity.zCoord + 1 - SBOX);
    }

    @Override
    public MovingObjectPosition collisionRayTrace(Vec3 vec3d, Vec3 vec3d1)
    {
        return MiscTools.collisionRayTrace(vec3d, vec3d1, getX(), getY(), getZ());
    }

    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool()
    {
        return AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(tileEntity.xCoord + CBOX, tileEntity.yCoord, tileEntity.zCoord + CBOX, tileEntity.xCoord + 1 - CBOX, tileEntity.yCoord + 1, tileEntity.zCoord + 1 - CBOX);
    }

//    @Override
//    public void onMinecartPass(EntityMinecart cart)
//    {
//        int meta = tileEntity.getBlockMetadata();
//        if(meta == 1) {
//            if((isReversed() && cart.motionX > 0.0D) || (!isReversed() && cart.motionX < 0.0D)) {
//                double distX = cart.posX - (tileEntity.xCoord + 0.5D);
////                    System.out.println("cartX=" + cart.posX + ", railX=" + (i + 0.5D) + ", railDir=" + isReversed());
//                if(!isReversed() && distX < -0.01 || isReversed() && distX > 0.01) {
////                        System.out.println("Setting Position");
//                    cart.setPosition(tileEntity.xCoord + 0.5D, cart.posY, cart.posZ);
//                }
////                    System.out.println("mX= " + cart.motionX + ", dist=" + distX);
//                if(!isReversed()) {
//                    cart.motionX = Math.abs(cart.motionX) * LOSS_FACTOR;
//                } else {
//                    cart.motionX = -Math.abs(cart.motionX) * LOSS_FACTOR;
//                }
//            }
//        } else if(meta == 0) {
//            if((isReversed() && cart.motionZ < 0.0D) || (!isReversed() && cart.motionZ > 0.0D)) {
//                double distZ = cart.posZ - (tileEntity.zCoord + 0.5D);
////                    System.out.println("cartZ=" + cart.posZ + ", railZ=" + (k + 0.5D) + ", railDir=" + isReversed());
//                if(isReversed() && distZ < -0.01 || !isReversed() && distZ > 0.01) {
////                        System.out.println("Setting Position");
//                    cart.setPosition(cart.posX, cart.posY, tileEntity.zCoord + 0.5D);
//                }
////                    System.out.println("mZ= " + cart.motionZ + ", dist=" + distZ);
//                if(isReversed()) {
//                    cart.motionZ = Math.abs(cart.motionZ) * LOSS_FACTOR;
//                } else {
//                    cart.motionZ = -Math.abs(cart.motionZ) * LOSS_FACTOR;
//                }
//            }
//        }
//    }
    @Override
    public boolean canMakeSlopes()
    {
        return false;
    }

    @Override
    public boolean isReversed()
    {
        return reversed;
    }

    @Override
    public void setReversed(boolean reversed)
    {
        this.reversed = reversed;
    }

    @Override
    public void writeToNBT(NBTTagCompound nbttagcompound)
    {
        super.writeToNBT(nbttagcompound);
        nbttagcompound.setBoolean("direction", reversed);
    }

    @Override
    public void readFromNBT(NBTTagCompound nbttagcompound)
    {
        super.readFromNBT(nbttagcompound);
        reversed = nbttagcompound.getBoolean("direction");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException
    {
        super.writePacketData(data);

        data.writeBoolean(reversed);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException
    {
        super.readPacketData(data);

        boolean r = data.readBoolean();

        if(reversed != r) {
            reversed = r;
            markBlockNeedsUpdate();
        }
    }
}
