package railcraft.common.blocks.tracks;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.api.tracks.ITrackEmitter;
import railcraft.common.api.tracks.ITrackReversable;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.util.misc.Game;

public class TrackDetectorDirection extends TrackBaseRailcraft implements ITrackReversable, ITrackEmitter
{

    private boolean reversed = false;
    private byte delay = 0;

    @Override
    public EnumTrack getTrackType()
    {
        return EnumTrack.DETECTOR_DIRECTION;
    }

    @Override
    public int getTextureIndex()
    {
        if(isTrackPowering()) {
            if(isReversed()) {
                return getTrackType().getTextureIndex() + 3;
            }
            return getTrackType().getTextureIndex() + 1;
        }
        if(isReversed()) {
            return getTrackType().getTextureIndex() + 2;
        }
        return getTrackType().getTextureIndex();
    }

    @Override
    public boolean canUpdate()
    {
        return true;
    }

    @Override
    public void updateEntity()
    {
        if(Game.isNotHost(getWorld())) {
            return;
        }
        if(delay > 0) {
            delay--;
            if(delay == 0) {
                notifyNeighbors();
            }
        }
    }

    @Override
    public void onMinecartPass(EntityMinecart cart)
    {
        int meta = tileEntity.getBlockMetadata();
        if(meta == 1 || meta == 2 || meta == 3) {
            if((isReversed() && cart.motionX < 0.0D) || (!isReversed() && cart.motionX > 0.0D)) {
                setTrackPowering();
            }
        } else if(meta == 0 || meta == 4 || meta == 5) {
            if((isReversed() && cart.motionZ > 0.0D) || (!isReversed() && cart.motionZ < 0.0D)) {
                setTrackPowering();
            }
        }
    }

    private void notifyNeighbors()
    {
        getWorld().notifyBlocksOfNeighborChange(getX(), getY(), getZ(), RailcraftBlocks.getBlockTrack().blockID);
        getWorld().notifyBlocksOfNeighborChange(getX(), getY() - 1, getZ(), RailcraftBlocks.getBlockTrack().blockID);
        sendUpdateToClient();
    }

    private void setTrackPowering()
    {
        boolean notify = delay == 0;
        delay = 3;
        if(notify) {
            notifyNeighbors();
        }
    }

    @Override
    public boolean isTrackPowering()
    {
        return delay > 0;
    }

    @Override
    public boolean isReversed()
    {
        return reversed;
    }

    @Override
    public void setReversed(boolean reversed)
    {
        this.reversed = reversed;
    }

    @Override
    public void writeToNBT(NBTTagCompound nbttagcompound)
    {
        super.writeToNBT(nbttagcompound);
        nbttagcompound.setBoolean("direction", reversed);
        nbttagcompound.setByte("delay", delay);
    }

    @Override
    public void readFromNBT(NBTTagCompound nbttagcompound)
    {
        super.readFromNBT(nbttagcompound);
        reversed = nbttagcompound.getBoolean("direction");
        delay = nbttagcompound.getByte("delay");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException
    {
        super.writePacketData(data);

        data.writeByte(delay);
        data.writeBoolean(reversed);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException
    {
        super.readPacketData(data);

        delay = data.readByte();
        reversed = data.readBoolean();

        markBlockNeedsUpdate();
    }
}
