package railcraft.common.blocks.tracks;

import net.minecraft.entity.item.EntityMinecart;
import railcraft.common.api.tracks.ITrackPowered;
import railcraft.common.api.tracks.ITrackReversable;

public class TrackGatedOneWay extends TrackGated implements ITrackReversable, ITrackPowered
{

    private static final double MOTION_MIN = 0.2;

    @Override
    public EnumTrack getTrackType()
    {
        return EnumTrack.GATED_ONEWAY;
    }

    @Override
    public int getTextureIndex()
    {
        int texture = getTrackType().getTextureIndex();
        if(isReversed()) {
            return texture + 1;
        }
        return texture;
    }

    @Override
    public void onMinecartPass(EntityMinecart cart)
    {
        if(isGateOpen()) {
            int meta = tileEntity.getBlockMetadata();
            if(meta == 0) {
                if(isReversed()) {
                    cart.motionZ = Math.max(Math.abs(cart.motionZ), MOTION_MIN);
                } else {
                    cart.motionZ = -Math.max(Math.abs(cart.motionZ), MOTION_MIN);
                }
            } else if(meta == 1) {
                if(isReversed()) {
                    cart.motionX = -Math.max(Math.abs(cart.motionX), MOTION_MIN);
                } else {
                    cart.motionX = Math.max(Math.abs(cart.motionX), MOTION_MIN);
                }
            }
        }
    }
}
