package railcraft.common.blocks.tracks;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.tracks.ITrackLockdown;
import railcraft.common.api.tracks.ITrackPowered;
import railcraft.common.util.misc.Game;

public class TrackHolding extends TrackLockingBase implements ITrackPowered, ITrackLockdown {

    protected static float DIR_THRESHOLD = 0.01f;
    protected boolean launchForward = true;
    protected byte reset = 0;

    @Override
    public EnumTrack getTrackType() {
        return EnumTrack.HOLDING;
    }

    @Override
    public void updateEntity() {
        if (getCurrentCart() != null && getCurrentCart().isDead) {
            setCurrentCart(null);
        }
        if (Game.isHost(getWorld())) {
            if (isPowered()) {
                delay = getDelayTime();
            } else if (delay > 0) {
                delay--;
                if (delay == 0) {
                    setCurrentCart(null);
                }
            }
            if (reset > 0) {
                reset--;
            }
        }

        if (prevDelay == 0 ^ delay == 0) {
            sendUpdateToClient();
        }
        prevDelay = delay;
    }

    @Override
    protected void checkCart(EntityMinecart cart) {
        if (delay > 0 && (cart != getCurrentCart() || reset == 0)) {
            delay = 0;
            setCurrentCart(cart);
            setLaunchDirection(cart);
        }
        reset = 10;
    }

    @Override
    public void onMinecartPass(EntityMinecart cart) {
        checkCart(cart);
        int meta = tileEntity.getBlockMetadata();
        if (isPowered() || delay > 0) {
            releaseCart(cart);
            double speed = CartTools.getCartSpeedUncapped(cart);
            double boostX = START_BOOST;
            double boostZ = START_BOOST;
            if (speed > 0.005D) {
                boostX = (Math.abs(cart.motionX) / speed) * BOOST_FACTOR;
                boostZ = (Math.abs(cart.motionZ) / speed) * BOOST_FACTOR;
            }
            if (meta == 0 || meta == 4 || meta == 5) {
                if (launchForward) {
                    cart.motionZ += boostZ;
                } else {
                    cart.motionZ -= boostZ;
                }
            } else if (meta == 1 || meta == 2 || meta == 3) {
                if (launchForward) {
                    cart.motionX += boostX;
                } else {
                    cart.motionX -= boostX;
                }
            }
        } else {
            if (getCurrentCart() == null) {
                setCurrentCart(cart);
                setLaunchDirection(cart);
            }
            if (getCurrentCart() == cart) {
                lockCart(cart);
                cart.motionX = 0.0D;
                cart.motionZ = 0.0D;
                if (meta == 0 || meta == 4 || meta == 5) {
                    cart.setPosition(cart.posX, cart.posY, tileEntity.zCoord + 0.5D);
                } else {
                    cart.setPosition(tileEntity.xCoord + 0.5D, cart.posY, cart.posZ);
                }
            } else {
//                double distX = cart.posX - (i + 0.5D);
//                double distZ = cart.posZ - (k + 0.5D);
//                if(Math.abs(distX) < 1.0 && Math.abs(distX) > 0.0) {
//                    cart.motionX += (1.0 / distX) * 0.01;
//                }
//                if(Math.abs(distZ) < 1.0 && Math.abs(distZ) > 0.0) {
//                    cart.motionZ += (1.0 / distZ) * 0.01;
//                }
            }
        }
    }

    protected void setLaunchDirection(EntityMinecart cart) {
        int meta = tileEntity.getBlockMetadata();
        double speed = CartTools.getCartSpeedUncapped(cart);
        if (speed > DIR_THRESHOLD) {
            boolean launch = launchForward;
            if (meta == 0 || meta == 4 || meta == 5) {
                launch = cart.motionZ > 0;
            } else if (meta == 1 || meta == 2 || meta == 3) {
                launch = cart.motionX > 0;
            }
            if (launchForward != launch) {
                launchForward = launch;
                sendUpdateToClient();
            }
        }
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setBoolean("launchForward", launchForward);
//        data.setByte("delay", delay);
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        launchForward = data.getBoolean("launchForward");
//        delay = data.getByte("delay");
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeBoolean(launchForward);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        launchForward = data.readBoolean();

        markBlockNeedsUpdate();
    }
}
