package railcraft.common.blocks.tracks;

import net.minecraft.entity.item.EntityMinecart;
import railcraft.common.api.tracks.ITrackLockdown;
import railcraft.common.api.tracks.ITrackPowered;
import railcraft.common.util.misc.Game;

public class TrackLockdown extends TrackLockingBase implements ITrackPowered, ITrackLockdown {

    @Override
    public EnumTrack getTrackType() {
        return EnumTrack.LOCKDOWN;
    }

    @Override
    public void updateEntity() {
        if (getCurrentCart() != null && getCurrentCart().isDead) {
            setCurrentCart(null);
        }
        if (Game.isHost(getWorld())) {
            if (isPowered()) {
                delay = getDelayTime();
            } else if (delay > 0) {
                delay--;
                if (delay == 0) {
                    setCurrentCart(null);
                }
            }
        }

        if (prevDelay == 0 ^ delay == 0) {
            sendUpdateToClient();
        }
        prevDelay = delay;
    }

    @Override
    public void onMinecartPass(EntityMinecart cart) {
        checkCart(cart);
        if (!isPowered() && delay <= 0) {
            if (getCurrentCart() == null) {
                setCurrentCart(cart);
            }
            if (getCurrentCart() == cart) {
                lockCart(cart);
                cart.motionX = 0.0D;
                cart.motionZ = 0.0D;
                int meta = tileEntity.getBlockMetadata();
                if (meta == 0 || meta == 4 || meta == 5) {
                    cart.posZ = tileEntity.zCoord + 0.5D;
                } else {
                    cart.posX = tileEntity.xCoord + 0.5D;
                }
            }
        } else {
            releaseCart(cart);
        }
    }
}
