/*
 * Copyright (c) CovertJaguar, 2011 http://railcraft.info
 * 
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.blocks.tracks;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.UUID;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.api.tracks.ITrackLockdown;
import railcraft.common.api.tracks.ITrackPowered;
import railcraft.common.carts.LinkageManager;
import railcraft.common.carts.Train;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class TrackLockingBase extends TrackBaseRailcraft implements ITrackLockdown, ITrackPowered {

    protected static double START_BOOST = 0.04;
    protected static double BOOST_FACTOR = 0.06;
    protected boolean powered = false;
    private EntityMinecart lockedCart;
    protected byte prevDelay = 0;
    protected byte delay = 0;
    private UUID uuid;

    @Override
    public int getTextureIndex() {
        if (isPowered() || delay > 0) {
            return getTrackType().getTextureIndex();
        }
        return getTrackType().getTextureIndex() + 1;
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    public EntityMinecart getCurrentCart() {
        return lockedCart;
    }

    @Override
    public void onBlockRemoved() {
        super.onBlockRemoved();
        setCurrentCart(null);
    }

    private UUID getUUID() {
        if (uuid == null) {
            uuid = UUID.randomUUID();
        }
        return uuid;
    }

    protected void setCurrentCart(EntityMinecart cart) {
        if (lockedCart != cart && lockedCart != null) {
            Train train = LinkageManager.getInstance(lockedCart.worldObj).getTrain(lockedCart);
            train.removeLockingTrack(getUUID());
        }
        lockedCart = cart;
    }

    protected void lockCart(EntityMinecart cart) {
        if (cart != null) {
            Train train = LinkageManager.getInstance(cart.worldObj).getTrain(cart);
            train.addLockingTrack(getUUID());
        }
    }

    protected void releaseCart(EntityMinecart cart) {
        if (cart != null) {
            Train train = LinkageManager.getInstance(cart.worldObj).getTrain(cart);
            train.removeLockingTrack(getUUID());
        }
    }

    protected void checkCart(EntityMinecart cart) {
        if (delay > 0 && cart != lockedCart) {
            delay = 0;
            setCurrentCart(cart);
        }
    }

    protected byte getDelayTime() {
        return 3;
    }

    @Override
    public void releaseCart() {
        delay = (byte) 10;
    }

    @Override
    public boolean isCartLockedDown(EntityMinecart cart) {
        return !powered && lockedCart == cart && delay == 0;
    }

    @Override
    public boolean isPowered() {
        return powered;
    }

    @Override
    public void setPowered(boolean powered) {
        this.powered = powered;
    }

    @Override
    public void writeToNBT(NBTTagCompound data) {
        super.writeToNBT(data);
        data.setBoolean("powered", powered);
        data.setLong("uuidHigh", getUUID().getMostSignificantBits());
        data.setLong("uuidLow", getUUID().getLeastSignificantBits());
    }

    @Override
    public void readFromNBT(NBTTagCompound data) {
        super.readFromNBT(data);
        powered = data.getBoolean("powered");

        if (data.hasKey("uuidHigh")) {
            uuid = new UUID(data.getLong("uuidHigh"), data.getLong("uuidLow"));
        }
    }

    @Override
    public void writePacketData(DataOutputStream data) throws IOException {
        super.writePacketData(data);

        data.writeBoolean(powered);
        data.writeByte(delay);
    }

    @Override
    public void readPacketData(DataInputStream data) throws IOException {
        super.readPacketData(data);

        powered = data.readBoolean();
        delay = data.readByte();

        markBlockNeedsUpdate();
    }
}
