package railcraft.common.blocks.tracks;

import net.minecraft.block.BlockRail;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.tracks.ITrackCustomPlaced;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.util.misc.Game;

public class TrackSuspended extends TrackBaseRailcraft implements ITrackCustomPlaced
{

    @Override
    public EnumTrack getTrackType()
    {
        return EnumTrack.SUSPENDED;
    }

    @Override
    public int getTextureIndex()
    {
        return getTrackSpec().getTextureIndex();
    }

    @Override
    public boolean canMakeSlopes()
    {
        return false;
    }

    @Override
    public void onBlockPlaced()
    {
        super.onBlockPlaced();
        if(!isSupported()) {
            breakRail();
        }
    }

    @Override
    public void onNeighborBlockChange(int id)
    {
        World world = getWorld();
        int i = tileEntity.xCoord;
        int j = tileEntity.yCoord;
        int k = tileEntity.zCoord;
        if(isSupported()) {
            int myId = RailcraftBlocks.getBlockTrack().blockID;
            if(id != myId) {
                world.notifyBlocksOfNeighborChange(i + 1, j, k, myId);
                world.notifyBlocksOfNeighborChange(i - 1, j, k, myId);
                world.notifyBlocksOfNeighborChange(i, j, k + 1, myId);
                world.notifyBlocksOfNeighborChange(i, j, k - 1, myId);
            }
        } else {
            breakRail();
        }
    }

    public void breakRail()
    {
        if(Game.isHost(getWorld())) {
            BlockTrack block = (BlockTrack)RailcraftBlocks.getBlockTrack();
            block.dropBlockAsItem(getWorld(), tileEntity.xCoord, tileEntity.yCoord, tileEntity.zCoord, 0, 0);
            getWorld().setBlockWithNotify(tileEntity.xCoord, tileEntity.yCoord, tileEntity.zCoord, 0);
        }
    }

    public boolean isSupportedRail(World world, int i, int j, int k, int meta)
    {
        if(!BlockRail.isRailBlockAt(world, i, j, k)) {
            return false;
        }
        if(isSupportedBelow(world, i, j, k)) {
            return true;
        }
        if(meta == EnumTrackMeta.NORTH_SOUTH.ordinal()) {
            if(isSupportedBelow(world, i, j, k + 1)) {
                return true;
            }
            if(isSupportedBelow(world, i, j, k - 1)) {
                return true;
            }
            return false;
        } else if(meta == EnumTrackMeta.EAST_WEST.ordinal()) {
            if(isSupportedBelow(world, i + 1, j, k)) {
                return true;
            }
            if(isSupportedBelow(world, i - 1, j, k)) {
                return true;
            }
            return false;
        }
        return false;
    }

    public boolean isSupportedBelow(World world, int i, int j, int k)
    {
        if(!world.blockExists(i, j, k)) {
            return true;
        }
        if(BlockRail.isRailBlockAt(world, i, j, k)) {
            return world.isBlockSolidOnSide(i, j - 1, k, ForgeDirection.UP);
        }
        return false;
    }

    public boolean isSupported()
    {
        int meta = tileEntity.getBlockMetadata();
        return isSupported(getWorld(), tileEntity.xCoord, tileEntity.yCoord, tileEntity.zCoord, meta);
    }

    public boolean isSupported(World world, int i, int j, int k, int meta)
    {
        if(isSupportedRail(world, i, j, k, meta)) {
            return true;
        }
        if(meta == EnumTrackMeta.NORTH_SOUTH.ordinal()) {
            if(isSupportedRail(world, i, j, k + 1, meta)
                || isSupportedRail(world, i, j, k - 1, meta)) {
                return true;
            }
            return false;
        } else if(meta == EnumTrackMeta.EAST_WEST.ordinal()) {
            if(isSupportedRail(world, i + 1, j, k, meta)
                || isSupportedRail(world, i - 1, j, k, meta)) {
                return true;
            }
            return false;
        }
        return false;
    }

    @Override
    public boolean canPlaceRailAt(World world, int i, int j, int k)
    {
//        if(BlockRail.isRailBlockAt(world, i, j - 1, k)) {
//            return false;
//        }
//        if(BlockRail.isRailBlockAt(world, i, j + 1, k)) {
//            return false;
//        }
        if(isSupported(world, i, j, k, 0) || isSupported(world, i, j, k, 1)) {
            return true;
        }
        return world.isBlockSolidOnSide(i, j - 1, k, ForgeDirection.UP);
    }
}
