package railcraft.common.blocks.tracks.speedcontroller;

import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.tracks.ITrackInstance;
import railcraft.common.blocks.tracks.TrackSpeed;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class SpeedControllerHighSpeed extends SpeedController
{

    private static final int LOOK_AHEAD_DIST = 2;
    private static float SPEED_SLOPE = 0.45f;
    private static SpeedController instance;

    public static SpeedController getInstance() {
        if(instance == null) {
            instance = new SpeedControllerHighSpeed();
        }
        return instance;
    }

    @Override
    public float getMaxSpeed(ITrackInstance track, EntityMinecart cart) {
        Float speed = null;
        if(track instanceof TrackSpeed) {
            speed = ((TrackSpeed)track).maxSpeed;
        }
        if(speed == null) {
            speed = speedForCurrentTrack(track);
        }
        if(track instanceof TrackSpeed) {
            ((TrackSpeed)track).maxSpeed = speed;
        }
        return speed;
    }

    public static float speedForCurrentTrack(ITrackInstance track) {
        World world = track.getWorld();
        int x = track.getX();
        int y = track.getY();
        int z = track.getZ();
        int id = world.getBlockId(x, y, z);
        if(BlockRail.isRailBlock(id)) {
            int meta = ((BlockRail)Block.blocksList[id]).getBasicRailMetadata(world, null, x, y, z);

            if(meta > 1 && meta < 6) {
                return SPEED_SLOPE;
            }
        }
        return speedForNextTrack(world, x, y, z, 0);
    }

    private static float speedForNextTrack(World world, int x, int y, int z, int dist) {
        float maxSpeed = RailcraftConfig.getMaxHighSpeed();
        if(dist < LOOK_AHEAD_DIST) {
            for(int side = 2; side < 6; side++) {
                ForgeDirection dir = ForgeDirection.getOrientation(side);
                int xx = MiscTools.getXOnSide(x, dir);
                int yy = y;
                int zz = MiscTools.getZOnSide(z, dir);
                int id = world.getBlockId(xx, yy, zz);
                if(!BlockRail.isRailBlock(id)) {
                    id = world.getBlockId(xx, yy + 1, zz);
                    if(BlockRail.isRailBlock(id)) {
                        yy = yy + 1;
                    } else {
                        id = world.getBlockId(xx, yy - 1, zz);
                        if(BlockRail.isRailBlock(id)) {
                            yy = yy - 1;
                        }
                    }
                }
                if(BlockRail.isRailBlock(id)) {
                    int meta = ((BlockRail)Block.blocksList[id]).getBasicRailMetadata(world, null, xx, yy, zz);
                    if(meta > 1 && meta < 6) {
                        return SPEED_SLOPE;
                    }
                    maxSpeed = speedForNextTrack(world, xx, yy, zz, dist + 1);
                    if(maxSpeed == SPEED_SLOPE) {
                        return SPEED_SLOPE;
                    }
                }
            }
        }
        return maxSpeed;
    }
}
