package railcraft.common.carts;

import buildcraft.api.inventory.ISpecialInventory;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import net.minecraft.block.Block;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.ChunkCoordIntPair;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeChunkManager;
import net.minecraftforge.common.ForgeChunkManager.Ticket;
import net.minecraftforge.common.ForgeChunkManager.Type;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.MinecartRegistry;
import railcraft.common.api.carts.ICartTextureInterface;
import railcraft.common.api.carts.IItemTransfer;
import railcraft.common.api.carts.TransferCartBase;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.effects.EffectManager;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.misc.ChunkManager;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.IAnchor;

public class EntityCartAnchor extends TransferCartBase implements ICartTextureInterface, IAnchor, ISpecialInventory {

    private static final long TICK_PER_HOUR = 72000;
    public static final byte TICKET_FLAG = 6;
    private static final byte ANCHOR_RADIUS = 1;
    private static final byte MAX_CHUNKS = 12;
    protected Ticket ticket;
    private Set<ChunkCoordIntPair> chunks;
    private boolean first = true;
    private long anchorFuel;
    private IInventory invWrapper = new InventoryMapper(this);
//    private int update = MiscTools.getRand().nextInt();

    public EntityCartAnchor(World world) {
        super(world);
        passThrough = true;
    }

    public EntityCartAnchor(World world, double x, double y, double z) {
        this(world);
        setPosition(x, y + (double) yOffset, z);
        motionX = 0.0D;
        motionY = 0.0D;
        motionZ = 0.0D;
        prevPosX = x;
        prevPosY = y;
        prevPosZ = z;
    }

    private boolean hasFuel() {
        return anchorFuel > 0;
    }

    public void setAnchorFuel(long fuel) {
        anchorFuel = fuel;
    }

    @Override
    public void onUpdate() {
        super.onUpdate();
        if (Game.isNotHost(worldObj)) {
            if (getFlag(TICKET_FLAG)) {
                if (chunks != null) {
                    EffectManager.instance.chunkLoaderEffect(worldObj, this, chunks);
                } else {
                    setupChunks(chunkCoordX, chunkCoordZ);
                }
            }
            return;
        }

        if (RailcraftConfig.deleteAnchors()) {
            setDead();
            return;
        }

        int hoursPerFeeding = getRefuelTime();
        if (hoursPerFeeding > 0) {
            if (ticket != null && anchorFuel > 0) {
                anchorFuel--;
            }
            if (anchorFuel <= 0) {
                stockFuel();
                ItemStack stack = getStackInSlot(0);
                if (stack == null || stack.stackSize <= 0) {
                    setInventorySlotContents(0, null);
                    releaseTicket();
                } else if (stack.itemID == RailcraftConfig.anchorFuelItemID()) {
                    decrStackSize(0, 1);
                    anchorFuel = hoursPerFeeding * TICK_PER_HOUR;
                }
            }
        }

        if (ticket == null) {
            requestTicket();
        }

//        update++;
//        if (update % 64 == 0) {
//            System.out.println("anchor Tick");
//        }
    }

    private void stockFuel() {
        LinkageManager lm = LinkageManager.getInstance(worldObj);
        EntityMinecart link = lm.getLinkedCartA(this);
        if (link instanceof IItemTransfer) {
            requestFuel((IItemTransfer) link);
        }

        link = lm.getLinkedCartB(this);
        if (link instanceof IItemTransfer) {
            requestFuel((IItemTransfer) link);
        }
    }

    private void requestFuel(IItemTransfer cart) {
        ItemStack stack = getStackInSlot(0);
        if (stack != null && stack.itemID != RailcraftConfig.anchorFuelItemID()) {
            stack = cart.offerItem(this, stack);
            setInventorySlotContents(0, stack);
            return;
        }
        if (stack == null) {
            stack = cart.requestItem(this, new ItemStack(RailcraftConfig.anchorFuelItemID(), 1, 0));
            if (stack != null) {
                InvTools.moveItemStack(stack, this);
            }
        }
    }

    protected Ticket getTicketFromForge() {
        return ForgeChunkManager.requestTicket(Railcraft.getMod(), worldObj, Type.ENTITY);
    }

    protected int getRefuelTime() {
        return RailcraftConfig.anchorRefuel();
    }

    protected boolean meetsTicketRequirements() {
        return hasFuel() || getRefuelTime() <= 0;
    }

    protected void releaseTicket() {
        ForgeChunkManager.releaseTicket(ticket);
        ticket = null;
        setFlag(TICKET_FLAG, false);
    }

    private boolean requestTicket() {
        if (meetsTicketRequirements()) {
            Ticket chunkTicket = getTicketFromForge();
            if (chunkTicket != null) {
//                System.out.println("Request Ticket: " + worldObj.getClass().getSimpleName());
                chunkTicket.getModData();
                chunkTicket.setChunkListDepth(MAX_CHUNKS);
                chunkTicket.bindEntity(this);
                setChunkTicket(chunkTicket);
                return true;
            }
        }
        return false;
    }

    public void setChunkTicket(Ticket tick) {
        if (this.ticket != tick) {
            ForgeChunkManager.releaseTicket(this.ticket);
        }
        this.ticket = tick;
        setFlag(TICKET_FLAG, ticket != null);
    }

    public void forceChunkLoading(int xChunk, int zChunk) {
        if (ticket == null) {
            return;
        }

        setupChunks(xChunk, zChunk);

//        System.out.println("Chunks Loaded = " + Arrays.toString(chunks.toArray()));

        for (ChunkCoordIntPair chunk : chunks) {
            ForgeChunkManager.forceChunk(ticket, chunk);
        }

        ChunkCoordIntPair myChunk = new ChunkCoordIntPair(xChunk, zChunk);
        ForgeChunkManager.forceChunk(ticket, myChunk);
        ForgeChunkManager.reorderChunk(ticket, myChunk);
    }

    public void setupChunks(int xChunk, int zChunk) {
        if (getFlag(TICKET_FLAG)) {
            chunks = ChunkManager.getInstance().getChunksAround(xChunk, zChunk, ANCHOR_RADIUS);
        } else {
            chunks = null;
        }
    }

    @Override
    protected void writeEntityToNBT(NBTTagCompound data) {
        super.writeEntityToNBT(data);

        data.setLong("anchorFuel", anchorFuel);
    }

    @Override
    protected void readEntityFromNBT(NBTTagCompound data) {
        super.readEntityFromNBT(data);

        if (getRefuelTime() > 0) {
            anchorFuel = data.getLong("anchorFuel");
        }
    }

    @Override
    public void setDead() {
        releaseTicket();
        super.setDead();
    }

    @Override
    public boolean doInteract(EntityPlayer player) {
        if (Game.isHost(getWorld()) && getRefuelTime() > 0) {
            GuiHandler.openGui(EnumGui.CART_ANCHOR, player, worldObj, this);
        }
        return true;
    }

    @Override
    public List<ItemStack> getItemsDropped() {
        List<ItemStack> items = new ArrayList<ItemStack>();
        items.add(getCartItem());
        return items;
    }

    @Override
    public ItemStack getCartItem() {
        ItemStack drop = MinecartRegistry.getItemForCart(this);
        if (getRefuelTime() > 0 && hasFuel()) {
            NBTTagCompound nbt = new NBTTagCompound("tag");
            nbt.setLong("fuel", anchorFuel);
            drop.setTagCompound(nbt);
        }
        return drop;
    }

    @Override
    public boolean doesCartMatchFilter(ItemStack stack, EntityMinecart cart) {
        if (ItemCart.getCartType(stack) == EnumCart.ANCHOR) {
            return true;
        }
        return false;
    }

    @Override
    public double getDrag() {
        return CartConstants.STANDARD_DRAG;
    }

    @Override
    public boolean canBeRidden() {
        return false;
    }

    @Override
    public int getSizeInventory() {
        return getRefuelTime() > 0 ? 1 : 0;
    }

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(EnumCart.ANCHOR.getTag());
    }

    @Override
    public Block getBlock() {
        return RailcraftBlocks.getBlockMachineAlpha();
    }

    @Override
    public int getBlockMetadata() {
        return EnumMachineAlpha.WORLD_ANCHOR.ordinal();
    }

    @Override
    public int getBlockTextureOnSide(int side) {
        if (side < 2 && !getFlag(TICKET_FLAG)) {
            return EnumMachineAlpha.WORLD_ANCHOR.getTexture(6);
        }
        return EnumMachineAlpha.WORLD_ANCHOR.getTexture(side);
    }

    @Override
    public long getAnchorFuel() {
        return anchorFuel;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (!RailcraftConfig.anchorsCanInteractWithPipes()) {
            return 0;
        }
        if (stack == null) {
            return 0;
        }
        if (stack.itemID != RailcraftConfig.anchorFuelItemID()) {
            return 0;
        }
        stack = stack.copy();
        IInventory inv = this;
        if (!doAdd) {
            inv = new InventoryCopy(inv);
        }
        ItemStack remainder = InvTools.moveItemStack(stack, inv);
        if (remainder == null) {
            return stack.stackSize;
        }
        return stack.stackSize - remainder.stackSize;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory inv = invWrapper;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return InvTools.removeItems(inv, maxItemCount);
    }
}
