/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.carts;

import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeChunkManager;
import net.minecraftforge.common.ForgeChunkManager.Type;
import railcraft.common.api.carts.CartTools;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class EntityCartAnchorPersonal extends EntityCartAnchor {

    private static final int MINUTES_BEFORE_DISABLE = 5;
    private long ticksSincePlayerLogged;

    public EntityCartAnchorPersonal(World world) {
        super(world);
    }

    public EntityCartAnchorPersonal(World world, double x, double y, double z) {
        super(world, x, y, z);
    }

    @Override
    public void onUpdate() {
        if (ticket != null) {
            if (MiscTools.isPlayerConnected(CartTools.getCartOwner(this))) {
                ticksSincePlayerLogged = 0;
            } else {
                ticksSincePlayerLogged++;
            }
            if (ticksSincePlayerLogged > RailcraftConstants.TICKS_PER_MIN * MINUTES_BEFORE_DISABLE) {
                releaseTicket();
            }
        }
        super.onUpdate();
    }

    @Override
    public boolean doesCartMatchFilter(ItemStack stack, EntityMinecart cart) {
        if (ItemCart.getCartType(stack) == EnumCart.ANCHOR_PERSONAL) {
            return true;
        }
        return false;
    }

    @Override
    protected ForgeChunkManager.Ticket getTicketFromForge() {
        return ForgeChunkManager.requestPlayerTicket(Railcraft.getMod(), CartTools.getCartOwner(this), worldObj, Type.ENTITY);
    }

    @Override
    protected int getRefuelTime() {
        return RailcraftConfig.anchorRefuelPersonal();
    }

    @Override
    protected boolean meetsTicketRequirements() {
        return MiscTools.isPlayerConnected(CartTools.getCartOwner(this)) && super.meetsTicketRequirements();
    }

    @Override
    public int getBlockMetadata() {
        return EnumMachineAlpha.PERSONAL_ANCHOR.ordinal();
    }

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(EnumCart.ANCHOR_PERSONAL.getTag());
    }

    @Override
    public int getBlockTextureOnSide(int side) {
        if (side < 2 && !getFlag(TICKET_FLAG)) {
            return EnumMachineAlpha.PERSONAL_ANCHOR.getTexture(6);
        }
        return EnumMachineAlpha.PERSONAL_ANCHOR.getTexture(side);
    }
}
