package railcraft.common.carts;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import railcraft.common.api.carts.CartBase;
import railcraft.common.api.carts.ICartRenderInterface;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.MiscTools;

public class EntityCartFurnace extends CartBase implements ICartRenderInterface {

    public EntityCartFurnace(World world) {
        super(world);
    }

    public EntityCartFurnace(World world, double d, double d1, double d2) {
        this(world);
        setPosition(d, d1 + (double) yOffset, d2);
        prevPosX = d;
        prevPosY = d1;
        prevPosZ = d2;
        fuel = 0;
        pushX = 0;
        pushZ = 0;
        minecartType = 2;
    }

    public boolean isActive() {
        return fuel > 0;
    }

    @Override
    protected void applyDragAndPushForces() {
        motionX *= getDrag();
        motionY *= 0.0D;
        motionZ *= getDrag();

        double d27 = MathHelper.sqrt_double(pushX * pushX + pushZ * pushZ);
        if (d27 > 0.01D) {
            pushX /= d27;
            pushZ /= d27;
            double d29 = 0.4;
            motionX += pushX * d29;
            motionZ += pushZ * d29;
        }
    }

    @Override
    protected void updatePushForces() {
        double d30 = MathHelper.sqrt_double(pushX * pushX + pushZ * pushZ);
        if (d30 > 0.01D && motionX * motionX + motionZ * motionZ > 0.001D) {
            pushX /= d30;
            pushZ /= d30;
            if (pushX * motionX + pushZ * motionZ < 0.0D) {
                pushX = 0.0D;
                pushZ = 0.0D;
            } else {
                pushX = motionX;
                pushZ = motionZ;
            }
        }
    }

    @Override
    public void onUpdate() {
        if (isMinecartPowered() && rand.nextInt(4) == 0) {
            worldObj.spawnParticle("largesmoke", posX, posY + 0.80000000000000004D, posZ, 0.0D, 0.0D, 0.0D);
        }

        super.onUpdate();
    }

    @Override
    public List<ItemStack> getItemsDropped() {
        List<ItemStack> items = new ArrayList<ItemStack>();
        if (RailcraftConfig.doCartsBreakOnDrop()) {
            items.add(new ItemStack(Item.minecartEmpty));
            items.add(new ItemStack(Block.stoneOvenIdle));
        } else {
            items.add(getCartItem());
        }
        return items;
    }

    @Override
    public ItemStack getCartItem() {
        return new ItemStack(Item.minecartPowered);
    }

    @Override
    public double getDrag() {
        return DRAG_FACTOR;
    }

    @Override
    public float getCartMaxSpeed() {
        return 0.39f;
    }

    @Override
    public boolean isPoweredCart() {
        return true;
    }

    @Override
    public boolean canBeRidden() {
        return false;
    }

    @Override
    public int getSizeInventory() {
        return 0;
    }

    @Override
    public String getInvName() {
        return "Steamcart";
    }

    @Override
    public boolean doInteract(EntityPlayer entityplayer) {
        ItemStack stack = entityplayer.inventory.getCurrentItem();
        if (stack != null) {
            int burnTime = MiscTools.getItemBurnTime(stack);

            if (burnTime > 0) {
                entityplayer.inventory.setInventorySlotContents(entityplayer.inventory.currentItem, InvTools.depleteItem(stack));
                fuel += burnTime;

                pushX = posX - entityplayer.posX;
                pushZ = posZ - entityplayer.posZ;
            }
        }

        return true;
    }
    private static final double DRAG_FACTOR = 0.99;
    private static final double PUSH_FACTOR = 0.1D;

    @Override
    public Block getBlock() {
        return Block.stoneOvenIdle;
    }

    @Override
    public int getBlockMetadata() {
        return 0;
    }
}
