package railcraft.common.carts;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.potion.PotionHelper;
import net.minecraft.world.World;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.RailcraftToolItems;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.Game;

public class EntityCartGift extends EntityCartTNT {

    private static final byte SPAWN_DIST = 2;
    private static final List<Gift> gifts = new ArrayList<Gift>();
    private static final List<Integer> potions = new ArrayList<Integer>();

    private static class Gift {

        public final int chance;
        public final ItemStack stack;

        public Gift(ItemStack stack, int chance) {
            this.stack = stack;
            this.chance = chance;
        }
    }

    static {
        gifts.add(new Gift(null /*potion*/, 100));
        gifts.add(new Gift(null /*potion*/, 100));
        gifts.add(new Gift(null /*potion*/, 100));
        gifts.add(new Gift(null /*potion*/, 100));
        gifts.add(new Gift(null /*potion*/, 100));

        gifts.add(new Gift(new ItemStack(Item.snowball, 16), 50));

        gifts.add(new Gift(new ItemStack(Block.cake), 25));
        gifts.add(new Gift(new ItemStack(Item.bowlSoup), 25));
        gifts.add(new Gift(new ItemStack(Item.cookie, 15), 75));
        gifts.add(new Gift(new ItemStack(Item.cookie, 10), 75));
        gifts.add(new Gift(new ItemStack(Item.cookie, 5), 75));
        gifts.add(new Gift(new ItemStack(Item.pumpkinPie), 20));

        gifts.add(new Gift(new ItemStack(Block.music), 5));
        gifts.add(new Gift(new ItemStack(Item.painting), 10));
        gifts.add(new Gift(new ItemStack(Item.flowerPot), 25));
        gifts.add(new Gift(new ItemStack(Item.compass), 25));
        gifts.add(new Gift(new ItemStack(Item.pocketSundial), 25));
        addGift(RailcraftToolItems.getCrowbar(), 20);

        gifts.add(new Gift(new ItemStack(Item.expBottle, 32), 5));
        gifts.add(new Gift(new ItemStack(Item.expBottle, 16), 10));
        gifts.add(new Gift(new ItemStack(Item.expBottle, 8), 20));
        gifts.add(new Gift(new ItemStack(Item.expBottle, 4), 40));
        gifts.add(new Gift(new ItemStack(Item.expBottle, 2), 80));

        gifts.add(new Gift(new ItemStack(Block.blockDiamond), 1));
        gifts.add(new Gift(new ItemStack(Block.blockEmerald), 2));
        gifts.add(new Gift(new ItemStack(Item.emerald), 30));
        gifts.add(new Gift(new ItemStack(Item.diamond), 20));
        gifts.add(new Gift(new ItemStack(Item.ingotGold), 30));
        gifts.add(new Gift(new ItemStack(Item.ingotGold, 2), 30));

        gifts.add(new Gift(new ItemStack(Item.goldNugget, 8), 80));
        gifts.add(new Gift(new ItemStack(Item.goldNugget, 16), 40));
        gifts.add(new Gift(new ItemStack(Item.goldNugget, 32), 20));

        gifts.add(new Gift(new ItemStack(Item.enderPearl), 30));
        gifts.add(new Gift(new ItemStack(Item.netherStar), 2));

        int recordChance = 1;

        gifts.add(new Gift(new ItemStack(Item.record11), recordChance));
        gifts.add(new Gift(new ItemStack(Item.record13), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordBlocks), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordCat), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordChirp), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordFar), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordMall), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordMellohi), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordStal), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordStrad), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordWard), recordChance));
        gifts.add(new Gift(new ItemStack(Item.recordWard), recordChance));

        int toolChance = 10;

        gifts.add(new Gift(new ItemStack(Item.fishingRod), toolChance));
        gifts.add(new Gift(new ItemStack(Item.bow), toolChance));
        gifts.add(new Gift(new ItemStack(Item.shears), toolChance));

        gifts.add(new Gift(new ItemStack(Item.axeDiamond), toolChance));
        gifts.add(new Gift(new ItemStack(Item.pickaxeDiamond), toolChance));
        gifts.add(new Gift(new ItemStack(Item.shovelDiamond), toolChance));
        gifts.add(new Gift(new ItemStack(Item.swordDiamond), toolChance));
        gifts.add(new Gift(new ItemStack(Item.hoeDiamond), toolChance));

        gifts.add(new Gift(new ItemStack(Item.axeGold), toolChance));
        gifts.add(new Gift(new ItemStack(Item.pickaxeGold), toolChance));
        gifts.add(new Gift(new ItemStack(Item.shovelGold), toolChance));
        gifts.add(new Gift(new ItemStack(Item.swordGold), toolChance));
        gifts.add(new Gift(new ItemStack(Item.hoeGold), toolChance));

        gifts.add(new Gift(new ItemStack(Item.axeSteel), toolChance));
        gifts.add(new Gift(new ItemStack(Item.pickaxeSteel), toolChance));
        gifts.add(new Gift(new ItemStack(Item.shovelSteel), toolChance));
        gifts.add(new Gift(new ItemStack(Item.swordSteel), toolChance));
        gifts.add(new Gift(new ItemStack(Item.hoeSteel), toolChance));

        addGift(RailcraftToolItems.getSteelAxe(), toolChance);
        addGift(RailcraftToolItems.getSteelPickaxe(), toolChance);
        addGift(RailcraftToolItems.getSteelSword(), toolChance);
        addGift(RailcraftToolItems.getSteelShovel(), toolChance);
        addGift(RailcraftToolItems.getSteelHoe(), toolChance);

        int armorChance = 5;

        gifts.add(new Gift(new ItemStack(Item.helmetDiamond), armorChance));
        gifts.add(new Gift(new ItemStack(Item.plateDiamond), armorChance));
        gifts.add(new Gift(new ItemStack(Item.legsDiamond), armorChance));
        gifts.add(new Gift(new ItemStack(Item.bootsDiamond), armorChance));

        gifts.add(new Gift(new ItemStack(Item.helmetGold), armorChance));
        gifts.add(new Gift(new ItemStack(Item.plateGold), armorChance));
        gifts.add(new Gift(new ItemStack(Item.legsGold), armorChance));
        gifts.add(new Gift(new ItemStack(Item.bootsGold), armorChance));

        gifts.add(new Gift(new ItemStack(Item.helmetSteel), armorChance));
        gifts.add(new Gift(new ItemStack(Item.plateSteel), armorChance));
        gifts.add(new Gift(new ItemStack(Item.legsSteel), armorChance));
        gifts.add(new Gift(new ItemStack(Item.bootsSteel), armorChance));

        gifts.add(new Gift(new ItemStack(Item.helmetLeather), armorChance));
        gifts.add(new Gift(new ItemStack(Item.plateLeather), armorChance));
        gifts.add(new Gift(new ItemStack(Item.legsLeather), armorChance));
        gifts.add(new Gift(new ItemStack(Item.bootsLeather), armorChance));

        addGift(RailcraftToolItems.getSteelHelm(), armorChance);
        addGift(RailcraftToolItems.getSteelArmor(), armorChance);
        addGift(RailcraftToolItems.getSteelLegs(), armorChance);
        addGift(RailcraftToolItems.getSteelBoots(), armorChance);

        for (int meta = 0; meta <= 32767; ++meta) {
            List effects = PotionHelper.getPotionEffects(meta, false);

            if (effects != null && !effects.isEmpty()) {
                potions.add(meta);
            }
        }
    }

    public static void addGift(ItemStack gift, int chance) {
        if (gift != null)
            gifts.add(new Gift(gift, chance));
    }

    public EntityCartGift(World world) {
        super(world);
    }

    public EntityCartGift(World world, double d, double d1, double d2) {
        this(world);
        setPosition(d, d1 + (double) yOffset, d2);
        motionX = 0.0D;
        motionY = 0.0D;
        motionZ = 0.0D;
        prevPosX = d;
        prevPosY = d1;
        prevPosZ = d2;
        setBlastRadius(1.5f);
    }

    @Override
    public List<ItemStack> getItemsDropped() {
        List<ItemStack> items = new ArrayList<ItemStack>();
        if (RailcraftConfig.doCartsBreakOnDrop()) {
            items.add(new ItemStack(Item.minecartEmpty));
            items.add(new ItemStack(Block.pumpkin));
        } else {
            items.add(getCartItem());
        }
        return items;
    }

    @Override
    protected byte getMinBlastRadius() {
        return 1;
    }

    @Override
    protected byte getMaxBlastRadius() {
        return 2;
    }

    @Override
    public void explode() {
        if (Game.isHost(getWorld())) {
            worldObj.createExplosion(this, posX, posY, posZ, getBlastRadius(), true);
            setDead();

            if (rand.nextInt(100) >= 50) {
                spawnGift();
                spawnGift();
            } else {
                spawnCoal();
            }
        }
    }

    private Gift getGift() {
        while (true) {
            int index = rand.nextInt(gifts.size());
            Gift gift = gifts.get(index);
            int weight = rand.nextInt(100);
            if (gift.chance >= weight)
                return gift;
        }
    }

    private void spawnGift() {
        Gift gift = getGift();

        if (gift.stack == null) {
            spawnPotion();
            return;
        }

        double x = posX + (rand.nextDouble() - rand.nextDouble()) * SPAWN_DIST;
        double y = (double) (posY + 1 + rand.nextInt(3) - 1);
        double z = posZ + (rand.nextDouble() - rand.nextDouble()) * SPAWN_DIST;
        InvTools.dropItem(gift.stack.copy(), worldObj, x, y, z);
    }

    private void spawnCoal() {
        double x = posX + (rand.nextDouble() - rand.nextDouble()) * SPAWN_DIST;
        double y = (double) (posY + 1 + rand.nextInt(3) - 1);
        double z = posZ + (rand.nextDouble() - rand.nextDouble()) * SPAWN_DIST;
        InvTools.dropItem(new ItemStack(Item.coal), worldObj, x, y, z);
    }

    private void spawnPotion() {
        int meta = potions.get(rand.nextInt(potions.size()));
        ItemStack potion = new ItemStack(Item.potion, 1, meta);

        double x = posX + (rand.nextDouble() - rand.nextDouble()) * SPAWN_DIST;
        double y = (double) (posY + 1 + rand.nextInt(3) - 1);
        double z = posZ + (rand.nextDouble() - rand.nextDouble()) * SPAWN_DIST;
        InvTools.dropItem(potion, worldObj, x, y, z);
    }

    @Override
    public String getInvName() {
        return "Gift Cart";
    }

    @Override
    public Block getBlock() {
        return Block.blockEmerald;
    }

    @Override
    public int getBlockMetadata() {
        return 0;
    }
}
