package railcraft.common.carts;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.client.model.ModelBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import railcraft.common.api.carts.CartBase;
import railcraft.common.api.carts.ICartRenderInterface;
import railcraft.common.api.carts.IExplosiveCart;
import railcraft.common.api.carts.IMinecartTextureProvider;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.IGuiReturnHandler;

public class EntityCartTNT extends CartBase implements IExplosiveCart, IGuiReturnHandler, ICartRenderInterface, IMinecartTextureProvider {

    private static final String TEXTURE = RailcraftConstants.CART_TEXTURE_FOLDER + "cart_wood.png";
    private static final byte FUSE_DATA_ID = 20;
    private static final byte BLAST_DATA_ID = 21;
    private static final byte PRIMED_DATA_ID = 22;
    private final static float BLAST_RADIUS_MULTIPLIER = 0.5f;
    private final static byte BLAST_RADIUS_MIN = 4;
    private final static byte BLAST_RADIUS_MAX = 12;
    public static final short MAX_FUSE = 500;
    public static final short MIN_FUSE = 0;
    private double prevMotionX;
    private double prevMotionZ;
    private boolean prevOnGround;
    private boolean prevOnRail;
    private short timeInAir = 0;

    public EntityCartTNT(World world) {
        super(world);
    }

    public EntityCartTNT(World world, double d, double d1, double d2) {
        this(world);
        setPosition(d, d1 + (double) yOffset, d2);
        motionX = 0.0D;
        motionY = 0.0D;
        motionZ = 0.0D;
        prevPosX = d;
        prevPosY = d1;
        prevPosZ = d2;
    }

    @Override
    protected void entityInit() {
        super.entityInit();
        dataWatcher.addObject(FUSE_DATA_ID, Short.valueOf((short) 80));
        dataWatcher.addObject(BLAST_DATA_ID, Byte.valueOf((byte) 9));
        dataWatcher.addObject(PRIMED_DATA_ID, Byte.valueOf((byte) 0));
    }

    @Override
    public List<ItemStack> getItemsDropped() {
        List<ItemStack> items = new ArrayList<ItemStack>();
        if (RailcraftConfig.doCartsBreakOnDrop()) {
            items.add(new ItemStack(Item.minecartEmpty));
            items.add(new ItemStack(Block.tnt));
        } else {
            items.add(getCartItem());
        }
        return items;
    }

    @Override
    public void onUpdate() {
        super.onUpdate();

        if (((motionX == 0 ^ prevMotionX == 0) && Math.abs(prevMotionX) > 0.4f && motionZ == 0)
                || ((motionZ == 0 ^ prevMotionZ == 0) && Math.abs(prevMotionZ) > 0.4f && motionX == 0)
                || (onGround && !prevOnGround && !prevOnRail && timeInAir > 5)) {
            explode();
        }
        prevMotionX = motionX;
        prevMotionZ = motionZ;
        prevOnGround = onGround;
        int i = MathHelper.floor_double(posX);
        int j = MathHelper.floor_double(posY);
        int k = MathHelper.floor_double(posZ);
        if (BlockRail.isRailBlockAt(worldObj, i, j - 1, k)) {
            j--;
        }
        int id = worldObj.getBlockId(i, j, k);
        prevOnRail = BlockRail.isRailBlock(id);
        if (!onGround && !prevOnRail) {
            timeInAir++;
        } else {
            timeInAir = 0;
        }

        if (isPrimed()) {
            setFuse((short) (getFuse() - 1));
            if (getFuse() <= 0) {
                explode();
            } else {
                worldObj.spawnParticle("smoke", posX, posY + 0.8D, posZ, 0.0D, 0.0D, 0.0D);
            }
        }
    }

    @Override
    public void explode() {
        if (Game.isHost(getWorld())) {
            worldObj.createExplosion(this, posX, posY, posZ, getBlastRadius(), true);
            setDead();
        }
    }

    @Override
    public boolean doInteract(EntityPlayer player) {
        ItemStack itemstack = player.inventory.getCurrentItem();
        if (itemstack != null) {
            if (itemstack.itemID == Item.flintAndSteel.itemID) {
                setPrimed(true);
                worldObj.playSoundAtEntity(this, "random.fuse", 1.0F, 1.0F);
            } else if (itemstack.itemID == Item.silk.itemID) {
                player.inventory.decrStackSize(player.inventory.currentItem, 1);
                GuiHandler.openGui(EnumGui.CART_TNT_FUSE, player, worldObj, this);
            } else if (itemstack.itemID == Item.gunpowder.itemID) {
                player.inventory.decrStackSize(player.inventory.currentItem, 1);
                GuiHandler.openGui(EnumGui.CART_TNT_BLAST, player, worldObj, this);
            }
        }
        return true;
    }

    @Override
    protected double getDrag() {
        return CartConstants.STANDARD_DRAG;
    }

    @Override
    public int getSizeInventory() {
        return 0;
    }

    @Override
    public boolean isStorageCart() {
        return false;
    }

    @Override
    public boolean canBeRidden() {
        return false;
    }

    @Override
    public String getInvName() {
        return "TNTcart";
    }

    @Override
    public boolean isPrimed() {
        return dataWatcher.getWatchableObjectByte(PRIMED_DATA_ID) != 0;
    }

    @Override
    public void setPrimed(boolean p) {
        dataWatcher.updateObject(PRIMED_DATA_ID, p ? Byte.valueOf((byte) 1) : Byte.valueOf((byte) 0));
    }

    @Override
    public int getFuse() {
        return dataWatcher.getWatchableObjectShort(FUSE_DATA_ID);
    }

    @Override
    public void setFuse(int f) {
        f = (short) Math.max(f, MIN_FUSE);
        f = (short) Math.min(f, MAX_FUSE);
        dataWatcher.updateObject(FUSE_DATA_ID, (short) f);
    }

    protected byte getMinBlastRadius() {
        return BLAST_RADIUS_MIN;
    }

    protected byte getMaxBlastRadius() {
        return BLAST_RADIUS_MAX;
    }

    @Override
    public float getBlastRadius() {
        return dataWatcher.getWatchableObjectByte(BLAST_DATA_ID) * BLAST_RADIUS_MULTIPLIER;
    }

    @Override
    public void setBlastRadius(float b) {
        b /= BLAST_RADIUS_MULTIPLIER;
        b = Math.max(b, getMinBlastRadius());
        b = Math.min(b, getMaxBlastRadius());
        dataWatcher.updateObject(BLAST_DATA_ID, Byte.valueOf((byte) b));
    }

    @Override
    protected void writeEntityToNBT(NBTTagCompound data) {
        super.writeEntityToNBT(data);
        data.setShort("Fuse", (short) getFuse());
        data.setByte("blastRadius", dataWatcher.getWatchableObjectByte(BLAST_DATA_ID));
        data.setBoolean("Primed", isPrimed());
    }

    @Override
    protected void readEntityFromNBT(NBTTagCompound data) {
        super.readEntityFromNBT(data);
        setFuse(data.getShort("Fuse"));
        setBlastRadius(data.getByte("blastRadius"));
        setPrimed(data.getBoolean("Primed"));
    }

    @Override
    public void writeGuiData(DataOutputStream data) throws IOException {
        data.writeShort(getFuse());
        data.writeByte(dataWatcher.getWatchableObjectByte(BLAST_DATA_ID));
    }

    @Override
    public void readGuiData(DataInputStream data) throws IOException {
        setFuse(data.readShort());
        setBlastRadius(data.readByte());
    }

    @Override
    public World getWorld() {
        return worldObj;
    }

    @Override
    public Block getBlock() {
        return Block.tnt;
    }

    @Override
    public int getBlockMetadata() {
        return 0;
    }

    @Override
    public String getTextureFile() {
        return TEXTURE;
    }
}
