package railcraft.common.carts;

import buildcraft.api.inventory.ISpecialInventory;
import com.google.common.io.ByteArrayDataInput;
import com.google.common.io.ByteArrayDataOutput;
import cpw.mods.fml.common.registry.IEntityAdditionalSpawnData;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.ISidedInventory;
import net.minecraftforge.common.MinecartRegistry;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import net.minecraftforge.liquids.LiquidTank;
import railcraft.common.api.carts.CartBase;
import railcraft.common.api.carts.ILiquidTransfer;
import railcraft.common.api.carts.IMinecart;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.inventory.PhantomInventory;
import railcraft.common.util.inventory.StandaloneInventory;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

public class EntityCartTank extends CartBase implements ITankContainer, ILiquidTransfer, IEntityAdditionalSpawnData, ISidedInventory, ISpecialInventory, IMinecart {

    public static Item item;

    public static ItemStack getFilterFromCartItem(ItemStack cart) {
        ItemStack filter = null;
        NBTTagCompound nbt = cart.getTagCompound();
        if (nbt != null) {
            int filterId = nbt.getInteger("filterId");
            if (filterId > 0) {
                int filterMeta = nbt.getInteger("filterMeta");
                filter = new ItemStack(filterId, 1, filterMeta);
            }
        }
        return filter;
    }

    public static ItemStack getCartItemForFilter(ItemStack filter) {
        ItemStack stack = new ItemStack(item);
        if (filter != null) {
            NBTTagCompound nbt = new NBTTagCompound("tag");
            nbt.setInteger("filterId", filter.itemID);
            nbt.setInteger("filterMeta", filter.getItemDamage());
            stack.setTagCompound(nbt);
        }
        return stack;
    }
    private static final byte LIQUID_ID_DATA_ID = 20;
    private static final byte LIQUID_META_DATA_ID = 21;
    private static final byte LIQUID_QTY_DATA_ID = 22;
    private static final byte FILLING_DATA_ID = 23;
    private static final int SLOT_INPUT = 0;
    private static final int SLOT_OUTPUT = 1;
    private int update = MiscTools.getRand().nextInt();
    private final TankManager tankManager = new TankManager();
    private StandardTank tank = new StandardTank(RailcraftConfig.getTankCartCapacity());
    private final PhantomInventory invFilter = new PhantomInventory(1, this);
    private final StandaloneInventory invBucket = new StandaloneInventory(2, this);

    public EntityCartTank(World world) {
        super(world);
        tankManager.addTank(tank);
    }

    public EntityCartTank(World world, double d, double d1, double d2) {
        this(world);
        setPosition(d, d1 + (double) yOffset, d2);
        motionX = 0.0D;
        motionY = 0.0D;
        motionZ = 0.0D;
        prevPosX = d;
        prevPosY = d1;
        prevPosZ = d2;
    }

    @Override
    protected void entityInit() {
        super.entityInit();
        dataWatcher.addObject(LIQUID_ID_DATA_ID, new Integer(0));
        dataWatcher.addObject(LIQUID_META_DATA_ID, new Integer(0));
        dataWatcher.addObject(LIQUID_QTY_DATA_ID, new Integer(0));
        dataWatcher.addObject(FILLING_DATA_ID, Byte.valueOf((byte) 0));
    }

    private void setLiquidQty(int qty) {
        dataWatcher.updateObject(LIQUID_QTY_DATA_ID, qty);
    }

    private int getLiquidQty() {
        return dataWatcher.getWatchableObjectInt(LIQUID_QTY_DATA_ID);
    }

    private void setLiquidId(int liquidId) {
        dataWatcher.updateObject(LIQUID_ID_DATA_ID, liquidId);
    }

    private int getLiquidId() {
        return dataWatcher.getWatchableObjectInt(LIQUID_ID_DATA_ID);
    }

    private void setLiquidMeta(int liquidId) {
        dataWatcher.updateObject(LIQUID_META_DATA_ID, liquidId);
    }

    private int getLiquidMeta() {
        return dataWatcher.getWatchableObjectInt(LIQUID_META_DATA_ID);
    }

    public TankManager getTankManager() {
        return tankManager;
    }

    @Override
    public void setDead() {
        super.setDead();
        InvTools.dropInventory(invBucket, worldObj, (int) posX, (int) posY, (int) posZ);
    }

    @Override
    public void onUpdate() {
        super.onUpdate();

        if (Game.isNotHost(worldObj)) {
            if (getLiquidId() != 0) {
                LiquidStack liquid = new LiquidStack(getLiquidId(), getLiquidQty(), getLiquidMeta());
                tank.setLiquid(liquid);
            } else {
                tank.setLiquid(null);
            }
            return;
        }

        LiquidStack liquid = tank.getLiquid();
        if (liquid != null) {
            if (liquid.itemID != getLiquidId()) {
                setLiquidId(liquid.itemID);
            }
            if (liquid.itemMeta != getLiquidMeta()) {
                setLiquidMeta(liquid.itemMeta);
            }
            if (liquid.amount != getLiquidQty()) {
                setLiquidQty(liquid.amount);
            }
        } else {
            if (getLiquidId() != 0) {
                setLiquidId(0);
                setLiquidMeta(0);
                setLiquidQty(0);
            }
        }

        update++;

        ItemStack topSlot = invBucket.getStackInSlot(SLOT_INPUT);
        if (topSlot != null && !LiquidManager.getInstance().isContainer(topSlot)) {
            invBucket.setInventorySlotContents(SLOT_INPUT, null);
            entityDropItem(topSlot, 1);
        }

        ItemStack bottomSlot = invBucket.getStackInSlot(SLOT_OUTPUT);
        if (bottomSlot != null && !LiquidManager.getInstance().isContainer(bottomSlot)) {
            invBucket.setInventorySlotContents(SLOT_OUTPUT, null);
            entityDropItem(bottomSlot, 1);
        }

        if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
            LiquidManager.getInstance().processContainers(this, 0, invBucket, SLOT_INPUT, SLOT_OUTPUT);
        }
    }

    @Override
    public boolean doInteract(EntityPlayer player) {
        if (Game.isHost(getWorld())) {
            if (LiquidManager.getInstance().handleRightClick(this, 0, player, true, true)) {
                return true;
            }
            GuiHandler.openGui(EnumGui.CART_TANK, player, worldObj, this);
        }
        return true;
    }

    @Override
    public ItemStack getCartItem() {
        ItemStack drop = MinecartRegistry.getItemForCart(this);
        if (hasFilter()) {
            NBTTagCompound nbt = new NBTTagCompound("tag");
            nbt.setInteger("filterId", getFilterItem().itemID);
            nbt.setInteger("filterMeta", getFilterItem().getItemDamage());
            drop.setTagCompound(nbt);
        }
        return drop;
    }

    @Override
    public List<ItemStack> getItemsDropped() {
        List<ItemStack> items = new ArrayList<ItemStack>();
        items.add(getCartItem());
        return items;
    }

    @Override
    protected double getDrag() {
        return CartConstants.STANDARD_DRAG;
    }

    @Override
    public boolean isStorageCart() {
        return false;
    }

    @Override
    public boolean canBeRidden() {
        return false;
    }

    @Override
    public int getSizeInventory() {
        return 0;
    }

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(EnumCart.TANK.getTag());
    }

    @Override
    protected void readEntityFromNBT(NBTTagCompound data) {
        super.readEntityFromNBT(data);

        invFilter.readFromNBT("invFilter", data);
        invBucket.readFromNBT("invBucket", data);

        tankManager.readTanksFromNBT(data);
    }

    @Override
    protected void writeEntityToNBT(NBTTagCompound data) {
        super.writeEntityToNBT(data);

        invFilter.writeToNBT("invFilter", data);
        invBucket.writeToNBT("invBucket", data);

        tankManager.writeTanksToNBT(data);
    }

    /**
     * Fills liquid into internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return fill(0, resource, doFill);
    }

    /**
     * Fills liquid into the specified internal tank.
     *
     * @param from ForgeDirection the liquid is pumped in from.
     * @param resource LiquidStack representing the maximum amount of liquid
     * filled into the ITankContainer
     * @param doFill If false filling will only be simulated.
     * @return Amount of resource that was filled into internal tanks.
     */
    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        if (tankIndex != 0 || resource == null) {
            return 0;
        }
        if (getFilterLiquid() == null || resource.isLiquidEqual(getFilterLiquid())) {
            return tank.fill(resource, doFill);
        }
        return 0;
    }

    /**
     * Drains liquid out of internal tanks, distribution is left to the
     * ITankContainer.
     *
     * @param from ForgeDirection the liquid is drained to.
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return drain(0, maxDrain, doDrain);
    }

    /**
     * Drains liquid out of the specified internal tank.
     *
     * @param maxDrain Maximum amount of liquid to drain.
     * @param doDrain If false draining will only be simulated.
     * @return LiquidStack representing the liquid and amount actually drained
     * from the ITankContainer
     */
    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        if (tankIndex != 0) {
            return null;
        }
        return tank.drain(maxDrain, doDrain);
    }

    /**
     * @return Array of {@link LiquidTank}s contained in this ITankContainer
     */
    @Override
    public ILiquidTank[] getTanks(ForgeDirection side) {
        return new ILiquidTank[]{tank};
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type) {
        return tank;
    }

    @Override
    public void setFilling(boolean fill) {
        dataWatcher.updateObject(FILLING_DATA_ID, Byte.valueOf(fill ? 1 : (byte) 0));
    }

    @Override
    public boolean isFilling() {
        return dataWatcher.getWatchableObjectByte(FILLING_DATA_ID) != 0;
    }

    public boolean hasFilter() {
        return getFilterItem() != null;
    }

    public LiquidStack getFilterLiquid() {
        ItemStack filter = getFilterItem();
        if (filter == null) {
            return null;
        }
        return LiquidManager.getInstance().getLiquidInContainer(filter);
    }

    public ItemStack getFilterItem() {
        return getFilter().getStackInSlot(0);
    }

    public void setFilter(ItemStack filter) {
        getFilter().setInventorySlotContents(0, filter);
    }

    public PhantomInventory getFilter() {
        return invFilter;
    }

    public IInventory getInvBucket() {
        return invBucket;
    }

    @Override
    public void writeSpawnData(ByteArrayDataOutput data) {
        data.writeInt(getFilterItem() != null ? getFilterItem().itemID : 0);
        data.writeInt(getFilterItem() != null ? getFilterItem().getItemDamage() : 0);
    }

    @Override
    public void readSpawnData(ByteArrayDataInput data) {
        int filterId = data.readInt();
        int filterMeta = data.readInt();
        if (filterId > 0) {
            ItemStack filter = new ItemStack(filterId, 1, filterMeta);
            setFilter(filter);
        }
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (LiquidManager.getInstance().isContainer(stack)) {
            IInventory slot = new InventoryMapper(this, SLOT_INPUT, 1);
            if (!doAdd) {
                slot = new InventoryCopy(slot);
            }
            ItemStack result = InvTools.moveItemStack(stack, slot);
            if (result != null && stack.stackSize != result.stackSize) {
                int used = stack.stackSize - result.stackSize;
                return used;
            } else if (result == null) {
                return stack.stackSize;
            }
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory slot = new InventoryMapper(this, SLOT_OUTPUT, 1);
        if (!doRemove) {
            slot = new InventoryCopy(slot);
        }
        ItemStack result = slot.decrStackSize(0, maxItemCount);
        if (result == null || result.stackSize <= 0) {
            return new ItemStack[0];
        }
        return new ItemStack[]{result};
    }

    @Override
    public int getStartInventorySide(ForgeDirection side) {
        if (side == ForgeDirection.UP) {
            return SLOT_INPUT;
        }
        return SLOT_OUTPUT;
    }

    @Override
    public int getSizeInventorySide(ForgeDirection side) {
        return 1;
    }

    @Override
    public boolean doesCartMatchFilter(ItemStack stack, EntityMinecart cart) {
        if (ItemCart.getCartType(stack) == EnumCart.TANK) {
            return true;
        }
        return false;
    }

    @Override
    public int offerLiquid(Object source, LiquidStack offer) {
        int qty = offer.amount;
        int used = fill(0, offer, true);

        offer.amount = qty - used;
        if (offer.amount <= 0) {
            return used;
        }

        LinkageManager lm = LinkageManager.getInstance(worldObj);

        EntityMinecart linkedCart = lm.getLinkedCartA(this);
        if (linkedCart != source && linkedCart instanceof ILiquidTransfer) {
            used += ((ILiquidTransfer) linkedCart).offerLiquid(this, offer);
        }

        offer.amount = qty - used;
        if (offer.amount <= 0) {
            return used;
        }

        linkedCart = lm.getLinkedCartB(this);
        if (linkedCart != source && linkedCart instanceof ILiquidTransfer) {
            used += ((ILiquidTransfer) linkedCart).offerLiquid(this, offer);
        }

        return used;
    }

    @Override
    public int requestLiquid(Object source, LiquidStack request) {
        LiquidStack acquired = drain(0, request.amount, false);
        if (acquired == null || !request.isLiquidEqual(acquired)) {
            return 0;
        }

        drain(0, request.amount, true);

        if (acquired.amount >= request.amount) {
            return acquired.amount;
        }

        LiquidStack newRequest = request.copy();
        newRequest.amount = request.amount - acquired.amount;

        LinkageManager lm = LinkageManager.getInstance(worldObj);

        EntityMinecart linkedCart = lm.getLinkedCartA(this);
        if (linkedCart != source && linkedCart instanceof ILiquidTransfer) {
            acquired.amount += ((ILiquidTransfer) linkedCart).requestLiquid(this, newRequest);
        }

        if (acquired.amount >= request.amount) {
            return acquired.amount;
        }

        newRequest.amount = request.amount - acquired.amount;

        linkedCart = lm.getLinkedCartB(this);
        if (linkedCart != source && linkedCart instanceof ILiquidTransfer) {
            acquired.amount += ((ILiquidTransfer) linkedCart).requestLiquid(this, newRequest);
        }

        return acquired.amount;
    }
}
