package railcraft.common.carts;

import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.MathHelper;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.core.items.ITrackItem;
import railcraft.common.api.tracks.RailTools;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.Game;

public class EntityCartTrackRelayer extends MaintanceCartBase {

    private static final int[] TEXTURE = new int[]{224, 224, 225, 225, 226, 226};
    private static final int SLOT_STOCK = 0;
    private static final int SLOT_EXIST = 0;
    private static final int SLOT_REPLACE = 1;

    public EntityCartTrackRelayer(World world) {
        super(world);
    }

    public EntityCartTrackRelayer(World world, double d, double d1, double d2) {
        this(world);
        setPosition(d, d1 + (double) yOffset, d2);
        motionX = 0.0D;
        motionY = 0.0D;
        motionZ = 0.0D;
        prevPosX = d;
        prevPosY = d1;
        prevPosZ = d2;
    }

    @Override
    public int[] getTextureArray() {
        return TEXTURE;
    }

    @Override
    public void onUpdate() {
        super.onUpdate();
        if (Game.isNotHost(getWorld())) {
            return;
        }

        stockItems(SLOT_REPLACE, SLOT_STOCK);
        replace();
    }

    private void replace() {
        int i = MathHelper.floor_double(this.posX);
        int j = MathHelper.floor_double(this.posY);
        int k = MathHelper.floor_double(this.posZ);

        if (BlockRail.isRailBlockAt(this.worldObj, i, j - 1, k)) {
            --j;
        }

        int id = this.worldObj.getBlockId(i, j, k);

        if (BlockRail.isRailBlock(id)) {
            ItemStack trackExist = patternInv.getStackInSlot(SLOT_EXIST);
            ItemStack trackStock = getStackInSlot(SLOT_STOCK);
            if (trackExist != null && trackStock != null) {
                if (trackExist.getItem() instanceof ITrackItem) {
                    ITrackItem trackItem = (ITrackItem) trackExist.getItem();
                    if (trackItem.getPlacedBlockId() == id) {
                        TileEntity tile = worldObj.getBlockTileEntity(i, j, k);
                        if (trackItem.isPlacedTileEntity(trackExist, tile)) {
                            int meta = removeOldTrack(i, j, k, id);
                            if (meta != -1) {
                                placeNewTrack(i, j, k, SLOT_STOCK, meta);
                            }
                        }
                    }
                } else if (trackExist.getItem() instanceof ItemBlock) {
                    if (trackExist.itemID == id) {
                        int meta = removeOldTrack(i, j, k, id);
                        if (meta != -1) {
                            placeNewTrack(i, j, k, SLOT_STOCK, meta);
                        }
                    }
                }
            }
        }
    }

    private int removeOldTrack(int x, int y, int z, int id) {
        List<ItemStack> drops = Block.blocksList[id].getBlockDropped(worldObj, x, y, z, 0, 0);

        for (ItemStack stack : drops) {
            CartTools.offerOrDropItem(this, stack);
        }
        int meta = worldObj.getBlockMetadata(x, y, z);
        if (((BlockRail) Block.blocksList[id]).hasPowerBit(worldObj, x, y, z)) {
            meta = meta & 7;
        }
        worldObj.setBlockWithNotify(x, y, z, 0);
        return meta;
    }

    private void placeNewTrack(int x, int y, int z, int slotStock, int meta) {
        ItemStack trackStock = getStackInSlot(slotStock);
        if (trackStock != null) {
            if (RailTools.placeRailAt(trackStock, worldObj, x, y, z)) {
                worldObj.setBlockMetadataWithNotify(x, y, z, meta);
                int id = worldObj.getBlockId(x, y, z);
                Block.blocksList[id].onNeighborBlockChange(worldObj, x, y, z, id);
                worldObj.markBlockForUpdate(x, y, z);
                decrStackSize(slotStock, 1);
                blink();
            }
        }
    }

    @Override
    public boolean doInteract(EntityPlayer player) {
        if (Game.isHost(worldObj)) {
            GuiHandler.openGui(EnumGui.CART_TRACK_RELAYER, player, worldObj, this);
        }
        return true;
    }

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(EnumCart.TRACK_RELAYER.getTag());
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        ItemStack trackReplace = patternInv.getStackInSlot(SLOT_REPLACE);
        ItemStack trackStock = getStackInSlot(SLOT_STOCK);
        if (InvTools.isItemEqual(stack, trackReplace)) {
            if (trackStock == null) {
                if (doAdd) {
                    setInventorySlotContents(SLOT_STOCK, stack.copy());
                }
                return stack.stackSize;
            } else if (trackStock.stackSize + stack.stackSize <= trackStock.getMaxStackSize()) {
                if (doAdd) {
                    trackStock.stackSize += stack.stackSize;
                }
                return stack.stackSize;
            } else {
                int used = trackStock.getMaxStackSize() - trackStock.stackSize;
                if (doAdd) {
                    trackStock.stackSize = trackStock.getMaxStackSize();
                }
                return used;
            }
        }
        return 0;
    }
}
