/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.carts;

import buildcraft.api.inventory.ISpecialInventory;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidContainerRegistry;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.gui.indicator.IIndicatorController;
import railcraft.common.gui.util.ToolTip;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.liquids.LiquidFilter;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.liquids.TankManager;
import railcraft.common.liquids.tanks.FilteredTank;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.util.effects.EffectManager;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryCopy;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.steam.Steam;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class EntityLocomotiveSteam extends EntityLocomotive implements ITankContainer, ISpecialInventory {

    private static final String[] TEXTURES = {
        RailcraftConstants.CART_TEXTURE_FOLDER + "loco_steam_1.png",
        RailcraftConstants.CART_TEXTURE_FOLDER + "loco_steam_2.png",
        RailcraftConstants.CART_TEXTURE_FOLDER + "loco_steam_3.png"};
    private static final String WHISTLE_SOUND = "locomotive.steam.whistle";
    private static final byte TICKS_PER_CONVERSION = 2;
    private static final int FUEL_PER_REQUEST = 3;
    private static final int INEFFICENCY_MULTIPLIER = Steam.INEFFICENCY_MULTIPLIER / 2;
    private static final int TANK_WATER = 0;
    private static final int SLOT_LIQUID_INPUT = 0;
    private static final int SLOT_LIQUID_OUTPUT = 1;
    private static final int SLOT_BURN = 2;
    private static final int SLOT_FUEL_A = 3;
    private static final int SLOT_FUEL_B = 4;
    private static final int SLOT_FUEL_C = 5;
    public static final byte SMOKE_FLAG = 6;
    public static final byte STEAM_FLAG = 7;
    private TankManager tankManager;
    private StandardTank tankWater;
    private StandardTank tankSteam;
    private IInventory invAll = new InventoryMapper(this);
    private InventoryMapper invWaterInput;
    private IInventory invWaterOutput = new InventoryMapper(this, SLOT_LIQUID_OUTPUT, 1);
    private IInventory invBurn = new InventoryMapper(this, SLOT_BURN, 1);
    private IInventory invStock = new InventoryMapper(this, SLOT_FUEL_A, 3);
    private IInventory invFuel = new InventoryMapper(this, SLOT_BURN, 4);
    private int burnCycle;
    private int update = rand.nextInt();
    public float heat = Steam.COLD_TEMP;
    public float burnTime;
    public float currentItemBurnTime;
    private IIndicatorController heatIndicator = new HeatIndicator();

    private class HeatIndicator implements IIndicatorController {

        List<ToolTip> tips = new ArrayList<ToolTip>();
        ToolTip tip = new ToolTip();

        private HeatIndicator() {
            tips.add(tip);
        }

        @Override
        public List<ToolTip> getToolTip() {
            tip.text = String.format("%.0f\u00B0C", heat);
            return tips;
        }

        @Override
        public int getScaledLevel(int size) {
            return (int) ((heat - Steam.COLD_TEMP) * size / (Steam.MAX_HEAT - Steam.COLD_TEMP));
        }
    };

    public IIndicatorController getHeatIndicator() {
        return heatIndicator;
    }

    public EntityLocomotiveSteam(World world) {
        super(world);
    }

    public EntityLocomotiveSteam(World world, double x, double y, double z) {
        super(world, x, y, z);
    }

    @Override
    protected void entityInit() {
        super.entityInit();
        tankManager = new TankManager();

        tankWater = new FilteredTank(LiquidContainerRegistry.BUCKET_VOLUME * 4, LiquidFilter.WATER);
        tankSteam = new FilteredTank(LiquidContainerRegistry.BUCKET_VOLUME * 16, LiquidFilter.STEAM);

        tankManager.addTank(tankWater);
        tankManager.addTank(tankSteam);
        
        invWaterInput = new InventoryMapper(this, SLOT_LIQUID_INPUT, 1);
        invWaterInput.setStackSizeLimit(4);
    }

    @Override
    public String getWhistle() {
        return WHISTLE_SOUND;
    }

    @Override
    protected ItemStack getCartItemBase() {
        return EnumCart.LOCO_STEAM.getCartItem();
    }

    @Override
    public String getInvName() {
        return RailcraftLanguage.translate(EnumCart.LOCO_STEAM.getTag());
    }

    @Override
    public int getSizeInventory() {
        return 6;
    }

    public TankManager getTankManager() {
        return tankManager;
    }

    public int getBurnProgressScaled(int i) {
        if (burnTime <= 0 || currentItemBurnTime <= 0) {
            return 0;
        }
        int scale = (int) (burnTime * i / currentItemBurnTime);
        scale = Math.min(scale, i);
        scale = Math.max(scale, 0);
        return scale;
    }

    private int getFuelPerConversion() {
        float fuelUsed = Steam.FUEL_PER_STEAM_CONVERSION;
        fuelUsed = fuelUsed + fuelUsed * (INEFFICENCY_MULTIPLIER - INEFFICENCY_MULTIPLIER * getHeatLevel());
        return Math.round(fuelUsed);
    }

    public float getHeatLevel() {
        return heat / Steam.MAX_HEAT;
    }

    private void increaseHeat() {
        float heatLevel = getHeatLevel();
        heat += Steam.HEAT_STEP;
        if (heatLevel < 0.25f) {
            heat += Steam.HEAT_STEP;
        }
        if (heatLevel < 0.5f) {
            heat += Steam.HEAT_STEP;
        }
        if (heatLevel < 0.75f) {
            heat += Steam.HEAT_STEP;
        }
        heat = Math.min(heat, Steam.MAX_HEAT);
    }

    private void reduceHeat() {
        float heatLevel = getHeatLevel();
        heat -= Steam.HEAT_STEP;
        if (heatLevel > 0.25f) {
            heat -= Steam.HEAT_STEP;
        }
        if (heatLevel > 0.5f) {
            heat -= Steam.HEAT_STEP;
        }
        if (heatLevel > 0.75f) {
            heat -= Steam.HEAT_STEP;
        }
        heat = Math.max(heat, Steam.COLD_TEMP);
    }

    @Override
    public void onUpdate() {
        super.onUpdate();

        if (Game.isHost(worldObj)) {
            update++;

            InvTools.moveOneItem(invStock, invBurn);
            InvTools.moveOneItem(invBurn, invWaterOutput, LiquidContainerRegistry.EMPTY_BUCKET);

            LiquidStack water = tankWater.getLiquid();
            if (water == null || water.amount <= 0) {
                setMode(LocoMode.SHUTDOWN);
            }

            LiquidStack steam = tankSteam.getLiquid();
            setSteaming(steam != null && steam.amount > 0);
            if (steam == null || tankSteam.getCapacity() - steam.amount >= Steam.STEAM_PER_UNIT_WATER || isShutdown()) {
                burnCycle++;
                if (burnCycle >= TICKS_PER_CONVERSION) {
                    burnCycle = 0;
                    float fuelNeeded = getFuelPerConversion();
                    while (burnTime < fuelNeeded) {
                        boolean addedFuel = addSolidFuel();
                        if (!addedFuel) {
                            break;
                        }
                    }
                    burnTime -= fuelNeeded;
                    if (burnTime <= 0) {
                        burnTime = 0;
                    }
                    if (heat >= Steam.BOILING_POINT) {
                        LiquidStack drainedWater = tankWater.drain(1, false);
                        if (drainedWater != null) {
                            tankWater.drain(1, true);
                            steam = LiquidItems.getSteamLiquid(Steam.STEAM_PER_UNIT_WATER);
                            tankSteam.fill(steam, true);
                        }
                    }
                }

                setSmoking(burnTime > 0);

                if (burnTime > 0) {
                    increaseHeat();
                } else {
                    reduceHeat();
                    ventSteam();
                }
            }

            if (update % LiquidManager.BUCKET_FILL_TIME == 0) {
                LiquidManager.getInstance().processContainers(this, TANK_WATER, this, SLOT_LIQUID_INPUT, SLOT_LIQUID_OUTPUT, true, false);
            }
        } else {
            if (isSmoking()) {
                if (rand.nextInt(3) == 0) {
                    double rads = renderYaw * Math.PI / 180D;
                    float offset = 0.4f;
                    worldObj.spawnParticle("largesmoke", posX - Math.cos(rads) * offset, posY + 1.2f, posZ - Math.sin(rads) * offset, 0, 0, 0);
                }
            }
            if (isSteaming()) {
                EffectManager.instance.steamEffect(worldObj, this, boundingBox.minY - posY - 0.3);
            }
        }
    }

    private void setSmoking(boolean smoke) {
        if (getFlag(SMOKE_FLAG) != smoke) {
            setFlag(SMOKE_FLAG, smoke);
        }
    }

    public boolean isSmoking() {
        return getFlag(SMOKE_FLAG);
    }

    private void setSteaming(boolean steam) {
        if (getFlag(STEAM_FLAG) != steam) {
            setFlag(STEAM_FLAG, steam);
        }
    }

    public boolean isSteaming() {
        return getFlag(STEAM_FLAG);
    }

    private void ventSteam() {
        tankSteam.drain(4, true);
    }

    private boolean addSolidFuel() {
        if (isShutdown()) {
            return false;
        }
        int newFuel = getMoreSoildFuel();
        burnTime += newFuel;
        currentItemBurnTime = burnTime;
        return newFuel > 0;
    }

    private int getMoreSoildFuel() {
        ItemStack burnable = getStackInSlot(SLOT_BURN);
        int burn = MiscTools.getItemBurnTime(burnable);

        if (burn > 0) {
            setInventorySlotContents(SLOT_BURN, InvTools.depleteItem(burnable));
        }
        return burn;
    }

    @Override
    public int getMoreGoJuice() {
        LiquidStack steam = tankSteam.getLiquid();
        if (steam != null && steam.amount >= tankSteam.getCapacity() / 2) {
            tankSteam.drain(Steam.STEAM_PER_UNIT_WATER, true);
            return FUEL_PER_REQUEST;
        }
        return 0;
//        return 100;
    }

    @Override
    public boolean doInteract(EntityPlayer player) {
        if (super.doInteract(player)) {
            return true;
        }
        if (Game.isHost(getWorld())) {
            if (LiquidManager.getInstance().handleRightClick(this, TANK_WATER, player, true, false)) {
                return true;
            }
            GuiHandler.openGui(EnumGui.LOCO_STEAM, player, worldObj, this);
        }
        return true;
    }

    @Override
    public void writeEntityToNBT(NBTTagCompound data) {
        super.writeEntityToNBT(data);
        tankManager.writeTanksToNBT(data);

        data.setFloat("heat", heat);
        data.setFloat("burnTime", burnTime);
        data.setFloat("currentItemBurnTime", currentItemBurnTime);
    }

    @Override
    public void readEntityFromNBT(NBTTagCompound data) {
        super.readEntityFromNBT(data);
        tankManager.readTanksFromNBT(data);

        heat = data.getFloat("heat");
        burnTime = data.getFloat("burnTime");
        currentItemBurnTime = data.getFloat("currentItemBurnTime");
    }

    @Override
    public String[] getLocomotiveTextures() {
        return TEXTURES;
    }

    public boolean isSafeToFill() {
        return heat < Steam.SUPER_HEATED || !tankWater.isEmpty();
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return fill(TANK_WATER, resource, doFill);
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        if (doFill && LiquidFilter.WATER.isLiquidEqual(resource)) {
            if (heat >= Steam.SUPER_HEATED) {
                LiquidStack water = tankWater.getLiquid();
                if (water == null || water.amount <= 0) {
                    explode();
                }
            }
        }
        return tankWater.fill(resource, doFill);
    }

    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection direction) {
        return tankManager.getTanks();
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type) {
        return tankWater;
    }

    @Override
    public int addItem(ItemStack stack, boolean doAdd, ForgeDirection from) {
        if (stack == null) {
            return 0;
        }
        if (MiscTools.getItemBurnTime(stack) > 0) {
            IInventory inv = invFuel;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remainder = InvTools.moveItemStack(stack, inv);
            if (remainder == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remainder.stackSize;
        }

        if (LiquidManager.getInstance().isFilledContainer(stack) && LiquidManager.getInstance().containsLiquid(stack, LiquidFilter.WATER)) {
            IInventory inv = invWaterInput;
            if (!doAdd) {
                inv = new InventoryCopy(inv);
            }
            ItemStack remainder = InvTools.moveItemStack(stack, inv);
            if (remainder == null) {
                return stack.stackSize;
            }
            return stack.stackSize - remainder.stackSize;
        }
        return 0;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        IInventory inv = invAll;
        if (!doRemove) {
            inv = new InventoryCopy(inv);
        }
        return InvTools.removeItems(inv, maxItemCount);
    }
}
