package railcraft.common.carts;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.util.DamageSource;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.MathHelper;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.common.MinecraftForge;
import railcraft.common.api.carts.CartBase;
import railcraft.common.api.carts.IItemTransfer;
import railcraft.common.api.carts.ILinkableCart;
import railcraft.common.api.carts.bore.IBoreHead;
import railcraft.common.api.carts.bore.IMineable;
import railcraft.common.api.tracks.RailTools;
import railcraft.common.blocks.tracks.EnumTrackMeta;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.EnumGui;
import railcraft.common.gui.GuiHandler;
import railcraft.common.util.inventory.EnumItemType;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.InventoryMapper;
import railcraft.common.util.misc.BallastRegistry;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

public class EntityTunnelBore extends CartBase implements IInventory, ILinkableCart {

    public static final float SPEED = 0.03F;
    public static final float LENGTH = 6f;
    public static final int MAX_FILL_DEPTH = 10;
    public static final int FAIL_DELAY = 200;
    public static final int STANDARD_DELAY = 5;
    public static final int LAYER_DELAY = 40;
    public static final int BALLAST_DELAY = 10;
    public static final int FUEL_CONSUMPTION = 12;
    public static final float HARDNESS_MULTIPLER = 8;
    protected static final int WATCHER_ID_MOVING = 20;
    protected static final int WATCHER_ID_BORE_HEAD = 21;
    protected static final int WATCHER_ID_FACING = 5;
//    protected static final int WATCHER_ID_BURN_TIME = 22;
    protected boolean degreeCalc = false;
    protected int delay = 0;
    private boolean active;
    private int update;
    private int burnTime;
    private boolean hasInit;
    protected boolean placeRail = false;
    protected boolean placeBallast = false;
    protected boolean boreLayer = false;
    protected final IInventory invFuel = new InventoryMapper(this, 1, 6);
    protected final IInventory invBallast = new InventoryMapper(this, 7, 9);
    protected final IInventory invRails = new InventoryMapper(this, 16, 9);
    protected int boreRotationAngle = 0;
    private static final Set<Mineable> mineableBlocks = new HashSet<Mineable>();
    private static final int[] mineable = {
        Block.blockClay.blockID,
        Block.blockSnow.blockID,
        Block.cactus.blockID,
        Block.carrot.blockID,
        Block.cobblestone.blockID,
        Block.cobblestoneMossy.blockID,
        Block.cocoaPlant.blockID,
        Block.crops.blockID,
        Block.deadBush.blockID,
        Block.dirt.blockID,
        Block.fire.blockID,
        Block.glowStone.blockID,
        Block.grass.blockID,
        Block.gravel.blockID,
        Block.ice.blockID,
        Block.leaves.blockID,
        Block.melon.blockID,
        Block.melonStem.blockID,
        Block.mushroomBrown.blockID,
        Block.mushroomCapBrown.blockID,
        Block.mushroomCapRed.blockID,
        Block.mushroomRed.blockID,
        Block.mycelium.blockID,
        Block.netherStalk.blockID,
        Block.netherrack.blockID,
        Block.obsidian.blockID,
        Block.oreCoal.blockID,
        Block.oreDiamond.blockID,
        Block.oreEmerald.blockID,
        Block.oreGold.blockID,
        Block.oreIron.blockID,
        Block.oreLapis.blockID,
        Block.oreRedstone.blockID,
        Block.oreRedstoneGlowing.blockID,
        Block.plantRed.blockID,
        Block.plantYellow.blockID,
        Block.potato.blockID,
        Block.pumpkin.blockID,
        Block.pumpkinStem.blockID,
        Block.reed.blockID,
        Block.sand.blockID,
        Block.sandStone.blockID,
        Block.sapling.blockID,
        Block.slowSand.blockID,
        Block.snow.blockID,
        Block.stone.blockID,
        Block.tallGrass.blockID,
        Block.tilledField.blockID,
        Block.torchWood.blockID,
        Block.vine.blockID,
        Block.waterlily.blockID,
        Block.web.blockID,
        Block.whiteStone.blockID,
        Block.wood.blockID,};
    public static final Set<Integer> replaceableBlocks = new HashSet<Integer>();
    private static final int[] replaceable = {
        Block.torchWood.blockID,
        Block.tallGrass.blockID,
        Block.deadBush.blockID,
        Block.vine.blockID,
        Block.mushroomBrown.blockID,
        Block.mushroomRed.blockID,
        Block.plantYellow.blockID,
        Block.plantRed.blockID,};

    static {
        for (int id : mineable) {
            addMineableBlock(id);
        }
        for (int id : replaceable) {
            replaceableBlocks.add(id);
        }
    }

    private static class Mineable {

        public int id;
        public int meta;

        public Mineable(int id, int meta) {
            this.id = id;
            this.meta = meta;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final Mineable other = (Mineable) obj;
            if (this.id != other.id) {
                return false;
            }
            if (this.meta != other.meta) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int hash = 7;
            hash = 41 * hash + this.id;
            hash = 41 * hash + this.meta;
            return hash;
        }
    }

    public EntityTunnelBore(World world, double i, double j, double k) {
        this(world, i, j, k, ForgeDirection.SOUTH);
    }

    public EntityTunnelBore(World world, double i, double j, double k, ForgeDirection f) {
        super(world);
        hasInit = true;
        setPosition(i, j + (double) yOffset, k);
        motionX = 0.0D;
        motionY = 0.0D;
        motionZ = 0.0D;
        prevPosX = i;
        prevPosY = j;
        prevPosZ = k;
//        cargoItems = new ItemStack[25];
        setFacing(f);
        setSize(LENGTH, 2.7F);

        update = MiscTools.getRand().nextInt();
    }

    public EntityTunnelBore(World world) {
        this(world, 0, 0, 0, ForgeDirection.SOUTH);
    }

    public static void addMineableBlock(int id) {
        addMineableBlock(id, -1);
    }

    public static void addMineableBlock(int id, int meta) {
        mineableBlocks.add(new Mineable(id, meta));
    }

    private boolean isMinableBlock(int id, int meta) {
        if (RailcraftConfig.boreMinesAllBlocks()) {
            return true;
        }
        Mineable mine = new Mineable(id, -1);
        if (mineableBlocks.contains(mine)) {
            return true;
        }
        mine.meta = meta;
        if (mineableBlocks.contains(mine)) {
            return true;
        }
        return false;
    }

    @Override
    protected void entityInit() {
        super.entityInit();
//        dataWatcher.addObject(WATCHER_ID_DELAY, Integer.valueOf(0));
        dataWatcher.addObject(WATCHER_ID_MOVING, Byte.valueOf((byte) 0));
        dataWatcher.addObject(WATCHER_ID_BORE_HEAD, Integer.valueOf(-1));
        dataWatcher.addObject(WATCHER_ID_FACING, Byte.valueOf((byte) 0));
//        dataWatcher.addObject(WATCHER_ID_BURN_TIME, Integer.valueOf(0));
    }

    @Override
    public boolean attackEntityFrom(DamageSource source, int damage) {
        if (!this.worldObj.isRemote && !this.isDead) {
            this.setRollingDirection(-this.getRollingDirection());
            this.setRollingAmplitude(10);
            this.setBeenAttacked();
            this.setDamage(this.getDamage() + damage * 10);

            if (source.getEntity() instanceof EntityPlayer && ((EntityPlayer) source.getEntity()).capabilities.isCreativeMode) {
                this.setDamage(500);
            }

            if (this.getDamage() > 120) {
                if (this.riddenByEntity != null) {
                    this.riddenByEntity.mountEntity(this);
                }

                this.setDead();
                dropCartAsItem();
            }

            return true;
        } else {
            return true;
        }
    }

    private void setYaw() {
        float yaw = 0;
        switch (getFacing()) {
            case NORTH:
                yaw = 90;
                break;
            case EAST:
                yaw = 0;
                break;
            case SOUTH:
                yaw = 270;
                break;
            case WEST:
                yaw = 180;
                break;
        }
        setRotation(yaw, rotationPitch);
    }

    @Override
    public int getSizeInventory() {
        return 25;
    }

    @Override
    public boolean canBePushed() {
        return false;
    }

    @Override
    public void setPosition(double i, double j, double k) {
        if (!hasInit) {
            super.setPosition(i, j, k);
            return;
        }

        posX = i;
        posY = j;
        posZ = k;
        double w = 2.7 / 2.0;
        double h = 2.7;
        double l = LENGTH / 2.0;
        double x1 = i;
        double x2 = i;
        double z1 = k;
        double z2 = k;
        if (getFacing() == ForgeDirection.WEST || getFacing() == ForgeDirection.EAST) {
            x1 -= l;
            x2 += l;
            z1 -= w;
            z2 += w;
        } else {
            x1 -= w;
            x2 += w;
            z1 -= l;
            z2 += l;
        }

        boundingBox.setBounds(x1, (j - (double) yOffset) + (double) ySize, z1, x2, (j - (double) yOffset) + (double) ySize + h, z2);
    }

    @Override
    public void onUpdate() {
        update++;

        if (Game.isHost(worldObj)) {
            if (update % 128 == 0) {
                forceUpdateBoreHead();
                setMinecartPowered(false);
                setMoving(false);
            } else {
                updateBoreHead();
            }

            stockBallast();
            stockTracks();
        }

        super.onUpdate();

        if (Game.isHost(worldObj)) {
//            if(update % 64 == 0){
//                System.out.println("bore tick");
//            }

            if (hasFuel() && getDelay() == 0) {
                setActive(true);
//            System.out.println("Yaw = " + MathHelper.floor_double(rotationYaw));

                int x;
                int y = MathHelper.floor_double(posY);
                int z;
                EnumTrackMeta dir = EnumTrackMeta.NORTH_SOUTH;
                if (getFacing() == ForgeDirection.WEST || getFacing() == ForgeDirection.EAST) {
                    dir = EnumTrackMeta.EAST_WEST;
                }

                if (getDelay() == 0) {
                    float offset = 1.5f;
                    x = MathHelper.floor_double(getXAhead(posX, offset));
                    z = MathHelper.floor_double(getZAhead(posZ, offset));

                    if (placeBallast) {
                        boolean placed = placeBallast(x, y - 1, z);
                        if (placed) {
                            setDelay(STANDARD_DELAY);
                        } else {
                            setDelay(FAIL_DELAY);
                            setActive(false);
                        }
                        placeBallast = false;
                    } else if (!worldObj.isBlockSolidOnSide(x, y - 1, z, ForgeDirection.UP)) {
                        placeBallast = true;
                        setDelay(BALLAST_DELAY);
                    }
                }

                if (getDelay() == 0) {
                    float offset = 0.8f;
                    x = MathHelper.floor_double(getXAhead(posX, offset));
                    z = MathHelper.floor_double(getZAhead(posZ, offset));

                    if (placeRail) {
                        boolean placed = placeTrack(x, y, z, dir);
                        if (placed) {
                            setDelay(STANDARD_DELAY);
                        } else {
                            setDelay(FAIL_DELAY);
                            setActive(false);
                        }
                        placeRail = false;
                    } else if (BlockRail.isRailBlockAt(worldObj, x, y, z)) {
                        if (!dir.isEqual(((BlockRail) Block.blocksList[worldObj.getBlockId(x, y, z)]).getBasicRailMetadata(worldObj, this, x, y, z))) {
                            worldObj.setBlockMetadata(x, y, z, dir.ordinal());
                            setDelay(STANDARD_DELAY);
                        }
                    } else {
                        int id = worldObj.getBlockId(x, y, z);
                        if (id == 0 || replaceableBlocks.contains(id)) {
                            placeRail = true;
                            setDelay(STANDARD_DELAY);
                        } else {
                            setDelay(FAIL_DELAY);
                            setActive(false);
                        }
                    }
                }

                if (getDelay() == 0) {
                    float offset = 3.3f;
                    x = MathHelper.floor_double(getXAhead(posX, offset));
                    z = MathHelper.floor_double(getZAhead(posZ, offset));

                    if (boreLayer) {
                        boolean bored = boreLayer(x, y, z, dir);
                        if (bored) {
                            setDelay(LAYER_DELAY);
                        } else {
                            setDelay(FAIL_DELAY);
                            setActive(false);
                        }
                        boreLayer = false;
                    } else {
                        if (checkForLava(x, y, z, dir)) {
                            setDelay(FAIL_DELAY);
                            setActive(false);
                        } else {
                            setDelay((int) Math.ceil(getLayerHardness(x, y, z, dir)));
                            if (getDelay() != 0) {
                                boreLayer = true;
                            }
                        }
                    }
                }
            }

            if (isMinecartPowered()) {
                double i = getXAhead(posX, 3.3);
                double k = getZAhead(posZ, 3.3);
                double size = 0.8;
                List entities = worldObj.getEntitiesWithinAABBExcludingEntity(this, AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(i - size, posY, k - size, i + size, posY + 2, k + size));
                for (Object e : entities) {
                    if (e instanceof EntityLiving) {
                        EntityLiving ent = (EntityLiving) e;
                        ent.attackEntityFrom(new DamageSourceBore(this), 2);
                    }
                }
            }

            setMoving(hasFuel() && getDelay() == 0);

            if (getDelay() > 0) {
                setDelay(getDelay() - 1);
            }
        }

        if (isMoving()) {
            float factorX = MathHelper.cos((float) Math.toRadians(rotationYaw));
            float factorZ = -MathHelper.sin((float) Math.toRadians(rotationYaw));
            motionX = SPEED * factorX;
            motionZ = SPEED * factorZ;
        } else {
            motionX = 0.0D;
            motionZ = 0.0D;
        }

        emitParticles();

        if (isMinecartPowered()) {
            boreRotationAngle += 5;
        }
    }

    @Override
    public float getCartMaxSpeed() {
        return SPEED;
    }

    @Override
    protected void updateFuel() {
        if (Game.isHost(worldObj)) {
            if (isMinecartPowered()) {
                spendFuel();
            }
            stockFuel();
            if (outOfFuel()) {
                addFuel();
            }
            setMinecartPowered(hasFuel() && isActive());
        }
    }

    protected double getXAhead(double x, double offset) {
        if (getFacing() == ForgeDirection.EAST) {
            x += offset;
        } else if (getFacing() == ForgeDirection.WEST) {
            x -= offset;
        }
        return x;
    }

    protected double getZAhead(double z, double offset) {
        if (getFacing() == ForgeDirection.NORTH) {
            z -= offset;
        } else if (getFacing() == ForgeDirection.SOUTH) {
            z += offset;
        }
        return z;
    }

    protected void emitParticles() {
        if (isMinecartPowered()) {
            double randomFactor = 0.125;

            double forwardOffset = -0.35;
            double smokeYOffset = 2.4;
            double flameYOffset = 0.7;
            double smokeSideOffset = 0.92;
            double flameSideOffset = 1.14;
            double smokeX1 = posX;
            double smokeX2 = posX;
            double smokeZ1 = posZ;
            double smokeZ2 = posZ;

            double flameX1 = posX;
            double flameX2 = posX;
            double flameZ1 = posZ;
            double flameZ2 = posZ;
            if (getFacing() == ForgeDirection.NORTH) {
                smokeX1 += smokeSideOffset;
                smokeX2 -= smokeSideOffset;
                smokeZ1 += forwardOffset;
                smokeZ2 += forwardOffset;

                flameX1 += flameSideOffset;
                flameX2 -= flameSideOffset;
                flameZ1 += forwardOffset + (rand.nextGaussian() * randomFactor);
                flameZ2 += forwardOffset + (rand.nextGaussian() * randomFactor);
            } else if (getFacing() == ForgeDirection.EAST) {
                smokeX1 -= forwardOffset;
                smokeX2 -= forwardOffset;
                smokeZ1 += smokeSideOffset;
                smokeZ2 -= smokeSideOffset;

                flameX1 -= forwardOffset + (rand.nextGaussian() * randomFactor);
                flameX2 -= forwardOffset + (rand.nextGaussian() * randomFactor);
                flameZ1 += flameSideOffset;
                flameZ2 -= flameSideOffset;
            } else if (getFacing() == ForgeDirection.SOUTH) {
                smokeX1 += smokeSideOffset;
                smokeX2 -= smokeSideOffset;
                smokeZ1 -= forwardOffset;
                smokeZ2 -= forwardOffset;

                flameX1 += flameSideOffset;
                flameX2 -= flameSideOffset;
                flameZ1 -= forwardOffset + (rand.nextGaussian() * randomFactor);
                flameZ2 -= forwardOffset + (rand.nextGaussian() * randomFactor);
            } else if (getFacing() == ForgeDirection.WEST) {
                smokeX1 += forwardOffset;
                smokeX2 += forwardOffset;
                smokeZ1 += smokeSideOffset;
                smokeZ2 -= smokeSideOffset;

                flameX1 += forwardOffset + (rand.nextGaussian() * randomFactor);
                flameX2 += forwardOffset + (rand.nextGaussian() * randomFactor);
                flameZ1 += flameSideOffset;
                flameZ2 -= flameSideOffset;
            }

            if (rand.nextInt(4) == 0) {
                worldObj.spawnParticle("largesmoke", smokeX1, posY + smokeYOffset, smokeZ1, 0.0D, 0.0D, 0.0D);
                worldObj.spawnParticle("flame", flameX1, posY + flameYOffset + (rand.nextGaussian() * randomFactor), flameZ1, 0.0D, 0.0D, 0.0D);
            }
            if (rand.nextInt(4) == 0) {
                worldObj.spawnParticle("largesmoke", smokeX2, posY + smokeYOffset, smokeZ2, 0.0D, 0.0D, 0.0D);
                worldObj.spawnParticle("flame", flameX2, posY + flameYOffset + (rand.nextGaussian() * randomFactor), flameZ2, 0.0D, 0.0D, 0.0D);
            }
        }
    }

    protected void stockBallast() {
        EntityMinecart link = LinkageManager.getInstance(worldObj).getLinkedCartA(this);
        if (link instanceof IItemTransfer) {
            IItemTransfer tranfer = (IItemTransfer) link;

            for (int slot = 0; slot < invBallast.getSizeInventory(); slot++) {
                ItemStack stack = invBallast.getStackInSlot(slot);
                if (stack != null && !BallastRegistry.isItemBallast(stack)) {
                    stack = tranfer.offerItem(this, stack);
                    invBallast.setInventorySlotContents(slot, stack);
                    return;
                }
                if (stack == null) {
                    stack = tranfer.requestItem(this, EnumItemType.BALLAST);
                    InvTools.moveItemStack(stack, invBallast);
                    return;
                }
            }
        }
    }

    protected boolean placeBallast(int i, int j, int k) {
        if (!worldObj.isBlockSolidOnSide(i, j, k, ForgeDirection.UP)) {
            for (int inv = 0; inv < invBallast.getSizeInventory(); inv++) {
                ItemStack stack = invBallast.getStackInSlot(inv);
                if (stack != null && BallastRegistry.isItemBallast(stack)) {
                    for (int y = j; y > j - MAX_FILL_DEPTH; y--) {
                        if (worldObj.isBlockSolidOnSide(i, y, k, ForgeDirection.UP)) {
                            invBallast.decrStackSize(inv, 1);
                            worldObj.setBlockAndMetadataWithNotify(i, j, k, stack.itemID, stack.getItemDamage());
                            return true;
                        }
                    }
                    return false;
                }
            }
        }
        return false;
    }

    protected void stockTracks() {
        EntityMinecart link = LinkageManager.getInstance(worldObj).getLinkedCartA(this);
        if (link instanceof IItemTransfer) {
            IItemTransfer tranfer = (IItemTransfer) link;

            for (int slot = 0; slot < invRails.getSizeInventory(); slot++) {
                ItemStack stack = invRails.getStackInSlot(slot);
                if (stack != null && !EnumItemType.TRACK.isItemType(stack)) {
                    stack = tranfer.offerItem(this, stack);
                    invRails.setInventorySlotContents(slot, stack);
                    return;
                }
                if (stack == null) {
                    stack = tranfer.requestItem(this, EnumItemType.TRACK);
                    InvTools.moveItemStack(stack, invRails);
                    return;
                }
            }
        }
    }

    protected boolean placeTrack(int i, int j, int k, EnumTrackMeta meta) {
        int id = worldObj.getBlockId(i, j, k);
        if (replaceableBlocks.contains(id)) {
            int m = worldObj.getBlockMetadata(i, j, k);
            worldObj.setBlockWithNotify(i, j, k, 0);
            Block.blocksList[id].dropBlockAsItem(worldObj, i, j, k, m, 0);
            id = worldObj.getBlockId(i, j, k);
        }
        if (id == 0 && worldObj.isBlockNormalCube(i, j - 1, k)) {
            for (int inv = 0; inv < invRails.getSizeInventory(); inv++) {
                ItemStack stack = invRails.getStackInSlot(inv);
                if (stack != null) {
                    boolean placed = RailTools.placeRailAt(stack, worldObj, i, j, k);
                    if (placed) {
                        worldObj.setBlockMetadataWithNotify(i, j, k, meta.ordinal());
                        invRails.decrStackSize(inv, 1);
                    }
                    return placed;
                }
            }
        }
        return false;
    }

    protected boolean checkForLava(int i, int j, int k, EnumTrackMeta dir) {
        int xStart = i - 1;
        int zStart = k - 1;
        int xEnd = i + 1;
        int zEnd = k + 1;
        if (dir == EnumTrackMeta.NORTH_SOUTH) {
            xStart = i - 2;
            xEnd = i + 2;
        } else {
            zStart = k - 2;
            zEnd = k + 2;
        }

        for (int jj = j; jj < j + 4; jj++) {
            for (int ii = xStart; ii <= xEnd; ii++) {
                for (int kk = zStart; kk <= zEnd; kk++) {
                    int id = worldObj.getBlockId(ii, jj, kk);
                    if (id == Block.lavaStill.blockID || id == Block.lavaMoving.blockID) {
                        return true;
                    }
                }
            }
        }

        return false;
    }

    protected boolean boreLayer(int i, int j, int k, EnumTrackMeta dir) {
        boolean clear = true;
        int ii = i;
        int kk = k;
        for (int jj = j; jj < j + 3; jj++) {
            clear = clear && mineBlock(ii, jj, kk, dir);
        }

        if (dir == EnumTrackMeta.NORTH_SOUTH) {
            ii--;
        } else {
            kk--;
        }
        for (int jj = j; jj < j + 3; jj++) {
            clear = clear && mineBlock(ii, jj, kk, dir);
        }

        ii = i;
        kk = k;
        if (dir == EnumTrackMeta.NORTH_SOUTH) {
            ii++;
        } else {
            kk++;
        }
        for (int jj = j; jj < j + 3; jj++) {
            clear = clear && mineBlock(ii, jj, kk, dir);
        }
        return clear;
    }

    protected boolean mineBlock(int i, int j, int k, EnumTrackMeta dir) {
        int id = worldObj.getBlockId(i, j, k);
        Block block = Block.blocksList[id];
        if (id == 0 || worldObj.isAirBlock(i, j, k)) {
            return true;
        } else if (BlockRail.isRailBlock(id)) {
            int trackMeta = ((BlockRail) block).getBasicRailMetadata(worldObj, this, i, j, k);
            if (dir.isEqual(trackMeta)) {
                return true;
            }
        } else if (id == Block.torchWood.blockID) {
            return true;
        }

        ItemStack head = getStackInSlot(0);
        if (head == null) {
            return false;
        }

        int meta = worldObj.getBlockMetadata(i, j, k);

        if (!canMineBlock(i, j, k, id, meta)) {
            return false;
        }

        ArrayList<ItemStack> items = block.getBlockDropped(worldObj, i, j, k, meta, 0);
        EntityMinecart link = LinkageManager.getInstance(worldObj).getLinkedCartA(this);

        for (ItemStack item : items) {
            if (EnumItemType.FUEL.isItemType(item)) {
                item = InvTools.moveItemStack(item, invFuel);
            }

            if (item != null && item.stackSize > 0 && item.itemID == Block.gravel.blockID) {
                item = InvTools.moveItemStack(item, invBallast);
            }

            if (item != null && item.stackSize > 0 && link instanceof IItemTransfer) {
                item = ((IItemTransfer) link).offerItem(this, item);
            }

            if (item != null && item.stackSize > 0 && !RailcraftConfig.boreDestroysBlocks()) {
                float f = 0.7F;
                double xr = (double) (worldObj.rand.nextFloat() - 0.5D) * f;
                double yr = (double) (worldObj.rand.nextFloat() - 0.5D) * f;
                double zr = (double) (worldObj.rand.nextFloat() - 0.5D) * f;
                EntityItem entityitem = new EntityItem(worldObj, getXAhead(posX, -3.2) + xr, posY + 0.3 + yr, getZAhead(posZ, -3.2) + zr, item);
                worldObj.spawnEntityInWorld(entityitem);
            }
        }

        worldObj.setBlockWithNotify(i, j, k, 0);

        head.setItemDamage(head.getItemDamage() + 1);
        if (head.getItemDamage() > head.getMaxDamage()) {
            setInventorySlotContents(0, null);
        }
        return true;
    }

    private boolean canMineBlock(int i, int j, int k, int id, int meta) {
        Block block = Block.blocksList[id];
        if (block instanceof IMineable) {
            ItemStack head = getStackInSlot(0);
            if (head == null) {
                return false;
            }
            return ((IMineable) block).canMineBlock(worldObj, i, j, k, this, head);
        }
        if (block.getBlockHardness(worldObj, i, j, k) < 0) {
            return false;
        }
        return isMinableBlock(id, meta) && canHeadHarvestBlock(id, meta);
    }

    private boolean canHeadHarvestBlock(int id, int meta) {
        ItemStack head = getStackInSlot(0);
        if (head == null) {
            return false;
        }

        if (head.getItem() instanceof IBoreHead) {
            IBoreHead boreHead = (IBoreHead) head.getItem();

            boolean mappingExists = false;

            int blockHarvestLevel = MinecraftForge.getBlockHarvestLevel(Block.blocksList[id], meta, "pickaxe");
            if (blockHarvestLevel > -1) {
                if (boreHead.getHarvestLevel() >= blockHarvestLevel) {
                    return true;
                }
                mappingExists = true;
            }

            blockHarvestLevel = MinecraftForge.getBlockHarvestLevel(Block.blocksList[id], meta, "axe");
            if (blockHarvestLevel > -1) {
                if (boreHead.getHarvestLevel() >= blockHarvestLevel) {
                    return true;
                }
                mappingExists = true;
            }

            blockHarvestLevel = MinecraftForge.getBlockHarvestLevel(Block.blocksList[id], meta, "shovel");
            if (blockHarvestLevel > -1) {
                if (boreHead.getHarvestLevel() >= blockHarvestLevel) {
                    return true;
                }
                mappingExists = true;
            }

            if (mappingExists) {
                return false;
            }
        }

        return true;
    }

    protected float getLayerHardness(int i, int j, int k, EnumTrackMeta dir) {
        float hardness = 0;
        int ii = i;
        int kk = k;
        for (int jj = j; jj < j + 3; jj++) {
            hardness += getBlockHardness(ii, jj, kk, dir);
        }

        if (dir == EnumTrackMeta.NORTH_SOUTH) {
            ii--;
        } else {
            kk--;
        }
        for (int jj = j; jj < j + 3; jj++) {
            hardness += getBlockHardness(ii, jj, kk, dir);
        }

        ii = i;
        kk = k;
        if (dir == EnumTrackMeta.NORTH_SOUTH) {
            ii++;
        } else {
            kk++;
        }
        for (int jj = j; jj < j + 3; jj++) {
            hardness += getBlockHardness(ii, jj, kk, dir);
        }

        hardness *= HARDNESS_MULTIPLER;

        if (cargoItems[0] != null && cargoItems[0].getItem() instanceof IBoreHead) {
            IBoreHead head = (IBoreHead) cargoItems[0].getItem();
            float dig = 2f - head.getDigModifier();
            hardness = hardness * dig;
        }

        return hardness;
    }

    protected float getBlockHardness(int i, int j, int k, EnumTrackMeta dir) {
        int id = worldObj.getBlockId(i, j, k);
        Block block = Block.blocksList[id];
        if (id == 0 || worldObj.isAirBlock(i, j, k)) {
            return 0;
        } else if (BlockRail.isRailBlockAt(worldObj, i, j, k)) {
            int trackMeta = ((BlockRail) block).getBasicRailMetadata(worldObj, this, i, j, k);
            if (dir.isEqual(trackMeta)) {
                return 0;
            }
        } else if (id == Block.torchWood.blockID) {
            return 0;
        } else if (id == Block.obsidian.blockID) {
            return 15;
        }
        int meta = worldObj.getBlockMetadata(i, j, k);
        if (!canMineBlock(i, j, k, id, meta)) {
            return 0.1f;
        }
        float hardness = block.getBlockHardness(worldObj, i, j, k);
        if (hardness <= 0) {
            hardness = 0.1f;
        }
        return hardness;
    }

    @Override
    public AxisAlignedBB getCollisionBox(Entity other) {
        if (other instanceof EntityLiving) {
            return other.boundingBox;
        }
        return null;
    }

    @Override
    public AxisAlignedBB getBoundingBox() {
        return boundingBox;
    }

    @Override
    public String getInvName() {
        return "Tunnel Bore";
    }

    public float getBoreRotationAngle() {
        return (float) Math.toRadians(boreRotationAngle);
    }

    @Override
    protected void writeEntityToNBT(NBTTagCompound data) {
//        fuel = getFuel();
        super.writeEntityToNBT(data);
        data.setByte("facing", (byte) getFacing().ordinal());
        data.setInteger("delay", getDelay());
        data.setBoolean("active", isActive());
        data.setInteger("burnTime", getBurnTime());
    }

    @Override
    protected void readEntityFromNBT(NBTTagCompound data) {
        super.readEntityFromNBT(data);
        setFacing(ForgeDirection.getOrientation(data.getByte("facing")));
        setDelay(data.getInteger("delay"));
        setActive(data.getBoolean("active"));
        setBurnTime(data.getInteger("burnTime"));
//        setFuel(fuel);

        update = MiscTools.getRand().nextInt();
    }

    protected void setDelay(int i) {
        delay = i;
//        dataWatcher.updateObject(WATCHER_ID_DELAY, Integer.valueOf(i));
    }

    protected int getDelay() {
        return delay;
//        return dataWatcher.getWatchableObjectInt(WATCHER_ID_DELAY);
    }

    @Override
    public boolean isMinecartPowered() {
        return super.isMinecartPowered();
    }

    protected boolean isActive() {
        return active;
//        return dataWatcher.getWatchableObjectByte(WATCHER_ID_ACTIVE) != 0;
    }

    protected void setActive(boolean active) {
        this.active = active;
//        dataWatcher.updateObject(WATCHER_ID_ACTIVE, Byte.valueOf((byte)(active ? 1 : 0)));
    }

    protected boolean isMoving() {
        return dataWatcher.getWatchableObjectByte(WATCHER_ID_MOVING) != 0;
    }

    protected void setMoving(boolean move) {
        dataWatcher.updateObject(WATCHER_ID_MOVING, Byte.valueOf((byte) (move ? 1 : 0)));
    }

    public int getBurnTime() {
        return burnTime;
//        return dataWatcher.getWatchableObjectInt(WATCHER_ID_BURN_TIME);
    }

    public void setBurnTime(int burnTime) {
        this.burnTime = burnTime;
//        dataWatcher.updateObject(WATCHER_ID_BURN_TIME, Integer.valueOf(burnTime));
    }

    public int getFuel() {
        return fuel;
//        return dataWatcher.getWatchableObjectInt(WATCHER_ID_FUEL);
    }

    public boolean outOfFuel() {
        return getFuel() <= FUEL_CONSUMPTION;
    }

    public boolean hasFuel() {
        return getFuel() > 0;
    }

    protected void stockFuel() {
        EntityMinecart link = LinkageManager.getInstance(worldObj).getLinkedCartA(this);
        if (link instanceof IItemTransfer) {
            IItemTransfer tranfer = (IItemTransfer) link;

            for (int slot = 0; slot < invFuel.getSizeInventory(); slot++) {
                ItemStack stack = invFuel.getStackInSlot(slot);
                if (stack != null && !EnumItemType.FUEL.isItemType(stack)) {
                    stack = tranfer.offerItem(this, stack);
                    invFuel.setInventorySlotContents(slot, stack);
                    return;
                }
                if (stack == null) {
                    stack = tranfer.requestItem(this, EnumItemType.FUEL);
                    if (stack != null) {
                        InvTools.moveItemStack(stack, invFuel);
                        return;
                    }
                }
            }
        }
    }

    protected void addFuel() {
        int burn = 0;
        for (int slot = 0; slot < invFuel.getSizeInventory(); slot++) {
            ItemStack stack = invFuel.getStackInSlot(slot);
            if (stack != null) {
                burn = MiscTools.getItemBurnTime(stack);
                if (burn > 0) {
                    if (stack.getItem().hasContainerItem()) {
                        invFuel.setInventorySlotContents(slot, new ItemStack(stack.getItem().getContainerItem()));
                    } else {
                        invFuel.decrStackSize(slot, 1);
                    }
                    break;
                }
            }
        }
        if (burn > 0) {
            setBurnTime(burn + getFuel());
            setFuel(getFuel() + burn);
        }
    }

    public int getBurnProgressScaled(int i) {
        int burnTime = getBurnTime();
        if (burnTime == 0) {
            return 0;
        }

        return getFuel() * i / burnTime;
    }

    public void setFuel(int i) {
        fuel = i;
//        dataWatcher.updateObject(WATCHER_ID_FUEL, Integer.valueOf(i));
    }

    protected void spendFuel() {
        setFuel(getFuel() - FUEL_CONSUMPTION);
    }

    protected void updateBoreHead() {
        int id = -1;
        if (cargoItems[0] != null) {
            id = cargoItems[0].itemID;
        }
        dataWatcher.updateObject(WATCHER_ID_BORE_HEAD, Integer.valueOf(id));
    }

    protected void forceUpdateBoreHead() {
        dataWatcher.updateObject(WATCHER_ID_BORE_HEAD, Integer.valueOf(-1));
        updateBoreHead();
    }

    public IBoreHead getBoreHead() {
        int id = dataWatcher.getWatchableObjectInt(WATCHER_ID_BORE_HEAD);
        IBoreHead head = null;
        if (id > 0 && id < Item.itemsList.length) {
            if (Item.itemsList[id] instanceof IBoreHead) {
                head = (IBoreHead) Item.itemsList[id];
            }
        }
        return head;
    }

    @Override
    protected void applyDragAndPushForces() {
        motionX *= getDrag();
        motionY *= 0.0D;
        motionZ *= getDrag();
    }

    @Override
    protected void updatePushForces() {
    }

    @Override
    public List<ItemStack> getItemsDropped() {
        List<ItemStack> items = new ArrayList<ItemStack>();
        items.add(getCartItem());
        return items;
    }

    @Override
    public boolean isPoweredCart() {
        return true;
    }

    @Override
    public boolean doInteract(EntityPlayer player) {
        if (Game.isHost(worldObj)) {
            GuiHandler.openGui(EnumGui.CART_BORE, player, worldObj, this);
        }
        return true;
    }

    @Override
    public void onInventoryChanged() {
        if (!isActive()) {
            setDelay(STANDARD_DELAY);
        }
    }

    public final ForgeDirection getFacing() {
        return ForgeDirection.getOrientation(dataWatcher.getWatchableObjectByte(WATCHER_ID_FACING));
    }

    protected final void setFacing(ForgeDirection facing) {
        dataWatcher.updateObject(WATCHER_ID_FACING, Byte.valueOf((byte) facing.ordinal()));

        setYaw();
    }

    @Override
    public boolean isLinkable() {
        return true;
    }

    @Override
    public boolean canLinkWithCart(EntityMinecart cart) {
        double x = getXAhead(posX, -LENGTH / 2);
        double z = getZAhead(posZ, -LENGTH / 2);

        if (cart.getDistance(x, posY, z) < LinkageManager.LINKAGE_DISTANCE * 2) {
            return true;
        }
        return false;
    }

    @Override
    public boolean hasTwoLinks() {
        return false;
    }

    @Override
    public float getLinkageDistance(EntityMinecart cart) {
        return 4f;
    }

    @Override
    public float getOptimalDistance(EntityMinecart cart) {
        return 3.1f;
    }

    @Override
    public void onLinkCreated(EntityMinecart cart) {
    }

    @Override
    public void onLinkBroken(EntityMinecart cart) {
    }

    @Override
    public boolean canBeAdjusted(EntityMinecart cart) {
        return false;
    }

    @Override
    public boolean shouldDoRailFunctions() {
        return false;
    }

    public IInventory getInventoryFuel() {
        return invFuel;
    }

    public IInventory getInventoryGravel() {
        return invBallast;
    }

    public IInventory getInventoryRails() {
        return invRails;
    }
}
