package railcraft.common.carts;

import net.minecraft.block.Block;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import railcraft.common.blocks.machine.beta.EnumMachineBeta;

public enum EnumCart {

    BASIC(1, 135, EntityCartBasic.class, null),
    CHEST(2, 151, EntityCartChest.class, new ItemStack(Block.chest)),
    FURNACE(3, 167, EntityCartFurnace.class, new ItemStack(Block.stoneOvenIdle)),
    TNT(4, 12 * 16 + 1, EntityCartTNT.class, new ItemStack(Block.tnt)),
    TANK(5, 12 * 16, EntityCartTank.class, new ItemStack(Block.glass, 8)),
    BATBOX(6, 198, EntityCartEnergy.class, null),
    MFE(6, 199, EntityCartEnergy.class, null),
    MFSU(6, 200, EntityCartEnergy.class, null),
    ANCHOR(7, 197, EntityCartAnchor.class, null),
    WORK(8, 196, EntityCartWork.class, new ItemStack(Block.workbench)),
    TRACK_RELAYER(9, 195, EntityCartTrackRelayer.class, null),
    UNDERCUTTER(10, 194, EntityCartUndercutter.class, null),
    PUMPKIN(11, 201, EntityCartPumpkin.class, new ItemStack(Block.pumpkin)),
    GIFT(12, 202, EntityCartGift.class, null),
    ANCHOR_PERSONAL(13, 203, EntityCartAnchorPersonal.class, null),
    LOCO_STEAM(30, 0, EntityLocomotiveSteam.class, null);
    private final int icon;
    private final Class<? extends EntityMinecart> type;
    private final byte id;
    private ItemStack contents = null;
    private ItemStack cartItem;

    private EnumCart(int id, int icon, Class<? extends EntityMinecart> type, ItemStack contents) {
        this.id = (byte) id;
        this.icon = icon;
        this.type = type;
        this.contents = contents;
    }

    public byte getId() {
        return id;
    }

    public int getIconIndex() {
        return icon;
    }

    public String getTag() {
        return "cart." + name().toLowerCase().replace('_', '.');
    }

    public String getItemTag() {
        return "rc.item.cart." + name().toLowerCase().replace('_', '.');
    }

    public Class<? extends EntityMinecart> getCartClass() {
        return type;
    }

    public void setContents(ItemStack stack) {
        contents = stack.copy();
    }

    public ItemStack getContents() {
        switch (this) {
            case TANK:
                if (EnumMachineBeta.TANK_IRON_GAUGE.isEnabled()) {
                    return EnumMachineBeta.TANK_IRON_GAUGE.getItem();
                }
            default: {
                if (contents == null) {
                    return null;
                }
                return contents.copy();
            }
        }
    }

    public EntityMinecart makeCart(ItemStack stack, World world, double i, double j, double k) {
        switch (this) {
            case CHEST:
                return new EntityCartChest(world, i, j, k);
            case FURNACE:
                return new EntityCartFurnace(world, i, j, k);
            case TNT:
                return new EntityCartTNT(world, i, j, k);
            case TANK: {
                EntityCartTank cart = new EntityCartTank(world, i, j, k);
                ItemStack filter = EntityCartTank.getFilterFromCartItem(stack);
                cart.setFilter(filter);
                return cart;
            }
            case BATBOX: {
                EntityMinecart cart = new EntityCartEnergy(world, i, j, k);
                cart.minecartType = 0;
                return cart;
            }
            case MFE: {
                EntityMinecart cart = new EntityCartEnergy(world, i, j, k);
                cart.minecartType = 1;
                return cart;
            }
            case MFSU: {
                EntityMinecart cart = new EntityCartEnergy(world, i, j, k);
                cart.minecartType = 2;
                return cart;
            }
            case ANCHOR: {
                EntityCartAnchor cart = new EntityCartAnchor(world, i, j, k);
                long fuel = ItemCartAnchor.getFuel(stack);
                cart.setAnchorFuel(fuel);
                return cart;
            }
            case ANCHOR_PERSONAL: {
                EntityCartAnchor cart = new EntityCartAnchorPersonal(world, i, j, k);
                long fuel = ItemCartAnchor.getFuel(stack);
                cart.setAnchorFuel(fuel);
                return cart;
            }
            case WORK:
                return new EntityCartWork(world, i, j, k);
            case TRACK_RELAYER:
                return new EntityCartTrackRelayer(world, i, j, k);
            case UNDERCUTTER:
                return new EntityCartUndercutter(world, i, j, k);
            case PUMPKIN:
                return new EntityCartPumpkin(world, i, j, k);
            case GIFT:
                return new EntityCartGift(world, i, j, k);
            case LOCO_STEAM:
                EntityLocomotiveSteam entity = new EntityLocomotiveSteam(world, i, j, k);
                entity.initEntityFromItem(stack);
                return entity;
        }

        return new EntityCartBasic(world, i, j, k);
    }

    /**
     * @return the cartItem
     */
    public ItemStack getCartItem() {
        if (cartItem == null) {
            return null;
        }
        return cartItem.copy();
    }

    /**
     * @param cartItem the cartItem to set
     */
    public void setCartItem(ItemStack cartItem) {
        this.cartItem = cartItem.copy();
    }
}
