package railcraft.common.carts;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.block.BlockRail;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.EnumRarity;
import net.minecraft.item.ItemMinecart;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.core.items.IMinecartItem;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.util.misc.Game;

public class ItemCart extends ItemMinecart implements IMinecartItem {

    private final EnumCart type;
    private int rarity = 0;

    public ItemCart(int i, EnumCart cart) {
        super(i, 0);
        maxStackSize = RailcraftConfig.getMinecartStackSize();
        this.type = cart;
        setIconIndex(cart.getIconIndex());
        setItemName(cart.getTag());
        setMaxDamage(0);
        setHasSubtypes(true);
    }

    public ItemCart setRarity(int rarity) {
        this.rarity = rarity;
        return this;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public EnumRarity getRarity(ItemStack par1ItemStack) {
        return EnumRarity.values()[rarity];
    }

    @Override
    public String getTextureFile() {
        return RailcraftConstants.MAIN_TEXTURE_FILE;
    }

    @Override
    public boolean onItemUse(ItemStack stack, EntityPlayer player, World world, int i, int j, int k, int l, float par8, float par9, float par10) {
        if (Game.isNotHost(world)) {
            return false;
        }
        EntityMinecart placedCart = placeCart(player.username, stack, world, i, j, k);
        if (placedCart != null) {
            stack.stackSize--;
            return true;
        }
        return false;
    }

    public EnumCart getCartType() {
        return type;
    }

    @Override
    public boolean canBePlacedByNonPlayer(ItemStack cart) {
        return true;
    }

    @Override
    public EntityMinecart placeCart(String owner, ItemStack cartStack, World world, int i, int j, int k) {
        int i1 = world.getBlockId(i, j, k);
        if (BlockRail.isRailBlock(i1)) {
            if (!CartTools.isMinecartAt(world, i, j, k, 0)) {
                EntityMinecart cart = type.makeCart(cartStack, world, (float) i + 0.5F, (float) j + 0.5F, (float) k + 0.5F);
                CartTools.setCartOwner(cart, owner);
                if (world.spawnEntityInWorld(cart)) {
                    return cart;
                }
            }
        }
        return null;
    }

    public static EnumCart getCartType(ItemStack cart) {
        if (cart != null && cart.getItem() instanceof ItemCart) {
            return ((ItemCart) cart.getItem()).getCartType();
        }
        return null;
    }
}
