/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.carts;

import java.util.List;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.EnumColor;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ItemLocomotive extends ItemCart {

    private final int icon1, icon2;

    public ItemLocomotive(int id, EnumCart cart, int icon1, int icon2) {
        super(id, cart);
        this.icon1 = icon1;
        this.icon2 = icon2;
        setMaxStackSize(1);
    }

    @Override
    public boolean requiresMultipleRenderPasses() {
        return true;
    }

    @Override
    public int getRenderPasses(int metadata) {
        return 2;
    }

    @Override
    public int getColorFromItemStack(ItemStack stack, int pass) {
        if (pass == 0) {
            return getPrimaryColor(stack).getHexColor();
        }
        if (pass == 1) {
            return getSecondaryColor(stack).getHexColor();
        }
        return super.getColorFromItemStack(stack, pass);
    }

    @Override
    public int getIconIndex(ItemStack stack, int pass) {
        if (pass == 0) {
            return icon1;
        }
        if (pass == 1) {
            return icon2;
        }
        return super.getIconIndex(stack, pass);
    }

    @Override
    public void addInformation(ItemStack stack, EntityPlayer player, List info, boolean adv) {
        EnumColor primary = getPrimaryColor(stack);
        String format = RailcraftLanguage.translate("gui.locomotive.tooltip.primary");
        info.add(String.format(format, primary.getTranslatedName()));

        EnumColor secondary = getSecondaryColor(stack);
        format = RailcraftLanguage.translate("gui.locomotive.tooltip.secondary");
        info.add(String.format(format, secondary.getTranslatedName()));

        float whistle = getWhistlePitch(stack);
        format = RailcraftLanguage.translate("gui.locomotive.tooltip.whistle");
        info.add(String.format(format, whistle < 0 ? "???" : String.format("%.2f", whistle)));
    }

    public static void setItemColorData(ItemStack stack, int primaryColor, int secondaryColor) {
        if (primaryColor < 0 || secondaryColor < 0) {
            return;
        }
        NBTTagCompound nbt = InvTools.getItemData(stack);
        nbt.setByte("primaryColor", (byte) primaryColor);
        nbt.setByte("secondaryColor", (byte) secondaryColor);

    }

    public static void setItemWhistleData(ItemStack stack, float whistlePitch) {
        NBTTagCompound nbt = InvTools.getItemData(stack);
        nbt.setFloat("whistlePitch", whistlePitch);
    }

    public static EnumColor getPrimaryColor(ItemStack stack) {
        NBTTagCompound nbt = stack.getTagCompound();
        if (nbt == null || !nbt.hasKey("primaryColor")) {
            return EnumColor.LIGHT_GRAY;
        }
        return EnumColor.fromId(nbt.getByte("primaryColor"));
    }

    public static EnumColor getSecondaryColor(ItemStack stack) {
        NBTTagCompound nbt = stack.getTagCompound();
        if (nbt == null || !nbt.hasKey("secondaryColor")) {
            return EnumColor.GRAY;
        }
        return EnumColor.fromId(nbt.getByte("secondaryColor"));
    }

    public static float getWhistlePitch(ItemStack stack) {
        NBTTagCompound nbt = stack.getTagCompound();
        if (nbt == null || !nbt.hasKey("whistlePitch")) {
            return -1;
        }
        return nbt.getFloat("whistlePitch");
    }
}
