/*
 * Copyright (c) CovertJaguar, 2011 http://railcraft.info
 * 
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.carts;

import java.util.List;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.world.World;
import net.minecraftforge.oredict.OreDictionary;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.EnumColor;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class LocomotivePaintingRecipe implements IRecipe {

    private static List<ItemStack>[] dyes = new List[16];

    static {
        for (int id = 0; id < 16; id++) {
            dyes[id] = OreDictionary.getOres(EnumColor.DYES[id]);
        }
    }
    private final ItemStack locomotive;

    public LocomotivePaintingRecipe(ItemStack locomotive) {
        this.locomotive = locomotive;
    }

    private boolean isDye(ItemStack stack) {
        return getDye(stack) != -1;
    }

    private int getDye(ItemStack stack) {
        for (int id = 0; id < 16; id++) {
            if (InvTools.isItemEqual(stack, dyes[id])) {
                return id;
            }
        }
        return -1;
    }

    private boolean isLocomotive(ItemStack loco) {
        return InvTools.isItemEqualIgnoreNBT(this.locomotive, loco);
    }

    @Override
    public boolean matches(InventoryCrafting craftingGrid, World var2) {
        if (craftingGrid.getSizeInventory() < getRecipeSize()) {
            return false;
        }
        ItemStack dyePrimary = craftingGrid.getStackInRowAndColumn(1, 0);
        if (!isDye(dyePrimary)) {
            return false;
        }
        ItemStack cart = craftingGrid.getStackInRowAndColumn(1, 1);
        if (!isLocomotive(cart)) {
            return false;
        }
        ItemStack dyeSecondary = craftingGrid.getStackInRowAndColumn(1, 2);
        if (!isDye(dyeSecondary)) {
            return false;
        }
        return true;
    }

    @Override
    public ItemStack getCraftingResult(InventoryCrafting craftingGrid) {
        ItemStack result = getRecipeOutput().copy();

        ItemStack dyePrimary = craftingGrid.getStackInRowAndColumn(1, 0);
        ItemStack dyeSecondary = craftingGrid.getStackInRowAndColumn(1, 2);

        ItemLocomotive.setItemColorData(result, getDye(dyePrimary), getDye(dyeSecondary));
        return result;
    }

    @Override
    public int getRecipeSize() {
        return 9;
    }

    @Override
    public ItemStack getRecipeOutput() {
        return locomotive;
    }
}
