package railcraft.common.carts;

import buildcraft.api.inventory.ISpecialInventory;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import railcraft.common.api.carts.CartBase;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.carts.IItemTransfer;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.StandaloneInventory;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class MaintanceCartBase extends CartBase implements ISpecialInventory {

    protected static final double DRAG_FACTOR = 0.9;
    protected static final float MAX_SPEED = 0.1f;
    private static final int BLINK_DURATION = 3;
    private static final int DATA_ID_BLINK = 20;
    protected final StandaloneInventory patternInv = new StandaloneInventory(6, this);

    public MaintanceCartBase(World world) {
        super(world);
    }

    @Override
    protected void entityInit() {
        super.entityInit();
        dataWatcher.addObject(DATA_ID_BLINK, new Byte((byte) 0));
    }

    public IInventory getPattern() {
        return patternInv;
    }

    @Override
    public int getSizeInventory() {
        return 1;
    }

    protected void blink() {
        dataWatcher.updateObject(DATA_ID_BLINK, Byte.valueOf((byte) BLINK_DURATION));
    }

    protected void setBlink(byte blink) {
        dataWatcher.updateObject(DATA_ID_BLINK, Byte.valueOf((byte) blink));
    }

    protected byte getBlink() {
        return dataWatcher.getWatchableObjectByte(DATA_ID_BLINK);
    }

    @Override
    public void onUpdate() {
        super.onUpdate();
        if (Game.isNotHost(getWorld())) {
            return;
        }

        if (isBlinking()) {
            setBlink((byte) (getBlink() - 1));
        }
    }

    public abstract int[] getTextureArray();

    @Override
    public List<ItemStack> getItemsDropped() {
        List<ItemStack> items = new ArrayList<ItemStack>();
        items.add(getCartItem());
        return items;
    }

    protected void stockItems(int slotReplace, int slotStock) {
        ItemStack stackReplace = patternInv.getStackInSlot(slotReplace);

        ItemStack stackStock = getStackInSlot(slotStock);

        if (stackStock != null && !InvTools.isItemEqual(stackReplace, stackStock)) {
            CartTools.offerOrDropItem(this, stackStock);
            setInventorySlotContents(slotStock, null);
        }

        if (stackReplace == null) {
            return;
        }

        stackStock = getStackInSlot(slotStock);

        EntityMinecart link_A = LinkageManager.getInstance(worldObj).getLinkedCartA(this);
        EntityMinecart link_B = LinkageManager.getInstance(worldObj).getLinkedCartB(this);

        if (stackStock == null || stackStock.stackSize < stackStock.getMaxStackSize()) {
            ItemStack stack = null;
            if (link_A instanceof IItemTransfer) {
                stack = ((IItemTransfer) link_A).requestItem(this, stackReplace);
                if (stack != null) {
                    if (stackStock == null) {
                        setInventorySlotContents(slotStock, stack);
                    } else {
                        stackStock.stackSize++;
                    }
                }
            }
            if (stack == null && link_B instanceof IItemTransfer) {
                stack = ((IItemTransfer) link_B).requestItem(this, stackReplace);
                if (stack != null) {
                    if (stackStock == null) {
                        setInventorySlotContents(slotStock, stack);
                    } else {
                        stackStock.stackSize++;
                    }
                }
            }
        }
    }

    @Override
    public boolean canBeRidden() {
        return false;
    }

    @Override
    public ItemStack[] extractItem(boolean doRemove, ForgeDirection from, int maxItemCount) {
        return new ItemStack[0];
    }

    @Override
    protected double getDrag() {
        return EntityCartTrackRelayer.DRAG_FACTOR;
    }

    @Override
    public float getCartMaxSpeed() {
        return MAX_SPEED;
    }

    public boolean isBlinking() {
        return dataWatcher.getWatchableObjectByte(DATA_ID_BLINK) > 0;
    }

    @Override
    public boolean isStorageCart() {
        return false;
    }

    @Override
    protected void writeEntityToNBT(NBTTagCompound data) {
        super.writeEntityToNBT(data);
        patternInv.writeToNBT("patternInv", data);
    }

    @Override
    protected void readEntityFromNBT(NBTTagCompound data) {
        super.readEntityFromNBT(data);
        patternInv.readFromNBT("patternInv", data);
    }
}
