/*
 * Copyright (c) CovertJaguar, 2011 http://railcraft.info
 * 
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.carts;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;
import net.minecraft.entity.item.EntityMinecart;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class Train {

    private final UUID uuid;
    private final Set<UUID> carts = new HashSet<UUID>();
    private final Set<UUID> safeSet = Collections.unmodifiableSet(carts);
    private final Set<UUID> lockingTracks = new HashSet<UUID>();

    public Train(EntityMinecart cart) {
        uuid = UUID.randomUUID();

        addCartsToTrain(cart);
    }

    private void addCartsToTrain(EntityMinecart cart) {
        addCart(cart);

        LinkageManager lm = LinkageManager.getInstance(cart.worldObj);
        EntityMinecart linkA = lm.getLinkedCartA(cart);
        EntityMinecart linkB = lm.getLinkedCartB(cart);

        if (linkA != null && !containsCart(linkA)) {
            addCartsToTrain(linkA);
        }

        if (linkB != null && !containsCart(linkB)) {
            addCartsToTrain(linkB);
        }
    }

    protected void releaseTrain(LinkageManager lm) {
        for (UUID id : carts) {
            EntityMinecart cart = lm.getCartFromLinkageId(id);
            if (cart != null) {
                cart.getEntityData().removeTag(LinkageManager.TRAIN_HIGH);
                cart.getEntityData().removeTag(LinkageManager.TRAIN_LOW);
            }
        }
        carts.clear();
        lockingTracks.clear();
    }

    public UUID getUUID() {
        return uuid;
    }

    private void addCart(EntityMinecart cart) {
        carts.add(cart.getPersistentID());
        cart.getEntityData().setLong(LinkageManager.TRAIN_HIGH, uuid.getMostSignificantBits());
        cart.getEntityData().setLong(LinkageManager.TRAIN_LOW, uuid.getLeastSignificantBits());
    }

    public boolean containsCart(EntityMinecart cart) {
        if (cart == null) {
            return false;
        }
        return carts.contains(cart.getPersistentID());
    }

    public Set<UUID> getCartsInTrain() {
        return safeSet;
    }

    public int getSize() {
        return carts.size();
    }

    public boolean isTrainLockedDown() {
        return !lockingTracks.isEmpty();
    }

    public void addLockingTrack(UUID track) {
        lockingTracks.add(track);
    }

    public void removeLockingTrack(UUID track) {
        lockingTracks.remove(track);
    }

    public static boolean isTrainLockedDown(EntityMinecart cart) {
        if (cart == null) {
            return false;
        }
        LinkageManager lm = LinkageManager.getInstance(cart.worldObj);
        Train train = lm.getTrain(cart);
        return train.isTrainLockedDown();
    }
}
