package railcraft.common.core;

import com.google.common.base.Splitter;
import com.google.common.collect.Iterables;
import com.google.common.primitives.Ints;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.Init;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.Mod.PostInit;
import cpw.mods.fml.common.Mod.PreInit;
import cpw.mods.fml.common.Mod.ServerStopping;
import cpw.mods.fml.common.SidedProxy;
import cpw.mods.fml.common.event.FMLFingerprintViolationEvent;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLInterModComms;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.event.FMLServerStoppingEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.registry.TickRegistry;
import cpw.mods.fml.relauncher.Side;
import java.io.File;
import java.util.logging.Level;
import net.minecraftforge.liquids.LiquidDictionary;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.api.fuel.FuelManager;
import railcraft.common.blocks.aesthetics.cube.EnumCube;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.machine.beta.EnumMachineBeta;
import railcraft.common.blocks.machine.gamma.EnumMachineGamma;
import railcraft.common.blocks.aesthetics.post.BlockPostMetal;
import railcraft.common.blocks.aesthetics.post.EnumPost;
import railcraft.common.blocks.signals.EnumSignal;
import railcraft.common.carts.LinkageManager;
import railcraft.common.modules.ModuleManager;
import railcraft.common.plugins.craftguide.CraftGuidePlugin;
import railcraft.common.util.inventory.EnumItemType;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.plugins.forge.Creeper$Plugin;
import railcraft.common.plugins.forge.Fishing$Plugin;
import railcraft.common.util.misc.BallastRegistry;
import railcraft.common.util.misc.BlinkTick;
import railcraft.common.util.misc.EnumColor;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.RailcraftPacket;
import railcraft.common.util.network.RailcraftPacketHandler;

@Mod(modid = "Railcraft", name = "Railcraft",
version = Railcraft.VERSION,
certificateFingerprint = "a0c255ac501b2749537d5824bb0f0588bf0320fa",
dependencies = "required-after:Forge@[6.6.0.516,);"
+ "after:BuildCraft|Core;"
+ "after:BuildCraft|Energy;"
+ "after:BuildCraft|Builders;"
+ "after:BuildCraft|Factory;"
+ "after:Forestry;"
+ "after:BasicComponents@[1.2.6,);"
+ "after:IC2")
@NetworkMod(channels = {RailcraftPacket.CHANNEL_NAME}, clientSideRequired = true, serverSideRequired = false, packetHandler = RailcraftPacketHandler.class)
public final class Railcraft {

    public static final String VERSION = "6.16.0.3";
    @Instance("Railcraft")
    public static Railcraft instance;
    private File configFolder;
//    public int totalMultiBlockUpdates = 0;
//    public int ticksSinceLastMultiBlockPrint = 0;
    @SidedProxy(clientSide = "railcraft.client.core.ClientProxy", serverSide = "railcraft.common.core.CommonProxy")
    public static CommonProxy proxy;

    public static CommonProxy getProxy() {
        return proxy;
    }

    public static Railcraft getMod() {
        return instance;
    }

    public static String getVersion() {
        return VERSION;
    }

    public File getConfigFolder() {
        return configFolder;
    }

    @Mod.IMCCallback
    public void processIMCRequests(FMLInterModComms.IMCEvent event) {
        Splitter splitter = Splitter.on("@").trimResults();
        for (FMLInterModComms.IMCMessage mess : event.getMessages()) {
            if (mess.key.equals("ballast")) {
                String[] tokens = Iterables.toArray(splitter.split(mess.getStringValue()), String.class);
                if (tokens.length != 2) {
                    Game.log(Level.WARNING, String.format("Mod %s attempted to register a ballast, but failed: %s", mess.getSender(), mess.getStringValue()));
                    continue;
                }
                Integer blockId = Ints.tryParse(tokens[0]);
                Integer metadata = Ints.tryParse(tokens[1]);
                if (blockId == null || metadata == null) {
                    Game.log(Level.WARNING, String.format("Mod %s attempted to register a ballast, but failed: %s", mess.getSender(), mess.getStringValue()));
                    continue;
                }
                BallastRegistry.registerBallast(blockId, metadata);
                Game.log(Level.WARNING, String.format("Mod %s registered %s as a valid ballast", mess.getSender(), mess.getStringValue()));
            } else if (mess.key.equals("boiler-fuel-liquid")) {
                String[] tokens = Iterables.toArray(splitter.split(mess.getStringValue()), String.class);
                if (tokens.length != 2) {
                    Game.log(Level.WARNING, String.format("Mod %s attempted to register a liquid Boiler fuel, but failed: %s", mess.getSender(), mess.getStringValue()));
                    continue;
                }
                LiquidStack liquid = LiquidDictionary.getLiquid(tokens[0], LiquidManager.BUCKET_VOLUME);
                Integer fuel = Ints.tryParse(tokens[1]);
                if (liquid == null || fuel == null) {
                    Game.log(Level.WARNING, String.format("Mod %s attempted to register a liquid Boiler fuel, but failed: %s", mess.getSender(), mess.getStringValue()));
                    continue;
                }
                FuelManager.addBoilerFuel(liquid, fuel);
                Game.log(Level.WARNING, String.format("Mod %s registered %s as a valid liquid Boiler fuel", mess.getSender(), mess.getStringValue()));
            } else if (mess.key.equals("securityViolation")) {
                Creeper$Plugin.doFix();
//                Game.log(Level.FINEST, "{0} asked for a bailout!", mess.getSender());
            }
        }
    }

    @Mod.FingerprintWarning
    public void fingerprintWarning(FMLFingerprintViolationEvent event) {
        if (Game.isObfuscated()) {
            Creeper$Plugin.pluckTheWeb("Tampering Detected. Please re-download Railcraft.");
            Runtime.getRuntime().halt(1);
        }
    }

    @PreInit
    public void preInit(FMLPreInitializationEvent event) {
        Game.log(Level.FINE, "Pre-Init Phase");

        Fishing$Plugin.reel$ItIn();

        configFolder = new File(event.getModConfigurationDirectory(), "railcraft");

        StartupChecks.checkForNewVersion();

        EnumItemType.initialize();

        ModuleManager.preInit();

        proxy.preInitClient();
    }

    @Init
    public void init(FMLInitializationEvent event) {
        Game.log(Level.FINE, "Init Phase");

        ModuleManager.init();

        TickRegistry.registerScheduledTickHandler(new BlinkTick(), Side.CLIENT);

        // Finish initializing ItemRegistry

        for (EnumMachineAlpha type : EnumMachineAlpha.values()) {
            if (type.isEnabled()) {
                ItemRegistry.registerItem(type.getTag(), type.getItem());
            }
        }

        for (EnumMachineBeta type : EnumMachineBeta.values()) {
            if (type.isEnabled()) {
                ItemRegistry.registerItem(type.getTag(), type.getItem());
            }
        }

        for (EnumMachineGamma type : EnumMachineGamma.values()) {
            if (type.isEnabled()) {
                ItemRegistry.registerItem(type.getTag(), type.getItem());
            }
        }

        for (EnumSignal type : EnumSignal.values()) {
            if (type.isEnabled()) {
                switch (type) {
                    case STONE_POST:
                    case WOOD_POST:
                    case METAL_POST:
                        break;
                    default:
                        ItemRegistry.registerItem(type.getTag(), type.getItem());
                }
            }
        }

        for (EnumCube type : EnumCube.values()) {
            if (type.isEnabled()) {
                ItemRegistry.registerItem(type.getTag(), type.getItem());
            }
        }

        for (EnumPost type : EnumPost.values()) {
            ItemRegistry.registerItem(type.getTag(), type.getItem());
        }

        for (EnumColor color : EnumColor.values()) {
            ItemRegistry.registerItem("post.metal." + color.getBasicTag(), BlockPostMetal.getItem(1, color.ordinal()));
        }
    }

    @PostInit
    public void postInit(FMLPostInitializationEvent event) {
        Game.log(Level.FINE, "Post-Init Phase");
        ModuleManager.postInit();

        proxy.initClient();

        CraftGuidePlugin.init();
    }

    @ServerStopping
    public void serverCleanUp(FMLServerStoppingEvent event) {
        LinkageManager.reset();
    }
}
