package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.gui.slots.SlotSimple;
import railcraft.common.util.misc.IAnchor;

public class ContainerAnchor extends RailcraftContainer {

    public IAnchor anchor;
    private Slot pearl;
    public short minutesRemaining;
    private short prevMinutesRemaining;

    public ContainerAnchor(InventoryPlayer inventoryplayer, IAnchor anchor) {
        super(anchor);
        this.anchor = anchor;
        addSlot(pearl = new SlotSimple(new ItemStack(RailcraftConfig.anchorFuelItemID(), 1, 0), anchor, 0, 60, 24));
        for (int i = 0; i < 3; i++) {
            for (int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 58 + i * 18));
            }

        }

        for (int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 116));
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        icrafting.sendProgressBarUpdate(this, 0, getMinutesRemaining(anchor.getAnchorFuel()));
    }

    private short getMinutesRemaining(long fuel) {
        return (short) Math.ceil((double) fuel / RailcraftConstants.TICKS_PER_MIN);
    }

    /**
     * Updates crafting matrix; called from onCraftMatrixChanged. Args: none
     */
    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        short mins = getMinutesRemaining(anchor.getAnchorFuel());

        for (int var1 = 0; var1 < this.crafters.size(); ++var1) {
            ICrafting var2 = (ICrafting) this.crafters.get(var1);

            if (this.prevMinutesRemaining != mins) {
                var2.sendProgressBarUpdate(this, 0, mins);
            }
        }

        this.prevMinutesRemaining = mins;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int id, int value) {
        if (id == 0) {
            minutesRemaining = (short) value;
        }
    }

//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack itemstack = null;
//        Slot slot = (Slot) inventorySlots.get(i);
//        if (slot != null && slot.getHasStack()) {
//            ItemStack itemstack1 = slot.getStack();
//            itemstack = itemstack1.copy();
//            if (i >= 1 && pearl.isItemValid(itemstack1)) {
//                if (!mergeItemStack(itemstack1, 0, 1, false)) {
//                    return null;
//                }
//            } else if (i >= 1 && i < 28) {
//                if (!mergeItemStack(itemstack1, 28, 37, false)) {
//                    return null;
//                }
//            } else if (i >= 28 && i < 37) {
//                if (!mergeItemStack(itemstack1, 1, 28, false)) {
//                    return null;
//                }
//            } else if (!mergeItemStack(itemstack1, 1, 37, false)) {
//                return null;
//            }
//            if (itemstack1.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if (itemstack1.stackSize != itemstack.stackSize) {
//                slot.onPickupFromSlot(player, itemstack1);
//            } else {
//                return null;
//            }
//        }
//        return itemstack;
//    }
}
