package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.inventory.SlotFurnace;
import railcraft.common.blocks.machine.alpha.TileBlastFurnace;
import railcraft.common.gui.slots.SlotBlastFurnaceFuel;
import railcraft.common.gui.slots.SlotBlastFurnaceInput;

public class ContainerBlastFurnace extends RailcraftContainer
{

    private TileBlastFurnace furnace;
    private int lastCookTime = 0;
    private int lastBurnTime = 0;
    private int lastItemBurnTime = 0;

    public ContainerBlastFurnace(InventoryPlayer player, TileBlastFurnace tile)
    {
        super(tile);
        this.furnace = tile;
        this.addSlot(new SlotBlastFurnaceInput(tile, 0, 56, 17));
        this.addSlot(new SlotBlastFurnaceFuel(tile, 1, 56, 53));
        this.addSlot(new SlotFurnace(player.player, tile, 2, 116, 35));
        int var3;

        for(var3 = 0; var3 < 3; ++var3) {
            for(int var4 = 0; var4 < 9; ++var4) {
                this.addSlot(new Slot(player, var4 + var3 * 9 + 9, 8 + var4 * 18, 84 + var3 * 18));
            }
        }

        for(var3 = 0; var3 < 9; ++var3) {
            this.addSlot(new Slot(player, var3, 8 + var3 * 18, 142));
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting)
    {
        super.addCraftingToCrafters(icrafting);
        icrafting.sendProgressBarUpdate(this, 0, furnace.getCookTime());
    }

    /**
     * Updates crafting matrix; called from onCraftMatrixChanged. Args: none
     */
    @Override
    public void detectAndSendChanges()
    {
        super.detectAndSendChanges();

        for(int var1 = 0; var1 < this.crafters.size(); ++var1) {
            ICrafting var2 = (ICrafting)this.crafters.get(var1);

            if(this.lastCookTime != this.furnace.getCookTime()) {
                var2.sendProgressBarUpdate(this, 0, this.furnace.getCookTime());
            }

            if(this.lastBurnTime != this.furnace.burnTime) {
                var2.sendProgressBarUpdate(this, 1, this.furnace.burnTime);
            }

            if(this.lastItemBurnTime != this.furnace.currentItemBurnTime) {
                var2.sendProgressBarUpdate(this, 2, this.furnace.currentItemBurnTime);
            }
        }

        this.lastCookTime = this.furnace.getCookTime();
        this.lastBurnTime = this.furnace.burnTime;
        this.lastItemBurnTime = this.furnace.currentItemBurnTime;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int par1, int par2)
    {
        if(par1 == 0) {
            this.furnace.setCookTime(par2);
        }

        if(par1 == 1) {
            this.furnace.burnTime = par2;
        }

        if(par1 == 2) {
            this.furnace.currentItemBurnTime = par2;
        }
    }
    
    /**
     * Called to transfer a stack from one inventory to the other eg. when shift clicking.
     */
//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int slotNum)
//    {
//        ItemStack original = null;
//        Slot slot = (Slot)this.inventorySlots.get(slotNum);
//
//        if(slot != null && slot.getHasStack()) {
//            ItemStack stack = slot.getStack();
//            original = stack.copy();
//
//            if(slotNum >= 0 && slotNum < 3) {
//                if(!this.mergeItemStack(stack, 3, 39, true)) {
//                    return null;
//                }
//            } else if(slotNum >= 3 && slotNum < 39 && SlotBlastFurnaceInput.canPlaceItem(stack)) {
//                if(!this.mergeItemStack(stack, 0, 1, false)) {
//                    return null;
//                }
//            } else if(slotNum >= 3 && slotNum < 39 && SlotBlastFurnaceFuel.canPlaceItem(stack)) {
//                if(!this.mergeItemStack(stack, 1, 2, false)) {
//                    return null;
//                }
//            } else if(slotNum >= 3 && slotNum < 30) {
//                if(!this.mergeItemStack(stack, 30, 39, false)) {
//                    return null;
//                }
//            } else if(slotNum >= 30 && slotNum < 39) {
//                if(!this.mergeItemStack(stack, 3, 30, false)) {
//                    return null;
//                }
//            } else if(!this.mergeItemStack(stack, 3, 39, false)) {
//                return null;
//            }
//
//            if(stack.stackSize == 0) {
//                slot.putStack((ItemStack)null);
//            } else {
//                slot.onSlotChanged();
//            }
//
//            if(stack.stackSize == original.stackSize) {
//                return null;
//            }
//
//            slot.onPickupFromSlot(player, stack);
//        }
//
//        return original;
//    }
}
