package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.machine.beta.TileBoilerFireboxLiquid;
import railcraft.common.gui.indicator.Indicator;
import railcraft.common.gui.slots.SlotLiquidContainerFilled;
import railcraft.common.gui.slots.SlotOutput;
import railcraft.common.liquids.LiquidGauge;
import railcraft.common.liquids.TankManager;

public class ContainerBoilerLiquid extends RailcraftContainer
{

    private TileBoilerFireboxLiquid tile;
    private Slot input;
    private Slot output;
    private float lastBurnTime;
    private float lastItemBurnTime;
    private float lastHeat;

    public ContainerBoilerLiquid(InventoryPlayer inventoryplayer, TileBoilerFireboxLiquid tile) {
        super(tile);
        this.tile = tile;

        addGauge(new LiquidGauge(tile.getTanks(null)[0], 116, 23, 16, 47));
        addGauge(new LiquidGauge(tile.getTanks(null)[1], 17, 23, 16, 47));
        addGauge(new LiquidGauge(tile.getTanks(null)[2], 89, 23, 16, 47));

        addIndicator(new Indicator(tile.getHeatIndicator(), 40, 25, 6, 43, 176, 61));

        addSlot(input = new SlotLiquidContainerFilled(tile, 0, 143, 21));
        addSlot(output = new SlotOutput(tile, 1, 143, 56));

        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 84 + i * 18));
            }
        }

        for(int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 142));
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.initGuiData(this, icrafting, 0);
            tMan.initGuiData(this, icrafting, 1);
            tMan.initGuiData(this, icrafting, 2);
        }

        icrafting.sendProgressBarUpdate(this, 10, Math.round(tile.burnTime));
        icrafting.sendProgressBarUpdate(this, 11, Math.round(tile.currentItemBurnTime));
        icrafting.sendProgressBarUpdate(this, 12, Math.round(tile.heat));
    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.updateGuiData(this, crafters, 0);
            tMan.updateGuiData(this, crafters, 1);
            tMan.updateGuiData(this, crafters, 2);
        }

        for(int var1 = 0; var1 < this.crafters.size(); ++var1) {
            ICrafting var2 = (ICrafting)this.crafters.get(var1);

            if(this.lastBurnTime != tile.burnTime) {
                var2.sendProgressBarUpdate(this, 10, Math.round(tile.burnTime));
            }

            if(this.lastItemBurnTime != tile.currentItemBurnTime) {
                var2.sendProgressBarUpdate(this, 11, Math.round(tile.currentItemBurnTime));
            }

            if(this.lastHeat != tile.heat) {
                var2.sendProgressBarUpdate(this, 12, Math.round(tile.heat));
            }
        }

        this.lastBurnTime = tile.burnTime;
        this.lastItemBurnTime = tile.currentItemBurnTime;
        this.lastHeat = tile.heat;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int id, int value) {
        TankManager tMan = tile.getTankManager();
        if (tMan != null) {
            tMan.processGuiUpdate(id, value);
        }

        switch (id) {
            case 10:
                tile.burnTime = value;
                break;
            case 11:
                tile.currentItemBurnTime = value;
                break;
            case 12:
                tile.heat = value;
                break;
        }
    }

//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack startStack = null;
//        Slot slot = (Slot)inventorySlots.get(i);
//        if(slot != null && slot.getHasStack()) {
//            ItemStack moveStack = slot.getStack();
//            startStack = moveStack.copy();
//             if(i >= 2 && input.isItemValid(moveStack)) {
//                if(!mergeItemStack(moveStack, 0, 1, false)) {
//                    return null;
//                }
//            } else if(i >= 2 && i < 29) {
//                if(!mergeItemStack(moveStack, 29, 38, false)) {
//                    return null;
//                }
//            } else if(i >= 29 && i < 38) {
//                if(!mergeItemStack(moveStack, 2, 29, false)) {
//                    return null;
//                }
//            } else if(!mergeItemStack(moveStack, 2, 38, false)) {
//                return null;
//            }
//            if(moveStack.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if(moveStack.stackSize != startStack.stackSize) {
//                slot.onPickupFromSlot(player, moveStack);
//            } else {
//                return null;
//            }
//        }
//        return startStack;
//    }
}
