package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.machine.alpha.TileCokeOven;
import railcraft.common.gui.slots.SlotCokeOven;
import railcraft.common.gui.slots.SlotLiquidContainerEmpty;
import railcraft.common.gui.slots.SlotOutput;
import railcraft.common.liquids.LiquidGauge;
import railcraft.common.liquids.TankManager;

public class ContainerCokeOven extends RailcraftContainer
{

    private TileCokeOven tile;
    private int lastCookTime;

    public ContainerCokeOven(InventoryPlayer inventoryplayer, TileCokeOven tile) {
        super(tile);
        lastCookTime = 0;
        this.tile = tile;

        addGauge(new LiquidGauge(tile.getTanks(null)[0], 90, 24, 48, 47));

        addSlot(new SlotCokeOven(tile, 0, 16, 43));
        addSlot(new SlotOutput(tile, 1, 62, 43));
        addSlot(new SlotOutput(tile, 2, 149, 57));
        addSlot(new SlotLiquidContainerEmpty(tile, 3, 149, 22));
        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 84 + i * 18));
            }

        }

        for(int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 142));
        }

    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();

        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.updateGuiData(this, crafters, 0);
        }

        for(int i = 0; i < crafters.size(); i++) {
            ICrafting icrafting = (ICrafting)crafters.get(i);
            if(lastCookTime != tile.getCookTime()) {
                icrafting.sendProgressBarUpdate(this, 10, tile.getCookTime());
            }
        }

        lastCookTime = tile.getCookTime();
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);

        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.initGuiData(this, icrafting, 0);
        }

        icrafting.sendProgressBarUpdate(this, 10, tile.getCookTime());
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int id, int data) {
        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.processGuiUpdate(id, data);
        }

        if(id == 10) {
            tile.setCookTime(data);
        }
    }

//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack itemstack = null;
//        Slot slot = (Slot)inventorySlots.get(i);
//        if(slot != null && slot.getHasStack()) {
//            ItemStack itemstack1 = slot.getStack();
//            itemstack = itemstack1.copy();
//            if(i >= 4 && i < 40 && SlotCokeOven.canPlaceItem(itemstack)) {
//                if(!mergeItemStack(itemstack1, 0, 1, false)) {
//                    return null;
//                }
//            } else if(i >= 4 && i < 40 && SlotLiquidContainerEmpty.canPlaceItem(itemstack)) {
//                if(!mergeItemStack(itemstack1, 3, 4, false)) {
//                    return null;
//                }
//            } else if(i >= 4 && i < 31) {
//                if(!mergeItemStack(itemstack1, 31, 40, false)) {
//                    return null;
//                }
//            } else if(i >= 31 && i < 40) {
//                if(!mergeItemStack(itemstack1, 4, 31, false)) {
//                    return null;
//                }
//            } else if(!mergeItemStack(itemstack1, 4, 40, false)) {
//                return null;
//            }
//            if(itemstack1.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if(itemstack1.stackSize != itemstack.stackSize) {
//                slot.onPickupFromSlot(player, itemstack1);
//            } else {
//                return null;
//            }
//        }
//        return itemstack;
//    }
}
