package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.machine.beta.TileEngineSteam;
import railcraft.common.gui.indicator.Indicator;
import railcraft.common.liquids.LiquidGauge;

public class ContainerEngineSteam extends RailcraftContainer
{

    private TileEngineSteam tile;
    private float lastEnergy;
    private float lastOutput;

    public ContainerEngineSteam(InventoryPlayer inventoryplayer, TileEngineSteam tile) {
        this.tile = tile;

        addGauge(new LiquidGauge(tile.getTanks(null)[0], 71, 23, 16, 47));

        addIndicator(new Indicator(tile.getEnergyIndicator(), 94, 25, 6, 43, 176, 47));

        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 84 + i * 18));
            }
        }

        for(int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 142));
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        tile.getTankManager().initGuiData(this, icrafting, 0);

        icrafting.sendProgressBarUpdate(this, 12, Math.round(tile.energy));
        icrafting.sendProgressBarUpdate(this, 13, Math.round(tile.currentOutput * 100));
    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        tile.getTankManager().updateGuiData(this, crafters, 0);

        for(int var1 = 0; var1 < this.crafters.size(); ++var1) {
            ICrafting var2 = (ICrafting)this.crafters.get(var1);

            if(this.lastEnergy != tile.energy) {
                var2.sendProgressBarUpdate(this, 12, Math.round(tile.energy));
            }

            if(this.lastOutput != tile.currentOutput) {
                var2.sendProgressBarUpdate(this, 13, Math.round(tile.currentOutput * 100));
            }
        }

        this.lastEnergy = tile.energy;
        this.lastOutput = tile.currentOutput;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int id, int value) {
        tile.getTankManager().processGuiUpdate(id, value);

        switch (id) {
            case 12:
                tile.energy = value;
                break;
            case 13:
                tile.currentOutput = value / 100f;
                break;
        }
    }

    @Override
    public boolean canInteractWith(EntityPlayer entityplayer) {
        return tile.isUseableByPlayer(entityplayer);
    }

//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack startStack = null;
//        Slot slot = (Slot)inventorySlots.get(i);
//        if(slot != null && slot.getHasStack()) {
//            ItemStack moveStack = slot.getStack();
//            startStack = moveStack.copy();
//            if(i < 27) {
//                if(!mergeItemStack(moveStack, 27, 36, false)) {
//                    return null;
//                }
//            } else if(!mergeItemStack(moveStack, 0, 27, false)) {
//                return null;
//            }
//            if(moveStack.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if(moveStack.stackSize != startStack.stackSize) {
//                slot.onPickupFromSlot(player, moveStack);
//            } else {
//                return null;
//            }
//        }
//        return startStack;
//    }
}
