package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.machine.beta.TileEngineSteamHobby;
import railcraft.common.gui.indicator.Indicator;
import railcraft.common.gui.slots.SlotFuel;
import railcraft.common.gui.slots.SlotOutput;
import railcraft.common.gui.slots.SlotWater;
import railcraft.common.liquids.LiquidGauge;

public class ContainerEngineSteamHobby extends RailcraftContainer
{

    private TileEngineSteamHobby tile;
    private Slot fuel;
    private Slot input;
    private Slot output;
    private float lastBurnTime;
    private float lastItemBurnTime;
    private float lastEnergy;
    private float lastOutput;
    private float lastHeat;

    public ContainerEngineSteamHobby(InventoryPlayer inventoryplayer, TileEngineSteamHobby tile) {
        super(tile);
        this.tile = tile;

        addGauge(new LiquidGauge(tile.getTanks(null)[0], 17, 23, 16, 47));
        addGauge(new LiquidGauge(tile.getTanks(null)[1], 107, 23, 16, 47));

        addIndicator(new Indicator(tile.getHeatIndicator(), 40, 25, 6, 43, 176, 61));
        addIndicator(new Indicator(tile.getEnergyIndicator(), 94, 25, 6, 43, 182, 61));

        addSlot(fuel = new SlotFuel(tile, 0, 62, 39));
        addSlot(input = new SlotWater(tile, 1, 143, 21));
        addSlot(output = new SlotOutput(tile, 2, 143, 56));

        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 84 + i * 18));
            }
        }

        for(int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 142));
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        tile.getTankManager().initGuiData(this, icrafting, 0);
        tile.getTankManager().initGuiData(this, icrafting, 1);

        icrafting.sendProgressBarUpdate(this, 10, Math.round(tile.burnTime));
        icrafting.sendProgressBarUpdate(this, 11, Math.round(tile.currentItemBurnTime));
        icrafting.sendProgressBarUpdate(this, 12, Math.round(tile.energy));
        icrafting.sendProgressBarUpdate(this, 13, Math.round(tile.currentOutput * 100));
        icrafting.sendProgressBarUpdate(this, 14, Math.round(tile.heat));
    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        tile.getTankManager().updateGuiData(this, crafters, 0);
        tile.getTankManager().updateGuiData(this, crafters, 1);

        for(int var1 = 0; var1 < this.crafters.size(); ++var1) {
            ICrafting var2 = (ICrafting)this.crafters.get(var1);

            if(this.lastBurnTime != tile.burnTime) {
                var2.sendProgressBarUpdate(this, 10, Math.round(tile.burnTime));
            }

            if(this.lastItemBurnTime != tile.currentItemBurnTime) {
                var2.sendProgressBarUpdate(this, 11, Math.round(tile.currentItemBurnTime));
            }

            if(this.lastEnergy != tile.energy) {
                var2.sendProgressBarUpdate(this, 12, Math.round(tile.energy));
            }

            if(this.lastOutput != tile.currentOutput) {
                var2.sendProgressBarUpdate(this, 13, Math.round(tile.currentOutput * 100));
            }

            if(this.lastHeat != tile.heat) {
                var2.sendProgressBarUpdate(this, 14, Math.round(tile.heat));
            }
        }

        this.lastBurnTime = tile.burnTime;
        this.lastItemBurnTime = tile.currentItemBurnTime;
        this.lastEnergy = tile.energy;
        this.lastOutput = tile.currentOutput;
        this.lastHeat = tile.heat;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int id, int value) {
        tile.getTankManager().processGuiUpdate(id, value);

        switch (id) {
            case 10:
                tile.burnTime = value;
                break;
            case 11:
                tile.currentItemBurnTime = value;
                break;
            case 12:
                tile.energy = value;
                break;
            case 13:
                tile.currentOutput = value / 100f;
                break;
            case 14:
                tile.heat = value;
                break;
        }
    }

//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack startStack = null;
//        Slot slot = (Slot)inventorySlots.get(i);
//        if(slot != null && slot.getHasStack()) {
//            ItemStack moveStack = slot.getStack();
//            startStack = moveStack.copy();
//            if(i >= 3 && fuel.isItemValid(moveStack)) {
//                if(!mergeItemStack(moveStack, 0, 1, false)) {
//                    return null;
//                }
//            } else if(i >= 3 && input.isItemValid(moveStack)) {
//                if(!mergeItemStack(moveStack, 1, 2, false)) {
//                    return null;
//                }
//            } else if(i >= 3 && i < 30) {
//                if(!mergeItemStack(moveStack, 30, 39, false)) {
//                    return null;
//                }
//            } else if(i >= 30 && i < 39) {
//                if(!mergeItemStack(moveStack, 3, 30, false)) {
//                    return null;
//                }
//            } else if(!mergeItemStack(moveStack, 3, 39, false)) {
//                return null;
//            }
//            if(moveStack.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if(moveStack.stackSize != startStack.stackSize) {
//                slot.onPickupFromSlot(player, moveStack);
//            } else {
//                return null;
//            }
//        }
//        return startStack;
//    }
}
