package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import railcraft.common.carts.EntityLocomotiveSteam;
import railcraft.common.gui.indicator.Indicator;
import railcraft.common.gui.slots.SlotFuel;
import railcraft.common.gui.slots.SlotOutput;
import railcraft.common.gui.slots.SlotWaterLimited;
import railcraft.common.liquids.LiquidGauge;
import railcraft.common.liquids.TankManager;

public class ContainerLocomotiveSteam extends RailcraftContainer {

    private EntityLocomotiveSteam loco;
    private Slot fuel;
    private Slot input;
    private Slot output;
    private float lastBurnTime;
    private float lastItemBurnTime;
    private float lastHeat;

    public ContainerLocomotiveSteam(InventoryPlayer inventoryplayer, EntityLocomotiveSteam loco) {
        super(loco);
        this.loco = loco;

        addGauge(new LiquidGauge(loco.getTanks(null)[0], 116, 23, 16, 47));
        addGauge(new LiquidGauge(loco.getTanks(null)[1], 17, 23, 16, 47));

        addIndicator(new Indicator(loco.getHeatIndicator(), 40, 25, 6, 43, 176, 61));

        addSlot(input = new SlotWaterLimited(loco, 0, 143, 21));
        addSlot(output = new SlotOutput(loco, 1, 143, 56));
        addSlot(fuel = new SlotFuel(loco, 2, 62, 39));
        addSlot(new SlotFuel(loco, 3, 89, 20));
        addSlot(new SlotFuel(loco, 4, 89, 38));
        addSlot(new SlotFuel(loco, 5, 89, 56));

        for (int i = 0; i < 3; i++) {
            for (int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 123 + i * 18));
            }
        }

        for (int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 181));
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        TankManager tMan = loco.getTankManager();
        if (tMan != null) {
            tMan.initGuiData(this, icrafting, 0);
            tMan.initGuiData(this, icrafting, 1);
        }

        icrafting.sendProgressBarUpdate(this, 10, Math.round(loco.burnTime));
        icrafting.sendProgressBarUpdate(this, 11, Math.round(loco.currentItemBurnTime));
        icrafting.sendProgressBarUpdate(this, 12, Math.round(loco.heat));
    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        TankManager tMan = loco.getTankManager();
        if (tMan != null) {
            tMan.updateGuiData(this, crafters, 0);
            tMan.updateGuiData(this, crafters, 1);
        }

        for (int var1 = 0; var1 < this.crafters.size(); ++var1) {
            ICrafting var2 = (ICrafting) this.crafters.get(var1);

            if (this.lastBurnTime != loco.burnTime) {
                var2.sendProgressBarUpdate(this, 10, Math.round(loco.burnTime));
            }

            if (this.lastItemBurnTime != loco.currentItemBurnTime) {
                var2.sendProgressBarUpdate(this, 11, Math.round(loco.currentItemBurnTime));
            }

            if (this.lastHeat != loco.heat) {
                var2.sendProgressBarUpdate(this, 12, Math.round(loco.heat));
            }
        }

        this.lastBurnTime = loco.burnTime;
        this.lastItemBurnTime = loco.currentItemBurnTime;
        this.lastHeat = loco.heat;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int id, int value) {
        TankManager tMan = loco.getTankManager();
        if (tMan != null) {
            tMan.processGuiUpdate(id, value);
        }

        switch (id) {
            case 10:
                loco.burnTime = value;
                break;
            case 11:
                loco.currentItemBurnTime = value;
                break;
            case 12:
                loco.heat = value;
                break;
        }
    }
//
//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack startStack = null;
//        Slot slot = (Slot) inventorySlots.get(i);
//        if (slot != null && slot.getHasStack()) {
//            ItemStack moveStack = slot.getStack();
//            startStack = moveStack.copy();
//            if (i >= 6 && fuel.isItemValid(moveStack)) {
//                if (!mergeItemStack(moveStack, 2, 6, false)) {
//                    return null;
//                }
//            } else if (i >= 2 && input.isItemValid(moveStack)) {
//                if (!mergeItemStack(moveStack, 0, 1, false)) {
//                    return null;
//                }
//            } else if (i >= 6 && i < 32) {
//                if (!mergeItemStack(moveStack, 32, 41, false)) {
//                    return null;
//                }
//            } else if (i >= 32 && i < 41) {
//                if (!mergeItemStack(moveStack, 6, 32, false)) {
//                    return null;
//                }
//            } else if (!mergeItemStack(moveStack, 6, 41, false)) {
//                return null;
//            }
//            if (moveStack.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if (moveStack.stackSize != startStack.stackSize) {
//                slot.onPickupFromSlot(player, moveStack);
//            } else {
//                return null;
//            }
//        }
//        return startStack;
//    }
}
