package railcraft.common.gui.containers;

import buildcraft.api.power.IPowerProvider;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.machine.alpha.TileRockCrusher;
import railcraft.common.gui.indicator.Indicator;
import railcraft.common.gui.slots.SlotOutput;
import railcraft.common.gui.slots.SlotRockCrusher;

public class ContainerRockCrusher extends RailcraftContainer
{

    private TileRockCrusher tile;
    private int lastProcessTime;
    private int lastEnergy;

    public ContainerRockCrusher(InventoryPlayer inventoryplayer, TileRockCrusher crusher) {
        super(crusher);
        this.tile = crusher;

        addIndicator(new Indicator(crusher.getEnergyIndicator(), 157, 23, 6, 48, 176, 53));

        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 3; k++) {
                addSlot(new SlotRockCrusher(crusher, i * 3 + k, 8 + k * 18, 21 + i * 18));
            }
        }
        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 3; k++) {
                addSlot(new SlotOutput(crusher, 9 + i * 3 + k, 98 + k * 18, 21 + i * 18));
            }
        }
        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 84 + i * 18));
            }

        }

        for(int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 142));
        }

    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        for(int i = 0; i < crafters.size(); i++) {
            ICrafting icrafting = (ICrafting)crafters.get(i);

            if(lastProcessTime != tile.getProcessTime()) {
                icrafting.sendProgressBarUpdate(this, 0, tile.getProcessTime());
            }

            IPowerProvider provider = tile.getPowerProvider();
            if(provider != null && lastEnergy != provider.getEnergyStored()) {
                icrafting.sendProgressBarUpdate(this, 1, (int)provider.getEnergyStored());
            }
        }

        lastProcessTime = tile.getProcessTime();
        if(tile.getPowerProvider() != null) {
            lastEnergy = (int)tile.getPowerProvider().getEnergyStored();
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        icrafting.sendProgressBarUpdate(this, 0, tile.getProcessTime());
        IPowerProvider provider = tile.getPowerProvider();
        if(provider != null) {
            icrafting.sendProgressBarUpdate(this, 1, (int)provider.getEnergyStored());
        }
    }

    @Override
    public void updateProgressBar(int id, int data) {
        switch (id) {
            case 0:
                tile.setProcessTime(data);
                break;
            case 1:
                tile.guiEnergy = data;
                break;
        }
    }
    
//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack itemstack = null;
//        Slot slot = (Slot)inventorySlots.get(i);
//        if(slot != null && slot.getHasStack()) {
//            ItemStack itemstack1 = slot.getStack();
//            itemstack = itemstack1.copy();
//            if(i >= 9 && i < 54 && SlotRockCrusher.canPlaceItem(itemstack)) {
//                if(!mergeItemStack(itemstack1, 0, 9, false)) {
//                    return null;
//                }
//            } else if(i >= 18 && i < 45) {
//                if(!mergeItemStack(itemstack1, 45, 54, false)) {
//                    return null;
//                }
//            } else if(i >= 45 && i < 54) {
//                if(!mergeItemStack(itemstack1, 18, 45, false)) {
//                    return null;
//                }
//            } else if(!mergeItemStack(itemstack1, 18, 54, false)) {
//                return null;
//            }
//            if(itemstack1.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if(itemstack1.stackSize != itemstack.stackSize) {
//                slot.onPickupFromSlot(player, itemstack1);
//            } else {
//                return null;
//            }
//        }
//        return itemstack;
//    }
}
