package railcraft.common.gui.containers;

import buildcraft.api.power.IPowerProvider;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryCraftResult;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.inventory.Slot;
import railcraft.common.blocks.machine.alpha.TileRollingMachine;
import railcraft.common.gui.indicator.Indicator;
import railcraft.common.gui.slots.SlotOutput;
import railcraft.common.gui.slots.SlotRollingMachine;
import railcraft.common.gui.slots.SlotUnshiftable;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.crafting.RollingMachineCraftingManager;
import railcraft.common.util.inventory.InvTools;

public class ContainerRollingMachine extends RailcraftContainer {

    private TileRollingMachine tile;
    private InventoryCrafting craftMatrix;
    private IInventory craftResult;
    private int lastEnergy, lastProgress;
    private ItemStack prevOutput;

    public ContainerRollingMachine(InventoryPlayer inventoryplayer, TileRollingMachine tile) {
        super(tile);
        this.tile = tile;
        craftMatrix = tile.getCraftMatrix();
        craftResult = new InventoryCraftResult();

        addIndicator(new Indicator(tile.getEnergyIndicator(), 157, 19, 6, 48, 176, 12));

        addSlot(new SlotRollingMachine(tile, craftResult, 0, 93, 27));
        addSlot(new SlotOutput(tile, 0, 124, 35));

        for (int l = 0; l < 3; l++) {
            for (int k1 = 0; k1 < 3; k1++) {
                addSlot(new SlotUnshiftable(craftMatrix, k1 + l * 3, 30 + k1 * 18, 17 + l * 18));
            }
        }

        for (int i1 = 0; i1 < 3; i1++) {
            for (int l1 = 0; l1 < 9; l1++) {
                addSlot(new Slot(inventoryplayer, l1 + i1 * 9 + 9, 8 + l1 * 18, 84 + i1 * 18));
            }
        }

        for (int j1 = 0; j1 < 9; j1++) {
            addSlot(new Slot(inventoryplayer, j1, 8 + j1 * 18, 142));
        }

        onCraftMatrixChanged(craftMatrix);
    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        for (int i = 0; i < crafters.size(); i++) {
            ICrafting icrafting = (ICrafting) crafters.get(i);

            if (lastProgress != tile.getProgress()) {
                icrafting.sendProgressBarUpdate(this, 0, tile.getProgress());
            }

            IPowerProvider provider = tile.getPowerProvider();
            if (provider != null && lastEnergy != provider.getEnergyStored()) {
                icrafting.sendProgressBarUpdate(this, 1, (int) provider.getEnergyStored());
            }
        }

        ItemStack output = tile.getStackInSlot(0);
        if (!InvTools.isItemEqualStrict(output, prevOutput)) {
            onCraftMatrixChanged(craftMatrix);
            prevOutput = output != null ? output.copy() : null;
        }

        lastProgress = tile.getProgress();

        if (tile.getPowerProvider() != null) {
            lastEnergy = (int) tile.getPowerProvider().getEnergyStored();
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        icrafting.sendProgressBarUpdate(this, 0, tile.getProgress());
        IPowerProvider provider = tile.getPowerProvider();
        if (provider != null) {
            icrafting.sendProgressBarUpdate(this, 1, (int) provider.getEnergyStored());
        }
    }

    @Override
    public void updateProgressBar(int id, int data) {
        switch (id) {
            case 0:
                tile.setProgress(data);
                break;
            case 1:
                tile.guiEnergy = data;
                break;
        }
    }

    @Override
    public final void onCraftMatrixChanged(IInventory inv) {
        ItemStack output = RollingMachineCraftingManager.getInstance().findMatchingRecipe(craftMatrix, tile.worldObj);
        if (output != null) {
            InvTools.addItemToolTip(output, "tip", RailcraftLanguage.translate("gui.rolling.machine.tip.craft"));
        }
        craftResult.setInventorySlotContents(0, output);
    }

    @Override
    public ItemStack slotClick(int i, int j, int modifier, EntityPlayer entityplayer) {
        ItemStack stack = super.slotClick(i, j, modifier, entityplayer);
        onCraftMatrixChanged(craftMatrix);
        return stack;
    }
}
