package railcraft.common.gui.containers;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraftforge.liquids.ILiquidTank;
import railcraft.common.blocks.machine.ITankTile;
import railcraft.common.gui.slots.SlotOutput;
import railcraft.common.liquids.LiquidGauge;
import railcraft.common.liquids.TankManager;

public class ContainerTank extends RailcraftContainer
{

    private ITankTile tile;
    private Slot input;

    public ContainerTank(InventoryPlayer inventoryplayer, ITankTile tile) {
        super(tile.getInventory());
        this.tile = tile;

        ILiquidTank tank = tile.getTank();
        if(tank != null) {
            addGauge(new LiquidGauge(tank, 35, 23, 48, 47));
        }

        addSlot(input = tile.getInputSlot(tile.getInventory(), 0, 116, 21));
        addSlot(new SlotOutput(tile.getInventory(), 1, 116, 56));
        for(int i = 0; i < 3; i++) {
            for(int k = 0; k < 9; k++) {
                addSlot(new Slot(inventoryplayer, k + i * 9 + 9, 8 + k * 18, 84 + i * 18));
            }

        }

        for(int j = 0; j < 9; j++) {
            addSlot(new Slot(inventoryplayer, j, 8 + j * 18, 142));
        }
    }

    @Override
    public void detectAndSendChanges() {
        super.detectAndSendChanges();
        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.updateGuiData(this, crafters, 0);
        }
    }

    @Override
    public void addCraftingToCrafters(ICrafting icrafting) {
        super.addCraftingToCrafters(icrafting);
        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.initGuiData(this, icrafting, 0);
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void updateProgressBar(int id, int data) {
        TankManager tMan = tile.getTankManager();
        if(tMan != null) {
            tMan.processGuiUpdate(id, data);
        }
    }

//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int i) {
//        ItemStack itemstack = null;
//        Slot slot = (Slot)inventorySlots.get(i);
//        if(slot != null && slot.getHasStack()) {
//            ItemStack itemstack1 = slot.getStack();
//            itemstack = itemstack1.copy();
//            if(i >= 2 && i < 38 && input.isItemValid(itemstack1)) {
//                if(!mergeItemStack(itemstack1, 0, 1, false)) {
//                    return null;
//                }
//            } else if(i >= 2 && i < 29) {
//                if(!mergeItemStack(itemstack1, 29, 38, false)) {
//                    return null;
//                }
//            } else if(i >= 29 && i < 38) {
//                if(!mergeItemStack(itemstack1, 2, 29, false)) {
//                    return null;
//                }
//            } else if(!mergeItemStack(itemstack1, 2, 38, false)) {
//                return null;
//            }
//            if(itemstack1.stackSize == 0) {
//                slot.putStack(null);
//            } else {
//                slot.onSlotChanged();
//            }
//            if(itemstack1.stackSize != itemstack.stackSize) {
//                slot.onPickupFromSlot(player, itemstack1);
//            } else {
//                return null;
//            }
//        }
//        return itemstack;
//    }
}
