package railcraft.common.gui.containers;

import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryCraftResult;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.inventory.Slot;
import net.minecraft.inventory.SlotCrafting;
import railcraft.common.carts.EntityCartWork;
import railcraft.common.gui.slots.SlotUnshiftable;

public class ContainerWorkCart extends RailcraftContainer {

    /**
     * The crafting matrix inventory (3x3).
     */
    public InventoryCrafting craftMatrix = new InventoryCrafting(this, 3, 3);
    public IInventory craftResult = new InventoryCraftResult();
    private EntityCartWork cart;

    public ContainerWorkCart(InventoryPlayer inv, EntityCartWork cart) {
        super(cart);
        this.cart = cart;
        this.addSlot(new SlotCrafting(inv.player, this.craftMatrix, this.craftResult, 0, 124, 35));
        int var6;
        int var7;

        for (var6 = 0; var6 < 3; ++var6) {
            for (var7 = 0; var7 < 3; ++var7) {
                this.addSlot(new SlotUnshiftable(this.craftMatrix, var7 + var6 * 3, 30 + var7 * 18, 17 + var6 * 18));
            }
        }

        for (var6 = 0; var6 < 3; ++var6) {
            for (var7 = 0; var7 < 9; ++var7) {
                this.addSlot(new Slot(inv, var7 + var6 * 9 + 9, 8 + var7 * 18, 84 + var6 * 18));
            }
        }

        for (var6 = 0; var6 < 9; ++var6) {
            this.addSlot(new Slot(inv, var6, 8 + var6 * 18, 142));
        }

        this.onCraftMatrixChanged(this.craftMatrix);
    }

    /**
     * Callback for when the crafting matrix is changed.
     */
    @Override
    public void onCraftMatrixChanged(IInventory par1IInventory) {
        this.craftResult.setInventorySlotContents(0, CraftingManager.getInstance().findMatchingRecipe(this.craftMatrix, cart.worldObj));
    }

    /**
     * Callback for when the crafting gui is closed.
     */
    @Override
    public void onCraftGuiClosed(EntityPlayer par1EntityPlayer) {
        super.onCraftGuiClosed(par1EntityPlayer);

        if (!cart.worldObj.isRemote) {
            for (int var2 = 0; var2 < 9; ++var2) {
                ItemStack var3 = this.craftMatrix.getStackInSlotOnClosing(var2);

                if (var3 != null) {
                    par1EntityPlayer.dropPlayerItem(var3);
                }
            }
        }
    }

    /**
     * Called to transfer a stack from one inventory to the other eg. when shift
     * clicking.
     */
//    @Override
//    public ItemStack transferStackInSlot(EntityPlayer player, int par1) {
//        ItemStack var2 = null;
//        Slot var3 = (Slot) this.inventorySlots.get(par1);
//
//        if (var3 != null && var3.getHasStack()) {
//            ItemStack var4 = var3.getStack();
//            var2 = var4.copy();
//
//            if (par1 == 0) {
//                if (!this.mergeItemStack(var4, 10, 46, true)) {
//                    return null;
//                }
//
////                var3.func_48433_a(var4, var2);
//            } else if (par1 >= 10 && par1 < 37) {
//                if (!this.mergeItemStack(var4, 37, 46, false)) {
//                    return null;
//                }
//            } else if (par1 >= 37 && par1 < 46) {
//                if (!this.mergeItemStack(var4, 10, 37, false)) {
//                    return null;
//                }
//            } else if (!this.mergeItemStack(var4, 10, 46, false)) {
//                return null;
//            }
//
//            if (var4.stackSize == 0) {
//                var3.putStack((ItemStack) null);
//            } else {
//                var3.onSlotChanged();
//            }
//
//            if (var4.stackSize == var2.stackSize) {
//                return null;
//            }
//
//            var3.onPickupFromSlot(player, var4);
//        }
//
//        return var2;
//    }
}
