package railcraft.common.items;

import com.google.common.collect.MapMaker;
import java.util.Map;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraftforge.event.ForgeSubscribe;
import net.minecraftforge.event.entity.player.EntityInteractEvent;
import railcraft.common.api.carts.ILinkableCart;
import railcraft.common.api.carts.ILinkageManager;
import railcraft.common.api.core.items.IToolCrowbar;
import railcraft.common.carts.EntityTunnelBore;
import railcraft.common.carts.IDirectionalCart;
import railcraft.common.carts.LinkageManager;
import railcraft.common.modules.ModuleManager;
import railcraft.common.modules.ModuleManager.Module;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class CrowbarHandler {

    public static final float SMACK_VELOCITY = 0.07f;
    private static Map<EntityPlayer, EntityMinecart> linkMap = new MapMaker().weakKeys().weakValues().makeMap();
    private static CrowbarHandler instance;

    public static CrowbarHandler getInstance() {
        if (instance == null) {
            instance = new CrowbarHandler();
        }
        return instance;
    }

    @ForgeSubscribe
    public void onEntityInteract(EntityInteractEvent event) {
        EntityPlayer thePlayer = event.entityPlayer;
        Entity entity = event.target;
        thePlayer.swingItem();
        if (Game.isNotHost(thePlayer.worldObj)) {
            return;
        }
        if (!ModuleManager.isModuleLoaded(Module.TRAIN)) {
            return;
        }
        boolean used = false;
        ItemStack stack = thePlayer.getCurrentEquippedItem();
        if (stack != null && stack.getItem() instanceof IToolCrowbar) {
            IToolCrowbar crowbar = (IToolCrowbar) stack.getItem();
            if (entity instanceof EntityMinecart) {
                EntityMinecart cart = (EntityMinecart) entity;

                if (crowbar.canLink(thePlayer, stack, cart)) {
                    boolean linkable = cart instanceof ILinkableCart;
                    if (!linkable || (linkable && ((ILinkableCart) cart).isLinkable())) {
                        if (linkMap.containsKey(thePlayer)) {
                            ILinkageManager lm = LinkageManager.getInstance(thePlayer.worldObj);
                            EntityMinecart last = linkMap.remove(thePlayer);
                            if (lm.areLinked(cart, last)) {
                                lm.breakLink(cart, last);
                                used = true;
                                thePlayer.addChatMessage("Link Broken");
                            } else {
                                used = lm.createLink((EntityMinecart) entity, (EntityMinecart) last);
                                if (used) {
                                    thePlayer.addChatMessage("Link Created");
                                }
                            }
                            if (!used) {
                                thePlayer.addChatMessage("Link Failed");
                            }
                        } else {
                            linkMap.put(thePlayer, (EntityMinecart) entity);
                            thePlayer.addChatMessage("Link Started");
                        }
                    }
                    if (used) {
                        crowbar.onLink(thePlayer, stack, cart);
                    }
                } else if (crowbar.canBoost(thePlayer, stack, cart)) {
                    thePlayer.addExhaustion(1F);

                    if (cart instanceof EntityTunnelBore) {
                        // NOOP
                    } else if (cart instanceof IDirectionalCart) {
                        ((IDirectionalCart) cart).reverse();
                    } else {
                        if (cart.posX < thePlayer.posX) {
                            cart.motionX -= SMACK_VELOCITY;
                        } else {
                            cart.motionX += SMACK_VELOCITY;
                        }
                        if (cart.posZ < thePlayer.posZ) {
                            cart.motionZ -= SMACK_VELOCITY;
                        } else {
                            cart.motionZ += SMACK_VELOCITY;
                        }
                    }
                    crowbar.onBoost(thePlayer, stack, cart);
                }
            }
        }
        event.setCanceled(used);
    }
}
