package railcraft.common.items;

import cpw.mods.fml.common.registry.GameRegistry;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ItemCircuit extends ItemRailcraft {

    public static enum EnumCircuit {

        CONTROLLER, RECEIVER, SIGNAL;
        public static EnumCircuit[] VALUES = values();
    };
    private static Item item;

    public static Item getCircuitItem() {
        if (item != null) {
            return item;
        }

        String tag = "rc.part.circuit";
        int id = RailcraftConfig.getItemId(tag);

        item = new ItemCircuit(id);
        GameRegistry.registerItem(item, tag);

        for (EnumCircuit circuit : EnumCircuit.VALUES) {
            ItemStack stack = new ItemStack(item, 1, circuit.ordinal());
            RailcraftLanguage.getInstance().registerItemName(stack, item.getItemNameIS(stack));
            ItemRegistry.registerItem(item.getItemNameIS(stack), stack);
        }

        CraftingPlugin.addShapedRecipe(new ItemStack(item, 1, EnumCircuit.CONTROLLER.ordinal()), new Object[]{
                    " #S",
                    "BGR",
                    "SRL",
                    'L', new ItemStack(Item.dyePowder, 1, 4),
                    '#', Item.redstoneRepeater,
                    'G', Item.ingotGold,
                    'S', new ItemStack(Block.cloth, 1, 14),
                    'R', Item.redstone,
                    'B', Item.slimeBall,});
        CraftingPlugin.addShapedRecipe(new ItemStack(item, 1, EnumCircuit.RECEIVER.ordinal()), new Object[]{
                    " #S",
                    "BGR",
                    "SRL",
                    'L', new ItemStack(Item.dyePowder, 1, 4),
                    '#', Item.redstoneRepeater,
                    'G', Item.ingotGold,
                    'S', new ItemStack(Block.cloth, 1, 13),
                    'R', Item.redstone,
                    'B', Item.slimeBall,});
        CraftingPlugin.addShapedRecipe(new ItemStack(item, 1, EnumCircuit.SIGNAL.ordinal()), new Object[]{
                    " #S",
                    "BGR",
                    "SRL",
                    'L', new ItemStack(Item.dyePowder, 1, 4),
                    '#', Item.redstoneRepeater,
                    'G', Item.ingotGold,
                    'S', new ItemStack(Block.cloth, 1, 4),
                    'R', Item.redstone,
                    'B', Item.slimeBall,});

        return item;
    }

    public static ItemStack getControllerCircuit() {
        return getControllerCircuit(1);
    }

    public static ItemStack getControllerCircuit(int qty) {
        return new ItemStack(getCircuitItem(), qty, EnumCircuit.CONTROLLER.ordinal());
    }

    public static ItemStack getReceiverCircuit() {
        return getReceiverCircuit(1);
    }

    public static ItemStack getReceiverCircuit(int qty) {
        return new ItemStack(getCircuitItem(), qty, EnumCircuit.RECEIVER.ordinal());
    }

    public static ItemStack getSignalCircuit() {
        return getSignalCircuit(1);
    }

    public static ItemStack getSignalCircuit(int qty) {
        return new ItemStack(getCircuitItem(), qty, EnumCircuit.SIGNAL.ordinal());
    }

    public ItemCircuit(int id) {
        super(id);
        setHasSubtypes(true);
        setMaxDamage(0);
        setItemName("rc.part.circuit");
    }

    @Override
    public void getSubItems(int id, CreativeTabs tab, List list) {
        for (EnumCircuit circuit : EnumCircuit.VALUES) {
            list.add(new ItemStack(this, 1, circuit.ordinal()));
        }
    }

    @Override
    public int getIconFromDamage(int damage) {
        if (damage >= EnumCircuit.values().length) {
            return 0;
        }
        switch (EnumCircuit.VALUES[damage]) {
            case CONTROLLER:
                return 150;
            case RECEIVER:
                return 166;
            case SIGNAL:
                return 133;
            default:
                return 166;
        }
    }

    @Override
    public String getItemNameIS(ItemStack stack) {
        int damage = stack.getItemDamage();
        if (damage < 0 || damage >= EnumCircuit.VALUES.length) {
            return "";
        }
        switch (EnumCircuit.VALUES[damage]) {
            case CONTROLLER:
                return "rc.part.circuit.controller";
            case RECEIVER:
                return "rc.part.circuit.receiver";
            case SIGNAL:
                return "rc.part.circuit.signal";
            default:
                return "";
        }
    }
}
