package railcraft.common.items;

import java.lang.reflect.Field;
import java.util.List;
import java.util.logging.Level;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.Game;

public class ItemGear extends ItemRailcraft {

    private static ItemStack stoneGear;
    private static ItemStack ironGear;
    private static ItemGear itemGear;

    static enum EnumGear {

        GOLD_PLATE, IRON, STEEL;
    }

    public ItemGear(int i) {
        super(i);
        setHasSubtypes(true);
        setMaxDamage(0);
        setItemName("rc.part.gear");
    }

    @Override
    public void getSubItems(int id, CreativeTabs tab, List list) {
        list.add(getGoldPlateGear());
        if (stoneGear.itemID == Block.cobblestone.blockID) {
            list.add(getIronGear());
        }
        list.add(getSteelGear());
    }

    @Override
    public int getIconFromDamage(int damage) {
        if (damage < 0 || damage >= EnumGear.values().length) {
            return 0;
        }
        switch (EnumGear.values()[damage]) {
            case GOLD_PLATE:
                return 9;
            case IRON:
                return 165;
            case STEEL:
                return 149;
            default:
                return 9;
        }
    }

    private static void init() {
        if (itemGear == null) {
            String tag = "part.gear";
            int id = RailcraftConfig.getItemId(tag);

            itemGear = new ItemGear(id);

            for (EnumGear gear : EnumGear.values()) {
                ItemStack stack = new ItemStack(itemGear, 1, gear.ordinal());
                RailcraftLanguage.getInstance().registerItemName(stack, itemGear.getItemNameIS(stack));
                RailcraftPartItems.registerItem(itemGear.getItemNameIS(stack), stack);
            }
        }

        if (stoneGear == null) {
            try {
                Class buildcraftCore = Class.forName("buildcraft.BuildCraftCore");
                Field stoneGearItem = buildcraftCore.getField("stoneGearItem");
                stoneGear = new ItemStack((Item) stoneGearItem.get(null));

                Field ironGearItem = buildcraftCore.getField("ironGearItem");
                ironGear = new ItemStack((Item) ironGearItem.get(null));
            } catch (Exception ex) {
                Game.log(Level.INFO, "BuildcraftCore not found, using alternate gear recipes.");
                stoneGear = new ItemStack(Block.cobblestone);
                ironGear = new ItemStack(itemGear, 1, EnumGear.IRON.ordinal());

                CraftingPlugin.addShapedRecipe(ironGear, new Object[]{
                            " I ",
                            "ISI",
                            " I ",
                            'I', Item.ingotIron,
                            'S', stoneGear,});
            }


            CraftingPlugin.addShapedRecipe(new ItemStack(itemGear, 1, EnumGear.GOLD_PLATE.ordinal()),
                    " G ",
                    "GSG",
                    " G ",
                    'G', new ItemStack(Item.goldNugget),
                    'S', stoneGear);

            CraftingPlugin.addShapedOreRecipe(new ItemStack(itemGear, 1, EnumGear.STEEL.ordinal()),
                    " I ",
                    "ISI",
                    " I ",
                    'I', "ingotSteel",
                    'S', stoneGear);
        }
    }

    public static ItemStack getStoneGear() {
        init();
        return stoneGear.copy();
    }

    public static ItemStack getIronGear() {
        init();
        return ironGear.copy();
    }

    public static ItemStack getSteelGear() {
        init();
        return new ItemStack(itemGear, 1, EnumGear.STEEL.ordinal());
    }

    public static ItemStack getGoldPlateGear() {
        return getGoldPlateGear(1);
    }

    public static ItemStack getGoldPlateGear(int qty) {
        init();
        return new ItemStack(itemGear, qty, EnumGear.GOLD_PLATE.ordinal());
    }

    @Override
    public String getItemNameIS(ItemStack stack) {
        int damage = stack.getItemDamage();
        if (damage < 0 || damage >= EnumGear.values().length) {
            return "";
        }
        switch (EnumGear.values()[damage]) {
            case GOLD_PLATE:
                return "rc.part.gear.gold.plate";
            case IRON:
                return "rc.part.gear.iron";
            case STEEL:
                return "rc.part.gear.steel";
            default:
                return "";
        }
    }
}
