/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.items;

import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemArmor;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraftforge.common.IArmorTextureProvider;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.core.RailcraftConstants;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ItemGoggles extends ItemArmor implements IArmorTextureProvider {

    private static final String TEXTURE = RailcraftConstants.ARMOR_TEXTURE_FOLDER + "goggles.png";
    private static ItemGoggles item;

    public static enum Aura {

        NONE("gui.goggles.aura.none"),
        ANCHOR("gui.goggles.aura.anchor"),
        TUNING("gui.goggles.aura.tuning");
        public static final Aura[] AURAS = {NONE, ANCHOR, TUNING};
        private String name;

        private Aura(String name) {
            this.name = name;
        }

        @Override
        public String toString() {
            return RailcraftLanguage.translate(name);
        }
    }

    public static void registerItem() {
        if (item == null) {
            String tag = "rc.armor.goggles";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemGoggles(id);
                item.setItemName(tag);
                item.setIconIndex(10);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedOreRecipe(new ItemStack(item), true,
                        "GCG",
                        "I I",
                        "LLL",
                        'C', ItemCircuit.getReceiverCircuit(),
                        'I', "ingotSteel",
                        'L', Item.leather,
                        'G', new ItemStack(Block.thinGlass));

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getItem() {
        if (item == null) {
            return null;
        }
        return new ItemStack(item);
    }

    public static Aura getCurrentAura(ItemStack goggles) {
        Aura aura = Aura.NONE;
        if (goggles != null && goggles.getItem() instanceof ItemGoggles) {
            NBTTagCompound data = goggles.getTagCompound();
            if (data != null) {
                aura = Aura.AURAS[data.getByte("aura")];
            }
        }
        return aura;
    }

    public static void incrementAura(ItemStack goggles) {
        if (goggles != null && goggles.getItem() instanceof ItemGoggles) {
            NBTTagCompound data = goggles.getTagCompound();
            if (data == null) {
                data = new NBTTagCompound("tag");
                goggles.setTagCompound(data);
            }
            byte aura = data.getByte("aura");
            aura++;
            if (aura >= Aura.AURAS.length) {
                aura = 0;
            }
            data.setByte("aura", aura);
        }
    }

    public static boolean areEnabled() {
        return item != null;
    }

    public static ItemStack getGoggles(EntityPlayer player) {
        if(player == null){
            return null;
        }
        ItemStack helm = player.getCurrentItemOrArmor(4);
        if (helm != null && helm.getItem() instanceof ItemGoggles) {
            return helm;
        }
        return null;
    }

    public static boolean isPlayerWearing(EntityPlayer player) {
        ItemStack helm = player.getCurrentItemOrArmor(4);
        return helm != null && helm.getItem() instanceof ItemGoggles;
    }

    public ItemGoggles(int id) {
        super(id, ItemMaterials.GOGGLES, 0, 0);
        setTextureFile(RailcraftConstants.MAIN_TEXTURE_FILE);
    }

    @Override
    public ItemStack onItemRightClick(ItemStack stack, World world, EntityPlayer player) {
        incrementAura(stack);
        if (Game.isNotHost(world)) {
            Aura aura = getCurrentAura(stack);
            String mode = RailcraftLanguage.translate("gui.goggles.mode");
            player.addChatMessage(String.format(mode, "\u00A75" + aura));
        }
        return stack.copy();
    }

    @Override
    public String getArmorTextureFile(ItemStack itemstack) {
        return TEXTURE;
    }

    @Override
    public boolean getIsRepairable(ItemStack itemToRepair, ItemStack stack) {
        return MiscTools.isOreClass(stack, "ingotSteel");
    }

    @Override
    public void addInformation(ItemStack stack, EntityPlayer player, List list, boolean adv) {
        NBTTagCompound data = stack.getTagCompound();
        Aura aura = getCurrentAura(stack);
        String mode = RailcraftLanguage.translate("gui.goggles.mode");
        String tip = RailcraftLanguage.translate("gui.goggles.tip");

        list.add(String.format(mode, "\u00A75" + aura));
        list.add(tip);
    }
}
