package railcraft.common.items;

import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.ForgeSubscribe;
import net.minecraftforge.event.entity.player.EntityInteractEvent;
import railcraft.common.api.carts.CartTools;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.RailcraftTileEntity;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.forge.Creeper$Plugin;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ItemMagnifyingGlass extends ItemRailcraft implements IActivationBlockingItem {

    private static Item item;

    public static void register() {
        if (item == null) {
            String tag = "tool.magnifying.glass";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemMagnifyingGlass(id);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedRecipe(new ItemStack(item), new Object[]{
                            " G",
                            "S ",
                            'S', new ItemStack(Item.stick),
                            'G', new ItemStack(Block.thinGlass)
                        });

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
            Creeper$Plugin.fixCreepers();
        }
    }

    public static ItemStack getItem() {
        if (item == null) {
            return null;
        }
        return new ItemStack(item);
    }

    public ItemMagnifyingGlass(int id) {
        super(id);
        setMaxDamage(0);
        setMaxStackSize(1);
        setIconIndex(23);
        setItemName("rcMagGlass");
        setFull3D();

        MinecraftForge.EVENT_BUS.register(this);

        setCreativeTab(CreativeTabs.tabTools);
    }

    @ForgeSubscribe
    public void onEntityInteract(EntityInteractEvent event) {
        EntityPlayer thePlayer = event.entityPlayer;
        if (Game.isNotHost(thePlayer.worldObj)) {
            return;
        }

        Entity entity = event.target;
        thePlayer.swingItem();
        ItemStack stack = thePlayer.getCurrentEquippedItem();
        if (stack != null && stack.getItem() instanceof ItemMagnifyingGlass) {
            if (entity instanceof EntityMinecart) {
                EntityMinecart cart = (EntityMinecart) entity;
                String message = RailcraftLanguage.translate("gui.mag.glass.placedby");
                thePlayer.addChatMessage(String.format(message, Railcraft.getProxy().getItemDisplayName(cart.getCartItem()), CartTools.getCartOwner(cart)));
                event.setCanceled(true);
            }
        }
    }

    @Override
    public boolean onItemUseFirst(ItemStack stack, EntityPlayer player, World world, int x, int y, int z, int side, float hitX, float hitY, float hitZ) {
        if (Game.isNotHost(world)) {
            return false;
        }
        TileEntity t = world.getBlockTileEntity(x, y, z);
        if (t instanceof RailcraftTileEntity) {
            RailcraftTileEntity tile = (RailcraftTileEntity) t;
            String message = RailcraftLanguage.translate("gui.mag.glass.placedby");
            player.addChatMessage(String.format(message, tile.getInvName(), tile.getOwner()));
            return true;
        }
        return false;
    }
}
