package railcraft.common.items;

import cpw.mods.fml.common.registry.GameRegistry;
import java.util.List;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.oredict.OreDictionary;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forestry.ForestryPlugin;
import railcraft.common.plugins.forge.CraftingPlugin;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ItemNugget extends ItemRailcraft {

    public static enum EnumNugget {

        IRON, STEEL;
        public static EnumNugget[] VALUES = values();
    };
    private static Item item;

    private static Item getNuggetItem() {
        if (item != null) {
            return item;
        }

        String tag = "rc.nugget";
        int id = RailcraftConfig.getItemId(tag);

        item = new ItemNugget(id);
        GameRegistry.registerItem(item, tag);

        for (EnumNugget nugget : EnumNugget.VALUES) {
            ItemStack stack = new ItemStack(item, 1, nugget.ordinal());
            RailcraftLanguage.getInstance().registerItemName(stack, item.getItemNameIS(stack));
            ItemRegistry.registerItem(item.getItemNameIS(stack), stack);
            ForestryPlugin.addBackpackItem("miner", stack);
        }

        OreDictionary.registerOre("nuggetIron", new ItemStack(item, 1, EnumNugget.IRON.ordinal()));
        OreDictionary.registerOre("nuggetSteel", new ItemStack(item, 1, EnumNugget.STEEL.ordinal()));

        CraftingPlugin.addShapelessRecipe(new ItemStack(item, 9, EnumNugget.IRON.ordinal()), new ItemStack(Item.ingotIron));
        CraftingPlugin.addShapelessRecipe(new ItemStack(item, 9, EnumNugget.STEEL.ordinal()), RailcraftPartItems.getIngotSteel());

        CraftingPlugin.addShapedOreRecipe(new ItemStack(Item.ingotIron),
                "NNN",
                "NNN",
                "NNN",
                'N', "nuggetIron");

        CraftingPlugin.addShapedOreRecipe(RailcraftPartItems.getIngotSteel(),
                "NNN",
                "NNN",
                "NNN",
                'N', "nuggetSteel");

        return item;
    }

    public static ItemStack getNuggetIron() {
        return getNuggetIron(1);
    }

    public static ItemStack getNuggetIron(int qty) {
        return new ItemStack(getNuggetItem(), qty, EnumNugget.IRON.ordinal());
    }

    public static ItemStack getNuggetSteel() {
        return getNuggetSteel(1);
    }

    public static ItemStack getNuggetSteel(int qty) {
        return new ItemStack(getNuggetItem(), qty, EnumNugget.STEEL.ordinal());
    }

    public ItemNugget(int id) {
        super(id);
        setHasSubtypes(true);
        setMaxDamage(0);
        setItemName("rc.nugget");
    }

    @Override
    public void getSubItems(int id, CreativeTabs tab, List list) {
        for (EnumNugget dust : EnumNugget.VALUES) {
            list.add(new ItemStack(this, 1, dust.ordinal()));
        }
    }

    @Override
    public int getIconFromDamage(int damage) {
        if (damage >= EnumNugget.values().length) {
            return 0;
        }
        switch (EnumNugget.VALUES[damage]) {
            case IRON:
                return 164;
            case STEEL:
                return 148;
            default:
                return 0;
        }
    }

    @Override
    public String getItemNameIS(ItemStack stack) {
        int damage = stack.getItemDamage();
        if (damage < 0 || damage >= EnumNugget.VALUES.length) {
            return "";
        }
        switch (EnumNugget.VALUES[damage]) {
            case IRON:
                return "rc.nugget.iron";
            case STEEL:
                return "rc.nugget.steel";
            default:
                return "";
        }
    }
}
