package railcraft.common.items;

import java.util.List;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraftforge.oredict.ShapedOreRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.util.crafting.RollingMachineCraftingManager;

public class ItemPlate extends ItemRailcraft {

    enum EnumPlate {

        IRON, STEEL, TIN
    }

    public ItemPlate(int i) {
        super(i);
        setHasSubtypes(true);
        setMaxDamage(0);
        setItemName("rc.parrt.plate");

        defineRecipes();
    }

    @Override
    public void getSubItems(int id, CreativeTabs tab, List list) {
        for (int i = 0; i < 3; i++) {
            list.add(new ItemStack(this, 1, i));
        }
    }

    @Override
    public int getIconFromDamage(int damage) {
        if (damage < 0 || damage >= EnumPlate.values().length) {
            return 35;
        }
        switch (EnumPlate.values()[damage]) {
            case IRON:
                return 35;
            case STEEL:
                return 51;
            case TIN:
                return 67;
            default:
                return 35;
        }
    }

    private void defineRecipes() {
        // Iron Plate
        RailcraftCraftingManager.rollingMachine.addRecipe(new ItemStack(this, 4), new Object[]{
                    "II",
                    "II",
                    'I', Item.ingotIron});

        // Steel Plate
        IRecipe recipe = new ShapedOreRecipe(new ItemStack(this, 4, 1), new Object[]{
                    "II",
                    "II",
                    'I', "ingotSteel"});
        RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

        // Tin Plate
        recipe = new ShapedOreRecipe(new ItemStack(this, 4, 2), new Object[]{
                    "IT",
                    "TI",
                    'I', Item.ingotIron,
                    'T', "ingotTin"});
        RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

        RailcraftCraftingManager.blastFurnace.addRecipe(itemID, EnumPlate.IRON.ordinal(), 1280, RailcraftPartItems.getIngotSteel());
    }

    @Override
    public String getItemNameIS(ItemStack stack) {
        int damage = stack.getItemDamage();
        if (damage < 0 || damage >= EnumPlate.values().length) {
            return "";
        }
        switch (EnumPlate.values()[damage]) {
            case IRON:
                return "rc.part.plate.iron";
            case STEEL:
                return "rc.part.plate.steel";
            case TIN:
                return "rc.part.plate.tin";
            default:
                return "";
        }
    }
}
