package railcraft.common.items;

import java.util.List;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraftforge.oredict.ShapedOreRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.util.crafting.RollingMachineCraftingManager;

public class ItemRail extends ItemRailcraft {

    enum EnumRail {

        STANDARD, ADVANCED, WOOD, SPEED, REINFORCED
    };

    public ItemRail(int i) {
        super(i);
        setHasSubtypes(true);
        setMaxDamage(0);
        setItemName("rc.part.rail");
    }

    @Override
    public void getSubItems(int id, CreativeTabs tab, List list) {
        for (int i = 0; i < EnumRail.values().length; i++) {
            list.add(new ItemStack(this, 1, i));
        }
    }

    public void defineRecipes() {
        // Standard
         RailcraftCraftingManager.rollingMachine.addRecipe(new ItemStack(this, 8), new Object[]{
                    "I I",
                    "I I",
                    "I I",
                    'I', new ItemStack(Item.ingotIron),});

        IRecipe recipe = new ShapedOreRecipe(new ItemStack(this, 6), new Object[]{
                    "I I",
                    "I I",
                    "I I",
                    'I', "ingotBronze"});
        RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

        if (!RailcraftConfig.getRecipeConfig("ic2.refinedIron.useSteel")) {
            recipe = new ShapedOreRecipe(new ItemStack(this, 10), new Object[]{
                        "I I",
                        "I I",
                        "I I",
                        'I', "ingotRefinedIron"});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);
        }

        recipe = new ShapedOreRecipe(new ItemStack(this, 16), new Object[]{
                    "I I",
                    "I I",
                    "I I",
                    'I', "ingotSteel"});
        RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

        // Advanced
         RailcraftCraftingManager.rollingMachine.addRecipe(new ItemStack(this, 8, 1), new Object[]{
                    "IRG",
                    "IRG",
                    "IRG",
                    'I', new ItemStack(this, 1, 0),
                    'R', new ItemStack(Item.redstone),
                    'G', new ItemStack(Item.ingotGold),});

        // Wooden
         RailcraftCraftingManager.rollingMachine.addRecipe(new ItemStack(this, 8, 2), new Object[]{
                    "W I",
                    "W I",
                    "W I",
                    'I', new ItemStack(this, 1, 0),
                    'W', RailcraftPartItems.getTieWood()
                });

        // Speed
        recipe = new ShapedOreRecipe(new ItemStack(this, 6, 3), new Object[]{
                    "IBG",
                    "IBG",
                    "IBG",
                    'I', "ingotSteel",
                    'B', Item.blazePowder,
                    'G', Item.ingotGold,});
        RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

        // Reinforced
        recipe = new ShapedOreRecipe(new ItemStack(this, 6, 4), new Object[]{
                    "IDI",
                    "IDI",
                    "IDI",
                    'I', "ingotSteel",
                    'D', "dustObsidian",});
        RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);
    }

    @Override
    public int getIconFromDamage(int damage) {
        if (damage < 0 || damage >= EnumRail.values().length) {
            return 105;
        }
        switch (EnumRail.values()[damage]) {
            case STANDARD:
                return 105;
            case ADVANCED:
                return 121;
            case WOOD:
                return 151;
            case SPEED:
                return 137;
            case REINFORCED:
                return 102;
            default:
                return 105;
        }
    }

    @Override
    public String getItemNameIS(ItemStack stack) {
        int damage = stack.getItemDamage();
        if (damage < 0 || damage >= EnumRail.values().length) {
            return "";
        }
        switch (EnumRail.values()[stack.getItemDamage()]) {
            case STANDARD:
                return "rc.part.rail.standard";
            case ADVANCED:
                return "rc.part.rail.advanced";
            case WOOD:
                return "rc.part.rail.wood";
            case SPEED:
                return "rc.part.rail.speed";
            case REINFORCED:
                return "rc.part.rail.reinforced";
            default:
                return "";
        }
    }
}
