package railcraft.common.items;

import cpw.mods.fml.common.registry.GameRegistry;
import ic2.api.Ic2Recipes;
import net.minecraft.block.Block;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.blocks.ore.ItemDust;
import railcraft.common.blocks.ore.ItemDust.EnumDust;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.ic2.IC2Plugin;
import railcraft.common.util.crafting.RollingMachineCraftingManager;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.plugins.forestry.ForestryPlugin;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.forge.LootPlugin;
import railcraft.common.util.misc.EnumColor;
import railcraft.common.util.misc.MiscTools;

public class RailcraftPartItems {

    public static Item itemRail;
    public static Item itemTie;
    public static Item itemRailbed;
    private static Item itemRebar;
    private static Item itemPlate;
    private static Item itemIngotSteel;
    private static Item itemDust;
    private static Item itemNugget;
    private static Item itemTurbineBlade;
    private static Item itemTurbineDisk;
    private static Item itemTurbineRotor;
    public static Item itemSignalLamp;

    public static void registerItem(String tag, ItemStack stack) {
        tag = MiscTools.cleanTag(tag);
        ItemRegistry.registerItem(tag, stack);
    }

    private static Item getRailItem() {
        if (itemRail != null) {
            return itemRail;
        }

        String tag = "rc.part.rail";
        int id = RailcraftConfig.getItemId(tag);

        ItemRail item = new ItemRail(id);
        GameRegistry.registerItem(item, tag);
        itemRail = item;

        item.defineRecipes();

        for (int meta = 0; meta < 5; meta++) {
            ItemStack stack = new ItemStack(itemRail, 1, meta);
            RailcraftLanguage.getInstance().registerItemName(stack, itemRail.getItemNameIS(stack));
            registerItem(itemRail.getItemNameIS(stack), stack);
        }

        return itemRail;
    }

    public static ItemStack getRailStandard() {
        return getRailStandard(1);
    }

    public static ItemStack getRailStandard(int qty) {
        if (RailcraftConfig.useOldRecipes()) {
            return new ItemStack(Item.ingotIron, qty);
        }

        return new ItemStack(getRailItem(), qty, 0);
    }

    public static ItemStack getRailAdvanced() {
        return getRailAdvanced(1);
    }

    public static ItemStack getRailAdvanced(int qty) {
        if (RailcraftConfig.useOldRecipes()) {
            return new ItemStack(Item.ingotGold, qty);
        }

        return new ItemStack(getRailItem(), qty, 1);
    }

    public static ItemStack getRailWood() {
        return getRailWood(1);
    }

    public static ItemStack getRailWood(int qty) {
        if (RailcraftConfig.useOldRecipes()) {
            return new ItemStack(Block.woodSingleSlab, 1);
        }

        return new ItemStack(getRailItem(), qty, 2);
    }

    public static ItemStack getRailSpeed() {
        return getRailSpeed(1);
    }

    public static ItemStack getRailSpeed(int qty) {
        if (RailcraftConfig.useOldRecipes()) {
            return getIngotSteel(qty);
        }

        return new ItemStack(getRailItem(), qty, 3);
    }

    public static ItemStack getRailReinforced() {
        return getRailReinforced(1);
    }

    public static ItemStack getRailReinforced(int qty) {
        if (RailcraftConfig.useOldRecipes()) {
            return getIngotSteel(qty);
        }

        return new ItemStack(getRailItem(), qty, 4);
    }

    public static ItemStack getRebar() {
        return getRebar(1);
    }

    public static ItemStack getRebar(int qty) {
        Item item = itemRebar;
        if (item != null) {
            return new ItemStack(item, qty);
        }

        String tag = "rc.part.rebar";
        int id = RailcraftConfig.getItemId(tag);

        if (id > 0) {
            item = new ItemRailcraft(id).setIconIndex(153).setItemName(tag);
            GameRegistry.registerItem(item, tag);
            ItemStack stack = new ItemStack(item);
            RailcraftLanguage.getInstance().registerItemName(stack, tag);
            registerItem(tag, stack);

            RailcraftCraftingManager.rollingMachine.addRecipe(new ItemStack(item, 4), new Object[]{
                        "  I",
                        " I ",
                        "I  ",
                        'I', new ItemStack(Item.ingotIron),});

            IRecipe recipe = new ShapedOreRecipe(new ItemStack(item, 4), true, new Object[]{
                        "  I",
                        " I ",
                        "I  ",
                        'I', "ingotBronze",});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

            recipe = new ShapedOreRecipe(new ItemStack(item, 6), true, new Object[]{
                        "  I",
                        " I ",
                        "I  ",
                        'I', "ingotRefinedIron",});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

            recipe = new ShapedOreRecipe(new ItemStack(item, 8), true, new Object[]{
                        "  I",
                        " I ",
                        "I  ",
                        'I', "ingotSteel",});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);


            itemRebar = item;
        } else {
            return new ItemStack(Item.ingotIron, qty);
        }

        return new ItemStack(item, qty);
    }

    private static Item getPlateItem() {
        if (itemPlate != null) {
            return itemPlate;
        }

        String tag = "rc.part.plate";
        int id = RailcraftConfig.getItemId(tag);

        itemPlate = new ItemPlate(id);
        GameRegistry.registerItem(itemPlate, tag);

        for (int meta = 0; meta < 3; meta++) {
            ItemStack stack = new ItemStack(itemPlate, 1, meta);
            RailcraftLanguage.getInstance().registerItemName(stack, itemPlate.getItemNameIS(stack));
            registerItem(itemPlate.getItemNameIS(stack), stack);
        }

        return itemPlate;
    }

    public static ItemStack getPlateIron() {
        return getPlateIron(1);
    }

    public static ItemStack getPlateIron(int qty) {
        return new ItemStack(getPlateItem(), qty, 0);
    }

    public static ItemStack getPlateSteel() {
        return getPlateSteel(1);
    }

    public static ItemStack getPlateSteel(int qty) {

        return new ItemStack(getPlateItem(), qty, 1);
    }

    public static ItemStack getPlateTin() {
        return getPlateTin(1);
    }

    public static ItemStack getPlateTin(int qty) {

        return new ItemStack(getPlateItem(), qty, 2);
    }

    private static Item getTieItem() {
        if (itemTie != null) {
            return itemTie;
        }

        String tag = "rc.part.tie";
        int id = RailcraftConfig.getItemId(tag);

        itemTie = new ItemTie(id);
        GameRegistry.registerItem(itemTie, tag);

        for (int meta = 0; meta < 2; meta++) {
            ItemStack stack = new ItemStack(itemTie, 1, meta);
            RailcraftLanguage.getInstance().registerItemName(stack, itemTie.getItemNameIS(stack));
            registerItem(itemTie.getItemNameIS(stack), stack);
        }

        ItemStack tieWood = getTieWood();
        CraftingPlugin.addShapedOreRecipe(tieWood, new Object[]{
                    " O ",
                    "###",
                    'O', LiquidItems.getCreosoteOilBucket(),
                    '#', "slabWood"
                });
        CraftingPlugin.addShapedOreRecipe(tieWood, new Object[]{
                    " O ",
                    "###",
                    'O', LiquidItems.getCreosoteOilBottle(),
                    '#', "slabWood"
                });
        CraftingPlugin.addShapedOreRecipe(tieWood, new Object[]{
                    " O ",
                    "###",
                    'O', LiquidItems.getCreosoteOilCan(),
                    '#', "slabWood"
                });
        CraftingPlugin.addShapedOreRecipe(tieWood, new Object[]{
                    " O ",
                    "###",
                    'O', LiquidItems.getCreosoteOilWax(),
                    '#', "slabWood"
                });
        CraftingPlugin.addShapedOreRecipe(tieWood, new Object[]{
                    " O ",
                    "###",
                    'O', LiquidItems.getCreosoteOilRefactory(),
                    '#', "slabWood"
                });
        CraftingPlugin.addShapedOreRecipe(tieWood, new Object[]{
                    " O ",
                    "###",
                    'O', LiquidItems.getCreosoteOilCell(),
                    '#', "slabWood"
                });

        LootPlugin.addLootMineshaft(tieWood, 4, 16, "tie.wood");

        ItemStack tieStone = getTieStone();
        CraftingPlugin.addShapedRecipe(tieStone, new Object[]{
                    " O ",
                    "###",
                    'O', getRebar(),
                    '#', new ItemStack(Block.stoneSingleSlab, 1, 0)
                });

        return itemTie;
    }

    public static ItemStack getTieWood() {
        return getTieWood(1);
    }

    public static ItemStack getTieWood(int qty) {

        return new ItemStack(getTieItem(), qty, 0);
    }

    public static ItemStack getTieStone() {
        return getTieStone(1);
    }

    public static ItemStack getTieStone(int qty) {

        return new ItemStack(getTieItem(), qty, 1);
    }

    private static Item getRailbedItem() {
        if (itemRailbed != null) {
            return itemRailbed;
        }

        String tag = "rc.part.railbed";
        int id = RailcraftConfig.getItemId(tag);

        itemRailbed = new ItemRailbed(id);
        GameRegistry.registerItem(itemRailbed, tag);

        for (int meta = 0; meta < 2; meta++) {
            ItemStack stack = new ItemStack(itemRailbed, 1, meta);
            RailcraftLanguage.getInstance().registerItemName(stack, itemRailbed.getItemNameIS(stack));
            registerItem(itemRailbed.getItemNameIS(stack), stack);
        }

        ItemStack output = new ItemStack(itemRailbed, 1, 0);
        CraftingManager.getInstance().addShapelessRecipe(output, new Object[]{
                    getTieWood(),
                    getTieWood(),
                    getTieWood(),
                    getTieWood()
                });

        output = new ItemStack(itemRailbed, 1, 1);
        CraftingManager.getInstance().addShapelessRecipe(output, new Object[]{
                    getTieStone(),
                    getTieStone(),
                    getTieStone(),
                    getTieStone()
                });

        return itemRailbed;
    }

    public static ItemStack getRailbedWood() {
        return getRailbedWood(1);
    }

    public static ItemStack getRailbedWood(int qty) {
        if (RailcraftConfig.useOldRecipes()) {
            return new ItemStack(Item.stick, qty);
        }
        return new ItemStack(getRailbedItem(), qty, 0);
    }

    public static ItemStack getRailbedStone() {
        return getRailbedStone(1);
    }

    public static ItemStack getRailbedStone(int qty) {
        if (RailcraftConfig.useOldRecipes()) {
            return new ItemStack(Block.stoneSingleSlab, 1, 0);
        }
        return new ItemStack(getRailbedItem(), qty, 1);
    }

    public static ItemStack getIngotSteel() {
        return getIngotSteel(1);
    }

    public static ItemStack getIngotSteel(int qty) {
        Item item = itemIngotSteel;
        if (item != null) {
            return new ItemStack(item, qty);
        }

        String tag = "rc.part.ingot.steel";
        int id = RailcraftConfig.getItemId(tag);

        if (id <= 0) {
            return new ItemStack(Item.ingotIron, qty);
        }

        item = new ItemRailcraft(id).setSmeltingExperiance(1).setIconIndex(181).setItemName(tag);
        GameRegistry.registerItem(item, tag);
        ItemStack stack = new ItemStack(item);
        RailcraftLanguage.getInstance().registerItemName(stack, tag);

        registerItem(tag, stack);

        itemIngotSteel = item;

        OreDictionary.registerOre("ingotSteel", new ItemStack(item));

        LootPlugin.addLootTool(new ItemStack(item), 5, 9, "steel.ingot");

        return new ItemStack(item, qty);
    }

    private static Item getDustItem() {
        if (itemDust != null) {
            return itemDust;
        }

        String tag = "rc.dust";
        int id = RailcraftConfig.getItemId(tag);

        itemDust = new ItemDust(id);
        GameRegistry.registerItem(itemDust, tag);

        for (EnumDust dust : EnumDust.values()) {
            ItemStack stack = new ItemStack(itemDust, 1, dust.ordinal());
            RailcraftLanguage.getInstance().registerItemName(stack, itemDust.getItemNameIS(stack));
            registerItem(itemDust.getItemNameIS(stack), stack);
            ForestryPlugin.addBackpackItem("miner", stack);
        }

        OreDictionary.registerOre("dustObsidian", new ItemStack(itemDust, 1, EnumDust.OBSIDIAN.ordinal()));
        OreDictionary.registerOre("dustSulfur", new ItemStack(itemDust, 1, EnumDust.SULFUR.ordinal()));
        OreDictionary.registerOre("dustSaltpeter", new ItemStack(itemDust, 1, EnumDust.SALTPETER.ordinal()));
        OreDictionary.registerOre("dustCharcoal", new ItemStack(itemDust, 1, EnumDust.CHARCOAL.ordinal()));

        if (IC2Plugin.isIC2Installed() && RailcraftConfig.getRecipeConfig("ic2.macerator.charcoal")) {
            Ic2Recipes.addMaceratorRecipe(new ItemStack(Item.coal, 1, 1), new ItemStack(itemDust, 1, EnumDust.CHARCOAL.ordinal()));
        }

        return itemDust;
    }

    public static ItemStack getDustObsidian() {
        return getDustObsidian(1);
    }

    public static ItemStack getDustObsidian(int qty) {
        return new ItemStack(getDustItem(), qty, EnumDust.OBSIDIAN.ordinal());
    }

    public static ItemStack getDustSulfur() {
        return getDustSulfur(1);
    }

    public static ItemStack getDustSulfur(int qty) {
        return new ItemStack(getDustItem(), qty, EnumDust.SULFUR.ordinal());
    }

    public static ItemStack getDustSaltpeter() {
        return getDustSaltpeter(1);
    }

    public static ItemStack getDustSaltpeter(int qty) {
        return new ItemStack(getDustItem(), qty, EnumDust.SALTPETER.ordinal());
    }

    public static ItemStack getDustCharcoal() {
        return getDustCharcoal(1);
    }

    public static ItemStack getDustCharcoal(int qty) {
        return new ItemStack(getDustItem(), qty, EnumDust.CHARCOAL.ordinal());
    }

    public static ItemStack getTurbineBlade() {
        return getTurbineBlade(1);
    }

    public static ItemStack getTurbineBlade(int qty) {
        Item item = itemTurbineBlade;
        if (item != null) {
            return new ItemStack(item, qty);
        }

        String tag = "rc.part.turbine.blade";
        int id = RailcraftConfig.getItemId(tag);

        if (id <= 0) {
            return getIngotSteel(qty);
        }

        item = new ItemRailcraft(id).setIconIndex(39).setItemName(tag);
        GameRegistry.registerItem(item, tag);
        ItemStack stack = new ItemStack(item);
        RailcraftLanguage.getInstance().registerItemName(stack, tag);

        registerItem(tag, stack);

        CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                    "I",
                    "I",
                    "I",
                    'I', "ingotSteel"
                });

        itemTurbineBlade = item;

        return new ItemStack(item, qty);
    }

    public static ItemStack getTurbineDisk() {
        return getTurbineDisk(1);
    }

    public static ItemStack getTurbineDisk(int qty) {
        Item item = itemTurbineDisk;
        if (item != null) {
            return new ItemStack(item, qty);
        }

        String tag = "rc.part.turbine.disk";
        int id = RailcraftConfig.getItemId(tag);

        if (id <= 0) {
            return null;
        }

        item = new ItemRailcraft(id).setIconIndex(40).setItemName(tag);
        GameRegistry.registerItem(item, tag);
        ItemStack stack = new ItemStack(item);
        RailcraftLanguage.getInstance().registerItemName(stack, tag);

        registerItem(tag, stack);

        ItemStack blade = getTurbineBlade();
        if (blade != null) {
            CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                        "BBB",
                        "BAB",
                        "BBB",
                        'A', "blockSteel",
                        'B', blade
                    });
        }

        itemTurbineDisk = item;

        return new ItemStack(item, qty);
    }

    public static ItemStack getTurbineRotor() {
        return getTurbineRotor(1);
    }

    public static ItemStack getTurbineRotor(int qty) {
        Item item = itemTurbineRotor;
        if (item != null) {
            return new ItemStack(item, qty);
        }

        String tag = "rc.part.turbine.rotor";
        int id = RailcraftConfig.getItemId(tag);

        if (id <= 0) {
            return null;
        }

        item = new ItemRailcraft(id).setIconIndex(24).setItemName(tag);
        item.setMaxDamage(30000).setMaxStackSize(1);
        GameRegistry.registerItem(item, tag);
        ItemStack stack = new ItemStack(item);
        RailcraftLanguage.getInstance().registerItemName(stack, tag);

        registerItem(tag, stack);

        ItemStack disk = getTurbineDisk();
        if (disk != null) {
            CraftingPlugin.addShapedRecipe(stack,
                    "DDD",
                    'D', disk);
        }

        itemTurbineRotor = item;

        return new ItemStack(item, qty);
    }

    public static ItemStack getSignalLamp() {
        return getSignalLamp(1);
    }

    public static ItemStack getSignalLamp(int qty) {
        Item item = itemSignalLamp;
        if (item != null) {
            return new ItemStack(item, qty);
        }

        String tag = "rc.part.signal.lamp";
        int id = RailcraftConfig.getItemId(tag);
        if (id <= 0) {
            return new ItemStack(Block.redstoneLampIdle, qty);
        }

        item = itemSignalLamp = new ItemRailcraft(id).setIconIndex(134).setItemName(tag);
        GameRegistry.registerItem(item, tag);
        ItemStack stackSignalLamp = new ItemStack(item);
        RailcraftLanguage.getInstance().registerItemName(stackSignalLamp, tag);
        CraftingPlugin.addShapedOreRecipe(stackSignalLamp, new Object[]{
                    "PG ",
                    "PYT",
                    "PRS",
                    'G', EnumColor.LIME.getDye(),
                    'Y', EnumColor.YELLOW.getDye(),
                    'R', EnumColor.RED.getDye(),
                    'S', Item.redstone,
                    'T', Item.lightStoneDust,
                    'P', Block.thinGlass,});
        ItemRegistry.registerItem(tag, stackSignalLamp);

        return new ItemStack(item, qty);
    }
}
