package railcraft.common.items;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.forge.LootPlugin;

public class RailcraftToolItems {

    private static Item itemCrowbar;
    private static Item itemSteelSword;
    private static Item itemSteelShovel;
    private static Item itemSteelPickaxe;
    private static Item itemSteelAxe;
    private static Item itemSteelHoe;
    private static Item itemSteelHelmet;
    private static Item itemSteelArmor;
    private static Item itemSteelLegs;
    private static Item itemSteelBoots;
    private static Item itemCoalCoke;
    private static Item itemOveralls;

    public static void initializeToolsArmor() {
        registerSteelSword();
        registerSteelShovel();
        registerSteelPickaxe();
        registerSteelAxe();
        registerSteelHoe();

        registerSteelHelmet();
        registerSteelArmor();
        registerSteelLegs();
        registerSteelBoots();

        registerOveralls();
    }

    public static void registerCrowbar() {
        if (itemCrowbar == null) {
            String tag = "rc.tool.crowbar";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                Item item = itemCrowbar = new ItemCrowbar(id);
                GameRegistry.registerItem(item, tag);
                MinecraftForge.setToolClass(item, "crowbar", 0);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedOreRecipe(new ItemStack(item), new Object[]{
                            " RI",
                            "RIR",
                            "IR ",
                            'I', Item.ingotIron,
                            'R', "dyeRed"
                        });

                RailcraftCraftingManager.blastFurnace.addRecipe(item.itemID, 0, 3840, RailcraftPartItems.getIngotSteel(3));

                LootPlugin.addLootTool(new ItemStack(item), 1, 1, tag);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getCrowbar() {
        if (itemCrowbar == null) {
            return null;
        }
        return new ItemStack(itemCrowbar);
    }

    private static void registerOveralls() {
        Item item = itemOveralls;
        if (item == null) {
            String tag = "rc.armor.overalls";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemOveralls = new ItemOveralls(id);
                item.setItemName(tag);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedRecipe(new ItemStack(item),
                        "III",
                        "I I",
                        "I I",
                        'I', new ItemStack(Block.cloth, 1, 3));

                ItemRegistry.registerItem(tag, new ItemStack(item));

//                LootPlugin.addLootWarrior(new ItemStack(item), 1, 1, tag);
            }
        }
    }

    public static ItemStack getOveralls() {
        if (itemOveralls == null) {
            return null;
        }
        return new ItemStack(itemOveralls);
    }

    private static void registerSteelSword() {
        Item item = itemSteelSword;
        if (item == null) {
            String tag = "rc.tool.steel.sword";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelSword = new ItemSteelSword(id);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedOreRecipe(new ItemStack(item), false,
                        " I ",
                        " I ",
                        " S ",
                        'I', "ingotSteel",
                        'S', new ItemStack(Item.stick));

                LootPlugin.addLootWarrior(new ItemStack(item), 1, 1, tag);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getSteelSword() {
        if (itemSteelSword == null) {
            return new ItemStack(Item.swordSteel);
        }
        return new ItemStack(itemSteelSword);
    }

    private static void registerSteelShovel() {
        Item item = itemSteelShovel;
        if (item == null) {
            String tag = "rc.tool.steel.shovel";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelShovel = new ItemSteelShovel(id);
                GameRegistry.registerItem(item, tag);
                MinecraftForge.setToolClass(item, "shovel", 2);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                IRecipe recipe = new ShapedOreRecipe(new ItemStack(item), false, new Object[]{
                            " I ",
                            " S ",
                            " S ",
                            'I', "ingotSteel",
                            'S', new ItemStack(Item.stick)
                        });
                CraftingManager.getInstance().getRecipeList().add(recipe);

                LootPlugin.addLootTool(new ItemStack(item), 1, 1, tag);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getSteelShovel() {
        if (itemSteelShovel == null) {
            return new ItemStack(Item.shovelSteel);
        }
        return new ItemStack(itemSteelShovel);
    }

    private static void registerSteelPickaxe() {
        Item item = itemSteelPickaxe;
        if (item == null) {
            String tag = "rc.tool.steel.pickaxe";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelPickaxe = new ItemSteelPickaxe(id);
                GameRegistry.registerItem(item, tag);
                MinecraftForge.setToolClass(item, "pickaxe", 2);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                IRecipe recipe = new ShapedOreRecipe(new ItemStack(item), false, new Object[]{
                            "III",
                            " S ",
                            " S ",
                            'I', "ingotSteel",
                            'S', new ItemStack(Item.stick)
                        });
                CraftingManager.getInstance().getRecipeList().add(recipe);

                LootPlugin.addLootTool(new ItemStack(item), 1, 1, tag);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getSteelPickaxe() {
        if (itemSteelPickaxe == null) {
            return new ItemStack(Item.pickaxeSteel);
        }
        return new ItemStack(itemSteelPickaxe);
    }

    private static void registerSteelAxe() {
        Item item = itemSteelAxe;
        if (item == null) {
            String tag = "rc.tool.steel.axe";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelAxe = new ItemSteelAxe(id);
                GameRegistry.registerItem(item, tag);
                MinecraftForge.setToolClass(item, "axe", 2);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                IRecipe recipe = new ShapedOreRecipe(new ItemStack(item), true, new Object[]{
                            "II ",
                            "IS ",
                            " S ",
                            'I', "ingotSteel",
                            'S', new ItemStack(Item.stick)
                        });
                CraftingManager.getInstance().getRecipeList().add(recipe);

                LootPlugin.addLootTool(new ItemStack(item), 1, 1, tag);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getSteelAxe() {
        if (itemSteelAxe == null) {
            return new ItemStack(Item.axeSteel);
        }
        return new ItemStack(itemSteelAxe);
    }

    private static void registerSteelHoe() {
        Item item = itemSteelHoe;
        if (item == null) {
            String tag = "rc.tool.steel.hoe";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelHoe = new ItemSteelHoe(id);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                IRecipe recipe = new ShapedOreRecipe(new ItemStack(item), true, new Object[]{
                            "II ",
                            " S ",
                            " S ",
                            'I', "ingotSteel",
                            'S', new ItemStack(Item.stick)
                        });
                CraftingManager.getInstance().getRecipeList().add(recipe);

                ItemRegistry.registerItem(tag, new ItemStack(item));
            }
        }
    }

    public static ItemStack getSteelHoe() {
        if (itemSteelHoe == null) {
            return new ItemStack(Item.hoeSteel);
        }
        return new ItemStack(itemSteelHoe);
    }

    private static void registerSteelHelmet() {
        Item item = itemSteelHelmet;
        if (item == null) {
            String tag = "rc.armor.steel.helmet";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelHelmet = new ItemSteelArmor(id, 0);
                item.setItemName(tag);
                item.setIconCoord(0, 5);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedOreRecipe(new ItemStack(item), true, new Object[]{
                            "III",
                            "I I",
                            'I', "ingotSteel",});

                ItemRegistry.registerItem(tag, new ItemStack(item));

                LootPlugin.addLootWarrior(new ItemStack(item), 1, 1, tag);
            }
        }
    }

    public static ItemStack getSteelHelm() {
        if (itemSteelHelmet == null) {
            return null;
        }
        return new ItemStack(itemSteelHelmet);
    }

    private static void registerSteelArmor() {
        Item item = itemSteelArmor;
        if (item == null) {
            String tag = "rc.armor.steel.plate";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelArmor = new ItemSteelArmor(id, 1);
                item.setItemName(tag);
                item.setIconCoord(0, 6);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedOreRecipe(new ItemStack(item), true, new Object[]{
                            "I I",
                            "III",
                            "III",
                            'I', "ingotSteel",});

                ItemRegistry.registerItem(tag, new ItemStack(item));

                LootPlugin.addLootWarrior(new ItemStack(item), 1, 1, tag);
            }
        }
    }

    public static ItemStack getSteelArmor() {
        if (itemSteelArmor == null) {
            return null;
        }
        return new ItemStack(itemSteelArmor);
    }

    private static void registerSteelLegs() {
        Item item = itemSteelLegs;
        if (item == null) {
            String tag = "rc.armor.steel.legs";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelLegs = new ItemSteelArmor(id, 2);
                item.setItemName(tag);
                item.setIconCoord(0, 7);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedOreRecipe(new ItemStack(item), true, new Object[]{
                            "III",
                            "I I",
                            "I I",
                            'I', "ingotSteel",});

                ItemRegistry.registerItem(tag, new ItemStack(item));

                LootPlugin.addLootWarrior(new ItemStack(item), 1, 1, tag);
            }
        }
    }

    public static ItemStack getSteelLegs() {
        if (itemSteelLegs == null) {
            return null;
        }
        return new ItemStack(itemSteelLegs);
    }

    private static void registerSteelBoots() {
        Item item = itemSteelBoots;
        if (item == null) {
            String tag = "rc.armor.steel.boots";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = itemSteelBoots = new ItemSteelArmor(id, 3);
                item.setItemName(tag);
                item.setIconCoord(0, 8);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                CraftingPlugin.addShapedOreRecipe(new ItemStack(item), true, new Object[]{
                            "I I",
                            "I I",
                            'I', "ingotSteel",});

                ItemRegistry.registerItem(tag, new ItemStack(item));

                LootPlugin.addLootWarrior(new ItemStack(item), 1, 1, tag);
            }
        }
    }

    public static ItemStack getSteelBoots() {
        if (itemSteelBoots == null) {
            return null;
        }
        return new ItemStack(itemSteelBoots);
    }

    public static void registerCoalCoke() {
        if (itemCoalCoke == null) {
            String tag = "rc.fuel.coke";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                Item item = itemCoalCoke = new ItemRailcraft(id).setIconIndex(70).setItemName(tag);
                GameRegistry.registerItem(itemCoalCoke, tag);
                RailcraftLanguage.getInstance().registerItemName(itemCoalCoke, tag);

                LootPlugin.addLootTool(new ItemStack(item), 4, 16, tag);

                ItemRegistry.registerItem("fuel.coke", new ItemStack(itemCoalCoke));
                OreDictionary.registerOre("fuelCoke", new ItemStack(itemCoalCoke));
            }
        }
    }

    public static ItemStack getCoalCoke() {
        if (itemCoalCoke == null) {
            return null;
        }
        return new ItemStack(itemCoalCoke);
    }
}
