package railcraft.common.liquids;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.liquids.LiquidDictionary;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.api.fuel.FuelManager;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CreativePlugin;
import railcraft.common.plugins.forge.LootPlugin;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class LiquidItems {

    private static LiquidStack liquidCreosoteOil;
    private static LiquidStack liquidSteam;
    private static Item itemCreosoteOil;
    private static Item itemSteam;
    private static Item itemCreosoteOilBucket;
    private static Item itemCreosoteOilBottle;
    private static Item itemSteamBottle;
    private static ItemLiquidContainer itemCreosoteOilCell;
    private static ItemLiquidContainer itemCreosoteOilCan;
    private static ItemLiquidContainer itemCreosoteOilWax;
    private static ItemLiquidContainer itemCreosoteOilRefactory;

    public static void initialize() {
        initCreosoteOil();
        initSteam();
    }

    public static ItemStack getCreosoteOil() {
        initCreosoteOil();
        return new ItemStack(itemCreosoteOil);
    }

    private static void initCreosoteOil() {
        if (itemCreosoteOil == null) {
            String tag = "rc.liquid.creosote.liquid";
            int id = RailcraftConfig.getItemId(tag);

            Item item = new ItemLiquid(id).setRarity(1).setIconIndex(0).setItemName(tag);
            GameRegistry.registerItem(item, tag);
            RailcraftLanguage.getInstance().registerItemName(item, tag);

            liquidCreosoteOil = LiquidDictionary.getOrCreateLiquid("Creosote Oil", new LiquidStack(item.itemID, 0));

            ItemRegistry.registerItem(tag, new ItemStack(item));

            LiquidStack liquid = liquidCreosoteOil.copy();
            liquid.amount = LiquidManager.BUCKET_VOLUME;
            FuelManager.addBoilerFuel(liquid, 3200);

            itemCreosoteOil = item;
        }
    }

    public static LiquidStack getCreosoteOilLiquid(int qty) {
        initCreosoteOil();
        LiquidStack liquid = liquidCreosoteOil.copy();
        liquid.amount = qty;
        return liquid;
    }

    public static ItemStack getCreosoteOilBucket() {
        Item item = itemCreosoteOilBucket;
        if (item == null) {
            String tag = "rc.liquid.creosote.bucket";
            int id = RailcraftConfig.getItemId(tag);

            item = new ItemLiquidContainer(id).setIconIndex(1).setItemName(tag).setMaxStackSize(1).setContainerItem(Item.bucketEmpty).setCreativeTab(CreativePlugin.TAB);
            GameRegistry.registerItem(item, tag);
            RailcraftLanguage.getInstance().registerItemName(item, tag);

            LiquidManager.getInstance().registerBucket(getCreosoteOilLiquid(LiquidManager.BUCKET_VOLUME), new ItemStack(item));

            ItemRegistry.registerItem(tag, new ItemStack(item));

            itemCreosoteOilBucket = item;
        }
        return new ItemStack(item);
    }

    public static ItemStack getCreosoteOilBottle() {
        return getCreosoteOilBottle(1);
    }

    public static ItemStack getCreosoteOilBottle(int qty) {
        Item item = itemCreosoteOilBottle;
        if (item == null) {
            String tag = "rc.liquid.creosote.bottle";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemLiquidContainer(id).setIconIndex(2).setItemName(tag).setCreativeTab(CreativePlugin.TAB);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                LootPlugin.addLootMineshaft(new ItemStack(item), 4, 16, "creosote.bottle");

                LiquidManager.getInstance().registerBottle(getCreosoteOilLiquid(LiquidManager.BUCKET_VOLUME), new ItemStack(item));

                ItemRegistry.registerItem(tag, new ItemStack(item));

                itemCreosoteOilBottle = item;
            } else {
                return null;
            }
        }
        return new ItemStack(item, qty);
    }

    public static ItemStack getCreosoteOilCan() {
        return getCreosoteOilCan(1);
    }

    public static ItemStack getCreosoteOilCan(int qty) {
        ItemLiquidContainer item = itemCreosoteOilCan;
        if (item == null) {
            String tag = "rc.liquid.creosote.can";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemLiquidContainer(id);
                item.setIconIndex(3);
                item.setItemName(tag);
                GameRegistry.registerItem(item, tag);

                RailcraftLanguage.getInstance().registerItemName(item, tag);

                boolean forestry = LiquidManager.getInstance().registerCan(getCreosoteOilLiquid(LiquidManager.BUCKET_VOLUME), new ItemStack(item));
                if (forestry) {
                    item.setCreativeTab(CreativePlugin.TAB);
                }

                ItemRegistry.registerItem(tag, new ItemStack(item));

                itemCreosoteOilCan = item;
            } else {
                return null;
            }
        }
        return new ItemStack(item, qty);
    }

    public static ItemStack getCreosoteOilWax() {
        return getCreosoteOilWax(1);
    }

    public static ItemStack getCreosoteOilWax(int qty) {
        ItemLiquidContainer item = itemCreosoteOilWax;
        if (item == null) {
            String tag = "rc.liquid.creosote.wax";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemLiquidContainer(id);
                item.setIconIndex(4);
                item.setItemName(tag);
                GameRegistry.registerItem(item, tag);

                RailcraftLanguage.getInstance().registerItemName(item, tag);

                boolean forestry = LiquidManager.getInstance().registerWax(getCreosoteOilLiquid(LiquidManager.BUCKET_VOLUME), new ItemStack(item));
                if (forestry) {
                    item.setCreativeTab(CreativePlugin.TAB);
                }

                ItemRegistry.registerItem(tag, new ItemStack(item));

                itemCreosoteOilWax = item;
            } else {
                return null;
            }
        }
        return new ItemStack(item, qty);
    }

    public static ItemStack getCreosoteOilRefactory() {
        return getCreosoteOilRefactory(1);
    }

    public static ItemStack getCreosoteOilRefactory(int qty) {
        ItemLiquidContainer item = itemCreosoteOilRefactory;
        if (item == null) {
            String tag = "rc.liquid.creosote.refactory";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemLiquidContainer(id);
                item.setIconIndex(5);
                item.setItemName(tag);
                GameRegistry.registerItem(item, tag);

                RailcraftLanguage.getInstance().registerItemName(item, tag);

                boolean forestry = LiquidManager.getInstance().registerRefactory(getCreosoteOilLiquid(LiquidManager.BUCKET_VOLUME), new ItemStack(item));
                if (forestry) {
                    item.setCreativeTab(CreativePlugin.TAB);
                }

                ItemRegistry.registerItem(tag, new ItemStack(item));

                itemCreosoteOilRefactory = item;
            } else {
                return null;
            }
        }
        return new ItemStack(item, qty);
    }

    public static ItemStack getCreosoteOilCell() {
        return getCreosoteOilCell(1);
    }

    public static ItemStack getCreosoteOilCell(int qty) {
        ItemLiquidContainer item = itemCreosoteOilCell;
        if (item == null) {
            String tag = "rc.liquid.creosote.cell";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                item = new ItemLiquidContainer(id);
                item.setIconIndex(6);
                item.setItemName(tag);
                GameRegistry.registerItem(item, tag);

                RailcraftLanguage.getInstance().registerItemName(item, tag);

                boolean ic2 = LiquidManager.getInstance().registerCell(getCreosoteOilLiquid(LiquidManager.BUCKET_VOLUME), new ItemStack(item));
                if (ic2) {
                    item.setCreativeTab(CreativePlugin.TAB);
                }

                ItemRegistry.registerItem(tag, new ItemStack(item));

                itemCreosoteOilCell = item;
            } else {
                return null;
            }
        }
        return new ItemStack(item, qty);
    }

    public static ItemStack getSteam() {
        initSteam();
        return new ItemStack(itemSteam);
    }

    private static void initSteam() {
        if (itemSteam == null) {
            String tag = "rc.liquid.steam";
            int id = RailcraftConfig.getItemId(tag);

            if (id > 0) {
                Item item = new ItemLiquid(id).setRarity(0).setIconIndex(16).setItemName(tag);
                GameRegistry.registerItem(item, tag);
                RailcraftLanguage.getInstance().registerItemName(item, tag);

                liquidSteam = LiquidDictionary.getOrCreateLiquid("Steam", new LiquidStack(item.itemID, 0));

                ItemRegistry.registerItem(tag, new ItemStack(item));

                itemSteam = item;
            }
        }
    }

    public static LiquidStack getSteamLiquid() {
        return getSteamLiquid(0);
    }

    public static LiquidStack getSteamLiquid(int qty) {
        initSteam();
        LiquidStack liquid = liquidSteam.copy();
        liquid.amount = qty;
        return liquid;
    }

//    public static ItemStack getSteamBottle(int qty) {
//        Item item = itemSteamBottle;
//        if (item == null) {
//            String tag = "rc.liquid.steam.bottle";
//            int id = 8000;
//
//            item = new ItemLiquidContainer(id).setIconIndex(18).setItemName(tag);
//            item.setCreativeTab(CreativeTabs.tabMisc);
//            GameRegistry.registerItem(item, tag);
////            RailcraftLanguage.getInstance().registerItemName(item, tag);
//
//            LiquidManager.getInstance().registerBottle(getSteamLiquid(LiquidManager.BUCKET_VOLUME), new ItemStack(item));
//
////            ItemRegistry.registerItem(tag, new ItemStack(item));
//
//            itemSteamBottle = item;
//        }
//        return new ItemStack(item, qty);
//    }
}
