package railcraft.common.liquids;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidContainerData;
import net.minecraftforge.liquids.LiquidContainerRegistry;
import net.minecraftforge.liquids.LiquidStack;
import net.minecraftforge.liquids.LiquidTank;
import railcraft.common.liquids.tanks.StandardTank;
import railcraft.common.util.inventory.InvTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public final class LiquidManager implements ILiquidAdaptor {

    public static final int BUCKET_FILL_TIME = 8;
    public static final int NETWORK_UPDATE_INTERVAL = 128;
    public static final int BUCKET_VOLUME = 1000;
    private static LiquidManager instance;
    private List<ILiquidAdaptor> adapters = new ArrayList<ILiquidAdaptor>();

    private LiquidManager() {
        adapters.add(ForestryLiquidAdapater.getInstance());
        adapters.add(ForgeLiquidAdapater.getInstance());

//        registerBottle(new LiquidStack(Block.waterStill, BUCKET_VOLUME), new ItemStack(Item.potion));
    }

    public static LiquidManager getInstance() {
        if (instance == null) {
            instance = new LiquidManager();
        }
        return instance;
    }

    public boolean handleRightClick(ITankContainer tank, int tankIndex, EntityPlayer player, boolean fill, boolean drain) {
        ItemStack current = player.inventory.getCurrentItem();
        if (current != null) {

            LiquidStack liquid = getLiquidInContainer(current);

            if (fill && liquid != null) {
                int used = tank.fill(tankIndex, liquid, true);

                if (used > 0) {
                    player.inventory.setInventorySlotContents(player.inventory.currentItem, InvTools.depleteItem(current));
                    return true;
                }

            } else if (drain) {

                LiquidStack available = tank.getTanks(ForgeDirection.UNKNOWN)[tankIndex].getLiquid();
                if (available != null) {
                    ItemStack filled = fillLiquidContainer(available, current);

                    liquid = getLiquidInContainer(filled);
                    if (liquid != null) {

                        if (current.stackSize > 1) {
                            if (!player.inventory.addItemStackToInventory(filled)) {
                                return false;
                            } else {
                                player.inventory.setInventorySlotContents(player.inventory.currentItem, InvTools.depleteItem(current));
                            }
                        } else {
                            player.inventory.setInventorySlotContents(player.inventory.currentItem, InvTools.depleteItem(current));
                            player.inventory.setInventorySlotContents(player.inventory.currentItem, filled);
                        }

                        tank.drain(tankIndex, liquid.amount, true);
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public void processContainers(StandardTank tank, IInventory inv, int inputSlot, int outputSlot) {
        processContainers(tank, inv, inputSlot, outputSlot, true, true);
    }

    public void processContainers(StandardTank tank, IInventory inv, int inputSlot, int outputSlot, boolean processFilled, boolean processEmpty) {
        TankManager tankManger = new TankManager();
        tankManger.addTank(tank);
        processContainers(tankManger, 0, inv, inputSlot, outputSlot, processFilled, processEmpty);
    }

    public void processContainers(ITankContainer tank, int tankIndex, IInventory inv, int inputSlot, int outputSlot) {
        processContainers(tank, tankIndex, inv, inputSlot, outputSlot, true, true);
    }

    public void processContainers(ITankContainer tank, int tankIndex, IInventory inv, int inputSlot, int outputSlot, boolean processFilled, boolean processEmpty) {
        ItemStack input = inv.getStackInSlot(inputSlot);
        ItemStack output = inv.getStackInSlot(outputSlot);

        if (input != null) {
            LiquidStack bucketLiquid = LiquidManager.getInstance().getLiquidInContainer(input);
            ItemStack emptyItem = input.getItem().getContainerItemStack(input);
            if (processFilled && bucketLiquid != null && (emptyItem == null || output == null || (output.stackSize < output.getMaxStackSize() && InvTools.isItemEqual(output, emptyItem)))) {
                int used = tank.fill(tankIndex, bucketLiquid, false);
                if (used >= bucketLiquid.amount) {
                    tank.fill(tankIndex, bucketLiquid, true);
                    if (emptyItem != null) {
                        if (output == null) {
                            inv.setInventorySlotContents(outputSlot, emptyItem);
                        } else {
                            output.stackSize++;
                        }
                    }
                    inv.decrStackSize(inputSlot, 1);
                }
            } else if (processEmpty && LiquidManager.getInstance().isEmptyContainer(input)) {
                ItemStack filled = LiquidManager.getInstance().fillLiquidContainer(tank.getTanks(ForgeDirection.UNKNOWN)[tankIndex].getLiquid(), input);
                if (filled != null && (output == null || (output.stackSize < output.getMaxStackSize() && InvTools.isItemEqual(filled, output)))) {
                    LiquidStack liquid = LiquidManager.getInstance().getLiquidInContainer(filled);
                    LiquidStack drain = tank.drain(tankIndex, liquid.amount, false);
                    if (drain != null && drain.amount > 0) {
                        tank.drain(tankIndex, liquid.amount, true);
                        if (output == null) {
                            inv.setInventorySlotContents(outputSlot, filled);
                        } else {
                            output.stackSize++;
                        }
                        inv.decrStackSize(inputSlot, 1);
                    }
                }
            }
        }
    }

    public boolean isBucket(ItemStack stack) {
        return LiquidContainerRegistry.isBucket(stack);
    }

    public boolean isContainer(ItemStack stack) {
        return LiquidContainerRegistry.isContainer(stack);
    }

    public boolean isFilledContainer(ItemStack stack) {
        return LiquidContainerRegistry.isFilledContainer(stack);
    }

    public boolean isEmptyContainer(ItemStack stack) {
        return LiquidContainerRegistry.isEmptyContainer(stack);
    }

    public ItemStack fillLiquidContainer(LiquidStack liquid, ItemStack empty) {
        if (liquid == null || empty == null) return null;
        return LiquidContainerRegistry.fillLiquidContainer(liquid, empty);
    }

    public ItemStack getFilledLiquidContainer(LiquidStack liquid, ItemStack empty) {
        if (liquid == null || empty == null) return null;
        liquid = liquid.copy();
        liquid.amount = Integer.MAX_VALUE;
        return LiquidContainerRegistry.fillLiquidContainer(liquid, empty);
    }

    public LiquidStack getLiquidInContainer(ItemStack stack) {
        return LiquidContainerRegistry.getLiquidForFilledItem(stack);
    }

    public boolean containsLiquid(ItemStack stack, LiquidStack liquid) {
        return LiquidContainerRegistry.containsLiquid(stack, liquid);
    }

    public boolean isLiquidEqual(LiquidStack L1, LiquidStack L2) {
        if (L1 == null || L2 == null) {
            return false;
        }
        return L1.isLiquidEqual(L2);
    }

    public void registerBucket(LiquidStack liquid, ItemStack filled) {
        ItemStack empty = new ItemStack(Item.bucketEmpty);
        LiquidContainerData container = new LiquidContainerData(liquid, filled, empty);
        registerContainer(container);
    }

    public boolean registerWax(LiquidStack liquid, ItemStack filled) {
        ItemStack empty = ForestryLiquidAdapater.getInstance().waxCapsule;
        if (empty != null) {
            LiquidContainerData container = new LiquidContainerData(liquid, filled, empty);
            registerContainer(container);
            return true;
        }
        return false;
    }

    public boolean registerRefactory(LiquidStack liquid, ItemStack filled) {
        ItemStack empty = ForestryLiquidAdapater.getInstance().refractoryEmpty;
        if (empty != null) {
            LiquidContainerData container = new LiquidContainerData(liquid, filled, empty);
            registerContainer(container);
            return true;
        }
        return false;
    }

    public boolean registerCan(LiquidStack liquid, ItemStack filled) {
        ItemStack empty = ForestryLiquidAdapater.getInstance().canEmpty;
        if (empty != null) {
            LiquidContainerData container = new LiquidContainerData(liquid, filled, empty);
            registerContainer(container);
            return true;
        }
        return false;
    }

    public boolean registerCell(LiquidStack liquid, ItemStack filled) {
        ItemStack empty = ForestryLiquidAdapater.getInstance().cellEmpty;
        if (empty != null) {
            LiquidContainerData container = new LiquidContainerData(liquid, filled, empty);
            registerContainer(container);
            return true;
        }
        return false;
    }

    public void registerBottle(LiquidStack liquid, ItemStack filled) {
        ItemStack empty = new ItemStack(Item.glassBottle);
        LiquidContainerData container = new LiquidContainerData(liquid, filled, empty);
        registerContainer(container);
    }

    @Override
    public void registerContainer(LiquidContainerData container) {
        for (ILiquidAdaptor adapter : adapters) {
            adapter.registerContainer(container);
        }
    }
}
