package railcraft.common.liquids;

import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;

/**
 * This class provides some convenience functions for ITankContainers
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TankToolkit implements ITankContainer {

    private ITankContainer tankContainer;

    public TankToolkit(ITankContainer c) {
        tankContainer = c;
    }

    public int getLiquidQty(LiquidStack liquid) {
        if (liquid == null) {
            return 0;
        }
        int amount = 0;
        for (ILiquidTank tank : getTanks(ForgeDirection.UNKNOWN)) {
            if (liquid.isLiquidEqual(tank.getLiquid())) {
                amount += tank.getLiquid().amount;
            }
        }
        return amount;
    }

    public boolean isTankEmpty(LiquidStack liquid) {
        return getLiquidQty(liquid) <= 0;
    }

    public boolean isTankFull(LiquidStack liquid) {
        if (liquid == null) {
            return false;
        }
        liquid = liquid.copy();
        liquid.amount = 1;
        int fill = fill(ForgeDirection.UNKNOWN, liquid, false);
        return fill <= 0;
    }

    public boolean areTanksFull() {
        for (ILiquidTank tank : getTanks(ForgeDirection.UNKNOWN)) {
            if (tank.getLiquid() == null || tank.getLiquid().amount < tank.getCapacity()) {
                return false;
            }
        }
        return true;
    }

    public boolean areTanksEmpty() {
        return !isLiquidInTank();
    }

    public boolean isLiquidInTank() {
        for (ILiquidTank tank : getTanks(ForgeDirection.UNKNOWN)) {
            boolean empty = tank.getLiquid() == null || tank.getLiquid().amount <= 0;
            if (!empty) {
                return true;
            }
        }
        return false;
    }

    public float getLiquidLevel() {
        int amount = 0;
        int capacity = 0;
        for (ILiquidTank tank : getTanks(ForgeDirection.UNKNOWN)) {
            LiquidStack liquid = tank.getLiquid();
            amount += liquid == null ? 0 : liquid.amount;
            capacity += tank.getCapacity();
        }
        return capacity == 0 ? 0 : amount / capacity;
    }

    public float getLiquidLevel(LiquidStack filter) {
        int amount = 0;
        int capacity = 0;
        for (ILiquidTank tank : getTanks(ForgeDirection.UNKNOWN)) {
            LiquidStack liquid = tank.getLiquid();
            if (liquid == null || !liquid.isLiquidEqual(filter)) {
                continue;
            }
            amount += liquid.amount;
            capacity += tank.getCapacity();
        }
        return capacity == 0 ? 0 : amount / (float)capacity;
    }

    public boolean canPutLiquid(ForgeDirection from, LiquidStack liquid) {
        if (liquid == null) {
            return false;
        }
        return fill(from, liquid, false) > 0;
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
        return tankContainer.fill(from, resource, doFill);
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill) {
        return tankContainer.fill(tankIndex, resource, doFill);
    }

    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return tankContainer.drain(from, maxDrain, doDrain);
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain) {
        return tankContainer.drain(tankIndex, maxDrain, doDrain);
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection side) {
        return tankContainer.getTanks(side);
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type) {
        return tankContainer.getTank(direction, type);
    }
}
