package railcraft.common.liquids.tanks;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.item.ItemStack;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.core.Railcraft;
import railcraft.common.gui.util.ToolTip;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class FilteredTank extends StandardTank {

    private final LiquidStack filter;

    public FilteredTank(int capacity, LiquidStack filter) {
        super(capacity);
        this.filter = filter;
    }

    @Override
    public int fill(LiquidStack resource, boolean doFill) {
        if (filter.isLiquidEqual(resource)) {
            return super.fill(resource, doFill);
        }
        return 0;
    }

    public LiquidStack getFilter() {
        return filter.copy();
    }

    public boolean liquidMatchesFilter(LiquidStack resource) {
        if (resource == null || filter == null) {
            return false;
        }
        return filter.isLiquidEqual(resource);
    }

    public List<ToolTip> getToolTip() {
        List<ToolTip> list = new ArrayList<ToolTip>();
        int amount = 0;
        LiquidStack liquidType = getFilter();
        if (liquidType != null && liquidType.itemID > 0) {
            ItemStack stack = liquidType.asItemStack();
            if (stack.getItem() == null) {
                return list;
            }
            list.add(new ToolTip(Railcraft.getProxy().getItemDisplayName(stack), Railcraft.getProxy().getItemRarityColor(stack)));
            if (getLiquid() != null) {
                amount = getLiquid().amount;
            }
        }
        list.add(new ToolTip(String.format("%,d", amount) + " / " + String.format("%,d", getCapacity())));
        return list;
    }
}
