package railcraft.common.modules;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecartRegistry;
import net.minecraftforge.common.MinecraftForge;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.aesthetics.cube.BlockCube;
import railcraft.common.blocks.aesthetics.cube.EnumCube;
import railcraft.common.blocks.detector.BlockDetector.Detector;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.machine.alpha.TamingInteractHandler;
import railcraft.common.blocks.machine.gamma.EnumMachineGamma;
import railcraft.common.carts.EntityCartTrackRelayer;
import railcraft.common.carts.EntityCartUndercutter;
import railcraft.common.carts.EntityTunnelBore;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemBoreHeadDiamond;
import railcraft.common.carts.ItemBoreHeadIron;
import railcraft.common.carts.ItemBoreHeadSteel;
import railcraft.common.carts.ItemCart;
import railcraft.common.carts.ItemTunnelBore;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.modules.orehandlers.BoreOreHandler;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.MiscTools;

public class ModuleAutomation extends RailcraftModule {

    @Override
    public void preInit() {
        MinecraftForge.EVENT_BUS.register(new BoreOreHandler());
    }

    @Override
    public void initFirst() {
        RailcraftBlocks.registerBlockDetector();
        BlockCube.registerBlock();

        Block blockDetector = RailcraftBlocks.getBlockDetector();

        if (blockDetector != null) {
            CraftingPlugin.addShapedOreRecipe(new ItemStack(blockDetector, 1, Detector.ITEM.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', "plankWood",
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.ANY.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', Block.stone,
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.EMPTY.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', new ItemStack(Block.stoneBrick, 1, 0),
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.MOB.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', new ItemStack(Block.stoneBrick, 1, 1),
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.MOB.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', Block.cobblestoneMossy,
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.POWERED.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', Block.cobblestone,
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.PLAYER.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', new ItemStack(Block.stoneSingleSlab, 1, 0),
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedOreRecipe(new ItemStack(blockDetector, 1, Detector.EXPLOSIVE.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', "slabWood",
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.ANIMAL.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', new ItemStack(Block.wood, 1, 0),
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.AGE.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', new ItemStack(Block.wood, 1, 1),
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedOreRecipe(new ItemStack(blockDetector, 1, Detector.ADVANCED.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', "ingotSteel",
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.TANK.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', Item.brick,
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.SHEEP.ordinal()), new Object[]{
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', Block.cloth,
                        'P', Block.pressurePlateStone,});
            CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.VILLAGER.ordinal()),
                    "XXX",
                    "XPX",
                    "XXX",
                    'X', Item.leather,
                    'P', Block.pressurePlateStone);
        }


        EnumMachineGamma gamma = EnumMachineGamma.DISPENSER_CART;
        if (RailcraftConfig.isSubBlockEnabled(gamma.getTag())) {
            RailcraftBlocks.registerBlockMachineGamma();
            Block blockUtility = RailcraftBlocks.getBlockMachineGamma();
            if (blockUtility != null) {
                ItemStack cartDispenser = gamma.getItem(1);
                CraftingPlugin.addShapedRecipe(cartDispenser,
                        "ML",
                        'M', Item.minecartEmpty,
                        'L', Block.dispenser);
                RailcraftLanguage.getInstance().registerItemName(cartDispenser, gamma.getTag());
            }
        }

        EnumMachineAlpha alpha = EnumMachineAlpha.FEED_STATION;
        if (RailcraftConfig.isSubBlockEnabled(alpha.getTag())) {
            RailcraftBlocks.registerBlockMachineAlpha();
            Block block = RailcraftBlocks.getBlockMachineAlpha();
            if (block != null) {
                ItemStack stack = alpha.getItem();
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "PCP",
                            "CSC",
                            "PCP",
                            'P', "plankWood",
                            'S', ModuleManager.isModuleLoaded(ModuleManager.Module.FACTORY) ? RailcraftPartItems.getPlateSteel() : new ItemStack(Block.blockSteel),
                            'C', new ItemStack(Item.goldenCarrot)});
                RailcraftLanguage.getInstance().registerItemName(stack, alpha.getTag());

                MinecraftForge.EVENT_BUS.register(new TamingInteractHandler());
            }
        }

        // Define Bore
        String tag = "cart.bore";
        boolean cartEnabled = RailcraftConfig.isCartEnabled(tag);

        if (cartEnabled) {
            MiscTools.registerMinecart(Railcraft.getMod(), EntityTunnelBore.class, tag, 50);

            int id = RailcraftConfig.getItemId("item.cart.bore");

            if (id > 0) {
                Item itemCartBore = new ItemTunnelBore(id);
                RailcraftLanguage.getInstance().registerItemName(itemCartBore, tag);
                MinecartRegistry.registerMinecart(EntityTunnelBore.class, new ItemStack(itemCartBore));

                CraftingPlugin.addShapedOreRecipe(new ItemStack(itemCartBore), new Object[]{
                            "ICI",
                            "FCF",
                            " S ",
                            'I', "blockSteel",
                            'S', Item.minecartCrate,
                            'F', Block.stoneOvenIdle,
                            'C', Item.minecartEmpty
                        });

                ItemRegistry.registerItem(tag, new ItemStack(itemCartBore));


                tag = "tool.bore.head.diamond";
                id = RailcraftConfig.getItemId(tag);
                if (id > 0) {
                    Item item = new ItemBoreHeadDiamond(id);
                    RailcraftLanguage.getInstance().registerItemName(item, tag);
                    CraftingPlugin.addShapedOreRecipe(new ItemStack(item), new Object[]{
                                "III",
                                "IDI",
                                "III",
                                'I', "ingotSteel",
                                'D', Block.blockDiamond,});
                    ItemRegistry.registerItem(tag, new ItemStack(item));
                }

                tag = "tool.bore.head.steel";
                id = RailcraftConfig.getItemId(tag);
                if (id > 0) {
                    Item item = new ItemBoreHeadSteel(id);
                    RailcraftLanguage.getInstance().registerItemName(item, tag);
                    CraftingPlugin.addShapedOreRecipe(new ItemStack(item), new Object[]{
                                "III",
                                "IDI",
                                "III",
                                'I', "ingotSteel",
                                'D', "blockSteel",});
                    ItemRegistry.registerItem(tag, new ItemStack(item));
                }

                tag = "tool.bore.head.iron";
                id = RailcraftConfig.getItemId(tag);
                if (id > 0) {
                    Item item = new ItemBoreHeadIron(id);
                    RailcraftLanguage.getInstance().registerItemName(item, tag);
                    CraftingPlugin.addShapedOreRecipe(new ItemStack(item), new Object[]{
                                "III",
                                "IDI",
                                "III",
                                'I', "ingotSteel",
                                'D', Block.blockSteel,});
                    ItemRegistry.registerItem(tag, new ItemStack(item));
                }

//                OreDictionary.registerOreHandler(new BoreOreHandler());
            }
        }

        // Define Track Relayer Cart

        EnumCart cart = EnumCart.TRACK_RELAYER;
        cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());

        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartTrackRelayer.class, cart.getTag(), cart.getId());
                Item item = new ItemCart(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "YLY",
                            "RSR",
                            "DMD",
                            'L', new ItemStack(Block.redstoneLampIdle),
                            'Y', "dyeYellow",
                            'R', new ItemStack(Item.blazeRod),
                            'D', new ItemStack(Item.pickaxeDiamond),
                            'S', "blockSteel",
                            'M', new ItemStack(Item.minecartEmpty)
                        });
                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartTrackRelayer.class, stack);

                ItemRegistry.registerItem("cart.track.relayer", stack);
            }
        }

        // Define Undercutter Cart

        cart = EnumCart.UNDERCUTTER;
        cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());

        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartUndercutter.class, cart.getTag(), cart.getId());
                Item item = new ItemCart(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "YLY",
                            "RSR",
                            "DMD",
                            'L', new ItemStack(Block.redstoneLampIdle),
                            'Y', "dyeYellow",
                            'R', new ItemStack(Block.pistonBase),
                            'D', new ItemStack(Item.shovelDiamond),
                            'S', "blockSteel",
                            'M', new ItemStack(Item.minecartEmpty)
                        });
                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartUndercutter.class, stack);


                ItemRegistry.registerItem("cart.undercutter", stack);
            }
        }
    }

    @Override
    public void initSecond() {
        Block blockDetector = RailcraftBlocks.getBlockDetector();

        if (blockDetector != null) {
            if (EnumCube.INFERNAL_BRICK.isEnabled()) {
                CraftingPlugin.addShapedRecipe(new ItemStack(blockDetector, 1, Detector.LOCOMOTIVE.ordinal()),
                        "XXX",
                        "XPX",
                        "XXX",
                        'X', EnumCube.INFERNAL_BRICK.getItem(),
                        'P', Block.pressurePlateStone);
            }
        }
    }
}
