package railcraft.common.modules;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.ForgeChunkManager;
import net.minecraftforge.common.MinecartRegistry;
import net.minecraftforge.common.MinecraftForge;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.machine.beta.EnumMachineBeta;
import railcraft.common.carts.EntityCartAnchor;
import railcraft.common.carts.EntityCartAnchorPersonal;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemCartAnchor;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.ChunkManager;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ModuleChunkLoading extends RailcraftModule {

    @Override
    public void initFirst() {
        ForgeChunkManager.setForcedChunkLoadingCallback(Railcraft.getMod(), ChunkManager.getInstance());
        MinecraftForge.EVENT_BUS.register(ChunkManager.getInstance());

        EnumMachineAlpha alpha = EnumMachineAlpha.WORLD_ANCHOR;
        if (RailcraftConfig.isSubBlockEnabled(alpha.getTag())) {
            RailcraftBlocks.registerBlockMachineAlpha();
            Block block = RailcraftBlocks.getBlockMachineAlpha();
            if (block != null) {
                ItemStack stack = alpha.getItem();

                if (RailcraftConfig.canCraftAnchors()) {
                    CraftingPlugin.addShapedRecipe(stack,
                            "gog",
                            "dpd",
                            "gog",
                            'd', Item.diamond,
                            'g', Item.ingotGold,
                            'p', Item.enderPearl,
                            'o', new ItemStack(Block.obsidian));
                }
                RailcraftLanguage.getInstance().registerItemName(stack, alpha.getTag());
            }
        }

        alpha = EnumMachineAlpha.PERSONAL_ANCHOR;
        if (RailcraftConfig.isSubBlockEnabled(alpha.getTag())) {
            RailcraftBlocks.registerBlockMachineAlpha();
            Block block = RailcraftBlocks.getBlockMachineAlpha();
            if (block != null) {
                ItemStack stack = alpha.getItem();

                if (RailcraftConfig.canCraftPersonalAnchors()) {
                    CraftingPlugin.addShapedRecipe(stack,
                            "gog",
                            "dpd",
                            "gog",
                            'd', Item.emerald,
                            'g', Item.ingotGold,
                            'p', Item.enderPearl,
                            'o', new ItemStack(Block.obsidian));
                }

                RailcraftLanguage.getInstance().registerItemName(stack, alpha.getTag());
            }
        }

        EnumMachineBeta beta = EnumMachineBeta.SENTINEL;
        if (RailcraftConfig.isSubBlockEnabled(beta.getTag())) {
            RailcraftBlocks.registerBlockMachineBeta();
            Block block = RailcraftBlocks.getBlockMachineBeta();
            if (block != null) {
                ItemStack stack = beta.getItem();
                if (RailcraftConfig.canCraftAnchors()) {
                    CraftingPlugin.addShapedRecipe(stack,
                            " p ",
                            " o ",
                            "ogo",
                            'g', Item.ingotGold,
                            'p', Item.enderPearl,
                            'o', new ItemStack(Block.obsidian));
                }
                RailcraftLanguage.getInstance().registerItemName(stack, beta.getTag());
            }
        }

        // Define Anchor Cart

        EnumCart cart = EnumCart.ANCHOR;
        boolean cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());

        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartAnchor.class, cart.getTag(), cart.getId());
                Item item = new ItemCartAnchor(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);
                ItemStack anchor = EnumMachineAlpha.WORLD_ANCHOR.getItem();
                if (RailcraftConfig.canCraftAnchors()) {
                    CraftingPlugin.addShapedRecipe(stack,
                            "A",
                            "M",
                            'A', anchor,
                            'M', Item.minecartEmpty);
                    CraftingPlugin.addShapelessRecipe(new ItemStack(Item.minecartEmpty), stack);
                }
                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartAnchor.class, stack);

                cart.setContents(anchor);

                ItemRegistry.registerItem("cart.anchor", stack);
            }
        }


        // Define Personal Anchor Cart

        cart = EnumCart.ANCHOR_PERSONAL;
        cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());

        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartAnchorPersonal.class, cart.getTag(), cart.getId());
                Item item = new ItemCartAnchor(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);
                ItemStack anchor = EnumMachineAlpha.PERSONAL_ANCHOR.getItem();
                if (RailcraftConfig.canCraftPersonalAnchors()) {
                    CraftingPlugin.addShapedRecipe(stack,
                            "A",
                            "M",
                            'A', anchor,
                            'M', Item.minecartEmpty);
                    CraftingPlugin.addShapelessRecipe(new ItemStack(Item.minecartEmpty), stack);
                }
                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartAnchorPersonal.class, stack);

                cart.setContents(anchor);

                ItemRegistry.registerItem("cart.anchor.personal", stack);
            }
        }
    }
}
