package railcraft.common.modules;

import cpw.mods.fml.common.network.NetworkRegistry;
import cpw.mods.fml.common.registry.GameRegistry;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;
import net.minecraft.block.Block;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.entity.monster.EntityEnderman;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.item.crafting.FurnaceRecipes;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.ChestGenHooks;
import net.minecraftforge.common.MinecartRegistry;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.liquids.LiquidStack;
import net.minecraftforge.oredict.ShapedOreRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.api.fuel.FuelManager;
import railcraft.common.api.signals.SignalTools;
import railcraft.common.blocks.machine.MachineTileRegistery;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.carts.EntityCartBasic;
import railcraft.common.carts.EntityCartChest;
import railcraft.common.carts.EntityCartFurnace;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemCartVanilla;
import railcraft.common.carts.LinkageHandler;
import railcraft.common.carts.LinkageManager;
import railcraft.common.carts.MinecartHooks;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.gui.GuiHandler;
import railcraft.common.items.CrowbarHandler;
import railcraft.common.items.FuelHandler;
import railcraft.common.items.ItemGoggles;
import railcraft.common.items.ItemMagnifyingGlass;
import railcraft.common.items.ItemNugget;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.items.RailcraftToolItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.buildcraft.BuildcraftPlugin;
import railcraft.common.util.crafting.BlastFurnaceCraftingManager;
import railcraft.common.util.crafting.CokeOvenCraftingManager;
import railcraft.common.util.crafting.CraftingHandler;
import railcraft.common.util.crafting.RockCrusherCraftingManager;
import railcraft.common.util.crafting.RollingMachineCraftingManager;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.liquids.LiquidManager;
import railcraft.common.plugins.forestry.ForestryPlugin;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.forge.Creeper$Plugin;
import railcraft.common.plugins.forge.LootPlugin;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;
import railcraft.common.util.network.PacketBuilder;

public class ModuleCore extends RailcraftModule {

    @Override
    public void preInit() {
        LinkageManager.reset();

        RailcraftCraftingManager.cokeOven = new CokeOvenCraftingManager();
        RailcraftCraftingManager.blastFurnace = new BlastFurnaceCraftingManager();
        RailcraftCraftingManager.rockCrusher = new RockCrusherCraftingManager();
        RailcraftCraftingManager.rollingMachine = new RollingMachineCraftingManager();

        SignalTools.packetBuilder = PacketBuilder.getInstance();

        LiquidItems.initialize();
    }

    @Override
    public void initFirst() {
        NetworkRegistry.instance().registerGuiHandler(Railcraft.getMod(), new GuiHandler());

        LootPlugin.increaseLootGen(1, 2,
                ChestGenHooks.MINESHAFT_CORRIDOR,
                ChestGenHooks.VILLAGE_BLACKSMITH);

        RailcraftToolItems.registerCrowbar();
        MinecraftForge.EVENT_BUS.register(CrowbarHandler.getInstance());

        ItemMagnifyingGlass.register();

        RailcraftToolItems.initializeToolsArmor();

        ItemNugget.getNuggetIron();

        EntityEnderman.carriableBlocks[Block.gravel.blockID] = false;

        BuildcraftPlugin.init();

        MinecraftForge.EVENT_BUS.register(MinecartHooks.getInstance());
        MinecraftForge.EVENT_BUS.register(LinkageHandler.getInstance());

        if (RailcraftConfig.useCollisionHandler()) {
            if (EntityMinecart.getCollisionHandler() != null) {
                Game.log(Level.CONFIG, "Existing Minecart Collision Handler detected, overwriting. Please check your configs to ensure this is desired behavior.");
            }
            EntityMinecart.setCollisionHandler(MinecartHooks.getInstance());
        }

        GameRegistry.registerCraftingHandler(new CraftingHandler());

        Set<Integer> testSet = new HashSet<Integer>();
        if (!RailcraftConfig.useOldRecipes()) {
            testSet.add(Block.rail.blockID);
            testSet.add(Block.railPowered.blockID);
            testSet.add(Block.railDetector.blockID);
        }
        
        if(RailcraftConfig.getRecipeConfig("railcraft.cart.furnace")){
            testSet.add(Item.minecartPowered.itemID);
        }

//        MiscTools.addShapelessRecipe(new ItemStack(Item.coal, 20), Block.dirt);

        Iterator it = CraftingManager.getInstance().getRecipeList().iterator();
        while (it.hasNext()) {
            IRecipe r = (IRecipe) it.next();
            ItemStack output = r.getRecipeOutput();
            if (output != null) {
                if (testSet.contains(output.itemID)) {
                    it.remove();
                }
            }
        }

        // Items
        int itemIdOffset = Item.shovelSteel.itemID;
        int itemIdCartBasic = Item.minecartEmpty.itemID;
        int itemIdCartChest = Item.minecartCrate.itemID;
        int itemIdCartSteam = Item.minecartPowered.itemID;


        Item.itemsList[itemIdCartBasic] = null;
        Item.itemsList[itemIdCartChest] = null;
        Item.itemsList[itemIdCartSteam] = null;

        Item itemCartBasic = new ItemCartVanilla(itemIdCartBasic - itemIdOffset, EnumCart.BASIC);
        Item itemCartChest = new ItemCartVanilla(itemIdCartChest - itemIdOffset, EnumCart.CHEST);
        Item itemCartPower = new ItemCartVanilla(itemIdCartSteam - itemIdOffset, EnumCart.FURNACE);

        GameRegistry.registerItem(itemCartBasic, EnumCart.BASIC.getItemTag());
        GameRegistry.registerItem(itemCartChest, EnumCart.CHEST.getItemTag());
        GameRegistry.registerItem(itemCartPower, EnumCart.FURNACE.getItemTag());

        MinecartRegistry.removeMinecart(EntityMinecart.class, 0);
        MinecartRegistry.removeMinecart(EntityMinecart.class, 1);
        MinecartRegistry.removeMinecart(EntityMinecart.class, 2);

        MinecartRegistry.registerMinecart(EntityCartBasic.class, new ItemStack(itemCartBasic));
        MinecartRegistry.registerMinecart(EntityCartChest.class, 1, new ItemStack(itemCartChest));
        MinecartRegistry.registerMinecart(EntityCartFurnace.class, 2, new ItemStack(itemCartPower));

        MiscTools.registerMinecart(Railcraft.getMod(), EntityCartBasic.class, "cart.basic", EnumCart.BASIC.getId());
        MiscTools.registerMinecart(Railcraft.getMod(), EntityCartChest.class, "cart.chest", EnumCart.CHEST.getId());
        MiscTools.registerMinecart(Railcraft.getMod(), EntityCartFurnace.class, "cart.furnace", EnumCart.FURNACE.getId());

        MinecraftForge.setBlockHarvestLevel(Block.rail, "crowbar", 0);
        MinecraftForge.setBlockHarvestLevel(Block.rail, "pickaxe", 0);
        MinecraftForge.setBlockHarvestLevel(Block.railPowered, "crowbar", 0);
        MinecraftForge.setBlockHarvestLevel(Block.railPowered, "pickaxe", 0);
        MinecraftForge.setBlockHarvestLevel(Block.railDetector, "crowbar", 0);
        MinecraftForge.setBlockHarvestLevel(Block.railDetector, "pickaxe", 0);

        // Define Recipies
        if (RailcraftConfig.getRecipeConfig("railcraft.cart.bronze")) {
            IRecipe recipe = new ShapedOreRecipe(new ItemStack(Item.minecartEmpty), false, new Object[]{
                        "I I",
                        "III",
                        'I', "ingotBronze",});
            CraftingManager.getInstance().getRecipeList().add(recipe);
        }

        if (RailcraftConfig.getRecipeConfig("railcraft.cart.steel")) {
            IRecipe recipe = new ShapedOreRecipe(new ItemStack(Item.minecartEmpty, 2), false, new Object[]{
                        "I I",
                        "III",
                        'I', "ingotSteel",});
            CraftingManager.getInstance().getRecipeList().add(recipe);
        }

        // Old rails
        if (!RailcraftConfig.useOldRecipes()) {
            ItemStack stackRailNormal = new ItemStack(Block.rail, 32);
            ItemStack stackRailBooster = new ItemStack(Block.railPowered, 16);
            ItemStack stackRailDetector = new ItemStack(Block.railDetector, 16);

            CraftingPlugin.addShapedRecipe(stackRailNormal, new Object[]{
                        "I I",
                        "I#I",
                        "I I",
                        'I', RailcraftPartItems.getRailStandard(),
                        '#', RailcraftPartItems.getRailbedWood(),});
            CraftingPlugin.addShapedRecipe(stackRailBooster, new Object[]{
                        "I I",
                        "I#I",
                        "IrI",
                        'I', RailcraftPartItems.getRailAdvanced(),
                        '#', RailcraftPartItems.getRailbedWood(),
                        'r', Item.redstone,});
            CraftingPlugin.addShapedRecipe(stackRailDetector, new Object[]{
                        "IsI",
                        "I#I",
                        "IrI",
                        'I', RailcraftPartItems.getRailStandard(),
                        '#', Block.pressurePlateStone,
                        'r', Item.redstone,
                        's', RailcraftPartItems.getRailbedWood(),});

            CraftingPlugin.addShapelessRecipe(RailcraftPartItems.getRailStandard(),
                    Block.rail,
                    Block.rail,
                    Block.rail,
                    Block.rail,
                    Block.rail,
                    Block.rail,
                    Block.rail,
                    Block.rail);

            RailcraftLanguage.getInstance().registerItemName(stackRailNormal, "track.standard");
            RailcraftLanguage.getInstance().registerItemName(stackRailBooster, "track.powered");
            RailcraftLanguage.getInstance().registerItemName(stackRailDetector, "track.detector");
        }


        // Old Carts
        ItemStack stackCartBasic = new ItemStack(itemCartBasic, 1);
        ItemStack stackCartChest = new ItemStack(itemCartChest, 1);
        ItemStack stackCartPower = new ItemStack(itemCartPower, 1);

        CraftingManager.getInstance().addShapelessRecipe(new ItemStack(Item.minecartEmpty), new Object[]{
                    itemCartChest,});
        CraftingManager.getInstance().addShapelessRecipe(new ItemStack(Item.minecartEmpty), new Object[]{
                    itemCartPower,});

        RailcraftLanguage.getInstance().registerItemName(stackCartBasic, EnumCart.BASIC.getTag());
        RailcraftLanguage.getInstance().registerItemName(stackCartChest, EnumCart.CHEST.getTag());
        RailcraftLanguage.getInstance().registerItemName(stackCartPower, EnumCart.FURNACE.getTag());

        LootPlugin.addLootMineshaft(stackCartBasic, 1, 1, "cart.basic");
        LootPlugin.addLootMineshaft(stackCartChest, 1, 1, "cart.chest");
        LootPlugin.addLootMineshaft(new ItemStack(Block.rail), 8, 32, "track.basic");

        MachineTileRegistery.registerTileEntities();
    }

    @Override
    public void initSecond() {
        if (RailcraftConfig.useCreosoteFurnaceRecipes() || !EnumMachineAlpha.COKE_OVEN.isEnabled()) {
            FurnaceRecipes.smelting().addSmelting(Item.coal.itemID, 0, LiquidItems.getCreosoteOilBottle(2), 0);
            FurnaceRecipes.smelting().addSmelting(Item.coal.itemID, 1, LiquidItems.getCreosoteOilBottle(1), 0);
        }

        ItemGoggles.registerItem();
    }

    @Override
    public void postInit() {
        GameRegistry.registerFuelHandler(new FuelHandler());

        addLiquidFuels();

        Creeper$Plugin.postSetup();
    }

    public static void addLiquidFuels() {
        ItemStack bioFuel = ForestryPlugin.getItem("liquidBiofuel");
        if (bioFuel != null) {
            FuelManager.addBoilerFuel(new LiquidStack(bioFuel.itemID, LiquidManager.BUCKET_VOLUME), 32000); // Biofuel
            Game.log(Level.FINER, "Registered Forestry BioFuel as a valid fuel source.");
        }

        Item fuel = null;
        try {
            fuel = (Item) Class.forName("buildcraft.BuildCraftEnergy").getField("fuel").get(null);
        } catch (Exception ex) {
            Game.log(Level.FINER, "Buildcraft Fuel not found.");
        }

        if (fuel != null) {
            FuelManager.addBoilerFuel(new LiquidStack(fuel, LiquidManager.BUCKET_VOLUME), 96000); // Fuel
            Game.log(Level.FINER, "Registered Buildcraft Fuel as a valid fuel source.");
        }
    }
}
