package railcraft.common.modules;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecartRegistry;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.tracks.EnumTrack;
import railcraft.common.carts.EntityCartTNT;
import railcraft.common.carts.EntityCartWork;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemCart;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.forge.LootPlugin;
import railcraft.common.util.misc.MiscTools;

public class ModuleExtras extends RailcraftModule {

    @Override
    public void initFirst() {
        RailcraftBlocks.registerBlockTrack();
        RailcraftBlocks.registerBlockRailElevator();

        // Define TNT Cart
        EnumCart cart = EnumCart.TNT;
        boolean cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());
        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartTNT.class, cart.getTag(), cart.getId());
                Item item = new ItemCart(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "WTW",
                            "WWW",
                            'T', Block.tnt,
                            'W', "slabWood"
                        });

                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartTNT.class, stack);
                LootPlugin.addLootMineshaft(stack.copy(), 1, 3, cart.getTag());
                ItemRegistry.registerItem(cart.getTag(), stack);
            }
        }

        // Define Work Cart
        cart = EnumCart.WORK;
        cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());
        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartWork.class, cart.getTag(), cart.getId());
                Item item = new ItemCart(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);
                CraftingPlugin.addShapedRecipe(stack, new Object[]{
                            "B",
                            "M",
                            'B', Block.workbench,
                            'M', Item.minecartEmpty
                        });
                CraftingManager.getInstance().addShapelessRecipe(new ItemStack(Item.minecartEmpty), new Object[]{
                            item,});
                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartWork.class, stack);
                LootPlugin.addLootMineshaft(stack.copy(), 1, 1, cart.getTag());

                ItemRegistry.registerItem(cart.getTag(), stack);
            }
        }

        Block blockTrack = RailcraftBlocks.getBlockTrack();
        if (blockTrack != null) {
            MiscTools.registerTrack(EnumTrack.PRIMING);
            MiscTools.registerTrack(EnumTrack.LAUNCHER);
            MiscTools.registerTrack(EnumTrack.SUSPENDED);
        }
    }
}
