package railcraft.common.modules;

import cpw.mods.fml.common.registry.EntityRegistry;
import forestry.api.recipes.RecipeManagers;
import ic2.api.Ic2Recipes;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.aesthetics.cube.BlockCube;
import railcraft.common.blocks.aesthetics.cube.EntityFallingCube;
import railcraft.common.blocks.aesthetics.cube.EnumCube;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.ItemNugget;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.items.RailcraftToolItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.ic2.IC2Plugin;
import railcraft.common.util.crafting.RollingMachineCraftingManager;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.forge.LootPlugin;
import railcraft.common.util.misc.BallastRegistry;
import railcraft.common.util.misc.Game;

public class ModuleFactory extends RailcraftModule {

    @Override
    public void initFirst() {
        RailcraftBlocks.registerBlockMachineAlpha();

        Block blockUtility = RailcraftBlocks.getBlockMachineAlpha();
        if (blockUtility == null) {
            postInitNotLoaded();
            return;
        }

        BlockCube.registerBlock();
        RailcraftToolItems.registerCoalCoke();

        EnumMachineAlpha typeUtility = EnumMachineAlpha.COKE_OVEN;
        if (RailcraftConfig.isSubBlockEnabled(typeUtility.getTag())) {
            ItemStack stack = typeUtility.getItem();
            CraftingPlugin.addShapedRecipe(stack, new Object[]{
                        "MBM",
                        "BMB",
                        "MBM",
                        'B', new ItemStack(Item.brick),
                        'M', new ItemStack(Block.sand),});

            RailcraftLanguage.getInstance().registerItemName(stack, typeUtility.getTag());

            if (RailcraftToolItems.getCoalCoke() != null) {
                RailcraftCraftingManager.cokeOven.addRecipe(Item.coal.itemID, 0, RailcraftToolItems.getCoalCoke(), LiquidItems.getCreosoteOilLiquid(500), 3000);
            }

            ItemStack rubberWood = IC2Plugin.getItem("rubberWood");
            if (rubberWood != null) {
                RailcraftCraftingManager.cokeOven.addRecipe(rubberWood.copy(), new ItemStack(Item.coal, 1, 1), LiquidItems.getCreosoteOilLiquid(250), 3000);
            }
        }

        typeUtility = EnumMachineAlpha.STEAM_OVEN;
        if (RailcraftConfig.isSubBlockEnabled(typeUtility.getTag())) {
            CraftingPlugin.addShapedRecipe(typeUtility.getItem(4), new Object[]{
                        "SSS",
                        "SFS",
                        "SSS",
                        'F', new ItemStack(Block.stoneOvenIdle),
                        'S', RailcraftPartItems.getPlateSteel(),});

            RailcraftLanguage.getInstance().registerItemName(typeUtility.getItem(), typeUtility.getTag());
        }

        typeUtility = EnumMachineAlpha.BLAST_FURNACE;
        if (RailcraftConfig.isSubBlockEnabled(typeUtility.getTag())) {
            ItemStack stack = typeUtility.getItem(4);
            CraftingPlugin.addShapedRecipe(stack, new Object[]{
                        "MBM",
                        "BPB",
                        "MBM",
                        'B', new ItemStack(Block.netherBrick),
                        'M', new ItemStack(Block.slowSand),
                        'P', Item.magmaCream,});

            RailcraftLanguage.getInstance().registerItemName(stack, typeUtility.getTag());

            int burnTime = 1200;
            if (IC2Plugin.isIC2Installed()
                    && RailcraftConfig.getRecipeConfig("ic2.refinedIron.disable")
                    && RailcraftConfig.getRecipeConfig("ic2.refinedIron.useSteel")) {
                burnTime = RailcraftConfig.getSteelCookTime();
            }
            RailcraftCraftingManager.blastFurnace.addRecipe(Item.ingotIron.itemID, burnTime, RailcraftPartItems.getIngotSteel(1));

            RailcraftCraftingManager.blastFurnace.addRecipe(Item.helmetSteel.itemID, 0, burnTime * 5, RailcraftPartItems.getIngotSteel(5));
            RailcraftCraftingManager.blastFurnace.addRecipe(Item.plateSteel.itemID, 0, burnTime * 8, RailcraftPartItems.getIngotSteel(8));
            RailcraftCraftingManager.blastFurnace.addRecipe(Item.legsSteel.itemID, 0, burnTime * 7, RailcraftPartItems.getIngotSteel(7));
            RailcraftCraftingManager.blastFurnace.addRecipe(Item.bootsSteel.itemID, 0, burnTime * 4, RailcraftPartItems.getIngotSteel(4));

            RailcraftCraftingManager.blastFurnace.addRecipe(Item.swordSteel.itemID, 0, burnTime * 2, RailcraftPartItems.getIngotSteel(2));
            RailcraftCraftingManager.blastFurnace.addRecipe(Item.shovelSteel.itemID, 0, burnTime, RailcraftPartItems.getIngotSteel(1));
            RailcraftCraftingManager.blastFurnace.addRecipe(Item.pickaxeSteel.itemID, 0, burnTime * 3, RailcraftPartItems.getIngotSteel(3));
            RailcraftCraftingManager.blastFurnace.addRecipe(Item.hoeSteel.itemID, 0, burnTime * 2, RailcraftPartItems.getIngotSteel(2));

            try {
                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getCrowbar(), true, burnTime * 3, RailcraftPartItems.getIngotSteel(3));

                int recycleTime = burnTime / 2;
                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getSteelHelm(), false, recycleTime * 4, RailcraftPartItems.getIngotSteel(4));
                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getSteelArmor(), false, recycleTime * 6, RailcraftPartItems.getIngotSteel(6));
                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getSteelLegs(), false, recycleTime * 5, RailcraftPartItems.getIngotSteel(5));
                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getSteelBoots(), false, recycleTime * 3, RailcraftPartItems.getIngotSteel(3));

                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getSteelSword(), false, recycleTime * 1, RailcraftPartItems.getIngotSteel(1));
                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getSteelPickaxe(), false, recycleTime * 2, RailcraftPartItems.getIngotSteel(2));
                RailcraftCraftingManager.blastFurnace.addRecipe(RailcraftToolItems.getSteelHoe(), false, recycleTime * 1, RailcraftPartItems.getIngotSteel(1));
            } catch (Throwable error) {
                Game.logErrorAPI("Railcraft", error);
            }
        }

        typeUtility = EnumMachineAlpha.ROCK_CRUSHER;
        if (RailcraftConfig.isSubBlockEnabled(typeUtility.getTag())) {
            ItemStack stack = typeUtility.getItem(4);
            CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                        "PDP",
                        "DSD",
                        "PDP",
                        'D', new ItemStack(Item.diamond),
                        'P', new ItemStack(Block.pistonBase),
                        'S', "blockSteel",});

            RailcraftLanguage.getInstance().registerItemName(stack, typeUtility.getTag());

            LinkedHashMap<ItemStack, Float> output = new LinkedHashMap<ItemStack, Float>();
            output.put(EnumCube.CRUSHED_OBSIDIAN.getItem(), 1.0f);
            output.put(RailcraftPartItems.getDustObsidian(), 0.25f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.obsidian.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(RailcraftPartItems.getDustObsidian(), 1.0f);
            output.put(RailcraftPartItems.getDustObsidian(), 0.25f);
            RailcraftCraftingManager.rockCrusher.addRecipe(EnumCube.CRUSHED_OBSIDIAN.getItem(), output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.brick, 3), 1.0f);
            output.put(new ItemStack(Item.brick), 0.5f);
            output.put(new ItemStack(Block.sand), 0.25f);
            output.put(new ItemStack(Block.sand), 0.25f);
            output.put(new ItemStack(Block.sand), 0.25f);
            output.put(new ItemStack(Block.sand), 0.25f);
            output.put(new ItemStack(Block.sand), 0.25f);
            RailcraftCraftingManager.rockCrusher.addRecipe(EnumMachineAlpha.COKE_OVEN.getItem(), output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.netherBrick), 0.75f);
            output.put(new ItemStack(Block.slowSand), 0.75f);
            output.put(new ItemStack(Item.blazePowder), 0.05f);
            RailcraftCraftingManager.rockCrusher.addRecipe(EnumMachineAlpha.BLAST_FURNACE.getItem(), output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(EnumCube.CRUSHED_OBSIDIAN.getItem(), 1.0f);
            output.put(EnumCube.CRUSHED_OBSIDIAN.getItem(), 0.5f);
            output.put(new ItemStack(Item.diamond), 0.5f);
            output.put(new ItemStack(Block.obsidian), 0.25f);
            output.put(RailcraftPartItems.getDustObsidian(), 0.25f);
            output.put(new ItemStack(Item.goldNugget, 16), 1.0f);
            output.put(new ItemStack(Item.goldNugget, 8), 0.5f);
            output.put(new ItemStack(Item.goldNugget, 8), 0.5f);
            output.put(new ItemStack(Item.goldNugget, 4), 0.5f);
            RailcraftCraftingManager.rockCrusher.addRecipe(EnumMachineAlpha.WORLD_ANCHOR.getItem(), output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(EnumCube.CRUSHED_OBSIDIAN.getItem(), 1.0f);
            output.put(EnumCube.CRUSHED_OBSIDIAN.getItem(), 0.5f);
            output.put(new ItemStack(Item.emerald), 0.5f);
            output.put(new ItemStack(Block.obsidian), 0.25f);
            output.put(RailcraftPartItems.getDustObsidian(), 0.25f);
            output.put(new ItemStack(Item.goldNugget, 16), 1.0f);
            output.put(new ItemStack(Item.goldNugget, 8), 0.5f);
            output.put(new ItemStack(Item.goldNugget, 8), 0.5f);
            output.put(new ItemStack(Item.goldNugget, 4), 0.5f);
            RailcraftCraftingManager.rockCrusher.addRecipe(EnumMachineAlpha.PERSONAL_ANCHOR.getItem(), output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.gravel), 1.0f);
            output.put(new ItemStack(Item.flint), 0.10f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.cobblestone.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.gravel), 1.0f);
            output.put(new ItemStack(Block.vine), 0.10f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.cobblestoneMossy.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.sand), 1.0f);
            output.put(new ItemStack(Item.goldNugget), 0.001f);
            output.put(new ItemStack(Item.diamond), 0.00005f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.gravel.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.cobblestone), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stone.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.sand), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.sandStone.blockID, -1, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.brick, 3), 1.0f);
            output.put(new ItemStack(Item.brick), 0.5f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.brick.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.clay, 4), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.blockClay.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.cobblestone), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stoneBrick.blockID, -1, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.gravel), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stairCompactCobblestone.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.cobblestone), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stairsStoneBrickSmooth.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.netherBrick), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stairsNetherBrick.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.brick, 4), 1.0f);
            output.put(new ItemStack(Item.brick), 0.5f);
            output.put(new ItemStack(Item.brick), 0.5f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stairsBrick.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.cobblestone), 0.45f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stoneSingleSlab.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.sand), 0.45f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stoneSingleSlab.blockID, 1, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.gravel), 0.45f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stoneSingleSlab.blockID, 3, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.brick), 1.0f);
            output.put(new ItemStack(Item.brick), 0.75f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stoneSingleSlab.blockID, 4, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.cobblestone), 0.45f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.stoneSingleSlab.blockID, 5, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.blockSnow), 0.85f);
            output.put(new ItemStack(Item.snowball), 0.25f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.ice.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Block.netherBrick), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.netherFence.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.lightStoneDust, 3), 1.0f);
            output.put(new ItemStack(Item.lightStoneDust), 0.75f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.glowStone.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.lightStoneDust, 3), 1.0f);
            output.put(new ItemStack(Item.lightStoneDust), 0.75f);
            output.put(new ItemStack(Item.redstone, 3), 1.0f);
            output.put(new ItemStack(Item.redstone), 0.75f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Block.redstoneLampIdle.blockID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.dyePowder, 4, 15), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Item.bone.itemID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(new ItemStack(Item.blazePowder, 2), 1.0f);
            output.put(new ItemStack(Item.blazePowder), 0.25f);
            output.put(new ItemStack(Item.blazePowder), 0.25f);
            output.put(new ItemStack(Item.blazePowder), 0.25f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Item.blazeRod.itemID, 0, output);

            output = new LinkedHashMap<ItemStack, Float>();
            output.put(RailcraftPartItems.getDustCharcoal(), 1.0f);
            RailcraftCraftingManager.rockCrusher.addRecipe(Item.coal.itemID, 1, output);
        }


        typeUtility = EnumMachineAlpha.ROLLING_MACHINE;
        if (RailcraftConfig.isSubBlockEnabled(typeUtility.getTag())) {
            ItemStack stack = typeUtility.getItem();
            CraftingPlugin.addShapedRecipe(stack, new Object[]{
                        "IPI",
                        "PCP",
                        "IPI",
                        'I', Item.ingotIron,
                        'P', Block.pistonBase,
                        'C', Block.workbench
                    });
            RailcraftLanguage.getInstance().registerItemName(stack, typeUtility.getTag());
        } else {
            postInitNotLoaded();
        }

        if (BlockCube.getBlock() != null) {
            EnumCube type = EnumCube.STEEL_BLOCK;
            if (RailcraftConfig.isSubBlockEnabled(type.getTag())) {
                ItemStack stack = type.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, type.getTag());
                IRecipe recipe = new ShapedOreRecipe(stack, new Object[]{
                            "III",
                            "III",
                            "III",
                            'I', "ingotSteel"});
                CraftingManager.getInstance().getRecipeList().add(recipe);

                CraftingPlugin.addShapelessRecipe(RailcraftPartItems.getIngotSteel(9), stack);

                LootPlugin.addLootTool(stack.copy(), 1, 1, "steel.block");

                OreDictionary.registerOre("blockSteel", stack);

                if (EnumMachineAlpha.BLAST_FURNACE.isEnabled()) {
                    RailcraftCraftingManager.blastFurnace.addRecipe(Block.blockSteel.blockID, 0, 11520, EnumCube.STEEL_BLOCK.getItem());
                }
            }

            type = EnumCube.CRUSHED_OBSIDIAN;
            if (RailcraftConfig.isSubBlockEnabled(type.getTag())) {
                ItemStack stack = type.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, type.getTag());

                BallastRegistry.registerBallast(BlockCube.getBlock().blockID, type.ordinal());

                EntityRegistry.registerModEntity(EntityFallingCube.class, "FallingCube", 52, Railcraft.getMod(), 160, 20, true);

                if (IC2Plugin.isIC2Installed() && RailcraftConfig.addObsidianRecipesToMacerator()) {
                    Ic2Recipes.addMaceratorRecipe(new ItemStack(Block.obsidian), stack);
                    Ic2Recipes.addMaceratorRecipe(stack, RailcraftPartItems.getDustObsidian());
                }
            }
        }
    }

    @Override
    public void initSecond() {
        if (ModuleManager.isModuleLoaded(ModuleManager.Module.STRUCTURES) && BlockCube.getBlock() != null) {
            if (EnumMachineAlpha.BLAST_FURNACE.isEnabled() && EnumCube.INFERNAL_BRICK.isEnabled()) {

                ItemStack stack = EnumMachineAlpha.BLAST_FURNACE.getItem(4);
                CraftingPlugin.addShapedRecipe(stack,
                        " B ",
                        "BPB",
                        " B ",
                        'B', EnumCube.INFERNAL_BRICK.getItem(),
                        'P', Item.magmaCream);
            }
            if (EnumMachineAlpha.COKE_OVEN.isEnabled() && EnumCube.SANDY_BRICK.isEnabled()) {
                ItemStack stack = EnumMachineAlpha.COKE_OVEN.getItem();
                CraftingPlugin.addShapedRecipe(stack,
                        " B ",
                        " S ",
                        " B ",
                        'B', EnumCube.SANDY_BRICK.getItem(),
                        'S', new ItemStack(Block.sand));
            }
        }
    }

    private void registerOreDustRecipe(ItemStack ore, ItemStack dust) {
        if (dust == null) {
            return;
        }
        dust = dust.copy();
        dust.stackSize = 2;

        HashMap<ItemStack, Float> output = new HashMap<ItemStack, Float>();
        output.put(dust, 1.0f);

        RailcraftCraftingManager.rockCrusher.addRecipe(ore, output);
    }

    @Override
    public void postInit() {
        if (OreDictionary.getOres("blockSteel").isEmpty()) {
            OreDictionary.registerOre("blockSteel", Block.blockSteel);
        }

        if (!EnumMachineAlpha.BLAST_FURNACE.isEnabled()) {
            registerAltSteelFurnaceRecipe();
        }

        List<ItemStack> logs = OreDictionary.getOres("logWood");
        for (ItemStack log : logs) {
            RailcraftCraftingManager.cokeOven.addRecipe(log, new ItemStack(Item.coal, 1, 1), LiquidItems.getCreosoteOilLiquid(250), 3000);
        }

        if (IC2Plugin.isIC2Installed()) {
            ItemStack dustIron = IC2Plugin.getItem("ironDust");
            ItemStack dustGold = IC2Plugin.getItem("goldDust");
            ItemStack dustCopper = IC2Plugin.getItem("copperDust");
            ItemStack dustTin = IC2Plugin.getItem("tinDust");
            ItemStack dustSilver = IC2Plugin.getItem("silverDust");

            if (RailcraftConfig.canCrushOres()) {
                registerOreDustRecipe(new ItemStack(Block.oreIron), dustIron);
                registerOreDustRecipe(new ItemStack(Block.oreGold), dustGold);

                List<ItemStack> ores = OreDictionary.getOres("oreCopper");
                for (ItemStack ore : ores) {
                    registerOreDustRecipe(ore, dustCopper);
                }

                ores = OreDictionary.getOres("oreTin");
                for (ItemStack ore : ores) {
                    registerOreDustRecipe(ore, dustTin);
                }

                ores = OreDictionary.getOres("oreSilver");
                for (ItemStack ore : ores) {
                    registerOreDustRecipe(ore, dustSilver);
                }
            }

            if (!RailcraftConfig.getRecipeConfig("ic2.macerator.ores")) {
                IC2Plugin.removeMaceratorDustRecipes(dustIron, dustGold, dustCopper, dustTin, dustSilver);
            }

            if (!RailcraftConfig.getRecipeConfig("ic2.macerator.bones")) {
                IC2Plugin.removeMaceratorRecipes(new ItemStack(Item.dyePowder, 1, 15));
            }

            if (!RailcraftConfig.getRecipeConfig("ic2.macerator.blaze")) {
                IC2Plugin.removeMaceratorRecipes(new ItemStack(Item.blazePowder));
            }

            if (!RailcraftConfig.getRecipeConfig("ic2.macerator.cobble")) {
                IC2Plugin.removeMaceratorRecipes(new ItemStack(Block.cobblestone));
            }

            if (!RailcraftConfig.getRecipeConfig("ic2.macerator.dirt")) {
                IC2Plugin.removeMaceratorRecipes(new ItemStack(Block.dirt));
            }
        }

        if (RecipeManagers.carpenterManager != null && RailcraftConfig.getRecipeConfig("forestry.carpenter.ties")) {
            RecipeManagers.carpenterManager.addRecipe(40, LiquidItems.getCreosoteOilLiquid(750), null, RailcraftPartItems.getTieWood(), new Object[]{
                        "###",
                        '#', "slabWood"
                    });
        }
    }

    public static int getFuelValues(int id) {
        if (RailcraftToolItems.getCoalCoke() != null && id == RailcraftToolItems.getCoalCoke().itemID) {
            return 3200;
        }
        return 0;
    }

    private static void registerAltSteelFurnaceRecipe() {
        List<ItemStack> iron = OreDictionary.getOres("nuggetIron");
        for (ItemStack nugget : iron) {
            CraftingPlugin.addFurnaceRecipe(nugget, ItemNugget.getNuggetSteel(), 0);
        }
    }

    @Override
    public void postInitNotLoaded() {
        RollingMachineCraftingManager.copyRecipesToWorkbench();
        registerAltSteelFurnaceRecipe();
    }
}
