package railcraft.common.modules;

import ic2.api.Ic2Recipes;
import java.util.List;
import java.util.logging.Level;
import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecartRegistry;
import net.minecraftforge.oredict.OreDictionary;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.detector.BlockDetector.Detector;
import railcraft.common.blocks.machine.gamma.EnumMachineGamma;
import railcraft.common.carts.EntityCartEnergy;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemCart;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.ItemRailcraft;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.ic2.IC2Plugin;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

public class ModuleIC2 extends RailcraftModule {

    public static Item lapotronUpgrade;
//    private static Item creosoteWood;

    @Override
    public boolean canModuleLoad() {
        return IC2Plugin.isIC2Installed();
    }

    @Override
    public void printLoadError() {
        Game.log(Level.FINER, "Module disabled: {0}, IC2 not detected", this);
    }

    @Override
    public void initFirst() {
        RailcraftBlocks.registerBlockDetector();
        RailcraftBlocks.registerBlockMachineGamma();

        EnumMachineGamma type = EnumMachineGamma.ENERGY_LOADER;
        if (type.isEnabled()) {

            ItemStack stack = type.getItem();
            RailcraftLanguage.getInstance().registerItemName(stack, type.getTag());
        }

        type = EnumMachineGamma.ENERGY_UNLOADER;
        if (type.isEnabled()) {

            ItemStack stack = type.getItem();
            RailcraftLanguage.getInstance().registerItemName(stack, type.getTag());
        }

        int id = RailcraftConfig.getItemId("ic2.upgrade.lapotron");
        if (id > 0) {
            lapotronUpgrade = new ItemRailcraft(id).setItemName("lapotronUpgrade").setIconIndex(183).setMaxStackSize(9);

            RailcraftLanguage.getInstance().registerItemName(lapotronUpgrade, "ic2.upgrade.lapotron");

            ItemRegistry.registerItem("ic2.upgrade.lapotron", new ItemStack(lapotronUpgrade));
        }



        int batId = RailcraftConfig.getItemId("item.cart.batbox");
        int mfeId = RailcraftConfig.getItemId("item.cart.mfe");
        int mfsuId = RailcraftConfig.getItemId("item.cart.mfsu");

        boolean batEnabled = RailcraftConfig.isCartEnabled("cart.energy.batbox") && batId > 0;
        boolean mfeEnabled = RailcraftConfig.isCartEnabled("cart.energy.mfe") && mfeId > 0;
        boolean mfsuEnabled = RailcraftConfig.isCartEnabled("cart.energy.mfsu") && mfsuId > 0;
        if (batEnabled || mfeEnabled || mfsuEnabled) {
            MiscTools.registerMinecart(Railcraft.getMod(), EntityCartEnergy.class, "cart.energy", EnumCart.BATBOX.getId());
        }

        if (batEnabled) {
            Item item = new ItemCart(batId, EnumCart.BATBOX);
            ItemStack stack = new ItemStack(item);
            MinecartRegistry.registerMinecart(EntityCartEnergy.class, 0, stack);
            RailcraftLanguage.getInstance().registerItemName(stack, EnumCart.BATBOX.getTag());
            ItemRegistry.registerItem("cart.energy.batbox", stack);
        }

        if (mfeEnabled) {
            Item item = new ItemCart(mfeId, EnumCart.MFE);
            ItemStack stack = new ItemStack(item);
            MinecartRegistry.registerMinecart(EntityCartEnergy.class, 1, stack);
            RailcraftLanguage.getInstance().registerItemName(stack, EnumCart.MFE.getTag());
            ItemRegistry.registerItem("cart.energy.mfe", stack);
        }

        if (mfsuEnabled) {
            Item item = new ItemCart(mfsuId, EnumCart.MFSU);
            ItemStack stack = new ItemStack(item);
            MinecartRegistry.registerMinecart(EntityCartEnergy.class, 2, stack);
            RailcraftLanguage.getInstance().registerItemName(stack, EnumCart.MFSU.getTag());
            ItemRegistry.registerItem("cart.energy.mfsu", stack);
        }

//        id = RailcraftConfig.getItemId("item.creosote.wood");
//        if(id > 0){
//            creosoteWood = new ItemRailcraft(id).setItemName("creosoteWood").setIconIndex(184);
//            ItemStack wood = new ItemStack(creosoteWood);
//            RailcraftLanguage.getInstance().registerItemName(creosoteWood, "Creosote Wood");
//
//            ItemStack oil = RailcraftPartItems.getCreosoteOil(2);
//            Ic2Recipes.addExtractorRecipe(wood, oil);
//
//            CropCard bush = new CreosoteBush(wood);
//
//            if(!CropCard.registerCrop(bush, 156)){
//                CropCard.registerCrop(bush);
//            }
//        }

    }

    @Override
    public void postInit() {

        if (RailcraftConfig.getRecipeConfig("ic2.refinedIron.disable")) {
            List<ItemStack> reIron = OreDictionary.getOres("ingotRefinedIron");
            reIron.clear();
        }
        if (RailcraftConfig.getRecipeConfig("ic2.refinedIron.useSteel")) {
            List<ItemStack> reIron = OreDictionary.getOres("ingotRefinedIron");
            reIron.addAll(OreDictionary.getOres("ingotSteel"));
        }

        createRecipes();
    }

    private static void createRecipes() {
        Block blockDetector = RailcraftBlocks.getBlockDetector();

        if (blockDetector != null) {
            ItemStack stack = new ItemStack(blockDetector, 1, Detector.ENERGY.ordinal());
            CraftingPlugin.addShapedOreRecipe(stack, false,
                    "XXX",
                    "XPX",
                    "XXX",
                    'X', "ingotTin",
                    'P', Block.pressurePlateStone);
        }

        ItemStack batbox = IC2Plugin.getItem("batBox");
        if (batbox != null) {
            EnumCart.BATBOX.setContents(batbox);
            ItemStack cart = MinecartRegistry.getItemForCart(EntityCartEnergy.class, 0);
            if (cart != null) {
                CraftingPlugin.addShapedRecipe(cart, new Object[]{
                            "E",
                            "M",
                            'E', batbox,
                            'M', Item.minecartEmpty
                        });
                CraftingPlugin.addShapelessRecipe(new ItemStack(Item.minecartEmpty), new Object[]{cart});
            }
        }

        ItemStack mfe = IC2Plugin.getItem("mfeUnit");
        if (mfe != null) {
            EnumCart.MFE.setContents(mfe);
            ItemStack cart = MinecartRegistry.getItemForCart(EntityCartEnergy.class, 1);
            if (cart != null) {
                CraftingPlugin.addShapedRecipe(cart, new Object[]{
                            "E",
                            "M",
                            'E', mfe,
                            'M', Item.minecartEmpty
                        });
                CraftingPlugin.addShapelessRecipe(new ItemStack(Item.minecartEmpty), new Object[]{cart});
            }
        }

        ItemStack mfsu = IC2Plugin.getItem("mfsUnit");
        if (mfsu != null) {
            EnumCart.MFSU.setContents(mfsu);
            ItemStack cart = MinecartRegistry.getItemForCart(EntityCartEnergy.class, 2);
            if (cart != null) {
                CraftingPlugin.addShapedRecipe(cart, new Object[]{
                            "E",
                            "M",
                            'E', mfsu,
                            'M', Item.minecartEmpty
                        });
                CraftingPlugin.addShapelessRecipe(new ItemStack(Item.minecartEmpty), new Object[]{cart});
            }
        }


        ItemStack battery = IC2Plugin.getItem("reBattery");
        ItemStack machine = IC2Plugin.getItem("machine");

        ItemStack detector = null;
        if (blockDetector != null) {
            detector = new ItemStack(blockDetector, 1, Detector.ENERGY.ordinal());
        } else {
            detector = new ItemStack(Block.pressurePlateStone);
        }

        if (battery != null && machine != null) {
            if (EnumMachineGamma.ENERGY_LOADER.isEnabled()) {
                Ic2Recipes.addCraftingRecipe(EnumMachineGamma.ENERGY_LOADER.getItem(), new Object[]{
                            "BLB",
                            "BIB",
                            "BDB",
                            'D', detector,
                            'B', battery,
                            'I', machine,
                            'L', new ItemStack(Block.dispenser, 1, 0)});
            }

            if (EnumMachineGamma.ENERGY_UNLOADER.isEnabled()) {
                Ic2Recipes.addCraftingRecipe(EnumMachineGamma.ENERGY_UNLOADER.getItem(), new Object[]{
                            "BDB",
                            "BIB",
                            "BLB",
                            'D', detector,
                            'B', battery,
                            'I', machine,
                            'L', new ItemStack(Block.dispenser, 1, 0),});
            }
        }

        int id = RailcraftConfig.getItemId("ic2.upgrade.lapotron");
        if (id > 0) {
            ItemStack lapotron = IC2Plugin.getItem("lapotronCrystal");
            ItemStack glassCable = IC2Plugin.getItem("glassFiberCableItem");
            ItemStack circuit = IC2Plugin.getItem("advancedCircuit");

            if (lapotron != null && glassCable != null && circuit != null) {
                lapotron.copy();
//                lapotron.setItemDamage(-1);
                Ic2Recipes.addCraftingRecipe(new ItemStack(lapotronUpgrade), new Object[]{
                            "GGG",
                            "wLw",
                            "GCG",
                            'G', new ItemStack(Block.glass, 1, 0),
                            'w', glassCable,
                            'C', circuit,
                            'L', lapotron,});
            }
        }
    }

    public static ItemStack getLapotronUpgrade() {
        if (lapotronUpgrade == null) {
            return null;
        }
        return new ItemStack(lapotronUpgrade);
    }
}
