/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.modules;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraftforge.common.MinecartRegistry;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.machine.beta.EnumMachineBeta;
import railcraft.common.blocks.tracks.EnumTrack;
import railcraft.common.carts.EntityLocomotiveSteam;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemLocomotive;
import railcraft.common.carts.LocomotivePaintingRecipe;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.ItemWhistleTuner;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ModuleLocomotives extends RailcraftModule {

    @Override
    public boolean canModuleLoad() {
        return true;
    }

    @Override
    public void initFirst() {

        MiscTools.registerTrack(EnumTrack.WHISTLE);
        MiscTools.registerTrack(EnumTrack.LOCOMOTIVE);
        MiscTools.registerTrack(EnumTrack.LIMITER);
        
        ItemWhistleTuner.registerItem();

        EnumCart cart = EnumCart.LOCO_STEAM;
        boolean cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());
        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityLocomotiveSteam.class, cart.getTag(), cart.getId());
                ItemLocomotive item = new ItemLocomotive(itemId, cart, 10 * 16, 11 * 16);
                item.setItemName(cart.getItemTag());
                item.setRarity(1);
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);
                cart.setCartItem(stack);
                paintLocomotive(stack);

                ItemStack tank;
                if (EnumMachineBeta.BOILER_TANK_HIGH_PRESSURE.isEnabled()) {
                    tank = EnumMachineBeta.BOILER_TANK_HIGH_PRESSURE.getItem();
                } else if (EnumMachineBeta.BOILER_TANK_LOW_PRESSURE.isEnabled()) {
                    tank = EnumMachineBeta.BOILER_TANK_LOW_PRESSURE.getItem();
                } else if (EnumMachineBeta.TANK_IRON_WALL.isEnabled()) {
                    tank = EnumMachineBeta.TANK_IRON_WALL.getItem();
                } else if (RailcraftPartItems.getIngotSteel() != null) {
                    tank = RailcraftPartItems.getIngotSteel();
                } else {
                    tank = new ItemStack(Item.ingotIron);
                }

                ItemStack firebox;
                if (EnumMachineBeta.BOILER_FIREBOX_SOLID.isEnabled()) {
                    firebox = EnumMachineBeta.BOILER_FIREBOX_SOLID.getItem();
                } else if (EnumMachineAlpha.BLAST_FURNACE.isEnabled()) {
                    firebox = EnumMachineAlpha.BLAST_FURNACE.getItem();
                } else {
                    firebox = new ItemStack(Block.stoneOvenIdle);
                }

                CraftingPlugin.addShapedRecipe(stack,
                        "TTF",
                        "TTF",
                        "BMM",
                        'T', tank,
                        'F', firebox,
                        'M', Item.minecartEmpty,
                        'B', new ItemStack(Block.fenceIron));

                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityLocomotiveSteam.class, stack);
                ItemRegistry.registerItem(cart.getTag(), stack);
            }
        }
    }

    private void paintLocomotive(ItemStack base) {
        IRecipe recipe = new LocomotivePaintingRecipe(base);
        CraftingPlugin.addRecipe(recipe);
    }
}
