package railcraft.common.modules;

import cpw.mods.fml.common.registry.GameRegistry;
import java.util.Calendar;
import java.util.logging.Level;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecartRegistry;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.carts.EntityCartGift;
import railcraft.common.carts.EntityCartPumpkin;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemCart;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.Game;
import railcraft.common.util.misc.MiscTools;

public class ModuleSeasonal extends RailcraftModule {

    @Override
    public void initFirst() {

        // Define Pumpkin Cart
        EnumCart cart = EnumCart.PUMPKIN;
        boolean cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());
        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartPumpkin.class, cart.getTag(), cart.getId());
                Item item = new ItemCart(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);

                Calendar cal = Calendar.getInstance();
                int month = cal.get(Calendar.MONTH);
                if (month == Calendar.OCTOBER || month == Calendar.NOVEMBER) {
                    Game.log(Level.INFO, "Activating Halloween Seasonal Pack");
                    CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                                "GGG",
                                "WPW",
                                "WWW",
                                'G', new ItemStack(Item.gunpowder),
                                'P', new ItemStack(Block.pumpkin),
                                'W', "slabWood"
                            });
                }

                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartPumpkin.class, stack);
                ItemRegistry.registerItem(cart.getTag(), stack);
            }
        }

        // Define Gift Cart
        cart = EnumCart.GIFT;
        cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());
        if (cartEnabled) {
            int itemId = RailcraftConfig.getItemId(cart.getItemTag());
            if (itemId > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartGift.class, cart.getTag(), cart.getId());
                Item item = new ItemCart(itemId, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stack = new ItemStack(item);

                Calendar cal = Calendar.getInstance();
                int month = cal.get(Calendar.MONTH);
                if (month == Calendar.DECEMBER || month == Calendar.JANUARY) {
                    Game.log(Level.INFO, "Activating Christmas Seasonal Pack");
                    CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                                "GGG",
                                "WEW",
                                "WWW",
                                'G', new ItemStack(Item.gunpowder),
                                'E', new ItemStack(Item.emerald),
                                'W', "slabWood"
                            });
                }

                RailcraftLanguage.getInstance().registerItemName(stack, cart.getTag());

                MinecartRegistry.registerMinecart(EntityCartGift.class, stack);
                ItemRegistry.registerItem(cart.getTag(), stack);
            }
        }
    }
}
