package railcraft.common.modules;

import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.signals.EnumSignal;
import railcraft.common.blocks.signals.ItemSignalBlockSurveyor;
import railcraft.common.blocks.signals.ItemSignalTuner;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.ItemCircuit;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;

public class ModuleSignals extends RailcraftModule {

    @Override
    public void initFirst() {
        RailcraftBlocks.registerBlockSignal();

        Block blockSignal = RailcraftBlocks.getBlockSignal();
        if (blockSignal != null) {

            ItemSignalBlockSurveyor.registerItem();
            ItemSignalTuner.registerItem();

            // Define Block Signal
            EnumSignal structure = EnumSignal.BLOCK_SIGNAL;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "LCI",
                            " BI",
                            "   ",
                            'C', ItemCircuit.getSignalCircuit(),
                            'I', Item.ingotIron,
                            'L', RailcraftPartItems.getSignalLamp(),
                            'B', "dyeBlack",});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Dual Head Block Signal
            structure = EnumSignal.DUAL_HEAD_BLOCK_SIGNAL;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "LCI",
                            " BI",
                            "LRI",
                            'C', ItemCircuit.getSignalCircuit(),
                            'R', ItemCircuit.getReceiverCircuit(),
                            'I', Item.ingotIron,
                            'L', RailcraftPartItems.getSignalLamp(),
                            'B', "dyeBlack",});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Distant Signal
            structure = EnumSignal.DISTANT_SIGNAL;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {
                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "LCI",
                            " BI",
                            "   ",
                            'C', ItemCircuit.getReceiverCircuit(),
                            'I', Item.ingotIron,
                            'L', RailcraftPartItems.getSignalLamp(),
                            'B', "dyeBlack",});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Dual Head Block Signal
            structure = EnumSignal.DUAL_HEAD_DISTANT_SIGNAL;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "LRI",
                            " BI",
                            "LRI",
                            'R', ItemCircuit.getReceiverCircuit(),
                            'I', Item.ingotIron,
                            'L', RailcraftPartItems.getSignalLamp(),
                            'B', "dyeBlack",});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Switch Lever
            structure = EnumSignal.SWITCH_LEVER;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "RBW",
                            "PLI",
                            'W', "dyeWhite",
                            'I', Item.ingotIron,
                            'L', Block.lever,
                            'P', Block.pistonBase,
                            'B', "dyeBlack",
                            'R', "dyeRed",});
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "RBW",
                            "ILP",
                            'W', "dyeWhite",
                            'I', Item.ingotIron,
                            'L', Block.lever,
                            'P', Block.pistonBase,
                            'B', "dyeBlack",
                            'R', "dyeRed",});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Switch Motor
            structure = EnumSignal.SWITCH_MOTOR;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "RBW",
                            "PCI",
                            'W', "dyeWhite",
                            'I', Item.ingotIron,
                            'P', Block.pistonBase,
                            'C', ItemCircuit.getReceiverCircuit(),
                            'B', "dyeBlack",
                            'R', "dyeRed",});
                CraftingPlugin.addShapedOreRecipe(stack, new Object[]{
                            "RBW",
                            "ICP",
                            'W', "dyeWhite",
                            'I', Item.ingotIron,
                            'P', Block.pistonBase,
                            'C', ItemCircuit.getReceiverCircuit(),
                            'B', "dyeBlack",
                            'R', "dyeRed",});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Receiver Box
            structure = EnumSignal.BOX_RECEIVER;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedRecipe(stack, new Object[]{
                            "ICI",
                            "IRI",
                            'I', Item.ingotIron,
                            'R', Item.redstone,
                            'C', ItemCircuit.getReceiverCircuit(),});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Controller Box
            structure = EnumSignal.BOX_CONTROLLER;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedRecipe(stack, new Object[]{
                            "ICI",
                            "IRI",
                            'I', Item.ingotIron,
                            'R', Item.redstone,
                            'C', ItemCircuit.getControllerCircuit(),});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Capacitor Box
            structure = EnumSignal.BOX_CAPACITOR;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedRecipe(stack, new Object[]{
                            "ICI",
                            "IRI",
                            'I', Item.ingotIron,
                            'R', Item.redstone,
                            'C', Item.redstoneRepeater,});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }

            // Define Signal Block Box
            structure = EnumSignal.BOX_BLOCK_RELAY;
            if (RailcraftConfig.isSubBlockEnabled(structure.getTag())) {

                ItemStack stack = structure.getItem();
                CraftingPlugin.addShapedRecipe(stack, new Object[]{
                            "ICI",
                            "IRI",
                            'I', Item.ingotIron,
                            'R', Item.redstone,
                            'C', ItemCircuit.getSignalCircuit(),});
                RailcraftLanguage.getInstance().registerItemName(stack, structure.getTag());
            }
        }
    }
}
