package railcraft.common.modules;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.oredict.ShapedOreRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.aesthetics.cube.BlockCube;
import railcraft.common.blocks.aesthetics.cube.EnumCube;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.aesthetics.post.BlockPostMetal;
import railcraft.common.blocks.aesthetics.post.EnumPost;
import railcraft.common.blocks.aesthetics.wall.BlockRailcraftWall;
import railcraft.common.blocks.aesthetics.wall.EnumWall;
import railcraft.common.blocks.signals.EnumSignal;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.modules.ModuleManager.Module;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.crafting.RollingMachineCraftingManager;
import railcraft.common.util.misc.EnumColor;

public class ModuleStructures extends RailcraftModule {

    @Override
    public void initFirst() {
        RailcraftBlocks.registerBlockSignal();
        RailcraftBlocks.registerBlockPost();
        RailcraftBlocks.registerBlockPostMetal();
        BlockRailcraftWall.registerBlock();

        Block blockPost = RailcraftBlocks.getBlockPost();
        if (blockPost != null) {
            MinecraftForge.setBlockHarvestLevel(blockPost, EnumPost.WOOD.ordinal(), "axe", 0);
            ItemStack stack = EnumPost.WOOD.getItem(4);
            CraftingManager.getInstance().addShapelessRecipe(stack, new Object[]{
                        RailcraftPartItems.getTieWood()
                    });
            RailcraftLanguage.getInstance().registerItemName(stack, EnumPost.WOOD.getTag());


            MinecraftForge.setBlockHarvestLevel(blockPost, EnumPost.STONE.ordinal(), "pickaxe", 1);
            stack = EnumPost.STONE.getItem(8);
            CraftingPlugin.addShapedRecipe(stack, new Object[]{
                        "SIS",
                        "SIS",
                        "SIS",
                        'I', RailcraftPartItems.getRebar(),
                        'S', Block.stone,});
            RailcraftLanguage.getInstance().registerItemName(EnumPost.STONE.getItem(), EnumPost.STONE.getTag());


            MinecraftForge.setBlockHarvestLevel(blockPost, EnumPost.METAL.ordinal(), "pickaxe", 1);
            stack = EnumPost.METAL.getItem(16);
            RailcraftLanguage.getInstance().registerItemName(EnumPost.METAL.getItem(), EnumPost.METAL.getTag());

            RailcraftCraftingManager.rollingMachine.addRecipe(stack, new Object[]{
                        "III",
                        " I ",
                        "III",
                        'I', Item.ingotIron,});
            RailcraftCraftingManager.rollingMachine.addRecipe(stack, new Object[]{
                        "I I",
                        "III",
                        "I I",
                        'I', Item.ingotIron,});

            stack = EnumPost.METAL.getItem(32);
            IRecipe recipe = new ShapedOreRecipe(stack, new Object[]{
                        "III",
                        " I ",
                        "III",
                        'I', "ingotSteel"});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);
            recipe = new ShapedOreRecipe(stack, new Object[]{
                        "I I",
                        "III",
                        "I I",
                        'I', "ingotSteel"});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

            stack = EnumPost.METAL.getItem(12);
            recipe = new ShapedOreRecipe(stack, new Object[]{
                        "III",
                        " I ",
                        "III",
                        'I', "ingotBronze"});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);
            recipe = new ShapedOreRecipe(stack, new Object[]{
                        "I I",
                        "III",
                        "I I",
                        'I', "ingotBronze"});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);

            stack = EnumPost.METAL.getItem(20);
            recipe = new ShapedOreRecipe(stack, new Object[]{
                        "III",
                        " I ",
                        "III",
                        'I', "ingotRefinedIron"});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);
            recipe = new ShapedOreRecipe(stack, new Object[]{
                        "I I",
                        "III",
                        "I I",
                        'I', "ingotRefinedIron"});
            RollingMachineCraftingManager.getInstance().getRecipeList().add(recipe);
        }

        Block blockPostMetal = RailcraftBlocks.getBlockPostMetal();
        if (blockPost != null && blockPostMetal != null) {
            ItemStack stackColored = BlockPostMetal.getItem(1, -1);
            ItemStack stackRaw = EnumPost.METAL.getItem();

            for (EnumColor color : EnumColor.values()) {
                ItemStack outputStack = new ItemStack(RailcraftBlocks.getBlockPostMetal(), 8, color.ordinal());
                CraftingPlugin.addShapedOreRecipe(outputStack, new Object[]{
                            "III",
                            "IDI",
                            "III",
                            'I', stackRaw,
                            'D', color.getDye(),});
                CraftingPlugin.addShapedOreRecipe(outputStack, new Object[]{
                            "III",
                            "IDI",
                            "III",
                            'I', stackColored,
                            'D', color.getDye(),});
                RailcraftLanguage.getInstance().registerItemName(outputStack, "post.metal." + color.getBasicTag());
            }
        }

        Block blockStructure = RailcraftBlocks.getBlockSignal();
        if (blockStructure != null) {

            // Define Iron Post
            EnumSignal structure = EnumSignal.METAL_POST;
            MinecraftForge.setBlockHarvestLevel(blockStructure, structure.ordinal(), "pickaxe", 1);
            GameRegistry.registerTileEntity(structure.getTileClass(), "RCTileStructureIronPost");
            
            // Define Wood Post
            structure = EnumSignal.WOOD_POST;
            MinecraftForge.setBlockHarvestLevel(blockStructure, structure.ordinal(), "axe", 0);
            GameRegistry.registerTileEntity(structure.getTileClass(), "RCTileStructureWoodPost");

            // Define Stone Post
            structure = EnumSignal.STONE_POST;
            MinecraftForge.setBlockHarvestLevel(blockStructure, structure.ordinal(), "pickaxe", 1);
            GameRegistry.registerTileEntity(structure.getTileClass(), "RCTileStructureStonePost");
        }

        EnumCube cubeType = EnumCube.CONCRETE_BLOCK;
        if (RailcraftConfig.isSubBlockEnabled(cubeType.getTag())) {
            BlockCube.registerBlock();
            Block cube = BlockCube.getBlock();
            if (cube != null) {
                ItemStack stack = cubeType.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, cubeType.getTag());
                if (ModuleManager.isModuleLoaded(Module.FACTORY)
                        && RailcraftBlocks.getBlockMachineAlpha() != null
                        && RailcraftConfig.isSubBlockEnabled(EnumMachineAlpha.ROLLING_MACHINE.getTag())) {
                    stack.stackSize = 8;
                    CraftingPlugin.addShapedRecipe(stack, new Object[]{
                                "SIS",
                                "ISI",
                                "SIS",
                                'I', RailcraftPartItems.getRebar(),
                                'S', Block.stone,});
                } else {
                    stack.stackSize = 4;
                    CraftingPlugin.addShapedRecipe(stack, new Object[]{
                                " S ",
                                "SIS",
                                " S ",
                                'I', Item.ingotIron,
                                'S', Block.stone,});
                }
            }
        }


        cubeType = EnumCube.SANDY_BRICK;
        if (RailcraftConfig.isSubBlockEnabled(cubeType.getTag())) {
            BlockCube.registerBlock();
            Block cube = BlockCube.getBlock();
            if (cube != null) {
                ItemStack stack = cubeType.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, cubeType.getTag());
                CraftingPlugin.addShapedRecipe(stack,
                        "BM",
                        "MB",
                        'B', new ItemStack(Item.brick),
                        'M', new ItemStack(Block.sand));
            }
        }

        cubeType = EnumCube.INFERNAL_BRICK;
        if (RailcraftConfig.isSubBlockEnabled(cubeType.getTag())) {
            BlockCube.registerBlock();
            Block cube = BlockCube.getBlock();
            if (cube != null) {
                ItemStack stack = cubeType.getItem(2);
                RailcraftLanguage.getInstance().registerItemName(stack, cubeType.getTag());
                CraftingPlugin.addShapedRecipe(stack,
                        "MB",
                        "BM",
                        'B', new ItemStack(Block.netherBrick),
                        'M', new ItemStack(Block.slowSand));
            }
        }

//        cubeType = EnumCube.BANDED_PLANKS;
//        if(RailcraftConfig.isSubBlockEnabled(cubeType.getTag())) {
//            RailcraftBlocks.registerBlockCube();
//            Block cube = RailcraftBlocks.getBlockCube();
//            if(cube != null) {
//                ItemStack stack = cubeType.getItem(8);
//                RailcraftLanguage.getInstance().registerItemName(stack, cubeType.getTag());
//                ModLoader.addRecipe(stack, new Object[]{
//                        "WWW",
//                        "III",
//                        "WWW",
//                        'I', new ItemStack(Item.ingotIron),
//                        'W', Block.planks});
//            }
//        }
    }

    @Override
    public void initSecond() {
        EnumWall.registerRecipes();
    }
}
