package railcraft.common.modules;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.block.Block;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecartRegistry;
import net.minecraftforge.liquids.LiquidContainerData;
import net.minecraftforge.liquids.LiquidContainerRegistry;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.detector.BlockDetector.Detector;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.machine.beta.EnumMachineBeta;
import railcraft.common.blocks.machine.gamma.EnumMachineGamma;
import railcraft.common.carts.EntityCartTank;
import railcraft.common.carts.EnumCart;
import railcraft.common.carts.ItemCart;
import railcraft.common.core.Railcraft;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.items.RailcraftToolItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.util.misc.MiscTools;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ModuleTransport extends RailcraftModule {

    @Override
    public void initFirst() {
        EnumMachineAlpha utility = EnumMachineAlpha.TANK_WATER;
        if (RailcraftConfig.isSubBlockEnabled(utility.getTag())) {
            RailcraftBlocks.registerBlockMachineAlpha();
            Block blockUtility = RailcraftBlocks.getBlockMachineAlpha();
            if (blockUtility != null) {
                ItemStack stack = utility.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, utility.getTag());
                CraftingPlugin.addShapedOreRecipe(utility.getItem(6), new Object[]{
                            "WWW",
                            "ISI",
                            "WWW",
                            'I', new ItemStack(Item.ingotIron),
                            'S', new ItemStack(Item.slimeBall),
                            'W', "plankWood"});
            }
        }

        EnumMachineBeta tankIronWall = EnumMachineBeta.TANK_IRON_WALL;
        if (RailcraftConfig.isSubBlockEnabled(tankIronWall.getTag())) {
            RailcraftBlocks.registerBlockMachineBeta();
            Block block = RailcraftBlocks.getBlockMachineBeta();
            if (block != null) {
                ItemStack stack = tankIronWall.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, tankIronWall.getTag());
                CraftingPlugin.addShapedRecipe(tankIronWall.getItem(4), new Object[]{
                            "PP",
                            "PP",
                            'P', RailcraftPartItems.getPlateIron(),});

                RailcraftCraftingManager.blastFurnace.addRecipe(stack.itemID, stack.getItemDamage(), 1280, RailcraftPartItems.getIngotSteel());
            }
        }

        EnumMachineBeta tankIronGauge = EnumMachineBeta.TANK_IRON_GAUGE;
        if (RailcraftConfig.isSubBlockEnabled(tankIronGauge.getTag())) {
            RailcraftBlocks.registerBlockMachineBeta();
            Block block = RailcraftBlocks.getBlockMachineBeta();
            if (block != null) {
                ItemStack stack = tankIronGauge.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, tankIronGauge.getTag());
                CraftingPlugin.addShapedRecipe(tankIronGauge.getItem(4), new Object[]{
                            "GPG",
                            "PGP",
                            "GPG",
                            'P', RailcraftPartItems.getPlateIron(),
                            'G', new ItemStack(Block.thinGlass)});

                RailcraftCraftingManager.blastFurnace.addRecipe(stack.itemID, stack.getItemDamage(), 1280, RailcraftPartItems.getIngotSteel());
            }
        }

        EnumMachineBeta tankIronValve = EnumMachineBeta.TANK_IRON_VALVE;
        if (RailcraftConfig.isSubBlockEnabled(tankIronValve.getTag())) {
            RailcraftBlocks.registerBlockMachineBeta();
            Block block = RailcraftBlocks.getBlockMachineBeta();
            if (block != null) {
                ItemStack stack = tankIronValve.getItem();
                RailcraftLanguage.getInstance().registerItemName(stack, tankIronValve.getTag());
                CraftingPlugin.addShapedRecipe(tankIronValve.getItem(4), new Object[]{
                            "GPG",
                            "PLP",
                            "GPG",
                            'P', RailcraftPartItems.getPlateIron(),
                            'L', new ItemStack(Block.lever),
                            'G', new ItemStack(Block.fenceIron)});

                RailcraftCraftingManager.blastFurnace.addRecipe(stack.itemID, stack.getItemDamage(), 1280, RailcraftPartItems.getIngotSteel());
            }
        }

        EnumMachineGamma itemLoader = EnumMachineGamma.ITEM_LOADER;
        if (RailcraftConfig.isSubBlockEnabled(itemLoader.getTag())) {
            RailcraftBlocks.registerBlockMachineGamma();
            Block blockUtility = RailcraftBlocks.getBlockMachineGamma();
            if (blockUtility != null) {
                ItemStack stack = itemLoader.getItem(1);
                Block blockDetector = RailcraftBlocks.getBlockDetector();
                if (blockDetector != null) {
                    CraftingPlugin.addShapedRecipe(stack, new Object[]{
                                "L",
                                "D",
                                'D', new ItemStack(blockDetector, 1, Detector.ITEM.ordinal()),
                                'L', Block.dispenser,});
                } else {
                    CraftingPlugin.addShapedRecipe(stack, new Object[]{
                                "L",
                                "D",
                                'D', Block.pressurePlateStone,
                                'L', Block.dispenser,});
                }
                RailcraftLanguage.getInstance().registerItemName(stack, itemLoader.getTag());


                itemLoader = EnumMachineGamma.ITEM_LOADER_ADVANCED;
                if (RailcraftConfig.isSubBlockEnabled(itemLoader.getTag())) {
                    ItemStack stackAdvanced = itemLoader.getItem(1);
                    CraftingPlugin.addShapedOreRecipe(stackAdvanced, new Object[]{
                                "IRI",
                                "RLR",
                                "ISI",
                                'I', "ingotSteel",
                                'R', Item.redstone,
                                'S', RailcraftToolItems.getSteelShovel(),
                                'L', stack,});

                    RailcraftLanguage.getInstance().registerItemName(stackAdvanced, itemLoader.getTag());
                }
            }
        }

        EnumMachineGamma itemUnloader = EnumMachineGamma.ITEM_UNLOADER;
        if (RailcraftConfig.isSubBlockEnabled(itemUnloader.getTag())) {
            RailcraftBlocks.registerBlockMachineGamma();
            Block blockUtility = RailcraftBlocks.getBlockMachineGamma();
            if (blockUtility != null) {
                ItemStack stack = itemUnloader.getItem(1);
                Block blockDetector = RailcraftBlocks.getBlockDetector();
                if (blockDetector != null) {
                    CraftingPlugin.addShapedRecipe(stack, new Object[]{
                                "D",
                                "L",
                                'D', new ItemStack(blockDetector, 1, Detector.ITEM.ordinal()),
                                'L', Block.dispenser,});
                } else {
                    CraftingPlugin.addShapedRecipe(stack, new Object[]{
                                "D",
                                "L",
                                'D', Block.pressurePlateStone,
                                'L', Block.dispenser,});
                }
                RailcraftLanguage.getInstance().registerItemName(stack, itemUnloader.getTag());

                itemUnloader = EnumMachineGamma.ITEM_UNLOADER_ADVANCED;
                if (RailcraftConfig.isSubBlockEnabled(itemUnloader.getTag())) {
                    ItemStack stackAdvanced = itemUnloader.getItem(1);
                    CraftingPlugin.addShapedOreRecipe(stackAdvanced, new Object[]{
                                "IRI",
                                "RLR",
                                "ISI",
                                'I', "ingotSteel",
                                'R', Item.redstone,
                                'S', RailcraftToolItems.getSteelShovel(),
                                'L', stack,});

                    RailcraftLanguage.getInstance().registerItemName(stackAdvanced, itemUnloader.getTag());
                }
            }
        }

        EnumMachineGamma liquidLoader = EnumMachineGamma.LIQUID_LOADER;
        if (RailcraftConfig.isSubBlockEnabled(liquidLoader.getTag())) {
            RailcraftBlocks.registerBlockMachineGamma();
            Block block = RailcraftBlocks.getBlockMachineGamma();
            if (block != null) {
                ItemStack stack = liquidLoader.getItem(1);
                Block blockDetector = RailcraftBlocks.getBlockDetector();
                ItemStack detector;
                if (blockDetector != null) {
                    detector = new ItemStack(blockDetector, 1, Detector.TANK.ordinal());
                } else {
                    detector = new ItemStack(Block.pressurePlateStone);
                }
                CraftingPlugin.addShapedRecipe(liquidLoader.getItem(), new Object[]{
                            "GLG",
                            "G G",
                            "GDG",
                            'D', detector,
                            'G', new ItemStack(Block.glass),
                            'L', Block.dispenser,});
                RailcraftLanguage.getInstance().registerItemName(stack, liquidLoader.getTag());
            }
        }

        EnumMachineGamma liquidUnloader = EnumMachineGamma.LIQUID_UNLOADER;
        if (RailcraftConfig.isSubBlockEnabled(liquidUnloader.getTag())) {
            RailcraftBlocks.registerBlockMachineGamma();
            Block block = RailcraftBlocks.getBlockMachineGamma();
            if (block != null) {
                ItemStack stack = liquidUnloader.getItem(1);
                Block blockDetector = RailcraftBlocks.getBlockDetector();
                ItemStack detector;
                if (blockDetector != null) {
                    detector = new ItemStack(blockDetector, 1, Detector.TANK.ordinal());
                } else {
                    detector = new ItemStack(Block.pressurePlateStone);
                }
                CraftingPlugin.addShapedRecipe(liquidUnloader.getItem(), new Object[]{
                            "GDG",
                            "G G",
                            "GLG",
                            'D', detector,
                            'G', new ItemStack(Block.glass),
                            'L', Block.dispenser,});
                RailcraftLanguage.getInstance().registerItemName(stack, liquidUnloader.getTag());
            }
        }

        EnumCart cart = EnumCart.TANK;
        boolean cartEnabled = RailcraftConfig.isCartEnabled(cart.getTag());
        if (cartEnabled) {
            int id = RailcraftConfig.getItemId(cart.getItemTag());
            if (id > 0) {
                MiscTools.registerMinecart(Railcraft.getMod(), EntityCartTank.class, cart.getTag(), cart.getId());
                Item item = EntityCartTank.item = new ItemCart(id, cart);
                item.setItemName(cart.getItemTag());
                GameRegistry.registerItem(item, cart.getItemTag());
                ItemStack stackCartTank = new ItemStack(EntityCartTank.item);
                MinecartRegistry.registerMinecart(EntityCartTank.class, stackCartTank);
                RailcraftLanguage.getInstance().registerItemName(stackCartTank, cart.getTag());
                if (EnumMachineBeta.TANK_IRON_GAUGE.isEnabled()) {
                    CraftingPlugin.addShapedRecipe(stackCartTank, new Object[]{
                                "T",
                                "M",
                                'T', EnumMachineBeta.TANK_IRON_GAUGE.getItem(),
                                'M', Item.minecartEmpty
                            });
                } else {
                    CraftingPlugin.addShapedRecipe(stackCartTank, new Object[]{
                                "GGG",
                                "GMG",
                                "GGG",
                                'G', new ItemStack(Block.glass),
                                'M', Item.minecartEmpty
                            });
                }
                CraftingManager.getInstance().addShapelessRecipe(new ItemStack(Item.minecartEmpty), new Object[]{
                            stackCartTank,});

                ItemRegistry.registerItem(cart.getTag(), stackCartTank);
            }
        }
    }

    @Override
    public void postInit() {
        if (EntityCartTank.item != null) {
            ItemStack stackCartTank = new ItemStack(EntityCartTank.item);

            for (LiquidContainerData container : LiquidContainerRegistry.getRegisteredLiquidContainerData()) {
                ItemStack filteredCart = EntityCartTank.getCartItemForFilter(container.filled);
                CraftingManager.getInstance().addShapelessRecipe(filteredCart, new Object[]{stackCartTank, container.filled});
                CraftingManager.getInstance().addShapelessRecipe(stackCartTank, new Object[]{filteredCart});
//                ItemRegistry.registerItem("cart.tank."+container.liquid.asItemStack().getItem().getItemDisplayName(container.liquid.asItemStack()).toLowerCase().replace(" ", "."), filteredCart);
            }
        }
    }
}
