/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.plugins.forestry;

import cpw.mods.fml.common.Loader;
import cpw.mods.fml.common.event.FMLInterModComms;
import cpw.mods.fml.common.registry.TickRegistry;
import cpw.mods.fml.relauncher.Side;
import forestry.api.core.ItemInterface;
import forestry.api.recipes.RecipeManagers;
import forestry.api.storage.BackpackManager;
import forestry.api.storage.EnumBackpackType;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.nbt.NBTTagString;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.plugins.forge.CraftingPlugin;
import railcraft.common.plugins.forge.CreativePlugin;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class ForestryPlugin {

    public static Boolean modLoaded = null;
    public static Item trackmanBackpackT1;
    public static Item trackmanBackpackT2;
    public static Item icemanBackpackT1;
    public static Item icemanBackpackT2;

    public static ItemStack getItem(String tag) {
        if (!isForestryInstalled()) {
            return null;
        }
        try {
            return ItemInterface.getItem(tag);
        } catch (Throwable error) {
            Game.logErrorAPI("Forestry", error);
        }
        return null;
    }

    public static void addBackpackItem(String pack, ItemStack stack) {
        if (stack == null) {
            return;
        }
        FMLInterModComms.sendMessage("Forestry", "add-backpack-items", String.format("%s@%d:%d", pack, stack.itemID, stack.getItemDamage()));
    }

    public static void registerBackpacks() {
        try {
            if (BackpackManager.backpackInterface == null) {
                return;
            }

            String tag = "rc.backpack.trackman.t1";
            int id = RailcraftConfig.getItemId(tag);
            if (id > 0) {
                trackmanBackpackT1 = BackpackManager.backpackInterface.addBackpack(id, TrackmanBackpack.getInstance(), EnumBackpackType.T1).setCreativeTab(CreativePlugin.TAB);;
                RailcraftLanguage.getInstance().registerItemName(trackmanBackpackT1, tag);

                ItemStack output = new ItemStack(trackmanBackpackT1);
                addBackpackTooltip(output);
                CraftingPlugin.addShapedRecipe(output,
                        "X#X",
                        "VYV",
                        "X#X",
                        '#', Block.cloth,
                        'V', new ItemStack(Block.rail),
                        'X', Item.silk,
                        'Y', new ItemStack(Block.chest));
            }

            tag = "rc.backpack.trackman.t2";
            id = RailcraftConfig.getItemId(tag);
            if (id > 0) {
                trackmanBackpackT2 = BackpackManager.backpackInterface.addBackpack(id, TrackmanBackpack.getInstance(), EnumBackpackType.T2).setCreativeTab(CreativePlugin.TAB);;
                RailcraftLanguage.getInstance().registerItemName(trackmanBackpackT2, tag);

                ItemStack silk = getItem("craftingMaterial");

                if (silk != null) {
                    silk.setItemDamage(3);

                    ItemStack output = new ItemStack(trackmanBackpackT2);
                    addBackpackTooltip(output);
                    RecipeManagers.carpenterManager.addRecipe(200, new LiquidStack(Block.waterStill, 1000), null, output, new Object[]{
                                "WXW",
                                "WTW",
                                "WWW",
                                'X', Item.diamond,
                                'W', silk,
                                'T', trackmanBackpackT1});
                }
            }

            tag = "rc.backpack.iceman.t1";
            id = RailcraftConfig.getItemId(tag);
            if (id > 0) {
                icemanBackpackT1 = BackpackManager.backpackInterface.addBackpack(id, IcemanBackpack.getInstance(), EnumBackpackType.T1).setCreativeTab(CreativePlugin.TAB);;
                RailcraftLanguage.getInstance().registerItemName(icemanBackpackT1, tag);

                ItemStack output = new ItemStack(icemanBackpackT1);
                addBackpackTooltip(output);
                CraftingPlugin.addShapedRecipe(output,
                        "X#X",
                        "VYV",
                        "X#X",
                        '#', Block.cloth,
                        'V', new ItemStack(Block.blockSnow),
                        'X', Item.silk,
                        'Y', new ItemStack(Block.chest));
            }

            tag = "rc.backpack.iceman.t2";
            id = RailcraftConfig.getItemId(tag);
            if (id > 0) {
                icemanBackpackT2 = BackpackManager.backpackInterface.addBackpack(id, IcemanBackpack.getInstance(), EnumBackpackType.T2).setCreativeTab(CreativePlugin.TAB);;
                RailcraftLanguage.getInstance().registerItemName(icemanBackpackT2, tag);

                ItemStack silk = getItem("craftingMaterial");

                if (silk != null) {
                    silk.setItemDamage(3);

                    ItemStack output = new ItemStack(icemanBackpackT2);
                    addBackpackTooltip(output);
                    RecipeManagers.carpenterManager.addRecipe(200, new LiquidStack(Block.waterStill, 1000), null, output, new Object[]{
                                "WXW",
                                "WTW",
                                "WWW",
                                'X', Item.diamond,
                                'W', silk,
                                'T', icemanBackpackT1});
                }
            }

            if (icemanBackpackT1 != null || icemanBackpackT2 != null) {
                TickRegistry.registerTickHandler(new IceManTickHandler(), Side.SERVER);
            }
        } catch (Throwable error) {
            Game.logErrorAPI("Forestry", error);
        }
    }

    private static void addBackpackTooltip(ItemStack stack) {
        NBTTagCompound nbt = InvTools.getItemData(stack);
        NBTTagCompound display = nbt.getCompoundTag("display");
        nbt.setCompoundTag("display", display);
        NBTTagList lore = display.getTagList("Lore");
        display.setTag("Lore", lore);
        lore.appendTag(new NBTTagString("madeby", "\u00a77\u00a7o" + RailcraftLanguage.translate("backpack.tooltip")));
    }

    public static void setupBackpackContents() {
        try {
            if (BackpackManager.backpackInterface == null) {
                return;
            }
            TrackmanBackpack.getInstance().setup();
            IcemanBackpack.getInstance().setup();
        } catch (Throwable error) {
            Game.logErrorAPI("Forestry", error);
        }
    }

    public static boolean isForestryInstalled() {
        if (modLoaded == null) {
            modLoaded = Loader.isModLoaded("Forestry");
        }
        return modLoaded;
    }
}
