/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.plugins.forestry;

import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import railcraft.common.blocks.aesthetics.wall.EnumWall;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.inventory.StandaloneInventory;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class IcemanBackpack extends BaseBackpack {

    private static IcemanBackpack instance;
    private static final ItemStack SNOWBALL = new ItemStack(Item.snowball);
    private static final ItemStack SNOWBLOCK = new ItemStack(Block.blockSnow);
    private static final String INV_TAG = "Items";

    public static IcemanBackpack getInstance() {
        if (instance == null) {
            instance = new IcemanBackpack();
        }
        return instance;
    }

    protected IcemanBackpack() {
    }

    public void setup() {
        addValidItem(Block.snow);
        addValidItem(Block.blockSnow);
        addValidItem(Block.ice);
        addValidItem(EnumWall.SNOW.getItem());
        addValidItem(EnumWall.ICE.getItem());
        addValidItem(Item.snowball);
    }

    public void compactInventory(ItemStack backpack) {
        StandaloneInventory inv = new StandaloneInventory(45);
        NBTTagCompound data = backpack.getTagCompound();
        if (data == null) return;
        InvTools.readInvFromNBT(inv, INV_TAG, data);
        int numSnowballs = InvTools.countItems(inv, SNOWBALL);
        if (numSnowballs >= 16) {
            for (int i = 0; i < 4; i++) {
                InvTools.removeOneItem(inv, SNOWBALL);
            }
            if (InvTools.moveItemStack(new ItemStack(Block.blockSnow), inv) == null) {
                InvTools.writeInvToNBT(inv, INV_TAG, data);
            }
        } else if (numSnowballs < 8 && InvTools.removeOneItem(inv, SNOWBLOCK) != null) {
            if (InvTools.moveItemStack(new ItemStack(Item.snowball, 4), inv) == null) {
                InvTools.writeInvToNBT(inv, INV_TAG, data);
            }
        }
    }

    @Override
    public String getKey() {
        return "ICEMAN";
    }

    @Override
    public String getName() {
        return RailcraftLanguage.translate("backpack.iceman.t1");
    }

    @Override
    public int getPrimaryColour() {
        return 0xFFFFFF;
    }

    @Override
    public int getSecondaryColour() {
        return 0xFFFFFF;
    }
}
