/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.plugins.forestry;

import net.minecraft.block.Block;
import net.minecraft.block.BlockRail;
import net.minecraft.item.Item;
import net.minecraft.item.ItemMinecart;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.machine.alpha.EnumMachineAlpha;
import railcraft.common.blocks.machine.beta.EnumMachineBeta;
import railcraft.common.blocks.signals.ItemSignalBlockSurveyor;
import railcraft.common.blocks.signals.ItemSignalTuner;
import railcraft.common.items.ItemCircuit;
import railcraft.common.items.ItemGoggles;
import railcraft.common.items.RailcraftPartItems;
import railcraft.common.items.RailcraftToolItems;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.liquids.LiquidItems;
import railcraft.common.modules.ModuleSignals;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class TrackmanBackpack extends BaseBackpack {

    private static TrackmanBackpack instance;

    public static TrackmanBackpack getInstance() {
        if (instance == null) {
            instance = new TrackmanBackpack();
        }
        return instance;
    }

    protected TrackmanBackpack() {
    }

    public void setup() {
        addValidItem(RailcraftToolItems.getCrowbar());
        addValidItem(ItemGoggles.getItem());
        addValidItem(ItemCircuit.getCircuitItem());
        addValidItem(ItemSignalBlockSurveyor.getItem());
        addValidItem(ItemSignalTuner.getItem());
        addValidItem(RailcraftToolItems.getOveralls());

        for (int blockId = 0; blockId < Block.blocksList.length; blockId++) {
            Block block = Block.blocksList[blockId];
            if (block == null) continue;
            if (BlockRail.isRailBlock(blockId)) {
                addValidItem(block);
            }
        }

        for (int itemId = Item.shovelSteel.itemID; itemId < Item.itemsList.length; itemId++) {
            Item item = Item.itemsList[itemId];
            if (item instanceof ItemMinecart) {
                addValidItem(item);
            }
        }

        addValidItem(LiquidItems.getCreosoteOilBottle());
        addValidItem(LiquidItems.getCreosoteOilBucket());
        addValidItem(LiquidItems.getCreosoteOilCell());
        addValidItem(LiquidItems.getCreosoteOilCan());
        addValidItem(LiquidItems.getCreosoteOilRefactory());
        addValidItem(LiquidItems.getCreosoteOilWax());

        addValidItem(EnumMachineAlpha.WORLD_ANCHOR.getItem());
        addValidItem(EnumMachineAlpha.PERSONAL_ANCHOR.getItem());
        addValidItem(EnumMachineBeta.SENTINEL.getItem());

        addValidItem(RailcraftPartItems.itemRail);
        addValidItem(RailcraftPartItems.itemRailbed);
        addValidItem(RailcraftPartItems.itemTie);
        addValidItem(RailcraftPartItems.itemSignalLamp);

        addValidItem(RailcraftBlocks.getBlockMachineGamma());
        addValidItem(RailcraftBlocks.getBlockElevator());
        addValidItem(RailcraftBlocks.getBlockSignal());
        addValidItem(RailcraftBlocks.getBlockDetector());
    }

    @Override
    public String getKey() {
        return "TRACKMAN";
    }

    @Override
    public String getName() {
        return RailcraftLanguage.translate("backpack.trackman.t1");
    }

    @Override
    public int getPrimaryColour() {
        return 0x0094FF;
    }

    @Override
    public int getSecondaryColour() {
        return 0xFFFFFF;
    }
}
