package railcraft.common.plugins.ic2;

import cpw.mods.fml.common.Loader;
import ic2.api.Ic2Recipes;
import ic2.api.Items;
import ic2.api.energy.event.EnergyTileLoadEvent;
import ic2.api.energy.event.EnergyTileSourceEvent;
import ic2.api.energy.event.EnergyTileUnloadEvent;
import ic2.api.energy.tile.IEnergySource;
import ic2.api.energy.tile.IEnergyTile;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraftforge.common.MinecraftForge;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.Game;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class IC2Plugin {

    private static Map<String, ItemStack> itemCache = new HashMap<String, ItemStack>();
    private static Boolean modLoaded = null;

    public static ItemStack getItem(String tag) {
        if (!isIC2Installed()) {
            return null;
        }
        ItemStack stack = itemCache.get(tag);
        if (stack != null) {
            return stack;
        }
        try {
            stack = Items.getItem(tag);
            if (stack != null) {
                itemCache.put(tag, stack.copy());
            }
            return stack;
        } catch (Throwable error) {
            Game.logErrorAPI("IC2", error);
        }
        return null;
    }

    public static void addTileToNet(IEnergyTile tile) {
        try {
            MinecraftForge.EVENT_BUS.post(new EnergyTileLoadEvent(tile));
        } catch (Throwable error) {
            Game.logErrorAPI("IC2", error);
        }
    }

    public static void removeTileFromNet(IEnergyTile tile) {
        try {
            MinecraftForge.EVENT_BUS.post(new EnergyTileUnloadEvent(tile));
        } catch (Throwable error) {
            Game.logErrorAPI("IC2", error);
        }
    }

    public static int emitEnergy(IEnergySource source, int amount) {
        try {
            EnergyTileSourceEvent event = new EnergyTileSourceEvent(source, amount);
            MinecraftForge.EVENT_BUS.post(event);
            return event.amount;
        } catch (Throwable error) {
            Game.logErrorAPI("IC2", error);
        }
        return amount;
    }

    public static void removeMaceratorRecipes(ItemStack... items) {
        try {
            List<Entry<ItemStack, ItemStack>> recipes = Ic2Recipes.getMaceratorRecipes();

            Iterator<Entry<ItemStack, ItemStack>> it = recipes.iterator();
            while (it.hasNext()) {
                Entry<ItemStack, ItemStack> entry = it.next();
                if (InvTools.isItemEqual(entry.getValue(), items)
                        || InvTools.isItemEqual(entry.getKey(), items)) {
                    it.remove();
                }
            }
        } catch (Throwable error) {
            Game.logErrorAPI("IC2", error);
        }
    }

    public static void removeMaceratorDustRecipes(ItemStack... items) {
        try {
            List<Entry<ItemStack, ItemStack>> recipes = Ic2Recipes.getMaceratorRecipes();

            Iterator<Entry<ItemStack, ItemStack>> it = recipes.iterator();
            while (it.hasNext()) {
                Entry<ItemStack, ItemStack> entry = it.next();
                ItemStack input = entry.getKey();
                if (input != null && input.getItem() instanceof ItemBlock && InvTools.isItemEqual(entry.getValue(), items)) {
                    it.remove();
                }
            }
        } catch (Throwable error) {
            Game.logErrorAPI("IC2", error);
        }
    }

    public static void nerfSyntheticCoal() {
        Iterator it = CraftingManager.getInstance().getRecipeList().iterator();
        while (it.hasNext()) {
            IRecipe r = (IRecipe) it.next();
            ItemStack output = r.getRecipeOutput();
            if (output != null) {
                if (output.itemID == Item.coal.itemID && output.stackSize == 20) {
                    output.stackSize = 5;
                }
            }
        }
    }

    private static void tagCompressorCoalRecipe() {
        try {
            if (isIC2Installed()) {
                ItemStack coal = new ItemStack(Item.coal);
                List<Map.Entry<ItemStack, ItemStack>> recipes = Ic2Recipes.getCompressorRecipes();
                for (Map.Entry<ItemStack, ItemStack> recipe : recipes) {
                    ItemStack output = recipe.getValue();
                    if (InvTools.isItemEqualIgnoreNBT(output, coal)) {
                        InvTools.markItemSynthetic(output);
                    }
                }
            }
        } catch (Throwable error) {
            Game.logErrorAPI("IC2", error);
        }
    }

    public static boolean isIC2Installed() {
        if (modLoaded == null) {
            modLoaded = Loader.isModLoaded("IC2");
        }
        return modLoaded;
    }
}
