package railcraft.common.util.crafting;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import railcraft.common.api.crafting.IBlastFurnaceCraftingManager;
import railcraft.common.api.crafting.IBlastFurnaceRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.items.RailcraftToolItems;

public class BlastFurnaceCraftingManager implements IBlastFurnaceCraftingManager {

    private final List<BlastFurnaceRecipe> recipes = new ArrayList<BlastFurnaceRecipe>();

    public static IBlastFurnaceCraftingManager getInstance() {
        return RailcraftCraftingManager.blastFurnace;
    }

    @Override
    public List<ItemStack> getFuels() {
        List<ItemStack> fuels = new ArrayList<ItemStack>();
        fuels.add(RailcraftToolItems.getCoalCoke());
        fuels.add(new ItemStack(Item.coal, 1, 1));
        return fuels;
    }

    @Override
    public List<IBlastFurnaceRecipe> getRecipes() {
        List<IBlastFurnaceRecipe> list = new ArrayList<IBlastFurnaceRecipe>(recipes);
        return list;
    }

    public static class BlastFurnaceRecipe implements IBlastFurnaceRecipe {

        private final int inputId;
        private final int inputDamage;
        private final int cookTime;
        private final ItemStack output;

        public BlastFurnaceRecipe(int inputId, int inputDamage, int cookTime, ItemStack output) {
            this.inputId = inputId;
            this.inputDamage = inputDamage;
            this.cookTime = cookTime;
            this.output = output;
        }

        @Override
        public boolean isRoomForOutput(ItemStack out) {
            if ((out == null || output == null || (out.isItemEqual(output)) && out.stackSize + output.stackSize <= output.getMaxStackSize())) {
                return true;
            }
            return false;
        }

        @Override
        public ItemStack getInput() {
            return new ItemStack(inputId, 1, inputDamage);
        }

        @Override
        public ItemStack getOutput() {
            if (output == null) {
                return null;
            }
            return output.copy();
        }

        @Override
        public int getOutputStackSize() {
            if (output == null) {
                return 0;
            }
            return output.stackSize;
        }

        @Override
        public int getCookTime() {
            return cookTime;
        }
    }

    @Override
    public void addRecipe(int inputId, int inputDamage, int cookTime, ItemStack output) {
        if (output != null) {
            recipes.add(new BlastFurnaceRecipe(inputId, inputDamage, cookTime, output));
        }
    }

    @Override
    public void addRecipe(int inputId, int cookTime, ItemStack output) {
        addRecipe(inputId, -1, cookTime, output);
    }

    @Override
    public void addRecipe(ItemStack input, boolean strictMatching, int cookTime, ItemStack output) {
        if (input != null && output != null) {
            recipes.add(new BlastFurnaceRecipe(input.itemID, strictMatching ? input.getItemDamage() : -1, cookTime, output));
        }
    }

    @Override
    public IBlastFurnaceRecipe getRecipe(int inputId, int inputDamage) {
        for (BlastFurnaceRecipe r : recipes) {
            if (r.inputId == inputId && r.inputDamage == inputDamage) {
                return r;
            }
        }
        for (BlastFurnaceRecipe r : recipes) {
            if (r.inputId == inputId && r.inputDamage == -1) {
                return r;
            }
        }
        return null;
    }

    @Override
    public IBlastFurnaceRecipe getRecipe(ItemStack stack) {
        if (stack == null) {
            return null;
        }
        return getRecipe(stack.itemID, stack.getItemDamage());
    }
}
