package railcraft.common.util.crafting;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.liquids.LiquidStack;
import railcraft.common.api.crafting.ICokeOvenCraftingManager;
import railcraft.common.api.crafting.ICokeOvenRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.util.misc.Game;

public class CokeOvenCraftingManager implements ICokeOvenCraftingManager {

    private final List<CokeOvenRecipe> recipes = new ArrayList<CokeOvenRecipe>();

    public static ICokeOvenCraftingManager getInstance() {
        return RailcraftCraftingManager.cokeOven;
    }

    @Override
    public List<ICokeOvenRecipe> getRecipes() {
        List<ICokeOvenRecipe> list = new ArrayList<ICokeOvenRecipe>(recipes);
        return list;
    }

    public static class CokeOvenRecipe implements ICokeOvenRecipe {

        private final int inputId;
        private final int inputDamage;
        private final LiquidStack liquidOutput;
        private final int cookTime;
        private final ItemStack output;

        public CokeOvenRecipe(int inputId, int inputDamage, ItemStack output, LiquidStack liquid, int cookTime) {
            this.inputId = inputId;
            this.inputDamage = inputDamage;
            this.output = output;
            this.liquidOutput = liquid;
            this.cookTime = cookTime;
        }

        @Override
        public ItemStack getInput() {
            return new ItemStack(inputId, 1, inputDamage);
        }

        @Override
        public ItemStack getOutput() {
            if (output == null) {
                return null;
            }
            return output.copy();
        }

        @Override
        public LiquidStack getLiquidOutput() {
            if (liquidOutput != null) {
                return liquidOutput.copy();
            }
            return null;
        }

        @Override
        public int getCookTime() {
            return cookTime;
        }
    }

    @Override
    public void addRecipe(ItemStack input, ItemStack output, LiquidStack liquidOutput, int cookTime) {
        addRecipe(input.itemID, input.getItemDamage(), output, liquidOutput, cookTime);
    }

    @Override
    public void addRecipe(int inputId, int inputDamage, ItemStack output, LiquidStack liquidOutput, int cookTime) {
        recipes.add(new CokeOvenRecipe(inputId, inputDamage, output, liquidOutput, cookTime));

        Game.log(Level.FINEST, "Adding Coke Oven recipe: {0}, {1}, {2}", Item.itemsList[inputId].getClass().getName(), inputId, inputDamage);
    }

    @Override
    public void addRecipe(int inputId, ItemStack output, LiquidStack liquidOutput, int cookTime) {
        addRecipe(inputId, -1, output, liquidOutput, cookTime);
    }

    @Override
    public ICokeOvenRecipe getRecipe(ItemStack stack) {
        if (stack == null) {
            return null;
        }
        return getRecipe(stack.itemID, stack.getItemDamage());
    }

    @Override
    public ICokeOvenRecipe getRecipe(int inputId, int inputDamage) {
        for (CokeOvenRecipe r : recipes) {
            if (r.inputId == inputId && r.inputDamage == inputDamage) {
                return r;
            }
        }
        for (CokeOvenRecipe r : recipes) {
            if (r.inputId == inputId && r.inputDamage == -1) {
                return r;
            }
        }
        return null;
    }
}
