package railcraft.common.util.crafting;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.minecraft.item.ItemStack;
import railcraft.common.api.crafting.IRockCrusherCraftingManager;
import railcraft.common.api.crafting.IRockCrusherRecipe;
import railcraft.common.api.crafting.RailcraftCraftingManager;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.util.misc.MiscTools;

public class RockCrusherCraftingManager implements IRockCrusherCraftingManager
{

    private final List<CrusherRecipe> recipes = new ArrayList<CrusherRecipe>();

    public static IRockCrusherCraftingManager getInstance()
    {
        return RailcraftCraftingManager.rockCrusher;
    }

    @Override
    public List<IRockCrusherRecipe> getRecipes()
    {
        List<IRockCrusherRecipe> list = new ArrayList<IRockCrusherRecipe>(recipes);
        return list;
    }

    public static class CrusherRecipe implements IRockCrusherRecipe
    {

        private final int inputId;
        private final int inputDamage;
        private final Map<ItemStack, Float> output;

        public CrusherRecipe(int inputId, int inputDamage, HashMap<ItemStack, Float> output)
        {
            this.inputId = inputId;
            this.inputDamage = inputDamage;
            this.output = output;
        }

        @Override
        public ItemStack getInput()
        {
            return new ItemStack(inputId, 1, inputDamage);
        }

        @Override
        public Map<ItemStack, Float> getOutputs()
        {
            return output;
        }

        @Override
        public List<ItemStack> getPossibleOuput()
        {
            List<ItemStack> list = new ArrayList<ItemStack>();
            for(ItemStack item : output.keySet()) {
                for(ItemStack saved : list) {
                    if(InvTools.isItemEqual(saved, item)) {
                        if(saved.stackSize + item.stackSize <= saved.getMaxStackSize()) {
                            saved.stackSize += item.stackSize;
                            item = null;
                        } else {
                            int diff = saved.getMaxStackSize() - saved.stackSize;
                            saved.stackSize = saved.getMaxStackSize();
                            item.stackSize -= diff;
                        }
                        break;
                    }
                }
                if(item != null) {
                    list.add(item.copy());
                }
            }
            return list;
        }

        @Override
        public List<ItemStack> getRandomizedOuput()
        {
            List<ItemStack> list = new ArrayList<ItemStack>();
            for(Map.Entry<ItemStack, Float> entry : output.entrySet()) {
                if(MiscTools.getRand().nextFloat() <= entry.getValue()) {
                    list.add(entry.getKey().copy());
                }
            }
            return list;
        }

        public int getNumberOfOutputs()
        {
            return output.size();
        }
    }

    /**
     *
     * @param inputId
     * @param inputDamage metadata or -1 for wildcard
     * @param output A map of outputs and chances. If more than 9 types of items, there will be unexpected behavior.
     */
    @Override
    public void addRecipe(int inputId, int inputDamage, HashMap<ItemStack, Float> output)
    {
        recipes.add(new CrusherRecipe(inputId, inputDamage, output));
    }

    /**
     *
     * @param input
     * @param output A map of outputs and chances.  If more than 9 types of items, there will be unexpected behavior.
     */
    @Override
    public void addRecipe(ItemStack input, HashMap<ItemStack, Float> output)
    {
        addRecipe(input.itemID, input.getItemDamage(), output);
    }

    @Override
    public CrusherRecipe getRecipe(ItemStack input)
    {
        return getRecipe(input.itemID, input.getItemDamage());
    }

    @Override
    public CrusherRecipe getRecipe(int inputId, int inputDamage)
    {
        for(CrusherRecipe r : recipes) {
            if(r.inputId == inputId && r.inputDamage == inputDamage) {
                return r;
            }
        }
        for(CrusherRecipe r : recipes) {
            if(r.inputId == inputId && r.inputDamage == -1) {
                return r;
            }
        }
        return null;
    }
}
