package railcraft.common.util.effects;

import cpw.mods.fml.common.SidedProxy;
import net.minecraft.entity.Entity;
import net.minecraft.tileentity.TileEntity;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class EffectManager {

    @SidedProxy(clientSide = "railcraft.client.util.effects.ClientProxy", serverSide = "railcraft.common.util.effects.CommonProxy")
    public static IEffectManager instance;

    public static interface IEffectSource {

        public double getX();

        public double getY();

        public double getZ();

        public boolean isDead();
    }

    public static class EffectSourceTile implements IEffectSource {

        private final TileEntity source;

        private EffectSourceTile(TileEntity source) {
            this.source = source;
        }

        @Override
        public double getX() {
            return source.xCoord + 0.5;
        }

        @Override
        public double getY() {
            return source.yCoord + 0.5;
        }

        @Override
        public double getZ() {
            return source.zCoord + 0.5;
        }

        @Override
        public boolean isDead() {
            return source.isInvalid();
        }
    }

    public static class EffectSourceEntity implements IEffectSource {

        private final Entity source;

        private EffectSourceEntity(Entity source) {
            this.source = source;
        }

        @Override
        public double getX() {
            return source.posX;
        }

        @Override
        public double getY() {
            return source.posY + source.yOffset;
        }

        @Override
        public double getZ() {
            return source.posZ;
        }

        @Override
        public boolean isDead() {
            return source.isDead;
        }
    }

    public static IEffectSource getEffectSource(Object source) {
        if (source instanceof TileEntity) {
            return new EffectSourceTile((TileEntity) source);
        } else if (source instanceof Entity) {
            return new EffectSourceEntity((Entity) source);
        }
        return null;
    }
}
