package railcraft.common.util.inventory;

import net.minecraft.block.BlockRail;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemSeeds;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.MinecartRegistry;
import railcraft.common.api.core.items.IItemType;
import railcraft.common.api.core.items.IMinecartItem;
import railcraft.common.api.core.items.ITrackItem;
import railcraft.common.util.misc.BallastRegistry;
import railcraft.common.util.misc.MiscTools;

/**
 * This interface is used with several of the functions in IItemTransfer
 * to provide a convenient means of dealing with entire classes of items without
 * having to specify each item individually.
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public enum EnumItemType implements IItemType
{

    FUEL, TRACK, MINECART, BALLAST, FEED;

    public static void initialize() {
        for(EnumItemType type : EnumItemType.values()) {
            IItemType.types.put(type.name(), type);
        }
    }

    @Override
    public boolean isItemType(ItemStack stack) {
        if(stack == null) {
            return false;
        }
        switch (this) {
            case FUEL:
                return MiscTools.getItemBurnTime(stack) > 0;
            case TRACK:
                return stack.getItem() instanceof ITrackItem || (stack.getItem() instanceof ItemBlock && BlockRail.isRailBlock(stack.itemID));
            case MINECART:
                return MinecartRegistry.getCartClassForItem(stack) != null || stack.getItem() instanceof IMinecartItem;
            case BALLAST:
                return BallastRegistry.isItemBallast(stack);
            case FEED:
                return stack.getItem() instanceof ItemFood || stack.itemID == Item.wheat.itemID || stack.getItem() instanceof ItemSeeds;
            default:
                return false;
        }
    }
}
