package railcraft.common.util.misc;

import java.util.HashSet;
import java.util.Set;
import net.minecraft.block.Block;
import net.minecraft.item.ItemStack;

/**
 * Register an item here to designate it as a possible ballast that can be used
 * in the Bore.
 *
 * It is expected that ballast is affected by gravity.
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class BallastRegistry {

    private static Set<BlockKey> ballastRegistry = new HashSet<BlockKey>();

    static {
        registerBallast(Block.gravel.blockID, 0);
    }

    public static void registerBallast(int blockID, int metadata) {
        ballastRegistry.add(new BlockKey(blockID, metadata));
    }

    public static boolean isItemBallast(ItemStack ballast) {
        return ballastRegistry.contains(new BlockKey(ballast.itemID, ballast.getItemDamage()));
    }

    public static Set<BlockKey> getRegisteredBallasts() {
        return ballastRegistry;
    }
}
