package railcraft.common.util.misc;

import cpw.mods.fml.common.FMLCommonHandler;
import cpw.mods.fml.common.registry.EntityRegistry;
import cpw.mods.fml.common.registry.GameRegistry;
import java.util.List;
import java.util.Random;
import java.util.logging.Level;
import net.minecraft.block.Block;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityFurnace;
import net.minecraft.util.MathHelper;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.LiquidStack;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import net.minecraftforge.oredict.ShapelessOreRecipe;
import railcraft.common.api.core.items.ItemRegistry;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.tracks.EnumTrack;
import railcraft.common.core.RailcraftConfig;
import railcraft.common.lang.RailcraftLanguage;
import railcraft.common.util.inventory.InvTools;
import railcraft.common.liquids.LiquidFilter;
import railcraft.common.liquids.LiquidManager;

public abstract class MiscTools {

    private static Random rand = new Random();

    /**
     * Returns a Random instance.
     *
     * @return Random
     */
    public static Random getRand() {
        return rand;
    }

    /**
     * Replacement function for fuel values since most users seem to overwrite
     * TileEntityFurnace.
     *
     * @param stack The item to test
     * @return The fuel value
     */
    public static int getItemBurnTime(ItemStack stack) {
        try {
            if (stack == null) {
                return 0;
            }

            int itemID = stack.getItem().itemID;

            if (stack.getItem() instanceof ItemBlock && Block.blocksList[itemID] != null) {
                Block block = Block.blocksList[itemID];

                String name = block.getBlockName();
                if (name != null && name.contains("blockScaffold")) {
                    return 0;
                }
            }

            LiquidStack liquid = LiquidManager.getInstance().getLiquidInContainer(stack);
            if (LiquidFilter.LAVA.isLiquidEqual(liquid)) {
                return liquid.amount;
            }

            if (itemID == Item.coal.itemID && stack.getItemDamage() == 0 && !InvTools.isSynthetic(stack)) {
                return 3200;
            }

            if (itemID == Item.blazeRod.itemID) {
                return 800;
            }

            String name = stack.getItem().getItemName();
            if (name != null && name.contains("itemScrap")) {
                return 0;
            }

            return TileEntityFurnace.getItemBurnTime(stack);
        } catch (Exception ex) {
            Game.logError("Error in Fuel Handler! Is some mod creating items that are not compliant with standards?", ex);
        }
        return 0;
    }

    public static boolean isOreClass(ItemStack stack, String tag) {
        List<ItemStack> ores = OreDictionary.getOres(tag);

        for (ItemStack ore : ores) {
            if (InvTools.isItemEqual(ore, stack)) {
                return true;
            }
        }
        return false;
    }

    public static boolean isPlayerConnected(String username) {
        return FMLCommonHandler.instance().getMinecraftServerInstance().getConfigurationManager().getPlayerForUsername(username) != null;
    }

    /**
     * Registers a subclass of EntityMinecart with the game engine.
     *
     * This is just a convenience function, it is not required to call this
     * function if you call ModLoader.registerEntityID() and
     * MinecraftForge.registerEntity() elsewhere.
     *
     * @param mod The mod doing the registration
     * @param type The class of the cart
     * @param tag The String identifier
     * @param internalId The mods internal entity id
     */
    public static void registerMinecart(Object mod, Class<? extends EntityMinecart> type, String tag, int internalId) {
        EntityRegistry.registerModEntity(type, tag, internalId, mod, 80, 3, true);
    }

    public static void registerTrack(EnumTrack rail) {
        RailcraftBlocks.registerBlockTrack();
        if (RailcraftBlocks.getBlockTrack() != null) {
            if (RailcraftConfig.isSubBlockEnabled(rail.getTag())) {
                if (rail.getTileClass() != null) {
                    GameRegistry.registerTileEntity(rail.getTileClass(), rail.getTileTag());
                }
                ItemStack stack = rail.registerRecipe();
                RailcraftLanguage.getInstance().registerItemName(stack, rail.getTag());

                ItemRegistry.registerItem(rail.getTag(), stack);
            }
        }
    }

    public static String cleanTag(String tag) {
        return tag.replace("rc.", "");
    }

    public static MovingObjectPosition collisionRayTrace(Vec3 vec3d, Vec3 vec3d1, int i, int j, int k) {
        vec3d = vec3d.addVector(-i, -j, -k);
        vec3d1 = vec3d1.addVector(-i, -j, -k);
        Vec3 vec3d2 = vec3d.getIntermediateWithXValue(vec3d1, 0);
        Vec3 vec3d3 = vec3d.getIntermediateWithXValue(vec3d1, 1);
        Vec3 vec3d4 = vec3d.getIntermediateWithYValue(vec3d1, 0);
        Vec3 vec3d5 = vec3d.getIntermediateWithYValue(vec3d1, 1);
        Vec3 vec3d6 = vec3d.getIntermediateWithZValue(vec3d1, 0);
        Vec3 vec3d7 = vec3d.getIntermediateWithZValue(vec3d1, 1);
        if (!isVecInsideYZBounds(vec3d2)) {
            vec3d2 = null;
        }
        if (!isVecInsideYZBounds(vec3d3)) {
            vec3d3 = null;
        }
        if (!isVecInsideXZBounds(vec3d4)) {
            vec3d4 = null;
        }
        if (!isVecInsideXZBounds(vec3d5)) {
            vec3d5 = null;
        }
        if (!isVecInsideXYBounds(vec3d6)) {
            vec3d6 = null;
        }
        if (!isVecInsideXYBounds(vec3d7)) {
            vec3d7 = null;
        }
        Vec3 vec3d8 = null;
        if (vec3d2 != null && (vec3d8 == null || vec3d.distanceTo(vec3d2) < vec3d.distanceTo(vec3d8))) {
            vec3d8 = vec3d2;
        }
        if (vec3d3 != null && (vec3d8 == null || vec3d.distanceTo(vec3d3) < vec3d.distanceTo(vec3d8))) {
            vec3d8 = vec3d3;
        }
        if (vec3d4 != null && (vec3d8 == null || vec3d.distanceTo(vec3d4) < vec3d.distanceTo(vec3d8))) {
            vec3d8 = vec3d4;
        }
        if (vec3d5 != null && (vec3d8 == null || vec3d.distanceTo(vec3d5) < vec3d.distanceTo(vec3d8))) {
            vec3d8 = vec3d5;
        }
        if (vec3d6 != null && (vec3d8 == null || vec3d.distanceTo(vec3d6) < vec3d.distanceTo(vec3d8))) {
            vec3d8 = vec3d6;
        }
        if (vec3d7 != null && (vec3d8 == null || vec3d.distanceTo(vec3d7) < vec3d.distanceTo(vec3d8))) {
            vec3d8 = vec3d7;
        }
        if (vec3d8 == null) {
            return null;
        }
        byte byte0 = -1;
        if (vec3d8 == vec3d2) {
            byte0 = 4;
        }
        if (vec3d8 == vec3d3) {
            byte0 = 5;
        }
        if (vec3d8 == vec3d4) {
            byte0 = 0;
        }
        if (vec3d8 == vec3d5) {
            byte0 = 1;
        }
        if (vec3d8 == vec3d6) {
            byte0 = 2;
        }
        if (vec3d8 == vec3d7) {
            byte0 = 3;
        }
        return new MovingObjectPosition(i, j, k, byte0, vec3d8.addVector(i, j, k));
    }

    private static boolean isVecInsideYZBounds(Vec3 vec3d) {
        if (vec3d == null) {
            return false;
        } else {
            return vec3d.yCoord >= 0 && vec3d.yCoord <= 1 && vec3d.zCoord >= 0 && vec3d.zCoord <= 1;
        }
    }

    private static boolean isVecInsideXZBounds(Vec3 vec3d) {
        if (vec3d == null) {
            return false;
        } else {
            return vec3d.xCoord >= 0 && vec3d.xCoord <= 1 && vec3d.zCoord >= 0 && vec3d.zCoord <= 1;
        }
    }

    private static boolean isVecInsideXYBounds(Vec3 vec3d) {
        if (vec3d == null) {
            return false;
        } else {
            return vec3d.xCoord >= 0 && vec3d.xCoord <= 1 && vec3d.yCoord >= 0 && vec3d.yCoord <= 1;
        }
    }

    /**
     * Performs a ray trace to determine which side of the block is under the
     * cursor.
     *
     * @param player EntityPlayer
     * @return a side value 0-5
     */
    public static ForgeDirection getCurrentMousedOverSide(EntityPlayer player) {
        double distance = player.capabilities.isCreativeMode ? 5.0F : 4.5F;
        Vec3 posVec = Vec3.createVectorHelper(player.posX, player.posY, player.posZ);
        Vec3 lookVec = player.getLook(1);
        posVec.yCoord += player.getEyeHeight();
        lookVec = posVec.addVector(lookVec.xCoord * distance, lookVec.yCoord * distance, lookVec.zCoord * distance);
        MovingObjectPosition mouseOver = player.worldObj.rayTraceBlocks(posVec, lookVec);
        if (mouseOver != null) {
            return ForgeDirection.getOrientation(mouseOver.sideHit);
        }
        return ForgeDirection.UNKNOWN;
    }

    /**
     * Returns the side closest to the player. Used in placement logic for
     * blocks.
     *
     * @param world
     * @param i
     * @param j
     * @param k
     * @param entityplayer
     * @return a side
     */
    public static ForgeDirection getSideClosestToPlayer(World world, int i, int j, int k, EntityLiving entityplayer) {
        if (MathHelper.abs((float) entityplayer.posX - (float) i) < 2.0F && MathHelper.abs((float) entityplayer.posZ - (float) k) < 2.0F) {
            double d = (entityplayer.posY + 1.82D) - (double) entityplayer.yOffset;
            if (d - (double) j > 2D) {
                return ForgeDirection.UP;
            }
            if ((double) j - d > 0.0D) {
                return ForgeDirection.DOWN;
            }
        }
        int dir = MathHelper.floor_double((double) ((entityplayer.rotationYaw * 4F) / 360F) + 0.5D) & 3;
        switch (dir) {
            case 0:
                return ForgeDirection.NORTH;
            case 1:
                return ForgeDirection.EAST;
            case 2:
                return ForgeDirection.SOUTH;
        }
        return dir != 3 ? ForgeDirection.DOWN : ForgeDirection.WEST;
    }

    /**
     * This function unlike getSideClosestToPlayer can only return north, south,
     * east, west.
     *
     * @param world
     * @param x
     * @param y
     * @param z
     * @param player
     * @return a side
     */
    public static ForgeDirection getHorizontalSideClosestToPlayer(World world, int i, int j, int k, EntityLiving player) {
        ForgeDirection facing = ForgeDirection.NORTH;
        int dir = MathHelper.floor_double((double) ((player.rotationYaw * 4.0F) / 360.0F) + 0.5) & 3;
        switch (dir) {
            case 0:
                return ForgeDirection.NORTH;
            case 1:
                return ForgeDirection.EAST;
            case 2:
                return ForgeDirection.SOUTH;
            case 3:
                return ForgeDirection.WEST;
        }
        return ForgeDirection.NORTH;
    }

    public static ForgeDirection getOppositeSide(int side) {
        int s = side;
        s = s % 2 == 0 ? s + 1 : s - 1;
        return ForgeDirection.getOrientation(s);
    }

    public static int getYOnSide(int y, ForgeDirection side) {
        switch (side) {
            case UP:
                return y + 1;
            case DOWN:
                return y - 1;
            default:
                return y;
        }
    }

    public static int getXOnSide(int x, ForgeDirection side) {
        switch (side) {
            case EAST:
                return x + 1;
            case WEST:
                return x - 1;
            default:
                return x;
        }
    }

    public static int getZOnSide(int z, ForgeDirection side) {
        switch (side) {
            case NORTH:
                return z - 1;
            case SOUTH:
                return z + 1;
            default:
                return z;
        }
    }

    public static boolean blockExistsOnSide(World world, int x, int y, int z, ForgeDirection side) {
        return world.blockExists(getXOnSide(x, side), getYOnSide(y, side), getZOnSide(z, side));
    }

    public static int getBlockMetadataOnSide(World world, int i, int j, int k, ForgeDirection side) {
        return world.getBlockMetadata(getXOnSide(i, side), getYOnSide(j, side), getZOnSide(k, side));
    }

    public static int getBlockIdOnSide(IBlockAccess world, int x, int y, int z, ForgeDirection side) {
        return world.getBlockId(getXOnSide(x, side), getYOnSide(y, side), getZOnSide(z, side));
    }

    public static TileEntity getBlockTileEntityOnSide(World world, int i, int j, int k, ForgeDirection side) {
        return world.getBlockTileEntity(getXOnSide(i, side), getYOnSide(j, side), getZOnSide(k, side));
    }

    public static void notifyBlocksOfNeighborChangeOnSide(World world, int i, int j, int k, int blockID, ForgeDirection side) {
        world.notifyBlocksOfNeighborChange(getXOnSide(i, side), getYOnSide(j, side), getZOnSide(k, side), blockID);
    }
}
