/*
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.util.network;

import cpw.mods.fml.common.network.PacketDispatcher;
import net.minecraft.server.management.PlayerInstance;
import net.minecraft.world.WorldServer;
import railcraft.common.api.signals.AbstractPair;
import railcraft.common.api.signals.ISignalPacketBuilder;
import railcraft.common.blocks.RailcraftTileEntity;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class PacketBuilder implements ISignalPacketBuilder {

    private static PacketBuilder instance;

    public static PacketBuilder getInstance() {
        if (instance == null) {
            instance = new PacketBuilder();
        }
        return instance;
    }

    private PacketBuilder() {
    }

    public void sendTileEntityPacket(RailcraftTileEntity tile) {
        if (tile.worldObj instanceof WorldServer) {
            WorldServer world = (WorldServer) tile.worldObj;
            PlayerInstance playerInstance = world.getPlayerManager().getOrCreateChunkWatcher(tile.xCoord >> 4, tile.zCoord >> 4, false);
            if (playerInstance != null) {
                PacketTileEntity pkt = new PacketTileEntity(tile);
                playerInstance.sendToAllPlayersWatchingChunk(pkt.getPacket());
            }
        }
    }
    
    @Override
    public void sendPairPacketUpdate(AbstractPair pairing) {
        PacketPairUpdate pkt = new PacketPairUpdate(pairing);
        PacketDispatcher.sendPacketToAllInDimension(pkt.getPacket(), pairing.getTile().worldObj.provider.dimensionId);
    }

    @Override
    public void sendPairPacketRequest(AbstractPair pairing) {
        PacketPairRequest pkt = new PacketPairRequest(pairing);
        PacketDispatcher.sendPacketToServer(pkt.getPacket());
    }

    public void sendGuiReturnPacket(IGuiReturnHandler handler) {
        PacketGuiReturn pkt = new PacketGuiReturn(handler);
        PacketDispatcher.sendPacketToServer(pkt.getPacket());
    }
}
