package railcraft.common.util.network;

import cpw.mods.fml.common.network.PacketDispatcher;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.entity.Entity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.DimensionManager;
import railcraft.common.core.Railcraft;

public class PacketGuiReturn extends RailcraftPacket
{

    private IGuiReturnHandler obj;

    public PacketGuiReturn() {
        super();
    }

    public PacketGuiReturn(IGuiReturnHandler obj) {
        this.obj = obj;
    }

    @Override
    public void writeData(DataOutputStream data) throws IOException {
        data.writeInt(obj.getWorld().provider.dimensionId);
        if(obj instanceof TileEntity) {
            TileEntity tile = (TileEntity)obj;
            data.writeBoolean(true);
            data.writeInt(tile.xCoord);
            data.writeInt(tile.yCoord);
            data.writeInt(tile.zCoord);
        } else if(obj instanceof Entity) {
            Entity entity = (Entity)obj;
            data.writeBoolean(false);
            data.writeInt(entity.entityId);
        } else {
            return;
        }
        obj.writeGuiData(data);
    }

    @Override
    public void readData(DataInputStream data) throws IOException {
        int dim = data.readInt();
        World world = DimensionManager.getWorld(dim);
        boolean tileReturn = data.readBoolean();
        if(tileReturn) {
            int x = data.readInt();
            int y = data.readInt();
            int z = data.readInt();

            TileEntity t = world.getBlockTileEntity(x, y, z);

            if(t instanceof IGuiReturnHandler) {
                ((IGuiReturnHandler)t).readGuiData(data);
            }

        } else {
            int entityId = data.readInt();
            Entity entity = world.getEntityByID(entityId);

            if(entity instanceof IGuiReturnHandler) {
                ((IGuiReturnHandler)entity).readGuiData(data);
            }
        }
    }

    public void sendPacket() {
        PacketDispatcher.sendPacketToServer(getPacket());
    }

    @Override
    public int getID() {
        return PacketType.GUI_RETURN.ordinal();
    }
}
