package railcraft.common.util.network;

import cpw.mods.fml.common.network.PacketDispatcher;
import cpw.mods.fml.common.network.Player;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.DimensionManager;
import railcraft.common.api.signals.AbstractPair;
import railcraft.common.api.signals.IControllerTile;
import railcraft.common.api.signals.IReceiverTile;
import railcraft.common.api.signals.SignalController;
import railcraft.common.api.signals.SignalReceiver;

public class PacketPairRequest extends RailcraftPacket {

    private AbstractPair pairing;
    private Player player;
    private PacketType packetType;

    public PacketPairRequest(Player player, PacketType type) {
        super();
        this.player = player;
        this.packetType = type;
    }

    public PacketPairRequest(AbstractPair pairing) {
        this.pairing = pairing;
    }

    @Override
    public void writeData(DataOutputStream data) throws IOException {
        TileEntity tile = pairing.getTile();
        data.writeInt(tile.worldObj.provider.dimensionId);
        data.writeInt(tile.xCoord);
        data.writeInt(tile.yCoord);
        data.writeInt(tile.zCoord);
    }

    @Override
    public void readData(DataInputStream data) throws IOException {
        World world = DimensionManager.getWorld(data.readInt());
        if (world == null) {
            return;
        }

        int x = data.readInt();
        int y = data.readInt();
        int z = data.readInt();

        TileEntity tile = world.getBlockTileEntity(x, y, z);

        switch (packetType) {
            case CONTROLLER_REQUEST:
                if (tile instanceof IControllerTile) {
                    pairing = ((IControllerTile) tile).getController();
                }
                break;
            case RECEIVER_REQUEST:
                if (tile instanceof IReceiverTile) {
                    pairing = ((IReceiverTile) tile).getReceiver();
                }
                break;
        }
        if (pairing != null && player != null) {
            PacketPairUpdate pkt = new PacketPairUpdate(pairing);
            PacketDispatcher.sendPacketToPlayer(pkt.getPacket(), player);
        }
    }

    @Override
    public int getID() {
        if (pairing instanceof SignalController) {
            return PacketType.CONTROLLER_REQUEST.ordinal();
        }
        if (pairing instanceof SignalReceiver) {
            return PacketType.RECEIVER_REQUEST.ordinal();
        }
        return -1;
    }
}
