package railcraft.common.util.network;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Collection;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import railcraft.common.api.core.WorldCoordinate;
import railcraft.common.api.signals.AbstractPair;
import railcraft.common.api.signals.IControllerTile;
import railcraft.common.api.signals.IReceiverTile;
import railcraft.common.api.signals.SignalController;
import railcraft.common.api.signals.SignalReceiver;
import railcraft.common.util.misc.Game;

public class PacketPairUpdate extends RailcraftPacket {

    private AbstractPair pairing;
    private PacketType packetType;

    public PacketPairUpdate(PacketType packetType) {
        super();
        this.packetType = packetType;
    }

    public PacketPairUpdate(AbstractPair pairing) {
        this.pairing = pairing;
    }

    @Override
    public void writeData(DataOutputStream data) throws IOException {
        data.writeInt(pairing.getTile().xCoord);
        data.writeInt(pairing.getTile().yCoord);
        data.writeInt(pairing.getTile().zCoord);

        Collection<WorldCoordinate> pairs = pairing.getPairs();
        data.writeByte(pairs.size());
        for (WorldCoordinate coord : pairs) {
            data.writeInt(coord.x);
            data.writeInt(coord.y);
            data.writeInt(coord.z);
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void readData(DataInputStream data) throws IOException {
        World world = Game.getWorld();
        int x = data.readInt();
        int y = data.readInt();
        int z = data.readInt();

        TileEntity tile = world.getBlockTileEntity(x, y, z);

        if (packetType == PacketType.CONTROLLER_UPDATE) {
            if (tile instanceof IControllerTile) {
                pairing = ((IControllerTile) tile).getController();
            }
        } else if (packetType == PacketType.RECEIVER_UPDATE) {
            if (tile instanceof IReceiverTile) {
                pairing = ((IReceiverTile) tile).getReceiver();
            }
        }
        if (pairing != null) {
            try {
                pairing.clearPairings();
            } catch (Throwable error) {
                Game.logErrorAPI("Railcraft", error);
            }
            int size = data.readByte();
            for (int i = 0; i < size; i++) {
                pairing.addPair(data.readInt(), data.readInt(), data.readInt());
            }
        }
    }

    @Override
    public int getID() {
        if (pairing instanceof SignalController) {
            return PacketType.CONTROLLER_UPDATE.ordinal();
        }
        if (pairing instanceof SignalReceiver) {
            return PacketType.RECEIVER_UPDATE.ordinal();
        }
        return -1;
    }
}
