package railcraft.common.util.network;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.block.Block;
import net.minecraft.network.packet.Packet;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import railcraft.common.api.tracks.ITrackInstance;
import railcraft.common.api.tracks.TrackRegistry;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.RailcraftTileEntity;
import railcraft.common.blocks.tracks.TileTrack;
import railcraft.common.util.misc.Game;

public class PacketTileEntity extends RailcraftPacket {

    private RailcraftTileEntity tile;

    public PacketTileEntity() {
        super();
    }

    public PacketTileEntity(RailcraftTileEntity tile) {
        this.tile = tile;
//        System.out.println("Created Tile Packet: " + tile.getClass().getSimpleName());
    }

    @Override
    public Packet getPacket() {
        Packet pkt = super.getPacket();
        pkt.isChunkDataPacket = true;
        return pkt;
    }

    @Override
    public void writeData(DataOutputStream data) throws IOException {
        data.writeInt(tile.xCoord);
        data.writeInt(tile.yCoord);
        data.writeInt(tile.zCoord);
        data.writeShort(tile.getId());
        tile.writePacketData(data);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void readData(DataInputStream data) throws IOException {
        World world = Game.getWorld();
        if (world == null) {
            return;
        }

        int x = data.readInt();
        int y = data.readInt();
        int z = data.readInt();
        short id = data.readShort();

        if (id < 0 || y < 0 || !world.blockExists(x, y, z)) {
            return;
        }

        TileEntity te = world.getBlockTileEntity(x, y, z);

        if (te instanceof RailcraftTileEntity) {
            tile = (RailcraftTileEntity) te;
            if (tile.getId() != id) {
                tile = null;
            }
        } else {
            tile = null;
        }

        if (tile == null) {
            int blockId = world.getBlockId(x, y, z);
            Block blockTrack = RailcraftBlocks.getBlockTrack();
            if (blockTrack != null && blockTrack.blockID == blockId) {
                ITrackInstance track = TrackRegistry.getTrackSpec(id).createInstanceFromSpec();
                tile = new TileTrack(track);
                world.setBlockTileEntity(x, y, z, tile);
            } else {
                world.removeBlockTileEntity(x, y, z);
                te = world.getBlockTileEntity(x, y, z);
                if (te instanceof RailcraftTileEntity) {
                    tile = (RailcraftTileEntity) te;
                }
            }
        }

        if (tile == null) {
            return;
        }

        tile.readPacketData(data);
    }

    @Override
    public int getID() {
        return PacketType.TILE_ENTITY.ordinal();
    }
}
