package railcraft.common.util.network;

import cpw.mods.fml.common.network.PacketDispatcher;
import cpw.mods.fml.common.network.Player;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import net.minecraft.network.packet.Packet;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.DimensionManager;

public class PacketTileRequest extends RailcraftPacket {

    private TileEntity tile;
    private Player player;

    public PacketTileRequest(Player player) {
        super();
        this.player = player;
    }

    public PacketTileRequest(TileEntity tile) {
        this.tile = tile;
    }

    @Override
    public void writeData(DataOutputStream data) throws IOException {
        data.writeInt(tile.worldObj.provider.dimensionId);
        data.writeInt(tile.xCoord);
        data.writeInt(tile.yCoord);
        data.writeInt(tile.zCoord);
    }

    @Override
    public void readData(DataInputStream data) throws IOException {
        World world = DimensionManager.getWorld(data.readInt());
        if (world == null) {
            return;
        }

        int x = data.readInt();
        int y = data.readInt();
        int z = data.readInt();

        tile = world.getBlockTileEntity(x, y, z);

        if (tile != null && player != null) {
            Packet pkt = tile.getDescriptionPacket();
            PacketDispatcher.sendPacketToPlayer(pkt, player);
        }
    }

    @Override
    public int getID() {
        return PacketType.TILE_REQUEST.ordinal();
    }
}
