package railcraft.common.util.network;

import cpw.mods.fml.common.network.IPacketHandler;
import cpw.mods.fml.common.network.Player;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.logging.Level;
import net.minecraft.network.INetworkManager;
import net.minecraft.network.packet.Packet250CustomPayload;
import railcraft.common.util.misc.Game;
import railcraft.common.util.network.RailcraftPacket.PacketType;

public class RailcraftPacketHandler implements IPacketHandler {

    private static PacketType[] packetTypes = PacketType.values();

    @Override
    public void onPacketData(INetworkManager manager, Packet250CustomPayload packet, Player player) {
        DataInputStream data = new DataInputStream(new ByteArrayInputStream(packet.data));
        try {
            RailcraftPacket pkt = null;

            byte packetID = data.readByte();

            if (packetID < 0) {
                return;
            }

//            System.out.println("Packet Received: " + packetID);

            PacketType type = packetTypes[packetID];
            switch (type) {
                case TILE_ENTITY:
                    pkt = new PacketTileEntity();
                    break;
                case GUI_RETURN:
                    pkt = new PacketGuiReturn();
                    break;
                case TILE_EXTRA_DATA:
                    pkt = new PacketTileExtraData();
                    break;
                case TILE_REQUEST:
                    pkt = new PacketTileRequest(player);
                    break;
                case GUI_UPDATE:
                    pkt = new PacketGuiUpdate();
                    break;
                case EFFECT:
                    pkt = new PacketEffect();
                    break;
                case CONTROLLER_UPDATE:
                case RECEIVER_UPDATE:
                    pkt = new PacketPairUpdate(type);
                    break;
                case CONTROLLER_REQUEST:
                case RECEIVER_REQUEST:
                    pkt = new PacketPairRequest(player, type);
                    break;
                default:
                    return;
            }
            pkt.readData(data);
        } catch (IOException e) {
            Game.log(Level.SEVERE, "Exception in PacketHandler.onPacketData {0}", e);
            e.printStackTrace();
        }
    }
}
