/*
 * Copyright (c) CovertJaguar, 2011 http://railcraft.info
 * 
 * This code is the property of CovertJaguar
 * and may only be used with explicit written
 * permission unless otherwise specified on the
 * license page at railcraft.wikispaces.com.
 */
package railcraft.common.util.sounds;

import java.util.HashMap;
import java.util.Map;
import net.minecraft.block.Block;
import net.minecraft.block.StepSound;
import railcraft.common.blocks.RailcraftBlocks;
import railcraft.common.blocks.aesthetics.cube.BlockCube;
import railcraft.common.blocks.aesthetics.cube.EnumCube;
import railcraft.common.blocks.aesthetics.post.EnumPost;
import railcraft.common.blocks.aesthetics.wall.BlockRailcraftWall;
import railcraft.common.blocks.aesthetics.wall.EnumWall;

/**
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public class SoundRegistry {

    private final static Map<Integer, Map<Integer, StepSound>> customSounds = new HashMap<Integer, Map<Integer, StepSound>>();

    public static StepSound getSound(int blockID, int meta) {
        Map<Integer, StepSound> blockSounds = customSounds.get(blockID);
        if (blockSounds != null) {
            return blockSounds.get(meta);
        }
        return null;
    }

    public static void setupBlockSounds() {
        Block block = RailcraftBlocks.getBlockPost();
        if (block != null) {
            registerCustomStepSound(block.blockID, EnumPost.WOOD.ordinal(), Block.soundWoodFootstep);
            registerCustomStepSound(block.blockID, EnumPost.STONE.ordinal(), Block.soundStoneFootstep);
        }

        block = BlockCube.getBlock();
        if (block != null) {

            for (EnumCube cube : EnumCube.VALUES) {
                registerCustomStepSound(block.blockID, cube.ordinal(), Block.soundStoneFootstep);
            }

            registerCustomStepSound(block.blockID, EnumCube.STEEL_BLOCK.ordinal(), Block.soundMetalFootstep);
            registerCustomStepSound(block.blockID, EnumCube.CRUSHED_OBSIDIAN.ordinal(), Block.soundGravelFootstep);
        }

        block = BlockRailcraftWall.getBlock();
        if (block != null) {

            for (EnumWall cube : EnumWall.VALUES) {
                registerCustomStepSound(block.blockID, cube.ordinal(), Block.soundStoneFootstep);
            }

            registerCustomStepSound(block.blockID, EnumWall.ICE.ordinal(), Block.soundGlassFootstep);
            registerCustomStepSound(block.blockID, EnumWall.SNOW.ordinal(), Block.soundSnowFootstep);
        }
    }

    private static void registerCustomStepSound(int blockID, int metadata, StepSound sound) {
        Map<Integer, StepSound> blockSounds = customSounds.get(blockID);
        if (blockSounds == null) {
            blockSounds = new HashMap<Integer, StepSound>();
            customSounds.put(blockID, blockSounds);
        }
        blockSounds.put(metadata, sound);
    }
}
